﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator("/Framework/debug.js", "/Framework/serviceLocator.js", "/Framework/telemetryUtilities.js");
(function(undefined) {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Actions");
    WinJS.Namespace.define("MS.Entertainment.UI.Actions", {ShowImmersiveDetails: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function showImmersiveDetails() {
            this.base()
        }, {
            automationId: MS.Entertainment.UI.AutomationIds.showImmersiveDetails, executed: function executed(param) {
                    MS.Entertainment.Instrumentation.PerfTrack.setStartTime(MS.Entertainment.Instrumentation.PerfTrack.StartPoints.PlayCommand);
                    var listOrMediaItem = this._extractMediaItem(param);
                    var showDetails = (param) ? param.showDetails : undefined;
                    var sessionId = (param && param.sessionId) ? param.sessionId : MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying;
                    var autoPlay = (param) ? param.autoPlay : undefined;
                    var hub = (param) ? param.hub : undefined;
                    var options = (param) ? param.options : {};
                    var startFullScreen = (param) ? param.startFullScreen : false;
                    var appView = Windows.UI.ViewManagement.ApplicationView;
                    var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                    var startPositionMsec = (param) ? param.startPositionMsec : undefined;
                    if (uiStateService.isSnapped && !appView.tryUnsnap())
                        return;
                    MS.Entertainment.Data.List.getData(listOrMediaItem, 0).then(function showImmersiveDetails(mediaItem) {
                        MS.Entertainment.UI.Actions.assert(mediaItem, "MS.Entertainment.Data.List.getData did not return a valid mediaItem");
                        var automationId = WinJS.Utilities.getMember("automationId", param);
                        if (autoPlay && (automationId === MS.Entertainment.UI.AutomationIds.smartButtonPlay || automationId === MS.Entertainment.UI.AutomationIds.smartButtonRestart || automationId === MS.Entertainment.UI.AutomationIds.smartButtonResume)) {
                            MS.Entertainment.Utilities.Telemetry.setCorrelationId(mediaItem.serviceId);
                            var telemetryParameterArray = [{
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.CorrelationID, parameterValue: MS.Entertainment.Utilities.Telemetry.getCorrelationId(mediaItem.serviceId)
                                    }, {
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ProductGuid, parameterValue: mediaItem.zuneId || mediaItem.serviceId || String.empty
                                    }, {
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaType, parameterValue: mediaItem.mediaType || String.empty
                                    }, {
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AutomationId, parameterValue: param.automationId || String.empty
                                    }];
                            if (mediaItem.seasonServiceId || mediaItem.canonicalSeasonId)
                                telemetryParameterArray.push({
                                    parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ParentProductGuid, parameterValue: mediaItem.seasonServiceId || mediaItem.canonicalSeasonId || String.empty
                                });
                            MS.Entertainment.Utilities.Telemetry.logPageAction({domElement: WinJS.Utilities.getMember("referenceContainer.domElement.parentElement.parentElement", this)}, {
                                uri: MS.Entertainment.Utilities.getCurrentUserLocation(), pageTypeId: MS.Entertainment.Utilities.Telemetry.PageTypeId.Details
                            }, null, telemetryParameterArray)
                        }
                        if (param && param.playPreviewOnly)
                            mediaItem.playPreviewOnly = true;
                        else if ("playPreviewOnly" in mediaItem)
                            mediaItem.playPreviewOnly = false;
                        if (param && param.hydrateMediaItem)
                            mediaItem = MS.Entertainment.ViewModels.MediaItemModel.augment(mediaItem);
                        if (autoPlay && (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.game || !mediaItem.inCollection))
                            MS.Entertainment.Utilities.Telemetry.logPlayPreview(mediaItem, false);
                        if (autoPlay && (((MS.Entertainment.Utilities.isVideoApp) && MS.Entertainment.Platform.PlaybackHelpers.isMovie(mediaItem)) || ((MS.Entertainment.Utilities.isMusicApp) && MS.Entertainment.Platform.PlaybackHelpers.isMusicTrackOrMusicVideo(mediaItem)))) {
                            MS.Entertainment.Instrumentation.PerfTrack.enableScenarioPlayProtectedInApp();
                            MS.Entertainment.Instrumentation.PerfTrack.enableScenarioPlayNonProtectedInApp()
                        }
                        if (MS.Entertainment.Utilities.Telemetry.isCurrentPageSearchPage()) {
                            var destination = "Details";
                            if (autoPlay)
                                destination = mediaItem.inCollection ? "Play" : "PlayPreview";
                            if (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeason || MS.Entertainment.Platform.PlaybackHelpers.isTVEpisode(mediaItem)) {
                                var seriesMediaItem = mediaItem;
                                if (seriesMediaItem.seriesId) {
                                    var seriesData = {
                                            libraryId: mediaItem.seriesLibraryId, Name: mediaItem.seriesTitle
                                        };
                                    if (mediaItem.seriesIdType && mediaItem.seriesIdType === MS.Entertainment.Data.Query.edsIdType.zuneCatalog)
                                        seriesData.ZuneId = mediaItem.seriesId;
                                    else
                                        seriesData.ID = mediaItem.seriesId;
                                    seriesMediaItem = MS.Entertainment.Data.augment(seriesData, MS.Entertainment.Data.Augmenter.Marketplace.EdsTVSeries);
                                    seriesMediaItem = MS.Entertainment.ViewModels.MediaItemModel.augment(seriesMediaItem)
                                }
                                else if (seriesMediaItem.canonicalSeriesId) {
                                    seriesMediaItem = MS.Entertainment.Data.augment({
                                        ID: mediaItem.canonicalSeriesId, title: {$value: mediaItem.canonicalSeriesTitle}
                                    }, MS.Entertainment.Data.Augmenter.Marketplace.EdsTVSeries);
                                    seriesMediaItem = MS.Entertainment.ViewModels.MediaItemModel.augment(seriesMediaItem)
                                }
                                seriesMediaItem.hydrate().then(function logSeasonSearchExit() {
                                    MS.Entertainment.Utilities.Telemetry.logSearchExit(seriesMediaItem, false, destination)
                                })
                            }
                            else
                                MS.Entertainment.Utilities.Telemetry.logSearchExit(mediaItem, false, destination)
                        }
                        if (param.showNowPlaying)
                            MS.Entertainment.Platform.PlaybackHelpers.showImmersive(null, {
                                sessionId: sessionId, startFullScreen: startFullScreen
                            });
                        else
                            MS.Entertainment.Platform.PlaybackHelpers.showImmersiveDetails(mediaItem, showDetails, autoPlay, hub, options, undefined, undefined, undefined, undefined, startPositionMsec)
                    }.bind(this))
                }, canExecute: function canExecute(param) {
                    return true
                }, _extractMediaItem: function _extractMediaItem(param) {
                    var mediaItem;
                    if (param && "mediaItem" in param)
                        mediaItem = param.mediaItem;
                    else if (param && "data" in param)
                        mediaItem = param.data;
                    else if (param)
                        mediaItem = param;
                    else {
                        var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                        if (sessionMgr && sessionMgr.nowPlayingSession && sessionMgr.primarySession.currentMedia)
                            mediaItem = sessionMgr.nowPlayingSession.currentMedia
                    }
                    return mediaItem
                }
        })});
    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, function() {
        return new MS.Entertainment.UI.Actions.ShowImmersiveDetails
    })
})()
