﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function(undefined) {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Actions");
    WinJS.Namespace.define("MS.Entertainment.UI.Actions", {DownloadActionBase: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function downloadActionBase() {
            this.base()
        }, {
            executed: function executed(param) {
                MS.Entertainment.UI.Actions.fail("DownloadActionBase.executed must be implemented in derived classes.")
            }, canExecute: function canExecute(param) {
                    return true
                }, _extractMediaItem: function _extractMediaItem(param) {
                    var mediaItem;
                    if ("mediaItem" in param)
                        mediaItem = param.mediaItem;
                    else
                        mediaItem = param;
                    if (MS.Entertainment.Data.List.isList(mediaItem))
                        return MS.Entertainment.Data.List.listToArray(mediaItem).then(function gotArray(mediaItemsArray) {
                                MS.Entertainment.UI.Actions.assert(mediaItemsArray.length === 1, "DownloadActionBase Action only accepts one mediaItem");
                                mediaItem = mediaItemsArray[0];
                                return WinJS.Promise.wrap(mediaItem)
                            }.bind(this));
                    return WinJS.Promise.wrap(mediaItem)
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.UI.Actions", {
        Purchase: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function purchase() {
            this.base()
        }, {
            eventHandlers: null, subTitle: null, dialogToHide: null, hideDialogIfNeeded: function hideDialogIfNeeded() {
                    if (this.dialogToHide) {
                        this.dialogToHide.hide();
                        this.dialogToHide = null
                    }
                }, executed: function executed(param) {
                    var listOrMediaItem = MS.Entertainment.UI.Actions.extractMediaItemFromParam(param);
                    var target = (param) ? param.target : null;
                    var purchaseType = (param) ? param.purchaseType : null;
                    this.eventHandlers = {onMediaAddedEvent: function onMediaAddedEvent() {
                            var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                            if (mediaContext)
                                mediaContext.dispatchEvent(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase)
                        }};
                    MS.Entertainment.Data.List.getData(listOrMediaItem, 0).done(function launchPurchaseFlow(mediaItem) {
                        if (param && param.hydrateMediaItem)
                            mediaItem = MS.Entertainment.ViewModels.MediaItemModel.augment(mediaItem);
                        var mediaIsTVSeason = MS.Entertainment.Platform.PlaybackHelpers.isTVSeason(mediaItem);
                        var productGuid = mediaIsTVSeason ? mediaItem.edsId || mediaItem.canonicalId : mediaItem.serviceId || mediaItem.zuneId;
                        MS.Entertainment.Utilities.Telemetry.setCorrelationId(productGuid);
                        var telemetryParameterArray = [{
                                    parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.CorrelationID, parameterValue: MS.Entertainment.Utilities.Telemetry.getCorrelationId(productGuid)
                                }, {
                                    parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ProductGuid, parameterValue: productGuid || String.empty
                                }, {
                                    parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaType, parameterValue: mediaItem.mediaType || String.empty
                                }, {
                                    parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AutomationId, parameterValue: param.automationId || String.empty
                                }];
                        if (param && param.offer)
                            telemetryParameterArray.concat([{
                                    parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.SignedOfferGuid, parameterValue: param.offer.offerId || String.empty
                                }, {
                                    parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.DeliveryFormat, parameterValue: param.offer.deliveryFormat || String.empty
                                }, {
                                    parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.DistributionRight, parameterValue: param.offer.licenseRight || String.empty
                                }]);
                        MS.Entertainment.Utilities.Telemetry.logPageAction({domElement: WinJS.Utilities.getMember("referenceContainer.domElement.parentElement.parentElement", this)}, {
                            uri: MS.Entertainment.Utilities.getCurrentUserLocation(), pageTypeId: MS.Entertainment.Utilities.Telemetry.PageTypeId.Details
                        }, null, telemetryParameterArray);
                        if (MS.Entertainment.Utilities.Telemetry.isCurrentPageSearchPage())
                            MS.Entertainment.Utilities.Telemetry.logSearchExit(mediaItem, false, "Buy");
                        var getRentalOptionPromise = WinJS.Promise.wrap();
                        var signedOffer = param && param.offer && param.offer.signedOffer;
                        var offerId = param && param.offer && param.offer.offerId;
                        var offer = param && param.offer;
                        if (param && !param.offer && param.streamingOffer && param.downloadOffer) {
                            if (!MS.Entertainment.UI.NetworkStatusService.isOnline()) {
                                var error = MS.Entertainment.Platform.Playback.Error.NS_E_WMPIM_USEROFFLINE;
                                MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_FAILED_PANEL_HEADER), error.code);
                                telemetryParameterArray.push({
                                    parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ErrorCode, parameterValue: error.code
                                });
                                telemetryParameterArray.push({
                                    parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ErrorId, parameterValue: error.name
                                });
                                MS.Entertainment.Utilities.Telemetry.logTelemetryEventWithParameterArray(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.UIFailure, telemetryParameterArray);
                                return
                            }
                            this.eventHandlers.onMessageEvent = this.hideDialogIfNeeded.bind(this);
                            this.eventHandlers.onFinishedEvent = this.hideDialogIfNeeded.bind(this);
                            this.eventHandlers.onCancelEvent = this.hideDialogIfNeeded.bind(this);
                            this.eventHandlers.onErrorEvent = this.hideDialogIfNeeded.bind(this);
                            this.eventHandlers.onShowWebHostDialogComplete = this.hideDialogIfNeeded.bind(this);
                            this.eventHandlers.onShowWebHostDialogError = this.hideDialogIfNeeded.bind(this);
                            getRentalOptionPromise = MS.Entertainment.UI.Controls.RentalOptionsDialog.showRentalOptionsDialog(param.mediaItem, param.streamingOffer, param.downloadOffer).then(function(rentalOptionsDialogResult) {
                                signedOffer = rentalOptionsDialogResult.offer.signedOffer;
                                offerId = rentalOptionsDialogResult.offer.offerId;
                                offer = rentalOptionsDialogResult.offer;
                                this.dialogToHide = rentalOptionsDialogResult.dialog;
                                if (this.dialogToHide && this.dialogToHide.buttons && this.dialogToHide.buttons[0])
                                    this.dialogToHide.buttons[0].isEnabled = false
                            }.bind(this))
                        }
                        getRentalOptionPromise.done(function showPurchaseFlowDialog() {
                            var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                            MS.Entertainment.Platform.PurchaseHelpers.launchPurchaseFlow(mediaItem, target, purchaseType, this.eventHandlers, offerId, null, signedInUser.gamerTag, signedOffer, offer)
                        }.bind(this))
                    }.bind(this), function getDataFailed() {
                        MS.Entertainment.UI.Actions.assert(false, "Execute purchase failed")
                    })
                }, canExecute: function canExecute(param) {
                    return this.isEnabled
                }
        }), PinToStartScreen: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function pinToStartScreen() {
                this.base()
            }, {
                executed: function executed(param) {
                    var listOrMediaItem = MS.Entertainment.UI.Actions.extractMediaItemFromParam(param);
                    var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                    MS.Entertainment.Data.List.getData(listOrMediaItem, 0).done(function getData_complete(media) {
                        return pinToStartScreenService.pinMediaAsync(media)
                    })
                }, canExecute: function canExecute(param) {
                        return this.isEnabled
                    }
            }), UnpinFromStartScreen: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function unpinFromStartScreen() {
                this.base()
            }, {
                executed: function executed(param) {
                    var listOrMediaItem = MS.Entertainment.UI.Actions.extractMediaItemFromParam(param);
                    var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                    MS.Entertainment.Data.List.getData(listOrMediaItem, 0).done(function getData_complete(media) {
                        return pinToStartScreenService.unpinMediaAsync(media)
                    })
                }, canExecute: function canExecute(param) {
                        return this.isEnabled
                    }
            }), PlayFreeOffer: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function playFreeOffer() {
                this.base()
            }, {
                freeOffer: null, freeOfferId: null, freeOfferCurrencyCode: null, freeOfferPurchaseType: null, freeOfferSignedOffer: null, startPlayback: true, subTitle: null, inPurchaseFlow: false, executed: function executed(param) {
                        if (this.inPurchaseFlow)
                            return;
                        this.inPurchaseFlow = true;
                        MS.Entertainment.UI.Actions.assert(this.freeOfferId, "freeOfferId required");
                        MS.Entertainment.UI.Actions.assert(this.freeOfferCurrencyCode, "freeOfferCurrencyCode required");
                        MS.Entertainment.UI.Actions.assert(this.freeOfferSignedOffer, "freeOfferSignedOffer required");
                        var listOrMediaItem = MS.Entertainment.UI.Actions.extractMediaItemFromParam(param);
                        MS.Entertainment.Data.List.getData(listOrMediaItem, 0).done(function launchPurchaseFlow(mediaItem) {
                            if (mediaItem) {
                                if (param && param.hydrateMediaItem)
                                    mediaItem = MS.Entertainment.ViewModels.MediaItemModel.augment(mediaItem);
                                var mediaIsTVSeason = MS.Entertainment.Platform.PlaybackHelpers.isTVSeason(mediaItem);
                                var productGuid = mediaIsTVSeason ? mediaItem.edsId || mediaItem.canonicalId : mediaItem.serviceId || mediaItem.zuneId;
                                MS.Entertainment.Utilities.Telemetry.setCorrelationId(productGuid);
                                var telemetryParameterArray = [{
                                            parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.CorrelationID, parameterValue: MS.Entertainment.Utilities.Telemetry.getCorrelationId(productGuid)
                                        }, {
                                            parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ProductGuid, parameterValue: productGuid || String.empty
                                        }, {
                                            parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaType, parameterValue: mediaItem.mediaType || String.empty
                                        }, {
                                            parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.SignedOfferGuid, parameterValue: this.freeOfferId || String.empty
                                        }, {
                                            parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.DeliveryFormat, parameterValue: "Streaming"
                                        }, {
                                            parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.DistributionRight, parameterValue: this.freeOfferPurchaseType || String.empty
                                        }, {
                                            parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AutomationId, parameterValue: WinJS.Utilities.getMember("automationId", param) || String.empty
                                        }];
                                MS.Entertainment.Utilities.Telemetry.logPageAction({domElement: WinJS.Utilities.getMember("referenceContainer.domElement.parentElement.parentElement", this)}, {
                                    uri: MS.Entertainment.Utilities.getCurrentUserLocation(), pageTypeId: MS.Entertainment.Utilities.Telemetry.PageTypeId.Details
                                }, null, telemetryParameterArray);
                                if (MS.Entertainment.Utilities.Telemetry.isCurrentPageSearchPage())
                                    MS.Entertainment.Utilities.Telemetry.logSearchExit(mediaItem, false, "PlayFreeOffer");
                                MS.Entertainment.Platform.PurchaseHelpers.freePurchaseFlow(mediaItem, this.freeOfferId, this.freeOfferCurrencyCode, this.freeOfferPurchaseType, this.startPlayback, this.freeOfferSignedOffer, this.freeOffer).then(function freePurchaseFlowDone() {
                                    this.inPurchaseFlow = false
                                }.bind(this))
                            }
                        }.bind(this))
                    }, canExecute: function canExecute(param) {
                        return this.isEnabled
                    }
            }), WatchOnXbox: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function watchOnXbox() {
                this.base()
            }, {
                executed: function executed(param) {
                    var listOrMediaItem = MS.Entertainment.UI.Actions.extractMediaItemFromParam(param);
                    MS.Entertainment.Data.List.getData(listOrMediaItem, 0).then(function watchOnXbox(mediaItem) {
                        if (param && param.hydrateMediaItem)
                            mediaItem = MS.Entertainment.ViewModels.MediaItemModel.augment(mediaItem);
                        if (MS.Entertainment.Utilities.Telemetry.isCurrentPageSearchPage())
                            MS.Entertainment.Utilities.Telemetry.logSearchExit(mediaItem, false, "WatchOnXBox");
                        MS.Entertainment.Platform.PlaybackHelpers.playToXboxPauseLocalPlayback(mediaItem)
                    }.bind(this))
                }, canExecute: function canExecute(param) {
                        return MS.Entertainment.Platform.PlaybackHelpers.isPlayToXboxFeatureEnabled()
                    }
            }), PlayOnXbox: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function playOnXbox() {
                this.base()
            }, {
                executed: function executed(param) {
                    var mediaItem = MS.Entertainment.UI.Actions.extractMediaItemFromParam(param);
                    if (param && param.hydrateMediaItem)
                        mediaItem = MS.Entertainment.ViewModels.MediaItemModel.augment(mediaItem);
                    if (MS.Entertainment.Utilities.Telemetry.isCurrentPageSearchPage())
                        MS.Entertainment.Utilities.Telemetry.logSearchExit(mediaItem, false, "PlayOnXBox");
                    MS.Entertainment.Platform.PlaybackHelpers.playToXboxPauseLocalPlayback(mediaItem, null)
                }, canExecute: function canExecute(param) {
                        return MS.Entertainment.Platform.PlaybackHelpers.isPlayToXboxFeatureEnabled()
                    }
            }), SwitchVideoDefinition: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function switchVideoDefinition() {
                this.base();
                this.videoDefinitionPreference = MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.hd
            }, {
                videoDefinitionPreference: null, executed: function executed(param) {
                        var hd = MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.hd;
                        var sd = MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.sd;
                        if (this.videoDefinitionPreference === hd)
                            this.videoDefinitionPreference = sd;
                        else
                            this.videoDefinitionPreference = hd;
                        if (this.isVoiceEnabled)
                            WinJS.Promise.timeout().done(function delay() {
                                XboxJS.UI.Voice.refreshVoiceElements()
                            });
                        var videoDetailsUiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.videoDetailsUiState);
                        videoDetailsUiStateService.fireVideoDefinitionChangeSignal(this.videoDefinitionPreference)
                    }, canExecute: function canExecute(param) {
                        return true
                    }, updateTitleForMediaResolution: function updateTitleForMediaResolution() {
                        var videoDefinition = MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition;
                        var videoDetailsUiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.videoDetailsUiState);
                        var useSdStrings = (this.videoDefinitionPreference === videoDefinition.hd);
                        if (this.isVoiceEnabled) {
                            this.title = useSdStrings ? String.load(String.id.IDS_VIDEO2_L3_SWITCH_TO_SD_BUTTON_VUI_GUI) : String.load(String.id.IDS_VIDEO2_L3_SWITCH_TO_HD_BUTTON_VUI_GUI);
                            this.voicePhrase = useSdStrings ? String.load(String.id.IDS_VIDEO2_L3_SWITCH_TO_SD_BUTTON_VUI_ALM) : String.load(String.id.IDS_VIDEO2_L3_SWITCH_TO_HD_BUTTON_VUI_ALM);
                            this.voicePhoneticPhrase = useSdStrings ? String.load(String.id.IDS_VIDEO2_L3_SWITCH_TO_SD_BUTTON_VUI_PRON) : String.load(String.id.IDS_VIDEO2_L3_SWITCH_TO_HD_BUTTON_VUI_PRON);
                            this.voiceConfidence = useSdStrings ? String.load(String.id.IDS_VIDEO2_L3_SWITCH_TO_SD_BUTTON_VUI_CONF) : String.load(String.id.IDS_VIDEO2_L3_SWITCH_TO_HD_BUTTON_VUI_CONF)
                        }
                        else
                            this.title = useSdStrings ? String.load(String.id.IDS_VIDEO_PRICE_SWITCH_SD) : String.load(String.id.IDS_VIDEO_PRICE_SWITCH_HD)
                    }
            }), SwitchLanguage: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function switchLanguage() {
                this.base()
            }, {
                subTitle: null, videoLanguagePreference: null, videoLanguagesAvailable: null, setVideoLanguages: function setVideoLanguages(languages) {
                        this.videoLanguagesAvailable = languages;
                        var curLang = MS.Entertainment.Utilities.getDisplayLanguageFromLanguageCode(MS.Entertainment.Utilities.getAccountLocale());
                        if (languages.length > 0) {
                            this.videoLanguagePreference = languages[0];
                            for (var i = 0; i < languages.length; i++) {
                                var matchLang = MS.Entertainment.Utilities.getDisplayLanguageFromLanguageCode(languages[i]);
                                if (curLang === matchLang) {
                                    this.videoLanguagePreference = languages[i];
                                    break
                                }
                            }
                        }
                    }, executed: function executed(param) {
                        var curLang = this.videoLanguagePreference;
                        var i = 0;
                        for (i = 0; i < this.videoLanguagesAvailable.length; i++)
                            if (curLang === this.videoLanguagesAvailable[i])
                                break;
                        i++;
                        if (i >= this.videoLanguagesAvailable.length)
                            i = 0;
                        this.videoLanguagePreference = this.videoLanguagesAvailable[i];
                        this.updateTitleForLanguageAndCount();
                        var videoDetailsUiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.videoDetailsUiState);
                        videoDetailsUiStateService.fireVideoLanguageChangeSignal(this.videoLanguagePreference)
                    }, canExecute: function canExecute(param) {
                        return true
                    }, updateTitleForLanguageAndCount: function updateTitleForLanguageAndCount() {
                        var videoDetailsUiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.videoDetailsUiState);
                        var languageCode = this.videoLanguagePreference;
                        var count = this.videoLanguagesAvailable.length;
                        if (this.isVoiceEnabled) {
                            this.title = String.load(String.id.IDS_VIDEO2_L3_SWITCH_LANGUAGE_BUTTON_VUI_GUI);
                            this.voicePhrase = String.load(String.id.IDS_VIDEO2_L3_SWITCH_LANGUAGE_BUTTON_VUI_ALM);
                            this.voicePhoneticPhrase = String.load(String.id.IDS_VIDEO2_L3_SWITCH_LANGUAGE_BUTTON_VUI_PRON);
                            this.voiceConfidence = String.load(String.id.IDS_VIDEO2_L3_SWITCH_LANGUAGE_BUTTON_VUI_CONF)
                        }
                        var displayLanguage = MS.Entertainment.Utilities.getDisplayLanguageFromLanguageCode(languageCode);
                        this.subTitle = String.load(String.id.IDS_VIDEO_PURCHASE_OPTIONS_CURRENT_LANGUAGE).format(displayLanguage, count)
                    }
            }), DownloadAgain: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.DownloadActionBase", function downloadAgain() {
                this.base()
            }, {executed: function executed(param) {
                    var mediaItemPromise = this._extractMediaItem(param);
                    mediaItemPromise.done(function getMediaItem(mediaItem) {
                        if (param && param.hydrateMediaItem)
                            mediaItem = MS.Entertainment.ViewModels.MediaItemModel.augment(mediaItem);
                        MS.Entertainment.Framework.DownloadInfo.getMarketplaceNativeFileDownloadsAsync(mediaItem).done(function getMarketplaceNativeFileDownloadsAsync_complete(result) {
                            for (var i = 0; i < result.length; i++) {
                                var task = result[i];
                                switch (task.taskStatus) {
                                    case Microsoft.Entertainment.FileTransferStatus.notStarted:
                                    case Microsoft.Entertainment.FileTransferStatus.error:
                                    case Microsoft.Entertainment.FileTransferStatus.canceled:
                                        if (task.canRetry)
                                            try {
                                                task.restart()
                                            }
                                            catch(exception) {}
                                        else
                                            task.closeAsync().done(function closeAsync_complete(task){}, function closeAsync_error(error){});
                                        break
                                }
                            }
                        })
                    })
                }}), DownloadPause: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.DownloadActionBase", function downloadPause() {
                this.base()
            }, {executed: function executed(param) {
                    var mediaItemPromise = this._extractMediaItem(param);
                    mediaItemPromise.done(function getMediaItem(mediaItem) {
                        MS.Entertainment.Framework.DownloadInfo.getMarketplaceNativeFileDownloadsAsync(mediaItem).done(function getMarketplaceNativeFileDownloadsAsync_complete(result) {
                            for (var i = 0; i < result.length; i++) {
                                var task = result[i];
                                switch (task.taskStatus) {
                                    case Microsoft.Entertainment.FileTransferStatus.pending:
                                    case Microsoft.Entertainment.FileTransferStatus.notStarted:
                                    case Microsoft.Entertainment.FileTransferStatus.running:
                                        task.pause();
                                        break
                                }
                            }
                        })
                    })
                }}), DownloadResume: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.DownloadActionBase", function downloadResume() {
                this.base()
            }, {executed: function executed(param) {
                    var mediaItemPromise = this._extractMediaItem(param);
                    mediaItemPromise.done(function getMediaItem(mediaItem) {
                        MS.Entertainment.Framework.DownloadInfo.getMarketplaceNativeFileDownloadsAsync(mediaItem).done(function getMarketplaceNativeFileDownloadsAsync_complete(result) {
                            for (var i = 0; i < result.length; i++) {
                                var task = result[i];
                                switch (task.taskStatus) {
                                    case Microsoft.Entertainment.FileTransferStatus.paused:
                                        task.resume();
                                        break
                                }
                            }
                        })
                    })
                }}), DownloadCancel: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.DownloadActionBase", function downloadCancel() {
                this.base()
            }, {executed: function executed(param) {
                    var mediaItemPromise = this._extractMediaItem(param);
                    mediaItemPromise.done(function getMediaItem(mediaItem) {
                        MS.Entertainment.Framework.DownloadInfo.getMarketplaceNativeFileDownloadsAsync(mediaItem).done(function getMarketplaceNativeFileDownloadsAsync_complete(result) {
                            for (var i = 0; i < result.length; i++) {
                                var task = result[i];
                                switch (task.taskStatus) {
                                    case Microsoft.Entertainment.FileTransferStatus.canceled:
                                    case Microsoft.Entertainment.FileTransferStatus.completed:
                                        break;
                                    default:
                                        task.cancel();
                                        break
                                }
                            }
                        })
                    })
                }}), DownloadRemove: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function downloadRemove() {
                this.base()
            }, {
                executed: function executed(param) {
                    var mediaItemPromise = this._extractMediaItem(param);
                    mediaItemPromise.done(function getMediaItem(mediaItem) {
                        if (param && param.hydrateMediaItem)
                            mediaItem = MS.Entertainment.ViewModels.MediaItemModel.augment(mediaItem);
                        MS.Entertainment.Framework.DownloadInfo.getMarketplaceNativeFileDownloadsAsync(mediaItem).done(function getMarketplaceNativeFileDownloadsAsync_complete(result) {
                            for (var i = 0; i < result.length; i++) {
                                var task = result[i];
                                switch (task.taskStatus) {
                                    case Microsoft.Entertainment.FileTransferStatus.notStarted:
                                    case Microsoft.Entertainment.FileTransferStatus.error:
                                    case Microsoft.Entertainment.FileTransferStatus.canceled:
                                        task.closeAsync().done(function closeAsync_complete(task){}, function closeAsync_error(error){});
                                        break
                                }
                            }
                        })
                    })
                }, canExecute: function canExecute(param) {
                        return true
                    }, _extractMediaItem: function _extractMediaItem(param) {
                        var mediaItem;
                        if ("mediaItem" in param)
                            mediaItem = param.mediaItem;
                        else
                            mediaItem = param;
                        if (MS.Entertainment.Data.List.isList(mediaItem))
                            return MS.Entertainment.Data.List.listToArray(mediaItem).then(function gotArray(mediaItemsArray) {
                                    MS.Entertainment.UI.Actions.assert(mediaItemsArray.length === 1, "DownloadRemove Action only accepts one mediaItem");
                                    mediaItem = mediaItemsArray[0];
                                    return WinJS.Promise.wrap(mediaItem)
                                }.bind(this));
                        return WinJS.Promise.wrap(mediaItem)
                    }
            }), DownloadDemo: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function downloadDemo() {
                this.base()
            }, {
                executed: function executed(param) {
                    var mediaItem = MS.Entertainment.UI.Actions.extractMediaItemFromParam(param);
                    if (param && param.hydrateMediaItem)
                        mediaItem = MS.Entertainment.ViewModels.MediaItemModel.augment(mediaItem);
                    if (mediaItem.gameDemo) {
                        var gameMediaItem = {
                                serviceId: mediaItem.gameDemo.id, mediaType: Microsoft.Entertainment.Queries.ObjectType.game
                            };
                        MS.Entertainment.Platform.PurchaseHelpers.launchPurchaseFlow(gameMediaItem)
                    }
                    if (MS.Entertainment.Utilities.Telemetry.isCurrentPageSearchPage())
                        MS.Entertainment.Utilities.Telemetry.logSearchExit(mediaItem, false, "DownloadDemo")
                }, canExecute: function canExecute(param) {
                        return true
                    }
            }), ActionPassThrough: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function invokeButton() {
                this.base()
            }, {
                executed: function executed(param){}, canExecute: function canExecute(param) {
                        return true
                    }
            })
    });
    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, function() {
            return new MS.Entertainment.UI.Actions.Purchase
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.pinToStartScreen))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.pinToStartScreen, function() {
            return new MS.Entertainment.UI.Actions.PinToStartScreen
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.unpinFromStartScreen))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.unpinFromStartScreen, function() {
            return new MS.Entertainment.UI.Actions.UnpinFromStartScreen
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.playFreeOffer))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.playFreeOffer, function() {
            return new MS.Entertainment.UI.Actions.PlayFreeOffer
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.watchOnXbox))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.watchOnXbox, function() {
            return new MS.Entertainment.UI.Actions.WatchOnXbox
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.playOnXbox))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.playOnXbox, function() {
            return new MS.Entertainment.UI.Actions.PlayOnXbox
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.switchVideoDefinition))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.switchVideoDefinition, function() {
            return new MS.Entertainment.UI.Actions.SwitchVideoDefinition
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.switchLanguage))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.switchLanguage, function() {
            return new MS.Entertainment.UI.Actions.SwitchLanguage
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadAgain))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadAgain, function() {
            return new MS.Entertainment.UI.Actions.DownloadAgain
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadCancel))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadCancel, function() {
            return new MS.Entertainment.UI.Actions.DownloadCancel
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadPause))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadPause, function() {
            return new MS.Entertainment.UI.Actions.DownloadPause
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadResume))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadResume, function() {
            return new MS.Entertainment.UI.Actions.DownloadResume
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadRemove))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadRemove, function() {
            return new MS.Entertainment.UI.Actions.DownloadRemove
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadDemo))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadDemo, function() {
            return new MS.Entertainment.UI.Actions.DownloadDemo
        });
    if (!actionService.isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.actionPassThrough))
        actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.actionPassThrough, function() {
            return new MS.Entertainment.UI.Actions.ActionPassThrough
        });
    WinJS.Namespace.define("MS.Entertainment.Pages", {ActionButtonsControl: MS.Entertainment.UI.Framework.defineUserControl("/Components/InlineDetails/ActionButtonsControl.html#actionButtonsControlTemplate", null, {
            replaceInline: false, defaultButtonClass: null, tabIndex: 0, limitFocusMovement: false, _buttonsChangedHandler: null, focusFirstItemOnPageLoad: false, _setDefaultFocusHandler: null, buttons: MS.Entertainment.UI.Framework.observableProperty("buttons", null), currentButtons: MS.Entertainment.UI.Framework.observableProperty("currentButtons", null), orientation: MS.Entertainment.UI.Framework.observableProperty("orientation", "vertical", true), initialize: function initialize() {
                    MS.Entertainment.UI.Actions.assert(this._actionButtons, "this._actionButtons is not valid");
                    MS.Entertainment.UI.Actions.assert(this._actionButtons.domElement, "this._actionButtons.domElement is not valid");
                    this._actionButtons.defaultButtonClass = this.defaultButtonClass;
                    this._actionButtons.tabIndex = this.tabIndex;
                    this.currentButtons = new MS.Entertainment.ObservableArray;
                    this._buttonsChangedHandler = MS.Entertainment.Utilities.addEventHandlers(this, {buttonsChanged: this._buttonsChanged.bind(this)});
                    if (this.focusFirstItemOnPageLoad)
                        this._setDefaultFocusHandler = MS.Entertainment.Utilities.addEventHandlers(this._actionButtons.domElement, {ActionsReady: function setDefaultFocus() {
                                MS.Entertainment.UI.Framework.tryAndFocusElementInSubTreeWithTimer(this._actionButtons.domElement, 0)
                            }.bind(this)});
                    this._buttonsChanged({detail: {newValue: this.buttons}});
                    if (this.orientation === "horizontal") {
                        WinJS.Utilities.addClass(this._actionButtons.domElement, "state-orientationHorizontal");
                        WinJS.Utilities.removeClass(this._actionButtons.domElement, "state-orientationVertical")
                    }
                    else {
                        WinJS.Utilities.removeClass(this._actionButtons.domElement, "state-orientationHorizontal");
                        WinJS.Utilities.addClass(this._actionButtons.domElement, "state-orientationVertical")
                    }
                }, unload: function unload() {
                    if (this._buttonsChangedHandler) {
                        this._buttonsChangedHandler.cancel();
                        this._buttonsChangedHandler = null
                    }
                    if (this._setDefaultFocusHandler) {
                        this._setDefaultFocusHandler.cancel();
                        this._setDefaultFocusHandler = null
                    }
                    MS.Entertainment.UI.Framework.UserControl.prototype.unload.call(this)
                }, _setFocusOverrides: function setFocusOverrides(actionButtons) {
                    var actionButtonControls = Array.prototype.map.call(actionButtons, function getWinControl(actionElement) {
                            return actionElement && actionElement.children.length > 0 && actionElement.children[0].winControl
                        });
                    actionButtonControls = actionButtonControls.filter(function filterFalsyElements(actionButtonControl) {
                        return !!actionButtonControl
                    });
                    var noOp = ".currentPage :focus";
                    for (var i = 1; i < actionButtonControls.length - 1; i++)
                        actionButtonControls[i].focusDirections = null;
                    if (actionButtonControls.length === 1)
                        actionButtonControls[0].focusDirections = {
                            down: noOp, up: noOp
                        };
                    else {
                        actionButtonControls[0].focusDirections = {up: noOp};
                        actionButtonControls[actionButtonControls.length - 1].focusDirections = {down: noOp}
                    }
                }, _buttonsChanged: function buttonInfoChanged(e) {
                    var newButtons = e && e.detail && e.detail.newValue;
                    if (!newButtons)
                        return;
                    WinJS.Utilities.removeClass(this._actionButtons.domElement, "hideFromDisplay");
                    if (!this.replaceInline)
                        this.currentButtons = newButtons;
                    else {
                        var i;
                        var enabledActions = {};
                        newButtons.forEach(function addId(action) {
                            MS.Entertainment.UI.assert(action.id, "Missing action Id");
                            enabledActions[action.id] = true
                        });
                        for (i = (this.currentButtons.length - 1); i >= 0; i--) {
                            var action = this.currentButtons.item(i);
                            if (!enabledActions[action.id])
                                this.currentButtons.removeAt(i)
                        }
                        var action;
                        var currentIndex = 0;
                        var actionLength = newButtons.length;
                        for (i = 0; i < actionLength; i++) {
                            action = newButtons[i];
                            if (this.currentButtons.length <= currentIndex)
                                this.currentButtons.push(action);
                            else if (this.currentButtons.item(currentIndex).id !== action.id)
                                this.currentButtons.insert(currentIndex, action);
                            currentIndex++
                        }
                        WinJS.Promise.timeout().then(function raiseReadyEvent() {
                            var domEvent = document.createEvent("Event");
                            domEvent.initEvent("ActionsReady", true, true);
                            this._actionButtons.domElement.dispatchEvent(domEvent);
                            if (this.limitFocusMovement && this._actionButtons.domElement.children.length > 0)
                                this._setFocusOverrides(this._actionButtons.domElement.children)
                        }.bind(this))
                    }
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.Pages", {ActionButtonsListControl: MS.Entertainment.UI.Framework.deriveUserControl("MS.Entertainment.UI.Controls.ItemsControl", null, function actionButtonsListControl(){}, {
            listViewModel: null, defaultButtonClass: null, tabIndex: 0, controlName: "ActionButtonsControl", initialize: function initialize() {
                    if (!this.itemTemplate)
                        this.itemTemplate = "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplate";
                    MS.Entertainment.UI.Controls.ItemsControl.prototype.initialize.apply(this, arguments)
                }, processItemData: function processItemData(itemData, index) {
                    itemData.execute = MS.Entertainment.Utilities.bindAndMarkSupportedForProcessing(itemData.execute, itemData);
                    if (!itemData.className) {
                        var className = this.defaultButtonClass;
                        if (itemData.extendedClassNames)
                            className += " " + itemData.extendedClassNames;
                        itemData.addProperty("className", className);
                        itemData.tabIndex = this.tabIndex
                    }
                    if (this.listViewModel)
                        return new MS.Entertainment.UI.Controls.ItemsControlViewModel(itemData, this.listViewModel, index);
                    else
                        return itemData
                }, itemsRendered: function itemsRendered() {
                    WinJS.Promise.timeout().done(function raiseReadyEvent() {
                        if (this.domElement) {
                            var domEvent = document.createEvent("Event");
                            domEvent.initEvent("ActionsReady", true, true);
                            this.domElement.dispatchEvent(domEvent)
                        }
                    }.bind(this))
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.Pages", {SmartBuyButtonsListControl: MS.Entertainment.UI.Framework.deriveUserControl("MS.Entertainment.Pages.ActionButtonsListControl", null, null, {
            _eventProxyHandler: null, _eventProxy: null, controlName: "ActionButtonsControl", eventProxy: {
                    get: function() {
                        return this._eventProxy
                    }, set: function(value) {
                            if (value !== this._eventProxy) {
                                this._eventProxy = value;
                                this._cancelEventHandlers();
                                if (this._eventProxy)
                                    this._eventProxyHandler = MS.Entertainment.Utilities.addEventHandlers(this._eventProxy, {currentButtonsChanged: this._setDataSourceFromEvent.bind(this)})
                            }
                        }
                }, unload: function unload() {
                    MS.Entertainment.Pages.ActionButtonsListControl.prototype.unload.apply(this, arguments);
                    this._cancelEventHandlers()
                }, _cancelEventHandlers: function _cancelEventHandlers() {
                    if (this._eventProxyHandler) {
                        this._eventProxyHandler.cancel();
                        this._eventProxyHandler = null
                    }
                }, _setDataSourceFromEvent: function _setDataSourceFromEvent(args) {
                    if (!this._unloaded)
                        this.dataSource = args.detail.value
                }
        })})
})()
