﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function(undefined) {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Actions");
    WinJS.Namespace.define("MS.Entertainment.UI.Actions", {
        AddToNowPlaying: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function addToNowPlaying() {
            this.base()
        }, {
            automationId: MS.Entertainment.UI.AutomationIds.addToNowPlaying, translateDragAndDropParams: function translateDragAndDropParams(source) {
                    return {mediaItem: source && source.data}
                }, executed: function executed(param) {
                    var executeActionObject = new MS.Entertainment.UI.Actions.AddToNowPlaying.AddToNowPlayingExecution(new MS.Entertainment.UI.Actions.AddToNowPlaying.AddToNowPlayingActionAppState);
                    executeActionObject.execute(param)
                }, canExecute: function canExecute(param) {
                    var canExecuteResult = true;
                    var mediaItem = MS.Entertainment.UI.Actions.extractMediaItemFromParam(param);
                    var appState = new MS.Entertainment.UI.Actions.AddToNowPlaying.AddToNowPlayingActionAppState;
                    if (!mediaItem)
                        canExecuteResult = false;
                    else if (Array.isArray(mediaItem))
                        canExecuteResult = mediaItem.length > 0;
                    else if (MS.Entertainment.Data.List.isList(mediaItem))
                        canExecuteResult = mediaItem.count > 0;
                    else if (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.track)
                        canExecuteResult = appState.canAddToNowPlaying(mediaItem);
                    return canExecuteResult
                }
        }), AddArtistToNowPlayingAction: MS.Entertainment.deferredDerive(MS.Entertainment.UI.Actions.Action, function addArtistToNowPlayingAction() {
                this.base()
            }, {
                executed: function executed(param) {
                    var query,
                        id;
                    var mediaItem = MS.Entertainment.UI.Actions.extractMediaItemFromParam(param);
                    var mediaItems = Array.isArray(mediaItem) ? mediaItem : [mediaItem];
                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                    var collectionFilter = mediaContext.collectionFilter;
                    MS.Entertainment.ViewModels.assert(mediaItems[0].mediaType === Microsoft.Entertainment.Queries.ObjectType.person && mediaItems[0].personType === Microsoft.Entertainment.Queries.PersonType.artist, "Invalid media type. Was expecting an artist media object");
                    mediaItems.forEach(function addMediaItemToNowPlaying(mediaItem) {
                        var itemCollection = null;
                        if (param.playCollectionItemsOnly) {
                            itemCollection = new MS.Entertainment.Data.Query.libraryTracks;
                            itemCollection.mediaAvailability = collectionFilter;
                            itemCollection.artistId = mediaItem.libraryId;
                            itemCollection.acquisitionData = new MS.Entertainment.Utilities.AcquisitionData(Microsoft.Entertainment.Platform.AcquisitionContext.collection, Microsoft.Entertainment.Platform.AcquisitionContextType.artist, MS.Entertainment.Utilities.isValidServiceId(mediaItem.canonicalId) ? mediaItem.canonicalId : String.empty);
                            if (param.librarySort)
                                itemCollection.sort = param.librarySort
                        }
                        else if (param.shouldUseTopSongsForArtistAddTo) {
                            itemCollection = new MS.Entertainment.Data.Query.Music.ArtistTopSongs;
                            itemCollection.id = mediaItem.canonicalId;
                            itemCollection.impressionGuid = mediaItem.impressionGuid;
                            itemCollection.hasTotalCount = true
                        }
                        else
                            itemCollection = mediaItem;
                        if (itemCollection)
                            MS.Entertainment.Platform.PlaybackHelpers.addToNowPlaying(itemCollection)
                    });
                    if (!param.suppressNotification)
                        MS.Entertainment.UI.Actions.Playlists._showPlaylistNotification(null, mediaItems, mediaItems[0].mediaType, true)
                }, canExecute: function canExecute(param) {
                        return true
                    }
            })
    });
    WinJS.Namespace.define("MS.Entertainment.UI.Actions.AddToNowPlaying", {
        AddToNowPlayingActionAppState: MS.Entertainment.UI.Framework.define(function addToNowPlayingActionAppState() {
            var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
            this.primarySessionIsSmartDJ = !!sessionMgr.primarySession.smartDJSeed;
            var signedInUserService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
            var featureEnablement = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.featureEnablement);
            this.marketplaceEnabled = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace);
            var freeStreamEnabled = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlay);
            var freePlayLimits = null;
            if (freeStreamEnabled && MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.freePlayLimits))
                freePlayLimits = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.freePlayLimits);
            this.userCanSubscriptionStream = this.marketplaceEnabled && signedInUserService && signedInUserService.isSubscription;
            this.userFreePlayLimitsNotExceeded = !freePlayLimits || !freePlayLimits.isMonthlyFreeLimitExceeded;
            this.userCanFreeStream = this.userFreePlayLimitsNotExceeded && this.marketplaceEnabled && freeStreamEnabled
        }, {
            marketplaceEnabled: false, primarySessionIsSmartDJ: false, purchaseErrors: 0, subscriptionErrors: 0, userCanFreeStream: false, userCanSubscriptionStream: false, userFreePlayLimitsNotExceeded: false, canAddToNowPlaying: function canAddToNowPlaying(track) {
                    var canAddToNowPlaying = false;
                    if (track.inCollection && track.canPlayLocally)
                        canAddToNowPlaying = true;
                    else if (this.marketplaceEnabled) {
                        canAddToNowPlaying = (track.canSubscriptionStream && track.hasPurchased) || (track.canSubscriptionStream && this.userCanSubscriptionStream) || (track.canFreeStream && this.userCanFreeStream);
                        if (!canAddToNowPlaying)
                            if (track.canSubscriptionStream && !this.userCanSubscriptionStream)
                                this.subscriptionErrors++;
                            else if (track.canPurchase || track.canPurchaseAlbum)
                                this.purchaseErrors++
                    }
                    return canAddToNowPlaying
                }
        }), AddToNowPlayingExecution: MS.Entertainment.UI.Framework.define(function addToNowPlayingExecution(appState) {
                this._appState = appState || new MS.Entertainment.UI.Actions.AddToNowPlaying.AddToNowPlayingActionAppState;
                this._appNotificationService = MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.appNotification) && MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appNotification)
            }, {
                _appState: null, _appNotificationService: null, _nowPlayingNotificationCategory: "addToNowPlaying", execute: function execute(param) {
                        var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                        var collectionFilter = mediaContext && mediaContext.collectionFilter;
                        var mediaItemParam = MS.Entertainment.UI.Actions.extractMediaItemFromParam(param);
                        var mediaItems = null;
                        var mediaItemType = null;
                        var showNotification = true;
                        var numMediaItemsAdded = 0;
                        var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                        if (sessionMgr.primarySession.smartDJSeed) {
                            if (!param.suppressNotification)
                                this._showPlaylistNotification(numMediaItemsAdded, mediaItemType)
                        }
                        else {
                            var sentCompleteNotification = false;
                            if (!param.suppressNotification && this._appNotificationService)
                                WinJS.Promise.timeout(500).done(function updateTrackNotifications() {
                                    if (!sentCompleteNotification)
                                        this._appNotificationService.send(new MS.Entertainment.UI.Notification({
                                            notificationType: MS.Entertainment.UI.Notification.Type.Informational, title: String.load(String.id.IDS_MUSIC_NOTIFICATION_ADDING_TO_NOW_PLAYING), subTitle: String.empty, moreDetails: null, icon: MS.Entertainment.UI.Icon.playlist, iconClassName: "icon-playlist", action: null, secondaryActions: null, category: this._nowPlayingNotificationCategory, isPersistent: false
                                        }))
                                }.bind(this));
                            MS.Entertainment.Data.VirtualList.copy(mediaItemParam).then(function gotVirtualList(mediaItemsVirtualList) {
                                return mediaItemsVirtualList.forEachAll(function addMediaItem(args) {
                                        var returnPromise = null;
                                        var mediaItem = args && args.item.data;
                                        var preventPlay = numMediaItemsAdded > 0;
                                        if (!mediaItem)
                                            return;
                                        mediaItemType = mediaItem.mediaType;
                                        if (MS.Entertainment.Utilities.isApp2) {
                                            MS.Entertainment.Platform.PlaybackHelpers.addToNowPlaying(mediaItem, collectionFilter, preventPlay);
                                            returnPromise = WinJS.Promise.as(1)
                                        }
                                        else if (mediaItemType === Microsoft.Entertainment.Queries.ObjectType.album)
                                            if (mediaItem.fromCollection && this._appState.userFreePlayLimitsNotExceeded) {
                                                MS.Entertainment.Platform.PlaybackHelpers.addToNowPlaying(mediaItem, collectionFilter, preventPlay);
                                                returnPromise = WinJS.Promise.as(1)
                                            }
                                            else
                                                returnPromise = this._expandAlbum(mediaItem, collectionFilter, preventPlay);
                                        else if (mediaItemType === Microsoft.Entertainment.Queries.ObjectType.track)
                                            returnPromise = this._addTrackToNowPlaying(mediaItem, collectionFilter, preventPlay);
                                        else if (mediaItem) {
                                            MS.Entertainment.Platform.PlaybackHelpers.addToNowPlaying(mediaItem, collectionFilter, preventPlay);
                                            returnPromise = WinJS.Promise.as(1)
                                        }
                                        return returnPromise.then(function addTotal(addedCount) {
                                                numMediaItemsAdded += addedCount
                                            })
                                    }.bind(this))
                            }.bind(this)).then(null, function failure(error) {
                                MS.Entertainment.UI.Actions.fail("Add to Now Playing encountered an error. Error message " + error && error.message)
                            }).done(function addMediaCompleted() {
                                if (!param.suppressNotification) {
                                    this._showPlaylistNotification(numMediaItemsAdded, mediaItemType);
                                    sentCompleteNotification = true
                                }
                                if (param.completeAction)
                                    param.completeAction()
                            }.bind(this))
                        }
                    }, _addTrackToNowPlaying: function _addTrackToNowPlaying(track, collectionFilter, preventPlay) {
                        if (!track) {
                            MS.Entertainment.UI.Actions.fail("Attempting to add a null track.");
                            return WinJS.Promise.as(0)
                        }
                        return MS.Entertainment.ViewModels.MediaItemModel.hydrateLibraryInfoAsync(track).then(function libraryInfoHydrated() {
                                var addedCount = 0;
                                if (this._appState.canAddToNowPlaying(track)) {
                                    addedCount = 1;
                                    MS.Entertainment.Platform.PlaybackHelpers.addToNowPlaying(track, collectionFilter, preventPlay)
                                }
                                return addedCount
                            }.bind(this), function libraryInfoHydrateFailed(error) {
                                MS.Entertainment.UI.Actions.fail("Failed to hydrate media item. Error message " + error && error.message);
                                return 0
                            })
                    }, _expandAlbum: function _expandAlbum(album, collectionFilter, preventPlay) {
                        if (!album)
                            return WinJS.Promise.as(0);
                        var tracksAdded = 0;
                        return this._getTracks(album).then(function handleTracks(tracks) {
                                var tracksPromise = null;
                                if (tracks)
                                    tracksPromise = tracks.forEachAll(function addTracks(track) {
                                        return this._addTrackToNowPlaying(track && track.item && track.item.data, collectionFilter, preventPlay || tracksAdded > 0).then(function addedTrack(addedCount) {
                                                tracksAdded += addedCount
                                            })
                                    }.bind(this));
                                return WinJS.Promise.as(tracksPromise)
                            }.bind(this)).then(function returnTotalAdded() {
                                return tracksAdded > 0 ? 1 : 0
                            })
                    }, _getTracks: function getTracks(album) {
                        var tracks;
                        if (album.tracks && album.tracks.count)
                            tracks = album.tracks;
                        else if (album.hasCanonicalId) {
                            var query = new MS.Entertainment.Data.Query.Music.AlbumSongs;
                            query.id = album.canonicalId;
                            query.impressionGuid = album.impressionGuid;
                            query.acquisitionData = album.acquisitionData;
                            query.relevancyTrackingContent = album.relevancyTrackingContent;
                            tracks = query.getItems()
                        }
                        else {
                            album = album.cloneWithHydration();
                            tracks = album.hydrate().then(function returnTracks() {
                                return album.tracks
                            })
                        }
                        return WinJS.Promise.as(tracks)
                    }, _showPlaylistNotification: function _showPlaylistNotification(numMediaItemsAdded, mediaItemType) {
                        if (this._appNotificationService)
                            this._appNotificationService.removeNotificationByCategory(this._nowPlayingNotificationCategory);
                        if (numMediaItemsAdded)
                            MS.Entertainment.UI.Actions.Playlists._showPlaylistNotification(null, null, mediaItemType, true, null, null, numMediaItemsAdded);
                        else if (this._appNotificationService) {
                            var subTitle = String.empty;
                            if (this._appState.primarySessionIsSmartDJ)
                                subTitle = String.load(String.id.IDS_MUSIC_NOTIFICATION_RADIO_LOCKED);
                            else if (this._appState.subscriptionErrors)
                                subTitle = String.load(String.id.IDS_MUSIC_NOTIFICATION_MUSIC_PASS_ONLY);
                            else
                                subTitle = String.load(String.id.IDS_MUSIC_NOTIFICATION_PURCHASE_ONLY);
                            this._appNotificationService.send(new MS.Entertainment.UI.Notification({
                                notificationType: MS.Entertainment.UI.Notification.Type.Informational, title: String.load(String.id.IDS_MUSIC_NOTIFICATION_ADD_ERROR), subTitle: subTitle, moreDetails: null, icon: MS.Entertainment.UI.Icon.playlist, iconClassName: "icon-playlist", action: null, secondaryActions: null, category: this._nowPlayingNotificationCategory, isPersistent: false
                            }))
                        }
                    }
            })
    });
    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.addToNowPlaying, function() {
        return new MS.Entertainment.UI.Actions.AddToNowPlaying
    });
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.addArtistToNowPlaying, function() {
        return new MS.Entertainment.UI.Actions.AddArtistToNowPlayingAction
    })
})()
