﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator("/Components/Playback/Controls/PlaybackControl.js", "/Framework/utilities.js");
(function() {
    "use strict";
    var MSEPlatform = WinJS.Namespace.define("MS.Entertainment.Platform", null);
    WinJS.Namespace.defineWithParent(MS.Entertainment.Platform.Playback, "ClosedCaptions", {
        RendererObservables: MS.Entertainment.defineObservable(function RendererObservables_ctor(){}, {ccLcid: 0}), getUserSettings: (function() {
                function app2SettingsSignature() {
                    var signature = "";
                    for (var p in MS.Entertainment.Platform.ClosedCaptions.ClosedCaptionProperties)
                        if (typeof p !== "function")
                            try {
                                signature += p + ": " + JSON.stringify(MS.Entertainment.Platform.ClosedCaptions.ClosedCaptionProperties[p])
                            }
                            catch(ex) {
                                signature += p
                            }
                    return signature
                }
                {};
                var edgeToShadowConverter = {
                        1: "none", 2: "-1px 0px 0px silver, 0px -1px 0px silver, 1px 1px 0px black, 2px 2px 0px black, 3px 3px 0px black", 3: "1px 1px 0px silver, 0px 1px 0px silver, -1px -1px 0px black, -1px 0px 0px black", 4: "0px 0px 4px black, 0px 0px 4px black, 0px 0px 4px black, 0px 0px 4px black", 5: "3px 3px 3px 2px black"
                    };
                function textEdgeShadow(edgeEnum) {
                    return edgeToShadowConverter[+edgeEnum]
                }
                function BaseUserSettingsPlatform() {
                    this.needToModifyDOMToApplySetttings = false;
                    this.loadAndUpdateClosedCaptionStyleSettings = function base_loadAndUpdateClosedCaptionStyleSettings() {
                        var ccSettings = this.loadClosedCaptionStyleSettings();
                        if (ccSettings)
                            this.updateClosedCaptionStyleSettings(ccSettings)
                    };
                    this.loadClosedCaptionStyleSettings = function base_loadClosedCaptionStyleSettings() {
                        var settingsStorage = Windows.Storage.ApplicationData.current.roamingSettings;
                        if (settingsStorage)
                            return {
                                    regionBackgroundColor: settingsStorage.values["CC_REGION_background-color"], regionOpacity: settingsStorage.values["CC_REGION_opacity"], textFontFamily: settingsStorage.values["CC_TEXT_font-family"], textFontSize: settingsStorage.values["CC_TEXT_font-size"], textEdgeAttributeEnum: settingsStorage.values["CC_TEXT_edge-attribute"], textColor: settingsStorage.values["CC_TEXT_color"], textOpacity: settingsStorage.values["CC_TEXT_opacity"], textBackgroundColor: settingsStorage.values["CC_TEXT_background-color"], textBackgroundOpacity: settingsStorage.values["CC_TEXT_background-opacity"]
                                };
                        else
                            return null
                    };
                    this.saveClosedCaptionStyleSettings = function base_saveClosedCaptionStyleSettings(ccSettings) {
                        if (!ccSettings)
                            return;
                        var settingsStorage = Windows.Storage.ApplicationData.current.roamingSettings;
                        if (ccSettings.hasOwnProperty("regionBackgroundColor"))
                            settingsStorage.values["CC_REGION_background-color"] = ccSettings["regionBackgroundColor"];
                        if (ccSettings.hasOwnProperty("regionOpacity"))
                            settingsStorage.values["CC_REGION_opacity"] = ccSettings["regionOpacity"];
                        if (ccSettings.hasOwnProperty("textFontFamily"))
                            settingsStorage.values["CC_TEXT_font-family"] = ccSettings["textFontFamily"];
                        if (ccSettings.hasOwnProperty("textFontSize"))
                            settingsStorage.values["CC_TEXT_font-size"] = ccSettings["textFontSize"];
                        if (ccSettings.hasOwnProperty("textEdgeAttributeEnum"))
                            settingsStorage.values["CC_TEXT_edge-attribute"] = ccSettings["textEdgeAttributeEnum"];
                        if (ccSettings.hasOwnProperty("textColor"))
                            settingsStorage.values["CC_TEXT_color"] = ccSettings["textColor"];
                        if (ccSettings.hasOwnProperty("textOpacity"))
                            settingsStorage.values["CC_TEXT_opacity"] = ccSettings["textOpacity"];
                        if (ccSettings.hasOwnProperty("textBackgroundColor"))
                            settingsStorage.values["CC_TEXT_background-color"] = ccSettings["textBackgroundColor"];
                        if (ccSettings.hasOwnProperty("textBackgroundOpacity"))
                            settingsStorage.values["CC_TEXT_background-opacity"] = ccSettings["textBackgroundOpacity"]
                    };
                    this.updateClosedCaptionStyleSettings = function base_updateClosedCaptionStyleSettings(ccSettings) {
                        var paragraphStyle = ".cc_layout P";
                        var spanStyle = ".cc_layout SPAN";
                        function rgbaFromSplitColor(settings, attr, opacity) {
                            var calculatedOpacity = opacity ? settings[opacity] : settings[attr + "Opacity"];
                            if (calculatedOpacity === String.empty)
                                return "rgb(" + settings[attr + "Color"] + ")";
                            return "rgba(" + settings[attr + "Color"] + "," + calculatedOpacity + ")"
                        }
                        var ccOverrideProcessor = {
                                regionOpacity: function(ctxt, ccSettings) {
                                    if (ccSettings["regionOpacity"]) {
                                        ctxt.rule(paragraphStyle).add("background-color", rgbaFromSplitColor(ccSettings, "regionBackground", "regionOpacity"));
                                        ctxt.rule(paragraphStyle).add("outline", rgbaFromSplitColor(ccSettings, "regionBackground", "regionOpacity") + " 15px solid")
                                    }
                                }, regionBackgroundColor: function(){}, textFontFamily: function(ctxt, ccSettings) {
                                        var textFontFamily = ccSettings["textFontFamily"];
                                        if (!textFontFamily)
                                            return;
                                        ctxt.rule(paragraphStyle).add("font-family", textFontFamily);
                                        ctxt.rule(spanStyle).add("font-family", textFontFamily);
                                        if (this.isFontUsedForSmallCaps(textFontFamily)) {
                                            ctxt.rule(paragraphStyle).add("font-variant", "small-caps");
                                            ctxt.rule(spanStyle).add("font-variant", "small-caps")
                                        }
                                    }, textFontSize: function(ctxt, ccSettings) {
                                        ctxt.rule(paragraphStyle).add("font-size", ccSettings["textFontSize"])
                                    }, textEdgeAttribute: function(ctxt, ccSettings) {
                                        ctxt.rule(paragraphStyle).add("text-shadow", ccSettings["textEdgeAttribute"])
                                    }, textEdgeAttributeEnum: function(ctxt, ccSettings) {
                                        ctxt.rule(paragraphStyle).add("text-shadow", textEdgeShadow(ccSettings["textEdgeAttributeEnum"]))
                                    }, textOpacity: function(){}, textColor: function(ctxt, ccSettings) {
                                        ctxt.rule(paragraphStyle).add("color", rgbaFromSplitColor(ccSettings, "text"));
                                        ctxt.rule(spanStyle).add("color", rgbaFromSplitColor(ccSettings, "text"))
                                    }, textBackgroundColor: function(){}, textBackgroundOpacity: function(ctxt, ccSettings) {
                                        this.needToModifyDOMToApplySetttings = (ccSettings["textBackgroundOpacity"] === String.empty) || (ccSettings["textBackgroundOpacity"] > 0);
                                        ctxt.rule(spanStyle).add("background-color", rgbaFromSplitColor(ccSettings, "textBackground"))
                                    }
                            };
                        function CSSRule() {
                            var attributes = {};
                            this.add = function(attribute, value, overridableByInline) {
                                if (value !== null && value !== undefined)
                                    attributes[attribute] = {
                                        value: value, overridableByInline: !!overridableByInline
                                    }
                            };
                            this.toString = function() {
                                var firstAttribute = true;
                                var cssText = "";
                                for (var attribute in attributes) {
                                    if (firstAttribute)
                                        firstAttribute = false;
                                    else
                                        cssText += "; ";
                                    cssText += attribute + ": ";
                                    cssText += attributes[attribute].value;
                                    if (!attributes[attribute].overridableByInline)
                                        cssText += " !important"
                                }
                                return "{" + cssText + "}"
                            }
                        }
                        {};
                        function CSSContext() {
                            var selectors = {};
                            this.rule = function rule(cssSelector) {
                                if (!selectors[cssSelector])
                                    selectors[cssSelector] = new CSSRule;
                                return selectors[cssSelector]
                            };
                            this.applyStyle = function() {
                                function getEmptyStyleElement() {
                                    var id = "CCStyleSheet";
                                    var ccStyleElement = document.getElementById(id);
                                    if (!ccStyleElement) {
                                        ccStyleElement = document.createElement("STYLE");
                                        document.documentElement.firstChild.appendChild(ccStyleElement);
                                        ccStyleElement.id = id
                                    }
                                    else if (ccStyleElement.sheet && ccStyleElement.sheet.rules) {
                                        var total = ccStyleElement.sheet.rules.length;
                                        for (var i = 0; i < total; i++)
                                            ccStyleElement.sheet.deleteRule(0)
                                    }
                                    return ccStyleElement
                                }
                                var ccStyleElement = getEmptyStyleElement();
                                for (var selector in selectors)
                                    ccStyleElement.sheet.insertRule(selector + selectors[selector], 0)
                            }
                        }
                        {};
                        this.needToModifyDOMToApplySetttings = false;
                        var overrideContext = new CSSContext;
                        for (var customSetting in ccSettings) {
                            MS.Entertainment.Platform.Playback.assert(ccOverrideProcessor[customSetting], "Fix this: Illegal CC custom setting or missing handler - " + customSetting);
                            if (ccOverrideProcessor[customSetting])
                                ccOverrideProcessor[customSetting].bind(this)(overrideContext, ccSettings)
                        }
                        overrideContext.applyStyle()
                    };
                    this.applySettings = function base_applySettings(ccContainer) {
                        if (this.needToModifyDOMToApplySetttings) {
                            var spanCandidates = ccContainer.querySelectorAll("P");
                            for (var i = 0; i < spanCandidates.length; i++) {
                                var pElements = ccContainer.querySelectorAll("P")[i].childNodes;
                                for (var j = 0; j < pElements.length; j++) {
                                    var pChildNode = pElements[j];
                                    if (pChildNode.nodeName === "#text") {
                                        var span = document.createElement('SPAN');
                                        span.className = "cc_text";
                                        pChildNode.parentNode.replaceChild(span, pChildNode);
                                        span.appendChild(pChildNode)
                                    }
                                }
                            }
                        }
                    };
                    this.isFontUsedForSmallCaps = function base_isFontUsedForSmallCaps(fontName) {
                        return fontName === "trebuchet ms"
                    }
                }
                function App2UserSettingsPlatform(){}
                {};
                App2UserSettingsPlatform.prototype = new BaseUserSettingsPlatform;
                App2UserSettingsPlatform.prototype.saveClosedCaptionStyleSettings = function(){};
                var app2CCFontFamilyConverter = {
                        0: String.empty, 1: "courier new", 2: "times new roman", 3: "lucida console", 4: "tahoma", 5: "comic sans ms", 6: "segoe script", 7: "trebuchet ms"
                    };
                App2UserSettingsPlatform.prototype.loadClosedCaptionStyleSettings = function app2_loadClosedCaptionStyleSettings() {
                    var fontSizeEnumConverter = {
                            0: "100%", 1: "50%", 2: "75%", 3: "150%", 4: "200%"
                        };
                    function fontFamily(app2Enum) {
                        return app2CCFontFamilyConverter[app2Enum]
                    }
                    {};
                    function percentagefontSize(enumFontSize) {
                        return fontSizeEnumConverter[enumFontSize]
                    }
                    function rgb(c) {
                        return c.r + "," + c.g + "," + c.b
                    }
                    function opacity(c) {
                        return c.a / 255
                    }
                    function addCCProperty(container, name, value) {
                        if (value === undefined || value === null)
                            return;
                        container[name] = value
                    }
                    var userSettings = MS.Entertainment.Platform.ClosedCaptions.ClosedCaptionProperties;
                    this.currentSignature = app2SettingsSignature();
                    var output = {};
                    if (userSettings.isEnabled && !userSettings.useDefaultOptions) {
                        addCCProperty(output, "regionBackgroundColor", rgb(userSettings.windowColor));
                        addCCProperty(output, "regionOpacity", opacity(userSettings.windowColor));
                        addCCProperty(output, "textFontFamily", fontFamily(userSettings.fontStyle));
                        addCCProperty(output, "textFontSize", percentagefontSize(userSettings.fontSize));
                        addCCProperty(output, "textEdgeAttribute", textEdgeShadow(userSettings.fontEdgeAttribute));
                        addCCProperty(output, "textColor", rgb(userSettings.fontColor));
                        addCCProperty(output, "textOpacity", opacity(userSettings.fontColor));
                        addCCProperty(output, "textBackgroundColor", rgb(userSettings.backgroundColor));
                        addCCProperty(output, "textBackgroundOpacity", opacity(userSettings.backgroundColor))
                    }
                    return output
                };
                var baseApplySettings = App2UserSettingsPlatform.prototype.applySettings;
                App2UserSettingsPlatform.prototype.applySettings = function(ccContainer) {
                    if (this.currentSignature !== app2SettingsSignature())
                        this.loadAndUpdateClosedCaptionStyleSettings();
                    baseApplySettings.bind(this)(ccContainer)
                };
                App2UserSettingsPlatform.prototype.isFontUsedForSmallCaps = function app2_isFontUsedForSmallCaps(fontName) {
                    return fontName === app2CCFontFamilyConverter[MS.Entertainment.Platform.ClosedCaptions.Meta.FontStyle.smallCapitals]
                };
                var ccUserSettingsImplementation = null;
                return function getUserSettingsPlatform() {
                        if (!ccUserSettingsImplementation)
                            try {
                                var testUserSettingsCapability = MS.Entertainment.Platform.ClosedCaptions.ClosedCaptionProperties;
                                ccUserSettingsImplementation = new App2UserSettingsPlatform
                            }
                            catch(ex) {
                                ccUserSettingsImplementation = new BaseUserSettingsPlatform
                            }
                        return ccUserSettingsImplementation
                    }
            })()
    });
    WinJS.Namespace.defineWithParent(MS.Entertainment.Platform.Playback, "ClosedCaptions", {Renderer: MS.Entertainment.UI.Framework.derive("MS.Entertainment.Platform.Playback.ClosedCaptions.RendererObservables", function Renderer_ctor(playbackControl, rendererContainer) {
            if (!playbackControl || !rendererContainer)
                return;
            MSEPlatform.Playback.ClosedCaptions.RendererObservables.prototype.constructor.call(this);
            WinJS.Utilities.addClass(rendererContainer, "hideFromDisplay");
            this._iPlaybackControl = playbackControl;
            this._ccContainer = rendererContainer;
            this._initialize(rendererContainer)
        }, {
            closedCaptionsOn: {
                set: function Renderer_closedCaptionsOn_set(value) {
                    MSEPlatform.Playback.Etw.traceClosedCaptionsOn(value);
                    if (value !== this._closedCaptionsOn)
                        this._onToggleCC(value)
                }, get: function Renderer_closedCaptionsOn_get() {
                        return this._closedCaptionsOn
                    }
            }, _closedCaptionsOn: false, _iPlaybackControl: null, _ccPresenter: null, _ttmlProcessor: null, _ccContainer: null, _ttmlFilepath: String.empty, _ttmlFileLoaded: false, _currentMedia: null, _currentMediaInstance: null, _renderingPaused: true, _playbackControlBindings: null, _resizeCallback: null, _processorLeadTime: 1000, _currentGeneratedAt: 0, _currentValidUntil: 0, _initialize: function Renderer_initialize(rendererContainer) {
                    if (!this._iPlaybackControl)
                        return;
                    if (MS.Entertainment.Utilities.isVideoApp || MS.Entertainment.Utilities.isMusicApp2) {
                        this._ttmlProcessor = new Microsoft.Entertainment.ClosedCaptions.TTMLProcessor;
                        this._ccPresenter = new MSEPlatform.Playback.ClosedCaptions.Presenter(this, rendererContainer);
                        if (this._closedCaptionsOn)
                            this._bindToPlaybackControl();
                        this.bind("ccLcid", this._onLcidChanged.bind(this))
                    }
                    else
                        this._closedCaptionsOn = false
                }, _bindToPlaybackControl: function Renderer_bindToPlaybackControl() {
                    this._playbackControlBindings = WinJS.Binding.bind(this._iPlaybackControl, {
                        currentMedia: this._onMediaChanged.bind(this), videoHeight: this._onVideoSizeChanged.bind(this), currentPosition: this._onPositionChanged.bind(this), currentTransportState: this._onTransportStateChanged.bind(this), seekedPosition: this._onSeeked.bind(this)
                    });
                    this._resizeCallback = this._setupRenderingSurface.bind(this);
                    MS.Entertainment.Utilities.attachResizeEvent(this._ccContainer, this._resizeCallback)
                }, _unbindFromPlaybackControl: function Renderer_unbindFromPlaybackControl() {
                    if (this._playbackControlBindings) {
                        this._playbackControlBindings.cancel();
                        this._playbackControlBindings = null
                    }
                    MS.Entertainment.Utilities.detachResizeEvent(this._ccContainer, this._resizeCallback)
                }, _reset: function Renderer_reset() {
                    this._ttmlFileLoaded = false;
                    this._ccPresenter.flush();
                    this._invalidateRenderingTimeWindow()
                }, _onToggleCC: function Renderer_onToggleCC(activate) {
                    this._closedCaptionsOn = activate;
                    if (activate) {
                        WinJS.Utilities.removeClass(this._ccContainer, "hideFromDisplay");
                        this._iPlaybackControl.enableTimeUpdate();
                        this._bindToPlaybackControl()
                    }
                    else {
                        WinJS.Utilities.addClass(this._ccContainer, "hideFromDisplay");
                        this._stopRendering();
                        this._unbindFromPlaybackControl()
                    }
                }, _onLcidChanged: function Renderer_onLcidChanged(newLcid, oldLcid) {
                    if (!oldLcid || !newLcid || !this._closedCaptionsOn)
                        return;
                    var currentMediaInstance = this._currentMediaInstance;
                    if (currentMediaInstance) {
                        this._ttmlFileLoaded = false;
                        this._onMediaChanged(currentMediaInstance)
                    }
                }, _onMediaChanged: function Renderer_onMediaChanged(newMedia) {
                    if (!newMedia)
                        return;
                    if (!this._closedCaptionsOn)
                        return;
                    if (newMedia.isEqual(this._currentMediaInstance) && this._ttmlFileLoaded) {
                        this._invalidateRenderingTimeWindow();
                        this._startRendering()
                    }
                    else {
                        this._reset();
                        this._currentMediaInstance = newMedia;
                        this._currentMedia = null;
                        if (newMedia._mediaItem)
                            this._currentMedia = newMedia._mediaItem.data;
                        this._getTTMLFilepath().then(function _gotTTMLFile(filePath) {
                            this._loadTTMLFile(filePath).then(function _loadedTTMLFile() {
                                this._startRendering()
                            }.bind(this), function _cantLoadTTMLFile(error){}.bind(this))
                        }.bind(this), function _noTTMLFile(error){}.bind(this))
                    }
                    this._setupRenderingSurface()
                }, _onVideoSizeChanged: function Renderer_onVideoSizeChanged(videoHeight) {
                    this._setupRenderingSurface()
                }, _onPositionChanged: function Renderer_onPositionChanged(playbackPosition) {
                    if (!this._closedCaptionsOn || !this._ttmlFileLoaded)
                        return;
                    this._checkAndProcessNextClosedCaptionsBlob(playbackPosition)
                }, _onTransportStateChanged: function Renderer_onTransportStateChanged(newTS) {
                    if (!this._closedCaptionsOn || !this._ttmlFileLoaded)
                        return;
                    switch (newTS) {
                        case MSEPlatform.Playback.TransportState.playing:
                            WinJS.Promise.timeout(500).then(function onDelayedPlaying() {
                                this._setupRenderingSurface()
                            }.bind(this));
                            this._startRendering();
                            break;
                        case MSEPlatform.Playback.TransportState.paused:
                            this._stopRendering();
                            break;
                        case MSEPlatform.Playback.TransportState.stopped:
                            this._stopRendering();
                            break
                    }
                }, _onSeeked: function Renderer_onSeeked(newPosition) {
                    if (!this._closedCaptionsOn || !this._ttmlFileLoaded)
                        return;
                    this._ccPresenter.flush();
                    this._invalidateRenderingTimeWindow();
                    this._renderingPaused = false;
                    this._checkAndProcessNextClosedCaptionsBlob(this._iPlaybackControl.currentPosition);
                    this._ccPresenter._presenterLoop()
                }, _setupRenderingSurface: function _setupRenderingSurface() {
                    var surfaceHeight = 0;
                    var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                    if (this._iPlaybackControl.videoWidth && this._iPlaybackControl.videoHeight) {
                        surfaceHeight = Math.floor(this._iPlaybackControl.videoHeight * (this._ccContainer.clientWidth / this._iPlaybackControl.videoWidth));
                        if (surfaceHeight && this._ccContainer.parentElement && this._ccContainer.parentElement.clientHeight) {
                            surfaceHeight = Math.min(surfaceHeight, this._ccContainer.parentElement.clientHeight);
                            this._ccContainer.style.height = surfaceHeight + "px";
                            this._ccContainer.style.top = ((this._ccContainer.parentElement.clientHeight - surfaceHeight) >> 1) + "px";
                            this._ccContainer.style.fontSize = Math.floor(surfaceHeight * 0.05) + "px"
                        }
                    }
                    if (uiStateService.isSnapped || surfaceHeight < 240) {
                        this._stopRendering();
                        this._ccPresenter.flush()
                    }
                    else
                        this._startRendering()
                }, _invalidateRenderingTimeWindow: function Renderer_invalidateRenderingTimeWindow() {
                    this._currentGeneratedAt = 0;
                    this._currentValidUntil = 0
                }, _findClosedCaptionsUrl: function Renderer_findClosedCaptionsUrl(miid) {
                    var ccInfo = null;
                    if (miid && this._currentMedia && this._currentMedia.closedCaptionFiles) {
                        var ccFiles = this._currentMedia.closedCaptionFiles;
                        var mediaInstanceId = null;
                        var lcid = null;
                        miid = miid.replace(/{/g, '');
                        miid = miid.replace(/}/g, '');
                        for (var i = 0; i < ccFiles.length; i++) {
                            mediaInstanceId = ccFiles[i].mediaInstanceId;
                            lcid = ccFiles[i].lcid;
                            if (mediaInstanceId) {
                                mediaInstanceId = mediaInstanceId.replace(/{/g, '');
                                mediaInstanceId = mediaInstanceId.replace(/}/g, '');
                                if (mediaInstanceId.toLowerCase() === miid && (!this.ccLcid || +lcid === +(this.ccLcid))) {
                                    ccInfo = {
                                        url: ccFiles[i].fileUri, name: ccFiles[i].name
                                    };
                                    break
                                }
                            }
                        }
                    }
                    return ccInfo
                }, _getTTMLFilepath: function Renderer_getTTMLFilepath() {
                    var smid = (this._currentMedia ? this._currentMedia.zuneId || this._currentMedia.serviceId : null);
                    var miid = (this._currentMediaInstance ? (this._currentMediaInstance.mediaInstanceId ? this._currentMediaInstance.mediaInstanceId.toLowerCase() : null) : null);
                    var url = String.empty;
                    var name = String.empty;
                    var ccInfo = this._findClosedCaptionsUrl(miid);
                    if (ccInfo) {
                        url = ccInfo.url;
                        name = ccInfo.name
                    }
                    MSEPlatform.Playback.Etw.traceCCRendererGetTTMLFilepath("begin", smid, miid, this.ccLcid, url);
                    if (smid && miid && url && name)
                        return new WinJS.Promise(function _getTTMLFilePromise(c, e, p) {
                                Microsoft.Entertainment.ClosedCaptionDownloader.getClosedCaptionFileAsync(url, smid, miid, name, this.ccLcid).then(function _getTTMLFile_success(path) {
                                    MSEPlatform.Playback.Etw.traceCCRendererGetTTMLFilepath("end", smid, miid, this.ccLcid, path);
                                    c(path)
                                }.bind(this), function _getTTMLFile_failed(error) {
                                    MSEPlatform.Playback.Etw.traceCCRendererGetTTMLFilepath("failed: " + error, smid, miid, this.ccLcid, url);
                                    e(error)
                                }.bind(this))
                            }.bind(this));
                    else {
                        MSEPlatform.Playback.Etw.traceCCRendererGetTTMLFilepath("failed: E_INVALIDARGS", smid, miid, this.ccLcid, url);
                        return WinJS.Promise.wrapError("E_INVALIDARGS")
                    }
                }, _ttmlProcessorLoadAsync: function _ttmlProcessorLoad(storageFile) {
                    if (!MS.Entertainment.Utilities.isApp2)
                        return this._ttmlProcessor.loadFromStorageFile(storageFile);
                    else
                        return Windows.Storage.FileIO.readTextAsync(storageFile).then(function loadTTMLAsString(ttmlAsString) {
                                try {
                                    this._ttmlProcessor.loadFromString(ttmlAsString)
                                }
                                catch(ex) {
                                    return WinJS.Promise.wrapError("_ttmlProcessor.loadFromString failed with " + ex)
                                }
                            }.bind(this))
                }, _loadTTMLFile: function Renderer_loadTTMLFile(filePath) {
                    if (!filePath)
                        return WinJS.Promise.wrapError("Renderer_loadTTMLFile: Error! Empty filePath.");
                    MSEPlatform.Playback.Etw.traceCCRendererLoadTTMLFile("begin", filePath);
                    return new WinJS.Promise(function _loadTTMLPromise(c, e, p) {
                            try {
                                Windows.Storage.StorageFile.getFileFromPathAsync(filePath).then(function _gotFile(storageFile) {
                                    this._ttmlProcessorLoadAsync(storageFile).then(function _loadTTMLFile_success() {
                                        MSEPlatform.Playback.Etw.traceCCRendererLoadTTMLFile("end", filePath);
                                        this._ttmlFilepath = filePath;
                                        this._ttmlFileLoaded = true;
                                        c()
                                    }.bind(this), function _loadTTMLFile_failed(errorFromLoad) {
                                        MSEPlatform.Playback.Etw.traceCCRendererLoadTTMLFile("failed: " + errorFromLoad, filePath);
                                        e(errorFromLoad)
                                    }.bind(this), function _loadTTMLFile_progress(progress) {
                                        MSEPlatform.Playback.Etw.traceCCRendererLoadTTMLFile("progress : " + progress, filePath)
                                    }.bind(this))
                                }.bind(this), function _noFileFromStorage(errorFromStorage) {
                                    MSEPlatform.Playback.Etw.traceCCRendererLoadTTMLFile("failed: " + errorFromStorage, filePath);
                                    e(errorFromStorage)
                                }.bind(this))
                            }
                            catch(ex) {
                                MSEPlatform.Playback.Etw.traceCCRendererLoadTTMLFile("failed: " + ex, filePath);
                                e(ex)
                            }
                        }.bind(this))
                }, _getHtmlBlob: function Renderer_getHtmlBlob(msecPosition) {
                    var ttmlOutput = null;
                    try {
                        ttmlOutput = this._ttmlProcessor.renderAt(msecPosition)
                    }
                    catch(ex) {}
                    return (ttmlOutput ? JSON.parse(ttmlOutput) : null)
                }, _applyUserSettings: function Renderer_applyUserSettings(htmlBlob) {
                    return htmlBlob
                }, _startRendering: function Renderer_startRendering() {
                    if (this._renderingPaused) {
                        MSEPlatform.Playback.Etw.traceCCEnterRenderingLoop(this._iPlaybackControl.currentPosition, this._iPlaybackControl.currentTransportState);
                        this._renderingPaused = false;
                        this._ccPresenter.start()
                    }
                }, _stopRendering: function Renderer_stopRendering() {
                    MSEPlatform.Playback.Etw.traceCCExitRenderingLoop(this._iPlaybackControl.currentPosition, this._iPlaybackControl.currentTransportState, this._closedCaptionsOn);
                    this._renderingPaused = true;
                    this._currentValidUntil = 0;
                    this._ccPresenter.stop()
                }, _checkAndProcessNextClosedCaptionsBlob: function Renderer_checkAndProcessNextClosedCaptionsBlob(currentPlaybackPosition) {
                    if (this._ttmlFileLoaded && !this._renderingPaused)
                        while (currentPlaybackPosition + this._processorLeadTime >= this._currentValidUntil) {
                            var requestTime = (this._currentValidUntil === 0 ? currentPlaybackPosition : this._currentValidUntil + 1);
                            var blob = this._getHtmlBlob(requestTime);
                            if (blob) {
                                this._currentGeneratedAt = blob.generatedAt;
                                this._currentValidUntil = blob.validUntil;
                                var finalHtml = this._applyUserSettings(blob.html);
                                this._ccPresenter.presentAt(this._currentGeneratedAt, this._currentValidUntil, finalHtml)
                            }
                            else
                                break
                        }
                }
        }, {
            _closedCaptionsStyleSheetId: "CCStyleSheet", loadAndUpdateClosedCaptionStyleSettings: function loadAndUpdateClosedCaptionStyleSettings() {
                    MS.Entertainment.Platform.Playback.ClosedCaptions.getUserSettings().loadAndUpdateClosedCaptionStyleSettings()
                }, loadClosedCaptionStyleSettings: function loadAndUpdateClosedCaptionStyleSettings() {
                    return MS.Entertainment.Platform.Playback.ClosedCaptions.getUserSettings().loadClosedCaptionStyleSettings()
                }, saveClosedCaptionStyleSettings: function saveClosedCaptionStyleSettings(ccSettings) {
                    MS.Entertainment.Platform.Playback.ClosedCaptions.getUserSettings().saveClosedCaptionStyleSettings(ccSettings)
                }, updateClosedCaptionStyleSettings: function updateClosedCaptionStyleSettings(ccSettings, isPreviewOnly) {
                    MS.Entertainment.Platform.Playback.ClosedCaptions.getUserSettings().updateClosedCaptionStyleSettings(ccSettings, isPreviewOnly)
                }
        })})
})()
