﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Platform.Playback");
    WinJS.Namespace.define("MS.Entertainment.Platform.Playback", {ConcurrentStreamingRestriction: MS.Entertainment.UI.Framework.define(function ConcurrentStreamingRestriction_constructor(playbackControl) {
            this._playbackControl = playbackControl;
            this._observe(playbackControl)
        }, {
            _playbackControl: null, _transportState: null, _currentMedia: null, _concurrentStreamingRestrictionModel: null, _mapStartToResume: false, _getPassportTicketPromise: null, _playToSenderConnectionState: Windows.Media.PlayTo.PlayToConnectionState.disconnected, _ticket: null, _concurrentStreamingRestrictionModelInitialized: false, _observe: function observe(playbackControl) {
                    var that = this;
                    this._playbackControl = playbackControl;
                    playbackControl.bind("currentMedia", function(newMediaInstance) {
                        that._onCurrentMediaChanged(newMediaInstance)
                    });
                    playbackControl.bind("currentTransportState", function(newTransportState) {
                        that._onCurrentTransportStateChanged(newTransportState)
                    });
                    playbackControl.bind("playToSenderConnectionState", function(newPlayToSenderConnectionState) {
                        that._onCurrentDlnaStateChanged(newPlayToSenderConnectionState)
                    })
                }, _onCurrentMediaChanged: function onCurrentMediaChanged(newMediaInstance) {
                    this._concurrentStreamingRestrictionModel = null;
                    this._transportState = null;
                    this._currentMedia = null;
                    this._mapStartToResume = false;
                    var that = this;
                    if (this._getPassportTicketPromise) {
                        this._getPassportTicketPromise.cancel();
                        this._getPassportTicketPromise = null
                    }
                    function onBlockStreaming(error) {
                        that._blockStreaming(that, error)
                    }
                    if (newMediaInstance) {
                        this._currentMedia = newMediaInstance;
                        if (this._shouldEnforceVideoConcurrentStreamingRestriction(newMediaInstance)) {
                            this._concurrentStreamingRestrictionModel = new Microsoft.Entertainment.Util.ConcurrentStreamingRestriction;
                            this._concurrentStreamingRestrictionModelInitialized = false;
                            this._concurrentStreamingRestrictionModel.addEventListener("blockstreaming", onBlockStreaming);
                            var signInProvider = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                            this._getPassportTicketPromise = signInProvider.getPassportTicket(MS.Entertainment.Utilities.SignIn.TicketType.MBI_SSL, MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_Passport)).then(function getPassportTicketComplete(ticket) {
                                that._ticket = ticket;
                                if (that._transportState !== null)
                                    that._reportStreamingState();
                                that._getPassportTicketPromise = null
                            }, function getPassportTicketError(error) {
                                that._blockStreaming(that, error);
                                that._getPassportTicketPromise = null
                            })
                        }
                    }
                }, _onCurrentTransportStateChanged: function onCurrentTransportStateChanged(newTransportState) {
                    this._transportState = newTransportState;
                    if (this._transportState === MS.Entertainment.Platform.Playback.TransportState.playing && this._concurrentStreamingRestrictionModel && (!this._concurrentStreamingRestrictionModelInitialized)) {
                        this._concurrentStreamingRestrictionModel.initialize(this._ticket, this._currentMedia.initialLease ? this._currentMedia.initialLease.status : "", this._currentMedia.initialLease ? this._currentMedia.initialLease.url : "", this._currentMedia.initialLease ? this._currentMedia.initialLease.pollAt : 0, this._currentMedia.mediaInstanceId, this._currentMedia.offerId, this._currentMedia.nativeLicenseRight);
                        this._concurrentStreamingRestrictionModelInitialized = true
                    }
                    this._reportStreamingState()
                }, _onCurrentDlnaStateChanged: function _onCurrentDlnaStateChanged(newPlayToSenderConnectionState) {
                    if (this._playToSenderConnectionState !== newPlayToSenderConnectionState) {
                        this._playToSenderConnectionState = newPlayToSenderConnectionState;
                        switch (newPlayToSenderConnectionState) {
                            case Windows.Media.PlayTo.PlayToConnectionState.connected:
                                break;
                            case Windows.Media.PlayTo.PlayToConnectionState.rendering:
                                if (this._concurrentStreamingRestrictionModel && this._concurrentStreamingRestrictionModelInitialized) {
                                    this._concurrentStreamingRestrictionModel.reportStreamingAction(Microsoft.Entertainment.Util.EStreamingActionType.eStreamingActionTypeStop, this._currentMedia.mediaInstanceId);
                                    this._concurrentStreamingRestrictionModel = null;
                                    this._concurrentStreamingRestrictionModelInitialized = false
                                }
                                break;
                            case Windows.Media.PlayTo.PlayToConnectionState.disconnected:
                                if (this._currentMedia && this._shouldEnforceVideoConcurrentStreamingRestriction(this._currentMedia)) {
                                    this._onCurrentMediaChanged(this._currentMedia);
                                    if (this._currentMedia.initialLease && this._concurrentStreamingRestrictionModel && !this._concurrentStreamingRestrictionModelInitialized) {
                                        this._concurrentStreamingRestrictionModel.initialize(this._ticket, this._currentMedia.initialLease ? this._currentMedia.initialLease.status : "", this._currentMedia.initialLease ? this._currentMedia.initialLease.url : "", this._currentMedia.initialLease ? this._currentMedia.initialLease.pollAt : 0, this._currentMedia.mediaInstanceId, this._currentMedia.offerId, this._currentMedia.nativeLicenseRight);
                                        this._concurrentStreamingRestrictionModelInitialized = true
                                    }
                                }
                                break;
                            default:
                                MS.Entertainment.UI.Controls.assert(false, "unknown playto sender state: " + newPlayToSenderConnectionState.toString());
                                break
                        }
                    }
                }, _reportStreamingState: function reportStreamingState() {
                    if (this._concurrentStreamingRestrictionModel && this._concurrentStreamingRestrictionModelInitialized && this._playToSenderConnectionState === Windows.Media.PlayTo.PlayToConnectionState.disconnected) {
                        var reportState = null;
                        switch (this._transportState) {
                            case MS.Entertainment.Platform.Playback.TransportState.playing:
                                if (this._mapStartToResume)
                                    reportState = Microsoft.Entertainment.Util.EStreamingActionType.eStreamingActionTypeResume;
                                else {
                                    reportState = Microsoft.Entertainment.Util.EStreamingActionType.eStreamingActionTypeStart;
                                    this._mapStartToResume = true
                                }
                                break;
                            case MS.Entertainment.Platform.Playback.TransportState.paused:
                                reportState = Microsoft.Entertainment.Util.EStreamingActionType.eStreamingActionTypePause;
                                break;
                            case MS.Entertainment.Platform.Playback.TransportState.stopped:
                                reportState = Microsoft.Entertainment.Util.EStreamingActionType.eStreamingActionTypeStop;
                                break;
                            default:
                                break
                        }
                        if (reportState !== null) {
                            var reportGuid = MS.Entertainment.Utilities.EMPTY_GUID;
                            if (this._currentMedia.hasOwnProperty("mediaInstanceId"))
                                reportGuid = this._currentMedia.mediaInstanceId;
                            this._concurrentStreamingRestrictionModel.reportStreamingAction(reportState, reportGuid)
                        }
                    }
                }, _blockStreaming: function _blockStreaming(context, errorCode)
                {
                    var error = {
                            code: parseInt(errorCode), innerError: errorCode
                        };
                    if (isNaN(error.code))
                        error = errorCode;
                    context._playbackControl.forceError(error)
                }, _shouldEnforceVideoConcurrentStreamingRestriction: function _shouldEnforceVideoConcurrentStreamingRestriction(mediaInstance) {
                    if (mediaInstance && mediaInstance.isVideo() && mediaInstance.hasOwnProperty("protectionState") && mediaInstance.protectionState === MS.Entertainment.Platform.Playback.ProtectionState.drmProtected && this._playToSenderConnectionState === Windows.Media.PlayTo.PlayToConnectionState.disconnected) {
                        var uri = new Windows.Foundation.Uri(mediaInstance.source);
                        if (uri.schemeName.toLocaleLowerCase() === "http" || uri.schemeName.toLocaleLowerCase() === "https")
                            if (mediaInstance.hasOwnProperty("mediaInstanceId") && mediaInstance.mediaInstanceId !== null && mediaInstance.mediaInstanceId !== undefined)
                                return true;
                            else {
                                MS.Entertainment.Platform.Playback.assert(false, "drmProtected content must have a media instance id");
                                this._blockStreaming(this, -1072885538);
                                return true
                            }
                    }
                    return false
                }
        }, {})})
})()
