﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Platform.Playback");
(function() {
    "use strict";
    var MSEPlatform = WinJS.Namespace.define("MS.Entertainment.Platform", null);
    function getPassportTicketAsync(returnEmptyOnFailure) {
        if (!MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.sessionManager))
            return WinJS.Promise.wrapError("No SessionMgr is registered");
        var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
        return sessionMgr.getPassportTicketAsync(returnEmptyOnFailure)
    }
    function isBlobUrl(url) {
        var blobUrlRegex = /^blob:/i;
        return blobUrlRegex.test(url)
    }
    WinJS.Namespace.defineWithParent(MSEPlatform, "Playback", {
        MediaPlayer: MS.Entertainment.UI.Framework.define(function MediaPlayer_constructor(playerType, mediaInstance, autoPlay) {
            if (!this._tagPlayer)
                this._tagPlayer = this._createHtmlTag(playerType, mediaInstance, autoPlay);
            this._autoPlay = autoPlay;
            this._media = mediaInstance
        }, {
            autoplay: {
                get: function autoplay_get() {
                    return this._autoPlay
                }, set: function autoplay_set(value) {
                        this._autoPlay = value
                    }
            }, muted: {
                    get: function muted_get() {
                        return this._tagPlayer.muted
                    }, set: function muted_set(value) {
                            this._tagPlayer.muted = value
                        }
                }, volume: {
                    get: function volume_get() {
                        return this._tagPlayer.volume
                    }, set: function volume_set(value) {
                            this._tagPlayer.volume = value
                        }
                }, currentTime: {
                    get: function currentTime_get() {
                        return this._tagPlayer.currentTime
                    }, set: function currentTime_set(value) {
                            this._tagPlayer.currentTime = value
                        }
                }, duration: {get: function duration_get() {
                        return this._tagPlayer.duration
                    }}, error: {get: function error_get() {
                        return this._tagPlayer.error
                    }}, media: {get: function media_get() {
                        return this._media
                    }}, playerType: {get: function playerType_get() {
                        return this._tagPlayer.tagName
                    }}, buffered: {get: function buffered_get() {
                        return this._tagPlayer.buffered
                    }}, ended: {get: function ended_get() {
                        return this._tagPlayer.ended
                    }}, isDisposed: {get: function isDisposed_get() {
                        return this._disposed
                    }}, msPlayToDisabled: {get: function msPlayToDisabled_get() {
                        return this._tagPlayer.msPlayToDisabled
                    }}, msPlayToSource: {get: function msPlayToSource_get() {
                        return this._tagPlayer.msPlayToSource
                    }}, play: function MediaPlayer_play() {
                    MSEPlatform.Playback.Etw.traceString("MediaPlayer::Play()");
                    this._tagPlayer.play()
                }, pause: function MediaPlayer_pause() {
                    MSEPlatform.Playback.Etw.traceString("MediaPlayer::Pause()");
                    this._tagPlayer.pause()
                }, stop: function MediaPlayer_stop() {
                    this.pause()
                }, seekToPosition: function MediaPlayer_seekToPosition(positionMsec) {
                    try {
                        var positionSec = positionMsec / 1000;
                        this._tagPlayer.currentTime = positionSec
                    }
                    catch(e) {
                        MSEPlatform.Playback.Etw.tracePlaybackError(e.code, e.msExtendedCode, "MediaPlayer::seekToPosition")
                    }
                }, forceTimeUpdate: function MediaPlayer_forceTimeUpdate() {
                    return this._tagPlayer.currentTime * 1000
                }, addEventListener: function MediaPlayer_addEventListener(type, listener, useCapture) {
                    this._tagPlayer.addEventListener(type, listener, useCapture)
                }, removeEventListener: function MediaPlayer_removeEventListener(type, listener, useCapture) {
                    this._tagPlayer.removeEventListener(type, listener, useCapture)
                }, getElement: function MediaPlayer_getElement() {
                    return this._tagPlayer
                }, getMediaDurationMS: function MediaPlayer_getMediaDurationMS() {
                    function getDurationMS(duration) {
                        if (duration) {
                            if (duration.getMinutes)
                                duration = ((duration.getHours() * 60 * 60) + (duration.getMinutes() * 60) + duration.getSeconds()) * 1000
                        }
                        else
                            duration = 0;
                        return duration
                    }
                    var duration = null;
                    if (this._media && this._media._mediaItem && this._media._mediaItem.data)
                        duration = this._media._mediaItem.data.duration;
                    return getDurationMS(duration)
                }, dispose: function MediaPlayer_dispose() {
                    MSEPlatform.Playback.Etw.traceString("MediaPlayer::dispose: Disposing tag");
                    if (this._tagPlayer._mediaProtectionManagerBindings) {
                        this._tagPlayer._mediaProtectionManagerBindings.cancel();
                        this._tagPlayer._mediaProtectionManagerBindings = null
                    }
                    this._tagPlayer.removeAttribute("src");
                    this._tagPlayer.load();
                    this._media = null;
                    this._disposed = true
                }, configureDRM: function MediaPlayer_configureDRM() {
                    var that = this;
                    var htmlTag = this._tagPlayer;
                    var mediaInstance = this._media;
                    var Playback = MSEPlatform.Playback;
                    var licenseLog = "";
                    var startLicenseAcquisitionTime = new Date;
                    MSEPlatform.Playback.assert(mediaInstance, "mediaInstance should not be null");
                    function getKidFromServiceRequest(e) {
                        return e && e.request && e.request.contentHeader ? e.request.contentHeader.keyIdString : String.empty
                    }
                    function serviceRequested(e) {
                        function onPlaySRCompleted(asyncOp) {
                            if (e && e.completion && e.completion.complete)
                                e.completion.complete(true);
                            MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "License acquisition succeeded");
                            var elapsedTime = new Date - startLicenseAcquisitionTime;
                            Playback.Etw.traceString("DRM:LA Time Elapsed (ms) : " + elapsedTime);
                            licenseLog = "";
                            if (Playback.AudioPlayer.fastStartBlockedOnLA)
                                Playback.AudioPlayer.fastStartBlockedOnLA = false;
                            if (mediaInstance.mediaType === Microsoft.Entertainment.Queries.ObjectType.track && !mediaInstance.trackLeafLicenseAcquired && mediaInstance.isLocal)
                                mediaInstance.trackLeafLicenseAcquired = true
                        }
                        function onPlaySRError(error) {
                            var errorCode = (error && error.number) ? error.number : "unknown";
                            Playback.Etw.traceString("DRM:onPlaySRError: " + errorCode);
                            MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "License acquisition failed with error " + errorCode);
                            var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                            if (error && error.number === MSEPlatform.Playback.Error.NS_E_DRM_INVALID_LICENSE.code && (mediaInstance.mediaType === Microsoft.Entertainment.Queries.ObjectType.track) && signedInUser && signedInUser.isSubscription && mediaInstance.isLocal) {
                                Playback.Etw.traceString("DRM:onPlaySRError: retry root LA for subscription downloaded content");
                                getPassportTicketAsync().then(function _gotTicket(ticket) {
                                    MSEPlatform.Playback.drmIndividualizationPromise.then(function _doAcquireRootLicense() {
                                        Microsoft.Entertainment.Util.PlayReadyHandler.acquireRootLicense(ticket)
                                    })
                                }).then(function _rootSucceeded() {
                                    Playback.Etw.traceString("DRM:onPlaySRError: root LA retry SUCCEEDED");
                                    MS.Entertainment.Utilities.Telemetry.logRootLicenseAcquisition("succeeded");
                                    if (e && e.completion && e.completion.complete)
                                        e.completion.complete(true)
                                }, function _rootFailed(error) {
                                    Playback.Etw.traceString("DRM:onPlaySRError: root LA retry FAILED: " + errorCode);
                                    MS.Entertainment.Utilities.Telemetry.logRootLicenseAcquisition("failed", errorCode);
                                    that._drmNotifyError(error, "DRM ServiceRequest Error (Root LA Retry): " + licenseLog, e)
                                })
                            }
                            else if (error && error.number === MSEPlatform.Playback.Error.MF_E_UNSUPPORTED_RATE.code && mediaInstance.mediaType !== Microsoft.Entertainment.Queries.ObjectType.track && mediaInstance.smidEnablerInEffect && mediaInstance.isLocal)
                                that._drmNotifyError(MSEPlatform.Playback.Error.ZEST_E_ASSET_LICENSE_RIGHT_NOT_OWNED, "Protected Video Content SMID is not owned. " + licenseLog, e);
                            else
                                that._drmNotifyError(error, "DRM ServiceRequest Error: " + licenseLog, e)
                        }
                        function onPlaySRProgress(report) {
                            if (report.indexOf("lease") === -1) {
                                var elapsedTime = new Date - startLicenseAcquisitionTime;
                                Playback.Etw.traceString("DRM:" + elapsedTime + ":" + report);
                                licenseLog += elapsedTime + " : " + report + "\n"
                            }
                            else
                                try {
                                    var leaseObject = JSON.parse(report);
                                    mediaInstance.initialLease = leaseObject.lease
                                }
                                catch(e) {
                                    Playback.Etw.traceString("Exception thrown when parsing the lease: " + report)
                                }
                        }
                        try {
                            if (!mediaInstance)
                                return;
                            if (e.request.type === Microsoft.Media.PlayReadyClient.PlayReadyStatics.individualizationServiceRequestType) {
                                MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, String.empty, "Individualization Requested");
                                e.request.beginServiceRequest().then(function indiv_complete() {
                                    MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, String.empty, "Individualization Successful");
                                    try {
                                        var next = e.request.nextServiceRequest();
                                        if (next) {
                                            e.request = next;
                                            serviceRequested(e)
                                        }
                                        else
                                            onPlaySRCompleted()
                                    }
                                    catch(ex) {
                                        MSEPlatform.Playback.MediaPlayer.drmNotifyException(htmlTag, ex, "DRM Individualization Service Request")
                                    }
                                }, function indiv_error() {
                                    MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, String.empty, "Individualization Failed");
                                    that._drmNotifyError(null, MSEPlatform.Playback.Error.NS_E_DRM_NEEDS_INDIVIDUALIZATION.name, e)
                                });
                                return
                            }
                            MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "License Needed");
                            mediaInstance.fillDownloadSubscriptionInfoAsync().then(function fillDownloadSubscriptionInfoAsync_complete() {
                                if (!mediaInstance.nativeLicenseRight && mediaInstance.mediaType === Microsoft.Entertainment.Queries.ObjectType.track && mediaInstance.isLocal)
                                    mediaInstance.nativeLicenseRight = Microsoft.Entertainment.Marketplace.MarketplaceMediaRights.subscriptionDownload;
                                var right = MSEPlatform.PurchaseHelpers.LicenseRightMap.toScript(mediaInstance.nativeLicenseRight);
                                return getPassportTicketAsync(right === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.FreeStream).then(function execute_drm_serviceRequest(ticket) {
                                        var handler;
                                        try {
                                            if (!mediaInstance.mediaInstanceId || MS.Entertainment.Utilities.isEmptyGuid(mediaInstance.mediaInstanceId) || !right) {
                                                if (mediaInstance.mediaType === Microsoft.Entertainment.Queries.ObjectType.track) {
                                                    MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "Track has no MIID or no rights");
                                                    that._drmNotifyError(MSEPlatform.Playback.Error.ZUNE_E_NO_SUBSCRIPTION_DOWNLOAD_RIGHTS, "No rights to this content", e);
                                                    return
                                                }
                                                else if (!MS.Entertainment.Utilities.isEmptyGuid(mediaInstance.serviceIdSafe)) {
                                                    mediaInstance.smidEnablerInEffect = true;
                                                    handler = new Microsoft.Entertainment.Util.PlayReadyHandler(ticket, mediaInstance.shouldLogToDrmDownloadHistory, mediaInstance.serviceIdSafe);
                                                    MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "Attempting SMID enabler")
                                                }
                                                else if (mediaInstance._mediaItem && mediaInstance._mediaItem.data && mediaInstance._mediaItem.data.fileItem) {
                                                    MSEPlatform.PurchaseHelpers.getServiceIdAsync(mediaInstance._mediaItem.data.fileItem).done(function crackedSmid(smid) {
                                                        if (smid) {
                                                            mediaInstance.smidEnablerInEffect = true;
                                                            mediaInstance._mediaItem.data.zuneId = smid;
                                                            handler = new Microsoft.Entertainment.Util.PlayReadyHandler(ticket, mediaInstance.shouldLogToDrmDownloadHistory, smid);
                                                            handler.beginServiceRequest(e.request).then(onPlaySRCompleted, onPlaySRError, onPlaySRProgress)
                                                        }
                                                        else {
                                                            MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "Media has no SMID assigned");
                                                            that._drmNotifyError(MSEPlatform.Playback.Error.X8_E_PLAYBACK_PROTECTED_MEDIA_NOT_IDENTIFIED, "Media has no SMID assigned", e)
                                                        }
                                                    }, function cannotCrackSmid(error) {
                                                        MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "Cannot crack SMID" + error);
                                                        that._drmNotifyError(MSEPlatform.Playback.Error.X8_E_PLAYBACK_PROTECTED_MEDIA_NOT_IDENTIFIED, "Cannot crack SMID " + error, e)
                                                    });
                                                    return
                                                }
                                            }
                                            else if (mediaInstance.mediaType === Microsoft.Entertainment.Queries.ObjectType.track && mediaInstance.trackLeafLicenseAcquired) {
                                                Playback.Etw.traceString("DRM:onPlaySRError: retry root LA for subscription downloaded content");
                                                handler = new Microsoft.Entertainment.Util.PlayReadyHandler(ticket);
                                                MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "Attempting root subscription enabler")
                                            }
                                            else {
                                                handler = new Microsoft.Entertainment.Util.PlayReadyHandler(ticket, right, mediaInstance.shouldLogToDrmDownloadHistory, mediaInstance.offerId ? mediaInstance.offerId : MS.Entertainment.Utilities.EMPTY_GUID, mediaInstance.mediaInstanceId);
                                                MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "Attempting MIID+Offer enabler")
                                            }
                                            handler.beginServiceRequest(e.request).then(onPlaySRCompleted, onPlaySRError, onPlaySRProgress)
                                        }
                                        catch(ex) {
                                            MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "Unexpected exception thrown");
                                            MSEPlatform.Playback.MediaPlayer._drmNotifyException(that, ex, "License Acquisition Service Request")
                                        }
                                    }, function sign_in_failure(error) {
                                        MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "Sign in Failed");
                                        that._drmNotifyError(null, MSEPlatform.Playback.Error.ZEST_E_SIGNIN_REQUIRED.name, e)
                                    })
                            }, function fillDownloadSubscriptionInfoAsync_error(error) {
                                MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "Media Detail Query failed");
                                that._drmNotifyError(MSEPlatform.Playback.Error.X8_E_PLAYBACK_PROTECTED_MEDIA_NOT_IDENTIFIED, "Media has no SMID assigned", e)
                            }).done(null, function reactiveLicenseAcquisitionFailed(error) {
                                MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "Unexpected outer exception thrown");
                                that._drmNotifyError(error, "serviceRequested", e)
                            })
                        }
                        catch(ex) {
                            MS.Entertainment.Utilities.Telemetry.logReactiveLicenseAcquisitionProgress(mediaInstance, getKidFromServiceRequest(e), "Unexpected outer exception thrown");
                            MSEPlatform.Playback.MediaPlayer._drmNotifyException(that, ex, "serviceRequested")
                        }
                    }
                    function componentLoadFailed(e) {
                        var msg = "";
                        try {
                            msg += e.information.items.size + " failed components!\n";
                            msg += "Components:\n";
                            for (var i = 0; i < e.information.items.size; i++)
                                msg += e.information.items[i].name + "\nReasons=0x" + e.information.items[i].reasons + "\n" + "Renewal Id=" + e.information.items[i].renewalId + "\n";
                            e.completion.complete(true);
                            msg += "Resumed source (false)\n";
                            MSEPlatform.Playback.Etw.traceString(msg);
                            that._drmNotifyError(null, MSEPlatform.Playback.Error.NS_E_WMP_BAD_DRIVER.name)
                        }
                        catch(e) {
                            MSEPlatform.Playback.MediaPlayer._drmNotifyException(that, e, "componentLoadFailed")
                        }
                    }
                    var mediaProtectionManager = new Windows.Media.Protection.MediaProtectionManager;
                    mediaProtectionManager.properties["Windows.Media.Protection.MediaProtectionSystemId"] = "{F4637010-03C3-42CD-B932-B48ADF3A6A54}";
                    var copyProtectionSystems = new Windows.Foundation.Collections.PropertySet;
                    copyProtectionSystems["{F4637010-03C3-42CD-B932-B48ADF3A6A54}"] = "Microsoft.Media.PlayReadyClient.PlayReadyWinRTTrustedInput";
                    mediaProtectionManager.properties["Windows.Media.Protection.MediaProtectionSystemIdMapping"] = copyProtectionSystems;
                    if (MS.Entertainment.Utilities.isVideoApp || MS.Entertainment.Utilities.isMusicApp2)
                        mediaProtectionManager.properties["Windows.Media.Protection.MediaProtectionContainerGuid"] = "{9A04F079-9840-4286-AB92-E65BE0885F95}";
                    htmlTag._mediaProtectionManagerBindings = MS.Entertainment.Utilities.addEventHandlers(mediaProtectionManager, {
                        componentloadfailed: componentLoadFailed, servicerequested: serviceRequested
                    });
                    try {
                        htmlTag.msSetMediaProtectionManager(mediaProtectionManager)
                    }
                    catch(e) {
                        MS.Entertainment.Platform.Playback.fail("_configureTagForDRM: Unexpected exception on msSetMediaProtectionManager: " + e.message);
                        MSEPlatform.Playback.MediaPlayer._drmNotifyException(that, e, "configureTagForDRM_msSetMediaProtectionManager")
                    }
                }, _createHtmlTag: function MediaPlayer_createHtmlTag(playerType, mediaInstance, autoPlay) {
                    MSEPlatform.Playback.Etw.traceString("MediaPlayer::createHtmlTag: creating audio/video tag.  mediaInstance.source = " + mediaInstance.source);
                    var Playback = MSEPlatform.Playback;
                    var htmlTag = Playback.MediaPlayer.createMediaTag(playerType);
                    if (!htmlTag)
                        throw"MediaPlayer::createHtmlTag(): Error! cannot create " + playerType + " tag. Out of memory?";
                    if (playerType === "audio")
                        try {
                            htmlTag.setAttribute("msAudioCategory", "backgroundCapableMedia")
                        }
                        catch(ex) {
                            MSEPlatform.Playback.Etw.traceString("MediaPlayer::constructor: failed to set backgroundCapableMedia.  exception: " + ex)
                        }
                    else
                        htmlTag.setAttribute("msAudioCategory", "foregroundOnlyMedia");
                    htmlTag.msPlayToDisabled = Playback.shouldDisableDlnaPlayTo(mediaInstance);
                    Playback.Etw.traceString("MediaPlayer::createHtmlTag(), autoplay= " + autoPlay);
                    htmlTag.autoplay = autoPlay;
                    htmlTag.style.width = "100%";
                    htmlTag.style.height = "100%";
                    return htmlTag
                }, _drmNotifyError: function MediaPlayer_drmNotifyError(error, source, serviceRequest) {
                    var that = this;
                    var bubbleError = true;
                    function _sendError(code) {
                        MSEPlatform.Playback.MediaPlayer._fireTagPlaybackError(that, code, source);
                        if (serviceRequest && serviceRequest.completion && serviceRequest.completion.complete) {
                            that._skipThisError = true;
                            serviceRequest.completion.complete(false)
                        }
                    }
                    {};
                    if (!error) {
                        var code;
                        switch (source) {
                            case MSEPlatform.Playback.Error.NS_E_DRM_NEEDS_INDIVIDUALIZATION.name:
                                code = MSEPlatform.Playback.Error.NS_E_DRM_NEEDS_INDIVIDUALIZATION.code;
                                break;
                            case MSEPlatform.Playback.Error.NS_E_WMP_BAD_DRIVER.name:
                                code = MSEPlatform.Playback.Error.NS_E_WMP_BAD_DRIVER.code;
                                break;
                            case MSEPlatform.Playback.Error.ZEST_E_SIGNIN_REQUIRED.name:
                                code = MSEPlatform.Playback.Error.ZEST_E_SIGNIN_REQUIRED.code;
                                break;
                            default:
                                code = MSEPlatform.Playback.Error.NS_E_WMP_DRM_GENERIC_LICENSE_FAILURE.code;
                                break
                        }
                        {};
                        _sendError(code)
                    }
                    else {
                        var code = error.number ? error.number : error.code;
                        if (code === MSEPlatform.Playback.Error.ZEST_E_ASSET_LICENSE_RIGHT_NOT_OWNED.code)
                            if (this._media)
                                if (this._media._mediaItem && this._media._mediaItem.data && this._media._mediaItem.data["playFromXbox"]) {
                                    bubbleError = false;
                                    this._media._mediaItem.data["playPreviewOnly"] = true;
                                    WinJS.Promise.timeout().then(function tryPreview() {
                                        MSEPlatform.PlaybackHelpers.playMedia(that._media._mediaItem.data, true)
                                    })
                                }
                                else if (this._media.smidEnablerInEffect) {
                                    code = MSEPlatform.Playback.Error.X8_E_PLAYBACK_PROTECTED_MEDIA_NOT_IDENTIFIED.code;
                                    error.code = code
                                }
                        if (bubbleError)
                            _sendError(code)
                    }
                }, _setSourceFromMediaInstance: function MediaPlayer_setSourceFromMediaInstance() {
                    this._tagPlayer.src = this._media.source
                }, _autoplay: false, _tagPlayer: null, _media: null, _disposed: false
        }, {
            createMediaTag: function MediaPlayer_createMediaTag(type) {
                return document.createElement(type)
            }, _fireTagPlaybackError: function MediaPlayer_fireTagPlaybackError(player, error, context) {
                    MSEPlatform.Playback.Etw.traceString("MediaPlayer::_fireTagPlaybackError:  error = " + MSEPlatform.Playback.mediaErrorToString(error));
                    MSEPlatform.Playback.assert(player && player.firePlaybackError, "Fix misconfigured media tag error handler");
                    if (player && player.firePlaybackError)
                        player.firePlaybackError(error, context)
                }, _drmNotifyException: function MediaPlayer_drmNotifyException(player, ex, source) {
                    MSEPlatform.Playback.Etw.traceString("MediaPlayer::_drmNotifyException: " + JSON.stringify(ex) + " source: " + source);
                    WinJS.Promise.timeout().then(function notifyDrmException() {
                        MSEPlatform.Playback.MediaPlayer._fireTagPlaybackError(player, MSEPlatform.Playback.Error.NS_E_WMP_DRM_GENERIC_LICENSE_FAILURE.code, source)
                    })
                }
        }), AudioPlayer: MS.Entertainment.UI.Framework.derive("MS.Entertainment.Platform.Playback.MediaPlayer", function AudioPlayer_constructor(mediaInstance, autoPlay) {
                MSEPlatform.Playback.Etw.traceString("AudioPlayer::constructor: Creating audio player");
                this.base("audio", mediaInstance, autoPlay);
                if (!this._usingPreActivatedTag && mediaInstance.protectionState !== MSEPlatform.Playback.ProtectionState.unprotected)
                    this.configureDRM();
                if (this._usingFastStartTag) {
                    MSEPlatform.Playback.Etw.traceString("AudioPlayer::constructor: usingFastStart ");
                    MSEPlatform.Playback.assert(!mediaInstance.isLocal, "fast start tag should only be used with streaming content");
                    MSEPlatform.Playback.assert(mediaInstance.protectionState === MSEPlatform.Playback.ProtectionState.drmProtected, "fast start tag should only be used with protected content");
                    this._tagPlayer.fastStartProperties["Url"] = mediaInstance.source
                }
                else if (!this._usingPreActivatedTag)
                    this._setSourceFromMediaInstance()
            }, {
                isFastStartSource: {get: function isFastStartSource_get() {
                        return this._usingFastStartTag
                    }}, isPreActivatedSource: {get: function isPreActivatedSource_get() {
                            return this._usingPreActivatedTag
                        }}, _createHtmlTag: function AudioPlayer_createHtmlTag(playerType, mediaInstance, autoPlay) {
                        var htmlTag;
                        this._usingFastStartTag = false;
                        this._usingPreActivatedTag = false;
                        var audioTagForFileActivation = MSEPlatform.Playback.AudioPlayer.audioTagForFileActivation;
                        if (audioTagForFileActivation) {
                            if (!audioTagForFileActivation.error) {
                                htmlTag = audioTagForFileActivation;
                                this._usingPreActivatedTag = true
                            }
                            else {
                                mediaInstance.mediaInstanceId = null;
                                autoPlay = true
                            }
                            MSEPlatform.Playback.AudioPlayer.audioTagForFileActivation = null
                        }
                        if (!this._usingPreActivatedTag)
                            if (!MS.Entertainment.Utilities.isMusicApp2 && autoPlay && MSEPlatform.Playback.AudioPlayer.fastStartTag && !mediaInstance.isLocal && !mediaInstance.disableFastStart && mediaInstance.protectionState === MSEPlatform.Playback.ProtectionState.drmProtected) {
                                MSEPlatform.Playback.Etw.traceString("AudioPlayer::constructor: Using FastStart");
                                htmlTag = MSEPlatform.Playback.AudioPlayer.fastStartTag;
                                htmlTag.autoplay = autoPlay;
                                MSEPlatform.Playback.AudioPlayer.fastStartTag = null;
                                this._usingFastStartTag = true
                            }
                            else
                                htmlTag = MSEPlatform.Playback.MediaPlayer.prototype._createHtmlTag.call(this, playerType, mediaInstance, autoPlay);
                        return htmlTag
                    }, _usingFastStartTag: false, _usingPreActivatedTag: false
            }, {
                createFastStartTag: function AudioPlayer_createFastStartTag() {
                    function configureFastStartTagForDRM(htmlTag) {
                        var disableFastStart = function disableFastStart(e) {
                                if (this.fastStartTag)
                                    this.disposeFastStartTag();
                                this.fastStartBlockedOnLA = true
                            }.bind(this);
                        var mediaProtectionManager = new Windows.Media.Protection.MediaProtectionManager;
                        mediaProtectionManager.properties["Windows.Media.Protection.MediaProtectionSystemId"] = "{F4637010-03C3-42CD-B932-B48ADF3A6A54}";
                        var copyProtectionSystems = new Windows.Foundation.Collections.PropertySet;
                        copyProtectionSystems["{F4637010-03C3-42CD-B932-B48ADF3A6A54}"] = "Microsoft.Media.PlayReadyClient.PlayReadyWinRTTrustedInput";
                        mediaProtectionManager.properties["Windows.Media.Protection.MediaProtectionSystemIdMapping"] = copyProtectionSystems;
                        htmlTag._mediaProtectionManagerBindings = MS.Entertainment.Utilities.addEventHandlers(mediaProtectionManager, {
                            componentloadfailed: disableFastStart, servicerequested: disableFastStart
                        });
                        try {
                            htmlTag.msSetMediaProtectionManager(mediaProtectionManager)
                        }
                        catch(e) {
                            MS.Entertainment.Platform.Playback.fail("_configureFastStartTagForDRM: Unexpected exception on msSetMediaProtectionManager: " + e.message);
                            MSEPlatform.Playback.MediaPlayer._drmNotifyException(htmlTag, e, "configureFastStartTagForDRM_msSetMediaProtectionManager")
                        }
                    }
                    if (this.fastStartTag) {
                        MSEPlatform.Playback.Etw.traceString("-AudioPlayer:_createFastStartTag exists already");
                        return
                    }
                    var extensionManager = new Windows.Media.MediaExtensionManager;
                    this.fastStartTag = MSEPlatform.Playback.MediaPlayer.createMediaTag("audio");
                    this.fastStartTag.fastStartProperties = new Windows.Foundation.Collections.PropertySet;
                    this.fastStartTag.fastStartProperties["Url"] = "";
                    extensionManager.registerSchemeHandler("Microsoft.Entertainment.Platform.Playback.FastStartSchemeHandler", "fsms:", this.fastStartTag.fastStartProperties);
                    this.fastStartTag.setAttribute("msAudioCategory", "backgroundCapableMedia");
                    this.fastStartTag.width = 0;
                    this.fastStartTag.height = 0;
                    this.fastStartTag.autoplay = false;
                    configureFastStartTagForDRM.call(this, this.fastStartTag);
                    this.fastStartTag.src = "fsms://#";
                    MSEPlatform.Playback.Etw.traceString("-AudioPlayer:createFastStartTag: created")
                }, disposeFastStartTag: function AudioPlayer_disposeFastStartTag() {
                        MSEPlatform.Playback.Etw.traceString("AudioPlayer::disposeFastStartTag: Disposing tag");
                        if (this.fastStartTag) {
                            if (this.fastStartTag._mediaProtectionManagerBindings) {
                                this.fastStartTag._mediaProtectionManagerBindings.cancel();
                                this.fastStartTag._mediaProtectionManagerBindings = null
                            }
                            this.fastStartTag.removeAttribute("src");
                            this.fastStartTag.load();
                            this.fastStartTag = null;
                            this.fastStartBlockedOnLA = false
                        }
                        else
                            MSEPlatform.Playback.Etw.traceString("AudioPlayer::disposeFastStartTag: Nothing to dispose")
                    }, hasValidFastStartTag: function AudioPlayer_hasValidFastStartTag() {
                        return !!this.fastStartTag
                    }, audioTagForFileActivation: null, fastStartBlockedOnLA: false, fastStartTag: null
            }), VideoPlayer: MS.Entertainment.UI.Framework.derive("MS.Entertainment.Platform.Playback.MediaPlayer", function VideoPlayer_constructor(mediaInstance, autoPlay) {
                MSEPlatform.Playback.Etw.traceString("VideoPlayer::constructor: Creating video player");
                this.base("video", mediaInstance, autoPlay);
                if (!isBlobUrl(mediaInstance.source)) {
                    if (mediaInstance.videoEncoding === Microsoft.Entertainment.Marketplace.VideoEncoding.h264) {
                        var adaptiveSourceOpenedEventHandler = this._adaptiveSourceOpened.bind(this);
                        MSEPlatform.Playback.VideoPlayer._adaptiveSourceManager.addEventListener("adaptivesourceopenedevent", adaptiveSourceOpenedEventHandler);
                        this._unhookEventHandlers = function _unhookEventHandlers() {
                            MSEPlatform.Playback.VideoPlayer._adaptiveSourceManager.removeEventListener("adaptivesourceopenedevent", adaptiveSourceOpenedEventHandler);
                            if (this._adaptiveStreamingEvents) {
                                this._adaptiveStreamingEvents.cancel();
                                this._adaptiveStreamingEvents = null
                            }
                        };
                        this._reporter = new Microsoft.Entertainment.Platform.Playback.SmoothStreamingPlaybackReporter;
                        this._reporter.setMainAttribute("URL", mediaInstance.source);
                        if (this._urlRetryMap)
                            this._urlRetryMap = {}
                    }
                    else if (MS.Entertainment.Utilities.isApp2 && !MS.Entertainment.isAppModeOverride) {
                        var smoothStreamingSessionCreatedEventHandler = this._smoothStreamingSessionCreated.bind(this);
                        var resourceAvailabilityChanged = this._resourceAvailabilityChanged.bind(this);
                        MSEPlatform.Playback.VideoPlayer._smoothStreamingSessionManager.addEventListener("sessioncreated", smoothStreamingSessionCreatedEventHandler);
                        Windows.Xbox.ApplicationModel.ApplicationResourceLimits.addEventListener("resourceavailabilitychanged", resourceAvailabilityChanged);
                        this._unhookEventHandlers = function _unhookEventHandlers() {
                            MSEPlatform.Playback.VideoPlayer._smoothStreamingSessionManager.removeEventListener("sessioncreated", smoothStreamingSessionCreatedEventHandler);
                            Windows.Xbox.ApplicationModel.ApplicationResourceLimits.removeEventListener("resourceavailabilitychanged", resourceAvailabilityChanged)
                        }
                    }
                    var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                    if (configurationManager)
                        this._startBitrate = configurationManager.playback.defaultStartBitrate
                }
                MSEPlatform.Playback.VideoPlayer.registerByteStreamHandlerForURL(mediaInstance.source, mediaInstance.videoEncoding);
                this._tagPlayer.videoEncoding = mediaInstance.videoEncoding;
                if (mediaInstance.protectionState !== MSEPlatform.Playback.ProtectionState.unprotected)
                    this.configureDRM();
                this._setSourceFromMediaInstance()
            }, {
                dispose: function VideoPlayer_dispose() {
                    MSEPlatform.Playback.Etw.traceString("VideoPlayer::dispose: Disposing video player");
                    this._unhookEventHandlers();
                    if (this._tagPlayer.videoEncoding === Microsoft.Entertainment.Marketplace.VideoEncoding.h264) {
                        var that = this;
                        this._reporter = null;
                        this._urlRetryMap = {};
                        this._flushTelemetryBuffer();
                        WinJS.Promise.timeout(1000).then(function() {
                            MSEPlatform.Playback.MediaPlayer.prototype.dispose.call(that)
                        })
                    }
                    else {
                        if (MS.Entertainment.Utilities.isApp2 && !MS.Entertainment.isAppModeOverride && this._smoothStreamingSession)
                            if (!this._isConstrained)
                                try {
                                    var lastBitrate = this._smoothStreamingSession.currentBitrate;
                                    if (lastBitrate > 0) {
                                        MSEPlatform.Playback.Etw.traceString("VideoPlayer::dispose: Updating recorded last bitrate seen = " + lastBitrate);
                                        MSEPlatform.Playback.VideoPlayer.lastBitrateUnconstrained = lastBitrate;
                                        Windows.Storage.ApplicationData.current.localSettings.values[MSEPlatform.Playback.VideoPlayer.lastBitrateUnconstrainedKey] = lastBitrate
                                    }
                                }
                                catch(e) {}
                        MSEPlatform.Playback.MediaPlayer.prototype.dispose.call(this)
                    }
                }, _unhookEventHandlers: function VideoPlayer_unhookEventHandlers() {
                        return
                    }, _isConstrained: {get: function VideoPlayer_isConstrained() {
                            return (Windows.Xbox.ApplicationModel.ApplicationResourceLimits.resourceAvailability === Windows.Xbox.ApplicationModel.ResourceAvailability.constrained)
                        }}, _smoothStreamingSessionCreated: function VideoPlayer_smoothStreamingSessionCreated(session) {
                        if (session.uri !== this._media.source)
                            return;
                        var Playback = MSEPlatform.Playback;
                        MSEPlatform.Playback.Etw.traceString("VideoPlayer::_smoothStreamingSessionCreated: Event: smoothstreamingsessioncreated");
                        MS.Entertainment.Platform.assert(this._smoothStreamingSession === null, "_smoothStreamingSessionCreated called while this._smoothStreamingSession was non-null");
                        this._smoothStreamingSession = session;
                        if (Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enableVideo2SmoothStreamingTelemetry && !isBlobUrl(session.uri))
                            try {
                                var reporter = new Microsoft.Entertainment.Platform.Playback.SmoothStreamingPlaybackReporter;
                                session.setReporter(reporter)
                            }
                            catch(e) {
                                MS.Entertainment.Platform.fail("Failed to set session reporter: " + (e && e.message) ? e.message : String.empty, e ? e.number : 0)
                            }
                        try {
                            Playback.VideoPlayer.lastBitrateUnconstrained = Windows.Storage.ApplicationData.current.localSettings.values[Playback.VideoPlayer.lastBitrateUnconstrainedKey]
                        }
                        catch(e) {
                            MS.Entertainment.Platform.fail("Failure to read " + Playback.VideoPlayer.lastBitrateUnconstrainedKey + " from localSettings: " + (e && e.message) ? e.message : "", e ? e.number : 0)
                        }
                        try {
                            if (this._isConstrained)
                                this._smoothStreamingSession.startBitrate = Math.min(this._startBitrate, Playback.VideoPlayer.maxBitrateConstrained);
                            else
                                this._smoothStreamingSession.startBitrate = Playback.VideoPlayer.lastBitrateUnconstrained ? Playback.VideoPlayer.lastBitrateUnconstrained : this._startBitrate
                        }
                        catch(e) {
                            MS.Entertainment.Platform.fail("Exception in _smoothStreamingSessionCreated: " + (e && e.message) ? e.message : 0, e ? e.number : 0)
                        }
                        if (this._smoothStreamingEvents) {
                            this._smoothStreamingEvents.cancel();
                            this._smoothStreamingEvents = null
                        }
                        this._smoothStreamingEvents = MS.Entertainment.Utilities.addEventHandlers(session, {closed: this._smoothStreamingSessionClosed.bind(this)});
                        this._updateSmoothStreamingSessionRates()
                    }, _smoothStreamingSessionClosed: function VideoPlayer_smoothStreamingSessionClosed(evt) {
                        MSEPlatform.Playback.Etw.traceString("VideoPlayer::_smoothStreamingSessionClosed: Event: smoothstreamingsessionclosed(" + evt.errorCode + ")");
                        if (this._smoothStreamingEvents) {
                            this._smoothStreamingEvents.cancel();
                            this._smoothStreamingEvents = null
                        }
                        this._smoothStreamingSession = null
                    }, _resourceAvailabilityChanged: function _resourceAvailabilityChanged(evt) {
                        MSEPlatform.Playback.Etw.traceString("VideoPlayer::_resourceAvailabilityChanged: Event: resourceavailabilitychanged");
                        var Playback = MSEPlatform.Playback;
                        if (this._smoothStreamingSession && Playback.VideoPlayer.respectConstrainedMode) {
                            this._updateSmoothStreamingSessionRates();
                            try {
                                if (this._isConstrained) {
                                    var currentBitrate = this._smoothStreamingSession.currentBitrate;
                                    if (currentBitrate > 0) {
                                        Playback.VideoPlayer.lastBitrateUnconstrained = currentBitrate;
                                        Windows.Storage.ApplicationData.current.localSettings.values[Playback.VideoPlayer.lastBitrateUnconstrainedKey] = currentBitrate
                                    }
                                    if (currentBitrate > Playback.VideoPlayer.maxBitrateConstrained) {
                                        MSEPlatform.Playback.Etw.traceString("VideoPlayer::_resourceAvailabilityChanged: Calling RequestBitrateChange(" + Playback.VideoPlayer.maxBitrateConstrained + ", false)");
                                        this._smoothStreamingSession.requestBitrateChange(Playback.VideoPlayer.maxBitrateConstrained, true)
                                    }
                                }
                                else {
                                    var newBitrate = Playback.VideoPlayer.lastBitrateUnconstrained && Playback.VideoPlayer.lastBitrateUnconstrained > 0 ? Playback.VideoPlayer.lastBitrateUnconstrained : this._startBitrate;
                                    if (newBitrate > this._smoothStreamingSession.currentBitrate) {
                                        MSEPlatform.Playback.Etw.traceString("VideoPlayer::_resourceAvailabilityChanged: Calling RequestBitrateChange(" + newBitrate + ", true)");
                                        this._smoothStreamingSession.requestBitrateChange(newBitrate, true)
                                    }
                                }
                            }
                            catch(e) {
                                MS.Entertainment.Platform.fail("Exception calling requestBitrateChange: " + (e && e.message) ? e.message : 0, e ? e.number : 0)
                            }
                            var positionMs = this.forceTimeUpdate();
                            this.seekToPosition(Math.max(positionMs - 100, 0))
                        }
                    }, _updateSmoothStreamingSessionRates: function VideoPlayer_updateSmoothStreamingSessionRates() {
                        try {
                            var Playback = MSEPlatform.Playback;
                            MS.Entertainment.Platform.assert(this._smoothStreamingSession, "_updateSmoothStreamingSessionRates called while this._smoothStreamingSession was null");
                            if (this._isConstrained) {
                                MSEPlatform.Playback.Etw.traceString("VideoPlayer::_updateSmoothStreamingSessionRates: mode values being applied to this._smoothStreamingSession");
                                this._smoothStreamingSession.maximumBitrate = Playback.VideoPlayer.maxBitrateConstrained;
                                this._smoothStreamingSession.maximumWidth = Playback.VideoPlayer.maxWidthConstrained;
                                this._smoothStreamingSession.maximumHeight = Playback.VideoPlayer.maxHeightConstrained
                            }
                            else {
                                MSEPlatform.Playback.Etw.traceString("VideoPlayer::_updateSmoothStreamingSessionRates: 'Full' (unconstrained) mode values being applied to this._smoothStreamingSession");
                                this._smoothStreamingSession.maximumBitrate = Playback.VideoPlayer.maxBitrateUnconstrained;
                                this._smoothStreamingSession.maximumWidth = Playback.VideoPlayer.maxWidthUnconstrained;
                                this._smoothStreamingSession.maximumHeight = Playback.VideoPlayer.maxHeightUnconstrained
                            }
                        }
                        catch(e) {
                            MS.Entertainment.Platform.fail("_updateSmoothStreamingSessionRates exception: " + (e && e.message) ? e.message : 0, e ? e.number : 0)
                        }
                    }, _adaptiveSourceOpened: function VideoPlayer_adaptiveSourceOpened(args) {
                        var adaptiveSource = args.adaptiveSource;
                        if (adaptiveSource.uri.absoluteUri !== this._media.source)
                            return;
                        this._adaptiveStreamingEvents = MS.Entertainment.Utilities.addEventHandlers(adaptiveSource, {
                            manifestreadyevent: this._onH264PiffManifestReady.bind(this), adaptivesourcestatusupdatedevent: this._onAdaptiveSourceStatusUpdated.bind(this), adaptivesourcefailedevent: this._onAdaptiveSourceFailed.bind(this)
                        })
                    }, _onH264PiffManifestReady: function VideoPlayer_onH264PiffManifestReady(args) {
                        this._currentH264VideoStream = null;
                        this._h264ChunksDownloaded = 0;
                        function getPreferredAudioStream(availableStreams) {
                            function AudioStreamDescriptor(fourCC, bitrate, index) {
                                this._fourCC = fourCC;
                                this._bitrate = bitrate;
                                this._index = index
                            }
                            function compareAudioStreamDescriptors(descriptorOne, descriptorTwo) {
                                var FourCCCodes = new Array(858604357, 1279476033, 1212367169);
                                if (descriptorOne._fourCC === descriptorTwo._fourCC)
                                    return (descriptorTwo._bitrate - descriptorOne._bitrate);
                                else {
                                    var fourCCAIndex = FourCCCodes.indexOf(descriptorOne._fourCC);
                                    var fourCCBIndex = FourCCCodes.indexOf(descriptorTwo._fourCC);
                                    return fourCCAIndex - fourCCBIndex
                                }
                            }
                            var audioStreamDescriptors = new Array;
                            for (var i = 0; i < availableStreams.size; i++)
                                if (availableStreams[i].type === Microsoft.Media.AdaptiveStreaming.MediaStreamType.audio) {
                                    var firstTrack = availableStreams[i].availableTracks[0];
                                    var fourCC = firstTrack.fourCC;
                                    var bitrate = firstTrack.bitrate;
                                    audioStreamDescriptors.push(new AudioStreamDescriptor(fourCC, bitrate, i))
                                }
                            var preferredAudioStream = null;
                            if (audioStreamDescriptors.length !== 0) {
                                audioStreamDescriptors.sort(compareAudioStreamDescriptors);
                                preferredAudioStream = availableStreams[audioStreamDescriptors[0]._index]
                            }
                            return preferredAudioStream
                        }
                        function selectVideoTracks(videoStream) {
                            var availableTracks = videoStream.availableTracks;
                            var selectedVideoTracks = [];
                            for (var i = 0; i < availableTracks.size; i++)
                                if (availableTracks[i].maxWidth <= MSEPlatform.Playback.VideoPlayer.maxWidthUnconstrained && availableTracks[i].maxHeight <= MSEPlatform.Playback.VideoPlayer.maxHeightUnconstrained)
                                    selectedVideoTracks.push(availableTracks[i]);
                            MS.Entertainment.Platform.Playback.assert(selectedVideoTracks.length > 0, "Video stream has no tracks available!");
                            videoStream.restrictTracks(selectedVideoTracks);
                            var topBitrate = 0;
                            for (var i = 0; i < selectedVideoTracks.length; i++)
                                if (selectedVideoTracks[i].bitrate >= topBitrate)
                                    topBitrate = selectedVideoTracks[i].bitrate;
                            var startingVideoTracks = [];
                            var startingBitrate = Math.min(topBitrate, this._startBitrate);
                            for (var i = 0; i < selectedVideoTracks.length; i++)
                                if (selectedVideoTracks[i].bitrate >= startingBitrate)
                                    startingVideoTracks.push(selectedVideoTracks[i]);
                            MS.Entertainment.Platform.Playback.assert(startingVideoTracks.length > 0, "Video stream has no tracks which can be selected!");
                            videoStream.selectTracks(startingVideoTracks)
                        }
                        function getPreferredVideoStream(availableStreams) {
                            var firstVideoStream = null;
                            for (var i = 0; i < availableStreams.size; i++)
                                if (availableStreams[i].type === Microsoft.Media.AdaptiveStreaming.MediaStreamType.video) {
                                    firstVideoStream = availableStreams[i];
                                    break
                                }
                            if (firstVideoStream)
                                selectVideoTracks.call(this, firstVideoStream);
                            return firstVideoStream
                        }
                        var manifest = args.adaptiveSource.manifest;
                        var availableStreams = manifest.availableStreams;
                        var selectedStreams = new Array;
                        var preferredAudioStream = getPreferredAudioStream(availableStreams);
                        if (preferredAudioStream) {
                            selectedStreams.push(preferredAudioStream);
                            var preferredVideoStream = getPreferredVideoStream.call(this, availableStreams);
                            if (preferredVideoStream) {
                                selectedStreams.push(preferredVideoStream);
                                manifest.selectStreamsAsync(selectedStreams);
                                this._currentH264VideoStream = preferredVideoStream
                            }
                        }
                    }, _onAdaptiveSourceStatusUpdated: function VideoPlayer_onAdaptiveSourceStatusUpdated(args) {
                        function populateUrlRetryMap(urlInfo) {
                            if (urlInfo)
                                if (this._urlRetryMap[urlInfo])
                                    this._urlRetryMap[urlInfo]++;
                                else
                                    this._urlRetryMap[urlInfo] = 1
                        }
                        if (args)
                            switch (args.updateType) {
                                case Microsoft.Media.AdaptiveStreaming.AdaptiveSourceStatusUpdateType.chunkDownloaded:
                                    this._h264ChunksDownloaded++;
                                    if (this._h264ChunksDownloaded === this._h264ChunksDownloadedThreshold && this._currentH264VideoStream)
                                        this._currentH264VideoStream.selectTracks(this._currentH264VideoStream.availableTracks);
                                    this._reportChunkDownloaded(args);
                                    if ((this._h264ChunksDownloaded % 20) === 0 && this._reporter)
                                        this._reporter.sendLogAsync("Report", false);
                                    break;
                                case Microsoft.Media.AdaptiveStreaming.AdaptiveSourceStatusUpdateType.chunkConnectHttpInvalid:
                                case Microsoft.Media.AdaptiveStreaming.AdaptiveSourceStatusUpdateType.nextChunkHttpInvalid:
                                case Microsoft.Media.AdaptiveStreaming.AdaptiveSourceStatusUpdateType.chunkHdrHttpInvalid:
                                case Microsoft.Media.AdaptiveStreaming.AdaptiveSourceStatusUpdateType.chunkHdrError:
                                    populateUrlRetryMap.call(this, args.additionalInfo);
                                    var responseValue = args.httpResponse ? args.httpResponse.toString() : "";
                                    this._reporter.setSubAttribute("hr", responseValue);
                                    this._reporter.sendLogAsync("MissingChunk", true);
                                    break;
                                default:
                                    break
                            }
                    }, _flushTelemetryBuffer: function VideoPlayer_flushTelemetryBuffer() {
                        if (this._reporter) {
                            this._reporter.sendLogAsync("Report", false);
                            this._reporter.sendLogAsync("Close", false)
                        }
                    }, _getTelemetryInfoFromChunkDownloadedArgs: function _getTelemetryInfoFromChunkDownloadedArgs(args) {
                        var TelemetryFields = {
                                chunkIndex: 0, url: 1, mediaStreamType: 2, chunkStartTimeHns: 3, chunkDurationns: 4, chunkBitrate: 5, chunkByteCount: 6, downloadRequestedTimeMs: 7, downloadCompletedTimeMs: 8, chunkPerceivedBandwidth: 9, avgPerceivedBandwidth: 10, bufferLevelAtRequested90kHz: 11, bufferLevelAtCompleted90kHz: 12, responseHeaders: 13
                            };
                        function uint32(val) {
                            var value = +val;
                            return isNaN(value) ? 0 : value
                        }
                        var HNS_PER_MILLISECOND = 10000;
                        var BUFFER_UNITS_90KHZ_TO_HNS = (1 / 90) * HNS_PER_MILLISECOND;
                        var additionalInfo = args.additionalInfo.split(";");
                        var info = {};
                        info.chunkId = uint32(additionalInfo[TelemetryFields.chunkIndex]);
                        info.url = additionalInfo[TelemetryFields.url];
                        info.bitrate = uint32(additionalInfo[TelemetryFields.chunkBitrate]);
                        info.startTick = uint32(additionalInfo[TelemetryFields.downloadRequestedTimeMs]) * HNS_PER_MILLISECOND;
                        info.openedTick = uint32(additionalInfo[TelemetryFields.downloadRequestedTimeMs]) * HNS_PER_MILLISECOND;
                        info.completedTicks = uint32(additionalInfo[TelemetryFields.downloadCompletedTimeMs]) * HNS_PER_MILLISECOND;
                        info.sizeInBytes = uint32(additionalInfo[TelemetryFields.chunkByteCount]);
                        info.hnsStartTime = uint32(additionalInfo[TelemetryFields.chunkStartTimeHns]);
                        info.avgBandwidth = uint32(additionalInfo[TelemetryFields.avgPerceivedBandwidth]);
                        info.lastBandwidth = uint32(additionalInfo[TelemetryFields.chunkPerceivedBandwidth]);
                        info.hnsBuffer = uint32(additionalInfo[TelemetryFields.bufferLevelAtCompleted90kHz]) * BUFFER_UNITS_90KHZ_TO_HNS;
                        return info
                    }, _reportChunkDownloaded: function VideoPlayer_reportChunkDownloaded(args) {
                        var info = this._getTelemetryInfoFromChunkDownloadedArgs(args);
                        var bitrateIndex = 0;
                        var width = 0;
                        var height = 0;
                        if (this._reporter) {
                            if (!this._urlRetryMap[info.url])
                                this._reporter.setMainAttribute("DownloadRetries", "0");
                            else
                                this._reporter.setMainAttribute("DownloadRetries", this._urlRetryMap[info.url].toString());
                            for (var i = 0; i < this._currentH264VideoStream.availableTracks.length; i++)
                                if (this._currentH264VideoStream.availableTracks[i].bitrate === info.bitrate) {
                                    bitrateIndex = this._currentH264VideoStream.availableTracks[i].trackIndex;
                                    width = this._currentH264VideoStream.availableTracks[i].maxWidth;
                                    height = this._currentH264VideoStream.availableTracks[i].maxHeight
                                }
                            this._reporter.addChunkInfo(bitrateIndex, info.bitrate, width, height);
                            this._reporter.addChunkHistory(info.chunkId, info.bitrate, info.bitrateIndex, info.hnsBuffer, info.avgBandwidth, info.lastBandwidth);
                            this._reporter.addDownloadData(info.startTick, info.openedTick, info.completedTicks, info.sizeInBytes, info.hnsStartTime, info.bitrate)
                        }
                    }, _onAdaptiveSourceFailed: function _onAdaptiveSourceFailed(args) {
                        if (this._reporter) {
                            var errorState = "unhandled";
                            for (var name in Microsoft.Media.AdaptiveStreaming.AdaptiveSourceFailedType)
                                if (args.failType === Microsoft.Media.AdaptiveStreaming.AdaptiveSourceFailedType[name]) {
                                    errorState = name;
                                    break
                                }
                            this._reporter.setSubAttribute("hr", errorState);
                            this._reporter.sendLogAsync("MissingChunk", true)
                        }
                    }, _smoothStreamingSession: null, _smoothStreamingEvents: null, _startBitrate: 2 * 1024 * 1024, _reporter: null, _adaptiveStreamingEvents: null, _currentH264VideoStream: null, _currentlyRegisteredByteStreamHandler: null, _h264ChunksDownloaded: 0, _h264ChunksDownloadedThreshold: 5, _urlRetryMap: {}
            }, {
                initializeVideoExtensions: function VideoPlayer_initializeVideoExtensions() {
                    if (MS.Entertainment.Utilities.isVideoApp1 || MS.Entertainment.Utilities.isApp2 || MS.Entertainment.Utilities.isTestApp) {
                        this._extensionManager = new Windows.Media.MediaExtensionManager;
                        this._adaptiveSourceManager = Microsoft.Media.AdaptiveStreaming.AdaptiveSourceManager.getDefault();
                        if (MS.Entertainment.Utilities.isApp2 && !MS.Entertainment.isAppModeOverride) {
                            this._smoothStreamingSessionManager = new Windows.Xbox.Media.SmoothStreamingSessionManager;
                            var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                            if (configurationManager) {
                                this.maxBitrateConstrained = configurationManager.playback.maxBitrateConstrained;
                                this.maxWidthConstrained = configurationManager.playback.maxWidthConstrained;
                                this.maxHeightConstrained = configurationManager.playback.maxHeightConstrained;
                                this.respectConstrainedMode = configurationManager.playback.respectConstrainedMode
                            }
                        }
                    }
                    if (MS.Entertainment.Utilities.isVideoApp1 && !MS.Entertainment.Utilities.isWindowsBlue)
                        this._extensionManager.registerByteStreamHandler("Microsoft.Entertainment.Platform.Playback.CFFByteStreamHandler", ".mp4", String.empty)
                }, registerByteStreamHandlerForURL: function VideoPlayer_registerByteStreamHandlerForURL(url, videoEncoding) {
                        if (!isBlobUrl(url) && (MS.Entertainment.Utilities.isVideoApp1 || MS.Entertainment.Utilities.isApp2 || MS.Entertainment.Utilities.isTestApp)) {
                            var isH264 = (videoEncoding === Microsoft.Entertainment.Marketplace.VideoEncoding.h264);
                            if (isH264) {
                                if (this._currentlyRegisteredByteStreamHandler !== this.VideoStreamFormat.H264) {
                                    this._currentlyRegisteredByteStreamHandler = this.VideoStreamFormat.H264;
                                    var ps = new Windows.Foundation.Collections.PropertySet;
                                    ps.insert("{A5CE1DE8-1D00-427B-ACEF-FB9A3C93DE2D}", this._adaptiveSourceManager);
                                    this._extensionManager.registerByteStreamHandler("Microsoft.Media.AdaptiveStreaming.SmoothByteStreamHandler", "", "text/xml", ps);
                                    MSEPlatform.Playback.Etw.traceString("VideoPlayer::_registerByteStreamHandlerForURL: Registered H.264 bytestream handler. Url:" + url)
                                }
                            }
                            else if (this._currentlyRegisteredByteStreamHandler !== this.VideoStreamFormat.VC1) {
                                this._currentlyRegisteredByteStreamHandler = this.VideoStreamFormat.VC1;
                                if (MS.Entertainment.Utilities.isApp2 && !MS.Entertainment.isAppModeOverride) {
                                    var ps = new Windows.Foundation.Collections.PropertySet;
                                    ps.insert("SmoothStreamingSessionManager", this._smoothStreamingSessionManager);
                                    this._extensionManager.registerByteStreamHandler("Windows.Xbox.Media.SmoothStreamingByteStreamHandler", "", "text/xml", ps)
                                }
                                else
                                    this._extensionManager.registerByteStreamHandler("Microsoft.Entertainment.Platform.Playback.MBRByteStreamHandler", "", "text/xml");
                                MSEPlatform.Playback.Etw.traceString("VideoPlayer::_registerByteStreamHandlerForURL: Registered VC-1 bytestream handler. Url:" + url)
                            }
                        }
                    }, maxWidthUnconstrained: {get: function maxWidthUnconstrained_get() {
                            return MS.Entertainment.Utilities.isApp1 ? 1366 : 1920
                        }}, maxHeightUnconstrained: {get: function maxHeightUnconstrained_get() {
                            return MS.Entertainment.Utilities.isApp1 ? 768 : 1080
                        }}, maxWidthConstrained: 1920, maxHeightConstrained: 720, maxBitrateConstrained: 850000, maxBitrateUnconstrained: 4294967295, respectConstrainedMode: true, lastBitrateUnconstrained: 0, lastBitrateUnconstrainedKey: "VideoPlayer._lastBitrateUnconstrained", VideoStreamFormat: {
                        H264: "H264", VC1: "VC1"
                    }, _smoothStreamingSessionManager: null, _adaptiveSourceManager: null, _extensionManager: null, _currentlyRegisteredByteStreamHandler: null
            })
    })
})()
