﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Platform.Playback");
(function() {
    "use strict";
    var MSEPlatform = WinJS.Namespace.define("MS.Entertainment.Platform", null);
    WinJS.Namespace.defineWithParent(MSEPlatform, "Playback", {PlaybackController: MS.Entertainment.UI.Framework.derive("MS.Entertainment.UI.Framework.ObservableBase", function PlaybackController_constructor() {
            var sessionMgr = null;
            var eventHandlers = MS.Entertainment.Utilities.addEvents(this, {
                    currentMediaChanged: function currentMediaChanged(e) {
                        var newMedia = e.detail.newValue;
                        var oldMedia = e.detail.oldValue;
                        MSEPlatform.Playback.Etw.traceString("PlaybackController::currentMediaChanged: ");
                        this._setMedia(newMedia);
                        if (!sessionMgr && MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.sessionManager))
                            sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                        if (newMedia && sessionMgr)
                            sessionMgr._notifyUpcomingStreaming(!newMedia.isLocal);
                        if (newMedia)
                            this._isVideo = newMedia.isVideo()
                    }.bind(this), nextMediaChanged: function nextMediaChanged(e) {
                            var newMedia = e.detail.newValue;
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::nextMediaChanged: " + (newMedia ? newMedia.source : "null"));
                            if (!!newMedia)
                                MS.Entertainment.Platform.Playback.Etw.tracePlaylistSetNextMedia(newMedia);
                            this._setNextMedia(newMedia)
                        }.bind(this), playerStateChanged: function playerStateChanged(e) {
                            var newState = e.detail.newValue;
                            var oldState = e.detail.oldValue;
                            if (!oldState)
                                oldState = "undefined";
                            MSEPlatform.Playback.Etw.tracePlayerStateChanged(newState, oldState)
                        }, currentTransportStateChanged: function currentTransportStateChanged(e) {
                            var newState = e.detail.newValue;
                            var oldState = e.detail.oldValue;
                            if (!oldState)
                                oldState = "undefined";
                            MSEPlatform.Playback.Etw.traceTransportStateChanged(newState, oldState, this._isVideo)
                        }.bind(this)
                });
            this._sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
            this.bind("currentTransportState", function onCurrentTransportStateChanged() {
                this._reschedulePrerollCallback()
            }.bind(this));
            this._configManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
            this._offlineNetworkRequestDelay = this._configManager.playback.offlineNetworkRequestDelay;
            var observablePromises = {};
            var lastPlayerStateError = null;
            this._observableNoLossAssign = function _observableNoLossAssign(name, value) {
                if (name === "playerState")
                    if (value === MS.Entertainment.Platform.Playback.PlayerState.error) {
                        if (!this._errorDescriptor.isCritical)
                            this._errorDescriptor.isCritical = this._isCriticalError(this._errorDescriptor);
                        if (this._currentMedia)
                            this._errorDescriptor.errorOrdinal = this._currentMedia.cookie;
                        this._observableNoLossAssign("error", this._errorDescriptor);
                        lastPlayerStateError = this._errorDescriptor
                    }
                    else if (lastPlayerStateError) {
                        this._observableNoLossAssign("error", null);
                        lastPlayerStateError = null
                    }
                if (!observablePromises[name])
                    observablePromises[name] = this.updateAndNotify(name, value);
                else
                    observablePromises[name] = observablePromises[name].then(function assign() {
                        return this.updateAndNotify(name, value)
                    }.bind(this))
            };
            this.waitForNoLossAssignmentsComplete = function waitForNoLossAssignmentsComplete() {
                return WinJS.Promise.join(observablePromises)
            }
        }, {
            dispose: function dispose() {
                this.reset();
                this.unbind("currentTransportState");
                if (this._playerEventHandlers) {
                    this._playerEventHandlers.cancel();
                    this._playerEventHandlers = null
                }
                this._player._currentMediaEventsCallback = null
            }, targetTransportState: {
                    get: function targetTransportState_get() {
                        return this._targetTransportState
                    }, set: function targetTransportState_set(value) {
                            if (!this._isPlayerSet())
                                return;
                            MSEPlatform.Playback.Etw.traceTargetTransportStateSet(value, this.currentTransportState);
                            if (this._isPlayerState(MSEPlatform.Playback.PlayerState.ready)) {
                                if ((this._targetTransportState !== value) || (MS.Entertainment.Platform.Playback.TransportState.playing === value && (this.playbackRate > 1 || this.playbackRate < -1)))
                                    this._applyTargetTransportState(value)
                            }
                            else if (this._isPlayerState(MSEPlatform.Playback.PlayerState.error))
                                try {
                                    this._applyTargetTransportState(value)
                                }
                                catch(e) {}
                            else
                                this._targetTransportState = value
                        }
                }, autoPlay: {
                    get: function autoPlay_get() {
                        if (!this._isPlayerSet())
                            return false;
                        return this._player.autoPlay
                    }, set: function autoPlay_set(value) {
                            if (!this._isPlayerSet())
                                return;
                            this._player.autoPlay = value
                        }
                }, muted: {
                    get: function muted_get() {
                        if (!this._isPlayerSet())
                            return false;
                        return this._player.muted
                    }, set: function muted_set(value) {
                            if (!this._isPlayerSet())
                                return;
                            this._player.muted = value
                        }
                }, seekToPosition: function seekToPosition(positionMsec) {
                    if (!this._isPlayerSet())
                        return;
                    if (positionMsec < 0)
                        positionMsec = 0;
                    if (positionMsec > this.duration)
                        positionMsec = this.duration;
                    if (!this._isPlayerState(MSEPlatform.Playback.PlayerState.ready))
                        this._targetPosition = positionMsec;
                    else {
                        if (this.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.playing) {
                            MS.Entertainment.Utilities.Telemetry.logPauseHappened(this, this.forceTimeUpdate());
                            MS.Entertainment.Utilities.Telemetry.logPlayHappened(this, positionMsec)
                        }
                        var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                        var isPlayToReceiverSessionRunning = sessionMgr && sessionMgr.primarySession && sessionMgr.primarySession.isPlayToReceiverSessionRunning;
                        if (!isPlayToReceiverSessionRunning)
                            if ((this.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.stopped) && (this._currentMedia && !this._currentMedia.videoAdSupported))
                                this._applyTargetTransportState(MS.Entertainment.Platform.Playback.TransportState.playing);
                        this._player.seekToPosition(positionMsec);
                        if (this.currentTransportState !== MS.Entertainment.Platform.Playback.TransportState.playing)
                            this._targetPosition = positionMsec
                    }
                }, fastFwd: function fastFwd() {
                    if (!this._isPlayerSet())
                        return;
                    this._player.fastFwd()
                }, fastReverse: function fastReverse() {
                    if (!this._isPlayerSet())
                        return;
                    this._player.rewind()
                }, slowFwd: function slowFwd() {
                    if (!this._isPlayerSet())
                        return;
                    this._player.slowFwd()
                }, slowReverse: function slowReverse() {
                    if (!this._isPlayerSet())
                        return;
                    this._player.slowRewind()
                }, reset: function reset(sendStop) {
                    this._internalCurrentMedia = null;
                    if (!this._isPlayerSet())
                        return;
                    if (sendStop && this._player && this.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.playing)
                        MS.Entertainment.Utilities.Telemetry.logSkipHappened(this, this.forceTimeUpdate());
                    this._player.reset(false);
                    this._player.reset(true);
                    if (this._prerollMediaItem) {
                        this._prerollMediaItem = null;
                        this._cancelPrerollCallback()
                    }
                    this.currentPosition = 0;
                    this._targetPosition = 0;
                    if (sendStop)
                        this.currentTransportState = MS.Entertainment.Platform.Playback.TransportState.stopped;
                    this._targetTransportState = MS.Entertainment.Platform.Playback.TransportState.unInitialize;
                    this.duration = 0;
                    this.errorDescriptor = null;
                    this._observableNoLossAssign("playerState", MS.Entertainment.Platform.Playback.PlayerState.notReady);
                    this.readyForNextMedia = false;
                    this.videoWidth = 0;
                    this.videoHeight = 0;
                    this.playbackRate = 1;
                    this.minPlaybackRate = 0;
                    this.maxPlaybackRate = 0
                }, forceError: function forceError(errorCode) {
                    if (!this._isPlayerSet())
                        return;
                    MSEPlatform.Playback.firePlaybackError(this._onPlayerEvent.bind(this), errorCode, "forceError");
                    this.reset(true)
                }, setPlayer: function setPlayer(player) {
                    if (player) {
                        this._player = player;
                        if (this._playerEventHandlers) {
                            this._playerEventHandlers.cancel();
                            this._playerEventHandlers = null
                        }
                        this._playerEventHandlers = MS.Entertainment.Utilities.addEvents(this._player, {
                            _nextMediaLoadedChanged: function _onNextMediaLoaded(e) {
                                var isLoaded = e.detail.newValue;
                                this._handleNextMediaLoaded(isLoaded, this)
                            }.bind(this), _nextMediaStartedChanged: function _onNextMediaStarted(e) {
                                    var isStarted = e.detail.newValue;
                                    this._handleNextMediaStarted(isStarted, this)
                                }.bind(this), _nextMediaErrorChanged: function _onNextMediaError(e) {
                                    var isError = e.detail.newValue;
                                    this._handleNextMediaError(isError)
                                }.bind(this), isRemoteSessionRunningChanged: function _onRemoteSessionChanged(e) {
                                    var isRemoteSessionRunningValue = e.detail.newValue;
                                    this.isRemoteSessionRunning = isRemoteSessionRunningValue
                                }.bind(this), playToSenderConnectionStateChanged: function _onPlayToSenderConnectionStateChanged(e) {
                                    var playToSenderConnectionStateValue = e.detail.newValue;
                                    this.playToSenderConnectionState = playToSenderConnectionStateValue
                                }.bind(this), mediaElementChanged: function _onMediaElementChanged(e) {
                                    var mediaElementValue = e.detail.newValue;
                                    this.mediaElement = mediaElementValue
                                }.bind(this)
                        });
                        this._player._currentMediaEventsCallback = this._onPlayerEvent.bind(this)
                    }
                }, enableTimeUpdate: function enableTimeUpdate() {
                    if (!this._isPlayerSet())
                        return;
                    this._player.enableTimeUpdate();
                    MSEPlatform.Playback.Etw.tracePlaybackEnableTimeUpdate(this.currentPosition)
                }, disableTimeUpdate: function disableTimeUpdate() {
                    if (!this._isPlayerSet())
                        return;
                    this._player.disableTimeUpdate();
                    MSEPlatform.Playback.Etw.tracePlaybackDisableTimeUpdate(this.currentPosition)
                }, forceTimeUpdate: function forceTimeUpdate() {
                    if (this._isPlayerSet() && this._player._currentPlayer) {
                        this.currentPosition = this._player.forceTimeUpdate();
                        MSEPlatform.Playback.Etw.tracePlaybackForceTimeUpdate(this.currentPosition)
                    }
                    return this.currentPosition
                }, isRemoteSession: function isRemoteSession() {
                    if (this._isPlayerSet())
                        return this._player.isRemoteSession();
                    return false
                }, notifyNetworkConnectionChanged: function notifyNetworkConnectionChanged(networkConnection) {
                    WinJS.Promise.timeout().then(this._handleNetworkConnectionChanged(networkConnection))
                }, skipToNextPrerolled: function skipToNextPrerolled(fromSkipButton) {
                    this._reportPrerollErrors();
                    MS.Entertainment.Utilities.Telemetry.logSkipHappened(this, this.forceTimeUpdate());
                    this._player._switchPlayerAsync().done(function switch_completed(succeeded) {
                        if (succeeded) {
                            if (this.targetTransportState === MSEPlatform.Playback.TransportState.playing || this.currentTransportState === MSEPlatform.Playback.TransportState.playing || (this.autoPlay && !fromSkipButton))
                                this._applyTargetTransportState(MSEPlatform.Playback.TransportState.playing)
                        }
                        else
                            this._skipButtonPressed = fromSkipButton
                    }.bind(this), null)
                }, hasPrerolledMedia: function hasPrerolledMedia() {
                    return (this._player && this._player._nextPlayer)
                }, prerollMediaItem: function PrerollMediaItem(mediaItem) {
                    var name = (mediaItem && mediaItem.data) ? mediaItem.data.name : String.empty;
                    MSEPlatform.Playback.Etw.traceString("PlaybackController::prerollMediaItem: Control has been asked to preroll \"" + name + "\"");
                    this._cancelPrerollCallback();
                    this._prerollMediaItem = mediaItem;
                    if (this._player)
                        this._player.controlIsProcessingNextMedia = true;
                    if (this._prerollMediaItem)
                        this._reschedulePrerollCallback();
                    else
                        this._setNextMedia(null)
                }, _reschedulePrerollCallback: function _reschedulePrerollCallback() {
                    MSEPlatform.Playback.Etw.traceString("PlaybackController::reschedulePrerollCallback");
                    this._cancelPrerollCallback();
                    if (!this._prerollMediaItem) {
                        MSEPlatform.Playback.Etw.traceString("PlaybackController::reschedulePrerollCallback: Control has nothing to preroll. Dropping reschedule request.");
                        if (this._player)
                            this._player.controlIsProcessingNextMedia = false;
                        return
                    }
                    MSEPlatform.Playback.Etw.traceString("PlaybackController::reschedulePrerollCallback: Control is scheduling next media.");
                    var myPrerollPromiseOrdinal = this._prerollPromiseOrdinal;
                    this._prerollPromise = this._collectPrerollInformation().then(function onCollectPrerollInformation(prerollInformation) {
                        if (myPrerollPromiseOrdinal !== this._prerollPromiseOrdinal) {
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::reschedulePrerollCallback: Promise was canceled but completed anyway. Dropping.");
                            if (this._player)
                                this._player.controlIsProcessingNextMedia = false;
                            return
                        }
                        if (this.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.stopped) {
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::reschedulePrerollCallback: Control was stopped, prerolling immediately.");
                            prerollInformation.delayTime = 0
                        }
                        if (prerollInformation.delayTime > 0 && this.currentTransportState !== MS.Entertainment.Platform.Playback.TransportState.playing)
                            if (this.currentMedia && this.currentMedia._errorDescriptor)
                                if (this._errorCount < this._maxSequentialErrors) {
                                    MSEPlatform.Playback.Etw.traceString("PlaybackController::reschedulePrerollCallback: Not playing and in error, prerolling immediately.");
                                    prerollInformation.delayTime = 0
                                }
                                else {
                                    MSEPlatform.Playback.Etw.traceString("PlaybackController::reschedulePrerollCallback: Not scheduling preroll. Too many errors. Report and deactivate playlist");
                                    this._setNextMedia(null);
                                    this._handleErrorsOnEndOfPlaylist()
                                }
                            else {
                                MSEPlatform.Playback.Etw.traceString("PlaybackController::reschedulePrerollCallback: Not scheduling preroll. Determined we should delay but pipeline is not playing");
                                return
                            }
                        MSEPlatform.Playback.Etw.traceString("PlaybackController::reschedulePrerollCallback: Scheduling preroll for " + (prerollInformation.delayTime / 1000) + " seconds from now");
                        return WinJS.Promise.timeout(prerollInformation.delayTime).then(function onPrerollTimeout() {
                                MSEPlatform.Playback.Etw.traceString("PlaybackController::reschedulePrerollCallback: Preroll timeout fired at " + (this.currentPosition / 1000) + " seconds into the file. ");
                                return this._requestNetwork().then(function() {
                                        return this._setPrerollItemAsNextInstance()
                                    }.bind(this))
                            }.bind(this), function onPrerollTimeoutError(error) {
                                if (WinJS.Promise.isCanceledError(error))
                                    MSEPlatform.Playback.Etw.traceString("PlaybackController::reschedulePrerollCallback: _prerollPromise canceled.")
                            }.bind(this))
                    }.bind(this), function onCollectPrerollInformationError(error) {
                        if (WinJS.Promise.isCanceledError(error))
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::reschedulePrerollCallback::onCollectPrerollInformationError: _prerollPromise canceled.")
                    })
                }, _cancelPrerollCallback: function PlaylistCore_cancelPrerollCallback() {
                    if (this._prerollPromise) {
                        MSEPlatform.Playback.Etw.traceString("PlaybackController::cancelPrerollCallback: Canceling scheduled preroll.");
                        this._prerollPromise.cancel();
                        this._prerollPromise = null;
                        this._prerollPromiseOrdinal++
                    }
                }, _collectPrerollInformation: function PlaylistCore_collectPrerollInformation() {
                    return this._hydrateItemForPreroll(this._prerollMediaItem).then(function onHydrateItemForPreroll(result) {
                            var prerollInformation = {
                                    isLocal: null, delayTime: 0
                                };
                            if (result && result.data) {
                                prerollInformation.isLocal = result.data.canPlayLocally;
                                MSEPlatform.Playback.Etw.traceString("PlaybackController::collectPrerollInformation: Preroll item can play locally: " + result.data.canPlayLocally + ".");
                                if (prerollInformation.isLocal === false)
                                    prerollInformation.delayTime = this._calculatePrerollTimeout()
                            }
                            return prerollInformation
                        }.bind(this))
                }, _hydrateItemForPreroll: function _hydrateItemForPreroll(mediaItem) {
                    if (mediaItem && !mediaItem.hydratedForPreroll && mediaItem.data && mediaItem.data.mediaType) {
                        MSEPlatform.Playback.Etw.traceString("PlaybackController::_hydrateItemForPreroll:  >> hydrateLibraryInfoAsync");
                        return MS.Entertainment.ViewModels.MediaItemModel.hydrateLibraryInfoAsync(mediaItem.data).then(function onHydrateLibraryInfoAsync() {
                                MSEPlatform.Playback.Etw.traceString("PlaybackController::_hydrateItemForPreroll:  << hydrateLibraryInfoAsync");
                                mediaItem.hydratedForPreroll = true;
                                return WinJS.Promise.wrap(mediaItem)
                            }, function onHydrateLibraryInfoAsyncError(e) {
                                MSEPlatform.Playback.Etw.traceString("PlaybackController::_hydrateItemForPreroll:  << hydrateLibraryInfoAsync Error");
                                return WinJS.Promise.wrapError(e)
                            })
                    }
                    else {
                        MSEPlatform.Playback.Etw.traceString("PlaybackController::_hydrateItemForPreroll:  mediaItem hydrated for preroll");
                        if (mediaItem)
                            mediaItem.hydratedForPreroll = true;
                        return WinJS.Promise.wrap(mediaItem)
                    }
                }, _calculatePrerollTimeout: function PlaylistCore_calculatePrerollTimeout() {
                    var currentPosition = this.forceTimeUpdate();
                    var currentDuration = this.duration ? this.duration : 0;
                    var timeout = currentDuration - currentPosition;
                    timeout = timeout - this._configManager.playback.streamingPrerollMS;
                    if (timeout < 0)
                        timeout = 0;
                    MSEPlatform.Playback.Etw.traceString("calculatePrerollTimeout: Determining a preroll timeout position: " + currentPosition + " duration: " + currentDuration + " timeout: " + timeout);
                    return timeout
                }, _setPrerollItemAsNextInstance: function _setPrerollItemAsNextInstance() {
                    MSEPlatform.Playback.Etw.traceString("PlaybackController::_setPrerollItemAsNextInstance: Prerolling at " + (this.currentPosition / 1000) + " seconds into the file.");
                    var onSetComplete = function onSetComplete() {
                            this._prerollMediaItem = null
                        }.bind(this);
                    MS.Entertainment.Platform.Playback.assert(this._prerollMediaItem, "Tried to convert a null preroll item. Dropping.");
                    if (!this._prerollMediaItem) {
                        MSEPlatform.Playback.Etw.traceString("Tried to convert a null preroll item. Dropping.");
                        return
                    }
                    return MS.Entertainment.Platform.Playback.Playlist.PlaylistCore.convertMediaItemToMediaInstance(this._prerollMediaItem, null, MS.Entertainment.Platform.Playback.UsageContext.automatic).then(function onConvertMediaItemToMediaInstance(mediaInstance) {
                            MSEPlatform.Playback.Etw.traceString("onConvertMediaItemToMediaInstance:  mediaInstance.source = " + mediaInstance.source);
                            this.nextMedia = mediaInstance;
                            onSetComplete()
                        }.bind(this), function onConvertMediaItemToMediaInstanceError(error) {
                            MSEPlatform.Playback.Etw.traceString("onConvertMediaItemToMediaInstanceError");
                            if (!WinJS.Promise.isCanceledError(error))
                                return MSEPlatform.Playback.MediaInstance.createInstanceAsync({
                                        cookie: this._prerollMediaItem.index, error: MSEPlatform.Playback.makePlaybackError(error, "prerollMediaItem_convertMediaItemToMediaInstance error"), mediaItem: this._prerollMediaItem
                                    }).then(function(errorMediaInstance) {
                                        this.nextMedia = errorMediaInstance;
                                        onSetComplete()
                                    }.bind(this));
                            else
                                MSEPlatform.Playback.Etw.traceString("onConvertMediaItemToMediaInstanceError: prerollMediaItem canceled")
                        }.bind(this))
                }, _isPlayerSet: function _isPlayerSet() {
                    if (!this._player)
                        MSEPlatform.Playback.Etw.traceString("PlaybackController::isPlayerSet: No, player is not set yet!");
                    return (this._player ? true : false)
                }, _isPlayerState: function _isPlayerState(state) {
                    return (this.playerState === state)
                }, _requestNetwork: function _requestNetwork() {
                    return WinJS.Promise.timeout(0)
                }, _releaseNetwork: function _releaseNetwork() {
                    MSEPlatform.Playback.Etw.traceString("PlaybackController::releaseNetwork");
                    if (this._networkUpRequest) {
                        var releaseMeLater = this._networkUpRequest;
                        this._networkUpRequest = null;
                        WinJS.Promise.timeout(5000).then(function() {
                            releaseMeLater.release()
                        })
                    }
                }, _setMedia: function _setMedia(mediaInstance) {
                    if (mediaInstance && mediaInstance._mediaItem && mediaInstance._mediaItem.data) {
                        var itemData = mediaInstance._mediaItem.data;
                        MSEPlatform.Playback.Etw.traceString("PlaybackController::_setMedia: mediaInstance = { name: " + itemData.name + ", libraryId: " + itemData.libraryId + ", serviceId: " + itemData.serviceId + " }")
                    }
                    else
                        MSEPlatform.Playback.Etw.traceString("PlaybackController::_setMedia: mediaInstance._mediaItem.data= undefined");
                    if (this._player)
                        this._player.controlIsProcessingNextMedia = false;
                    if (!mediaInstance) {
                        MS.Entertainment.Utilities.Telemetry.logSkipHappened(this, this.forceTimeUpdate());
                        if (this._canResetOnNullMediaInstance)
                            this.reset(true);
                        else
                            this._canResetOnNullMediaInstance = true;
                        return
                    }
                    if (!this._isPlayerSet())
                        return;
                    if (mediaInstance.isEqual(this._internalCurrentMedia))
                        return;
                    MS.Entertainment.Utilities.Telemetry.logPlaybackAttempted(mediaInstance);
                    this._clearReportedErrors();
                    this.endOfPlaylist = false;
                    if (this._player && ((this.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.playing) || (this.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.paused)))
                        MS.Entertainment.Utilities.Telemetry.logSkipHappened(this, this.forceTimeUpdate());
                    this.isPreview = mediaInstance.isPreview;
                    if (mediaInstance.videoAdSupported)
                        this.autoPlay = false;
                    this._handleMediaInstance(mediaInstance)
                }, _handleMediaInstance: function _handleMediaInstance(mediaInstance) {
                    this._nextMediaInstanceAfterAd = null;
                    this.isAudioAd = false;
                    this._setPlayerMedia(mediaInstance)
                }, _clearReportedErrors: function _clearReportedErrors() {
                    MSEPlatform.Playback.Etw.traceString("PlaybackController::_setPlayerMedia: clear all reported errors");
                    var playbackEventNotifications = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.playbackEventNotifications);
                    playbackEventNotifications.clearErrors();
                    this._erroredMediaInstances = [];
                    this._lastErrorEncountered = null;
                    this._hasSameErrors = true;
                    this._hasPlayedSong = false;
                    this._errorCount = 0
                }, _setPlayerMedia: function _setPlayerMedia(mediaInstance) {
                    MSEPlatform.Playback.Etw.traceSetMedia(mediaInstance);
                    this.reset(true);
                    if (!MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.contentRestrictionService)) {
                        var errorCode = this._isParentallyBlocked(mediaInstance);
                        if (errorCode) {
                            var mediaItem = (mediaInstance._mediaItem ? mediaInstance._mediaItem.data : null);
                            MSEPlatform.Playback.firePlaybackError(this._onPlayerEvent.bind(this), errorCode, "_setMedia_isParentallyBlocked", mediaItem);
                            return
                        }
                    }
                    if (mediaInstance && mediaInstance._errorDescriptor) {
                        var mediaItem = (mediaInstance._mediaItem ? mediaInstance._mediaItem.data : null);
                        MSEPlatform.Playback.firePlaybackError(this._onPlayerEvent.bind(this), mediaInstance._errorDescriptor, "_setMedia_preexistingCondition", mediaItem);
                        return
                    }
                    if (this.autoPlay)
                        this.currentTransportState = MSEPlatform.Playback.TransportState.starting;
                    this._canResetOnNullMediaInstance = !MS.Entertainment.Platform.Playback.XPlayer.audioTagForFileActivation;
                    this._player.currentMedia = mediaInstance
                }, _setNextMedia: function _setNextMedia(mediaInstance) {
                    if (!this._isPlayerSet())
                        return;
                    var name = (mediaInstance && mediaInstance._mediaItem && mediaInstance._mediaItem.data) ? mediaInstance._mediaItem.data.name : String.empty;
                    MSEPlatform.Playback.Etw.traceString("PlaybackController::_setNextMedia: name= " + name);
                    if (!mediaInstance) {
                        this._player.nextMedia = null;
                        this._player.controlIsProcessingNextMedia = false;
                        this.endOfPlaylist = true;
                        if ((this.autoPlay && !this.activating && (this.currentTransportState !== MSEPlatform.Playback.TransportState.playing && this.currentTransportState !== MSEPlatform.Playback.TransportState.starting)) || (this.currentTransportState === MSEPlatform.Playback.TransportState.starting && this.error))
                            this._handleErrorsOnEndOfPlaylist();
                        return
                    }
                    MS.Entertainment.Utilities.Telemetry.logPlaybackAttempted(mediaInstance);
                    MSEPlatform.Playback.Etw.traceSetNextMedia(this.readyForNextMedia, mediaInstance);
                    if (!MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.contentRestrictionService)) {
                        var errorCode = this._isParentallyBlocked(mediaInstance);
                        if (errorCode) {
                            mediaInstance._errorDescriptor = {msExtendedCode: errorCode.code};
                            this._handleNextMediaError(true, null, mediaInstance);
                            this._player.controlIsProcessingNextMedia = false;
                            return
                        }
                    }
                    if (mediaInstance && mediaInstance._errorDescriptor) {
                        this._handleNextMediaError(true, null, mediaInstance);
                        this._player.controlIsProcessingNextMedia = false;
                        return
                    }
                    this._player.nextMedia = mediaInstance;
                    this._player.controlIsProcessingNextMedia = false
                }, _isParentallyBlocked: function _isParentallyBlocked(mediaInstance) {
                    var errorCode = null;
                    if (mediaInstance._mediaItem) {
                        var mediaItem = mediaInstance._mediaItem.data;
                        if (mediaItem)
                            if (mediaItem.isExplicit && !mediaItem.inCollection) {
                                var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                                if (!signedInUser.xuid)
                                    errorCode = MS.Entertainment.Platform.Playback.Error.X8_E_PLAYBACK_LOGON_TO_PLAY_EXPLICIT_CONTENT;
                                else if (!signedInUser.hasExplicitPrivilege)
                                    errorCode = MS.Entertainment.Platform.Playback.makeParentallyBlockedError()
                            }
                    }
                    return errorCode
                }, _onPlayerEvent: function _onPlayerEvent(event) {
                    switch (event.type)
                    {
                        case"loadedmetadata":
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::onPlayerEvent, loadedmetadata");
                            this.errorDescriptor = null;
                            if (event.srcElement.durationOverrideMS)
                                this.duration = event.srcElement.durationOverrideMS;
                            else if (isFinite(event.srcElement.duration))
                                this.duration = Math.round(event.srcElement.duration * 1000);
                            if (!this._player) {
                                MSEPlatform.Playback.Etw.traceString("PlaybackController::onPlayerEvent, loadedmetadata - player is null");
                                MS.Entertainment.Utilities.fail("loadedmetadata - player is null")
                            }
                            if (!this._player._currentMedia) {
                                MSEPlatform.Playback.Etw.traceString("PlaybackController::onPlayerEvent, loadedmetadata - currentMedia is null");
                                MS.Entertainment.Utilities.fail("loadedmetadata - currentMedia is null")
                            }
                            MSEPlatform.Playback.Etw.traceMediaLoaded(false, this._player._currentMedia, this.duration);
                            if (this._player._currentMedia)
                                this._player._currentMedia.alternateSource = null;
                            this._observableNoLossAssign("playerState", MS.Entertainment.Platform.Playback.PlayerState.ready);
                            if (this._player && this._player._currentMedia && !this._player._currentMedia.isAudioAd) {
                                if (!this.hasPrerolledMedia()) {
                                    MSEPlatform.Playback.Etw.traceString("PlaybackController::onPlayerEvent, loadedmetadata - raising readyForNextMedia");
                                    this.readyForNextMedia = true
                                }
                            }
                            else if (this._nextMediaInstanceAfterAd)
                                this._setNextMedia(this._nextMediaInstanceAfterAd);
                            this.videoWidth = event.srcElement.videoWidth;
                            this.videoHeight = event.srcElement.videoHeight;
                            if (this._targetPosition === 0 && this._player && this._player._currentMedia && this._player._currentMedia.startPosition > 0 && !this.isRemoteSession())
                                this._targetPosition = this._player._currentMedia.startPosition;
                            if (this._targetTransportState !== MSEPlatform.Playback.TransportState.unInitialize && this._targetTransportState !== this.currentTransportState)
                                this._applyTargetTransportState(this._targetTransportState);
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::onPlayerEvent, loadedmetadata, autoplay=" + this.autoPlay);
                            if (this.autoPlay && this._player) {
                                MSEPlatform.Playback.Etw.traceString("PlaybackController::onPlayerEvent, loadedmetadata: calling xplayer.play(), autoplay=true");
                                this._player.play()
                            }
                            break;
                        case"timeupdate":
                            if (event.srcElement && event.srcElement.currentTime)
                                this.currentPosition = Math.round(event.srcElement.currentTime * 1000);
                            break;
                        case"durationchange":
                            if (event.srcElement.durationOverrideMS)
                                this.duration = event.srcElement.durationOverrideMS;
                            else if (event.srcElement && event.srcElement.duration && isFinite(event.srcElement.duration))
                                this.duration = Math.round(event.srcElement.duration * 1000);
                            break;
                        case"playing":
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::onPlayerEvent: Playing");
                            if (this._targetPosition > 0) {
                                this.seekToPosition(this._targetPosition);
                                this._targetPosition = 0
                            }
                            this._targetTransportState = MSEPlatform.Playback.TransportState.playing;
                            this.currentTransportState = MSEPlatform.Playback.TransportState.playing;
                            if (this._player && this._player.currentMedia && this._player.currentMedia.protectionState && (this._player.currentMedia.protectionState === MS.Entertainment.Platform.Playback.ProtectionState.drmProtected)) {
                                MS.Entertainment.Instrumentation.PerfTrack.triggerScenarioAppLaunchPlayProtectedContent();
                                MS.Entertainment.Instrumentation.PerfTrack.triggerScenarioPlayProtectedInApp()
                            }
                            else {
                                MS.Entertainment.Instrumentation.PerfTrack.triggerScenarioAppLaunchPlayNonProtectedContent();
                                MS.Entertainment.Instrumentation.PerfTrack.triggerScenarioPlayNonProtectedInApp()
                            }
                            MS.Entertainment.Instrumentation.PerfTrack.disableScenarioAppLaunchPlayProtectedContent();
                            MS.Entertainment.Instrumentation.PerfTrack.disableScenarioPlayProtectedInApp();
                            MS.Entertainment.Instrumentation.PerfTrack.disableScenarioAppLaunchPlayNonProtectedContent();
                            MS.Entertainment.Instrumentation.PerfTrack.disableScenarioPlayNonProtectedInApp();
                            this._hasPlayedSong = true;
                            this._errorCount = 0;
                            MS.Entertainment.Utilities.Telemetry.logPlayHappened(this, this.forceTimeUpdate());
                            break;
                        case"pause":
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::onPlayerEvent: pause");
                            if (this._targetTransportState === MSEPlatform.Playback.TransportState.stopped)
                                this.currentTransportState = MSEPlatform.Playback.TransportState.stopped;
                            else if (this.currentTransportState === MSEPlatform.Playback.TransportState.stopped) {
                                MSEPlatform.Playback.Etw.traceString("PlaybackController::onPlayerEvent: pause - Current transport state is stopped. Ignoring paused request.");
                                return
                            }
                            else {
                                this._targetTransportState = MSEPlatform.Playback.TransportState.paused;
                                this.currentTransportState = MSEPlatform.Playback.TransportState.paused;
                                MS.Entertainment.Utilities.Telemetry.logPauseHappened(this, this.forceTimeUpdate())
                            }
                            break;
                        case"ended":
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::onPlayerEvent: ended");
                            if (!this.hasPrerolledMedia()) {
                                this.currentTransportState = MSEPlatform.Playback.TransportState.stopped;
                                this._targetTransportState = MSEPlatform.Playback.TransportState.stopped;
                                if (this.endOfPlaylist || (this._lastErrorEncountered && this._lastErrorEncountered.isCritical))
                                    this._handleErrorsOnEndOfPlaylist()
                            }
                            MS.Entertainment.Utilities.Telemetry.logEndHappened(this, this.duration);
                            this.currentPosition = 0;
                            this._reportPrerollErrors();
                            if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.playbackErrorDisplayService)) {
                                var displayService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.playbackErrorDisplayService);
                                if (displayService)
                                    displayService.showDialogForNonCriticalErrors = false
                            }
                            break;
                        case"seeked":
                            this._fireSeekedPositionChanged(this.currentPosition);
                            break;
                        case"error":
                            MS.Entertainment.Instrumentation.PerfTrack.disableAllStartupScenarios();
                            MS.Entertainment.Instrumentation.PerfTrack.disableScenarioPlayProtectedInApp();
                            MS.Entertainment.Instrumentation.PerfTrack.disableScenarioPlayNonProtectedInApp();
                            this.activating = false;
                            var currentMediaInstance = this._player ? this._player.currentMedia || this._currentMedia : null;
                            if (currentMediaInstance === null || (this.isRemoteSessionRunning && event.target.error.code === MSEPlatform.Playback.MediaTagError.MEDIA_ERR_SRC_NOT_SUPPORTED.code)) {
                                MSEPlatform.Playback.Etw.traceString("PlaybackController::onPlayerEvent: Error to ignore: error: code=" + MSEPlatform.Playback.errorToString(event.target.error.code));
                                break
                            }
                            var name = (currentMediaInstance && currentMediaInstance._mediaItem && currentMediaInstance._mediaItem.data) ? currentMediaInstance._mediaItem.data.name : String.empty;
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::onPlayerEvent: error: " + MSEPlatform.Playback.mediaErrorToString(event.target.error) + ", itemName=" + name);
                            if (event.srcElement)
                                event.srcElement.errorCode = event.target.error.code;
                            if (currentMediaInstance && currentMediaInstance.alternateVideoEncoding !== null) {
                                currentMediaInstance.videoEncoding = currentMediaInstance.alternateVideoEncoding;
                                currentMediaInstance.alternateVideoEncoding = null;
                                this._setMedia(currentMediaInstance)
                            }
                            else if (currentMediaInstance && currentMediaInstance.alternateSource) {
                                currentMediaInstance.source = currentMediaInstance.alternateSource;
                                currentMediaInstance.alternateSource = null;
                                this._setMedia(currentMediaInstance)
                            }
                            else if (event.srcElement && event.srcElement.fastStartProperties && event.srcElement.error && event.srcElement.error.msExtendedCode === MSEPlatform.Playback.Error.MF_E_DRM_UNSUPPORTED.code) {
                                currentMediaInstance.disableFastStart = true;
                                this._setMedia(currentMediaInstance)
                            }
                            else {
                                var err = event.target.error;
                                err.context = "onPlayerEvent: Error from tag";
                                this._handleCurrentMediaError(err)
                            }
                            break
                    }
                }, _fireSeekedPositionChanged: function _fireSeekedPositionChanged(currentPosition) {
                    this.seekedPosition = currentPosition;
                    this._reschedulePrerollCallback()
                }, _handleErrorsOnEndOfPlaylist: function _handleErrorsOnEndOfPlaylist() {
                    var displayError = null;
                    if (this._lastErrorEncountered) {
                        var displayError = MS.Entertainment.Platform.Playback.Error.NS_E_WMP_MULTIPLE_ERROR_IN_PLAYLIST;
                        if (this._hasSameErrors)
                            displayError = this._lastErrorEncountered;
                        if (!this._hasPlayedSong)
                            displayError.isCritical = true
                    }
                    this._player._nextMediaStarted = false;
                    this._player._switchPlayerAsync().done(function setErrorStateOnEnd() {
                        if (displayError) {
                            this.errorDescriptor = displayError;
                            this._observableNoLossAssign("playerState", MS.Entertainment.Platform.Playback.PlayerState.error);
                            this._lastErrorEncountered = null
                        }
                    }.bind(this), function dontcare(){})
                }, _handleNextMediaLoaded: function _handleNextMediaLoaded(isLoaded, iPlayback) {
                    if (isLoaded) {
                        iPlayback.readyForNextMedia = false;
                        this._releaseNetwork();
                        MSEPlatform.Playback.Etw.traceMediaLoaded(true, iPlayback._player._nextMedia, iPlayback._player._nextPlayer ? iPlayback._player._nextPlayer.duration * 1000 : 0);
                        if (this._skipButtonPressed || (iPlayback.currentMedia && iPlayback.currentMedia._errorDescriptor)) {
                            this._skipButtonPressed = false;
                            iPlayback.skipToNextPrerolled()
                        }
                    }
                }, _handleNextMediaStarted: function _handleNextMediaStarted(isStarted, iPlayback) {
                    MSEPlatform.Playback.Etw.traceString("PlaybackController::_handleNextMediaStarted: " + isStarted);
                    if (isStarted) {
                        iPlayback._internalCurrentMedia = iPlayback._player._currentMedia;
                        if (!iPlayback._internalCurrentMedia) {
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::_handleNextMediaStarted: currentMedia null");
                            iPlayback.currentMedia = null
                        }
                        if (iPlayback._player._currentMedia && !iPlayback._player._currentMedia.isAudioAd) {
                            iPlayback.currentMedia = iPlayback._player._currentMedia;
                            iPlayback.errorDescriptor = iPlayback._player._currentMedia._errorDescriptor
                        }
                        iPlayback.isAudioAd = iPlayback._player._currentMedia && iPlayback._player._currentMedia.isAudioAd;
                        iPlayback._observableNoLossAssign("playerState", MS.Entertainment.Platform.Playback.PlayerState.ready);
                        if (iPlayback._player._currentPlayer)
                            iPlayback.duration = Math.round(iPlayback._player._currentPlayer.duration * 1000);
                        if (iPlayback.currentTransportState !== MSEPlatform.Playback.TransportState.playing)
                            iPlayback.currentPosition = 0;
                        if (iPlayback.currentMedia) {
                            MSEPlatform.Playback.Etw.traceNextMediaStarted(iPlayback.currentMedia.source);
                            MS.Entertainment.Utilities.Telemetry.logPlaybackHappened(iPlayback.currentMedia)
                        }
                        this._startProcessingNextMedia()
                    }
                }, _startProcessingNextMedia: function _startProcessingNextMedia() {
                    if (this._internalCurrentMedia) {
                        MSEPlatform.Playback.Etw.traceString("PlaybackController::_handleNextMediaStarted - raising readyForNextMedia");
                        this._signalForNextMedia()
                    }
                }, _handleNextMediaError: function _handleNextMediaError(isError, unReferrencedParam, mediaInstance) {
                    if (isError) {
                        var error;
                        if (!mediaInstance)
                            mediaInstance = this._player._nextMedia;
                        if (mediaInstance && mediaInstance.alternateVideoEncoding !== null) {
                            mediaInstance.videoEncoding = mediaInstance.alternateVideoEncoding;
                            mediaInstance.alternateVideoEncoding = null;
                            this._player._setNextMedia(mediaInstance);
                            return
                        }
                        if (mediaInstance) {
                            if (!this._shouldIgnoreThisErrorForBlocking(mediaInstance._errorDescriptor.code))
                                this._addErroredMediaInstance(mediaInstance);
                            error = this.errorDescriptor;
                            MS.Entertainment.Utilities.Telemetry.logPlaybackError(mediaInstance, error, "PlaybackController_handleNextMediaError")
                        }
                        this._player.reset(true);
                        this._reportPrerollErrors();
                        if (!this._isCriticalError(error))
                            this._signalForNextMedia();
                        else if (!this._hasPlayedSong && this._errorCount >= this._maxSequentialErrors)
                            this._observableNoLossAssign("playerState", MS.Entertainment.Platform.Playback.PlayerState.error)
                    }
                }, _remapPlaybackError: function _remapPlaybackError(error) {
                    return error
                }, _handleCurrentMediaError: function _handleCurrentMediaError(error) {
                    error = this._remapPlaybackError(error);
                    if (this.isRemoteSession() && error && error.code === 3)
                        this.errorDescriptor = {msExtendedCode: MS.Entertainment.Platform.Playback.Error.X8_E_PLAYBACK_PLAYTO_ERR_DECODE.code};
                    else {
                        this.errorDescriptor = error;
                        this.errorDescriptor.mediaItem = (this.currentMedia && this.currentMedia._mediaItem) ? this.currentMedia._mediaItem.data : null
                    }
                    if (this._player && this._player.currentMedia) {
                        error.errorOrdinal = this._player.currentMedia.cookie;
                        this._player.currentMedia._errorDescriptor = error
                    }
                    this._observableNoLossAssign("playerState", MS.Entertainment.Platform.Playback.PlayerState.error);
                    var errorContext = error.context || "PlaybackController_handleCurrentMediaError";
                    MSEPlatform.Playback.Etw.tracePlaybackError(error.code, error.msExtendedCode, errorContext);
                    this._lastErrorEncountered = error;
                    MS.Entertainment.Utilities.Telemetry.logPlaybackError(this.currentMedia, error, errorContext);
                    this._processPlaybackErrorPostMapping(error)
                }, _processPlaybackErrorPostMapping: function _processPlaybackErrorPostMapping(error){}, _signalForNextMedia: function _signalForNextMedia() {
                    this._player.controlIsProcessingNextMedia = true;
                    this.readyForNextMedia = false;
                    MSEPlatform.Playback.Etw.traceString("PlaybackController::signalForNextMedia requested");
                    if (this._signalForNextMediaPromise) {
                        MSEPlatform.Playback.Etw.traceString("PlaybackController::signalForNextMedia: cancel previous");
                        this._signalForNextMediaPromise.cancel()
                    }
                    this._signalForNextMediaPromise = WinJS.Promise.timeout(10);
                    this._signalForNextMediaPromise.then(function() {
                        MSEPlatform.Playback.Etw.traceString("PlaybackController::signalForNextMedia: raising readyForNextMedia");
                        this.readyForNextMedia = true;
                        this._signalForNextMediaPromise = null
                    }.bind(this))
                }, _isCriticalError: function _isCriticalError(error) {
                    if (!error)
                        return false;
                    else if (this._errorCount >= this._maxSequentialErrors)
                        return true;
                    switch (error.msExtendedCode) {
                        case MSEPlatform.Playback.Error.X8_E_PLAYBACK_STOPPED_DATA_LIMIT_APPROACHING.code:
                        case MSEPlatform.Playback.Error.E_MDS_ROAMING_LIMIT.code:
                        case MSEPlatform.Playback.Error.ERROR_GRAPHICS_ONLY_CONSOLE_SESSION_SUPPORTED.code:
                        case MSEPlatform.Playback.Error.MF_E_AUDIO_PLAYBACK_DEVICE_INVALIDATED.code:
                        case MSEPlatform.Playback.Error.MF_E_CANNOT_CREATE_SINK.code:
                        case MSEPlatform.Playback.Error.MF_E_DEBUGGING_NOT_ALLOWED.code:
                        case MSEPlatform.Playback.Error.MF_E_HIGH_SECURITY_LEVEL_CONTENT_NOT_ALLOWED.code:
                        case MSEPlatform.Playback.Error.ZEST_E_MW_CONCURRENT_STREAM.code:
                        case MSEPlatform.Playback.Error.ZEST_E_MEDIAINSTANCE_STREAMING_OCCUPIED.code:
                        case MSEPlatform.Playback.Error.ZEST_E_MULTITUNER_CONCURRENTSTREAMING_DETECTED.code:
                        case MSEPlatform.Playback.Error.NS_E_COMPRESSED_DIGITAL_AUDIO_PROTECTION_LEVEL_UNSUPPORTED.code:
                        case MSEPlatform.Playback.Error.NS_E_UNCOMPRESSED_DIGITAL_AUDIO_PROTECTION_LEVEL_UNSUPPORTED.code:
                        case MSEPlatform.Playback.Error.NS_E_DRM_DRIVER_AUTH_FAILURE.code:
                        case MSEPlatform.Playback.Error.NS_E_DRM_NEEDS_INDIVIDUALIZATION.code:
                        case MSEPlatform.Playback.Error.NS_E_DRM_UNABLE_TO_INITIALIZE.code:
                        case MSEPlatform.Playback.Error.NS_E_WMP_AUDIO_HW_PROBLEM.code:
                        case MSEPlatform.Playback.Error.NS_E_WMP_BAD_DRIVER.code:
                        case MSEPlatform.Playback.Error.E_MDS_UNAUTHENTICATED_TRACK_LIMIT.code:
                            return true;
                        default:
                            return false
                    }
                }, _fireCriticalPlaybackError: function _fireCriticalPlaybackError(error) {
                    var mediaItem = (this.currentMedia && this.currentMedia._mediaItem ? this.currentMedia._mediaItem.data : null);
                    error.isCritical = true;
                    MSEPlatform.Playback.firePlaybackError(this._onPlayerEvent.bind(this), error, "PlaybackController_fireCriticalError", mediaItem);
                    this._errorCount = 0
                }, _shouldIgnoreThisErrorForBlocking: function _shouldIgnoreThisErrorForBlocking(error) {
                    return (error === MS.Entertainment.Platform.Playback.Error.X8_E_PLAYBACK_MEDIA_ERR_NOT_LOCAL.code)
                }, _addErroredMediaInstance: function _addErroredMediaInstance(mediaInstance) {
                    if (!mediaInstance)
                        return;
                    var error = null;
                    var errorCode = null;
                    if (mediaInstance._errorDescriptor) {
                        error = MS.Entertainment.Platform.Playback.makePlaybackError(mediaInstance._errorDescriptor);
                        errorCode = error.msExtendedCode;
                        error.errorOrdinal = mediaInstance.cookie
                    }
                    if (this._hasSameErrors && errorCode && this._lastErrorEncountered && this._lastErrorEncountered.msExtendedCode !== errorCode)
                        this._hasSameErrors = false;
                    this._lastErrorEncountered = error;
                    this._erroredMediaInstances.push(mediaInstance);
                    this._errorCount++;
                    if (this._isCriticalError(error)) {
                        error.isCritical = true;
                        this.errorDescriptor = error
                    }
                }, _reportPrerollErrors: function _reportPrerollErrors() {
                    var playbackEventNotifications = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.playbackEventNotifications);
                    while (this._erroredMediaInstances.length > 0) {
                        var mediaInstance = this._erroredMediaInstances.pop();
                        var mediaItem = null;
                        var errorCode = null;
                        if (mediaInstance) {
                            if (mediaInstance._mediaItem) {
                                mediaItem = mediaInstance._mediaItem.data;
                                if (mediaItem && mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.video && mediaInstance._mediaItem.trackItem)
                                    mediaItem = mediaInstance._mediaItem.trackItem.data
                            }
                            if (mediaInstance._errorDescriptor) {
                                var error = MS.Entertainment.Platform.Playback.makePlaybackError(mediaInstance._errorDescriptor);
                                errorCode = error.msExtendedCode
                            }
                        }
                        playbackEventNotifications.setError(MS.Entertainment.Platform.Playback.PlaybackEventNotifications.getEventingMediaId(mediaItem), errorCode)
                    }
                }, _handleNetworkConnectionChanged: function _handleNetworkConnectionChanged(networkConnection) {
                    switch (networkConnection) {
                        case MS.Entertainment.Platform.NetworkConnection.approachingDataLimit:
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::_handleNetworkConnectionChanged: approachingDataLimit");
                            this._pauseStreamingAndFireError(MSEPlatform.Playback.Error.X8_E_PLAYBACK_STOPPED_DATA_LIMIT_APPROACHING);
                            break;
                        case MS.Entertainment.Platform.NetworkConnection.overDataLimit:
                        case MS.Entertainment.Platform.NetworkConnection.switchedToMetered:
                            MSEPlatform.Playback.Etw.traceString("PlaybackController::_handleNetworkConnectionChanged: overDataLimit");
                            break
                    }
                }, _pauseStreamingAndFireError: function _pauseStreamingAndFireError(error) {
                    MSEPlatform.Playback.Etw.traceString("PlaybackController::_pauseStreamingAndFireError");
                    if (!this._isPlayerSet() || this.currentTransportState === MSEPlatform.Playback.TransportState.unInitialize || this.currentTransportState === MSEPlatform.Playback.TransportState.stopped || this.currentTransportState === MSEPlatform.Playback.TransportState.paused)
                        return;
                    if (!this.currentMedia || this.currentMedia.isLocal)
                        return;
                    this._player.pause();
                    this._targetTransportState = MSEPlatform.Playback.TransportState.paused;
                    this._fireCriticalPlaybackError(error)
                }, _applyTargetTransportState: function _applyTargetTransportState(value) {
                    this._targetTransportState = value;
                    try {
                        switch (value)
                        {
                            case MSEPlatform.Playback.TransportState.stopped:
                                if (this.currentTransportState === MSEPlatform.Playback.TransportState.paused)
                                    this.currentTransportState = MSEPlatform.Playback.TransportState.stopped;
                                else if (this.currentTransportState !== MSEPlatform.Playback.TransportState.stopped)
                                    this._player.stop();
                                break;
                            case MSEPlatform.Playback.TransportState.paused:
                                this._player.pause();
                                if (!this._currentMedia || !this._currentMedia.videoAdSupported)
                                    this.autoPlay = false;
                                break;
                            case MSEPlatform.Playback.TransportState.playing:
                                if (this.currentTransportState !== MSEPlatform.Playback.TransportState.paused)
                                    this.currentTransportState = MSEPlatform.Playback.TransportState.starting;
                                this._player.play();
                                this.activating = false;
                                if (!this._currentMedia || !this._currentMedia.videoAdSupported)
                                    this.autoPlay = true;
                                break;
                            default:
                                throw"PlaybackController_applyTargetTransportState: Error! Unsupported state - " + value;
                        }
                    }
                    catch(exception) {
                        if (this._player._currentPlayer === null)
                            this.currentTransportState = MSEPlatform.Playback.TransportState.stopped;
                        else
                            throw exception;
                    }
                }, currentMedia: MS.Entertainment.UI.Framework.observableProperty("currentMedia", null), currentPosition: MS.Entertainment.UI.Framework.observableProperty("currentPosition", 0), currentTransportState: MS.Entertainment.UI.Framework.observableProperty("currentTransportState", MS.Entertainment.Platform.Playback.TransportState.stopped), isAudioAd: MS.Entertainment.UI.Framework.observableProperty("isAudioAd", false), isPreview: MS.Entertainment.UI.Framework.observableProperty("isPreview", false), duration: MS.Entertainment.UI.Framework.observableProperty("duration", 0), errorDescriptor: MS.Entertainment.UI.Framework.observableProperty("errorDescriptor", null), nextMedia: MS.Entertainment.UI.Framework.observableProperty("nextMedia", null), playerState: MS.Entertainment.UI.Framework.observableProperty("playerState", MS.Entertainment.Platform.Playback.PlayerState.notReady), readyForNextMedia: MS.Entertainment.UI.Framework.observableProperty("readyForNextMedia", false), videoWidth: MS.Entertainment.UI.Framework.observableProperty("videoWidth", 0), videoHeight: MS.Entertainment.UI.Framework.observableProperty("videoHeight", 0), playbackRate: MS.Entertainment.UI.Framework.observableProperty("playbackRate", 1), minPlaybackRate: MS.Entertainment.UI.Framework.observableProperty("minPlaybackRate", 0), maxPlaybackRate: MS.Entertainment.UI.Framework.observableProperty("maxPlaybackRate", 0), _isVideo: MS.Entertainment.UI.Framework.observableProperty("_isVideo", false), seekedPosition: MS.Entertainment.UI.Framework.observableProperty("seekedPosition", 0), isRemoteSessionRunning: MS.Entertainment.UI.Framework.observableProperty("isRemoteSessionRunning", false), playToSenderConnectionState: MS.Entertainment.UI.Framework.observableProperty("playToSenderConnectionState", Windows.Media.PlayTo.PlayToConnectionState.disconnected), mediaElement: MS.Entertainment.UI.Framework.observableProperty("mediaElement", false), error: MS.Entertainment.UI.Framework.observableProperty("error", null), _configManager: null, _offlineNetworkRequestDelay: 100, _internalCurrentMedia: null, _player: null, _targetTransportState: MSEPlatform.Playback.TransportState.unInitialize, _targetPosition: 0, _sessionMgr: null, _erroredMediaInstances: [], _signalForNextMediaPromise: null, _nextMediaInstanceAfterAd: null, _skipButtonPressed: false, _prerollPromise: null, _prerollPromiseOrdinal: 0, _lastErrorEncountered: null, _hasSameErrors: true, _hasPlayedSong: false, _errorCount: 0, _maxSequentialErrors: 25, _canResetOnNullMediaInstance: true
        }, {createInstance: function createInstance() {
                if (MS.Entertainment.Utilities.isMusicApp1)
                    return new MS.Entertainment.Platform.Playback.PlaybackControllerMusic1;
                else if (MS.Entertainment.Utilities.isMusicApp2)
                    return new MS.Entertainment.Platform.Playback.PlaybackControllerMusic;
                else if (MS.Entertainment.Utilities.isVideoApp)
                    return new MS.Entertainment.Platform.Playback.PlaybackControllerVideo;
                else if (MS.Entertainment.Utilities.isTestApp)
                    return new MS.Entertainment.Platform.Playback.PlaybackController;
                else
                    MS.Entertainment.Utilities.fail("PlaybackController.createInstance() Wrong app mode")
            }})})
})()
