﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Platform.Playback");
(function() {
    "use strict";
    var MSEPlatform = WinJS.Namespace.define("MS.Entertainment.Platform", null);
    function tackOnTimeAfterExecution(fun, timeInMs, id) {
        return new WinJS.Promise(function init(c) {
                MSEPlatform.Playback.Etw.traceString("XPlayer::_watchTimeSeparated execution of " + id);
                var value = fun();
                return WinJS.Promise.timeout(timeInMs).then(function timeExpired() {
                        c(value)
                    })
            })
    }
    WinJS.Namespace.defineWithParent(MSEPlatform, "Playback", {
        shouldDisableDlnaPlayTo: function shouldDisableDlnaPlayTo(mediaInstance) {
            var mediaType = mediaInstance.mediaType;
            var disable = false;
            if (mediaType === Microsoft.Entertainment.Queries.ObjectType.video) {
                if (!MS.Entertainment.Utilities.isWindowsBlue && mediaInstance.protectionState === MSEPlatform.Playback.ProtectionState.drmProtected)
                    disable = true
            }
            else if (!MS.Entertainment.Utilities.isWindowsBlue && !mediaInstance.isLocal && mediaInstance.protectionState === MSEPlatform.Playback.ProtectionState.drmProtected)
                disable = true;
            return disable
        }, XPlayer: MS.Entertainment.UI.Framework.derive("MS.Entertainment.UI.Framework.ObservableBase", function XPlayer_constructor(playerContainer) {
                this._playerContainer = playerContainer;
                this._setupDlnaPlayToMgr();
                if (MS.Entertainment.Utilities.SignIn)
                    MS.Entertainment.Utilities.SignIn.addEventListener("signInComplete", function() {
                        this._createFastStartAudioTag()
                    }.bind(this));
                this._onAppResume = this._onAppResume.bind(this);
                this._onAppSuspending = this._onAppSuspending.bind(this);
                Windows.UI.WebUI.WebUIApplication.addEventListener("resuming", this._onAppResume);
                Windows.UI.WebUI.WebUIApplication.addEventListener("suspending", this._onAppSuspending);
                if (MS.Entertainment.Utilities.isMusicApp1) {
                    this._volumeController = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.volumeService);
                    this._volumeControllerBindings = WinJS.Binding.bind(this._volumeController, {
                        volume: this._onVolumeValueStateChange.bind(this), mute: this._onVolumeValueStateChange.bind(this)
                    });
                    if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.uiState)) {
                        var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                        MS.Entertainment.UI.Framework.addEventHandlers(uiStateService, {isAppVisibleChanged: this._onAppVisibleChanged.bind(this)})
                    }
                }
            }, {
                _contentRestrictionService: {get: (function() {
                        var contentRestrictionServiceHolder = null;
                        return function _contentRestrictionServiceGetter() {
                                if (contentRestrictionServiceHolder === null && MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.contentRestrictionService))
                                    contentRestrictionServiceHolder = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.contentRestrictionService);
                                return contentRestrictionServiceHolder
                            }
                    })()}, dispose: function dispose() {
                        this._cleanupDlnaPlayToMgr();
                        if (MS.Entertainment.Utilities.SignIn)
                            MS.Entertainment.Utilities.SignIn.removeEventListener("signInComplete");
                        Windows.UI.WebUI.WebUIApplication.removeEventListener("resuming", this._onAppResume);
                        Windows.UI.WebUI.WebUIApplication.removeEventListener("suspending", this._onAppSuspending);
                        if (this._volumeControllerBindings) {
                            this._volumeControllerBindings.cancel();
                            this._volumeControllerBindings = null
                        }
                        this._playerContainer = null
                    }, autoPlay: {
                        get: function autoPlay_get() {
                            return this._autoPlay
                        }, set: function autoPlay_set(value) {
                                this._autoPlay = value
                            }
                    }, currentMedia: {
                        get: function currentMedia_get() {
                            return this._currentMedia
                        }, set: function currentMedia_set(value) {
                                this._setMedia(value)
                            }
                    }, nextMedia: {
                        get: function nextMedia_get() {
                            return this._nextMedia
                        }, set: function nextMedia_set(value) {
                                this._setNextMedia(value)
                            }
                    }, muted: {
                        get: function muted_get() {
                            this._ensurePipeline();
                            return this._currentPlayer.muted
                        }, set: function muted_set(value) {
                                this._ensurePipeline();
                                this._currentPlayer.muted = value
                            }
                    }, play: function play() {
                        MSEPlatform.Playback.Etw.traceString("XPlayer::Play()");
                        this._ensurePipeline();
                        this._currentPlayer.play()
                    }, pause: function pause() {
                        this._ensurePipeline();
                        this._currentPlayer.pause()
                    }, stop: function stop() {
                        this._ensurePipeline();
                        this._currentPlayer.stop();
                        if (this.currentMedia)
                            this.currentMedia.stopPosition = this.forceTimeUpdate()
                    }, seekToPosition: function seekToPosition(positionMsec) {
                        this._ensurePipeline();
                        try {
                            this._currentMedia.maxPosition = this._currentPlayer.currentTime;
                            this._currentPlayer.seekToPosition(positionMsec);
                            this._currentMedia.maxPosition = positionMsec / 1000
                        }
                        catch(e) {
                            MSEPlatform.Playback.Etw.tracePlaybackError(e.code, e.msExtendedCode, "XPlayer::seekToPosition")
                        }
                    }, fastFwd: function fastFwd(){}, rewind: function rewind(){}, slowFwd: function slowFwd(){}, slowRewind: function slowRewind(){}, _removePlayerFromDOM: function _removePlayerFromDOM(player) {
                        var playerElement = player.getElement();
                        if (this._playerContainer && this._playerContainer.contains(playerElement))
                            this._playerContainer.removeChild(playerElement);
                        if (player.parentNode && player.parentNode.contains(playerElement))
                            player.parentNode.removeChild(playerElement)
                    }, _removePlayerSrc: function _removePlayerSrc(player, removeFromDOM) {
                        if (player) {
                            if (removeFromDOM)
                                this._removePlayerFromDOM(player);
                            if (player.media && !player.media._errorDescriptor)
                                if (player.media.source && player.media.source.match && player.media.source.match(/^blob*:/i))
                                    URL.revokeObjectURL(player.media.source);
                            player.dispose()
                        }
                    }, reset: function reset(resetNext) {
                        MSEPlatform.Playback.Etw.traceString("XPlayer::_reset:  resetNext = " + resetNext);
                        if (resetNext) {
                            if (this._nextPlayer) {
                                this._unhookNextMediaEvents();
                                this._nextMedia = null;
                                this._removePlayerSrc(this._nextPlayer, false);
                                this._nextPlayer = null
                            }
                        }
                        else if (this._currentPlayer && !this._isDlnaConnectionPresent()) {
                            this._unhookCurrentMediaEvents();
                            this._currentMedia = null;
                            this._removePlayerSrc(this._currentPlayer, true);
                            this._currentPlayer = null
                        }
                    }, enableTimeUpdate: function enableTimeUpdate() {
                        if (this._currentMediaEventsCallback && this._currentPlayer) {
                            this._currentPlayer.removeEventListener("timeupdate", this._currentMediaEventsCallback);
                            this._currentPlayer.addEventListener("timeupdate", this._currentMediaEventsCallback, false)
                        }
                    }, disableTimeUpdate: function disableTimeUpdate() {
                        if (this._currentMediaEventsCallback && this._currentPlayer)
                            this._currentPlayer.removeEventListener("timeupdate", this._currentMediaEventsCallback)
                    }, forceTimeUpdate: function forceTimeUpdate() {
                        if (this._currentPlayer)
                            return this._currentPlayer.currentTime * 1000
                    }, isRemoteSession: function isRemoteSession() {
                        return this._isDlnaConnectionPresent()
                    }, _prePlayAsync: function _prePlay_Async(mediaInstance) {
                        function fireExplicitContentBlockedErrorAndFail(error, context, message) {
                            if (error) {
                                message += " Details: ";
                                try {
                                    message += JSON.stringify(error)
                                }
                                catch(ex) {
                                    message += error.toString()
                                }
                            }
                            MSEPlatform.Playback.Etw.traceString("XPlayer::_fireExplicitContentBlockedErrorAndFail: explicit content blocked.  context: " + context + ", message: " + message);
                            MSEPlatform.Playback.fail(message);
                            var playbackError = MS.Entertainment.Platform.Playback.makePlaybackError(MSEPlatform.Playback.makeParentallyBlockedError(), context);
                            playbackError.innerError = error;
                            return WinJS.Promise.wrapError(playbackError)
                        }
                        if (!mediaInstance || !mediaInstance._ratedContentDescriptionPromise)
                            return WinJS.Promise.as();
                        return mediaInstance._ratedContentDescriptionPromise.then(function gotRatedContentDescription(ratedContentDescription) {
                                return this._contentRestrictionService.requestContentAccessAsync(ratedContentDescription)
                            }.bind(this), function gotRatedContentDescription_failed(error) {
                                var msg = "_ratedContentDescriptionPromise retuned an unexpected async error.";
                                return fireExplicitContentBlockedErrorAndFail(error, "gotRatedContentDescription_failed", msg)
                            }).then(function requestContentAccessAsync_complete(canPlay) {
                                if (canPlay)
                                    return;
                                var itemName = "No title";
                                try {
                                    itemName = mediaInstance._mediaItem.data.name
                                }
                                catch(ex) {
                                    {}
                                }
                                MS.Entertainment.Platform.Playback.Etw.traceString("RESTRICTED ITEM: " + itemName);
                                var errorCode = MSEPlatform.Playback.makeParentallyBlockedError();
                                var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                                if (!signedInUser.xuid)
                                    errorCode = MSEPlatform.Playback.Error.X8_E_PLAYBACK_LOGON_TO_PLAY_EXPLICIT_CONTENT;
                                return WinJS.Promise.wrapError(MSEPlatform.Playback.makePlaybackError(errorCode, "ParentalControl"))
                            }, function requestContentAccessAsync_failed(error) {
                                var msg = "requestContentAccessAsync retuned an unexpected async error.";
                                return fireExplicitContentBlockedErrorAndFail(error, "requestContentAccessAsync_failed", msg)
                            })
                    }, _setMedia: function _setMedia(mediaInstance) {
                        if (!mediaInstance) {
                            MSEPlatform.Playback.Etw.traceString("XPlayer::setMedia():  null mediaInstance!");
                            throw"XPlayer::setMedia(): null mediaInstance!";
                        }
                        this._nextMediaStarted = false;
                        this._prePlayAsync(mediaInstance).done(function _prePlayAsync_set_media_completed() {
                            var name = (mediaInstance._mediaItem && mediaInstance._mediaItem.data) ? mediaInstance._mediaItem.data.name : String.empty;
                            MSEPlatform.Playback.Etw.traceString("+XPlayer::_setMedia: name=[" + name + "], source=[" + mediaInstance.source + "]");
                            if (this._isDlnaConnectionPresent()) {
                                var dlnaNextPlayer;
                                if (!MSEPlatform.Playback.shouldDisableDlnaPlayTo(mediaInstance)) {
                                    this._unhookCurrentMediaEvents();
                                    try {
                                        dlnaNextPlayer = this._createMediaPlayer(mediaInstance, false);
                                        if (!dlnaNextPlayer.msPlayToDisabled) {
                                            MSEPlatform.Playback.Etw.traceString("XPlayer::_setMedia, set PlayToSource.next on current player and start playto");
                                            this._configurePlayToByRefMediaPlayer(dlnaNextPlayer, mediaInstance);
                                            this._setVolumeValueAndStateOnTag(dlnaNextPlayer);
                                            this._currentPlayer.msPlayToSource.next = dlnaNextPlayer.msPlayToSource;
                                            if (MS.Entertainment.Utilities.isVideoApp1 && mediaInstance.startPosition) {
                                                var startTime = Math.floor((+mediaInstance.startPosition) / 1000) + 1;
                                                dlnaNextPlayer.currentTime = startTime
                                            }
                                            this._currentPlayer.msPlayToSource.playNext()
                                        }
                                        this._currentMedia = null;
                                        this._removePlayerSrc(this._currentPlayer, true);
                                        this._currentPlayer = dlnaNextPlayer;
                                        this._currentMedia = mediaInstance;
                                        this._hookupCurrentMediaEvents();
                                        this._playerContainer.appendChild(this._currentPlayer.getElement())
                                    }
                                    catch(ex) {
                                        MS.Entertainment.Platform.Playback.Etw.traceString("XPlayer::_setMedia: Exception thrown when configuring dlnaNextPlayer: " + ex);
                                        var playbackError = MS.Entertainment.Platform.Playback.makePlaybackError(MS.Entertainment.Platform.Playback.Error.X8_E_PLAYBACK_PLAYTO_UNEXPECTED_ERR, "XPlayer::_setMedia() threw exception when configuring dlnaNextPlayer");
                                        this._firePlaybackError(playbackError, playbackError.context)
                                    }
                                }
                                if (!dlnaNextPlayer || dlnaNextPlayer.msPlayToDisabled)
                                    if (this._currentPlayer && this._currentMedia) {
                                        this._nextMedia = mediaInstance;
                                        this._nextMediaLoaded = false;
                                        this._nextMediaStarted = false;
                                        this._nextMediaError = false;
                                        this._fireNextMediaError(MSEPlatform.Playback.Error.X8_E_PLAYBACK_MEDIA_ERR_SRC_NOT_SUPPORTED, "XPlayer::_setMedia, playTo is disabled for media. Current media playing, treat this as next media")
                                    }
                                    else
                                        this._firePlaybackError(MSEPlatform.Playback.Error.X8_E_PLAYBACK_MEDIA_ERR_SRC_NOT_SUPPORTED, "XPlayer::_setMedia, playTo is disabled for media");
                                dlnaNextPlayer = null
                            }
                            else {
                                this._tagLock.reset();
                                if (this._currentPlayer)
                                    this.reset(false);
                                this._currentMedia = mediaInstance;
                                this._currentPlayer = this._createMediaPlayer(mediaInstance, this._autoPlay);
                                this._configurePlayToByRefMediaPlayer(this._currentPlayer, this._currentMedia);
                                this._hookupCurrentMediaEvents();
                                this._setVolumeValueAndStateOnTag(this._currentPlayer);
                                MSEPlatform.Playback.Etw.traceString("XPlayer::_setMedia: audio/video tag appending");
                                this._playerContainer.appendChild(this._currentPlayer.getElement())
                            }
                            MSEPlatform.Playback.Etw.traceString("-XPlayer::_setMedia:")
                        }.bind(this), function _prePlayAsync_set_media_failed(error) {
                            MSEPlatform.Playback.Etw.traceString("-XPlayer::_setMedia: _prePlayAsync_set_media_failed");
                            this._currentMedia = mediaInstance;
                            var playbackError = error;
                            if (!error || !error.code || !error.context)
                                playbackError = MSEPlatform.Playback.makePlaybackError(MSEPlatform.Playback.Error.X8_E_PLAYBACK_MEDIA_GENERIC, "XPlayer::_setMedia() failed");
                            return this._firePlaybackError(playbackError, playbackError.context)
                        }.bind(this))
                    }, _setNextMedia: function _setNextMedia(mediaInstance) {
                        if (mediaInstance) {
                            var name = WinJS.Utilities.getMember("_mediaItem.data.name", mediaInstance) || String.empty;
                            MSEPlatform.Playback.Etw.traceString("+XPlayer::_setNextMedia: name=[" + name + "], source=[" + mediaInstance.source + "]")
                        }
                        else
                            MSEPlatform.Playback.Etw.traceString("+XPlayer::_setNextMedia: null, prerolled media is disposed");
                        var succeeded = true;
                        if (this._nextPlayer)
                            this.reset(true);
                        function finalizeThePlayer() {
                            this._nextMedia = mediaInstance;
                            this._nextMediaLoaded = false;
                            this._nextMediaStarted = false;
                            this._nextMediaError = false;
                            if (this._isDlnaConnectionPresent())
                                if (!this._nextPlayer || this._nextPlayer.msPlayToDisabled) {
                                    this._fireNextMediaError(MSEPlatform.Playback.Error.X8_E_PLAYBACK_MEDIA_ERR_SRC_NOT_SUPPORTED, "XPlayer::_setNextMedia, playTo is disabled for media");
                                    succeeded = false
                                }
                            if (succeeded)
                                this._hookupNextMediaEvents();
                            MSEPlatform.Playback.Etw.traceString("-XPlayer::_setNextMedia")
                        }
                        if (!mediaInstance)
                            return;
                        else if (!this._isDlnaConnectionPresent() || !MSEPlatform.Playback.shouldDisableDlnaPlayTo(mediaInstance))
                            this._watchTimeSeparation(function nextHTML() {
                                this._nextPlayer = this._createMediaPlayer(mediaInstance, false);
                                finalizeThePlayer.bind(this)()
                            }.bind(this), "_setNextMedia::_createMediaPlayer");
                        else
                            finalizeThePlayer.bind(this)()
                    }, _fireNextMediaError: function _fireNextMediaError(error, context) {
                        MSEPlatform.Playback.firePlaybackError(this._handleNextMediaEvents.bind(this), this._remapErrorForPlayTo(error), context)
                    }, _firePlaybackError: function _firePlaybackError(error, context) {
                        MSEPlatform.Playback.firePlaybackError(this._currentMediaEventsCallback, this._remapErrorForPlayTo(error), context)
                    }, _remapErrorForPlayTo: function _remapErrorForPlayTo(error) {
                        var mappedError = error;
                        var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                        var isSubscription = signedInUser && signedInUser.isSubscription;
                        MSEPlatform.Playback.Etw.traceString("XPlayer::_remapErrorForPlayTo: user is " + isSubscription ? "MUSIC PASS user" : "NON-MusicPass user");
                        if (this._isDlnaConnectionPresent() && error && (typeof(error) === "object") && (error.code === MSEPlatform.Playback.Error.X8_E_PLAYBACK_MEDIA_ERR_SRC_NOT_SUPPORTED.code))
                            if (isSubscription)
                                mappedError = MSEPlatform.Playback.Error.X8_E_PLAYBACK_MEDIA_CANT_PLAYTO_PREMIUM;
                            else
                                mappedError = MSEPlatform.Playback.Error.X8_E_PLAYBACK_MEDIA_CANT_PLAYTO;
                        return mappedError
                    }, _createMediaPlayer: function _createMediaPlayer(mediaInstance, autoPlay) {
                        MSEPlatform.Playback.Etw.traceString("XPlayer::_createMediaPlayer: creating audio/video player.  mediaInstance.source = " + mediaInstance.source);
                        var Playback = MSEPlatform.Playback;
                        var mediaPlayer;
                        if (mediaInstance.mediaType === Microsoft.Entertainment.Queries.ObjectType.video)
                            mediaPlayer = new Playback.VideoPlayer(mediaInstance, autoPlay);
                        else {
                            mediaPlayer = new Playback.AudioPlayer(mediaInstance, autoPlay);
                            var mediaElement = mediaPlayer.getElement();
                            if (mediaPlayer.isFastStartSource) {
                                WinJS.Promise.timeout().then(function fastStartSimulateMediaLoaded() {
                                    if (!mediaPlayer.error) {
                                        mediaElement.durationOverrideMS = mediaPlayer.getMediaDurationMS();
                                        this._currentMediaEventsCallback({
                                            srcElement: mediaElement, type: "loadedmetadata"
                                        })
                                    }
                                    else {
                                        var target = {error: this.mediaPlayer.error};
                                        this._currentMediaEventsCallback({
                                            srcElement: mediaElement, type: "error", target: target
                                        })
                                    }
                                }.bind(this));
                                MSEPlatform.Playback.assert(autoPlay, "fast start tag should only be used with autoplay");
                                mediaPlayer.play()
                            }
                            else if (mediaPlayer.isPreActivatedSource) {
                                Playback.Etw.traceString("XPlayer::_CreateMediaPlayer() pre created audioTag ");
                                WinJS.Promise.timeout().then(function preCreatedHtmlTagSimulateEvents() {
                                    if (!mediaPlayer.error) {
                                        if (this._currentMedia && this._currentMedia._mediaItem && this._currentMedia._mediaItem.data)
                                            if (!this._currentMedia._mediaItem.data.duration || this._currentMedia._mediaItem.data.duration <= 0)
                                                this._currentMedia._mediaItem.data.duration = Math.round(this._currentPlayer.duration * 1000);
                                        this._currentMediaEventsCallback({
                                            srcElement: mediaElement, type: "loadedmetadata"
                                        });
                                        this._currentMediaEventsCallback({
                                            srcElement: mediaElement, type: "playing"
                                        });
                                        if (mediaPlayer.ended)
                                            this._currentMediaEventsCallback({
                                                srcElement: mediaElement, type: "ended"
                                            })
                                    }
                                }.bind(this))
                            }
                        }
                        return mediaPlayer
                    }, _skipThisError: false, _errorFilterHandler: null, _hookupCurrentMediaEvents: function _hookupCurrentMediaEvents() {
                        var currentPlayer = this._currentPlayer;
                        if (!this._errorFilterHandler)
                            this._errorFilterHandler = function errorFilter(e) {
                                var primarySession;
                                var shouldSkipForApp2 = false;
                                var shouldSimulateEnded = false;
                                var isPlayToReceiverSessionRunning = false;
                                if (MS.Entertainment.Utilities.isApp2) {
                                    if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.sessionManager)) {
                                        primarySession = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager).primarySession;
                                        isPlayToReceiverSessionRunning = primarySession && primarySession.isPlayToReceiverSessionRunning
                                    }
                                    shouldSkipForApp2 = e.target.endOfStream || isPlayToReceiverSessionRunning || (e.target.error.msExtendedCode === MSEPlatform.Playback.Error.E_ENHANCED_STORAGE.code) || (e.target.error.msExtendedCode === MSEPlatform.Playback.Error.E_ABORT.code) || (e.target.error.msExtendedCode === MSEPlatform.Playback.Error.E_FILE_NOT_FOUND.code) || (e.target.error.msExtendedCode === MSEPlatform.Playback.Error.MF_E_UNSUPPORTED_RATE.code);
                                    if (shouldSkipForApp2)
                                        if (e.target.endOfStream)
                                            MSEPlatform.Playback.Etw.traceString("XPlayer::_errorFilter() Error after EOS, msExtendedCode = " + e.target.error.msExtendedCode);
                                        else {
                                            MSEPlatform.Playback.Etw.traceString("XPlayer::_errorFilter() Errors on Video2 ignored, msExtendedCode = " + e.target.error.msExtendedCode);
                                            if (primarySession) {
                                                var currentPosition = primarySession.currentPosition;
                                                var duration = +(primarySession.duration);
                                                if (!isPlayToReceiverSessionRunning && (duration !== 0) && ((currentPosition / duration) >= 0.95))
                                                    shouldSimulateEnded = true
                                            }
                                        }
                                }
                                var shouldSkipThisError = this._skipThisError || shouldSkipForApp2;
                                if (this._skipThisError)
                                    this._skipThisError = false;
                                if (!shouldSkipThisError)
                                    this._currentMediaEventsCallback(e);
                                else if (shouldSimulateEnded)
                                    this._currentMediaEventsCallback({
                                        srcElement: e.target, type: "ended", target: e.target
                                    })
                            }.bind(this);
                        if (this._currentMediaEventsCallback) {
                            currentPlayer.addEventListener("loadedmetadata", this._currentMediaEventsCallback, false);
                            currentPlayer.addEventListener("playing", this._currentMediaEventsCallback, false);
                            currentPlayer.addEventListener("ended", this._currentMediaEventsCallback, false);
                            currentPlayer.addEventListener("pause", this._currentMediaEventsCallback, false);
                            currentPlayer.addEventListener("error", this._errorFilterHandler, false);
                            currentPlayer.addEventListener("seeked", this._currentMediaEventsCallback, false);
                            currentPlayer.addEventListener("durationchange", this._currentMediaEventsCallback, false)
                        }
                        this.enableTimeUpdate();
                        var currentMediahandler = this._handleCurrentMediaEvents.bind(this);
                        currentPlayer.addEventListener("ended", currentMediahandler, false);
                        currentPlayer.addEventListener("playing", currentMediahandler, false);
                        currentPlayer.addEventListener("pause", currentMediahandler, false);
                        currentPlayer.addEventListener("error", currentMediahandler, false);
                        currentPlayer.addEventListener("progress", currentMediahandler, false);
                        if (currentPlayer && !currentPlayer.msPlayToDisabled)
                            try {
                                if (currentPlayer.msPlayToSource && currentPlayer.msPlayToSource.connection) {
                                    var dlnaHandler = this._handleDlnaConnectionEvents.bind(this);
                                    currentPlayer.msPlayToSource.connection.addEventListener("error", dlnaHandler, false);
                                    currentPlayer.msPlayToSource.connection.addEventListener("statechanged", dlnaHandler, false);
                                    currentPlayer.msPlayToSource.connection.addEventListener("transferred", dlnaHandler, false)
                                }
                            }
                            catch(ex) {
                                MS.Entertainment.UI.Debug.writeLine("Failed to hook up DLNA Connection events: " + ex)
                            }
                        this._unhookCurrentMediaEvents = function() {
                            if (currentPlayer) {
                                currentPlayer.removeEventListener("ended", currentMediahandler, false);
                                currentPlayer.removeEventListener("playing", currentMediahandler, false);
                                currentPlayer.removeEventListener("pause", currentMediahandler, false);
                                currentPlayer.removeEventListener("error", currentMediahandler, false);
                                currentPlayer.removeEventListener("progress", currentMediahandler, false);
                                if (this._currentMediaEventsCallback) {
                                    currentPlayer.removeEventListener("loadedmetadata", this._currentMediaEventsCallback);
                                    currentPlayer.removeEventListener("playing", this._currentMediaEventsCallback);
                                    currentPlayer.removeEventListener("ended", this._currentMediaEventsCallback);
                                    currentPlayer.removeEventListener("pause", this._currentMediaEventsCallback);
                                    currentPlayer.removeEventListener("error", this._errorFilterHandler);
                                    currentPlayer.removeEventListener("seeked", this._currentMediaEventsCallback);
                                    currentPlayer.removeEventListener("durationchange", this._currentMediaEventsCallback)
                                }
                                this.disableTimeUpdate();
                                if (currentPlayer && !currentPlayer.msPlayToDisabled)
                                    try {
                                        if (currentPlayer.msPlayToSource && currentPlayer.msPlayToSource.connection) {
                                            currentPlayer.msPlayToSource.connection.removeEventListener("error", dlnaHandler);
                                            currentPlayer.msPlayToSource.connection.removeEventListener("statechanged", dlnaHandler);
                                            currentPlayer.msPlayToSource.connection.removeEventListener("transferred", dlnaHandler)
                                        }
                                    }
                                    catch(ex) {
                                        MS.Entertainment.UI.Debug.writeLine("Failed to unhook DLNA Connection events: " + ex)
                                    }
                            }
                        }
                    }, _configurePlayToByRefMediaPlayer: function _configurePlayToByRefMediaPlayer(player, media) {
                        if (MS.Entertainment.Utilities.isWindowsBlue)
                            if (player && !player.msPlayToDisabled)
                                try {
                                    if (player.msPlayToSource) {
                                        MSEPlatform.Playback.Etw.traceString("XPlayer::_configurePlayToByRefMediaPlayer - Hooking up DLNA PlayToByRef");
                                        var mediaIsProtected = false;
                                        var mediaIdExists = false;
                                        var playToUrl = MS.Entertainment.Utilities.isMusicApp ? "ms-playtoapp-xboxmusic://media?" : "ms-playtoapp-xboxvideo://media?";
                                        if (media) {
                                            mediaIsProtected = (media.protectionState !== MSEPlatform.Playback.ProtectionState.unprotected);
                                            if (media._mediaItem && media._mediaItem.data)
                                                if (MS.Entertainment.Utilities.isValidServiceId(media._mediaItem.data.zuneId)) {
                                                    mediaIdExists = true;
                                                    playToUrl = this._appendParamToUrl(playToUrl, "X_id={" + MS.Entertainment.Utilities.trimGuid(media._mediaItem.data.zuneId) + "}")
                                                }
                                                else if (MS.Entertainment.Utilities.isValidServiceId(media.serviceId)) {
                                                    mediaIdExists = true;
                                                    playToUrl = this._appendParamToUrl(playToUrl, "X_id={" + MS.Entertainment.Utilities.trimGuid(media.serviceId) + "}")
                                                }
                                        }
                                        if (mediaIdExists && mediaIsProtected) {
                                            var playToUri = new Windows.Foundation.Uri(playToUrl);
                                            MSEPlatform.Playback.Etw.traceString("XPlayer::_configurePlayToByRefMediaPlayer - setting this._currentPlayer.msPlayToSource.preferredSourceUri");
                                            player.msPlayToSource.preferredSourceUri = playToUri
                                        }
                                    }
                                }
                                catch(ex) {
                                    MSEPlatform.Playback.Etw.traceString("XPlayer::_configurePlayToByRefMediaPlayer - this._currentPlayer.msPlayToSource.preferredSourceUri exception", ex.toString())
                                }
                    }, _appendParamToUrl: function _appendParamToUrl(url, param) {
                        var newUrl = url;
                        if (newUrl[newUrl.length - 1] !== '?')
                            newUrl += '&';
                        newUrl += param;
                        return newUrl
                    }, _unhookCurrentMediaEvents: function _unhookCurrentMediaEvents(){}, _hookupNextMediaEvents: function _hookupNextMediaEvents() {
                        var handler = this._handleNextMediaEvents.bind(this);
                        var nextPlayer = this._nextPlayer;
                        this._unhookNextMediaEvents();
                        nextPlayer.addEventListener("loadedmetadata", handler, false);
                        nextPlayer.addEventListener("error", handler, false);
                        nextPlayer.addEventListener("progress", handler, false);
                        this._unhookNextMediaEvents = function() {
                            nextPlayer.removeEventListener("loadedmetadata", handler);
                            nextPlayer.removeEventListener("error", handler);
                            nextPlayer.removeEventListener("progress", handler)
                        }
                    }, _unhookNextMediaEvents: function _unhookNextMediaEvents(){}, _onSwitchPlayerError: function _onSwitchPlayerError(error) {
                        var msg = "Async errors should have been handled by  _switchPlayerAsync: ";
                        try {
                            msg += JSON.stringify(error)
                        }
                        catch(ex) {
                            {}
                        }
                        MSEPlatform.Playback.fail(msg);
                        MSEPlatform.Playback.Etw.traceString("XPlayer::_onSwitchPlayerError.  " + MSEPlatform.Playback.mediaErrorToString(error))
                    }, _handleCurrentMediaEvents: function _handleCurrentMediaEvents(event) {
                        var traceInfo = String.empty;
                        if (event.type === "error" && event.target && event.target.error)
                            traceInfo = MSEPlatform.Playback.mediaErrorToString(event.target.error);
                        MSEPlatform.Playback.Etw.traceString("XPlayer::_handleCurrentMediaEvents event from audio/video tag: " + event.type + " " + traceInfo);
                        if (this._currentMedia) {
                            this._currentMedia.maxPosition = this._currentPlayer ? this._currentPlayer.currentTime : 0;
                            this._currentMedia._lastStateChangeEvent = event
                        }
                        switch (event.type) {
                            case"ended":
                                event.target.endOfStream = true;
                                if (this.controlIsProcessingNextMedia)
                                    MSEPlatform.Playback.Etw.traceString("XPlayer::_handleCurrentMediaEvents : playback ended before preroll completed");
                                else
                                    this._switchPlayerAsync().done(function _switchPlayerAsync_completed(succeeded) {
                                        this._nextMediaStarted = false;
                                        if (succeeded)
                                            this._currentPlayer.play()
                                    }.bind(this), this._onSwitchPlayerError.bind(this));
                                break;
                            case"playing":
                                if (!MSEPlatform.Playback.AudioPlayer.hasValidFastStartTag && MS.Entertainment.Utilities.isMusicApp1)
                                    if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.uiState)) {
                                        var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                                        if (uiStateService.isAppVisible)
                                            this._watchTimeSeparation(this._createFastStartAudioTag.bind(this), "_createFastStartAudioTag on playing")
                                    }
                                break;
                            case"progress":
                                MS.Entertainment.Utilities.Telemetry.logPlayerBufferState(this._currentPlayer.buffered, this._currentPlayer.currentTime);
                                break;
                            default:
                                break
                        }
                    }, _handleNextMediaEvents: function _handleNextMediaEvents(event) {
                        if (this._nextMedia)
                            this._nextMedia._lastStateChangeEvent = event;
                        switch (event.type) {
                            case"loadedmetadata":
                                MSEPlatform.Playback.Etw.traceString("XPlayer::_handleNextMediaEvents - loadedmetadata");
                                if (this._nextMedia) {
                                    MSEPlatform.Playback.Etw.traceString("XPlayer::_handleNextMediaEvents - loadedmetadata, resetting errorDescriptor");
                                    this._nextMedia._errorDescriptor = null;
                                    this._nextMediaError = false;
                                    this._nextMediaLoaded = true;
                                    if (this._currentPlayer && (this._currentPlayer.ended || this._currentMedia._errorDescriptor)) {
                                        MSEPlatform.Playback.Etw.traceString("XPlayer::_handleNextMediaEvents - preroll completed after previous track ended. Calling switchPlayer.");
                                        this._switchPlayerAsync().done(function _switchPlayerAsync_completed(succeeded) {
                                            if (succeeded)
                                                this._currentPlayer.play()
                                        }.bind(this), this._onSwitchPlayerError.bind(this))
                                    }
                                }
                                else
                                    MSEPlatform.Playback.Etw.traceString("XPlayer::_handleNextMediaEvents - loadedmetadata, nextMedia null?");
                                break;
                            case"error":
                                MSEPlatform.Playback.Etw.traceString("XPlayer::_handleNextMediaEvents - error");
                                MSEPlatform.Playback.Etw.traceString("XPlayer::_handleNextMediaEvents - error," + " src = " + (event.target.src ? event.target.src : "null") + ", currentsrc = " + (event.target.currentSrc ? event.target.currentSrc : "null") + ", " + MSEPlatform.Playback.mediaErrorToString(event.target.error));
                                if (event.target.error.msExtendedCode === MSEPlatform.Playback.Error.MF_E_UNSUPPORTED_RATE.code) {
                                    MSEPlatform.Playback.Etw.traceString("MF_E_UNSUPPORTED_RATE ignored in a prerolled item");
                                    break
                                }
                                if (this._nextMedia) {
                                    this._nextMedia._errorDescriptor = event.target.error;
                                    this._nextMediaError = true
                                }
                                else
                                    MSEPlatform.Playback.Etw.traceString("XPlayer::_handleNextMediaEvents - error, nextMedia null?");
                                break;
                            case"progress":
                                MS.Entertainment.Utilities.Telemetry.logPlayerBufferState(this._nextPlayer.buffered, this._nextPlayer.currentTime);
                                break;
                            default:
                                break
                        }
                    }, _handleDlnaConnectionEvents: function _handleDlnaConnectionEvents(event) {
                        switch (event.type) {
                            case"error":
                                if (this._currentMedia && this._currentMedia.alternateVideoEncoding !== null) {
                                    this._firePlaybackError(MSEPlatform.Playback.Error.MF_E_UNSUPPORTED_BYTESTREAM, "Dlna translated error. Not subject to DLNA telemetry.");
                                    break
                                }
                                var telemetryParameter = [{
                                            parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ErrorCode, parameterValue: event.code ? event.code : 0
                                        }];
                                MS.Entertainment.Utilities.Telemetry.logTelemetryEventWithParameterArray(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlayToSenderError, telemetryParameter);
                                var errorString = MSEPlatform.Playback.errorToString(event.code);
                                MSEPlatform.Playback.Etw.traceString("XPlayer::_handleDlnaConnectionEvents - Error = " + errorString);
                                MS.Entertainment.UI.Debug.writeLine("DLNA ConnectionError: " + errorString);
                                break;
                            case"statechanged":
                                if (this._currentPlayer && !this._currentPlayer.msPlayToDisabled && this._currentPlayer.msPlayToSource && this._currentPlayer.msPlayToSource.connection) {
                                    MS.Entertainment.UI.Debug.writeLine("_currentPlayer connection state: " + this._currentPlayer.msPlayToSource.connection.state);
                                    this.playToSenderConnectionState = this._currentPlayer.msPlayToSource.connection.state;
                                    if (MS.Entertainment.Utilities.isWindowsBlue && this._currentMedia)
                                        this._currentMedia._remoteTrackingMode = true;
                                    MSEPlatform.Playback.Etw.traceString("XPlayer::_handleDlnaConnectionEvents - _currentPlayer Connection StateChanged = " + this.playToSenderConnectionState)
                                }
                                else {
                                    MS.Entertainment.UI.Debug.writeLine("DLNA Connection StateChanged: " + event.currentState);
                                    this.playToSenderConnectionState = event.currentState;
                                    MSEPlatform.Playback.Etw.traceString("XPlayer::_handleDlnaConnectionEvents - DLNA Connection StateChanged = " + this.playToSenderConnectionState)
                                }
                                if (!this._currentMedia._telemetryDataSent && this.playToSenderConnectionState === Windows.Media.PlayTo.PlayToConnectionState.rendering) {
                                    var currentMediaIsProtected = (this._currentMedia.protectionState !== MS.Entertainment.Platform.Playback.ProtectionState.unprotected);
                                    var currentMediaHasSmid = false;
                                    if ((this._currentMedia._mediaItem && this._currentMedia._mediaItem.data && MS.Entertainment.Utilities.isValidServiceId(this._currentMedia._mediaItem.data.zuneId)) || MS.Entertainment.Utilities.isValidServiceId(this._currentMedia.serviceId))
                                        currentMediaHasSmid = true;
                                    var playToType = (currentMediaHasSmid && currentMediaIsProtected) ? MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.PlayToByRef : MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.PlayToUGC;
                                    MS.Entertainment.Utilities.Telemetry.logTelemetryEventWithParameterArray(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlayToSenderPlaybackHappened, [{
                                            parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.PlayToType, parameterValue: playToType
                                        }]);
                                    this._currentMedia._telemetryDataSent = true
                                }
                                if (this.playToSenderConnectionState === Windows.Media.PlayTo.PlayToConnectionState.disconnected) {
                                    this.dmrName = "";
                                    this.isRemoteSessionRunning = false;
                                    if (MS.Entertainment.Utilities.isWindowsBlue && this._currentMedia)
                                        this._currentMedia._remoteTrackingMode = false;
                                    if (this._currentPlayer)
                                        this._setVolumeValueAndStateOnTag(this._currentPlayer);
                                    if (this._currentMedia)
                                        this._currentMedia.startPosition = 0;
                                    if (this._currentPlayer && this._currentPlayer.msPlayToDisabled)
                                        this._firePlaybackError(MSEPlatform.Playback.Error.X8_E_PLAYBACK_MEDIA_ERR_SRC_NOT_SUPPORTED, "DLNA actual device error");
                                    if (MS.Entertainment.Utilities.isVideoApp1 && !MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).nowPlayingVisible)
                                        this.reset(false);
                                    if (this._currentPlayer && this._currentPlayer.isFastStartSource && MSEPlatform.Playback.AudioPlayer.hasValidFastStartTag) {
                                        this._nextPlayer = this._createMediaPlayer(this._currentMedia, false);
                                        this._nextMedia = this._currentMedia;
                                        this._configurePlayToByRefMediaPlayer(this._nextPlayer, this._nextMedia);
                                        this._destroyCurrentAndPromoteNextPlayer()
                                    }
                                }
                                break;
                            case"transferred":
                                MSEPlatform.Playback.Etw.traceString("XPlayer::_handleDlnaConnectionEvents - DLNA Connection Transferred");
                                MS.Entertainment.UI.Debug.writeLine("DLNA Connection Transferred");
                                break
                        }
                    }, _isDlnaConnectionPresent: function _isDlnaConnectionPresent() {
                        return (this._dlnaPlayToMgr && this.playToSenderConnectionState !== Windows.Media.PlayTo.PlayToConnectionState.disconnected)
                    }, _switchPlayerAsync: function _switchPlayerAsync() {
                        return this._prePlayAsync(this._nextMedia).then(function _prePlayAsync_succeeded() {
                                MSEPlatform.Playback.Etw.traceString("+XPlayer:_switchPlayer.  this._nextPlayer = " + this._nextPlayer);
                                if (this._nextPlayer) {
                                    MSEPlatform.Playback.Etw.traceSwitchingToNextMedia(this._nextMedia.source);
                                    this._unhookCurrentMediaEvents();
                                    if (this._isDlnaConnectionPresent()) {
                                        if (this._currentPlayer && this._currentPlayer.msPlayToSource)
                                            if (this._currentPlayer.msPlayToSource.next) {
                                                MSEPlatform.Playback.Etw.traceString("Xplayer::_switchPlayer, PlayToSource.next already exists, start playto");
                                                this._setVolumeValueAndStateOnTag(this._nextPlayer);
                                                this._currentPlayer.msPlayToSource.playNext()
                                            }
                                            else if (this._nextPlayer.msPlayToDisabled)
                                                this._fireNextMediaError(MSEPlatform.Playback.Error.X8_E_PLAYBACK_MEDIA_ERR_SRC_NOT_SUPPORTED, "XPlayer::_switchPlayer, playTo is disabled for media");
                                            else
                                                try {
                                                    MSEPlatform.Playback.Etw.traceString("Xplayer::_switchPlayer, PlayToSource.next does not exist, set PlayToSource.next on current player and start playto");
                                                    this._configurePlayToByRefMediaPlayer(this._nextPlayer, this._nextMedia);
                                                    this._setVolumeValueAndStateOnTag(this._nextPlayer);
                                                    this._currentPlayer.msPlayToSource.next = this._nextPlayer.msPlayToSource;
                                                    this._currentPlayer.msPlayToSource.playNext()
                                                }
                                                catch(ex) {
                                                    MSEPlatform.Playback.Etw.traceString("XPlayer::_switchPlayer.  Caught exception in _configurePlayToByRefMediaPlayer");
                                                    this._currentPlayer.msPlayToSource = null
                                                }
                                    }
                                    else
                                        this._configurePlayToByRefMediaPlayer(this._nextPlayer, this._nextMedia)
                                }
                                if (this._nextPlayer === null)
                                    if (this.controlIsProcessingNextMedia || (this.controlIsProcessingNextMedia === undefined && MS.Entertainment.Utilities.isMusicApp)) {
                                        MSEPlatform.Playback.Etw.traceString("-XPlayer::_switchPlayer.  returning false");
                                        return false
                                    }
                                    else if (!(this.isRemoteSessionRunning || MS.Entertainment.Utilities.isVideoApp1)) {
                                        this._destroyCurrentAndPromoteNextPlayer();
                                        MSEPlatform.Playback.Etw.traceString("-XPlayer:_switchPlayer, setting _nextMediaStarted with currentPlayer  " + !!this._currentPlayer);
                                        this._nextMediaStarted = true
                                    }
                                    else {
                                        MSEPlatform.Playback.Etw.traceString("-XPlayer:_switchPlayer.  returning false");
                                        return false
                                    }
                                else {
                                    this._destroyCurrentAndPromoteNextPlayer();
                                    this._nextMediaStarted = true
                                }
                                var success = !!this._currentPlayer;
                                MSEPlatform.Playback.Etw.traceString("-XPlayer::_switchPlayer, switchSucceeded: returning " + success);
                                return success
                            }.bind(this), function _prePlayAsync_switch_player_failed(error) {
                                MSEPlatform.Playback.Etw.traceString("-XPlayer::_prePlayAsync_switch_player_failed");
                                if (error && error.code && error.context)
                                    this._fireNextMediaError(error.code, error.context);
                                else {
                                    var playbackError = MSEPlatform.Playback.makePlaybackError(MSEPlatform.Playback.Error.X8_E_PLAYBACK_MEDIA_GENERIC, "_prePlayAsync failed without an error code");
                                    this._fireNextMediaError(playbackError.error, playbackError.context)
                                }
                                return false
                            }.bind(this))
                    }, _destroyCurrentAndPromoteNextPlayer: function _destroyCurrentAndPromoteNextPlayer() {
                        MSEPlatform.Playback.Etw.traceString("+XPlayer::_destroyCurrentAndPromoteNextPlayer");
                        try {
                            if (this._currentPlayer)
                                this._removePlayerSrc(this._currentPlayer, true)
                        }
                        catch(ex) {
                            MSEPlatform.Playback.assert(ex, "Error in media tag removal");
                            MS.Entertainment.Utilities.Telemetry.logErrorInMediaTagRemoval(ex)
                        }
                        this._currentPlayer = this._nextPlayer;
                        this._currentMedia = this._nextMedia;
                        if (this._currentPlayer) {
                            this._setVolumeValueAndStateOnTag(this._currentPlayer);
                            this._playerContainer.appendChild(this._currentPlayer.getElement());
                            this._hookupCurrentMediaEvents()
                        }
                        this._unhookNextMediaEvents();
                        this._nextPlayer = null;
                        this._nextMedia = null;
                        if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.playbackErrorDisplayService)) {
                            var displayService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.playbackErrorDisplayService);
                            if (displayService)
                                displayService.showDialogForNonCriticalErrors = false
                        }
                        MSEPlatform.Playback.Etw.traceString("-XPlayer::_destroyCurrentAndPromoteNextPlayer")
                    }, _onSourceRequested: function _onSourceRequested(event) {
                        if (this._currentPlayer && this._currentPlayer.msPlayToDisabled)
                            return;
                        try {
                            var sourceRequest = event.sourceRequest;
                            var deferral = sourceRequest.getDeferral();
                            if (this._currentPlayer) {
                                sourceRequest.setSource(this._currentPlayer.msPlayToSource);
                                if (this._nextMedia && MSEPlatform.Playback.shouldDisableDlnaPlayTo(this._nextMedia))
                                    this._fireNextMediaError(MSEPlatform.Playback.Error.X8_E_PLAYBACK_MEDIA_ERR_SRC_NOT_SUPPORTED, "XPlayer::_onSourceRequested, playTo is disabled for next media")
                            }
                            else
                                sourceRequest.setSource(null);
                            deferral.complete()
                        }
                        catch(ex) {
                            this._firePlaybackError(ex.number, "DLNA SourceRequested")
                        }
                    }, _onSourceSelected: function _onSourceSelected(event) {
                        if (event && event.friendlyName) {
                            var telemetryParameter = [{
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.DMRFriendlyName, parameterValue: event.friendlyName
                                    }];
                            MS.Entertainment.Utilities.Telemetry.logTelemetryEventWithParameterArray(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlayToSenderAttempted, telemetryParameter);
                            MSEPlatform.Playback.Etw.traceString("Xplayer::_onSourceSelected, PlayTo DMR Selected: " + event.friendlyName);
                            MS.Entertainment.UI.Debug.writeLine("DLNA SourceSelected: DMR = " + event.friendlyName);
                            this.isRemoteSessionRunning = true;
                            this.dmrName = event.friendlyName
                        }
                    }, _setupDlnaPlayToMgr: function _setupDlnaPlayToMgr() {
                        this.playToSenderConnectionState = Windows.Media.PlayTo.PlayToConnectionState.disconnected;
                        try {
                            this._dlnaPlayToMgr = Windows.Media.PlayTo.PlayToManager.getForCurrentView();
                            if (this._dlnaPlayToMgr) {
                                this._dlnaPlayToMgr.defaultSourceSelection = false;
                                this._dlnaPlayToMgr.addEventListener("sourcerequested", this._onSourceRequested.bind(this), false);
                                this._dlnaPlayToMgr.addEventListener("sourceselected", this._onSourceSelected.bind(this), false)
                            }
                        }
                        catch(ex) {
                            this._dlnaPlayToMgr = null
                        }
                    }, _cleanupDlnaPlayToMgr: function _cleanupDlnaPlayToMgr() {
                        if (this._dlnaPlayToMgr) {
                            this._dlnaPlayToMgr.removeEventListener("sourcerequested", this._onSourceRequested);
                            this._dlnaPlayToMgr.removeEventListener("sourceselected", this._onSourceSelected);
                            this._dlnaPlayToMgr = null
                        }
                    }, _createFastStartAudioTag: function _createFastStartAudioTag() {
                        MSEPlatform.Playback.Etw.traceString("+XPlayer:_createFastStartAudioTag");
                        if (MS.Entertainment.Utilities.isMusicApp1) {
                            var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                            var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                            if (!signIn.isSignedIn) {
                                MSEPlatform.Playback.Etw.traceString("-XPlayer:_createFastStartAudioTag.  signIn.isSignedIn = false");
                                return
                            }
                            MSEPlatform.Playback.AudioPlayer.createFastStartTag()
                        }
                    }, _ensurePipeline: function _ensurePipeline() {
                        if (this._currentPlayer === undefined || this._currentPlayer === null || this._currentPlayer.isDisposed) {
                            MSEPlatform.Playback.Etw.traceString("XPlayer::_ensurePipeline() error! AV pipeline not set yet.");
                            throw"XPlayer::_ensurePipeline() error! AV pipeline not set yet.";
                        }
                    }, _onAppResume: function _onAppResume() {
                        if (MSEPlatform.Playback.AudioPlayer.hasValidFastStartTag)
                            try {
                                MSEPlatform.Playback.Etw.traceString("XPlayer::_onAppResume: Destroying FastStart tag on resume. Windows invalidates inactive DRM pipelines on suspend");
                                MSEPlatform.Playback.AudioPlayer.disposeFastStartTag()
                            }
                            catch(e) {
                                MSEPlatform.Playback.Etw.traceString("XPlayer::_onAppResume: error! AV pipeline not set yet.");
                                MSEPlatform.Playback.assert(false, "failed to destroy fast start tag on resume. exception:" + e)
                            }
                    }, _onAppSuspending: function _onAppSuspending() {
                        try {
                            MSEPlatform.Playback.Etw.traceString("XPlayer::_onAppSuspending: close out tracking items");
                            for (var trackingId in this._trackingIdUsage) {
                                var media = this._mediaByTrackingId(trackingId);
                                this._closeTrackingTransaction(media);
                                delete this._trackingIdUsage[trackingId];
                                this._globalDeleteTrackingIdState[trackingId]
                            }
                        }
                        catch(e) {
                            MSEPlatform.Playback.Etw.traceString("XPlayer::_onAppSuspending: failed to close out tracking items on suspend.  exception: " + e);
                            MSEPlatform.Playback.assert(false, "XPlayer::_onAppSuspending: failed to close out tracking items on suspend. exception: " + e)
                        }
                    }, _onAppVisibleChanged: function _onAppVisibleChanged() {
                        if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.uiState)) {
                            var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                            if (uiStateService.isAppVisible)
                                this._createFastStartAudioTag();
                            else if (MSEPlatform.Playback.AudioPlayer.hasValidFastStartTag)
                                MSEPlatform.Playback.AudioPlayer.disposeFastStartTag()
                        }
                    }, _onGlobalTrackingIdState: function _onGlobalTrackingIdState(id, state) {
                        if (id) {
                            var trackingIdState = MSEPlatform.SessionManager.trackingIdState;
                            MSEPlatform.Playback.Etw.traceString("Was: " + trackingIdState[id] + " tracking id: " + id);
                            trackingIdState[id] = state;
                            MSEPlatform.Playback.Etw.traceString("Now:" + state + ": " + id)
                        }
                    }, _globalDeleteTrackingIdState: function _globalDeleteTrackingIdState(id) {
                        if (id) {
                            var trackingIdState = MSEPlatform.SessionManager.trackingIdState;
                            MSEPlatform.Playback.Etw.traceString("Deleted " + trackingIdState[id] + " tracking id: " + id);
                            delete trackingIdState[id]
                        }
                    }, _mediaByTrackingId: function _mediaByTrackingId(trackingId) {
                        if (this._trackingIdUsage[trackingId] & MSEPlatform.Playback.XPlayer._mediaBitmask.current)
                            return this._currentMedia;
                        if (this._trackingIdUsage[trackingId] & MSEPlatform.Playback.XPlayer._mediaBitmask.next)
                            return this._nextMedia;
                        return null
                    }, _releaseTrackingRef: function _releaseTrackingRef(media, mediaBitmask) {
                        if (media && media.hasOwnProperty("trackingId") && !!media.trackingId) {
                            if (!this._trackingIdUsage[media.trackingId])
                                this._trackingIdUsage[media.trackingId] = 0;
                            this._trackingIdUsage[media.trackingId] &= ~mediaBitmask
                        }
                    }, _addTrackingRef: function _addTrackingRef(media, mediaBitmask) {
                        if (media && media.hasOwnProperty("trackingId") && !!media.trackingId) {
                            if (!this._trackingIdUsage[media.trackingId])
                                this._trackingIdUsage[media.trackingId] = 0;
                            this._trackingIdUsage[media.trackingId] |= mediaBitmask
                        }
                    }, _checkTracking: function _checkTracking(media, mediaBitmask) {
                        if (media && media.hasOwnProperty("trackingId") && !!media.trackingId && this._trackingIdUsage[media.trackingId] === 0) {
                            if (!media._lastStateChangeEvent || media._lastStateChangeEvent.type !== "ended" || media._remoteTrackingMode) {
                                if (mediaBitmask === MSEPlatform.Playback.XPlayer._mediaBitmask.current)
                                    media.maxPosition = this._currentPlayer ? this._currentPlayer.currentTime : 0;
                                this._onTrackedMediaOutOfScope(media)
                            }
                            else
                                this._onGlobalTrackingIdState(media.trackingId, "SkipReported");
                            delete this._trackingIdUsage[media.trackingId];
                            this._globalDeleteTrackingIdState(media.trackingId)
                        }
                    }, _onTrackedMediaOutOfScope: function _onTrackedMediaOutOfScope(media) {
                        this._closeTrackingTransaction(media)
                    }, _closeTrackingTransaction: function _closeTrackingTransaction(media) {
                        if (!media)
                            return;
                        Microsoft.Entertainment.Marketplace.Marketplace.sendPlaybackDurationAsync(media.trackingId, media.maxPosition);
                        this._onGlobalTrackingIdState(media.trackingId, "Reported");
                        MSEPlatform.Playback.Etw.traceString("XPlayer::_closeTrackingTransaction(" + media.trackingId + ", " + media.maxPosition + ")")
                    }, _overwriteMediaRefCount: function _overwriteMediaRefCount(destinationMedia, sourceMedia, mediaBitmask) {
                        this._releaseTrackingRef(destinationMedia, mediaBitmask);
                        this._addTrackingRef(sourceMedia, mediaBitmask);
                        this._checkTracking(destinationMedia, mediaBitmask)
                    }, _overwriteCurrentMedia: function _overwriteCurrentMedia(value) {
                        if (value)
                            this._onGlobalTrackingIdState(value.trackingId, "Current");
                        this._overwriteMediaRefCount(this._currentMediaValueHolder, value, MSEPlatform.Playback.XPlayer._mediaBitmask.current);
                        this._currentMediaValueHolder = value
                    }, _overwriteNextMedia: function _overwriteNextMedia(value) {
                        if (value)
                            this._onGlobalTrackingIdState(value.trackingId, "Next");
                        this._overwriteMediaRefCount(this._nextMediaValueHolder, value, MSEPlatform.Playback.XPlayer._mediaBitmask.next);
                        this._nextMediaValueHolder = value
                    }, _currentMedia: {
                        get: function internal_currentMedia_get() {
                            return this._currentMediaValueHolder
                        }, set: function internal_currentMedia_set(value) {
                                this._overwriteCurrentMedia(value)
                            }
                    }, _nextMedia: {
                        get: function internal_nextMedia_get() {
                            return this._nextMediaValueHolder
                        }, set: function internal_nextMedia_set(value) {
                                this._overwriteNextMedia(value)
                            }
                    }, _onVolumeValueStateChange: function VolumeValueStateChange(arg) {
                        if (this._currentPlayer)
                            this._setVolumeValueAndStateOnTag(this._currentPlayer)
                    }, _setVolumeValueAndStateOnTag: function setVolumeValueAndStateOnTag(tag) {
                        try {
                            if (tag && MS.Entertainment.Utilities.isMusicApp1 && this._volumeController) {
                                tag.volume = this._volumeController.volume;
                                tag.muted = this._volumeController.mute
                            }
                            else if (tag && MS.Entertainment.Utilities.isMusicApp2)
                                tag.volume = (tag.playerType === "AUDIO") ? (Microsoft.Entertainment.Configuration.ConfigurationManager().playback.music2VolumeValue) / 100 : 1.0
                        }
                        catch(e) {
                            MSEPlatform.Playback.Etw.traceString("XPlayer::SetVolumeValueAndStateOnTag threw an error" + e.toString())
                        }
                    }, _watchTimeSeparation: function _watchTimeSeparation(fun, id) {
                        if (id !== this._lastTimeSeparationId) {
                            this._lastTimeSeparationId = id;
                            MSEPlatform.Playback.Etw.traceString("XPlayer::_watchTimeSeparation for " + id);
                            return this._tagLock.enter(tackOnTimeAfterExecution.bind(this, fun, 2500, id))
                        }
                        else {
                            MSEPlatform.Playback.Etw.traceString("XPlayer::_watchTimeSeparation: synchronous " + id);
                            return WinJS.Promise.wrap(fun())
                        }
                    }, _currentPlayer: {
                        get: function _currentPlayer_get() {
                            return this._currentMediaPlayer
                        }, set: function _currentPlayer_set(mediaPlayer) {
                                if (this._currentMediaPlayer !== mediaPlayer) {
                                    this._currentMediaPlayer = mediaPlayer;
                                    if (this._currentMediaPlayer)
                                        this.mediaElement = this._currentMediaPlayer.getElement();
                                    else
                                        this.mediaElement = null;
                                    if (!!mediaPlayer)
                                        this._currentMediaPlayer.firePlaybackError = this._firePlaybackError.bind(this)
                                }
                            }
                    }, _nextPlayer: {
                        get: function _nextPlayer_get() {
                            return this._nextMediaPlayer
                        }, set: function _nextPlayer_set(mediaPlayer) {
                                if (this._nextMediaPlayer !== mediaPlayer) {
                                    this._nextMediaPlayer = mediaPlayer;
                                    if (!!mediaPlayer)
                                        this._nextMediaPlayer.firePlaybackError = this._fireNextMediaError.bind(this)
                                }
                            }
                    }, _playerContainer: null, _currentMediaPlayer: null, _nextMediaPlayer: null, _currentMediaValueHolder: null, _currentMediaEventsCallback: null, _nextMediaValueHolder: null, _dlnaPlayToMgr: null, _trackingIdUsage: {}, _volumeController: null, _volumeControllerBindings: null, _lastTimeSeparationId: null, _usingFastStartTag: false, _nextMediaLoaded: MS.Entertainment.UI.Framework.observableProperty("_nextMediaLoaded", false), _nextMediaStarted: MS.Entertainment.UI.Framework.observableProperty("_nextMediaStarted", false), _nextMediaError: MS.Entertainment.UI.Framework.observableProperty("_nextMediaError", false), dmrName: MS.Entertainment.UI.Framework.observableProperty("dmrName", String.empty), isRemoteSessionRunning: MS.Entertainment.UI.Framework.observableProperty("isRemoteSessionRunning", false), playToSenderConnectionState: MS.Entertainment.UI.Framework.observableProperty("playToSenderConnectionState", Windows.Media.PlayTo.PlayToConnectionState.disconnected), mediaElement: MS.Entertainment.UI.Framework.observableProperty("mediaElement", null)
            }, {
                createInstance: function XPlayer_createInstance(playerContainer) {
                    var isWWA = true;
                    var player = null;
                    if (isWWA)
                        player = new MSEPlatform.Playback.XPlayer(playerContainer);
                    player._tagLock = new MSEPlatform.Playback.Playlist.AccessSerializer;
                    return player
                }, createMediaTag: function createMediaTag(type) {
                        return MSEPlatform.Playback.MediaPlayer.createMediaTag(type)
                    }, audioTagForFileActivation: {
                        get: function audioTagForFileActivation_get() {
                            return MSEPlatform.Playback.AudioPlayer.audioTagForFileActivation
                        }, set: function audioTagForFileActivation_set(tag) {
                                MSEPlatform.Playback.AudioPlayer.audioTagForFileActivation = tag
                            }
                    }, _mediaBitmask: {
                        current: 1, next: 2
                    }
            })
    })
})()
