﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Platform.PlaybackHelpers");
    WinJS.Namespace.define("MS.Entertainment.Platform", {PlayAction: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function playAction() {
            this.base()
        }, {
            _executing: false, executed: function executed(param) {
                    if (MS.Entertainment.Utilities.isApp2 && this._executing)
                        return;
                    this._executing = true;
                    this.requeryCanExecute();
                    var mediaQueued = WinJS.Promise.as(this.executedPlay(param));
                    mediaQueued.done(function waitForPlayback(wasMediaQueued) {
                        if (wasMediaQueued)
                            MS.Entertainment.Platform.PlaybackHelpers.waitForTransportState(MS.Entertainment.Platform.Playback.TransportState.playing).then(null, function ignoreError(){}).done(function enableAction() {
                                this._executing = false;
                                this.requeryCanExecute()
                            }.bind(this));
                        else {
                            this._executing = false;
                            this.requeryCanExecute()
                        }
                    }.bind(this), function ignoreError(){});
                    return mediaQueued
                }, executedPlay: function executedPlay(param) {
                    return false
                }, canExecute: function canExecute(param) {
                    return this.canExecutePlay(param) && (MS.Entertainment.Utilities.isApp2 || !this._executing)
                }, canExecutePlay: function canExecutePlay(param) {
                    return false
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.Platform.PlaybackHelpers", {
        _getFeatureEnablement: (function _getFeatureEnablement_closure() {
            var featureEnablement = null;
            return function _getFeatureEnablement_getter() {
                    if (!featureEnablement)
                        featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                    return featureEnablement
                }
        })(), waitForTransportState: function waitForTransportState(transportStates) {
                var bindings;
                var completed;
                var failed;
                var promise;
                var playbackSession = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager).nowPlayingSession;
                if (transportStates && !Array.isArray(transportStates))
                    transportStates = [transportStates];
                function onPlaybackChanged() {
                    if (!bindings)
                        return;
                    if (playbackSession.playerState === MS.Entertainment.Platform.Playback.PlayerState.error)
                        failed();
                    else if (transportStates.indexOf(playbackSession.currentTransportState) >= 0)
                        completed()
                }
                if (transportStates)
                    promise = new WinJS.Promise(function initializePromise(c, f) {
                        completed = c;
                        failed = f;
                        bindings = WinJS.Binding.bind(playbackSession, {
                            currentTransportState: onPlaybackChanged, playerState: onPlaybackChanged
                        })
                    }, function canceled(){});
                else
                    promise = WinJS.Promise.wrap();
                promise.then(null, function ignoreError(){}).done(function clearBindings() {
                    if (bindings) {
                        bindings.cancel();
                        bindings = null
                    }
                });
                return promise
            }, getMediaByServiceId: function getMediaByServiceId(serviceId, idType) {
                var featureEnablement = this._getFeatureEnablement();
                return new WinJS.Promise(function(c, e, p) {
                        if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.tvMarketplace) || featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.movieTrailersMarketplace) || featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.moviesMarketplace) || featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace) || featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicVideosMarketplace)) {
                            var contentTypeQuery = new MS.Entertainment.Data.Query.GenericDetails;
                            contentTypeQuery.serviceId = serviceId;
                            contentTypeQuery.idType = (idType && idType.toLocaleLowerCase() === MS.Entertainment.Data.Query.edsIdType.canonical.toLocaleLowerCase()) ? MS.Entertainment.Data.Query.edsIdType.canonical : MS.Entertainment.Data.Query.edsIdType.zuneCatalog;
                            contentTypeQuery.execute().then(function contentTypeQuerySuccess(query) {
                                var mediaItemQuery = null;
                                if (query.result.item)
                                    switch (query.result.item.contentType) {
                                        case"Movie":
                                            if (query.result.item.type === MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.MovieTrailer) {
                                                if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.movieTrailersMarketplace)) {
                                                    mediaItemQuery = new MS.Entertainment.Data.Query.Video.EdsDetailMovie;
                                                    mediaItemQuery.id = query.result.item.serviceId;
                                                    mediaItemQuery.idType = query.result.item.serviceIdType;
                                                    mediaItemQuery.impressionGuid = query.result.item.impressionGuid;
                                                    mediaItemQuery.relevancyTrackingContent = query.result.item.relevancyTrackingContent
                                                }
                                            }
                                            else if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.moviesMarketplace)) {
                                                mediaItemQuery = new MS.Entertainment.Data.Query.Video.EdsDetailMovie;
                                                mediaItemQuery.id = query.result.item.serviceId;
                                                mediaItemQuery.idType = query.result.item.serviceIdType;
                                                mediaItemQuery.impressionGuid = query.result.item.impressionGuid;
                                                mediaItemQuery.relevancyTrackingContent = query.result.item.relevancyTrackingContent
                                            }
                                            break;
                                        case"MovieTrailer":
                                            if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.movieTrailersMarketplace)) {
                                                mediaItemQuery = new MS.Entertainment.Data.Query.Video.EdsDetailMovie;
                                                mediaItemQuery.id = query.result.item.serviceId;
                                                mediaItemQuery.idType = query.result.item.serviceIdType;
                                                mediaItemQuery.impressionGuid = query.result.item.impressionGuid;
                                                mediaItemQuery.relevancyTrackingContent = query.result.item.relevancyTrackingContent
                                            }
                                            break;
                                        case"Track":
                                            if (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.networkStatus).isEnabled(MS.Entertainment.UI.NetworkStatusService.NetworkedFeature.musicMarketplace) && featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace)) {
                                                mediaItemQuery = new MS.Entertainment.Data.Query.Music.SongDetails;
                                                mediaItemQuery.id = query.result.item.serviceId;
                                                mediaItemQuery.idType = query.result.item.serviceIdType;
                                                mediaItemQuery.impressionGuid = query.result.item.impressionGuid;
                                                mediaItemQuery.acquisitionData = new MS.Entertainment.Utilities.AcquisitionData(Microsoft.Entertainment.Platform.AcquisitionContext.store, Microsoft.Entertainment.Platform.AcquisitionContextType.track)
                                            }
                                            break;
                                        case"Album":
                                            if (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.networkStatus).isEnabled(MS.Entertainment.UI.NetworkStatusService.NetworkedFeature.musicMarketplace) && featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace)) {
                                                mediaItemQuery = new MS.Entertainment.Data.Query.Music.AlbumWithTracks;
                                                mediaItemQuery.id = query.result.item.serviceId;
                                                mediaItemQuery.idType = query.result.item.serviceIdType;
                                                mediaItemQuery.impressionGuid = query.result.item.impressionGuid;
                                                mediaItemQuery.acquisitionData = new MS.Entertainment.Utilities.AcquisitionData(Microsoft.Entertainment.Platform.AcquisitionContext.store, Microsoft.Entertainment.Platform.AcquisitionContextType.album)
                                            }
                                            break;
                                        case"Artist":
                                            if (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.networkStatus).isEnabled(MS.Entertainment.UI.NetworkStatusService.NetworkedFeature.musicMarketplace) && featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace)) {
                                                mediaItemQuery = new MS.Entertainment.Data.Query.Music.ArtistDetails;
                                                mediaItemQuery.id = query.result.item.serviceId;
                                                mediaItemQuery.idType = query.result.item.serviceIdType;
                                                mediaItemQuery.impressionGuid = query.result.item.impressionGuid;
                                                mediaItemQuery.acquisitionData = new MS.Entertainment.Utilities.AcquisitionData(Microsoft.Entertainment.Platform.AcquisitionContext.store, Microsoft.Entertainment.Platform.AcquisitionContextType.artist)
                                            }
                                            break;
                                        case"MusicVideo":
                                            if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace)) {
                                                mediaItemQuery = new MS.Entertainment.Data.Query.Music.MusicVideoDetails;
                                                mediaItemQuery.id = query.result.item.serviceId;
                                                mediaItemQuery.idType = query.result.item.serviceIdType;
                                                mediaItemQuery.impressionGuid = query.result.item.impressionGuid
                                            }
                                            break;
                                        case"Episode":
                                            if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.tvMarketplace)) {
                                                mediaItemQuery = new MS.Entertainment.Data.Query.Video.EdsDetailTVEpisode;
                                                mediaItemQuery.id = query.result.item.serviceId;
                                                mediaItemQuery.idType = query.result.item.serviceIdType;
                                                mediaItemQuery.impressionGuid = query.result.item.impressionGuid
                                            }
                                            break;
                                        case"Series":
                                            if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.tvMarketplace)) {
                                                mediaItemQuery = new MS.Entertainment.Data.Query.Video.EdsDetailTVSeries;
                                                mediaItemQuery.id = query.result.item.serviceId;
                                                mediaItemQuery.idType = query.result.item.serviceIdType;
                                                mediaItemQuery.impressionGuid = query.result.item.impressionGuid;
                                                mediaItemQuery.relevancyTrackingContent = query.result.item.relevancyTrackingContent
                                            }
                                            break;
                                        case"Season":
                                            if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.tvMarketplace)) {
                                                mediaItemQuery = new MS.Entertainment.Data.Query.Video.EdsDetailTVSeason;
                                                mediaItemQuery.id = query.result.item.serviceId;
                                                mediaItemQuery.idType = query.result.item.serviceIdType;
                                                mediaItemQuery.impressionGuid = query.result.item.impressionGuid
                                            }
                                            break;
                                        default:
                                            break
                                    }
                                if (mediaItemQuery)
                                    mediaItemQuery.execute().then(function mediaItemQuerySuccess(query) {
                                        c(query.result.item)
                                    }, function mediaItemQueryError(error) {
                                        e(error.errorCode)
                                    });
                                else
                                    e()
                            }, function contentTypeQueryError(error) {
                                e(error.errorCode)
                            })
                        }
                        else
                            e()
                    })
            }, getGameMediaByTitleId: function getGameMediaByTitleId(titleId) {
                titleId = parseInt(titleId);
                if (!titleId)
                    return WinJS.Promise.wrapError(null);
                var edsMediaItem = MS.Entertainment.Data.augment({TitleId: titleId}, MS.Entertainment.Data.Augmenter.Marketplace.XboxGame);
                MS.Entertainment.ViewModels.MediaItemModel.augment(edsMediaItem);
                if (edsMediaItem.hydrate)
                    return edsMediaItem.hydrate().then(function hydrateSuccess() {
                            return edsMediaItem
                        }, function hydrateFailed() {
                            return edsMediaItem
                        });
                else
                    return WinJS.Promise.wrap(edsMediaItem)
            }, getMusicMediaByServiceId: function _getArtistMediaByServiceId(serviceId, serviceIdType, mediaType) {
                var featureEnablement = this._getFeatureEnablement();
                return new WinJS.Promise(function(c, e, p) {
                        if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace) && MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.networkStatus).isEnabled(MS.Entertainment.UI.NetworkStatusService.NetworkedFeature.musicMarketplace)) {
                            var useCanonicalId = true;
                            if (serviceIdType) {
                                var idType = serviceIdType.toLocaleLowerCase();
                                if (idType === MS.Entertainment.Data.Query.edsIdType.canonical.toLocaleLowerCase())
                                    useCanonicalId = true;
                                else if (idType.toLocaleLowerCase() === MS.Entertainment.Data.Query.edsIdType.zuneCatalog.toLocaleLowerCase())
                                    useCanonicalId = false;
                                else
                                    MS.Entertainment.UI.Components.Shell.fail("Unknown media ID type.")
                            }
                            var item = useCanonicalId ? {ID: serviceId} : {ZuneId: serviceId};
                            item.location = MS.Entertainment.Data.ItemLocation.marketplace;
                            switch (mediaType.toLocaleLowerCase()) {
                                case MS.Entertainment.Data.Query.edsMediaType.musicArtist.toLocaleLowerCase():
                                    var artist = MS.Entertainment.Data.augment(item, MS.Entertainment.Data.Augmenter.Marketplace.EDSArtist);
                                    if (artist) {
                                        MS.Entertainment.ViewModels.MediaItemModel.augment(artist);
                                        artist.hydrate().done(function completeHydrate(media) {
                                            c(media)
                                        }, function hydrateError(error) {
                                            e(error)
                                        })
                                    }
                                    else {
                                        error = new Error("Failed to augment artist");
                                        e(error)
                                    }
                                    break;
                                case MS.Entertainment.Data.Query.edsMediaType.album.toLocaleLowerCase():
                                    var album = MS.Entertainment.Data.augment(item, MS.Entertainment.Data.Augmenter.Marketplace.EDSAlbum);
                                    if (album) {
                                        MS.Entertainment.ViewModels.MediaItemModel.augment(album);
                                        album.hydrate().done(function completeHydrate(media) {
                                            c(media)
                                        }, function hydrateError(error) {
                                            e(error)
                                        })
                                    }
                                    else {
                                        error = new Error("Failed to augment album");
                                        e(error)
                                    }
                                    break;
                                case MS.Entertainment.Data.Query.edsMediaType.track.toLocaleLowerCase():
                                    var track = MS.Entertainment.Data.augment(item, MS.Entertainment.Data.Augmenter.Marketplace.EDSTrack);
                                    if (track) {
                                        MS.Entertainment.ViewModels.MediaItemModel.augment(track);
                                        track.hydrate({forceUpdate: true}).done(function completeHydrate(media) {
                                            c(media)
                                        }, function hydrateError(error) {
                                            e(error)
                                        })
                                    }
                                    else {
                                        error = new Error("Failed to augment track");
                                        e(error)
                                    }
                                    break;
                                case MS.Entertainment.Data.Query.edsMediaType.musicPlaylist.toLocaleLowerCase():
                                    var error;
                                    if (serviceId) {
                                        error = new Error("Marketplace playlists not implemented");
                                        e(error)
                                    }
                                    else {
                                        error = new Error("No service id given");
                                        e(error)
                                    }
                                    break;
                                case MS.Entertainment.Data.Query.edsMediaType.musicVideo.toLocaleLowerCase():
                                    var musicVideo = MS.Entertainment.Data.augment(item, MS.Entertainment.Data.Augmenter.Marketplace.EDSMusicVideo);
                                    if (musicVideo) {
                                        MS.Entertainment.ViewModels.MediaItemModel.augment(musicVideo);
                                        musicVideo.hydrate({forceUpdate: true}).done(function completeHydrate(media) {
                                            c(media)
                                        }, function hydrateError(error) {
                                            e(error)
                                        })
                                    }
                                    else {
                                        error = new Error("Failed to augment musicVideo");
                                        e(error)
                                    }
                                    break;
                                default:
                                    MS.Entertainment.UI.Components.Shell.fail("Unknown desired media type.");
                                    break
                            }
                        }
                        else
                            e(MS.Entertainment.Platform.Playback.Error.NS_E_WMPIM_USEROFFLINE.code)
                    })
            }, isXboxConsoleAvailableInRegion: function _isXboxConsoleAvailableInRegion() {
                var result = true;
                result = this._getFeatureEnablement().isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.xbox360ConsoleRegions);
                return result
            }, isPlayToXboxFeatureEnabled: function _isPlayToXboxFeatureEnabled(mediaItem) {
                var result = this.isXboxConsoleAvailableInRegion();
                if (result)
                    if (MS.Entertainment.Utilities.isVideoApp) {
                        var featureEnablement = this._getFeatureEnablement();
                        if (mediaItem)
                            if (this.isTVEpisode(mediaItem))
                                result = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.tvMarketplace);
                            else if (this.isMusicVideo(mediaItem))
                                result = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicVideosMarketplace);
                            else
                                result = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.moviesMarketplace);
                        else
                            result = (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.tvMarketplace) || featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.moviesMarketplace) || featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicVideosMarketplace))
                    }
                return result
            }, isClosedCaptionFeatureEnabled: function _isClosedCaptionFeatureEnabled() {
                return this._getFeatureEnablement().isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.closedCaptionsEnabled) && MS.Entertainment.Utilities.isVideoApp
            }, playMedia: function playMedia(mediaItem, showDetails, startPositionMsec) {
                this.playMedia2(mediaItem, {
                    sessionId: MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying, autoPlay: true, startPositionMsec: startPositionMsec, showImmersive: true, immersiveOptions: {
                            sessionId: MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying, startFullScreen: !showDetails
                        }
                })
            }, playMediaOnXbox: function playMediaOnXbox(mediaItem, titleId, deepLinkInfo, startPositionMsec) {
                this.playMedia2(mediaItem, {
                    sessionId: MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.remoteLRC, titleId: titleId, autoPlay: true, deepLinkInfo: deepLinkInfo, startPositionMsec: startPositionMsec, showImmersive: true, showSmartGlassActivity: true, showAppBar: false, appBarHideTimeoutMS: MS.Entertainment.UI.Controls.BottomAppBar.defaultCompanionHideTimeoutMS, immersiveOptions: {
                            sessionId: MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.remoteLRC, startFullScreen: false
                        }
                })
            }, showImmersiveDetails: function showImmersiveDetails(mediaItem, showDetails, autoPlay, hub, options, sessionId, completeCallBack, titleId, deepLink, startPositionMsec) {
                var immersiveOptions = {
                        startFullScreen: !showDetails, hub: hub, panelOptions: options, sessionId: sessionId
                    };
                var playPreviewOnly = (mediaItem && mediaItem.playPreviewOnly) ? mediaItem.playPreviewOnly : false;
                var playOptions = {
                        autoPlay: autoPlay, showImmersive: true, immersiveOptions: immersiveOptions, sessionId: sessionId, titleId: titleId, deepLink: deepLink, playPreviewOnly: playPreviewOnly, startPositionMsec: startPositionMsec, playbackSource: MS.Entertainment.Platform.PlaybackHelpers.PlaybackSource.details
                    };
                if (autoPlay)
                    this.playMedia2(mediaItem, playOptions);
                else
                    this.showImmersive(mediaItem, immersiveOptions)
            }, addToNowPlaying: function addToNowPlaying(mediaItem, collectionFilter, preventPlay) {
                this.playMedia2(mediaItem, {
                    autoPlay: true, showAppBar: true, queueMedia: true, showImmersive: false, preservePlayContext: true, collectionFilter: collectionFilter, preventPlay: preventPlay
                })
            }, waitForTransportStateOrTimeout: function waitForTransportStateOrTimeout(tsToWaitFor, tmoMsec) {
                MS.Entertainment.Platform.Playback.Etw.traceString("+PlaybackHelpers::waitForTransportStateOrTimeout(" + tsToWaitFor + ", " + tmoMsec + ")");
                var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                var playbackSession = sessionMgr.primarySession;
                var uberPromise = new WinJS.Promise(function _uberPromise(c, e, p) {
                        var tmoPromise = null;
                        var tsChanged = function _tsChanged(transportState) {
                                if (transportState === tsToWaitFor) {
                                    MS.Entertainment.Platform.Playback.Etw.traceString("PlaybackHelpers::waitForTransportStateOrTimeout, tsChanged : " + transportState);
                                    playbackSession.unbind("currentTransportState", tsChanged);
                                    if (tmoPromise)
                                        tmoPromise.cancel();
                                    c()
                                }
                                else if (playbackSession.playerState === MS.Entertainment.Platform.Playback.PlayerState.error && playbackSession.errorDescriptor) {
                                    MS.Entertainment.Platform.Playback.Etw.traceString("PlaybackHelpers::waitForTransportStateOrTimeout, playerState error");
                                    playbackSession.unbind("currentTransportState", tsChanged);
                                    if (tmoPromise)
                                        tmoPromise.cancel();
                                    c()
                                }
                            };
                        var tmoExpired = function _tmoExpired() {
                                MS.Entertainment.Platform.Playback.Etw.traceString("PlaybackHelpers::waitForTransportStateOrTimeout, timed out");
                                playbackSession.unbind("currentTransportState", tsChanged);
                                c()
                            };
                        tmoPromise = WinJS.Promise.timeout(tmoMsec).then(tmoExpired);
                        playbackSession.bind("currentTransportState", tsChanged)
                    });
                MS.Entertainment.Platform.Playback.Etw.traceString("-PlaybackHelpers::waitForTransportStateOrTimeout");
                return uberPromise
            }, playMedia2: function playMedia2(mediaItem, options) {
                var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                sessionMgr.nowPlayingSession.isPlayToReceiverSessionRunning = !!options.isPlayToReceiverSessionRunning;
                var playOptions = options || {};
                if (!mediaItem.activationFilePath) {
                    MS.Entertainment.Instrumentation.PerfTrack.enableScenarioPlayProtectedInApp();
                    MS.Entertainment.Instrumentation.PerfTrack.enableScenarioPlayNonProtectedInApp()
                }
                if (playOptions.playContext) {
                    MS.Entertainment.Platform.PlaybackHelpers.assert(!playOptions.preservePlayContext, "preservePlayContext and playContext both supplied; the session playContext will not be changed.");
                    MS.Entertainment.Platform.PlaybackHelpers.assert(!playOptions.playContext.isSmartDJ || playOptions.playContext.artistSeed, "isSmartDJ parameter is set in the playback context, but no SmartDJ seed was specified.")
                }
                playOptions.autoPlay = (playOptions.autoPlay !== undefined) ? playOptions.autoPlay : true;
                playOptions.showImmersive = (playOptions.showImmersive !== undefined) ? playOptions.showImmersive : false;
                MS.Entertainment.Platform.PlaybackHelpers.playActionInitiated = true;
                MS.Entertainment.Platform.PlaybackHelpers.lastPlaybackSource = playOptions.playbackSource || String.empty;
                MS.Entertainment.Platform.Playback.Etw.traceString("+PlaybackHelpers::playMedia2" + ", autoPlay=" + playOptions.autoPlay + ", showImmersive=" + playOptions.showImmersive + ", showAppbar=" + playOptions.showAppBar + ", preventNavigateToDefault=" + playOptions.preventNavigateToDefault + ", callback=" + (playOptions.completeCallBack ? true : false));
                var immersiveOptions = playOptions.immersiveOptions ? playOptions.immersiveOptions : {};
                if (!playOptions.sessionId)
                    playOptions.sessionId = sessionMgr.primarySession.sessionId;
                if (!playOptions.setOnly && !options.queueMedia && !playOptions.doNotNavigate && this.forceFullScreenNowPlaying) {
                    playOptions.showImmersive = true;
                    playOptions.showAppBar = false
                }
                if (mediaItem && mediaItem.playPreviewOnly)
                    playOptions.sessionId = MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying;
                immersiveOptions.sessionId = playOptions.sessionId;
                var validateExplicitPrivilege = true;
                var doPlay = function doPlay(userCancelled) {
                        if (userCancelled)
                            return;
                        var navigationService;
                        if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.navigation))
                            navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                        MS.Entertainment.Platform.Playback.Etw.traceString("+PlaybackHelpers::doPlay");
                        this._validateMediaItem(mediaItem, validateExplicitPrivilege).then(function validateMediaItem_completed() {
                            MS.Entertainment.Platform.PlaybackHelpers._play(mediaItem, playOptions);
                            if (playOptions.navigateToDetailsPage)
                                MS.Entertainment.Platform.PlaybackHelpers.navigateToDetailsPage(playOptions.album ? playOptions.album : mediaItem);
                            if (playOptions.showImmersive)
                                MS.Entertainment.Platform.PlaybackHelpers.waitForTransportStateOrTimeout(MS.Entertainment.Platform.Playback.TransportState.starting, 2500).then(function _delayedShowImmersive() {
                                    var playbackSession = sessionMgr.nowPlayingSession;
                                    if (playbackSession.playerState !== MS.Entertainment.Platform.Playback.PlayerState.error)
                                        MS.Entertainment.Platform.PlaybackHelpers.showImmersive(mediaItem, immersiveOptions);
                                    else if (navigationService && !navigationService.currentPage)
                                        navigationService.navigateToDefaultPage()
                                });
                            else if (!playOptions.preventNavigateToDefault)
                                if (navigationService && !navigationService.currentPage)
                                    navigationService.navigateToDefaultPage();
                            if (!MS.Entertainment.Utilities.isApp2 && playOptions.showAppBar && !window.onNewMusicPage) {
                                var playbackSession = sessionMgr.nowPlayingSession;
                                var showPromise = null;
                                var appBarShown = false;
                                var showPromiseTimeoutMS = 1200;
                                var onPlaybackStarted = function showAppBarOnPlaybackStart(currentPlaybackState) {
                                        if (currentPlaybackState === MS.Entertainment.Platform.Playback.TransportState.starting || currentPlaybackState === MS.Entertainment.Platform.Playback.TransportState.playing) {
                                            if (showPromise)
                                                showPromise.cancel();
                                            showAppBar()
                                        }
                                        else if (playbackSession.errorDescriptor)
                                            playbackSession.unbind("currentTransportState", onPlaybackStarted)
                                    };
                                var showAppBar = function showAppBar() {
                                        if (appBarShown)
                                            return;
                                        appBarShown = true;
                                        playbackSession.unbind("currentTransportState", onPlaybackStarted);
                                        showPromise = null;
                                        var appBarControl = MS.Entertainment.globalControls.getControl(MS.Entertainment.globalControls.GlobalControl.appBar);
                                        if (appBarControl)
                                            appBarControl.show((isNaN(playOptions.appBarHideTimeoutMS) ? MS.Entertainment.UI.Controls.BottomAppBar.defaultHideTimeoutMS : playOptions.appBarHideTimeoutMS))
                                    };
                                showPromise = WinJS.Promise.timeout(showPromiseTimeoutMS).done(showAppBar);
                                playbackSession.bind("currentTransportState", onPlaybackStarted)
                            }
                            if (playOptions.completeCallBack)
                                playOptions.completeCallBack();
                            {}
                        }.bind(this));
                        MS.Entertainment.Platform.Playback.Etw.traceString("-PlaybackHelpers::doPlay")
                    }.bind(this);
                if (sessionMgr.lrcSession && playOptions.sessionId === sessionMgr.lrcSession.sessionId) {
                    validateExplicitPrivilege = false;
                    if (!playOptions.titleId)
                        playOptions.titleId = MS.Entertainment.Platform.LivingRoomCompanion.WellKnownTitleId.ze;
                    if (sessionMgr.lrcSession && sessionMgr.lrcSession.sessionState === MS.Entertainment.Platform.LivingRoomCompanion.SessionState.connected)
                        if (parseInt(playOptions.titleId) === sessionMgr.lrcSession.currentTitleId || sessionMgr.lrcSession.currentTitleId === MS.Entertainment.Platform.LivingRoomCompanion.WellKnownTitleId.dd || sessionMgr.lrcSession.currentTitleId === MS.Entertainment.Platform.LivingRoomCompanion.WellKnownTitleId.bb || sessionMgr.lrcSession.currentTitleId === MS.Entertainment.Platform.LivingRoomCompanion.WellKnownTitleId.ze || sessionMgr.lrcSession.currentTitleId === MS.Entertainment.Platform.LivingRoomCompanion.WellKnownTitleId.mc)
                            doPlay();
                        else {
                            MS.Entertainment.UI.Controls.XBoxControls.hide();
                            this._showConfirmationDialog(doPlay, function handleNoInterrupt() {
                                var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                                if (!navigationService.currentPage)
                                    navigationService.navigateToDefaultPage()
                            }.bind(this))
                        }
                }
                else
                    doPlay();
                MS.Entertainment.Platform.Playback.Etw.traceString("-PlaybackHelpers::playMedia2")
            }, navigateToDetailsPage: function navigateToDetailsPage(mediaItem) {
                if (MS.Entertainment.Utilities.isApp1 && MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.winJSNavigation)) {
                    var actionId;
                    var actionParameter = {
                            data: mediaItem, isTrack: false
                        };
                    switch (mediaItem.mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                            actionId = MS.Entertainment.UI.Actions.ActionIdentifiers.artistDetailsNavigate;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                            actionId = MS.Entertainment.UI.Actions.ActionIdentifiers.albumDetailsNavigate;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                            actionId = MS.Entertainment.UI.Actions.ActionIdentifiers.albumDetailsNavigate;
                            actionParameter.isTrack = true;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.playlist:
                            actionId = MS.Entertainment.UI.Actions.ActionIdentifiers.playlistDetailsNavigate;
                            break;
                        default:
                            MS.Entertainment.UI.fail("Not a supported media type. Type: " + mediaItem.mediaType)
                    }
                    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                    var actionToExecute = actionService.getAction(actionId);
                    if (actionToExecute) {
                        actionToExecute.parameter = actionParameter;
                        actionToExecute.execute()
                    }
                }
            }, showImmersive: function showImmersive(mediaItem, immersiveOptions) {
                var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                immersiveOptions = immersiveOptions ? immersiveOptions : {};
                immersiveOptions.startFullScreen = (immersiveOptions.startFullScreen !== undefined) ? immersiveOptions.startFullScreen : false;
                if (!immersiveOptions.startFullScreen)
                    if (MS.Entertainment.Platform.PlaybackHelpers.isMovie(mediaItem)) {
                        mediaItem.hydrate();
                        MS.Entertainment.Instrumentation.PerfTrack.setStartTime(MS.Entertainment.Instrumentation.PerfTrack.StartPoints.MovieDetailsRequest)
                    }
                    else if (MS.Entertainment.Platform.PlaybackHelpers.isTVEpisode(mediaItem) || MS.Entertainment.Platform.PlaybackHelpers.isTVSeason(mediaItem) || MS.Entertainment.Platform.PlaybackHelpers.isTVSeries(mediaItem)) {
                        mediaItem.hydrate();
                        MS.Entertainment.Instrumentation.PerfTrack.setStartTime(MS.Entertainment.Instrumentation.PerfTrack.StartPoints.TVDetailsRequest)
                    }
                if (!MS.Entertainment.Data.List.isListOrArray(mediaItem) && !MS.Entertainment.Data.MainQuery.isQuery(mediaItem))
                    immersiveOptions.mediaItem = mediaItem;
                if (immersiveOptions.sessionId) {
                    if (!immersiveOptions.mediaItem) {
                        var playbackSession = sessionMgr.getSession(immersiveOptions.sessionId);
                        immersiveOptions.mediaItem = playbackSession.currentMedia
                    }
                    if (this.useFullScreenNowPlaying || this.forceFullScreenNowPlaying) {
                        var fullScreenNowPlayingOptions = null;
                        if (MS.Entertainment.Utilities.isVideoApp)
                            fullScreenNowPlayingOptions = {
                                mediaItem: immersiveOptions.mediaItem, showNotifications: false
                            };
                        immersiveOptions.startFullScreen = true;
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateTo(MS.Entertainment.UI.Monikers.fullScreenNowPlaying, null, null, fullScreenNowPlayingOptions);
                        if (immersiveOptions.completeCallback)
                            immersiveOptions.completeCallback(true);
                        {};
                        return
                    }
                }
                if (immersiveOptions.mediaItem) {
                    var forcePageChange = true;
                    var currentPageIsSameImmersive = false;
                    if (immersiveOptions.overridePageChange !== undefined)
                        forcePageChange = immersiveOptions.overridePageChange;
                    else {
                        var currentLocation = MS.Entertainment.Utilities.getCurrentUserLocation();
                        if (WinJS.Utilities.getMember("MS.Entertainment.UI.Monikers") && currentLocation === MS.Entertainment.UI.Monikers.immersiveDetails) {
                            var currentPage = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).currentPage;
                            if (currentPage.options.mediaItem.isEqual && immersiveOptions.mediaItem.isChildOf && immersiveOptions.mediaItem.sharesParentWith) {
                                currentPageIsSameImmersive = currentPage.options.mediaItem.isEqual(immersiveOptions.mediaItem) || immersiveOptions.mediaItem.isChildOf(currentPage.options.mediaItem) || immersiveOptions.mediaItem.sharesParentWith(currentPage.options.mediaItem);
                                forcePageChange = !currentPageIsSameImmersive
                            }
                        }
                    }
                    var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                    if (MS.Entertainment.Utilities.isMusicApp && featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace) && immersiveOptions.sessionId)
                        immersiveOptions.hideBackButtonOnNavigate = true;
                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateTo(MS.Entertainment.UI.Monikers.immersiveDetails, null, null, immersiveOptions, forcePageChange);
                    if (currentPageIsSameImmersive && immersiveOptions.completeCallback)
                        immersiveOptions.completeCallback(false)
                }
                else if (immersiveOptions.forceNavigate)
                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateTo(MS.Entertainment.UI.Monikers.immersiveDetails, null, null, immersiveOptions, false);
                else {
                    var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                    if (!navigationService.currentPage)
                        navigationService.navigateToDefaultPage();
                    if (immersiveOptions.completeCallback)
                        immersiveOptions.completeCallback(false);
                    {}
                }
            }, playToXbox: function playToXbox(mediaItem, provider, startPositionMsec) {
                if (!mediaItem)
                    return;
                var mediaItemModel = mediaItem;
                var _getInfoAndLaunchCompanion = function _getInfoAndLaunchCompanion() {
                        var titleId = provider ? provider.partnerApplicationLaunchInfoList[0].TitleId : MS.Entertainment.Platform.LivingRoomCompanion.WellKnownTitleId.ze;
                        var deepLinkInfo = provider ? provider.partnerApplicationLaunchInfoList[0].deepLinkInfo : null;
                        var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                        var session = sessionMgr.primarySession;
                        if (mediaItemModel.mediaType === Microsoft.Entertainment.Queries.ObjectType.game)
                            titleId = mediaItemModel.titleId;
                        var launchInfo = "xboxsmartglass://launchTitle/?";
                        launchInfo += "titleId=" + titleId + "&mediaType=" + mediaItemModel.mediaType;
                        if (mediaItemModel.hasZuneId)
                            launchInfo += "&serviceId=" + mediaItemModel.zuneId;
                        else if (mediaItemModel.hasServiceId)
                            launchInfo += "&serviceId=" + mediaItemModel.serviceId;
                        if (deepLinkInfo)
                            launchInfo += "&deepLinkInfo=" + encodeURIComponent(deepLinkInfo);
                        if (startPositionMsec)
                            launchInfo += "&startPositionMsec=" + startPositionMsec;
                        launchInfo += "&firstAction=playnow";
                        var appAction = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.launchApp);
                        appAction.automationId = MS.Entertainment.UI.AutomationIds.launchAppPlayToXbox;
                        appAction.parameter = {
                            uri: launchInfo, appendSource: true, appendGamerTag: true
                        };
                        appAction.execute()
                    }.bind(this);
                if (!mediaItem.hydrated) {
                    var mediaItemModel = MS.Entertainment.ViewModels.MediaItemModel.augment(mediaItem);
                    if (mediaItemModel.hydrate)
                        mediaItemModel.hydrate().then(function _hydrated() {
                            _getInfoAndLaunchCompanion()
                        })
                }
                else
                    _getInfoAndLaunchCompanion()
            }, playToXboxPauseLocalPlayback: function _playToXboxPauseLocalPlayback(mediaItem, provider, startPositionMsec) {
                var sessionManager = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                if (sessionManager && sessionManager.primarySession.sessionId === MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying)
                    switch (sessionManager.primarySession.targetTransportState) {
                        case MS.Entertainment.Platform.Playback.TransportState.playing:
                        case MS.Entertainment.Platform.Playback.TransportState.starting:
                        case MS.Entertainment.Platform.Playback.TransportState.buffering:
                            sessionManager.primarySession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.paused;
                            break
                    }
                this.playToXbox(mediaItem, provider, startPositionMsec)
            }, playFromXbox: function playFromXbox(mediaItem, startPositionMsec) {
                if (!mediaItem)
                    return;
                MS.Entertainment.Platform.PlaybackHelpers.assert(mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.video, "playFromXbox: only video can be transfered to X8");
                var mediaItemModel = mediaItem;
                function _getInfoAndLaunchX8() {
                    var launchInfo = "microsoftvideo://play/?id=" + mediaItemModel.serviceId;
                    if (startPositionMsec)
                        launchInfo += "&startPositionMsec=" + startPositionMsec;
                    var appAction = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.launchApp);
                    appAction.automationId = MS.Entertainment.UI.AutomationIds.launchAppVideoApp;
                    appAction.parameter = {
                        uri: launchInfo, appendSource: true, appendGamerTag: true
                    };
                    appAction.execute()
                }
                if (!mediaItem.hydrated && !mediaItem.hasServiceId) {
                    var mediaItemModel = MS.Entertainment.ViewModels.MediaItemModel.augment(mediaItem);
                    if (mediaItemModel.hydrate)
                        mediaItemModel.hydrate().then(function _hydrated() {
                            _getInfoAndLaunchX8()
                        })
                }
                else
                    _getInfoAndLaunchX8()
            }, showPlaybackError: function showPlaybackError(error) {
                var title = String.load(String.id.IDS_PLAYBACK_ERROR_MESSAGE_TITLE);
                return MS.Entertainment.UI.Shell.showError(title, error.code)
            }, showItemDetails: function showItemDetails(popOverOptions, immersiveOptions) {
                var checkOffline = false;
                var mediaItem = null;
                var location = null;
                immersiveOptions = immersiveOptions || {};
                var forceDetails = immersiveOptions.forceDetails || false;
                var completeCallback = immersiveOptions.completeCallback || null;
                var configManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                if (popOverOptions && popOverOptions.dataContext) {
                    mediaItem = popOverOptions.dataContext.data;
                    location = popOverOptions.dataContext.location
                }
                MS.Entertainment.Platform.PlaybackHelpers.assert(mediaItem, "popOverOptions.dataContext.data required to show item details");
                if (mediaItem) {
                    var showImmersive = false;
                    if (MS.Entertainment.Utilities.isVideoApp) {
                        showImmersive = this.isMovie(mediaItem) || this.isTVSeries(mediaItem) || this.isTVSeason(mediaItem) || (this.isTVEpisode(mediaItem) && forceDetails);
                        checkOffline = true
                    }
                    else if (MS.Entertainment.Utilities.isMusicApp2)
                        if (this.isMusicArtist(mediaItem)) {
                            showImmersive = true;
                            mediaItem.fromCollection = popOverOptions.dataContext.location === MS.Entertainment.Pages.BaseMediaInlineDetails.Location.collection
                        }
                    if (checkOffline && !MS.Entertainment.UI.NetworkStatusService.isOnline() && showImmersive && location === MS.Entertainment.Data.ItemLocation.marketplace) {
                        var errorCode = MS.Entertainment.Platform.Playback.Error.NS_E_WMPIM_USEROFFLINE.code;
                        MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_FAILED_PANEL_HEADER), errorCode)
                    }
                    else if (this.isMovie(mediaItem) && MS.Entertainment.Utilities.isVideoApp2)
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateTo(MS.Entertainment.UI.Monikers.movieDetailsPage, null, null, {
                            mediaItem: mediaItem, completeCallback: completeCallback || null
                        }, true);
                    else if ((this.isTVSeries(mediaItem) || this.isTVSeason(mediaItem) || (this.isTVEpisode(mediaItem) && forceDetails)) && MS.Entertainment.Utilities.isVideoApp2) {
                        var model = new MS.Entertainment.Components.Video.TvDetailsModel;
                        model.start(mediaItem);
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateTo(MS.Entertainment.UI.Monikers.tvDetailsPage, null, null, {
                            mediaItem: mediaItem, model: model, completeCallback: completeCallback || null
                        }, true)
                    }
                    else if (showImmersive)
                        MS.Entertainment.Platform.PlaybackHelpers.showImmersive(mediaItem, immersiveOptions);
                    else
                        MS.Entertainment.UI.Controls.PopOver.showPopOver(popOverOptions)
                }
            }, shouldDisplayRatingImage: function shouldDisplayRatingImage() {
                var playbackSession = null;
                var displayRatingImage = false;
                var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                if (sessionMgr && sessionMgr.nowPlayingSession && sessionMgr.primarySession)
                    playbackSession = sessionMgr.nowPlayingSession;
                var globalizationManager = new Microsoft.Entertainment.Util.GlobalizationManager;
                var currentRegion = globalizationManager.getRegion().toLowerCase();
                var config = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                var supportedRegions = config.playback.videoRatingAvailable;
                if ((supportedRegions.toLowerCase().indexOf(currentRegion.toLowerCase()) >= 0) && playbackSession && playbackSession.getProtectionState() !== MS.Entertainment.Platform.Playback.ProtectionState.unprotected)
                    displayRatingImage = true;
                return displayRatingImage
            }, getVideoRatingImageAsync: function getVideoRatingImageAsync() {
                var playbackSession = null;
                var ratingImage = String.empty;
                if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.sessionManager))
                    playbackSession = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager).nowPlayingSession;
                if (playbackSession && playbackSession.currentMedia && playbackSession.currentMedia.hasServiceId && playbackSession.currentMedia.ratingImage) {
                    ratingImage = playbackSession.currentMedia.ratingImage;
                    return WinJS.Promise.wrap(ratingImage)
                }
                else if (playbackSession.currentMedia.activationFilePath && playbackSession.getProtectionState() !== MS.Entertainment.Platform.Playback.ProtectionState.unprotected) {
                    var query = new MS.Entertainment.Data.Query.libraryVideos;
                    query.videoUrl = playbackSession.currentMedia.activationFilePath;
                    return query.execute().then(function getVideoComplete(q) {
                            return q.result.firstItem.ratingImage
                        }, function getVideoError() {
                            return WinJS.Promise.as(ratingImage)
                        })
                }
                else
                    return WinJS.Promise.wrap(ratingImage)
            }, _validateMediaItem: function _validateMediaItem(mediaItem, validateExplicitPrivilege) {
                var validateAccountHasExplicitPrivilege = function validateAccountHasExplicitPrivilege() {
                        var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                        if (!validateExplicitPrivilege || signedInUser.hasExplicitPrivilege)
                            return WinJS.Promise.wrap();
                        else
                            return this.showPlaybackError(MS.Entertainment.Platform.Playback.makeParentallyBlockedError()).then(function() {
                                    return WinJS.Promise.wrapError(error)
                                })
                    }.bind(this);
                var validateGameMediaItem = function validateGameMediaItem() {
                        var signInService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                        if (signInService.isSignedIn)
                            return validateAccountHasExplicitPrivilege();
                        else
                            return signInService.signIn().then(function signin_completed() {
                                    if (signInService.isSignedIn)
                                        return validateAccountHasExplicitPrivilege();
                                    else
                                        return this.showPlaybackError(MS.Entertainment.Platform.Playback.Error.NS_E_EXPLICIT_CONTENT_SIGNIN_REQUIRED).then(function() {
                                                return WinJS.Promise.wrapError(error)
                                            })
                                }.bind(this))
                    }.bind(this);
                if (mediaItem && mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.game)
                    return validateGameMediaItem();
                else
                    return WinJS.Promise.wrap()
            }, _setProperty: function _setProperty(item, propertyName, propertyValue) {
                if (propertyName in item)
                    item[propertyName] = propertyValue;
                else
                    item.addProperty(propertyName, propertyValue)
            }, _setMediaAvailability: function _setMediaAvailability(mediaItem, mediaAvailabilty) {
                var mediaItems = Array.isArray(mediaItem) ? mediaItem : [mediaItem];
                mediaItems.forEach(function setMediaAvailability(mediaItem) {
                    if (mediaItem.inCollection && mediaItem.fromCollection && (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.album || mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.person || mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.playlist))
                        if (mediaAvailabilty || mediaItem._mediaAvailability)
                            this._setProperty(mediaItem, "_mediaAvailability", mediaAvailabilty)
                }, this)
            }, _play: function _play(mediaItem, options) {
                MS.Entertainment.UI.Controls.assert(mediaItem, "PlayAction::_play.  mediaItem argument is invalid");
                MS.Entertainment.UI.Controls.assert(options, "PlayAction::_play.  options argument is invalid");
                if (mediaItem && options) {
                    var trackToSearchFor = options.track;
                    if (trackToSearchFor)
                        trackToSearchFor.playPreviewOnly = options.playPreviewOnly ? options.playPreviewOnly : false;
                    var compareTrackItem = function compareTrackItem() {
                            return this.data.isEqual(trackToSearchFor)
                        };
                    compareTrackItem.track = trackToSearchFor;
                    mediaItem.playPreviewOnly = !!options.playPreviewOnly;
                    mediaItem.enableFallbackToPreview = !!options.enableFallbackToPreview;
                    MS.Entertainment.Platform.Playback.Etw.traceString("+PlaybackHelpers::_play");
                    var ensurePreownedMediaAddedAsyncPromise = WinJS.Promise.wrap();
                    if (!MS.Entertainment.Utilities.isMusicApp && mediaItem && !mediaItem.playPreviewOnly)
                        ensurePreownedMediaAddedAsyncPromise = MS.Entertainment.Platform.PurchaseHelpers.ensurePreownedMediaAddedAsync(mediaItem);
                    MS.Entertainment.Platform.Playback.Etw.traceString("+PlaybackHelpers::ensurePreownedMediaAddedAsyncPromise");
                    return ensurePreownedMediaAddedAsyncPromise.then(function ensurePreownedMediaAddedAsync_complete(dbResult) {
                            MS.Entertainment.Platform.Playback.Etw.traceString("-PlaybackHelpers::ensurePreownedMediaAddedAsyncPromise");
                            var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                            var playbackSession = sessionMgr.setPrimarySession(options.sessionId);
                            playbackSession._ensureSession().done(function verifiedSessionIsInitialized() {
                                playbackSession.shuffle = options.shuffle !== undefined ? options.shuffle : playbackSession.shuffle;
                                playbackSession.smartDJSeed = options.smartDJSeed !== undefined ? options.smartDJSeed : null;
                                var startPositionMsec = 0 | options.startPositionMsec;
                                var autoPlay = (typeof options.autoPlay) === "boolean" ? options.autoPlay : playbackSession.autoPlay;
                                playbackSession.autoPlay = autoPlay;
                                if (options.sessionId === MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying) {
                                    this._setMediaAvailability(mediaItem, options.collectionFilter);
                                    if (options.queueMedia && !options.setOnly) {
                                        MS.Entertainment.Platform.Playback.Etw.traceString("PlaybackHelpers queueItem: appending");
                                        var getCountPromise = playbackSession.mediaCollection ? playbackSession.mediaCollection.getCount() : WinJS.Promise.wrap(0);
                                        getCountPromise.done(function insertNewMedia(currentCount) {
                                            playbackSession.insertAtEnd(null, mediaItem).done(function playIfPaused() {
                                                MS.Entertainment.Platform.Playback.Etw.traceString("PlaybackHelpers queueItem: append complete");
                                                if (options.autoPlay && !options.preventPlay && (playbackSession.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.paused || playbackSession.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.stopped || playbackSession.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.unInitialize))
                                                    playbackSession.playAt(currentCount)
                                            }, function errorWhileAppending(error) {
                                                MS.Entertainment.Platform.Playback.Etw.traceString("PlaybackHelpers queueItem: error while appending " + JSON.stringify(error))
                                            })
                                        })
                                    }
                                    else {
                                        playbackSession.setDataSource(mediaItem).done(function dataSourceSet(mediaCollection) {
                                            if (!!options.setOnly) {
                                                if (mediaCollection) {
                                                    var list = mediaCollection.createListBinding();
                                                    var index = options.offset || 0;
                                                    list.fromIndex(index).then(function first_in_collection(item) {
                                                        if (item) {
                                                            if (!item.data.hydrate) {
                                                                item.data.location = MS.Entertainment.Data.ItemLocation.marketplace;
                                                                MS.Entertainment.ViewModels.MediaItemModel.augment(item.data)
                                                            }
                                                            var setCurrentMedia = function setCurrentMedia() {
                                                                    playbackSession.currentMedia = item.data;
                                                                    playbackSession.pendingOrdinal = index
                                                                };
                                                            if (!item.data.hydrated)
                                                                item.data.hydrate({forceUpdate: true}).done(setCurrentMedia, setCurrentMedia);
                                                            else
                                                                setCurrentMedia()
                                                        }
                                                        list.release()
                                                    })
                                                }
                                                return
                                            }
                                            if (typeof(options.offset) !== "number" && options.offset < 0)
                                                options.offset = null;
                                            if (startPositionMsec > 0)
                                                playbackSession.playAt(options.offset || 0, startPositionMsec);
                                            else {
                                                playbackSession.autoPlay = autoPlay;
                                                if (options.offset === null)
                                                    playbackSession.activate(document.createElement("div"));
                                                else if (!trackToSearchFor)
                                                    playbackSession.playAt(options.offset);
                                                else
                                                    playbackSession.playAt(options.offset, 0, compareTrackItem)
                                            }
                                        }.bind(this), function dataSourceSetFailed(error){});
                                        if (!options.preservePlayContext && !options.queueMedia)
                                            sessionMgr.playContext = options.playContext || {}
                                    }
                                    var isPlaylist = mediaItem.execute !== undefined || this.isMusicTrack(mediaItem) || this.isMusicAlbum(mediaItem) || this.isPerson(mediaItem) || this.isMusicPlaylist(mediaItem) || this.isMusicVideo(mediaItem) || (Array.isArray(mediaItem) && mediaItem.length > 1) || (MS.Entertainment.Data.List.isList(mediaItem) && mediaItem.count > 0) || (mediaItem.source && mediaItem.source.length > 1);
                                    if ((options.saveNowPlaying === undefined || options.saveNowPlaying) && isPlaylist && !options.playPreviewOnly) {
                                        if (MS.Entertainment.Platform.PlaybackHelpers._saveNowPlayingPromise) {
                                            MS.Entertainment.Platform.PlaybackHelpers._saveNowPlayingPromise.cancel();
                                            MS.Entertainment.Platform.PlaybackHelpers._saveNowPlayingPromise = null
                                        }
                                        MS.Entertainment.Platform.PlaybackHelpers._saveNowPlayingPromise = WinJS.Promise.timeout(MS.Entertainment.Platform.PlaybackHelpers._saveNowPlayingTimeout).then(function saveNowPlaying() {
                                            playbackSession.savePlaylist(null, true);
                                            MS.Entertainment.Platform.PlaybackHelpers._saveNowPlayingPromise = null
                                        })
                                    }
                                }
                                else if (options.sessionId === MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.remoteLRC) {
                                    if (!options.titleId)
                                        options.titleId = MS.Entertainment.Platform.LivingRoomCompanion.WellKnownTitleId.ze;
                                    sessionMgr.nowPlayingSession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.stopped;
                                    MS.Entertainment.UI.Controls.ActivityOverlay.lastAutoLaunchId = null;
                                    if (options.queueMedia)
                                        playbackSession.insertAtEnd(null, mediaItem);
                                    else
                                        playbackSession.setDataSource(mediaItem, options.deepLinkInfo).then(function dataSourceSet() {
                                            playbackSession.playAt(options.titleId, startPositionMsec, MS.Entertainment.Platform.LivingRoomCompanion.LaunchFirstAction.playNow)
                                        }.bind(this))
                                }
                                MS.Entertainment.Platform.Playback.Etw.traceString("-PlaybackHelpers::_play")
                            }.bind(this), function sessionCannotBeInitialized(errorObj) {
                                MS.Entertainment.UI.Controls.assert(false, "Session initialization failure: " + JSON.stringify(errorObj))
                            }.bind(this))
                        }.bind(this), function ensurePreownedMediaAddedAsync_error(e) {
                            MS.Entertainment.UI.Controls.assert(false, "ensurePreownedMediaAddedAsync invoked the error handler.")
                        }.bind(this))
                }
            }, _showConfirmationDialog: function _showConfirmationDialog(yesCallback, noCallback) {
                MS.Entertainment.UI.Shell.showMessageBox(String.load(String.id.IDS_APPBAR_MESSAGE_XBOX_INTERUPTION_TITLE), String.load(String.id.IDS_APPBAR_MESSAGE_XBOX_INTERUPTION_TEXT), {
                    width: "65%", height: "50%", buttons: [{
                                title: String.load(String.id.IDS_YES_BUTTON), execute: function(overlay) {
                                        overlay.hide();
                                        yesCallback()
                                    }, isEnabled: true, isAvailable: true
                            }, {
                                title: String.load(String.id.IDS_NO_BUTTON), execute: function(overlay) {
                                        overlay.hide();
                                        noCallback()
                                    }, isEnabled: true, isAvailable: true
                            }], automationId: "LeaveXboxSessionMessageBox"
                })
            }, reloadNowPlaying: function reloadNowPlaying(startOnLoad) {
                var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                var isSubscription = configurationManager.service.lastSignedInUserSubscription;
                var firstFreeStringsLaunch = configurationManager.music.firstFreeStringsLaunch && !isSubscription;
                var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                if (startOnLoad || (!firstFreeStringsLaunch && MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).activationKind === Windows.ApplicationModel.Activation.ActivationKind.launch && !MS.Entertainment.Platform.PlaybackHelpers.playActionInitiated) && (!sessionMgr.nowPlayingSession || sessionMgr.nowPlayingSession.currentMedia === null)) {
                    var query = new MS.Entertainment.Data.Query.libraryPlaylists;
                    query.playlistType = Microsoft.Entertainment.Platform.PlaylistType.nowPlaying;
                    query.chunkSize = 1;
                    query.execute().then(function loadPlaylist(query) {
                        if (query.result.totalCount > 0)
                            query.result.items.toArray(0, 1).then(function playPlaylist(playlist) {
                                if (playlist.length >= 1 && (startOnLoad || !MS.Entertainment.Platform.PlaybackHelpers.playActionInitiated) && (!sessionMgr.nowPlayingSession || sessionMgr.nowPlayingSession.currentMedia === null)) {
                                    var queryPromise = WinJS.Promise.as();
                                    var mediaItemsQuery = null;
                                    var playOptions = null;
                                    if (playlist[0].count > 0) {
                                        mediaItemsQuery = new MS.Entertainment.Data.Query.libraryPlaylistMediaItems;
                                        mediaItemsQuery.playlistId = playlist[0].libraryId;
                                        playOptions = {
                                            autoPlay: true, setOnly: !startOnLoad, saveNowPlaying: false, showAppBar: false, queueMedia: !startOnLoad, showImmersive: false, doNotNavigate: true, offset: Math.max((playlist[0].currentItemIndex || 0), 0)
                                        };
                                        sessionMgr.nowPlayingSession.pendingOrdinal = playOptions.offset
                                    }
                                    else if (playlist[0].smartDjSeed && MS.Entertainment.Utilities.isValidGuid(playlist[0].smartDjSeed))
                                        queryPromise = MS.Entertainment.Platform.PlaybackHelpers.getMusicMediaByServiceId(playlist[0].smartDjSeed, MS.Entertainment.Data.Query.edsIdType.canonical, MS.Entertainment.Data.Query.edsMediaType.musicArtist).then(function loadMedia(media) {
                                            if (media) {
                                                mediaItemsQuery = new MS.Entertainment.Data.Query.Music.SmartDJ;
                                                mediaItemsQuery.id = media.canonicalId;
                                                mediaItemsQuery.acquisitionData = new MS.Entertainment.Utilities.AcquisitionData(Microsoft.Entertainment.Platform.AcquisitionContext.smartDJ, Microsoft.Entertainment.Platform.AcquisitionContextType.artist, media.canonicalId);
                                                playOptions = {
                                                    autoPlay: true, setOnly: !startOnLoad, saveNowPlaying: false, showAppBar: false, queueMedia: !startOnLoad, showImmersive: false, playbackSource: MS.Entertainment.Platform.PlaybackHelpers.PlaybackSource.radio, smartDJSeed: media, doNotNavigate: true, playContext: {
                                                            artistSeed: media, isSmartDJ: true
                                                        }
                                                }
                                            }
                                        });
                                    var onPlayerStateChanged = function onPlayerStateChanged(currentPlayerState) {
                                            if (!currentPlayerState)
                                                return;
                                            if (currentPlayerState === MS.Entertainment.Platform.Playback.PlayerState.notReady) {
                                                if (!MS.Entertainment.Platform.PlaybackHelpers.playActionInitiated) {
                                                    if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.adService))
                                                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.adService).skipAudioAdForNextTrack = true;
                                                    if (mediaItemsQuery && playOptions)
                                                        MS.Entertainment.Platform.PlaybackHelpers.playMedia2(mediaItemsQuery, playOptions);
                                                    else
                                                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).engageVisible = true
                                                }
                                            }
                                            else if (currentPlayerState === MS.Entertainment.Platform.Playback.PlayerState.error)
                                                MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.playbackErrorDisplayService).suppressNextPlaybackErrorDialog = true;
                                            sessionMgr.primarySession.unbind("playerState", onPlayerStateChanged)
                                        };
                                    var primarySessionIdChanged = function primarySessionIdChanged() {
                                            if (sessionMgr.primarySession) {
                                                sessionMgr.primarySession.bind("playerState", onPlayerStateChanged);
                                                MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).unbind("primarySessionId", primarySessionIdChanged)
                                            }
                                        };
                                    queryPromise.done(function initializeBinding() {
                                        if (mediaItemsQuery && playOptions)
                                            MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).bind("primarySessionId", primarySessionIdChanged);
                                        else
                                            MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).engageVisible = true
                                    }, function onError(e) {
                                        MS.Entertainment.fail("Error loading smartDj playlist: " + e.toString())
                                    })
                                }
                                else
                                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).engageVisible = true
                            });
                        else
                            MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).engageVisible = true
                    })
                }
                else
                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).engageVisible = true;
                if (firstFreeStringsLaunch)
                    configurationManager.music.firstFreeStringsLaunch = false
            }, isMovie: function MediaInstance_isMovie(mediaItem) {
                var isType = false;
                if (mediaItem)
                    isType = (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.video && mediaItem.videoType === Microsoft.Entertainment.Queries.VideoType.movie);
                return isType
            }, isTVEpisode: function MediaInstance_isTVEpisode(mediaItem) {
                var isType = false;
                if (mediaItem)
                    isType = (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.video && mediaItem.videoType === Microsoft.Entertainment.Queries.VideoType.tvEpisode);
                return isType
            }, isTVSeason: function MediaInstance_isTVSeason(mediaItem) {
                var isType = false;
                if (mediaItem)
                    isType = Microsoft.Entertainment.Queries.ObjectType.tvSeason === mediaItem.mediaType || (MS.Entertainment.Data.Augmenter.ServiceTypes.editorialItem === mediaItem.serviceType && MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Season === mediaItem.type);
                return isType
            }, isTVSeries: function MediaInstance_isTVSeries(mediaItem) {
                var isType = false;
                if (mediaItem)
                    isType = (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeries);
                return isType
            }, isAnyTV: function MediaInstance_isTV(mediaItem) {
                if (mediaItem)
                    return this.isTVEpisode(mediaItem) || this.isTVSeason(mediaItem) || this.isTVSeries(mediaItem)
            }, isVideo: function MediaInstance_isVideo(mediaItem) {
                var isType = false;
                if (mediaItem)
                    isType = (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.video && mediaItem.videoType !== Microsoft.Entertainment.Queries.VideoType.musicVideo);
                return isType
            }, isMusicArtist: function MediaInstance_isMusicArtist(mediaItem) {
                var isType = false;
                if (mediaItem)
                    isType = (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.person);
                return isType
            }, isMusicVideo: function MediaInstance_isMusicVideo(mediaItem) {
                var isType = false;
                if (mediaItem)
                    isType = (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.video && mediaItem.videoType === Microsoft.Entertainment.Queries.VideoType.musicVideo);
                return isType
            }, isMusicTrack: function MediaInstance_isMusicTrack(mediaItem) {
                var isType = false;
                if (mediaItem)
                    isType = (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.track);
                return isType
            }, isMusicTrackOrMusicVideo: function MediaInstance_isMusicTrackOrMusicVideo(mediaItem) {
                var isType = false;
                if (mediaItem)
                    isType = (MS.Entertainment.Platform.PlaybackHelpers.isMusicTrack(mediaItem) || MS.Entertainment.Platform.PlaybackHelpers.isMusicVideo(mediaItem));
                return isType
            }, isMusicAlbum: function MediaInstance_isMusicAlbum(mediaItem) {
                if (mediaItem)
                    return (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.album)
            }, isMusicPlaylist: function MediaInstance_isMusicPlaylist(mediaItem) {
                if (mediaItem)
                    return (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.playlist)
            }, isMusicOrMusicVideo: function MediaInstance_isMusicOrMusicVideo(mediaItem) {
                if (mediaItem)
                    return this.isMusicTrack(mediaItem) || this.isMusicAlbum(mediaItem) || this.isMusicVideo(mediaItem) || this.isMusicPlaylist(mediaItem)
            }, isAnyMusic: function MediaInstance_isAnyMusic(mediaItem) {
                if (mediaItem)
                    return this.isMusicTrack(mediaItem) || this.isMusicAlbum(mediaItem) || this.isMusicVideo(mediaItem) || this.isMusicArtist(mediaItem) || this.isMusicPlaylist(mediaItem)
            }, isGame: function MediaInstance_isGame(mediaItem) {
                if (mediaItem)
                    return (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.game)
            }, isPerson: function MediaInstance_isPerson(mediaItem) {
                var isType = false;
                if (mediaItem)
                    isType = (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.person);
                return isType
            }, forceFullScreenNowPlaying: {get: function() {
                    var uiState = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                    return (!uiState.isSnapped && MS.Entertainment.Utilities.isMusicApp2) || MS.Entertainment.Utilities.isVideoApp
                }}, useFullScreenNowPlaying: {get: function() {
                    return MS.Entertainment.Utilities.isMusicApp2 || MS.Entertainment.Utilities.isVideoApp
                }}, PlaybackSource: {
                fileActivation: "fileActivation", details: "details", radio: "radio", playlist: "playlist", playTo: "playTo", deeplink: "deeplink", postRoll: "postRoll"
            }, _saveNowPlayingTimeout: 10000, _saveNowPlayingPromise: null, deferredUpdateTimeout: 100, playActionInitiated: false, lastPlaybackSource: ""
    });
    WinJS.Namespace.define("MS.Entertainment.UI.Actions", {
        RemoveFromPlaybackSession: MS.Entertainment.deferredDerive("MS.Entertainment.UI.ToolbarAction", function RemoveFromPlaybackSession() {
            this.base()
        }, {
            automationId: MS.Entertainment.UI.AutomationIds.playlistRemoveFromNowPlaying, executed: function executed(param) {
                    param = this.transformParameter(param);
                    var keyPromise;
                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                    if (param && param.mediaItem && param.mediaItem.mediaCollection)
                        if (param.key)
                            keyPromise = WinJS.Promise.wrap(param.key);
                        else if (param.mediaItem.mediaCollection.itemFromIndex)
                            keyPromise = param.mediaItem.mediaCollection.itemFromIndex(param.offset).then(function gotItem(item) {
                                return item.key
                            });
                    if (keyPromise)
                        keyPromise.then(function gotKey(key) {
                            if (mediaContext)
                                mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.removeFromNowPlaying, param);
                            param.mediaItem.mediaCollection.remove(key)
                        }, function failedToGetKey(error) {
                            if (mediaContext)
                                mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.removeFromNowPlaying, param);
                            MS.Entertainment.Platform.PlaybackHelpers.fail("Failed to remove item from now playing. Error message " + error && error.message)
                        })
                }, transformParameter: function transformParameter(parameter) {
                    return parameter
                }, canExecute: function canExecute(param) {
                    param = this.transformParameter(param);
                    var canExecute = !!param && MS.Entertainment.Platform.Playback.PlaybackSession.isPlaybackSession(param.mediaItem) && !!param.mediaItem.mediaCollection && (!!param.key || (typeof param.offset === "number" & param.offset >= 0));
                    return canExecute
                }
        }), PlayMedia: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function PlayMedia() {
                this.base()
            }, {
                automationId: MS.Entertainment.UI.AutomationIds.playMedia, executed: function executed(param) {
                        MS.Entertainment.Instrumentation.PerfTrack.setStartTime(MS.Entertainment.Instrumentation.PerfTrack.StartPoints.PlayCommand);
                        var isImmersive = false;
                        if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.navigation)) {
                            var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                            isImmersive = navigationService.checkUserLocation(MS.Entertainment.UI.Monikers.immersiveDetails)
                        }
                        var playContext = {};
                        if (param.artistSeed) {
                            playContext.artistSeed = param.artistSeed;
                            playContext.isSmartDJ = param.isSmartDJ
                        }
                        else if (param.playlist)
                            playContext.playlist = param.playlist;
                        else if (param.mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.playlist)
                            playContext.playlist = param.mediaItem;
                        var playbackSource = String.empty;
                        if (playContext.isSmartDJ)
                            playbackSource = MS.Entertainment.Platform.PlaybackHelpers.PlaybackSource.radio;
                        else if (playContext.playlist)
                            playbackSource = MS.Entertainment.Platform.PlaybackHelpers.PlaybackSource.playlist;
                        if (MS.Entertainment.Platform.Playback.PlaybackSession.isPlaybackSession(param.mediaItem)) {
                            var playbackEventNotifications = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.playbackEventNotifications);
                            playbackEventNotifications.clearErrors();
                            var playbackSession = param.mediaItem;
                            playbackSession.autoPlay = true;
                            playbackSession.playAt(param.offset || 0)
                        }
                        else {
                            if (param.playPreviewOnly)
                                param.mediaItem.playPreviewOnly = param.playPreviewOnly;
                            else
                                param.mediaItem.playPreviewOnly = false;
                            var mediaItem = param.playMusicVideo ? param.mediaItem.musicVideo : param.mediaItem;
                            var isFilteredCollection = !!param.collectionFilter && param.collectionFilter !== Microsoft.Entertainment.Platform.MediaAvailability.available && param.collectionFilter !== Microsoft.Entertainment.Platform.MediaAvailability.undefined;
                            MS.Entertainment.UI.Actions.assert(!(mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.album && isFilteredCollection && param.playPreviewOnly), "Applying a filter and requesting preview only on an album hasn't been validated to work. You are attempting to do this. Please verify this works");
                            var album;
                            if (!isFilteredCollection && mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.album && !param.dontUseTrackListForPlayback && mediaItem.tracks) {
                                album = mediaItem;
                                mediaItem = mediaItem.tracks
                            }
                            MS.Entertainment.Instrumentation.PerfTrack.enableScenarioPlayProtectedInApp();
                            MS.Entertainment.Instrumentation.PerfTrack.enableScenarioPlayNonProtectedInApp();
                            MS.Entertainment.Platform.PlaybackHelpers.playMedia2(mediaItem, {
                                sessionId: MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying, autoPlay: true, saveNowPlaying: true, startPositionMsec: param.startPositionMS, showImmersive: !!param.showImmersive, showAppBar: (!!param.showAppBar && !isImmersive), shuffle: param.shuffle, queueMedia: !!param.queueMedia, playbackSource: playbackSource, immersiveOptions: {
                                        sessionId: MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying, startFullScreen: !param.showDetails, overridePageChange: param.overridePageChange
                                    }, offset: param.offset, track: param.track, playContext: playContext, playPreviewOnly: param.playPreviewOnly, preservePlayContext: param.preservePlayContext, collectionFilter: param.collectionFilter, album: album, navigateToDetailsPage: param.navigateToDetailsPage, enableFallbackToPreview: !!param.enableFallbackToPreview
                            })
                        }
                    }.bind(this), canExecute: function canExecute(param) {
                        return param && param.mediaItem && (!param.playMusicVideo || param.mediaItem.hasMusicVideoId)
                    }
            }), ShuffleNowPlaying: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function ShuffleNowPlaying() {
                this.base()
            }, {
                forceTitleChange: true, automationId: MS.Entertainment.UI.AutomationIds.transportShuffle, executed: function executed(param) {
                        var playbackSession = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager).nowPlayingSession;
                        playbackSession.shuffle = !playbackSession.shuffle;
                        if (!param || !param.ignoreLabel)
                            this.title = playbackSession.shuffle ? String.load(String.id.IDS_TRANSPORT_CONTROLS_SHUFFLE_ON_BUTTON) : String.load(String.id.IDS_TRANSPORT_CONTROLS_SHUFFLE_OFF_BUTTON)
                    }, canExecute: function canExecute(param) {
                        var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                        return !!(sessionMgr && sessionMgr.nowPlayingSession && sessionMgr.nowPlayingSession.canShuffle)
                    }
            }), RepeatNowPlaying: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function RepeatNowPlaying() {
                this.base()
            }, {
                forceTitleChange: true, automationId: MS.Entertainment.UI.AutomationIds.transportRepeat, executed: function executed(param) {
                        var playbackSession = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager).nowPlayingSession;
                        playbackSession.repeat = !playbackSession.repeat;
                        this.title = playbackSession.repeat ? String.load(String.id.IDS_TRANSPORT_CONTROLS_REPEAT_ON_BUTTON) : String.load(String.id.IDS_TRANSPORT_CONTROLS_REPEAT_OFF_BUTTON)
                    }, canExecute: function canExecute(param) {
                        return true
                    }
            }), ShowItemDetails: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function showItemDetails() {
                this.base()
            }, {
                automationId: MS.Entertainment.UI.AutomationIds.showItemDetails, executed: function executed(popOverParameters) {
                        if (popOverParameters && popOverParameters.clearBackStack && MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.navigation)) {
                            var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                            if (navigationService)
                                navigationService.clearBackStackOnNextNavigate(true)
                        }
                        MS.Entertainment.Platform.PlaybackHelpers.showItemDetails(popOverParameters)
                    }, canExecute: function canExecute(param) {
                        return true
                    }
            }), SkipBackward: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function skipBackward() {
                this.base();
                this.executed = this.executed.bind(this);
                this.canExecute = this.canExecute.bind(this)
            }, {
                automationId: MS.Entertainment.UI.AutomationIds.playbackSkipBackward, executed: function executed() {
                        var session = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager).primarySession;
                        if (session)
                            session.skipBack()
                    }, canExecute: function canExecute() {
                        var session = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager).primarySession;
                        return !!(session && session.canSkipBackward)
                    }
            }), SkipForward: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function skipForward() {
                this.base();
                this.executed = this.executed.bind(this);
                this.canExecute = this.canExecute.bind(this)
            }, {
                automationId: MS.Entertainment.UI.AutomationIds.playbackSkipForward, executed: function executed() {
                        var session = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager).primarySession;
                        if (session)
                            session.skipFwd()
                    }, canExecute: function canExecute() {
                        var session = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager).primarySession;
                        return !!(session && session.canSkipForward)
                    }
            }), ToggleMusicVideoPlayback: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function toggleMusicVideoPlayback() {
                this.base()
            }, {
                automationId: MS.Entertainment.UI.AutomationIds.toggleMusicVideoPlayback, executed: function executed() {
                        var session = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager).primarySession;
                        if (session)
                            session.musicVideosEnabled = !session.musicVideosEnabled
                    }, canExecute: function canExecute() {
                        var session = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager).primarySession;
                        return !!(session)
                    }
            })
    });
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia, function() {
        return new MS.Entertainment.UI.Actions.PlayMedia
    });
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.removeFromPlaybackSession, function() {
        return new MS.Entertainment.UI.Actions.RemoveFromPlaybackSession
    });
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.shuffleNowPlaying, function() {
        return new MS.Entertainment.UI.Actions.ShuffleNowPlaying
    });
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.repeatNowPlaying, function() {
        return new MS.Entertainment.UI.Actions.RepeatNowPlaying
    });
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.showItemDetails, function() {
        return new MS.Entertainment.UI.Actions.ShowItemDetails
    });
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.skipBackward, function() {
        return new MS.Entertainment.UI.Actions.SkipBackward
    });
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.skipForward, function() {
        return new MS.Entertainment.UI.Actions.SkipForward
    });
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.toggleMusicVideoPlayback, function() {
        return new MS.Entertainment.UI.Actions.ToggleMusicVideoPlayback
    })
})()
