﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Platform.Playback");
(function() {
    "use strict";
    var MSEPlatform = WinJS.Namespace.define("MS.Entertainment.Platform", null);
    var playbackControlMixin = {
            _playbackControlCreateInstance: function _playbackControlCreateInstance() {
                if (!this._playbackControl) {
                    var parkedPlaybackHost = document.createElement("div");
                    WinJS.Utilities.addClass(parkedPlaybackHost, "removeFromDisplay");
                    document.body.appendChild(parkedPlaybackHost);
                    this._parkedPlaybackHost = parkedPlaybackHost;
                    this._playbackControlContainer = document.createElement("div");
                    this._parkedPlaybackHost.appendChild(this._playbackControlContainer);
                    if (!this._playbackControlContainer)
                        throw"_playbackControlCreateInstance: Error! Cannot create playbackControlContainer. Out of memory?";
                    this._playbackControl = new MSEPlatform.Playback.PlaybackControl(this._playbackControlContainer, {});
                    if (!this._playbackControl)
                        throw"_playbackControlCreateInstance: Error! Cannot create playbackControl. Out of memory?";
                }
            }, _parkedPlaybackHost: null, _playbackControlHost: null, _playbackControlContainer: null, _playbackControl: null, _playbackControlEvents: null, _iPlayback: null
        };
    var playlistMixin = {
            _playlistCreateInstance: function _playlistCreateInstance() {
                if (!this._playlist)
                    this._playlist = new MSEPlatform.Playback.Playlist.PlaylistCore(null, {});
                if (!this._playlist)
                    throw"_playlistCreateInstance: Error! Cannot create playlist. Out of memory?";
                this._playlist.addEventListener("shuffleChanged", function _shuffleChanged(e) {
                    if (e.detail && e.detail.oldValue !== undefined)
                        try {
                            if (!this.smartDJSeed) {
                                this._saveSessionStateValue("shuffle", e.detail.newValue);
                                this._persistedShuffle = e.detail.newValue
                            }
                            else
                                MS.Entertainment.Platform.Playback.assert(!e.detail.newValue, "Shuffle should not be set to on while playing Radio")
                        }
                        catch(ex) {
                            MSEPlatform.Playback.Etw.traceString("PlaybackSession_shuffleChanged: ignored non-fatal exception: " + e)
                        }
                }.bind(this));
                this._playlist.addEventListener("repeatChanged", function _repeatChanged(e) {
                    if (e.detail && e.detail.oldValue !== undefined)
                        try {
                            if (!this.smartDJSeed) {
                                this._saveSessionStateValue("repeat", e.detail.newValue);
                                this._persistedRepeat = e.detail.newValue
                            }
                            else
                                MS.Entertainment.Platform.Playback.assert(!e.detail.newValue, "Repeat should not be set to on while playing Radio")
                        }
                        catch(ex) {
                            MSEPlatform.Playback.Etw.traceString("PlaybackSession_repeatChanged: ignored non-fatal exception: " + ex)
                        }
                }.bind(this))
            }, _playlist: null
        };
    var ccRendererMixin = {
            _ccRendererCreateInstance: function _ccRendererCreateInstance(playbackControl, rendererContainer) {
                if (!this._ccRenderer)
                    this._ccRenderer = new MSEPlatform.Playback.ClosedCaptions.Renderer(playbackControl, rendererContainer)
            }, _ccRenderer: null
        };
    WinJS.Namespace.define("MS.Entertainment.Platform.Playback", {
        _musicVideosEnabled: false, _musicVideosParentalBlockSet: false, _isMusicVideoModeInEffect: {get: function Playback_isMusicVideoModeInEffect() {
                    return MS.Entertainment.Platform.Playback._musicVideosEnabled && !MS.Entertainment.Platform.Playback._musicVideosParentalBlockSet
                }}
    });
    var playbackSessionMixin = {
            sessionId: {get: function PlaybackSession_sessionId_get() {
                    return this._sessionId
                }}, isPlayingVideo: {get: function PlaybackSession_isPlayingVideo_get() {
                        return this.mediaElement && this.mediaElement.tagName === "VIDEO"
                    }}, _setMusicVideosEnabled: function _playbackSessionMixin_setMusicVideoEnabled(value, isPersistedValue) {
                    function hasMusicVideoIdAsync(item) {
                        var q = new MS.Entertainment.Data.Query.Music.SongDetails;
                        q.id = item.serviceId;
                        q.idType = item.serviceIdType;
                        q.impressionGuid = item.impressionGuid;
                        if (item.acquisitionData && item.acquisitionData.acquisitionContext && item.acquisitionData.acquisitionContextType) {
                            var acquisitionAdditionalContext = item.acquisitionData.acquisitionAdditionalContext || item.playlistName || String.empty;
                            q.acquisitionData = new MS.Entertainment.Utilities.AcquisitionData(item.acquisitionData.acquisitionContext, item.acquisitionData.acquisitionContextType, item.acquisitionData.acquisitionContextID, acquisitionAdditionalContext)
                        }
                        return q.execute().then(function trackQueryCompleted(completedQuery) {
                                var marketplaceItem = null;
                                if (completedQuery && completedQuery.result) {
                                    marketplaceItem = completedQuery.result;
                                    if (marketplaceItem && marketplaceItem.data)
                                        return marketplaceItem.data.hasMusicVideoId
                                }
                                return false
                            }, function trackQueryError() {
                                return false
                            })
                    }
                    MS.Entertainment.Platform.Playback.assert(MS.Entertainment.Utilities.isMusicApp2, "At this time this property is expected to be manipulated by Music2App only. Remove the assert if this is no longer true " + "or check if unintended call happened and remove it");
                    var oldValue = MS.Entertainment.Platform.Playback._isMusicVideoModeInEffect;
                    if (isPersistedValue) {
                        MS.Entertainment.Platform.Playback._musicVideosEnabled = value;
                        this._saveSessionStateValue("musicVideosEnabled", value)
                    }
                    else
                        MS.Entertainment.Platform.Playback._musicVideosParentalBlockSet = value;
                    if (!this.currentMedia)
                        return;
                    value = MS.Entertainment.Platform.Playback._isMusicVideoModeInEffect;
                    if (oldValue === !value)
                        if ((value && this.currentMedia.hasMusicVideoId) || (!value && !this.currentMedia.hasMusicVideoId && !this.currentMedia.hasServiceId))
                            this._playlist.activate(this.currentOrdinal);
                        else if (value && !this.currentMedia.hasMusicVideoId && this.currentMedia.hasServiceId && !MS.Entertainment.Platform.PlaybackHelpers.isMusicVideo(this.currentMedia))
                            hasMusicVideoIdAsync(this.currentMedia).done(function hasMusicVideoIdAsync_completes(hasMusicVideoId) {
                                if (hasMusicVideoId)
                                    this._playlist.activate(this.currentOrdinal)
                            }.bind(this), function ignoreError() {
                                MS.Entertainment.Platform.Playback.fail("hasMusicVideoIdAsync should not return on async error path. True or False only. Ignored.")
                            })
                }, musicVideosEnabled: {
                    get: function PlaybackSession_musicVideosEnabled_get() {
                        return MS.Entertainment.Platform.Playback._musicVideosEnabled
                    }, set: function PlaybackSession_musicVideosEnabled_set(value) {
                            this._setMusicVideosEnabled(value, true)
                        }
                }, musicVideosParentalBlockSet: {
                    get: function PlaybackSession_musicVideosParentalBlockSet() {
                        return MS.Entertainment.Platform.Playback._musicVideosParentalBlockSet
                    }, set: function PlaybackSession_musicVideosParentalBlockSet(value) {
                            if (value !== MS.Entertainment.Platform.Playback._musicVideosParentalBlockSet)
                                this._setMusicVideosEnabled(value, false)
                        }
                }, isMusicVideoModeInEffect: {get: function PlaybackSession_isMusicVideoModeInEffect() {
                        return MS.Entertainment.Platform.Playback._isMusicVideoModeInEffect
                    }}, playbackControlDom: {
                    get: function PlaybackSession_playbackControlDom_get() {
                        return this._playbackControlContainer
                    }, set: function PlaybackSession_playbackControlDom_set() {
                            throw"Error! playbackControlDom property is not settable";
                        }
                }, setDataSource: function PlaybackSession_setDataSource(dataSource) {
                    var context = this;
                    if (this.smartDJSeed) {
                        this.canShuffle = false;
                        this.canRepeat = false;
                        this.shuffle = false;
                        this.repeat = false
                    }
                    else {
                        this.canShuffle = true;
                        this.canRepeat = true;
                        this.shuffle = this._persistedShuffle;
                        this.repeat = this._persistedRepeat
                    }
                    this._setDataSourcePromise = this._ensureSession().then(function _setDataSource2() {
                        context._iPlayback.reset(true);
                        return context._playlist.setDataSource(dataSource).then(function(mediaCollection) {
                                return mediaCollection
                            })
                    });
                    return this._setDataSourcePromise
                }, activate: function PlaybackSession_activate(playbackControlHost, index) {
                    if (!playbackControlHost)
                        return;
                    var context = this;
                    this._ensureSession().then(function _activate2() {
                        playbackControlHost.appendChild(context._playbackControlContainer);
                        context._playbackControlHost = playbackControlHost;
                        context.pendingOrdinal = -1;
                        switch (context._iPlayback.currentTransportState) {
                            case MSEPlatform.Playback.TransportState.paused:
                                context._iPlayback.targetTransportState = MSEPlatform.Playback.TransportState.playing;
                                break;
                            case MSEPlatform.Playback.TransportState.stopped:
                                context._playlist.activate(index);
                                break;
                            case MSEPlatform.Playback.TransportState.playing:
                                break;
                            default:
                                break
                        }
                    })
                }, deactivate: function PlaybackSession_deactivate(pausePlayback) {
                    if (!this._playbackControlHost)
                        return;
                    var pause = pausePlayback || false;
                    var context = this;
                    this._ensureSession().then(function _deactivate2() {
                        if (pause)
                            context._iPlayback.targetTransportState = MSEPlatform.Playback.TransportState.paused;
                        var control = context._playbackControlHost.children[0];
                        context._playbackControlHost = null;
                        context._parkedPlaybackHost.appendChild(control)
                    })
                }, clear: function PlaybackSession_clear() {
                    if (this._iPlayback) {
                        this._iPlayback.targetTransportState = MSEPlatform.Playback.TransportState.stopped;
                        this._iPlayback.reset(true)
                    }
                    if (this._playlist && this._playlist._dataSource)
                        this._playlist._dataSource.dispose();
                    this.setDataSource(null);
                    this.currentMedia = null;
                    this.pendingOrdinal = null;
                    MS.Entertainment.Platform.PlaybackHelpers.playActionInitiated = false
                }, relocate: function PlaybackSession_relocate(newHost) {
                    if (!newHost)
                        return;
                    if (typeof(newHost) === "object")
                        this._relocateLocal(newHost)
                }, playAt: function PlaybackSession_playAt(index, startTimeMsec, searchFor, maxSearchDistance) {
                    if (index < 0)
                        return;
                    var context = this;
                    this._ensureSession().then(function _playAt2() {
                        context._playlist.activate(index, startTimeMsec, searchFor, maxSearchDistance)
                    })
                }, isMediaCurrentlyLoaded: function PlaybackSession_isMediaCurrentlyLoaded(media) {
                    var type = typeof media;
                    var match = false;
                    if (this.currentMedia)
                        switch (type) {
                            case"number":
                                match = MS.Entertainment.Utilities.isValidLibraryId(media) && this.currentMedia.libraryId === media;
                                break;
                            case"string":
                                match = !MS.Entertainment.Utilities.isEmptyGuid(media) && (this.currentMedia.zuneId === media || this.currentMedia.canonicalId === media || this.currentMedia.serviceId === media);
                                break;
                            case"object":
                                match = media && this.currentMedia.isEqual(media);
                                break
                        }
                    return match
                }, getProtectionState: function getProtectionState() {
                    if (this._iPlayback && this._iPlayback._player && this._iPlayback._player._currentMedia)
                        return this._iPlayback._player._currentMedia.protectionState;
                    else
                        return null
                }, filterClosedCaptions: function PlaybackSession_filterClosedCaptions(media) {
                    var closedCaptions = [];
                    var mediaInstanceId = String.empty;
                    if (this.currentMedia && this._iPlayback && this._iPlayback._player && this._iPlayback._player._currentMedia && this._iPlayback._player._currentMedia.mediaInstanceId && !this.currentMedia.mediaInstanceId)
                        mediaInstanceId = this._iPlayback._player._currentMedia.mediaInstanceId;
                    var filterToRelevantClosedCaptionFiles = function filterToRelevantClosedCaptionFiles(closedCaptionFile) {
                            var ccFileMediaInstanceID = ("{" + closedCaptionFile.mediaInstanceId + "}").toUpperCase();
                            if (ccFileMediaInstanceID !== mediaInstanceId)
                                return false;
                            return true
                        };
                    if (media && media.closedCaptionFiles && media.closedCaptionFiles.length)
                        closedCaptions = media.closedCaptionFiles.filter(filterToRelevantClosedCaptionFiles);
                    return closedCaptions
                }, _relocateLocal: function PlaybackSession_relocateLocal(newHost) {
                    var context = this;
                    this._ensureSession().then(function _relocate2() {
                        newHost.appendChild(context._playbackControlContainer);
                        context._playbackControlHost = newHost
                    })
                }, _restoreSessionState: function PlaybackSession_restoreSessionState() {
                    var shuffle = false;
                    var repeat = false;
                    try {
                        shuffle = MS.Entertainment.Utilities.getUserRoamingSetting("shuffle") || false;
                        repeat = MS.Entertainment.Utilities.getUserRoamingSetting("repeat") || false;
                        MS.Entertainment.Platform.Playback._musicVideosEnabled = MS.Entertainment.Utilities.getUserRoamingSetting("musicVideosEnabled") || false
                    }
                    catch(e) {}
                    this._playlist.shuffle = shuffle;
                    this._playlist.repeat = repeat;
                    this._persistedShuffle = shuffle;
                    this._persistedRepeat = repeat
                }, _saveSessionStateValue: function PlaybackSession_saveSessionStateValue(key, value) {
                    MS.Entertainment.Utilities.setUserRoamingSetting(key, value)
                }, _ensureSession: function PlaybackSession_ensureSession() {
                    if (!this._playbackControl)
                        return WinJS.Promise.wrapError("PlaybackSession_ensureSession: Error! PlaybackControl not created.");
                    return this._sessionInitializedPromise
                }, _repositionOnError: function PlaybackSession__repositionOnError() {
                    if (this._playlist && this._playlist._dataSource && this._playlist._dataSource.size === 1)
                        return null;
                    if (this.error.isCritical) {
                        if (this.error.code === MS.Entertainment.Platform.Playback.Error.ZEST_E_MW_CONCURRENT_STREAM.code) {
                            var startOrdinal = this.error.errorOrdinal;
                            if (startOrdinal === +startOrdinal)
                                return startOrdinal;
                            else
                                return this.first
                        }
                        return null
                    }
                    else
                        return this.first;
                    return null
                }, _onPlaylistEnded: function PlaybackSession_onPlaylistEnded()
                {
                    this._iPlayback.waitForNoLossAssignmentsComplete().then(function assignsComplete() {
                        MSEPlatform.Playback.Etw.traceString("PlaybackSession:: _onPlaylistEnded");
                        if (MS.Entertainment.Utilities.isMusicApp && !this.isPlayToReceiverSessionRunning) {
                            var repositionOrdinal = this.first;
                            this.pendingOrdinal = repositionOrdinal;
                            if (this.error) {
                                var repositionOnErrorOrdinal = this._repositionOnError();
                                if (repositionOnErrorOrdinal === null)
                                    return;
                                else
                                    repositionOrdinal = repositionOnErrorOrdinal
                            }
                            WinJS.Promise.timeout(100).done(function repositionPlayback() {
                                this.autoPlay = false;
                                this.playAt(repositionOrdinal);
                                this.pendingOrdinal = repositionOrdinal
                            }.bind(this))
                        }
                    }.bind(this))
                }, _onPlaylistMediaChanged: function PlaybackSession_onPlaylistMediaChanged(e) {
                    var newMediaItem = (e && e.detail) ? e.detail.newValue : null;
                    if (!newMediaItem && !this._playlist.settingDataSource) {
                        this.currentMedia = null;
                        this._onPlaylistEnded();
                        return
                    }
                    this._updateCurrentMedia(newMediaItem);
                    this.canTransfer = false;
                    if (!newMediaItem)
                        return;
                    if (this._canTransferMediaItemPromise) {
                        this._canTransferMediaItemPromise.cancel();
                        this._canTransferMediaItemPromise = null
                    }
                    var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                    this._canTransferMediaItemPromise = sessionMgr.canTransferMediaItemAsync(newMediaItem, MSEPlatform.Playback.WellKnownPlaybackSessionId.nowPlaying, MSEPlatform.Playback.WellKnownPlaybackSessionId.remoteLRC);
                    var onPromiseComplete = function canTransferMediaItemAsync_complete(canTransfer) {
                            this.canTransfer = canTransfer;
                            this._canTransferMediaItemPromise = null
                        }.bind(this);
                    this._canTransferMediaItemPromise.then(onPromiseComplete, function canTransferMediaItemAsync_error(e) {
                        onPromiseComplete(false)
                    }.bind(this));
                    if (MS.Entertainment.Utilities.isMusicApp1)
                        if (!MS.Entertainment.UI.Shell.ImageLoader.isBlobUrl(newMediaItem.imageUri))
                            MS.Entertainment.UI.Shell.ImageLoader.getServiceImageUrl(newMediaItem, 100, 100).then(function gotAlbumArt(uri) {
                                MSEPlatform.Playback.Etw.traceString("PlaybackSession_onPlaylistMediaChanged: albumArtUri = " + uri);
                                var artUri = new Windows.Foundation.Uri(uri);
                                this.assignTrackNameAndAlbumArt(artUri)
                            }.bind(this), function noAlbumArt(error) {
                                this.assignTrackNameAndAlbumArt(null)
                            }.bind(this));
                        else if (MS.Entertainment.Utilities.isMusicApp1)
                            MSEPlatform.Playback.makeLocalImagePromise.done(function assignImageToMTC() {
                                var imagePath = "ms-appdata:///local/" + MS.Entertainment.Activation.localImageFileNameforMTC;
                                var artUri = new Windows.Foundation.Uri(imagePath);
                                this.assignTrackNameAndAlbumArt(artUri)
                            }.bind(this), function assignImageToMTC_error() {
                                this.assignTrackNameAndAlbumArt(null)
                            }.bind(this))
                }, assignTrackNameAndAlbumArt: (function PlaybackSession_assignTrackNameAndAlbumArt_closure() {
                    var mediaControlsAvailable = true;
                    return function assignTrackNameAndAlbumArt(uri) {
                            try {
                                if (mediaControlsAvailable && Windows.Media.MediaControl) {
                                    Windows.Media.MediaControl.albumArt = uri;
                                    if (this.currentMedia) {
                                        Windows.Media.MediaControl.trackName = this.currentMedia.name;
                                        MSEPlatform.Playback.Etw.traceString("PlaybackSession_assignTrackNameAndAlbumArt::Assigned trackname to MTC:" + this.currentMedia.name);
                                        if (this.currentMedia.artistName) {
                                            Windows.Media.MediaControl.artistName = this.currentMedia.artistName;
                                            MSEPlatform.Playback.Etw.traceString("PlaybackSession_assignTrackNameAndAlbumArt::Assigned artistname to MTC:" + this.currentMedia.artistName)
                                        }
                                        else
                                            Windows.Media.MediaControl.artistName = ""
                                    }
                                    else
                                        Windows.Media.MediaControl.trackName = "";
                                    MSEPlatform.Playback.Etw.traceString("PlaybackSession_assignTrackNameAndAlbumArt::Assigned album art uri to MTC")
                                }
                            }
                            catch(ex) {
                                mediaControlsAvailable = false
                            }
                        }
                })(), _onAudioAdChanged: function _onAudioAdChanged() {
                    this.canSeek = !this.isAudioAd;
                    this._updateCanSkipBackward();
                    this._updateCanSkipForward();
                    this._updateCurrentMediaForAudioAdChange()
                }, _updateCanSkipBackward: function _updateCanSkipBackward() {
                    var canSkip = !this._iPlayback.isAudioAd;
                    if (canSkip !== this.canSkipBackward)
                        this.canSkipBackward = canSkip
                }, _updateCanSkipForward: function _updateCanSkipForward() {
                    var canSkip = true;
                    if (MS.Entertainment.Utilities.isMusicApp)
                        canSkip = !this._iPlayback.isAudioAd && this._playlist.canSkipForward;
                    if (canSkip !== this.canSkipForward)
                        this.canSkipForward = canSkip
                }, _onVisibilityChanged: function _onVisibilityChanged() {
                    if (document.hidden)
                        this.disableTimeUpdate();
                    else {
                        this.enableTimeUpdate();
                        this.forceTimeUpdate()
                    }
                }, _updateCurrentMediaForAudioAdChange: function _updateCurrentMediaForAudioAdChange(newMediaItem) {
                    if (MS.Entertainment.Utilities.isApp2)
                        return;
                    if (this._iPlayback._player && this._iPlayback._player._currentMedia && this._iPlayback._player._currentMedia._mediaItem)
                        this.currentMedia = this._iPlayback._player._currentMedia._mediaItem.data;
                    else if (!this._iPlayback._player || !this._iPlayback._player._currentMedia)
                        this.currentMedia = null
                }, _updateCurrentMedia: function _updateCurrentMedia(newMediaItem) {
                    if (MS.Entertainment.Utilities.isApp2 || newMediaItem === null)
                        this.currentMedia = newMediaItem;
                    else
                        this._updateCurrentMediaForAudioAdChange()
                }, _onPlayerStateChanged: function _onPlayerStateChanged(){}, _onRemoteSessionRunningChanged: function _onRemoteSessionRunningChanged(e) {
                    if (e) {
                        this._closedCaptionSavedState = this.closedCaptionsOn;
                        this.closedCaptionsOn = false
                    }
                    else if (this._closedCaptionSavedState !== null)
                        this.closedCaptionsOn = this._closedCaptionSavedState
                }, companionExperience: {get: function PlaybackSession_companionExperience_get() {
                        return this._companionExperience
                    }}, skipFwd: function PlaybackSession_skipFwd() {
                    MS.Entertainment.Platform.Playback.assert(this.canSkipForward, "skipFwd() should not be called if canSkipForward is false");
                    this._playlist.skipFwd()
                }, skipBack: function PlaybackSession_skipBack(forcePreviousTrack) {
                    MS.Entertainment.Platform.Playback.assert(this.canSkipBackward, "skipBack() should not be called if canSkipBackward is false");
                    var skipBackThreshold = 5000;
                    if ((forcePreviousTrack || this.currentPosition <= skipBackThreshold) && this._playlist.canSkipBackward)
                        this._playlist.skipBack();
                    else
                        this._iPlayback.seekToPosition(0)
                }, _sessionId: null, _sessionInitializedPromise: null, _setDataSourcePromise: WinJS.Promise.as(), _companionExperience: null, _canTransferMediaItemPromise: null, _persistedShuffle: false, _persistedRepeat: false
        };
    WinJS.Namespace.defineWithParent(MSEPlatform, "Playback", {PlaybackSessionBase: WinJS.Class.mix(function mixConstructor(){}, MS.Entertainment.UI.Framework.EventMixin, MS.Entertainment.UI.Framework.UpdatePropertyMixin, playbackSessionMixin, playbackControlMixin, playlistMixin, ccRendererMixin, WinJS.Binding.mixin)});
    WinJS.Namespace.defineWithParent(MSEPlatform, "Playback", {
        PlaybackSession: MS.Entertainment.UI.Framework.derive("MS.Entertainment.Platform.Playback.PlaybackSessionBase", function playbackSession_constructor(id) {
            var completePromise;
            this._initObservable({});
            this._sessionInitializedPromise = new WinJS.Promise(function(c, e, p) {
                completePromise = c
            });
            this._sessionId = (!id ? MSEPlatform.SessionManager._createSessionId() : id);
            this._playbackControlCreateInstance();
            this._signOutHandlerBound = this._signOutHandler.bind(this);
            var onPlaybackSessionInitialized = function() {
                    var isInitialized;
                    if (this._playbackControlEvents) {
                        this._playbackControlEvents.cancel();
                        this._playbackControlEvents = null;
                        isInitialized = true
                    }
                    if (isInitialized) {
                        this._playlistCreateInstance();
                        this._iPlayback = this._playbackControl.getPlaybackInterface();
                        this._playlist.setPlaybackControl(this._iPlayback);
                        if (MS.Entertainment.Utilities.isVideoApp || MS.Entertainment.Utilities.isApp2)
                            this._ccRendererCreateInstance(this._iPlayback, this._playbackControl.getClosedCaptionsContainer());
                        MS.Entertainment.ProxyHelpers.proxyProperties(this, this._iPlayback, ["targetTransportState", "autoPlay", "muted"]);
                        MS.Entertainment.ProxyHelpers.proxyObservables(this, this._iPlayback, ["playerState", "error", "currentTransportState", "isAudioAd", "isPreview", "duration", "currentPosition", "videoWidth", "videoHeight", "errorDescriptor", "playbackRate", "minPlaybackRate", "maxPlaybackRate", "isRemoteSessionRunning", "mediaElement"]);
                        MS.Entertainment.ProxyHelpers.delegateFunctions(this, this._iPlayback, ["seekToPosition", "fastFwd", "fastReverse", "slowFwd", "slowReverse", "enableTimeUpdate", "disableTimeUpdate", "forceTimeUpdate", "isRemoteSession", "notifyNetworkConnectionChanged"]);
                        MS.Entertainment.ProxyHelpers.proxyProperties(this, this._playlist, ["first"]);
                        MS.Entertainment.ProxyHelpers.proxyObservables(this, this._playlist, ["shuffle", "repeat", "mediaCollection", "currentOrdinal", "currentNowPlayingPlaylistLibraryId", "smartDJSeed"]);
                        MS.Entertainment.ProxyHelpers.delegateFunctions(this, this._playlist, ["before", "after", "savePlaylist", "insertAtEnd"]);
                        if (MS.Entertainment.Utilities.isVideoApp || MS.Entertainment.Utilities.isApp2) {
                            MS.Entertainment.ProxyHelpers.proxyProperties(this, this._ccRenderer, ["closedCaptionsOn"]);
                            MS.Entertainment.ProxyHelpers.proxyObservables(this, this._ccRenderer, ["ccLcid"])
                        }
                        try {
                            this._playLog = new MS.Entertainment.Platform.Playback.PlayLog(this._iPlayback)
                        }
                        catch(e) {
                            MS.Entertainment.Platform.Playback.fail("PlayLog should always be loaded before PlaybackSession. " + (e && e.toString()))
                        }
                        this._concurrentStreamingRestriction = new MS.Entertainment.Platform.Playback.ConcurrentStreamingRestriction(this._iPlayback);
                        this._playlist.addEventListener("currentMediaChanged", this._onPlaylistMediaChanged.bind(this));
                        completePromise();
                        MS.Entertainment.Platform.Playback.Etw.traceSessionInitialized();
                        this._restoreSessionState();
                        this._iPlayback.bind("isAudioAd", this._onAudioAdChanged.bind(this));
                        this._iPlayback.bind("playerState", this._onPlayerStateChanged.bind(this));
                        this._iPlayback.bind("isRemoteSessionRunning", this._onRemoteSessionRunningChanged.bind(this));
                        this._playlist.bind("canSkipBackward", this._updateCanSkipBackward.bind(this));
                        this._playlist.bind("canSkipForward", this._updateCanSkipForward.bind(this));
                        this._visibilityChanged = this._onVisibilityChanged.bind(this);
                        document.addEventListener("visibilitychange", this._visibilityChanged, false);
                        if (this._constructionTime)
                            MSEPlatform.Playback.Etw.traceString("session is initialized " + (new Date - this._constructionTime) + "ms after construction");
                        else
                            MSEPlatform.Playback.Etw.traceString("session is initialized during construction")
                    }
                }.bind(this);
            if (this._playbackControl.controlInitialized)
                onPlaybackSessionInitialized();
            else
                this._playbackControlEvents = MS.Entertainment.Utilities.addEventHandlers(this._playbackControl, {onControlInitialized: onPlaybackSessionInitialized});
            this._constructionTime = new Date;
            MSEPlatform.Playback.Etw.traceString("session constructed")
        }, {
            canShuffle: MS.Entertainment.UI.Framework.observableProperty("canShuffle", false), canRepeat: MS.Entertainment.UI.Framework.observableProperty("canRepeat", false), canTransfer: MS.Entertainment.UI.Framework.observableProperty("canTransfer", false), canControlMedia: MS.Entertainment.UI.Framework.observableProperty("canControlMedia", true), canFastForward: MS.Entertainment.UI.Framework.observableProperty("canFastForward", false), canSlowForward: MS.Entertainment.UI.Framework.observableProperty("canSlowForward", false), canFastReverse: MS.Entertainment.UI.Framework.observableProperty("canFastReverse", false), canSlowReverse: MS.Entertainment.UI.Framework.observableProperty("canSlowReverse", false), canSeek: MS.Entertainment.UI.Framework.observableProperty("canSeek", true), currentMedia: MS.Entertainment.UI.Framework.observableProperty("currentMedia", null), canSkipBackward: MS.Entertainment.UI.Framework.observableProperty("canSkipBackward", true), canSkipForward: MS.Entertainment.UI.Framework.observableProperty("canSkipForward", true), lastPlayedMedia: MS.Entertainment.UI.Framework.observableProperty("lastPlayedMedia", null), lastPlayedDuration: MS.Entertainment.UI.Framework.observableProperty("lastPlayedDuration", 0), lastPlayedPosition: MS.Entertainment.UI.Framework.observableProperty("lastPlayedPosition", 0), isPlayToReceiverSessionRunning: MS.Entertainment.UI.Framework.observableProperty("isPlayToReceiverSessionRunning", false), _visibilityChanged: null, _closedCaptionSavedState: null, _signOutHandlerBound: null, _signInBound: false, _signOutHandler: function _signOutHandler() {
                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                    if (!signIn.isSignedIn)
                        this.setLastPlayedMedia(null)
                }, currentMediaKey: {get: function get_CurrentMediaKey() {
                        return this._playlist ? this._playlist._currentMediaKey : null
                    }}, setLastPlayedMedia: function setLastPlayedMedia(media) {
                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                    if (media && signIn.isSignedIn) {
                        if (!this._signInBound) {
                            signIn.bind("isSignedIn", this._signOutHandlerBound);
                            this._signInBound = true
                        }
                    }
                    else if (this._signInBound) {
                        signIn.unbind("isSignedIn", this._signOutHandlerBound);
                        this._signInBound = false
                    }
                    MS.Entertainment.Platform.Playback.assert(media === null || media === this.currentMedia, "setLastPlayedMedia called with something other than currentMedia");
                    this.lastPlayedMedia = media;
                    this.lastPlayedDuration = (media === null) ? 0 : this.duration;
                    this.lastPlayedPosition = (media === null) ? 0 : this.currentPosition
                }, isVideoAdSupported: {get: function() {
                        return this._playbackControl.isVideoAdSupported
                    }}
        }, {isPlaybackSession: function isPlaybackSession(object) {
                return MS.Entertainment.Platform.Playback.PlaybackSession.prototype.isPrototypeOf(object)
            }}), makeLocalImageFileFromThumbnailPromise: (function PlaybackSession_makeLocalImageFileFromThumbnailPromise_closure() {
                var makeLocalImageFilePromise;
                return function makeLocalImageFileFromThumbnailPromise(thumbnail) {
                        if (makeLocalImageFilePromise) {
                            makeLocalImageFilePromise.cancel();
                            makeLocalImageFilePromise = null
                        }
                        makeLocalImageFilePromise = new WinJS.Promise(function makeLocalImageFile(completed, error) {
                            var timer = function attemptFileCreation(count) {
                                    Windows.Storage.ApplicationData.current.localFolder.createFileAsync(MS.Entertainment.Activation.localImageFileNameforMTC, Windows.Storage.CreationCollisionOption.replaceExisting).done(completed, function createFileAsync_Error(createFileError) {
                                        if (count <= 6)
                                            WinJS.Promise.timeout(500).then(function retryOnError() {
                                                timer(count + 1)
                                            });
                                        else
                                            error(createFileError)
                                    })
                                };
                            timer(0)
                        });
                        var size;
                        var reader;
                        var localFile;
                        var inputStream;
                        makeLocalImageFilePromise = makeLocalImageFilePromise.then(function(file) {
                            inputStream = thumbnail.getInputStreamAt(0);
                            reader = new Windows.Storage.Streams.DataReader(inputStream);
                            size = thumbnail.size;
                            localFile = file;
                            return reader.loadAsync(size)
                        }).then(function loadAsync_success() {
                            var buffer = new Array(size);
                            reader.readBytes(buffer);
                            return Windows.Storage.FileIO.writeBytesAsync(localFile, buffer)
                        }, function loadAsync_error(error) {
                            MS.Entertainment.Platform.Playback.Etw.traceString("imageLoader::_loadBytesAsync_Error")
                        }).then(function() {
                            if (reader)
                                reader.close();
                            if (inputStream)
                                inputStream.close()
                        }).then(function() {
                            makeLocalImageFilePromise = null
                        });
                        return makeLocalImageFilePromise
                    }
            })(), makeLocalImagePromise: null
    })
})()
