﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    WinJS.Namespace.define("MS.Entertainment.UI.Controls", {VolumeService: MS.Entertainment.UI.Framework.derive("MS.Entertainment.UI.Framework.ObservableBase", function VolumeServiceConstructor() {
            if (MS.Entertainment.Utilities.isMusicApp) {
                this._configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                this.updateAndNotify("volume", (this._configurationManager.playback.musicVolumeValue) / 100);
                this.updateAndNotify("mute", this._configurationManager.playback.musicMuteState)
            }
            else
                try {
                    this._volumeManager = new Microsoft.Entertainment.AppVolumeManager.AppVolume;
                    this._volumeManager.addEventListener("systemvolumestatechanged", this._systemVolumeStateChanged.bind(this));
                    this._volumeManager.addEventListener("audioendpointchanged", this._audioEndpointChanged.bind(this));
                    this._initializeStates()
                }
                catch(e) {
                    this.updateAndNotify("isAudioEndpointAvailable", false)
                }
        }, {
            _systemVolumeStateChangedBindingMethod: null, _volumeManager: null, _volume: 0.80, _mute: false, _isAudioEndpointAvailable: true, _configurationManager: null, volume: {
                    get: function getVolume() {
                        return this._volume
                    }, set: function setVolume(newValue) {
                            if (MS.Entertainment.Utilities.isMusicApp)
                                this._configurationManager.playback.musicVolumeValue = newValue * 100;
                            else
                                try {
                                    if (this._volumeManager)
                                        this._volumeManager.setVolumeLevelAsync(newValue)
                                }
                                catch(e) {
                                    MS.Entertainment.UI.Controls.fail(e && e.toString())
                                }
                            if (this._volume !== newValue)
                                this.updateAndNotify("volume", newValue)
                        }
                }, mute: {
                    get: function getMute() {
                        return this._mute
                    }, set: function setMute(newValue) {
                            if (MS.Entertainment.Utilities.isMusicApp)
                                this._configurationManager.playback.musicMuteState = newValue;
                            else
                                try {
                                    if (this._volumeManager)
                                        this._volumeManager.setMutedAsync(newValue)
                                }
                                catch(e) {
                                    MS.Entertainment.UI.Controls.fail(e && e.toString())
                                }
                            if (this._mute !== newValue)
                                this.updateAndNotify("mute", newValue)
                        }
                }, isAudioEndpointAvailable: {
                    get: function getIsAudioEndpointAvailable() {
                        return this._isAudioEndpointAvailable
                    }, set: function setIsAudioEndpointAvailable(newState) {
                            this.updateAndNotify("isAudioEndpointAvailable", newValue)
                        }
                }, _systemVolumeStateChanged: function systemVolumeStateChanged() {
                    this._updateSystemVolumeStates()
                }, _initializeStates: function initializeStates() {
                    this._updateSystemVolumeStates()
                }, _updateSystemVolumeStates: function updateSystemVolumeStates() {
                    try {
                        if (this._volumeManager) {
                            this._volumeManager.getVolumeLevelAsync().done(function getVolumeLevelAsync_complete(volume) {
                                if (this._volume !== volume.value)
                                    this.updateAndNotify("volume", volume.value)
                            }.bind(this), function getVolumeLevelAsync_error(error) {
                                MS.Entertainment.UI.Controls.fail("Failed to get system volume level with error:" + (error && error.toString()))
                            });
                            this._volumeManager.getMutedAsync().done(function getMutedAsync_complete(muteState) {
                                if (this._mute !== muteState.value)
                                    this.updateAndNotify("mute", muteState.value)
                            }.bind(this), function getMutedAsync_error(error) {
                                MS.Entertainment.UI.Controls.fail("Failed to get system mute level with error:" + (error && error.toString()))
                            })
                        }
                    }
                    catch(e) {
                        MS.Entertainment.UI.Controls.fail(e && e.toString())
                    }
                }, _audioEndpointChanged: function _audioEndpointChanged(newState) {
                    if (newState.detail.length > 0)
                        if (newState.detail[0] === 1) {
                            this.updateAndNotify("isAudioEndpointAvailable", true);
                            this._updateSystemVolumeStates()
                        }
                        else
                            this.updateAndNotify("isAudioEndpointAvailable", false)
                }
        }, {factory: function factory() {
                return new MS.Entertainment.UI.Controls.VolumeService
            }})});
    MS.Entertainment.ServiceLocator.register(MS.Entertainment.Services.volumeService, MS.Entertainment.UI.Controls.VolumeService.factory)
})()
