﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Platform");
    var MSEPlatform = WinJS.Namespace.define("MS.Entertainment.Platform", null);
    WinJS.Namespace.defineWithParent(MSEPlatform, "LivingRoomCompanion", {
        BrowserUrlState: {
            updated: "updated", updating: "updating"
        }, BrowserInputMode: {
                browser: "browser", dpad: "dpad"
            }, SessionState: {
                unInitialize: "unInitialize", disconnecting: "disconnecting", disconnected: "disconnected", connecting: "connecting", connected: "connected", transferringToRemote: "transferringToRemote", transferringToLocal: "transferringToLocal", error: "error", connectedTemporaryError: "connectedTemporaryError"
            }, AppLifetime: {
                none: "none", suspending: "suspending", resuming: "resuming", running: "running", shutdown: "shutdown"
            }, WellKnownTitleId: {
                ze: 1481115739, hu: 810026961, nx: 960956369, dd: 4294838225, mc: 1480918995, bb: 1481115776
            }, isWellKnownTitle: function isWellKnownTitle(titleId, wellKnownTitleId) {
                if (!titleId || isNaN(titleId))
                    return false;
                return (parseInt(titleId) === wellKnownTitleId)
            }, LaunchFirstAction: {
                playNow: "PlayNow", addNow: "AddNow"
            }
    });
    WinJS.Namespace.defineWithParent(MS.Entertainment, "Platform", {SessionManager: MS.Entertainment.UI.Framework.define(function SessionManager_constructor() {
            this._createMediaControlMethods();
            this._bindMediaControls();
            this._bindSystemMediaTransportControls()
        }, {
            nowPlayingSession: {get: function SessionManager_nowPlayingSession_get() {
                    return this._nowPlayingSession
                }}, primarySession: {get: function SessionManager_primarySession_get() {
                        return this._primarySession
                    }}, setPrimarySession: function SessionManager_setPrimarySession(sessionId) {
                    if (!this.sessions || !sessionId)
                        return null;
                    if (this._stopPromise && sessionId === MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying) {
                        this._stopPromise.cancel();
                        this._stopPromise = null
                    }
                    if (this._primarySession && this._primarySession.sessionId === sessionId)
                        return this._primarySession;
                    if (this._primarySession)
                        if (Windows && Windows.Media && Windows.Media.MediaControl) {
                            this._primarySession.unbind("canSkipForward", this._bindMediaControlsNext);
                            this._primarySession.unbind("canSkipBackward", this._bindMediaControlsPrevious)
                        }
                    var activeSession = this.getSession(sessionId);
                    if (activeSession) {
                        this._primarySession = activeSession;
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).primarySessionId = this._primarySession.sessionId;
                        if (Windows && Windows.Media && Windows.Media.MediaControl) {
                            activeSession.bind("canSkipForward", this._bindMediaControlsNext);
                            activeSession.bind("canSkipBackward", this._bindMediaControlsPrevious)
                        }
                    }
                    return activeSession
                }, _stopPromise: null, createSession: function SessionManager_createSession() {
                    this.sessions = this.sessions || {};
                    var session = new MSEPlatform.Playback.PlaybackSession;
                    this.sessions[session.sessionId] = session;
                    return session
                }, getSession: function SessionManager_getSession(sessionId) {
                    var session = null;
                    if (sessionId in this.sessions)
                        session = this.sessions[sessionId];
                    return session
                }, activateSession: function SessionManager_activateSession(controlHost, sessionId) {
                    var session = (!sessionId ? this._nowPlayingSession : this.sessions[sessionId]);
                    if (session)
                        WinJS.Promise.join([session._setDataSourcePromise]).then(function() {
                            session.activate(controlHost)
                        })
                }, deactivateSession: function SessionManager_deactivateSession(pausePlayback, sessionId) {
                    var session = (!sessionId ? this._nowPlayingSession : this.sessions[sessionId]);
                    if (session)
                        session.deactivate(pausePlayback)
                }, relocateSession: function SessionManager_relocateSession(newHost, sessionId) {
                    var session = (!sessionId ? this._nowPlayingSession : this.sessions[sessionId]);
                    if (session)
                        session.relocate(newHost)
                }, canTransferMediaItemAsync: function SessionManager_canTransferMediaItem(mediaItem, fromSessionId, toSessionId, currentTitleId) {
                    if (fromSessionId === toSessionId)
                        return WinJS.Promise.wrap(false);
                    if (toSessionId === MSEPlatform.Playback.WellKnownPlaybackSessionId.nowPlaying)
                        return WinJS.Promise.wrap(false);
                    else if (toSessionId === MSEPlatform.Playback.WellKnownPlaybackSessionId.remoteLRC)
                        return this._canTransferToRemoteAsync(mediaItem);
                    return WinJS.Promise.wrap(false)
                }, displayRequestActive: function SessionManager_displayRequestActive() {
                    if (this._displayRequest) {
                        if (this._isDisplayRequestActive)
                            return;
                        try {
                            this._displayRequest.requestActive();
                            this._isDisplayRequestActive = true
                        }
                        catch(e) {}
                    }
                }, displayRequestRelease: function SessionManager_displayRequestRelease() {
                    if (this._displayRequest) {
                        if (!this._isDisplayRequestActive)
                            return;
                        try {
                            this._displayRequest.requestRelease();
                            this._isDisplayRequestActive = false
                        }
                        catch(e) {}
                    }
                }, clearCachedPBMState: function SessionManager_clearCachedPBMState() {
                    if (this._transportStateBeforePaused !== MS.Entertainment.Platform.Playback.TransportState.unInitialize) {
                        MSEPlatform.Playback.Etw.traceString("Clearing cached state '" + this._transportStateBeforePaused + "' from when we paused due to PBM event");
                        this._transportStateBeforePaused = MS.Entertainment.Platform.Playback.TransportState.unInitialize
                    }
                }, _createMediaControlMethods: function _createMediaControlMethods() {
                    var mediaControls = null;
                    var systemMediaTransportControls = null;
                    if (!MS.Entertainment.Utilities.isApp2)
                        mediaControls = WinJS.Utilities.getMember("Media.MediaControl", Windows);
                    if (WinJS.Utilities.getMember("Media.SystemMediaTransportControls", Windows) && MS.Entertainment.Utilities.isMusicApp2)
                        systemMediaTransportControls = Windows.Media.SystemMediaTransportControls.getForCurrentView();
                    this._bindMediaControlsNext = MS.Entertainment.Utilities.bindAndMarkSupportedForProcessing(function SessionManager_bindMediaControlsNext(canSkipForward) {
                        if (mediaControls)
                            if (canSkipForward && !this._nextTrackPressedListener)
                                try {
                                    this._nextTrackPressedListener = MS.Entertainment.UI.Framework.addEventHandlers(mediaControls, {nexttrackpressed: function nextTrackPressed() {
                                            this._mediaControlNext()
                                        }.bind(this)}, false)
                                }
                                catch(ex) {
                                    MSEPlatform.Playback.Etw.traceString("sessionManager::_createMediaControlMethods(): Non-fatal exception caught in addEventListener(nexttrackpressed): " + ex)
                                }
                            else if (!canSkipForward && this._nextTrackPressedListener) {
                                this._nextTrackPressedListener.cancel();
                                this._nextTrackPressedListener = null
                            }
                        if (systemMediaTransportControls) {
                            systemMediaTransportControls.isNextEnabled = canSkipForward;
                            systemMediaTransportControls.displayUpdater.update()
                        }
                    }, this);
                    this._bindMediaControlsPrevious = MS.Entertainment.Utilities.bindAndMarkSupportedForProcessing(function SessionManager_bindMediaControlsPrevious(canSkipBack) {
                        if (mediaControls)
                            if (canSkipBack && !this._previousTrackPressedListener)
                                try {
                                    this._previousTrackPressedListener = MS.Entertainment.UI.Framework.addEventHandlers(mediaControls, {previoustrackpressed: function previousTrackPressed() {
                                            this._mediaControlPrevious()
                                        }.bind(this)}, false)
                                }
                                catch(ex) {
                                    MSEPlatform.Playback.Etw.traceString("sessionManager::_createMediaControlMethods(): Non-fatal exception caught in addEventListener(previoustrackpressed): " + ex)
                                }
                            else if (!canSkipBack && this._previousTrackPressedListener) {
                                this._previousTrackPressedListener.cancel();
                                this._previousTrackPressedListener = null
                            }
                        if (systemMediaTransportControls) {
                            systemMediaTransportControls.isPreviousEnabled = canSkipBack;
                            systemMediaTransportControls.displayUpdater.update()
                        }
                    }, this);
                    this._mediaControlNext = MS.Entertainment.Utilities.bindAndMarkSupportedForProcessing(function SessionManager_mediaControlNext(source) {
                        source = source ? source : "mediaKey";
                        if (this.primarySession)
                            this.primarySession._ensureSession().then(function sessionMgr_doMediaControlNext() {
                                if (MS.Entertainment.Platform.PlaybackHelpers.isMusicTrackOrMusicVideo(this.primarySession.currentMedia))
                                    this.primarySession.skipFwd();
                                else {
                                    var positionMs = this.primarySession.forceTimeUpdate();
                                    this.primarySession.seekToPosition(positionMs + 29000)
                                }
                            }.bind(this));
                        MS.Entertainment.Utilities.Telemetry.logNextClicked(source)
                    }, this);
                    this._mediaControlPrevious = MS.Entertainment.Utilities.bindAndMarkSupportedForProcessing(function SessionManager_mediaControlPrevious(source, forcePreviousTrack) {
                        source = source ? source : "mediaKey";
                        if (this.primarySession)
                            this.primarySession._ensureSession().then(function sessionMgr_doMediaControlPrevious() {
                                if (MS.Entertainment.Platform.PlaybackHelpers.isMusicTrackOrMusicVideo(this.primarySession.currentMedia))
                                    this.primarySession.skipBack(forcePreviousTrack);
                                else {
                                    var positionMs = this.primarySession.forceTimeUpdate();
                                    this.primarySession.seekToPosition(positionMs - 15000)
                                }
                            }.bind(this));
                        MS.Entertainment.Utilities.Telemetry.logPreviousClicked(source)
                    }, this);
                    this._mediaControlPlayPauseToggle = MS.Entertainment.Utilities.bindAndMarkSupportedForProcessing(function SessionManager_mediaControlPlayPauseToggle(source) {
                        source = source ? source : "mediaKey";
                        if (this.primarySession) {
                            this.primarySession._ensureSession().then(function sessionMgr_doMediaControlPlayPauseToggle() {
                                if (this.primarySession.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.playing) {
                                    this.primarySession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.paused;
                                    MS.Entertainment.Utilities.Telemetry.logPauseClicked(source)
                                }
                                else if (this.primarySession.playerState === MS.Entertainment.Platform.Playback.PlayerState.ready)
                                    this.primarySession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.playing;
                                else
                                    this.primarySession.playAt(0)
                            }.bind(this));
                            MS.Entertainment.Utilities.Telemetry.logPlayClicked(source)
                        }
                    }, this)
                }, _bindMediaControls: function _bindMediaControls() {
                    if (!Windows || !Windows.Media || !Windows.Media.MediaControl || MS.Entertainment.Utilities.isApp2)
                        return;
                    var mediaControls = Windows.Media.MediaControl;
                    try {
                        mediaControls.addEventListener("playpausetogglepressed", function mediaControlPlayPause() {
                            this._mediaControlPlayPauseToggle()
                        }.bind(this), false);
                        mediaControls.addEventListener("playpressed", function mediaControlPlay() {
                            this.primarySession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.playing;
                            MS.Entertainment.Utilities.Telemetry.logPlayClicked("mediaKey")
                        }.bind(this), false);
                        mediaControls.addEventListener("stoppressed", function mediaControlStop() {
                            this.primarySession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.stopped;
                            MS.Entertainment.Utilities.Telemetry.logStopClicked("mediaKey")
                        }.bind(this), false);
                        mediaControls.addEventListener("pausepressed", function mediaControlPause() {
                            this.primarySession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.paused;
                            MS.Entertainment.Utilities.Telemetry.logPauseClicked("mediaKey")
                        }.bind(this), false);
                        mediaControls.addEventListener("soundlevelchanged", this._onSoundLevelChanged.bind(this), false)
                    }
                    catch(ex) {
                        MSEPlatform.Playback.Etw.traceString("Non-fatal exception caught in mediaControls.addEventListener: " + ex)
                    }
                }, _bindSystemMediaTransportControls: function _bindSystemMediaTransportControls() {
                    if (!WinJS.Utilities.getMember("Media.SystemMediaTransportControls", Windows) || !MS.Entertainment.Utilities.isMusicApp2)
                        return;
                    var systemMediaTransportControls = Windows.Media.SystemMediaTransportControls.getForCurrentView();
                    var systemMediaButtons = Windows.Media.SystemMediaTransportControlsButton;
                    systemMediaTransportControls.addEventListener("buttonpressed", function buttonPressed(evt) {
                        evt.detail.forEach(function eventIterator(event) {
                            switch (event.button) {
                                case systemMediaButtons.stop:
                                    this.primarySession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.stopped;
                                    MS.Entertainment.Utilities.Telemetry.logStopClicked("voiceCommand");
                                    break;
                                case systemMediaButtons.play:
                                    this.primarySession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.playing;
                                    MS.Entertainment.Utilities.Telemetry.logPlayClicked("voiceCommand");
                                    break;
                                case systemMediaButtons.pause:
                                    this.primarySession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.paused;
                                    MS.Entertainment.Utilities.Telemetry.logPauseClicked("voiceCommand");
                                    break;
                                case systemMediaButtons.next:
                                    this._mediaControlNext("voiceCommand");
                                    break;
                                case systemMediaButtons.previous:
                                    this._mediaControlPrevious("voiceCommand", true);
                                    break
                            }
                        }.bind(this))
                    }.bind(this));
                    systemMediaTransportControls.addEventListener("propertychanged", function propertyChanged(event) {
                        var updater = systemMediaTransportControls.displayUpdater;
                        switch (event.property) {
                            case Windows.Media.SystemMediaTransportControlsProperty.playbackPosition:
                                if (updater.type === Windows.Media.MediaPlaybackType.music)
                                    if (isFinite(updater.musicProperties.playbackPosition) && updater.musicProperties.playbackPosition >= 0)
                                        this.primarySession.seekToPosition(updater.musicProperties.playbackPosition);
                                break;
                            default:
                                break
                        }
                    }.bind(this))
                }, _onSoundLevelChanged: function SessionManager_onSoundLevelChanged() {
                    if (this.primarySession && this.primarySession.isRemoteSession && this.primarySession.isRemoteSession())
                        return;
                    try {
                        var soundLevel = Windows.Media.MediaControl.soundLevel;
                        switch (soundLevel) {
                            case Windows.Media.SoundLevel.full:
                                this._onSoundLevelFull();
                                break;
                            case Windows.Media.SoundLevel.low:
                                this._onSoundLevelLow();
                                break;
                            case Windows.Media.SoundLevel.muted:
                                this._onSoundLevelMuted();
                                break;
                            default:
                                MS.Entertainment.Platform.assert(false, "Unexpected value for soundLevel: " + soundLevel);
                                break
                        }
                    }
                    catch(ex) {}
                }, _onSoundLevelFull: function SessionManager_onSoundLevelFull() {
                    var currentTransportState = MS.Entertainment.Platform.Playback.TransportState.unInitialize;
                    MSEPlatform.Playback.Etw.traceString("+PBM Sound Full");
                    if (this.primarySession) {
                        currentTransportState = this.primarySession.currentTransportState;
                        if (this._transportStateBeforePaused && this._transportStateBeforePaused !== MS.Entertainment.Platform.Playback.TransportState.unInitialize) {
                            if (this._transportStateBeforePaused !== MS.Entertainment.Platform.Playback.TransportState.starting) {
                                MSEPlatform.Playback.Etw.traceString("PBM Sound full. Target TransportState to stored value:", this._transportStateBeforePaused);
                                this.primarySession.targetTransportState = this._transportStateBeforePaused
                            }
                            else {
                                MSEPlatform.Playback.Etw.traceString("PBM Sound full. Target TransportState to playing");
                                this.primarySession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.playing
                            }
                            this._transportStateBeforePaused = MS.Entertainment.Platform.Playback.TransportState.unInitialize
                        }
                    }
                    MSEPlatform.Playback.Etw.tracePlaybackPBMSoundLevelChanged("full", currentTransportState)
                }, _onSoundLevelLow: function SessionManager_onSoundLevelLow() {
                    var currentTransportState = MS.Entertainment.Platform.Playback.TransportState.unInitialize;
                    MSEPlatform.Playback.Etw.traceString("+PBM Sound Low");
                    if (this.primarySession) {
                        currentTransportState = this.primarySession.currentTransportState;
                        if (!MS.Entertainment.Utilities.isMusicApp && currentTransportState !== MS.Entertainment.Platform.Playback.TransportState.stopped) {
                            MSEPlatform.Playback.Etw.traceString("PBM Sound Low pausing from " + this.primarySession.currentTransportState);
                            this._transportStateBeforePaused = this.primarySession.currentTransportState;
                            this.primarySession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.paused
                        }
                    }
                    MSEPlatform.Playback.Etw.tracePlaybackPBMSoundLevelChanged("low", currentTransportState)
                }, _onSoundLevelMuted: function SessionManager_onSoundLevelMuted() {
                    MSEPlatform.Playback.Etw.traceString("+PBM Sound Muted");
                    if (this.primarySession && this.primarySession.currentTransportState !== MS.Entertainment.Platform.Playback.TransportState.stopped) {
                        this._transportStateBeforePaused = this.primarySession.currentTransportState;
                        this.primarySession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.paused
                    }
                    MSEPlatform.Playback.Etw.tracePlaybackPBMSoundLevelChanged("muted", this._transportStateBeforePaused)
                }, _canTransferToRemoteAsync: function SessionManager_canTransferToRemote(mediaItem) {
                    var mediaType = mediaItem.mediaType;
                    var smid = mediaItem.serviceId;
                    switch (mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.game:
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.video:
                            if (mediaItem.playFromXbox)
                                return WinJS.Promise.wrap(true);
                            else
                                return MS.Entertainment.ViewModels.SmartBuyStateEngine.queryMediaStateAsync(mediaItem).then(function queryMediaStateAsync_complete(stateInfo) {
                                        return WinJS.Promise.wrap(stateInfo.marketplace.canStream && !stateInfo.marketplace.exclusivelyRented)
                                    }.bind(this));
                            break;
                        default:
                            break
                    }
                    return WinJS.Promise.wrap(false)
                }, _rootLicenseRefresh: (function SessionManager_rootLicenseRefresh_closure() {
                    var configurationManager;
                    return function SessionManager_rootLicenseRefresh(signIn) {
                            if (!configurationManager)
                                configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                            var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                            var tunerActivated = (signedInUser && signedInUser.isTunerActivated) ? true : false;
                            var subscription = (signedInUser && signedInUser.isSubscription) ? true : false;
                            MSEPlatform.Playback.Etw.traceDRMRootLicenseRefreshAccount(subscription);
                            if (!(subscription && tunerActivated))
                                return WinJS.Promise.wrap();
                            var now = new Date;
                            var spanMs = configurationManager.drm.timeBetweenRefreshSessionsMins * 60 * 1000;
                            var lastRefresh = configurationManager.drm.lastFullRefresh;
                            MSEPlatform.Playback.Etw.traceDRMRootLicenseRefreshDetails(now, lastRefresh, spanMs);
                            if (now - lastRefresh < spanMs)
                                return WinJS.Promise.wrap();
                            MSEPlatform.Playback.Etw.traceDRMRootLicenseRefreshInvoked();
                            return signIn.getPassportTicket(MS.Entertainment.Utilities.SignIn.TicketType.MBI_SSL, MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_Passport)).then(function gotTicket(ticket) {
                                    var drmIndividualizationPromise = WinJS.Promise.as(MS.Entertainment.Platform.Playback.drmIndividualizationPromise);
                                    return drmIndividualizationPromise.then(function _doAcquireRootLicense() {
                                            return Microsoft.Entertainment.Util.PlayReadyHandler.acquireRootLicense(ticket)
                                        })
                                }).then(function succeeded() {
                                    configurationManager.drm.lastFullRefresh = now;
                                    MSEPlatform.Playback.Etw.traceDRMRootLicenseRefreshCompleted("succeeded");
                                    return WinJS.Promise.wrap()
                                }, function failed(error) {
                                    var errorCode = (error && error.number) ? error.number : "unknown";
                                    MSEPlatform.Playback.Etw.traceDRMRootLicenseRefreshCompleted(error + " (error code: " + errorCode + ")");
                                    return WinJS.Promise.wrapError(error)
                                })
                        }
                })(), _reportMetering: function SessionManager_reportMetering(signIn) {
                    var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                    var subscription = (signedInUser && signedInUser.isSubscription) ? true : false;
                    var tunerActivated = (signedInUser && signedInUser.isTunerActivated) ? true : false;
                    var meteringCertificate = (signedInUser) ? signedInUser.meteringCertificate : null;
                    MSEPlatform.Playback.Etw.traceDRMReportMeteringAccount(subscription, tunerActivated, meteringCertificate ? "set" : "not set");
                    if (subscription && tunerActivated) {
                        MSEPlatform.Playback.Etw.traceDRMReportMeteringInvoked();
                        signIn.getPassportTicket(MS.Entertainment.Utilities.SignIn.TicketType.MBI_SSL, MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_Passport)).then(function gotTicket(ticket) {
                            return Microsoft.Entertainment.Util.PlayReadyHandler.reportMetering(ticket, meteringCertificate)
                        }).then(function succeeded() {
                            MSEPlatform.Playback.Etw.traceDRMReportMeteringCompleted("succeeded")
                        }, function failed(error) {
                            var errorCode = (error && error.number) ? error.number : "unknown";
                            MSEPlatform.Playback.Etw.traceDRMReportMeteringCompleted(error + "(error code: " + errorCode + ")")
                        })
                    }
                }, getPassportTicketAsync: function SessionManager_getPassportTicketAsync(returnEmptyOnFailure) {
                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                    if (!signIn)
                        return WinJS.Promise.wrapError("No SignIn service registered");
                    function getPassportTicketAsync_signedIn(returnEmptyOnFailure) {
                        return signIn.getPassportTicket(MS.Entertainment.Utilities.SignIn.TicketType.MBI_SSL, MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_Passport)).then(function getPassportTicketAsync_success(ticket) {
                                if (!ticket) {
                                    if (returnEmptyOnFailure)
                                        return String.empty;
                                    return WinJS.Promise.wrapError(new Error("Unable to get a valid passport ticket."))
                                }
                                return ticket
                            }, function getPassportTicketAsync_error(e) {
                                if (returnEmptyOnFailure)
                                    return String.empty;
                                return WinJS.Promise.wrapError(e)
                            })
                    }
                    if (signIn.isSignedIn || returnEmptyOnFailure)
                        return getPassportTicketAsync_signedIn(returnEmptyOnFailure);
                    else {
                        var waitForSignIn = new WinJS.Promise(function SessionManager_initWaitForSignIn(c, e, p) {
                                function _waitForSignInAsync_signInComplete() {
                                    MS.Entertainment.Utilities.SignIn.removeEventListener("signInComplete", _waitForSignInAsync_signInComplete, false);
                                    c()
                                }
                                MS.Entertainment.Utilities.SignIn.addEventListener("signInComplete", _waitForSignInAsync_signInComplete, false);
                                if (!signIn.isSigningIn)
                                    signIn.signIn(false, true, true)
                            });
                        return waitForSignIn.then(getPassportTicketAsync_signedIn.bind(this, returnEmptyOnFailure))
                    }
                }, _onSignInCompleted: function SessionManager_onSignInCompleted() {
                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                    if (signIn && signIn.isSignedIn) {
                        var retries = 0;
                        var DRMOnSignInActivities = function DRMOnSignIn() {
                                if (MS.Entertainment.Platform.Playback.drmIndividualizationPromise)
                                    MS.Entertainment.Platform.Playback.drmIndividualizationPromise.done(function indivDone(individualized) {
                                        if (individualized) {
                                            if (MS.Entertainment.Utilities.isMusicApp)
                                                this._rootLicenseRefresh(signIn).then(function SessionManager_rootLicenseRefresh_Completed() {
                                                    this._reportMetering(signIn)
                                                }.bind(this))
                                        }
                                        else {
                                            MS.Entertainment.Utilities.DRM.individualizationAsync();
                                            if (retries < 5) {
                                                retries++;
                                                MSEPlatform.Playback.Etw.traceString("Re-indiv on sign-in. Retry#" + retries);
                                                DRMOnSignInActivities()
                                            }
                                            else
                                                MSEPlatform.Playback.Etw.traceString("Re-indiv on sign-in. Does not work - giving up after " + retries + " retries.")
                                        }
                                    }.bind(this), function uhOh(error) {
                                        MS.Entertainment.Platform.fail("DRM indiv promise should not ever fail: only false/true success is expected")
                                    })
                            }.bind(this);
                        DRMOnSignInActivities()
                    }
                }, _onApproachingDataLimitChanged: function SessionManager_onApproachingDataLimitChanged(approachingDataLimit) {
                    if (approachingDataLimit)
                        if (this._nowPlayingSession)
                            this._nowPlayingSession.notifyNetworkConnectionChanged(MS.Entertainment.Platform.NetworkConnection.approachingDataLimit)
                }, _initialize: function SessionManager_initialize() {
                    MSEPlatform.Playback.VideoPlayer.initializeVideoExtensions();
                    if (MS.Entertainment.Utilities.SignIn)
                        MS.Entertainment.Utilities.SignIn.addEventListener("signInComplete", this._onSignInCompleted.bind(this), false);
                    this.sessions = this.sessions || {};
                    var sessionId = MSEPlatform.Playback.WellKnownPlaybackSessionId.nowPlaying;
                    this.sessions[sessionId] = new MSEPlatform.Playback.PlaybackSession(sessionId);
                    this._nowPlayingSession = this.sessions[sessionId];
                    this._nowPlayingSession.addEventListener("currentOrdinalChanged", this._onPlaylistOrdinalChanged.bind(this));
                    this._nowPlayingSession.addEventListener("currentNowPlayingPlaylistLibraryIdChanged", this._currentNowPlayingPlaylistLibraryIdChanged.bind(this));
                    this._nowPlayingSession.addEventListener("currentTransportStateChanged", this._currentTransportStateChanged.bind(this));
                    this.setPrimarySession(sessionId)
                }, _displayRequest: {get: function get_displayRequest() {
                        if (!this.__displayRequest && Windows && Windows.System && Windows.System.Display)
                            if (Windows && Windows.System && Windows.System.Display)
                                this.__displayRequest = new Windows.System.Display.DisplayRequest;
                        return this.__displayRequest
                    }}, _currentTransportStateChanged: function _currentTransportStateChanged(e) {
                    if (this._mediaStore && this.primarySession && this.primarySession.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.stopped && this.primarySession.currentNowPlayingPlaylistLibraryId && !this.primarySession.canSkipForward)
                        this._onPlaylistOrdinalChanged(this.primarySession.first)
                }, _dataLimitHandler: null, _notifyUpcomingStreaming: function _sessionMgr_notifyUpcomingStreaming(isStream) {
                    if (isStream)
                        if (!this._dataLimitHandler) {
                            this._dataLimitHandler = this._onApproachingDataLimitChanged.bind(this);
                            WinJS.Binding.bind(MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.networkStatus), {approachingDataLimit: this._dataLimitHandler})
                        }
                }, _currentNowPlayingPlaylistLibraryIdChanged: function _currentNowPlayingPlaylistLibraryIdChanged(e) {
                    var newId = (e && e.detail) ? e.detail.newValue : null;
                    this._currentNowPlayingPlaylistLibraryId = newId
                }, _onPlaylistOrdinalChanged: function _onPlaylistOrdinalChanged(e) {
                    var newOrdinal = typeof e === "number" ? e : (e && e.detail) ? e.detail.newValue : null;
                    if (newOrdinal === null)
                        return;
                    if (!this._mediaStore)
                        this._mediaStore = new Microsoft.Entertainment.Platform.MediaStore;
                    var provider = this._mediaStore.playlistProvider;
                    if (this._currentNowPlayingPlaylistLibraryId >= 0)
                        provider.savePlaylistCurrentItemIndexAsync(this._currentNowPlayingPlaylistLibraryId, newOrdinal);
                    else {
                        this._updatePendingOrdinal = newOrdinal;
                        if (this._playlistIdLookupPromise)
                            return;
                        var query = new MS.Entertainment.Data.Query.libraryPlaylists;
                        query.playlistType = Microsoft.Entertainment.Platform.PlaylistType.nowPlaying;
                        query.chunkSize = 1;
                        this._playlistIdLookupPromise = query.execute().then(function loadPlaylist(query) {
                            if (query.result.totalCount > 0)
                                query.result.items.toArray(0, 1).then(function updatePlaylist(playlist) {
                                    if (playlist && playlist.length > 0) {
                                        this._currentNowPlayingPlaylistLibraryId = playlist[0].libraryId;
                                        provider.savePlaylistCurrentItemIndexAsync(playlist[0].libraryId, this._updatePendingOrdinal)
                                    }
                                    this._playlistIdLookupPromise = null
                                }.bind(this))
                        }.bind(this))
                    }
                }, requestNetwork: function SessionManager_requestNetwork(requestOwner) {
                    var releaseCount = 0;
                    MS.Entertainment.Platform.assert(MS.Entertainment.Utilities.isMusicApp, "requestNetwork should not be needed outside of the music app");
                    if (!this._networkKeepAliveTag)
                        try {
                            if (!this._extensionManager)
                                this._extensionManager = new Windows.Media.MediaExtensionManager;
                            this._networkKeepAliveSettings = new Windows.Foundation.Collections.PropertySet;
                            this._networkKeepAliveSettings["DoesNotUseNetwork"] = false;
                            this._extensionManager.registerSchemeHandler("Microsoft.Entertainment.Platform.Playback.ConnectedStandbySchemeHandler", "shape:", this._networkKeepAliveSettings);
                            this._networkKeepAliveTag = document.createElement("video");
                            this._networkKeepAliveTag.width = 0;
                            this._networkKeepAliveTag.height = 0;
                            this._networkKeepAliveTag.src = "shape://circle";
                            this._networkKeepAliveTag.autoplay = false
                        }
                        catch(exception) {
                            MSEPlatform.Playback.Etw.traceString("Non-fatal exception caught in requestNetwork: " + exception)
                        }
                    if (this._networkKeepAliveSettings && this._networkKeepAliveRefCount++ === 0)
                        this._networkKeepAliveSettings["DoesNotUseNetwork"] = false;
                    MSEPlatform.Playback.Etw.traceString("requestNetwork from " + requestOwner + ", " + "refCount: " + this._networkKeepAliveRefCount);
                    return {release: function releaseNetwork() {
                                MS.Entertainment.Platform.assert(releaseCount++ === 0, "double release from " + requestOwner);
                                if (this._networkKeepAliveSettings && --this._networkKeepAliveRefCount === 0) {
                                    this._networkKeepAliveSettings["DoesNotUseNetwork"] = true;
                                    this._networkKeepAliveTag.removeAttribute("src");
                                    this._networkKeepAliveTag = null
                                }
                                MSEPlatform.Playback.Etw.traceString("releaseNetwork from " + requestOwner + ", " + "refCount: " + this._networkKeepAliveRefCount)
                            }.bind(this)}
                }, requestNetworkAndDelay: function sessionManager_requestNetworkAndDelay(requestOwner) {
                    var networkReleaseHandle = this.requestNetwork(requestOwner);
                    return WinJS.Promise.timeout(this._networkRequestDelay).then(function onNetworkRequestDelayed() {
                            return networkReleaseHandle
                        }.bind(this))
                }, sessions: null, tmfAuthToken: null, __displayRequest: null, _isDisplayRequestActive: false, _nowPlayingSession: null, _primarySession: null, _transportStateBeforePaused: MS.Entertainment.Platform.Playback.TransportState.unInitialize, _networkKeepAliveTag: null, _networkKeepAliveSettings: null, _networkKeepAliveRefCount: 0, _currentNowPlayingPlaylistLibraryId: -1, _updatePendingOrdinal: -1, _playlistIdLookupPromise: null, _mediaStore: null, _networkRequestDelay: 2000
        }, {
            trackingIdState: {}, _createSessionId: (function createSessionId_closure() {
                    var sessionCounter = 0;
                    return function _createSessionId() {
                            sessionCounter++;
                            return ("sessionId_" + sessionCounter)
                        }
                })()
        })});
    WinJS.Namespace.defineWithParent(MSEPlatform, "Playback", {WellKnownPlaybackSessionId: {
            nowPlaying: "nowPlaying", remoteLRC: "remoteLRC"
        }});
    WinJS.Namespace.defineWithParent(MS.Entertainment, "Platform", {NetworkConnection: {
            approachingDataLimit: "approachingDataLimit", overDataLimit: "overDataLimit", switchedToMetered: "switchedToMetered"
        }});
    MS.Entertainment.ServiceLocator.register(MS.Entertainment.Services.sessionManager, function sessionManagerFactory() {
        var sessionMgr = new MSEPlatform.SessionManager;
        sessionMgr._initialize();
        return sessionMgr
    }, true)
})()
