﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
var MS;
(function(MS) {
    (function(Entertainment) {
        (function(Components) {
            (function(Video) {
                var Augmenter = MS.Entertainment.Data.Augmenter;
                var IEpisode = MS.Entertainment.Data.Augmenter.IEpisode;
                var ISeason = MS.Entertainment.Data.Augmenter.ISeason;
                var EpisodeProgressionHelper = (function() {
                        function EpisodeProgressionHelper(series) {
                            this._series = series;
                            if (!this._series)
                                Trace.fail("No series provided")
                        }
                        EpisodeProgressionHelper.prototype.findNextEpisode = function(selectedSeason, lastWatchedEpisode) {
                            var _this = this;
                            if (!this._series) {
                                Trace.fail("No series provided");
                                return null
                            }
                            return this._hydrateModel(this._series, selectedSeason).then(function(hydrateResults) {
                                    _this._series = hydrateResults.series;
                                    selectedSeason = hydrateResults.season;
                                    return _this._findNextEpisode(selectedSeason, lastWatchedEpisode)
                                }).then(function() {
                                    return _this._hydrateModel(_this._series, _this._season, _this._episode)
                                })
                        };
                        EpisodeProgressionHelper.prototype._findNextEpisode = function(selectedSeason, lastWatchedEpisode) {
                            var _this = this;
                            var useBookmark = !lastWatchedEpisode;
                            var libraryEpisodesPromise = WinJS.Promise.wrap();
                            var lastWatchedPromise;
                            libraryEpisodesPromise = EpisodeProgressionHelper.getLibraryEpisodes(this._series).then(function(episodes) {
                                return WinJS.Promise.wrap(episodes)
                            }, function() {
                                return WinJS.Promise.wrap(null)
                            });
                            if (useBookmark)
                                lastWatchedPromise = this._getLastWatchedBookmark().then(function(bookmark) {
                                    return WinJS.Promise.wrap(bookmark)
                                }, function() {
                                    return WinJS.Promise.wrap(null)
                                });
                            return WinJS.Promise.join({
                                    libraryEpisodes: libraryEpisodesPromise, lastWatchedEpisodeBookmark: lastWatchedPromise
                                }).then(function(results) {
                                    if (!results)
                                        return WinJS.Promise.wrapError("No results from previous Promise");
                                    var lastWatchedEpisodeBookmark = results.lastWatchedEpisodeBookmark;
                                    var libraryEpisodes = results.libraryEpisodes;
                                    var foundLastWatchedEpisode = false;
                                    var lastWatchedSeasonNumber = null;
                                    var lastWatchedEpisodeNumber = null;
                                    var lastWatchedEpisodeLibraryId = -1;
                                    var lastWatchedComplete = false;
                                    var minWatchedPercentage = 0.95;
                                    if (!selectedSeason && !libraryEpisodes)
                                        if (_this._series && _this._series.seasons)
                                            return _this._series.seasons.toArrayAll().then(function(seasons) {
                                                    _this._season = (seasons[seasons.length - 1]);
                                                    return WinJS.Promise.wrap(_this._season)
                                                });
                                    if (lastWatchedEpisode) {
                                        foundLastWatchedEpisode = true;
                                        lastWatchedSeasonNumber = lastWatchedEpisode.seasonNumber;
                                        lastWatchedEpisodeNumber = lastWatchedEpisode.episodeNumber;
                                        lastWatchedEpisodeLibraryId = lastWatchedEpisode.libraryId;
                                        lastWatchedComplete = true
                                    }
                                    else if (lastWatchedEpisodeBookmark) {
                                        foundLastWatchedEpisode = true;
                                        lastWatchedSeasonNumber = lastWatchedEpisodeBookmark.tvSeasonNumber;
                                        lastWatchedEpisodeNumber = lastWatchedEpisodeBookmark.tvEpisodeNumber;
                                        lastWatchedEpisodeLibraryId = lastWatchedEpisodeBookmark.tvEpisodeId;
                                        var lastPosition = lastWatchedEpisodeBookmark.tvEpisodeBookmark;
                                        var duration = lastWatchedEpisodeBookmark.tvEpisodeDuration;
                                        if (lastPosition > 0)
                                            lastWatchedComplete = ((lastPosition / duration) > minWatchedPercentage);
                                        else
                                            lastWatchedComplete = lastWatchedEpisodeBookmark.hasPlayed
                                    }
                                    var seasonHydrationPromise = WinJS.Promise.wrap();
                                    var nextEpisodeNumber = -1;
                                    var currentEpisode = null;
                                    var nextEpisode = null;
                                    if (libraryEpisodes && !foundLastWatchedEpisode) {
                                        for (var x = 0; x < libraryEpisodes.length; x++)
                                            if (libraryEpisodes[x].episodeNumber > 0 && libraryEpisodes[x].seasonNumber > 0) {
                                                _this._episode = libraryEpisodes[x];
                                                break
                                            }
                                    }
                                    else if (libraryEpisodes) {
                                        var useNextEpisode = false;
                                        var nextEpisodeByNumber = null;
                                        var nextEpisodeByPosition = null;
                                        nextEpisodeNumber = (lastWatchedEpisodeNumber + 1);
                                        for (var x = 0; x < libraryEpisodes.length; x++) {
                                            if (lastWatchedEpisodeLibraryId === libraryEpisodes[x].libraryId) {
                                                currentEpisode = libraryEpisodes[x];
                                                useNextEpisode = true
                                            }
                                            if (useNextEpisode && !nextEpisodeByPosition && lastWatchedEpisodeNumber === libraryEpisodes[x].episodeNumber && lastWatchedSeasonNumber === libraryEpisodes[x].seasonNumber && lastWatchedEpisodeLibraryId !== libraryEpisodes[x].libraryId)
                                                nextEpisodeByPosition = libraryEpisodes[x];
                                            if (nextEpisodeNumber === libraryEpisodes[x].episodeNumber && lastWatchedSeasonNumber === libraryEpisodes[x].seasonNumber) {
                                                nextEpisodeByNumber = libraryEpisodes[x];
                                                break
                                            }
                                        }
                                        nextEpisode = nextEpisodeByNumber || nextEpisodeByPosition;
                                        if (!lastWatchedComplete && currentEpisode)
                                            _this._episode = currentEpisode;
                                        else if (lastWatchedComplete && nextEpisode)
                                            _this._episode = nextEpisode;
                                        else if (lastWatchedComplete && _this._series && _this._series.seasons) {
                                            var jumpToNextSeason = false;
                                            _this._series.seasons.toArrayAll().then(function(seasons) {
                                                for (var x = 0; x < seasons.length; x++) {
                                                    var currentSeason = seasons[x];
                                                    if (jumpToNextSeason) {
                                                        nextEpisodeNumber = 1;
                                                        seasonHydrationPromise = currentSeason.refresh();
                                                        break
                                                    }
                                                    else if (lastWatchedSeasonNumber === currentSeason.seasonNumber)
                                                        if (lastWatchedEpisodeNumber === currentSeason.latestEpisode) {
                                                            jumpToNextSeason = true;
                                                            seasonHydrationPromise = currentSeason.refresh()
                                                        }
                                                        else if (nextEpisodeNumber <= currentSeason.latestEpisode) {
                                                            seasonHydrationPromise = currentSeason.refresh();
                                                            break
                                                        }
                                                }
                                            })
                                        }
                                    }
                                    return seasonHydrationPromise.then(function(season) {
                                            if (season && season.episodes)
                                                season.episodes.toArrayAll().then(function(episodes) {
                                                    for (var x = 0; x < episodes.length; x++)
                                                        if (nextEpisodeNumber === episodes[x].episodeNumber) {
                                                            _this._episode = episodes[x];
                                                            break
                                                        }
                                                    if (!_this._episode)
                                                        _this._episode = currentEpisode
                                                });
                                            var seasons = null;
                                            if (_this._series)
                                                if (_this._series.seasons && _this._series.seasons.count > 0)
                                                    seasons = _this._series.seasons;
                                                else
                                                    seasons = _this._series.librarySeasons;
                                            Trace.assert(seasons && seasons.count, ("No seasons found for this series! " + "Series: Name {0}, CanonicalID {1}").format(_this._series.name, _this._series.canonicalId));
                                            if (seasons)
                                                return seasons.toArrayAll().then(function(seasons) {
                                                        if (selectedSeason)
                                                            for (var x = 0; x < seasons.length; x++)
                                                                if (selectedSeason.seasonNumber === seasons[x].seasonNumber) {
                                                                    _this._season = seasons[x];
                                                                    return WinJS.Promise.wrap(_this._season)
                                                                }
                                                        for (var x = 0; x < seasons.length; x++)
                                                            if (_this._episode && _this._episode.seasonNumber === seasons[x].seasonNumber) {
                                                                _this._season = seasons[x];
                                                                return WinJS.Promise.wrap(_this._season)
                                                            }
                                                        if (seasons.length > 0) {
                                                            _this._season = seasons[seasons.length - 1];
                                                            return WinJS.Promise.wrap(_this._season)
                                                        }
                                                    });
                                            Trace.assert(_this._season, ("No season found for this series! " + "Series: Name {0}, CanonicalID {1}").format(_this._series.name, _this._series.canonicalId))
                                        });
                                    return WinJS.Promise.wrap(_this._season)
                                })
                        };
                        EpisodeProgressionHelper.prototype._getLastWatchedBookmark = function() {
                            if (!this._series || this._series.libraryId === undefined || this._series.libraryId === -1)
                                return WinJS.Promise.wrap(null);
                            var findSeriesZuneIdPromise = WinJS.Promise.wrap(this._series.zuneId);
                            if (!this._series.hasZuneId && this._series.librarySeasons)
                                findSeriesZuneIdPromise = this._series.librarySeasons.toArrayAll().then(function(sortedLibrarySeasons) {
                                    for (var x = 0; x < sortedLibrarySeasons.length; x++)
                                        if (!MS.Entertainment.Utilities.isEmptyGuid(sortedLibrarySeasons[x].seriesZuneId))
                                            return WinJS.Promise.wrap(sortedLibrarySeasons[x].seriesZuneId)
                                });
                            var pendingBookmarkWriteOperations;
                            if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.bookmarkOperationsWatcher)) {
                                var bookmarkWatcher = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.bookmarkOperationsWatcher);
                                pendingBookmarkWriteOperations = bookmarkWatcher.waitForPendingOperations()
                            }
                            return WinJS.Promise.join({
                                    zuneId: findSeriesZuneIdPromise, pendingBookmarkWriteOperations: WinJS.Promise.as(pendingBookmarkWriteOperations)
                                }).then(function(results) {
                                    var zuneId = results && results.zuneId;
                                    if (MS.Entertainment.Utilities.isEmptyGuid(zuneId))
                                        return WinJS.Promise.wrap(null);
                                    var ms = new Microsoft.Entertainment.Platform.MediaStore;
                                    return ms.videoProvider.getLastWatchedEpisodeForSeriesAsync(zuneId)
                                }).then(function(bookmark) {
                                    if (bookmark && bookmark.tvEpisodeId >= 0)
                                        return WinJS.Promise.wrap(bookmark);
                                    else
                                        return WinJS.Promise.wrap(null)
                                }, function() {
                                    return WinJS.Promise.wrap(null)
                                })
                        };
                        EpisodeProgressionHelper.prototype._hydrateModel = function(series, season, episode) {
                            var seriesHydrate;
                            if (series && series.hydrate && !series.hydrated)
                                seriesHydrate = series.hydrate();
                            else
                                seriesHydrate = WinJS.Promise.as(series);
                            var seasonHydrate;
                            if (season && season.hydrate && !season.hydrated)
                                seasonHydrate = season.hydrate();
                            else
                                seasonHydrate = WinJS.Promise.as(season);
                            var episodeHydrate;
                            if (episode && episode.hydrate && !episode.hydrated)
                                episodeHydrate = episode.hydrate();
                            else
                                episodeHydrate = WinJS.Promise.as(episode);
                            return WinJS.Promise.join({
                                    series: seriesHydrate, season: seasonHydrate, episode: episodeHydrate
                                })
                        };
                        EpisodeProgressionHelper.getLibraryEpisodes = function(series) {
                            if (!series)
                                return WinJS.Promise.wrap(null);
                            return MS.Entertainment.ViewModels.MediaItemModel.hydrateLibraryInfoAsync(series).then(function() {
                                    if (!series || series.libraryId === -1)
                                        return WinJS.Promise.wrap(null);
                                    var query = new MS.Entertainment.Data.Query.libraryVideoTV;
                                    query.seriesId = series.libraryId;
                                    query.sort = Microsoft.Entertainment.Queries.VideosSortBy.seriesTitleSeasonNumberEpisodeNumber;
                                    query.isLive = false;
                                    query.chunkSize = 1000;
                                    return query.getItemsArrayAndIgnoreErrors()
                                })
                        };
                        return EpisodeProgressionHelper
                    })();
                Video.EpisodeProgressionHelper = EpisodeProgressionHelper
            })(Components.Video || (Components.Video = {}));
            var Video = Components.Video
        })(Entertainment.Components || (Entertainment.Components = {}));
        var Components = Entertainment.Components
    })(MS.Entertainment || (MS.Entertainment = {}));
    var Entertainment = MS.Entertainment
})(MS || (MS = {}))
