﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Components.Shell");
(function() {
    "use strict";
    WinJS.Namespace.define("MS.Entertainment.UI.Components.Shell", {OpenFileAction: MS.Entertainment.deferredDerive("MS.Entertainment.UI.ToolbarAction", function() {
            this.base();
            this.id = MS.Entertainment.UI.AppBarActions.openFile;
            this.automationId = MS.Entertainment.UI.AutomationIds.appBarOpenFile;
            this.title = String.load(String.id.IDS_DETAILS_FILE_OPEN_LABEL);
            this.icon = WinJS.UI.AppBarIcon.openfile
        }, {
            executed: function executed(params) {
                this.parameter = {
                    actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: MS.Entertainment.UI.Actions.ExecutionLocation.canvas
                };
                var fileOpenPicker = null;
                try {
                    fileOpenPicker = new Windows.Storage.Pickers.FileOpenPicker
                }
                catch(e) {
                    MS.Entertainment.UI.Components.Shell.fail("failed to create file open picker");
                    return
                }
                var openFile = function openFile(file) {
                        if (file && file.size !== 0) {
                            if (MS.Entertainment.Utilities.isVideoApp)
                                file = [file];
                            WinJS.Promise.as(MS.Entertainment.Activation.retrievePropertiesAndPlay(file, 0)).done(null, function(error) {
                                MS.Entertainment.UI.Components.Shell.fail("OpenFileAction::openFile(). Failed to retrieve file properties. Error: " + (error && error.message))
                            })
                        }
                    };
                if (MS.Entertainment.Utilities.isVideoApp)
                    try {
                        var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                        var playbackSession = sessionMgr.getSession(MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying);
                        var initialPlaybackState = playbackSession.currentTransportState;
                        if (initialPlaybackState === MS.Entertainment.Platform.Playback.TransportState.playing)
                            playbackSession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.paused;
                        var validFileExtensions = [".3g2", ".3gp", ".3gpp", ".asf", ".avi", ".divx", ".m2ts", ".m2t", ".m4v", ".mod", ".mov", ".mp4", ".mp4v", ".mpe", ".mpeg", ".mpg", ".mpv2", ".mts", ".tod", ".ts", ".tts", ".wm", ".wmv", ".xvid"];
                        fileOpenPicker.fileTypeFilter.replaceAll(validFileExtensions);
                        fileOpenPicker.suggestedStartLocation = Windows.Storage.Pickers.PickerLocationId.videosLibrary;
                        fileOpenPicker.viewMode = Windows.Storage.Pickers.PickerViewMode.thumbnail;
                        fileOpenPicker.pickSingleFileAsync().then(function fileOpenPickerFunction(file) {
                            if (file && file.size !== 0)
                                openFile(file);
                            else if (initialPlaybackState === MS.Entertainment.Platform.Playback.TransportState.playing)
                                playbackSession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.playing
                        })
                    }
                    catch(e) {
                        MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_MEDIA_ERROR_CAPTION), e.number)
                    }
                else if (MS.Entertainment.Utilities.isMusicApp)
                    try {
                        fileOpenPicker.fileTypeFilter.replaceAll([".aac", ".adt", ".adts", ".m3u", ".m4a", ".mp3", ".mp4", ".mpa", ".wav", ".wma", ".wpl", ".zpl"]);
                        fileOpenPicker.suggestedStartLocation = Windows.Storage.Pickers.PickerLocationId.musicLibrary;
                        fileOpenPicker.viewMode = Windows.Storage.Pickers.PickerViewMode.list;
                        fileOpenPicker.pickMultipleFilesAsync().then(function fileOpenPickerFunction(file) {
                            openFile(file)
                        })
                    }
                    catch(e) {
                        MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_MEDIA_ERROR_CAPTION), e.number)
                    }
            }, canExecute: function canExecute() {
                    return (MS.Entertainment.Utilities.isVideoApp || MS.Entertainment.Utilities.isMusicApp)
                }
        }, {
            getOpenFileAction: function getOpenFileAction() {
                if (!MS.Entertainment.UI.Components.Shell.OpenFileAction.openFileAction)
                    MS.Entertainment.UI.Components.Shell.OpenFileAction.openFileAction = new MS.Entertainment.UI.Components.Shell.OpenFileAction;
                return {action: MS.Entertainment.UI.Components.Shell.OpenFileAction.openFileAction}
            }, openFileAction: null
        })});
    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.openFile, function() {
        if (!MS.Entertainment.UI.Components.Shell.OpenFileAction.openFileAction)
            MS.Entertainment.UI.Components.Shell.OpenFileAction.openFileAction = new MS.Entertainment.UI.Components.Shell.OpenFileAction;
        return MS.Entertainment.UI.Components.Shell.OpenFileAction.openFileAction
    })
})()
