﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function(undefined) {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Actions");
    WinJS.Namespace.define("MS.Entertainment.UI.Actions", {Download: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function download() {
            this.base()
        }, {
            executed: function executed(param) {
                var mediaItem = this._extractMediaItem(param);
                MS.Entertainment.Platform.PurchaseHelpers.ensurePreownedMediaAddedAsync(mediaItem).then(function ensurePreownedMediaAddedAsync_complete(dbItems) {
                    MS.Entertainment.Platform.PurchaseHelpers.downloadMedia(mediaItem, dbItems.dbMediaIds, dbItems.dbMediaTypes)
                }, function ensurePreownedMediaAddedAsync_error(errorCode) {
                    var fileTransferNotifications = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.fileTransferNotifications);
                    if (mediaItem.hasServiceId)
                        fileTransferNotifications.setItemError(mediaItem.serviceId, errorCode);
                    if (mediaItem.inCollection)
                        fileTransferNotifications.setItemError(mediaItem.libraryId, errorCode)
                })
            }, canExecute: function canExecute(param) {
                    var mediaItem = this._extractMediaItem(param);
                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                    return (signIn.isSignedIn && !mediaItem.hasLocalContent)
                }, _extractMediaItem: function _extractMediaItem(param) {
                    var mediaItem;
                    if (param && param.mediaItem)
                        mediaItem = param.mediaItem;
                    else
                        mediaItem = param;
                    if (MS.Entertainment.Data.List.isList(mediaItem))
                        MS.Entertainment.Data.List.listToArray(mediaItem).then(function gotArray(mediaItemsArray) {
                            MS.Entertainment.UI.Actions.assert(mediaItemsArray.length === 1, "Download Action only accepts one mediaItem");
                            mediaItem = mediaItemsArray[0]
                        });
                    return mediaItem
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.UI.Actions", {SubscriptionDownload: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function subscriptionDownload() {
            this.base()
        }, {
            automationId: MS.Entertainment.UI.AutomationIds.subscriptionDownload, _executing: false, _reEnableAction: function _reEnableAction() {
                    this._executing = false;
                    this.requeryCanExecute()
                }, executed: function executed(param) {
                    this._executing = true;
                    this.requeryCanExecute();
                    var mediaItems = this._extractMediaItems(param);
                    var mediaContext;
                    if (!mediaItems) {
                        mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                        mediaItems = this._ensureListOrArray(mediaContext.mediaItem)
                    }
                    var downloadPromise = WinJS.Promise.wrap();
                    if (mediaItems)
                        downloadPromise = MS.Entertainment.UI.SubscriptionDownload.verifyMachineActivationIsNotAtLimit().then(function verifyMachineActivationIsNotAtLimit() {
                            return MS.Entertainment.UI.SubscriptionDownload.subscriptionDownload(mediaItems)
                        }).then(function downloadMedia_complete(addedItems) {
                            if (mediaContext)
                                mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.subscriptionDownload, {addedItems: addedItems});
                            return WinJS.Promise.timeout()
                        }, function errorOccurred(errorCode) {
                            return MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_DOWNLOAD_ERROR_CAPTION), errorCode)
                        });
                    downloadPromise.done(this._reEnableAction.bind(this), this._reEnableAction.bind(this))
                }, canExecute: function canExecute(param) {
                    return !this._executing
                }, _extractMediaItems: function _extractMediaItem(param) {
                    var mediaItems;
                    if (param && param.mediaItem)
                        mediaItems = param.mediaItem;
                    else
                        mediaItems = param;
                    return MS.Entertainment.Data.List.isListOrArray(mediaItems) ? mediaItems : [mediaItems]
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.UI.Actions", {SubscriptionAddToMyMusic: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function subscriptionAddToMyMusic() {
            this.base()
        }, {
            automationId: MS.Entertainment.UI.AutomationIds.smartButtonAddToMyMusic, _executing: false, _appStateLoaded: false, _dtoMarketplace: false, _marketplaceEnabled: false, _refreshAppState: function _refreshAppState() {
                    var signedInUserService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                    var featureEnablementService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.featureEnablement);
                    var marketplaceEnabled = !!featureEnablementService.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace);
                    var freeStreamEnabled = !!featureEnablementService.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlay);
                    var userCanSubscriptionStream = marketplaceEnabled && signedInUserService.isSubscription;
                    this._dtoMarketplace = marketplaceEnabled && !freeStreamEnabled && !userCanSubscriptionStream;
                    this._marketplaceEnabled = marketplaceEnabled
                }, _reEnableAction: function _reEnableAction() {
                    this._executing = false;
                    this.requeryCanExecute()
                }, _showCloudGrovelInfoDialog: function _showCloudGrovelInfoDialog() {
                    MS.Entertainment.Music.cloudGrovelInfoDialog.show()
                }, _showErrorNotification: function _showErrorNotification() {
                    if (MS.Entertainment.UI.Controls.CommandingPopOver && MS.Entertainment.UI.Controls.CommandingPopOver.popoverOpen) {
                        var notificationText = String.load(String.id.IDS_MUSIC2_PANEL_ADD_ERROR);
                        MS.Entertainment.UI.Controls.CommandingPopOver.displayPopoverNotification(notificationText, false)
                    }
                }, _showNotification: function _showNotification(addedItems, baseMediaItem) {
                    var count = addedItems.dbMediaIds.length;
                    MS.Entertainment.UI.Actions.assert(count > 0, "_showNotification called with no items added.");
                    var isCloudEnabled = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.cloudCollection).isEnabled;
                    var action = isCloudEnabled ? this._showCloudGrovelInfoDialog : null;
                    if (MS.Entertainment.UI.Controls.CommandingPopOver && MS.Entertainment.UI.Controls.CommandingPopOver.popoverOpen) {
                        var notificationText = String.empty;
                        switch (baseMediaItem[0].mediaType) {
                            case Microsoft.Entertainment.Queries.ObjectType.video:
                            case Microsoft.Entertainment.Queries.ObjectType.track:
                                notificationText = String.load(String.id.IDS_MUSIC2_PANEL_ADD_1_ITEM).format(String.load(String.id.IDS_MUSIC2_PANEL_ADD_SONG), String.load(String.id.IDS_MUSIC2_PANEL_ADD_COLLECTION));
                                break;
                            case Microsoft.Entertainment.Queries.ObjectType.album:
                                notificationText = String.load(String.id.IDS_MUSIC2_PANEL_ADD_1_ITEM).format(String.load(String.id.IDS_MUSIC2_PANEL_ADD_ALBUM), String.load(String.id.IDS_MUSIC2_PANEL_ADD_COLLECTION));
                                break;
                            case Microsoft.Entertainment.Queries.ObjectType.person:
                                notificationText = String.load(String.id.IDS_MUSIC2_PANEL_ADD_N_ITEMS).format(String.load(String.id.IDS_MUSIC2_PANEL_ADD_TOP_SONGS), String.load(String.id.IDS_MUSIC2_PANEL_ADD_COLLECTION));
                                break;
                            default:
                                MS.Entertainment.UI.Actions.fail("Unexpected media type added to collection.")
                        }
                        MS.Entertainment.UI.Controls.CommandingPopOver.displayPopoverNotification(notificationText, true)
                    }
                    else if (MS.Entertainment.Services.appNotification) {
                        var title = String.empty;
                        var subTitle = String.empty;
                        if (isCloudEnabled) {
                            switch (addedItems.dbMediaTypes[0]) {
                                case Microsoft.Entertainment.Queries.ObjectType.track:
                                    title = MS.Entertainment.Formatters.countText(count, String.id.IDS_MUSIC_CLOUD_SONGS_ADDED_NOTIFICATION_LINE1_PLURAL);
                                    break;
                                case Microsoft.Entertainment.Queries.ObjectType.album:
                                    title = MS.Entertainment.Formatters.countText(count, String.id.IDS_MUSIC_CLOUD_ALBUMS_ADDED_NOTIFICATION_LINE1_PLURAL);
                                    break;
                                case Microsoft.Entertainment.Queries.ObjectType.person:
                                    title = MS.Entertainment.Formatters.countText(count, String.id.IDS_MUSIC_CLOUD_ARTISTS_ADDED_NOTIFICATION_LINE1_PLURAL);
                                    break;
                                default:
                                    MS.Entertainment.UI.Actions.fail("Unexpected media type added to collection.")
                            }
                            subTitle = String.load(String.id.IDS_MUSIC_CLOUD_SONGS_ADDED_NOTIFICATION_LINE2)
                        }
                        else {
                            title = String.load(String.id.IDS_MUSIC_NOTIFICATION_ADDED_TO_MY_MUSIC);
                            switch (addedItems.dbMediaTypes[0]) {
                                case Microsoft.Entertainment.Queries.ObjectType.track:
                                    subTitle = MS.Entertainment.Formatters.countText(count, String.id.IDS_MUSIC_NOTIFICATION_TRACKS_ADDED_PLURAL);
                                    break;
                                case Microsoft.Entertainment.Queries.ObjectType.album:
                                    subTitle = MS.Entertainment.Formatters.countText(count, String.id.IDS_MUSIC_NOTIFICATION_ALBUMS_ADDED_PLURAL);
                                    break;
                                case Microsoft.Entertainment.Queries.ObjectType.person:
                                    subTitle = MS.Entertainment.Formatters.countText(count, String.id.IDS_MUSIC_NOTIFICATION_ARTISTS_ADDED_PLURAL);
                                    break
                            }
                        }
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appNotification).send(new MS.Entertainment.UI.Notification({
                            notificationType: MS.Entertainment.UI.Notification.Type.Informational, title: title, subTitle: subTitle, moreDetails: null, icon: MS.Entertainment.UI.Icon.musicInCollection, action: action, secondaryActions: null, category: null, isPersistent: false
                        }));
                        MS.Entertainment.UI.CollectionChangeNotifierService.blockGlobalCollectionChangeEvents = false;
                        MS.Entertainment.UI.CollectionChangeNotifierService.blockGlobalCollectionUpSyncEvents = false
                    }
                }, translateDragAndDropParams: function translateDragAndDropParams(source) {
                    return {mediaItem: source && source.data}
                }, executed: function executed(param) {
                    param = param || {};
                    var mediaItems = this._extractMediaItems(param);
                    var mediaContext;
                    var completeAction = param.completeAction;
                    if (!mediaItems) {
                        mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                        completeAction = function completeAction(addedItems) {
                            if (mediaContext)
                                mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.addToMyMusic, {addedItems: addedItems})
                        };
                        if (!mediaContext.mediaItem)
                            return;
                        mediaItems = MS.Entertainment.Data.List.isListOrArray(mediaContext.mediaItem) ? mediaContext.mediaItem : [mediaContext.mediaItem]
                    }
                    this._executing = true;
                    this.requeryCanExecute();
                    var downloadMedia = false;
                    if (MS.Entertainment.Utilities.isMusicApp1) {
                        var cloudCollectionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.cloudCollection);
                        downloadMedia = cloudCollectionService && cloudCollectionService.shouldDownloadOnAdd;
                        var cloudEnabled = cloudCollectionService.isEnabled;
                        MS.Entertainment.UI.CollectionChangeNotifierService.blockGlobalCollectionChangeEvents = cloudEnabled;
                        MS.Entertainment.UI.CollectionChangeNotifierService.blockGlobalCollectionUpSyncEvents = cloudEnabled
                    }
                    var mediaToDownload = param.containerItem || mediaItems;
                    return this._addToMyMusicNoDownload(mediaItems, param.containerItem, param.suppressNotification || downloadMedia, completeAction).then(function download() {
                            return downloadMedia ? this._downloadMediaItems(mediaToDownload, param.containerItem, completeAction) : WinJS.Promise.wrap()
                        }.bind(this)).then(this._signalDone.bind(this), this._signalError.bind(this)).then(this._reEnableAction.bind(this))
                }, canExecute: function canExecute(param) {
                    this._refreshAppState();
                    return !this._executing && !this._dtoMarketplace && this._marketplaceEnabled
                }, _addToMyMusicNoDownload: function _addToMyMusicNoDownload(mediaItems, containerItem, suppressNotification, completeAction) {
                    var signInService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                    var executePromise;
                    var addMedia = function addMedia(mediaItems, containerItem) {
                            return MS.Entertainment.Platform.PurchaseHelpers.addSubscriptionMedia(mediaItems, containerItem).then(function addSubscriptionMediaComplete(addedItems) {
                                    if (containerItem)
                                        return addMedia.call(this, containerItem);
                                    else {
                                        if (!suppressNotification)
                                            this._showNotification(addedItems, mediaItems);
                                        if (completeAction)
                                            return completeAction(addedItems)
                                    }
                                }.bind(this), function addSubscriptionMediaError(errorCode) {
                                    var showErrorPromise;
                                    if (MS.Entertainment.Utilities.isMusicApp2) {
                                        this._showErrorNotification();
                                        showErrorPromise = WinJS.Promise.as()
                                    }
                                    else
                                        showErrorPromise = (typeof errorCode === "number") ? MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_MEDIA_ERROR_CAPTION), errorCode) : WinJS.Promise.as();
                                    return showErrorPromise.then(function showErrorPromiseComplete() {
                                            return WinJS.Promise.wrapError()
                                        })
                                }.bind(this))
                        }.bind(this);
                    if (signInService.isSignedIn)
                        executePromise = addMedia(mediaItems, containerItem);
                    else
                        executePromise = new WinJS.Promise(function init(signInComplete) {
                            MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_ADD_TO_MY_MUSIC_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signUp, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], signInComplete)
                        }).then(function signInComplete(result) {
                            if (result === MS.Entertainment.Utilities.SignIn.SignInResult.success)
                                return addMedia(mediaItems, containerItem)
                        });
                    return executePromise
                }, _downloadMediaItems: function _downloadMediaItems(mediaToDownload, containerItem, completeAction) {
                    return MS.Entertainment.UI.SubscriptionDownload.verifyMachineActivationIsNotAtLimit().then(function subscriptionDownloadAllowed() {
                            return MS.Entertainment.UI.SubscriptionDownload.subscriptionDownload(mediaToDownload)
                        }, function subscriptionDownloadNotAllowed(errorCode) {
                            if (errorCode !== MS.Entertainment.UI.SubscriptionDownload.deviceActivationLimitExceededErrorCode)
                                return MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_DOWNLOAD_ERROR_CAPTION), errorCode);
                            MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_DOWNLOAD_ERROR_CAPTION), errorCode);
                            var addToMyMusicPromise = this._addToMyMusicNoDownload(mediaToDownload, containerItem);
                            (Microsoft.Entertainment.Configuration.ConfigurationManager()).music.musicPassDownloadEnabled = false;
                            var cloudCollectionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.cloudCollection);
                            if (cloudCollectionService)
                                cloudCollectionService.autoDownloadEnabled = false;
                            return addToMyMusicPromise
                        }.bind(this)).then(function downloadMedia_complete(addedItems) {
                            if (completeAction)
                                return completeAction(addedItems);
                            return null
                        }, function downloadMedia_error(error) {
                            return MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_DOWNLOAD_ERROR_CAPTION), error)
                        })
                }, _signalError: function _signalError(error) {
                    var errorMessage = (error && error.message);
                    if (errorMessage)
                        MS.Entertainment.UI.Actions.fail("Failed to add to music. Error message: " + errorMessage);
                    this._signalDone()
                }, _signalDone: function _signalDone() {
                    var referenceContainer = WinJS.Binding.unwrap(this.referenceContainer);
                    if (referenceContainer && referenceContainer.domElement) {
                        var domEvent = document.createEvent("Event");
                        domEvent.initEvent("iconButtonClicked", true, true);
                        domEvent.automationId = this.automationId;
                        domEvent.id = this.id;
                        referenceContainer.domElement.dispatchEvent(domEvent)
                    }
                }, _extractMediaItems: function _extractMediaItems(param) {
                    var mediaItems;
                    if (param && param.mediaItem)
                        mediaItems = param.mediaItem;
                    else
                        mediaItems = param;
                    return this._ensureListOrArray(mediaItems)
                }, _ensureListOrArray: function _ensureListOrArray(mediaItems) {
                    var result = mediaItems;
                    if (!MS.Entertainment.Data.List.isListOrArray(mediaItems) && mediaItems)
                        result = [mediaItems];
                    return result
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.UI", {SubscriptionDownload: {
            _getTrackLibraryId: function _getTrackLibraryId(mediaItem) {
                var libraryItems = [{
                            libraryId: mediaItem.libraryId, libraryType: mediaItem.mediaType
                        }];
                return WinJS.Promise.wrap(libraryItems)
            }, _getMediaContainerLibraryIds: function _getMediaContainerLibraryIds(mediaContainer) {
                    var libraryItems = [];
                    var tracksPromise = WinJS.Promise.wrap();
                    if (mediaContainer)
                        if (mediaContainer.tracks)
                            tracksPromise = WinJS.Promise.wrap(mediaContainer.tracks);
                        else {
                            MS.Entertainment.UI.Actions.assert(mediaContainer.mediaType === Microsoft.Entertainment.Queries.ObjectType.album, "Expected to expand an album if tracks are missing.");
                            var query = new MS.Entertainment.Data.Query.libraryTracks;
                            var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                            var collectionFilter = mediaContext.collectionFilter || Microsoft.Entertainment.Platform.MediaAvailability.undefined;
                            query.albumId = mediaContainer.libraryId;
                            query.mediaAvailability = collectionFilter;
                            tracksPromise = query.getItemsAndIgnoreErrors()
                        }
                    return tracksPromise.then(function _gotTracks(tracks) {
                            if (tracks)
                                return tracks.forEachAll(function _getItemLibraryId(arg) {
                                        if (arg.item.data && arg.item.data.hasZuneId)
                                            libraryItems.push({
                                                libraryId: arg.item.data.libraryId, libraryType: arg.item.data.mediaType
                                            })
                                    })
                        }).then(function _gotItemsArray(array) {
                            return libraryItems
                        })
                }, _canDownloadCollectionMedia: function _canDownloadCollectionMedia(stateInfo) {
                    if (stateInfo.download.hasFailed && stateInfo.download.hasRetryable)
                        return true;
                    var hasDownloads = false;
                    var playabilityItems = stateInfo.collection.playability.items;
                    var entitlements = stateInfo.marketplace.entitlements;
                    if (playabilityItems && playabilityItems.length > 0 && entitlements && entitlements.length > 0) {
                        var hasNonLocalMedia = false;
                        var nonLocalMedia = {};
                        for (var i = 0; i < playabilityItems.length; i++) {
                            var playabilityItem = playabilityItems[i];
                            if (playabilityItem && playabilityItem.mediaId && !playabilityItem.isDownloading && !playabilityItem.locallyPlayable) {
                                nonLocalMedia[playabilityItem.mediaId.toLowerCase()] = playabilityItem;
                                hasNonLocalMedia = true
                            }
                        }
                        if (hasNonLocalMedia)
                            for (var i = 0; i < entitlements.length; i++) {
                                var entitlement = entitlements[i];
                                if (entitlement && entitlement.canFullyDownload && nonLocalMedia[entitlement.serviceMediaId.toLowerCase()])
                                    return true
                            }
                    }
                    return false
                }, subscriptionDownload: function subscriptionDownload(mediaItems) {
                    var downloadableMediaItems = [];
                    var downloadStatePromises = [];
                    var downloadabilityCheckPromise = WinJS.Promise.wrap();
                    downloadabilityCheckPromise = MS.Entertainment.Data.List.listToArray(mediaItems).then(function _gotArray(mediaItemsArray) {
                        mediaItemsArray.forEach(function _getDownloadInfo(mediaItem) {
                            var hydratePromise = mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.track ? WinJS.Promise.wrap(mediaItem) : mediaItem.hydrate();
                            downloadStatePromises.push(hydratePromise.then(function checkState() {
                                var singleDownloadabilityCheckPromise;
                                if (mediaItem.fromCollection && mediaItem.canSubscriptionDownload)
                                    downloadableMediaItems.push(mediaItem);
                                else if (mediaItem.fromCollection && mediaItem.inCollection)
                                    singleDownloadabilityCheckPromise = MS.Entertainment.ViewModels.SmartBuyStateEngine.queryMediaStateAsync(mediaItem).then(function _getMediaStateComplete(stateInfo) {
                                        if (MS.Entertainment.UI.SubscriptionDownload._canDownloadCollectionMedia(stateInfo))
                                            downloadableMediaItems.push(mediaItem)
                                    });
                                else
                                    singleDownloadabilityCheckPromise = MS.Entertainment.ViewModels.MusicSmartBuyStateHandlers.getMusicRights(mediaItem).then(function _getMusicRightsComplete(rights) {
                                        if (rights && rights.subscriptionDownload)
                                            downloadableMediaItems.push(mediaItem)
                                    });
                                return singleDownloadabilityCheckPromise
                            }).then(null, function ignoreError(error) {
                                MS.Entertainment.UI.Actions.fail("Failed to get subscription downloadability for media item. error: " + (error && error.message))
                            }))
                        });
                        return WinJS.Promise.join(downloadStatePromises)
                    });
                    return downloadabilityCheckPromise.then(function _gotDownloadableItems() {
                            var hydrateLibraryInfoPromises = [];
                            downloadableMediaItems.forEach(function notifyDownloadAndHydrateLibraryId(mediaItem) {
                                if (mediaItem.contentNotifications) {
                                    var notification = new MS.Entertainment.UI.ContentNotification.Notification(WinJS.UI.AppBarIcon.download, String.load(String.id.IDS_FILE_TRANSFER_DOWNLOADING_QUEUE_SHORT), String.load(String.id.IDS_FILE_TRANSFER_DOWNLOADING_QUEUE_LONG));
                                    if (!mediaItem.allTracksLocal)
                                        mediaItem.contentNotifications.sendNotification(MS.Entertainment.UI.ContentNotification.NotificationType.marketplaceDownloadStatus, notification)
                                }
                                hydrateLibraryInfoPromises.push(MS.Entertainment.ViewModels.MediaItemModel.hydrateLibraryInfoAsync(mediaItem))
                            });
                            return WinJS.Promise.join(hydrateLibraryInfoPromises)
                        }).then(function _gotLibraryIds() {
                            if (!downloadableMediaItems || !downloadableMediaItems.length)
                                return WinJS.Promise.wrap();
                            var collectionMediaItems = [];
                            var marketplaceMediaItems = [];
                            var downloadMediaPromises = [];
                            downloadableMediaItems.forEach(function _splitDownloadableMediaItems(mediaItem) {
                                if ((mediaItem.fromCollection && mediaItem.inCollection) || mediaItem.isFromPlaylist)
                                    collectionMediaItems.push(mediaItem);
                                else
                                    marketplaceMediaItems.push(mediaItem)
                            });
                            MS.Entertainment.UI.Actions.assert((collectionMediaItems.length + marketplaceMediaItems.length) === downloadableMediaItems.length, "Expected all downloadable items to be categorized as either collection or marketplace items.");
                            var result = {itemsAddedToCollection: null};
                            if (marketplaceMediaItems.length)
                                downloadMediaPromises.push(MS.Entertainment.UI.SubscriptionDownload._downloadMarketplaceMedia(marketplaceMediaItems, result));
                            if (collectionMediaItems.length)
                                downloadMediaPromises.push(MS.Entertainment.UI.SubscriptionDownload._downloadCollectionMedia(collectionMediaItems));
                            return WinJS.Promise.join(downloadMediaPromises).then(function downloadsCompleted() {
                                    return result.itemsAddedToCollection
                                })
                        })
                }, _downloadMarketplaceMedia: function _downloadMarketplaceMedia(marketplaceMediaItems, result) {
                    return MS.Entertainment.Platform.PurchaseHelpers.addSubscriptionMedia(marketplaceMediaItems).then(function addSubscriptionMedia_complete(dbItems) {
                            result.itemsAddedToCollection = dbItems;
                            return MS.Entertainment.Platform.PurchaseHelpers.downloadMedia(null, dbItems.dbMediaIds, dbItems.dbMediaTypes)
                        }, function addSubscriptionMedia_error(errorCode) {
                            marketplaceMediaItems.forEach(function _clearNotifications(mediaItem) {
                                if (mediaItem && mediaItem.contentNotifications)
                                    mediaItem.contentNotifications.clear();
                                if (typeof errorCode === "number") {
                                    var fileTransferNotifications = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.fileTransferNotifications);
                                    if (mediaItem.serviceId && mediaItem.serviceId !== MS.Entertainment.Utilities.EMPTY_GUID)
                                        fileTransferNotifications.setItemError(mediaItem.serviceId, errorCode);
                                    if (mediaItem.libraryId && mediaItem.libraryId >= 0)
                                        fileTransferNotifications.setItemError(mediaItem.libraryId, errorCode)
                                }
                            });
                            return WinJS.Promise.wrapError(errorCode)
                        })
                }, _downloadCollectionMedia: function _downloadCollectionMedia(collectionMediaItems) {
                    var getCollectionTrackPromises = collectionMediaItems.map(function _getTracksToDownload(mediaItem) {
                            switch (mediaItem.mediaType) {
                                case Microsoft.Entertainment.Queries.ObjectType.track:
                                    return MS.Entertainment.UI.SubscriptionDownload._getTrackLibraryId(mediaItem);
                                case Microsoft.Entertainment.Queries.ObjectType.album:
                                    return MS.Entertainment.UI.SubscriptionDownload._getMediaContainerLibraryIds(mediaItem);
                                case Microsoft.Entertainment.Queries.ObjectType.playlist:
                                    return mediaItem.hydrate().then(MS.Entertainment.UI.SubscriptionDownload._getMediaContainerLibraryIds);
                                default:
                                    MS.Entertainment.ViewModels.fail("SubscriptionDownloadAction received an invalid media type");
                                    return WinJS.Promise.wrapError()
                            }
                        });
                    return WinJS.Promise.join(getCollectionTrackPromises).then(function downloadItemsArray(downloadItemLists) {
                            if (downloadItemLists) {
                                var downloadItems = [].concat.apply([], downloadItemLists);
                                var libraryIds = downloadItems.map(function _getLibraryId(item) {
                                        return item.libraryId
                                    });
                                var libraryTypes = downloadItems.map(function _getLibraryType(item) {
                                        return item.libraryType
                                    });
                                return MS.Entertainment.Platform.PurchaseHelpers.downloadMedia(null, libraryIds, libraryTypes, false, false)
                            }
                        })
                }, verifyMachineActivationIsNotAtLimit: function verifyMachineActivationIsNotAtLimit() {
                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                    var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                    if (signIn && signIn.isSignedIn && signedInUser) {
                        var allowSubscriptionDownload = (signedInUser.isTunerActivated || signedInUser.isTunerActivatable);
                        if (allowSubscriptionDownload)
                            return WinJS.Promise.wrap();
                        else {
                            var refreshSignInStateCompletedOrError = function refreshSignInStateCompletedOrError() {
                                    if (signIn.isSignedIn) {
                                        var allowSubscriptionDownload = (signedInUser.isTunerActivated || signedInUser.isTunerActivatable);
                                        if (allowSubscriptionDownload || !signedInUser.isSubscription)
                                            return WinJS.Promise.wrap();
                                        else
                                            return WinJS.Promise.wrapError(MS.Entertainment.UI.SubscriptionDownload.deviceActivationLimitExceededErrorCode)
                                    }
                                    else
                                        return WinJS.Promise.wrapError(MS.Entertainment.Utilities.SignInErrors.ZEST_E_SIGNIN_REQUIRED)
                                };
                            return signIn.refreshSignInState().then(refreshSignInStateCompletedOrError, refreshSignInStateCompletedOrError)
                        }
                    }
                    else
                        return WinJS.Promise.wrapError(MS.Entertainment.Utilities.SignInErrors.ZEST_E_SIGNIN_REQUIRED)
                }, deviceActivationLimitExceededErrorCode: 0xC1010008 - 0xFFFFFFFF - 1
        }});
    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.download, function() {
        return new MS.Entertainment.UI.Actions.Download
    });
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.subscriptionDownload, function() {
        return new MS.Entertainment.UI.Actions.SubscriptionDownload
    });
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.subscriptionAddToMyMusic, function() {
        return new MS.Entertainment.UI.Actions.SubscriptionAddToMyMusic
    })
})()
