﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
(function() {
    "use strict";
    scriptValidator();
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI");
    WinJS.Namespace.define("MS.Entertainment.UI", {FileTransferNotificationService: MS.Entertainment.UI.Framework.define(function FileTransferNotificationServiceConstructor() {
            this._listeners = {};
            this._itemErrors = {};
            this._itemTransferState = {};
            this._containerErrors = {};
            this._internalHandlers = {}
        }, {
            _internalHandlers: null, _listeners: null, _itemErrors: null, _itemTransferState: null, _fileTransferListenerId: String.Empty, startListening: function startListening(handlers) {
                    MS.Entertainment.UI.assert(handlers, "invalid value passed for handlers");
                    var fileTransferService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.fileTransfer);
                    if (Array.isArray(handlers)) {
                        this._fileTransferListenerId = [];
                        for (var i = 0; i < handlers.length; i++) {
                            this._fileTransferListenerId[i] = "FileTransferNotificationServiceListener_" + Math.random();
                            MS.Entertainment.UI.assert(handlers[i].taskKeyGetter, "invalid taskKeyGetter");
                            MS.Entertainment.UI.assert(handlers[i].notifier, "invalid notifier");
                            fileTransferService.registerListener(this._fileTransferListenerId[i], handlers[i].taskKeyGetter, null, handlers[i].notifier, this)
                        }
                    }
                    else {
                        this._fileTransferListenerId = "FileTransferNotificationServiceListener_" + Math.random();
                        MS.Entertainment.UI.assert(handlers.taskKeyGetter, "invalid taskKeyGetter");
                        MS.Entertainment.UI.assert(handlers.notifier, "invalid notifier");
                        fileTransferService.registerListener(this._fileTransferListenerId, handlers.taskKeyGetter, null, handlers.notifier, this)
                    }
                }, stopListening: function stopListening() {
                    if (this._fileTransferListenerId) {
                        var fileTransferService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.fileTransfer);
                        if (Array.isArray(this._fileTransferListenerId))
                            for (var i = 0; i < this._fileTransferListenerId.length; i++)
                                fileTransferService.unregisterListener(this._fileTransferListenerId[i]);
                        else
                            fileTransferService.unregisterListener(this._fileTransferListenerId);
                        this._fileTransferListenerId = String.Empty
                    }
                }, attachListener: function attachListener(mediaId, listener) {
                    MS.Entertainment.UI.assert(this._isValidMediaId(mediaId), "invalid mediaId param");
                    MS.Entertainment.UI.assert(listener instanceof Function, "listener param is not a function");
                    var listeners = this._listeners[mediaId];
                    if (!listeners)
                        this._listeners[mediaId] = [listener];
                    else
                        listeners.push(listener);
                    var errorCode = this._itemErrors[mediaId];
                    if (errorCode)
                        listener(mediaId, MS.Entertainment.UI.FileTransferNotificationService.Event.transferError, errorCode);
                    if (this._itemTransferState[mediaId])
                        listener(mediaId, this._itemTransferState[mediaId], true)
                }, detachListener: function detachListener(mediaId, listener) {
                    MS.Entertainment.UI.assert(this._isValidMediaId(mediaId), "invalid mediaId param");
                    MS.Entertainment.UI.assert(listener instanceof Function, "listener param is not a function");
                    var listeners = this._listeners[mediaId];
                    if (listeners)
                        if (listeners.length === 1) {
                            MS.Entertainment.UI.assert(listeners[0] === listener, "listener not attached");
                            if (listeners[0] === listener)
                                delete this._listeners[mediaId]
                        }
                        else {
                            var index = listeners.indexOf(listener);
                            MS.Entertainment.UI.assert(index >= 0, "listener not attached");
                            if (index >= 0)
                                listeners.splice(index, 1)
                        }
                }, setItemError: function setItemError(mediaId, errorCode) {
                    MS.Entertainment.UI.assert(this._isValidMediaId(mediaId), "invalid mediaId param");
                    MS.Entertainment.UI.assert(!isNaN(errorCode), "invalid errorCode param");
                    if (errorCode === 0)
                        delete this._itemErrors[mediaId];
                    else
                        this._itemErrors[mediaId] = errorCode;
                    this._sendEvent(mediaId, MS.Entertainment.UI.FileTransferNotificationService.Event.transferError, errorCode)
                }, setItemTransferState: function setItemTransferState(mediaId, state, value) {
                    MS.Entertainment.UI.assert(this._isValidMediaId(mediaId), "invalid mediaId param");
                    MS.Entertainment.UI.assert(state === MS.Entertainment.UI.FileTransferNotificationService.Event.download, "Unsupported State passed into setTransferState");
                    if (!value)
                        delete this._itemTransferState[mediaId];
                    else
                        this._itemTransferState[mediaId] = state;
                    this._sendEvent(mediaId, state, value)
                }, _sendEvent: function _sendEvent(mediaId, event, value) {
                    var listeners = this._listeners[mediaId];
                    if (listeners)
                        listeners.forEach(function(listener) {
                            listener(mediaId, event, value)
                        })
                }, _isValidMediaId: function _isValidMediaId(mediaId) {
                    return ((typeof mediaId === "number" && mediaId >= 0) || (typeof mediaId === "string" && mediaId !== MS.Entertainment.Utilities.EMPTY_GUID))
                }
        }, {
            Instance: null, Event: {
                    download: "download", transferError: "transferError"
                }
        })});
    MS.Entertainment.ServiceLocator.register(MS.Entertainment.Services.fileTransferNotifications, function FileTransferNotificationServiceFactory() {
        if (!MS.Entertainment.UI.FileTransferNotificationService.Instance)
            MS.Entertainment.UI.FileTransferNotificationService.Instance = new MS.Entertainment.UI.FileTransferNotificationService;
        return MS.Entertainment.UI.FileTransferNotificationService.Instance
    }, true)
})()
