﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
var __extends = this.__extends || function(d, b) {
        for (var p in b)
            if (b.hasOwnProperty(p))
                d[p] = b[p];
        function __() {
            this.constructor = d
        }
        __.prototype = b.prototype;
        d.prototype = new __
    };
var MS;
(function(MS) {
    (function(Entertainment) {
        (function(UI) {
            var NetworkStatusService = (function(_super) {
                    __extends(NetworkStatusService, _super);
                    function NetworkStatusService() {
                        _super.call(this);
                        this._approachingDataLimit = false;
                        this._networkStatusChangedBinding = null;
                        this._overLimit = false;
                        this._uiStateService = null;
                        this._offlineDelayPromise = null;
                        this._offlineDelayTimerInMS = 5000;
                        this._networkedFeatureStatusTable = null;
                        this._lastNotificationId = "-1";
                        this._uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                        var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                        if (configurationManager.shell.retailExperience || configurationManager.shell.mockNetworkStatusOffline)
                            this._uiStateService.networkStatus = 5;
                        else
                            this._startListening();
                        this._initializeNetworkedFeaturesTable()
                    }
                    Object.defineProperty(NetworkStatusService.prototype, "approachingDataLimit", {
                        get: function() {
                            return this._approachingDataLimit
                        }, set: function(value) {
                                this.updateAndNotify("approachingDataLimit", value)
                            }, enumerable: true, configurable: true
                    });
                    NetworkStatusService.isOnline = function() {
                        var isOnline = true;
                        switch (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).networkStatus) {
                            case null:
                            case 0:
                            case 1:
                            case 2:
                            case 3:
                                isOnline = true;
                                break;
                            case 4:
                            case 5:
                                isOnline = false;
                                break;
                            default:
                                MS.Entertainment.UI.fail("Current network state is not a valid MS.Entertainment.UI.NetworkStatusService.NetworkStatus value.")
                        }
                        return isOnline
                    };
                    NetworkStatusService.prototype.isEnabled = function(feature) {
                        MS.Entertainment.Utilities.validateIsMemberOrThrow(feature, NetworkStatusService.NetworkedFeature);
                        var networkStatus = !this._uiStateService.networkStatus ? 5 : this._uiStateService.networkStatus;
                        var featureStatusArray = this._networkedFeatureStatusTable[feature];
                        MS.Entertainment.UI.assert(featureStatusArray, "Unknown feature");
                        MS.Entertainment.UI.assert(featureStatusArray[networkStatus] !== undefined, "Unknown network status for this feature");
                        return featureStatusArray ? featureStatusArray[networkStatus] : false
                    };
                    NetworkStatusService.prototype.dispose = function() {
                        if (this._networkStatusChangedBinding) {
                            this._networkStatusChangedBinding.cancel();
                            this._networkStatusChangedBinding = null
                        }
                    };
                    NetworkStatusService.prototype._initializeNetworkedFeaturesTable = function() {
                        this._networkedFeatureStatusTable = {};
                        var musicMarketplace = {};
                        musicMarketplace[0] = true;
                        musicMarketplace[1] = true;
                        musicMarketplace[2] = true;
                        musicMarketplace[3] = false;
                        musicMarketplace[4] = false;
                        musicMarketplace[5] = false;
                        this._networkedFeatureStatusTable[0] = musicMarketplace;
                        var videoMarketplace = {};
                        videoMarketplace[0] = true;
                        videoMarketplace[1] = true;
                        videoMarketplace[2] = true;
                        videoMarketplace[3] = false;
                        videoMarketplace[4] = false;
                        videoMarketplace[5] = false;
                        this._networkedFeatureStatusTable[1] = videoMarketplace
                    };
                    NetworkStatusService.prototype._startListening = function() {
                        var _this = this;
                        this._networkStatusChangedBinding = MS.Entertainment.Utilities.addEventHandlers(Windows.Networking.Connectivity.NetworkInformation, {networkstatuschanged: function() {
                                return _this._onNetworkStatusChanged()
                            }});
                        this._updateNetworkStatus(true)
                    };
                    NetworkStatusService.prototype._onNetworkStatusChanged = function() {
                        this._updateNetworkStatus(false)
                    };
                    NetworkStatusService.prototype._updateNetworkStatus = function(immediateStatusUpdate) {
                        var _this = this;
                        if (MS.Entertainment.Utilities.isApp2) {
                            this._uiStateService.networkStatus = 1;
                            return
                        }
                        if (this._offlineDelayPromise) {
                            this._offlineDelayPromise.cancel();
                            this._offlineDelayPromise = null
                        }
                        var internetProfile = null;
                        try {
                            internetProfile = Windows.Networking.Connectivity.NetworkInformation.getInternetConnectionProfile()
                        }
                        catch(e) {
                            MS.Entertainment.UI.fail("Exception accessing: NetworkInformation.getInternetConnectionProfile: " + e.toString())
                        }
                        if (!internetProfile) {
                            if (immediateStatusUpdate)
                                this._uiStateService.networkStatus = 5;
                            else
                                this._offlineDelayPromise = WinJS.Promise.timeout(this._offlineDelayTimerInMS).then(function() {
                                    return _this._uiStateService.networkStatus = 5
                                });
                            return
                        }
                        var stringId = String.id.IDS_NETWORK_STATUS_UNKNOWN;
                        var raiseNotification = false;
                        var isCritical = false;
                        try {
                            switch (internetProfile.getNetworkConnectivityLevel()) {
                                case Windows.Networking.Connectivity.NetworkConnectivityLevel.none:
                                    this._uiStateService.networkStatus = 5;
                                    break;
                                case Windows.Networking.Connectivity.NetworkConnectivityLevel.localAccess:
                                case Windows.Networking.Connectivity.NetworkConnectivityLevel.constrainedInternetAccess:
                                    this._uiStateService.networkStatus = 4;
                                    stringId = String.id.IDS_NETWORK_STATUS_LOCAL_ONLY;
                                    raiseNotification = false;
                                    break;
                                case Windows.Networking.Connectivity.NetworkConnectivityLevel.internetAccess:
                                    var connectionCost = internetProfile.getConnectionCost();
                                    try {
                                        this.approachingDataLimit = connectionCost.approachingDataLimit;
                                        switch (connectionCost.networkCostType) {
                                            case Windows.Networking.Connectivity.NetworkCostType.unknown:
                                            case Windows.Networking.Connectivity.NetworkCostType.unrestricted:
                                                this._uiStateService.networkStatus = 1;
                                                break;
                                            case Windows.Networking.Connectivity.NetworkCostType.fixed:
                                            case Windows.Networking.Connectivity.NetworkCostType.variable:
                                                if (connectionCost.overDataLimit) {
                                                    this._uiStateService.networkStatus = 3;
                                                    stringId = String.id.IDS_NETWORK_STATUS_OVER_THE_LIMIT;
                                                    raiseNotification = true;
                                                    isCritical = true
                                                }
                                                else if (connectionCost.roaming) {
                                                    this._uiStateService.networkStatus = 3;
                                                    stringId = String.id.IDS_NETWORK_STATUS_ROAMING;
                                                    raiseNotification = true;
                                                    isCritical = true
                                                }
                                                else {
                                                    this._uiStateService.networkStatus = 2;
                                                    stringId = String.id.IDS_NETWORK_STATUS_THROTTLED;
                                                    raiseNotification = true
                                                }
                                                break;
                                            default:
                                                MS.Entertainment.UI.assert(false, "Unknown ConnectivityLevel");
                                                this._uiStateService.networkStatus = 0;
                                                stringId = String.id.IDS_NETWORK_STATUS_UNKNOWN;
                                                break
                                        }
                                    }
                                    catch(e) {
                                        MS.Entertainment.UI.fail("Exception accessing: networkCostType: " + e.toString());
                                        this._uiStateService.networkStatus = 0;
                                        stringId = String.id.IDS_NETWORK_STATUS_UNKNOWN
                                    }
                                    break
                            }
                        }
                        catch(e) {
                            MS.Entertainment.UI.fail("Exception accessing: Windows.Networking.Connectivity.NetworkConnectivityLevel: " + e.toString());
                            this._uiStateService.networkStatus = 5
                        }
                        this._sendTelemetry(this._uiStateService.networkStatus);
                        if (raiseNotification)
                            this._sendNotification(stringId, isCritical);
                        else
                            this._lastNotificationId = stringId
                    };
                    NetworkStatusService.prototype._sendTelemetry = function(status) {
                        var telemetryParameterArray = [{
                                    parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.NetworkStatus, parameterValue: NetworkStatusService.NetworkStatus[status]
                                }];
                        MS.Entertainment.Utilities.Telemetry.logTelemetryEventWithParameterArray(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.NetworkService, telemetryParameterArray)
                    };
                    NetworkStatusService.prototype._sendNotification = function(stringId, isCritical) {
                        if (stringId === this._lastNotificationId)
                            return;
                        var appNotificationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appNotification);
                        appNotificationService.removeNotificationByCategory(NetworkStatusService._notificationCategory);
                        this._lastNotificationId = stringId;
                        appNotificationService.send(new MS.Entertainment.UI.Notification({
                            notificationType: isCritical ? MS.Entertainment.UI.Notification.Type.Critical : MS.Entertainment.UI.Notification.Type.Informational, title: String.load(String.id.IDS_NETWORK_STATUS_HEADER), subTitle: String.load(stringId), moreDetails: null, icon: MS.Entertainment.UI.Icon.inlineStreaming, action: null, secondaryActions: null, category: NetworkStatusService._notificationCategory, isPersistent: !!isCritical
                        }))
                    };
                    NetworkStatusService._notificationCategory = "networkStatus";
                    return NetworkStatusService
                })(MS.Entertainment.UI.Framework.ObservableBase);
            UI.NetworkStatusService = NetworkStatusService;
            (function(NetworkStatusService) {
                (function(NetworkStatus) {
                    NetworkStatus[NetworkStatus["unknown"] = 0] = "unknown";
                    NetworkStatus[NetworkStatus["unrestricted"] = 1] = "unrestricted";
                    NetworkStatus[NetworkStatus["throttled"] = 2] = "throttled";
                    NetworkStatus[NetworkStatus["onDemand"] = 3] = "onDemand";
                    NetworkStatus[NetworkStatus["localOnly"] = 4] = "localOnly";
                    NetworkStatus[NetworkStatus["none"] = 5] = "none"
                })(NetworkStatusService.NetworkStatus || (NetworkStatusService.NetworkStatus = {}));
                var NetworkStatus = NetworkStatusService.NetworkStatus;
                (function(NetworkedFeature) {
                    NetworkedFeature[NetworkedFeature["musicMarketplace"] = 0] = "musicMarketplace";
                    NetworkedFeature[NetworkedFeature["videoMarketplace"] = 1] = "videoMarketplace"
                })(NetworkStatusService.NetworkedFeature || (NetworkStatusService.NetworkedFeature = {}));
                var NetworkedFeature = NetworkStatusService.NetworkedFeature
            })(UI.NetworkStatusService || (UI.NetworkStatusService = {}));
            var NetworkStatusService = UI.NetworkStatusService;
            MS.Entertainment.ServiceLocator.register(MS.Entertainment.Services.networkStatus, function() {
                return new NetworkStatusService
            })
        })(Entertainment.UI || (Entertainment.UI = {}));
        var UI = Entertainment.UI
    })(MS.Entertainment || (MS.Entertainment = {}));
    var Entertainment = MS.Entertainment
})(MS || (MS = {}))
