﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
var __extends = this.__extends || function(d, b) {
        for (var p in b)
            if (b.hasOwnProperty(p))
                d[p] = b[p];
        function __() {
            this.constructor = d
        }
        __.prototype = b.prototype;
        d.prototype = new __
    };
var MS;
(function(MS) {
    (function(Entertainment) {
        (function(UI) {
            var BasePlaybackErrorDisplayServiceImpl = (function() {
                    function BasePlaybackErrorDisplayServiceImpl(){}
                    BasePlaybackErrorDisplayServiceImpl.prototype.postErrorNavigationHandler = function(errorCode, isCritical){};
                    return BasePlaybackErrorDisplayServiceImpl
                })();
            UI.BasePlaybackErrorDisplayServiceImpl = BasePlaybackErrorDisplayServiceImpl;
            var VideoPlaybackErrorDisplayServiceImpl = (function(_super) {
                    __extends(VideoPlaybackErrorDisplayServiceImpl, _super);
                    function VideoPlaybackErrorDisplayServiceImpl() {
                        _super.apply(this, arguments)
                    }
                    VideoPlaybackErrorDisplayServiceImpl.prototype.postErrorNavigationHandler = function(errorCode, isCritical) {
                        try {
                            var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                            if (navigationService && navigationService.currentPage.iaNode.moniker === MS.Entertainment.UI.Monikers.fullScreenNowPlaying)
                                navigationService.navigateBack()
                        }
                        catch(exception) {}
                    };
                    return VideoPlaybackErrorDisplayServiceImpl
                })(BasePlaybackErrorDisplayServiceImpl);
            UI.VideoPlaybackErrorDisplayServiceImpl = VideoPlaybackErrorDisplayServiceImpl;
            function createPlaybackErrorDisplayServiceImpl() {
                if (MS.Entertainment.Utilities.isVideoApp)
                    return new VideoPlaybackErrorDisplayServiceImpl;
                return new BasePlaybackErrorDisplayServiceImpl
            }
            var PlaybackErrorDisplayService = (function() {
                    function PlaybackErrorDisplayService() {
                        this._playbackSession = null;
                        this._sessionMgr = null;
                        this._sessionEventHandlers = null;
                        this._uiStateService = null;
                        this._primarySessionIdBinding = null;
                        this.suppressNextPlaybackErrorDialog = false;
                        this.showDialogForNonCriticalErrors = false;
                        this._impl = createPlaybackErrorDisplayServiceImpl()
                    }
                    PlaybackErrorDisplayService.prototype.initialize = function() {
                        var _this = this;
                        this._uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                        this._sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                        this._primarySessionIdBinding = this._uiStateService.bind("primarySessionId", function() {
                            _this.playbackSession = _this._sessionMgr.primarySession
                        })
                    };
                    PlaybackErrorDisplayService.prototype.unload = function() {
                        this._detachBindings();
                        if (this._primarySessionIdBinding) {
                            this._primarySessionIdBinding.cancel();
                            this._primarySessionIdBinding = null
                        }
                    };
                    Object.defineProperty(PlaybackErrorDisplayService.prototype, "playbackSession", {
                        get: function() {
                            return this._playbackSession
                        }, set: function(newValue) {
                                this._playbackSession = newValue;
                                this._detachBindings();
                                this._sessionEventHandlers = MS.Entertainment.Utilities.addEvents(this.playbackSession, {errorChanged: this._errorStateChanged.bind(this)})
                            }, enumerable: true, configurable: true
                    });
                    PlaybackErrorDisplayService.prototype._errorStateChanged = function(e) {
                        var newVal = e.detail.newValue;
                        if (newVal)
                            this._handlePlaybackError(newVal)
                    };
                    PlaybackErrorDisplayService.prototype._detachBindings = function() {
                        if (this._sessionEventHandlers) {
                            this._sessionEventHandlers.cancel();
                            this._sessionEventHandlers = null
                        }
                    };
                    PlaybackErrorDisplayService.prototype._handlePlaybackError = function(errorDescriptor) {
                        if (this.playbackSession.sessionId === MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying) {
                            var error = errorDescriptor;
                            var errorCode = MS.Entertainment.Platform.Playback._mapMediaElementErrorCodes(error.code, error.msExtendedCode);
                            var isCritical = errorDescriptor.isCritical;
                            this._displayError(errorCode, isCritical)
                        }
                    };
                    PlaybackErrorDisplayService.prototype._displayError = function(errorCode, isCritical) {
                        if (!this.suppressNextPlaybackErrorDialog && !this._shouldIgnoreThisError(errorCode))
                            MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_PLAYBACK_ERROR_MESSAGE_TITLE), errorCode).done(this._impl.postErrorNavigationHandler.bind(this._impl, errorCode, isCritical));
                        this.suppressNextPlaybackErrorDialog = false
                    };
                    PlaybackErrorDisplayService.prototype._shouldIgnoreThisError = function(errorCode) {
                        var shouldIgnoreThisError = false;
                        if (MS.Entertainment.Utilities.isVideoApp2)
                            if (errorCode === MS.Entertainment.Platform.Playback.Error.X8_E_PLAYBACK_HDMI_OUTPUT_LOST.code)
                                shouldIgnoreThisError = true;
                        return shouldIgnoreThisError
                    };
                    return PlaybackErrorDisplayService
                })();
            UI.PlaybackErrorDisplayService = PlaybackErrorDisplayService;
            var CriticalOnlyPlaybackErrorDisplayService = (function(_super) {
                    __extends(CriticalOnlyPlaybackErrorDisplayService, _super);
                    function CriticalOnlyPlaybackErrorDisplayService() {
                        _super.call(this);
                        this._errorPromise = null
                    }
                    CriticalOnlyPlaybackErrorDisplayService.prototype._displayError = function(errorCode, isCritical) {
                        if (this.playbackSession.sessionId === MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying) {
                            if (isCritical || this.showDialogForNonCriticalErrors) {
                                if (this._errorPromise) {
                                    this._errorPromise.cancel();
                                    this._errorPromise = null
                                }
                                this._errorPromise = WinJS.Promise.timeout(30).then(function showError() {
                                    MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_PLAYBACK_ERROR_MESSAGE_TITLE), errorCode)
                                })
                            }
                            this.showDialogForNonCriticalErrors = false
                        }
                    };
                    return CriticalOnlyPlaybackErrorDisplayService
                })(PlaybackErrorDisplayService);
            UI.CriticalOnlyPlaybackErrorDisplayService = CriticalOnlyPlaybackErrorDisplayService;
            MS.Entertainment.ServiceLocator.register(MS.Entertainment.Services.playbackErrorDisplayService, function createPlaybackErrorDisplayService() {
                if (MS.Entertainment.Utilities.isMusicApp)
                    return new MS.Entertainment.UI.CriticalOnlyPlaybackErrorDisplayService;
                else
                    return new MS.Entertainment.UI.PlaybackErrorDisplayService
            })
        })(Entertainment.UI || (Entertainment.UI = {}));
        var UI = Entertainment.UI
    })(MS.Entertainment || (MS.Entertainment = {}));
    var Entertainment = MS.Entertainment
})(MS || (MS = {}))
