﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
var MS;
(function(MS) {
    (function(Entertainment) {
        (function(Utilities) {
            var PluralizationRule = (function() {
                    function PluralizationRule(captureExpression, resultString) {
                        this._matches = PluralizationRule._createMatchFunction(captureExpression);
                        this._resultString = resultString
                    }
                    PluralizationRule._stringStartsWith = function(str, expression) {
                        return str.length !== expression.length && str.lastIndexOf(expression, 0) === 0
                    };
                    PluralizationRule._countEndsWith = function(count, expression, expressionModuloFactor) {
                        return count !== expression && (count % expressionModuloFactor) === expression
                    };
                    PluralizationRule._createRangeRuleFunction = function(captureExpression, indexOfRange) {
                        var firstNumber = Number(captureExpression.substr(0, indexOfRange));
                        var lastNumber = Number(captureExpression.substr(indexOfRange + 1));
                        MS.Entertainment.Utilities.assert(firstNumber < lastNumber, "Malformed rule detected. First number in a range must be less than the last number.");
                        if (!isNaN(firstNumber) && !isNaN(lastNumber))
                            return function(count) {
                                    return count >= firstNumber && count <= lastNumber
                                };
                        MS.Entertainment.Utilities.fail("Malformed rule detected. One or both ends of a range rule were not valid numbers.", captureExpression);
                        return function(count) {
                                return false
                            }
                    };
                    PluralizationRule._createWildcardRuleFunction = function(captureExpression, indexOfWildcard) {
                        if (captureExpression === "*")
                            return function(count) {
                                    return true
                                };
                        else if (indexOfWildcard === 0) {
                            var captureExpressionEnding = Number(captureExpression.substr(1));
                            if (!isNaN(captureExpressionEnding)) {
                                var expressionModuloFactor = Math.pow(10, captureExpression.length - 1);
                                return function(count) {
                                        return PluralizationRule._countEndsWith(count, captureExpressionEnding, expressionModuloFactor)
                                    }
                            }
                            else
                                MS.Entertainment.Utilities.fail("Malformed rule detected. Wildcard rule must end with a number.")
                        }
                        else if (indexOfWildcard === captureExpression.length - 1) {
                            var captureExpressionBeginning = captureExpression.substr(0, captureExpression.length - 1);
                            return function(count) {
                                    var countText = String.empty + count;
                                    return PluralizationRule._stringStartsWith(countText, captureExpressionBeginning)
                                }
                        }
                        MS.Entertainment.Utilities.fail("Malformed rule detected. Wildcard was found in the middle of the expression.", captureExpression);
                        return function(count) {
                                return false
                            }
                    };
                    PluralizationRule._createMatchFunction = function(captureExpression) {
                        var indexOfRange = captureExpression.indexOf("-");
                        if (indexOfRange >= 0)
                            return PluralizationRule._createRangeRuleFunction(captureExpression, indexOfRange);
                        else {
                            var indexOfWildcard = captureExpression.indexOf("*");
                            if (indexOfWildcard >= 0)
                                return PluralizationRule._createWildcardRuleFunction(captureExpression, indexOfWildcard);
                            else {
                                var numericForm = Number(captureExpression);
                                if (!isNaN(numericForm))
                                    if (numericForm >= 0)
                                        return function(count) {
                                                return count === numericForm
                                            };
                                    else
                                        MS.Entertainment.Utilities.fail("Malformed rule detected. Negative numbers are not supported for pluralization.", captureExpression);
                                else
                                    MS.Entertainment.Utilities.fail("Malformed rule detected.", captureExpression)
                            }
                        }
                        return function(count) {
                                return false
                            }
                    };
                    Object.defineProperty(PluralizationRule.prototype, "matches", {
                        get: function() {
                            return this._matches
                        }, enumerable: true, configurable: true
                    });
                    Object.defineProperty(PluralizationRule.prototype, "resultString", {
                        get: function() {
                            return this._resultString
                        }, enumerable: true, configurable: true
                    });
                    return PluralizationRule
                })();
            Utilities.PluralizationRule = PluralizationRule;
            var Pluralization = (function() {
                    function Pluralization(){}
                    Pluralization._resetState = function() {
                        this._initializedIdVisionCheck = false;
                        this._initializedPseudoLocCheck = false;
                        MS.Entertainment.Utilities.Pluralization._pluralizationRulesets = {}
                    };
                    Pluralization.getPluralizedString = function(stringId, count) {
                        MS.Entertainment.Utilities.Pluralization._pluralizationRulesets = MS.Entertainment.Utilities.Pluralization._pluralizationRulesets || {};
                        if (!MS.Entertainment.Utilities.Pluralization._pluralizationRulesets[stringId]) {
                            var newRuleset = MS.Entertainment.Utilities.Pluralization._createRuleset(stringId);
                            if (!newRuleset || !Array.isArray(newRuleset)) {
                                MS.Entertainment.Utilities.fail("Expected ruleset for this string ID to be a defined array.", stringId);
                                newRuleset = []
                            }
                            MS.Entertainment.Utilities.Pluralization._pluralizationRulesets[stringId] = newRuleset
                        }
                        var resultString = String.empty;
                        var ruleset = MS.Entertainment.Utilities.Pluralization._pluralizationRulesets[stringId];
                        var matchFound = ruleset.some(function(rule) {
                                if (rule.matches(count)) {
                                    resultString = rule.resultString;
                                    return true
                                }
                                return false
                            });
                        if (!matchFound)
                            MS.Entertainment.Utilities.fail("Failed to find a pluralization rule that matches {0} in ruleset {1}.".format(count, stringId));
                        return resultString
                    };
                    Object.defineProperty(Pluralization, "_useIdVision", {
                        get: function() {
                            if (!Pluralization._initializedIdVisionCheck) {
                                var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                                Pluralization.__useIdVision = configurationManager.shell.idVision;
                                Pluralization._initializedIdVisionCheck = true
                            }
                            return Pluralization.__useIdVision
                        }, enumerable: true, configurable: true
                    });
                    Pluralization._createRuleset = function(stringId) {
                        var ruleset = [];
                        var rulesString = String.load(stringId, true);
                        if (rulesString) {
                            var firstPseudoLocSectionLength = 0;
                            var lastPseudoLocSectionLength = 0;
                            var hasPseudoLocSections = false;
                            if (!Pluralization._initializedPseudoLocCheck || Pluralization._isPseudoLoc) {
                                var firstPseudoLocSectionLength = rulesString.indexOf("[[");
                                var lastPseudoLocSectionLength = rulesString.length - rulesString.lastIndexOf("]") - 1;
                                hasPseudoLocSections = (firstPseudoLocSectionLength > 0 || lastPseudoLocSectionLength > 0);
                                if (!Pluralization._initializedPseudoLocCheck)
                                    Pluralization._isPseudoLoc = hasPseudoLocSections;
                                Pluralization._initializedPseudoLocCheck = true
                            }
                            var actualRulesLength = rulesString.length - (firstPseudoLocSectionLength + lastPseudoLocSectionLength);
                            var idVisionId = null;
                            if (Pluralization._useIdVision)
                                idVisionId = "[" + String.idVisionIds[stringId] + "] ";
                            var regexp = /(?:\[\[([0-9\-\*]+)\]([^\]]*?)\])+?/g;
                            var numGroups = 3;
                            var parts;
                            while (parts = regexp.exec(rulesString)) {
                                MS.Entertainment.Utilities.assert(parts.length === numGroups, "Expected one result string for each capture expression.");
                                var captureExpression = parts[1];
                                var resultString = parts[2];
                                if (hasPseudoLocSections) {
                                    var resultPseudoLocRatio = actualRulesLength > 0 ? resultString.length / actualRulesLength : 0;
                                    var beginningPseudoLoc = firstPseudoLocSectionLength > 0 ? rulesString.substr(0, Math.ceil(firstPseudoLocSectionLength * resultPseudoLocRatio)) : String.empty;
                                    var endingPseudoLoc = lastPseudoLocSectionLength > 0 ? rulesString.substr(rulesString.length - Math.ceil(lastPseudoLocSectionLength * resultPseudoLocRatio)) : String.empty;
                                    resultString = beginningPseudoLoc + resultString + endingPseudoLoc
                                }
                                if (idVisionId)
                                    resultString = idVisionId + resultString;
                                ruleset.push(new PluralizationRule(captureExpression, resultString))
                            }
                            MS.Entertainment.Utilities.assert(ruleset.length > 0, "Parsing failure for string " + stringId)
                        }
                        return ruleset
                    };
                    Pluralization._pluralizationRulesets = {};
                    return Pluralization
                })();
            Utilities.Pluralization = Pluralization
        })(Entertainment.Utilities || (Entertainment.Utilities = {}));
        var Utilities = Entertainment.Utilities
    })(MS.Entertainment || (MS.Entertainment = {}));
    var Entertainment = MS.Entertainment
})(MS || (MS = {}))
