﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
var MS;
(function(MS) {
    (function(Entertainment) {
        (function(Framework) {
            (function(ScriptUtilities) {
                function setOptions(target, options, setPublicMembersOnly) {
                    if (!target || !options || typeof options !== "object")
                        return;
                    Object.keys(options).forEach(function(key) {
                        var privateKey = "_" + key;
                        var value = options[key];
                        if (privateKey in target && !setPublicMembersOnly)
                            target[privateKey] = value;
                        else if (key in target)
                            target[key] = value
                    })
                }
                ScriptUtilities.setOptions = setOptions;
                function waitForSnappedIfNeeded(disallowUserDismiss) {
                    var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                    var waitForSnappedPromise = WinJS.Promise.as();
                    if (uiStateService.isSnapped)
                        waitForSnappedPromise = MS.Entertainment.UI.Framework.loadTemplate("/Controls/PleaseResizeDialog.html", "pleaseResizeDialog", true).then(function() {
                            return MS.Entertainment.UI.Controls.PleaseResizeDialog.waitForWindowResize(!!disallowUserDismiss)
                        });
                    return waitForSnappedPromise
                }
                ScriptUtilities.waitForSnappedIfNeeded = waitForSnappedIfNeeded;
                function _getTabStopsForElementsSubtree(element, includeNegativeTabStops) {
                    var tabStops = WinJS.Utilities.query("*", element).filter(function(candidateTabstop) {
                            if ((candidateTabstop.tabIndex < 0) && !includeNegativeTabStops)
                                return false;
                            if (candidateTabstop.hasAttribute("tabindex"))
                                return true;
                            switch (candidateTabstop.tagName.toLocaleLowerCase()) {
                                case"button":
                                    return true;
                                default:
                                    return false
                            }
                        });
                    return tabStops
                }
                function suppressTabbing(element) {
                    _getTabStopsForElementsSubtree(element, false).forEach(function(item) {
                        item.previousTabIndex = item.tabIndex;
                        item.tabIndex = -1
                    })
                }
                ScriptUtilities.suppressTabbing = suppressTabbing;
                function restoreTabbing(element) {
                    _getTabStopsForElementsSubtree(element, true).forEach(function(item) {
                        if (item.previousTabIndex === undefined)
                            return;
                        item.tabIndex = item.previousTabIndex;
                        item.previousTabIndex = undefined
                    })
                }
                ScriptUtilities.restoreTabbing = restoreTabbing
            })(Framework.ScriptUtilities || (Framework.ScriptUtilities = {}));
            var ScriptUtilities = Framework.ScriptUtilities;
            (function(PlayValidation) {
                var MediaPlayValidation = (function() {
                        function MediaPlayValidation(){}
                        MediaPlayValidation.prototype.validatePlayMedia = function(mediaItemToPlay) {
                            var _this = this;
                            return this.getSigningInPromise().then(function() {
                                    return _this.mediaPlaybackPromise(mediaItemToPlay)
                                })
                        };
                        MediaPlayValidation.prototype.getSigningInPromise = function() {
                            var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                            var signedInSignal = new MS.Entertainment.UI.Framework.Signal;
                            var signInBindings;
                            var signInPromise = WinJS.Promise.wrap();
                            var cleanupSignInBindings = function() {
                                    if (!signInBindings)
                                        return;
                                    signInBindings.cancel();
                                    signInBindings = null;
                                    if (signedInSignal)
                                        signedInSignal.complete()
                                };
                            if (signIn.isSigningIn) {
                                signInBindings = WinJS.Binding.bind(signIn, {isSigningIn: cleanupSignInBindings});
                                signInPromise = signedInSignal.promise
                            }
                            return signInPromise
                        };
                        MediaPlayValidation.prototype.mediaPlaybackPromise = function(invokedMedia) {
                            if (!MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.adService))
                                return WinJS.Promise.wrap();
                            var signal = new MS.Entertainment.UI.Framework.Signal;
                            var adService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.adService);
                            adService.isStreamingBlocked(invokedMedia).done(function(signInNeeded) {
                                var dialogDismissedSignal = new MS.Entertainment.UI.Framework.Signal;
                                var dialogDismissed = function() {
                                        dialogDismissedSignal.complete()
                                    };
                                dialogDismissedSignal.promise.then(function() {
                                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                                    return (!signInNeeded || signIn.isSignedIn) ? adService.playVideoAdIfRequired(invokedMedia, null) : false
                                }).then(function(adPlayedIfNeeded) {
                                    if (adPlayedIfNeeded)
                                        signal.complete();
                                    else
                                        signal.promise.cancel()
                                }, function(error) {
                                    MS.Entertainment.ViewModels.fail("playVideoAdIfRequired_failed: " + (error && error.message));
                                    signal.complete()
                                });
                                if (signInNeeded) {
                                    var featureEnablement = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.featureEnablement);
                                    var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                                    if (!featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlayAnonymous))
                                        MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signUp, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], dialogDismissed);
                                    else if (configurationManager.service.lastSignedInUserXuid)
                                        MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_EXISTING_USER_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_EXISTING_USER_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signInNowForExistingUser, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], dialogDismissed);
                                    else
                                        MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_NEW_USER_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_NEW_USER_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signInNowForNewUser, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], dialogDismissed)
                                }
                                else
                                    dialogDismissed()
                            }, function(error) {
                                MS.Entertainment.ViewModels.fail("isStreamingBlocked_failed: " + (error && error.message));
                                signal.error(error)
                            });
                            return signal.promise
                        };
                        return MediaPlayValidation
                    })();
                PlayValidation.MediaPlayValidation = MediaPlayValidation
            })(Framework.PlayValidation || (Framework.PlayValidation = {}));
            var PlayValidation = Framework.PlayValidation
        })(Entertainment.Framework || (Entertainment.Framework = {}));
        var Framework = Entertainment.Framework
    })(MS.Entertainment || (MS.Entertainment = {}));
    var Entertainment = MS.Entertainment
})(MS || (MS = {}))
