﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
(function() {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Framework");
    WinJS.Namespace.define("MS.Entertainment.Framework", {
        UserTimeTelemetryManager: MS.Entertainment.UI.Framework.define(function UserTimeTelemetryManager() {
            this._musicPlaybackTimeTracker = new MS.Entertainment.Framework.PlaybackTimeTracker("Music");
            this._videoPlaybackTimeTracker = new MS.Entertainment.Framework.PlaybackTimeTracker("Video");
            var now = Date.now();
            this._totalTimeCounterStart = now;
            this._snappedModeTimeCounterStart = now;
            this._unsnappedModeTimeCounterStart = now;
            try {
                this._configuration = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                this._userUsageTimeout = this._configuration.telemetry.userTimeEventTimeout
            }
            catch(e) {
                MS.Entertainment.Framework.fail("Failed to access config to get the user time event timeout " + e.toString());
                this._userUsageTimeout = 900000
            }
            if (!MS.Entertainment.Utilities.isTestApp && this._userUsageTimeout && this._userUsageTimeout > 0)
                this._eventFireTimerPromise = WinJS.Promise.timeout(this._userUsageTimeout).then(this._fireTimerTelemetry.bind(this))
        }, {
            _totalTimeCounterStart: 0, _accumulatedTotalTime: 0, _snappedModeTimeCounterStart: 0, _accumulatedSnappedTime: 0, _unsnappedModeTimeCounterStart: 0, _accumulatedUnsnappedTime: 0, _musicPlaybackTimeTracker: null, _videoPlaybackTimeTracker: null, _configuration: null, _startedSnapped: false, _viewModeSwitched: false, _userUsageTimeout: 0, _eventFireTimerPromise: null, _sendDataPoint: function _sendDataPoint() {
                    var totalTime = 0;
                    var snappedTime = 0;
                    var notSnappedTime = 0;
                    var invalidNumbers = false;
                    if (!this._configuration)
                        this._configuration = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                    if (this._configuration.telemetry.timeSpent) {
                        totalTime = this._configuration.telemetry.timeSpent;
                        snappedTime = this._configuration.telemetry.snappedMode;
                        notSnappedTime = this._configuration.telemetry.notSnapped;
                        if ((totalTime < 0) || (snappedTime < 0) || (notSnappedTime < 0))
                            invalidNumbers = true;
                        if (!invalidNumbers) {
                            var telemetryParameterArray = [{
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.TimeSpent, parameterValue: totalTime
                                    }, {
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.SnappedMode, parameterValue: snappedTime
                                    }, {
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.NotSnapped, parameterValue: notSnappedTime
                                    }, {
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MusicPlaybackBackground, parameterValue: this._configuration.telemetry.musicPlaybackBackground
                                    }, {
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MusicPlaybackSnapped, parameterValue: this._configuration.telemetry.musicPlaybackSnapped
                                    }, {
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MusicPlaybackUnsnapped, parameterValue: this._configuration.telemetry.musicPlaybackUnsnapped
                                    }, {
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.VideoPlaybackBackground, parameterValue: this._configuration.telemetry.videoPlaybackBackground
                                    }, {
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.VideoPlaybackSnapped, parameterValue: this._configuration.telemetry.videoPlaybackSnapped
                                    }, {
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.VideoPlaybackUnsnapped, parameterValue: this._configuration.telemetry.videoPlaybackUnsnapped
                                    }];
                            MS.Entertainment.Utilities.Telemetry.logTelemetryEventWithParameterArray(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.X8UsageTime, telemetryParameterArray)
                        }
                        this._configuration.telemetry.timeSpent = 0;
                        this._configuration.telemetry.snappedMode = 0;
                        this._configuration.telemetry.notSnapped = 0;
                        this._configuration.telemetry.musicPlaybackBackground = 0;
                        this._configuration.telemetry.musicPlaybackSnapped = 0;
                        this._configuration.telemetry.musicPlaybackUnsnapped = 0;
                        this._configuration.telemetry.videoPlaybackBackground = 0;
                        this._configuration.telemetry.videoPlaybackSnapped = 0;
                        this._configuration.telemetry.videoPlaybackUnsnapped = 0
                    }
                    if (!this._eventFireTimerPromise)
                        if (this._userUsageTimeout && this._userUsageTimeout > 0)
                            this._eventFireTimerPromise = WinJS.Promise.timeout(this._userUsageTimeout).then(this._fireTimerTelemetry.bind(this))
                }, appActivated: function appActivated() {
                    this._sendDataPoint();
                    this._startedSnapped = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isSnapped
                }, appResumed: function appResumed() {
                    var now = Date.now();
                    this._sendDataPoint();
                    this._totalTimeCounterStart = now;
                    this._accumulatedTotalTime = 0;
                    this._snappedModeTimeCounterStart = now;
                    this._accumulatedSnappedTime = 0;
                    this._unsnappedModeTimeCounterStart = now;
                    this._accumulatedUnsnappedTime = 0;
                    this._viewModeSwitched = false;
                    this._startedSnapped = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isSnapped;
                    var sessionManager = MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.sessionManager);
                    if (sessionManager) {
                        var currentSession = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager).nowPlayingSession;
                        if (currentSession && currentSession.currentMedia && currentSession.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.playing)
                            if (MS.Entertainment.Platform.PlaybackHelpers.isVideo(currentSession.currentMedia))
                                this._videoPlaybackTimeTracker.playbackStart();
                            else if (MS.Entertainment.Platform.PlaybackHelpers.isMusicOrMusicVideo(currentSession.currentMedia))
                                this._musicPlaybackTimeTracker.playbackStart()
                    }
                }, _fireTimerTelemetry: function _fireTimerTelemetry() {
                    this.appSuspended();
                    this.appResumed()
                }, appSuspended: function appSuspended() {
                    if (!this._configuration)
                        this._configuration = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                    var backgroundTime = 0;
                    this._musicPlaybackTimeTracker.playbackStop();
                    this._videoPlaybackTimeTracker.playbackStop();
                    if (this._eventFireTimerPromise) {
                        this._eventFireTimerPromise.cancel();
                        this._eventFireTimerPromise = null
                    }
                    if (MS.Entertainment.Utilities.isMusicApp)
                        backgroundTime = this._musicPlaybackTimeTracker.accumulatedBackgroundPlaybackTime;
                    this._musicPlaybackTimeTracker.dumpDataPoints();
                    this._videoPlaybackTimeTracker.dumpDataPoints();
                    var accumulatedTotalTimeChange = Date.now() - this._totalTimeCounterStart;
                    this._accumulatedTotalTime += accumulatedTotalTimeChange;
                    this._accumulatedUnsnappedTime += backgroundTime;
                    try
                    {
                        if (this._viewModeSwitched) {
                            this._fixCurrentAccumulatedTime();
                            this._configuration.telemetry.snappedMode = this._accumulatedSnappedTime;
                            this._configuration.telemetry.notSnapped = this._accumulatedUnsnappedTime
                        }
                        else if (this._startedSnapped) {
                            this._configuration.telemetry.snappedMode = this._accumulatedTotalTime / 1000;
                            this._configuration.telemetry.notSnapped = "0"
                        }
                        else {
                            this._configuration.telemetry.snappedMode = "0";
                            this._configuration.telemetry.notSnapped = this._accumulatedTotalTime / 1000
                        }
                        this._configuration.telemetry.timeSpent = this._accumulatedTotalTime / 1000
                    }
                    catch(exception) {
                        MS.Entertainment.Framework.fail("exception in suspend " + exception.toString())
                    }
                }, _fixCurrentAccumulatedTime: function _fixCurrentAccumulatedTime() {
                    var now = Date.now();
                    if (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isSnapped) {
                        var accumulatedSnappedChange = now - this._snappedModeTimeCounterStart;
                        this._accumulatedSnappedTime += (accumulatedSnappedChange / 1000)
                    }
                    else {
                        var accumulatedUnsnappedChange = now - this._unsnappedModeTimeCounterStart;
                        this._accumulatedUnsnappedTime += (accumulatedUnsnappedChange / 1000)
                    }
                }, switchedToSnappedMode: function switchedToSnappedMode() {
                    var now = Date.now();
                    var accumulatedUnsnappedChange = now - this._unsnappedModeTimeCounterStart;
                    this._accumulatedUnsnappedTime += (accumulatedUnsnappedChange / 1000);
                    this._unsnappedModeTimeCounterStart = now;
                    this._snappedModeTimeCounterStart = now;
                    this._musicPlaybackTimeTracker.snappedModeChanged();
                    this._videoPlaybackTimeTracker.snappedModeChanged();
                    this._viewModeSwitched = true
                }, switchedToUnsnappedMode: function switchedToUnsnappedMode() {
                    var now = Date.now();
                    var accumulatedSnappedChange = now - this._snappedModeTimeCounterStart;
                    this._accumulatedSnappedTime += (accumulatedSnappedChange / 1000);
                    this._snappedModeTimeCounterStart = now;
                    this._unsnappedModeTimeCounterStart = now;
                    this._musicPlaybackTimeTracker.snappedModeChanged();
                    this._videoPlaybackTimeTracker.snappedModeChanged();
                    this._viewModeSwitched = true
                }, appVisibilityChanged: function appVisibilityChanged() {
                    if (!MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isAppVisible) {
                        var accumulatedTotalTimeChange = Date.now() - this._totalTimeCounterStart;
                        this._accumulatedTotalTime += accumulatedTotalTimeChange;
                        if (this._viewModeSwitched)
                            this._fixCurrentAccumulatedTime()
                    }
                    else {
                        var now = Date.now();
                        this._snappedModeTimeCounterStart = now;
                        this._unsnappedModeTimeCounterStart = now;
                        this._totalTimeCounterStart = now
                    }
                    this._musicPlaybackTimeTracker.appVisibilityChanged();
                    this._videoPlaybackTimeTracker.appVisibilityChanged()
                }, musicPlaybackStart: function musicPlaybackStart() {
                    this._musicPlaybackTimeTracker.playbackStart()
                }, videoPlaybackStart: function videoPlaybackStart() {
                    this._videoPlaybackTimeTracker.playbackStart()
                }, musicPlaybackStop: function musicPlaybackStop() {
                    this._musicPlaybackTimeTracker.playbackStop()
                }, videoPlaybackStop: function videoPlaybackStop() {
                    this._videoPlaybackTimeTracker.playbackStop()
                }
        }, null), PlaybackTimeTracker: MS.Entertainment.UI.Framework.define(function PlaybackTimeTracker(name) {
                this._name = name
            }, {
                accumulatedUnsnappedPlaybackTime: 0, accumulatedSnappedPlaybackTime: 0, accumulatedBackgroundPlaybackTime: 0, _playbackTimeCounterStart: 0, _isSnapped: false, _isVisible: false, _name: "", _configuration: null, _accumulatedBackgroundConfig: null, _accumulatedSnappedConfig: null, _accumulatedUnsnappedConfig: null, lastStateChange: 0, currentlyPlaying: false, _sleepTimeFudgeFactor: 1500000, snappedModeChanged: function switchedSnappedMode() {
                        if (this._playbackTimeCounterStart !== 0)
                            if (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isSnapped !== this._isSnapped) {
                                this.playbackStop();
                                this.playbackStart()
                            }
                    }, appVisibilityChanged: function appVisibilityChanged() {
                        if (this._playbackTimeCounterStart !== 0)
                            if (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isAppVisible !== this._isVisible) {
                                this.playbackStop();
                                this.playbackStart()
                            }
                    }, playbackStart: function playbackStart() {
                        this.playbackStop();
                        this._playbackTimeCounterStart = Date.now();
                        this._isSnapped = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isSnapped;
                        this._isVisible = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isAppVisible
                    }, playbackStop: function playbackStop() {
                        var now = Date.now();
                        if (this._playbackTimeCounterStart !== 0) {
                            if (this._isVisible)
                                if (this._isSnapped)
                                    this.accumulatedSnappedPlaybackTime += (now - this._playbackTimeCounterStart) / 1000;
                                else
                                    this.accumulatedUnsnappedPlaybackTime += (now - this._playbackTimeCounterStart) / 1000;
                            else if (this._name === "Music") {
                                if (now - this.lastStateChange <= this._sleepTimeFudgeFactor)
                                    this.accumulatedBackgroundPlaybackTime += (now - this._playbackTimeCounterStart) / 1000
                            }
                            else
                                this.accumulatedBackgroundPlaybackTime += (now - this._playbackTimeCounterStart) / 1000;
                            this._playbackTimeCounterStart = 0;
                            this.lastStateChange = now
                        }
                    }, dumpDataPoints: function dumpDataPoints(dataPoint) {
                        try {
                            if (!this._configuration)
                                this._configuration = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                            if (this._name === "Music") {
                                this._configuration.telemetry.musicPlaybackBackground = this.accumulatedBackgroundPlaybackTime;
                                this._configuration.telemetry.musicPlaybackSnapped = this.accumulatedSnappedPlaybackTime;
                                this._configuration.telemetry.musicPlaybackUnsnapped = this.accumulatedUnsnappedPlaybackTime
                            }
                            else if (this._name === "Video") {
                                this._configuration.telemetry.videoPlaybackBackground = this.accumulatedBackgroundPlaybackTime;
                                this._configuration.telemetry.videoPlaybackSnapped = this.accumulatedSnappedPlaybackTime;
                                this._configuration.telemetry.videoPlaybackUnsnapped = this.accumulatedUnsnappedPlaybackTime
                            }
                            this.accumulatedBackgroundPlaybackTime = 0;
                            this.accumulatedSnappedPlaybackTime = 0;
                            this.accumulatedUnsnappedPlaybackTime = 0
                        }
                        catch(error) {}
                    }
            }, null)
    });
    MS.Entertainment.ServiceLocator.register(MS.Entertainment.Services.userTimeTelemetryManager, function getUserTimeTelemetryManager() {
        return new MS.Entertainment.Framework.UserTimeTelemetryManager
    }, true);
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.userTimeTelemetryManager)
})()
