﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
(function() {
    "use strict";
    scriptValidator();
    WinJS.Namespace.define("MS.Entertainment.Framework", {
        KeyboardInteractionListener: MS.Entertainment.UI.Framework.define(function KeyboardInteractionListener() {
            var keyboardHandler = this._onKeyDown.bind(this);
            var mouseHandler = this._onMouseDown.bind(this);
            var focusInHandler = this._handleFocusIn.bind(this);
            var focusOutHandler = this._handleFocusOut.bind(this);
            var keyDownHandler = this.handleKeyDown.bind(this);
            var keyUpHandler = this._handleKeyUp.bind(this);
            var pointerDownHandler = this._handlePointerDown.bind(this);
            var pointerUpHandler = this._handlePointerUp.bind(this);
            var pointOutHandler = this._handlePointerOut.bind(this);
            var gestureEngagedHandler = this._gestureEngagedHandler.bind(this);
            var gestureDisengagedHandler = this._gestureDisengagedHandler.bind(this);
            MS.Entertainment.Framework.KeyboardInteractionListener.showKeyboardFocus = MS.Entertainment.Utilities.isApp2;
            document.addEventListener("keydown", keyboardHandler, true);
            document.addEventListener("MSPointerDown", mouseHandler, false);
            document.addEventListener("pointerdown", mouseHandler, false);
            if (MS.Entertainment.Utilities.isApp2) {
                document.addEventListener("MSPointerMove", mouseHandler, false);
                document.addEventListener("MSPointerHover", mouseHandler, false);
                document.addEventListener("pointermove", mouseHandler, false);
                document.addEventListener("pointerhover", mouseHandler, false);
                document.addEventListener("keydown", keyDownHandler, true);
                document.addEventListener("keyup", keyUpHandler, true);
                document.addEventListener("MSPointerDown", pointerDownHandler, false);
                document.addEventListener("MSPointerUp", pointerUpHandler, false);
                document.addEventListener("MSPointerOut", pointOutHandler, false);
                window.addEventListener("focusin", focusInHandler, false);
                window.addEventListener("focusout", focusOutHandler, false);
                window.addEventListener("gestureengaged", gestureEngagedHandler, false);
                window.addEventListener("gesturedisengaged", gestureDisengagedHandler, false);
                this._documentLastHadFocus = document.hasFocus()
            }
            if (WinJS.Utilities.hasWinRT) {
                var commandUI = Windows.UI.Input.EdgeGesture.getForCurrentView();
                commandUI.addEventListener("completed", this._onAppBarGesture.bind(this))
            }
        }, {
            _keyboardStyleSet: false, _mouseStyleSet: false, _mouseHoverTimeout: null, _mouseHoverResetTime: 10000, _documentLastHadFocus: false, _onKeyDown: function KeyboardInteractionListener_onKeyDown(evt) {
                    if (!this._keyboardStyleSet && MS.Entertainment.Framework.KeyboardInteractionListener._isKeyboardNavigationEvent(evt))
                        this._setKeyboardMode(evt)
                }, _setKeyboardMode: function _setKeyboardMode(evt) {
                    this._keyboardStyleSet = true;
                    this._mouseStyleSet = false;
                    WinJS.Utilities.addClass(document.body, MS.Entertainment.Framework.KeyboardInteractionListener.keyboardFocusClassName);
                    WinJS.Utilities.removeClass(document.body, MS.Entertainment.Framework.KeyboardInteractionListener.mouseHoverClassName);
                    MS.Entertainment.Framework.KeyboardInteractionListener.showKeyboardFocus = true;
                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).nuiEngaged = false
                }, _onAppBarGesture: function _onAppBarGesture(evt) {
                    if (evt && evt.kind === Windows.UI.Input.EdgeGestureKind.keyboard) {
                        this._keyboardStyleSet = true;
                        WinJS.Utilities.addClass(document.body, MS.Entertainment.Framework.KeyboardInteractionListener.keyboardFocusClassName);
                        WinJS.Utilities.removeClass(document.body, MS.Entertainment.Framework.KeyboardInteractionListener.mouseHoverClassName);
                        MS.Entertainment.Framework.KeyboardInteractionListener.showKeyboardFocus = true
                    }
                }, _onMouseDown: function KeyboardInteractionListener_onMouseDown(evt) {
                    if (!this._mouseStyleSet) {
                        this._keyboardStyleSet = false;
                        this._mouseStyleSet = true;
                        WinJS.Utilities.removeClass(document.body, MS.Entertainment.Framework.KeyboardInteractionListener.keyboardFocusClassName);
                        WinJS.Utilities.addClass(document.body, MS.Entertainment.Framework.KeyboardInteractionListener.mouseHoverClassName);
                        if (MS.Entertainment.Utilities.isApp2) {
                            if (this._mouseHoverTimeout)
                                this._mouseHoverTimeout.cancel();
                            this._mouseHoverTimeout = WinJS.Promise.timeout(this._mouseHoverResetTime).then(this._setKeyboardMode.bind(this))
                        }
                    }
                    MS.Entertainment.Framework.KeyboardInteractionListener.showKeyboardFocus = false;
                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).nuiEngaged = true
                }, _handleFocusIn: function _handleFocusIn(evt) {
                    var documentHasFocus = document.hasFocus();
                    if (documentHasFocus && !this._documentLastHadFocus) {
                        this._setKeyboardMode();
                        WinJS.Utilities.removeClass(document.body, "win-visiblebutnotfocused");
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).visibleButNotFocused = false
                    }
                    this._documentLastHadFocus = documentHasFocus
                }, _handleFocusOut: function _handleFocusOut(evt) {
                    var documentHasFocus = document.hasFocus();
                    if (!documentHasFocus) {
                        WinJS.Utilities.removeClass(document.body, MS.Entertainment.Framework.KeyboardInteractionListener.keyboardFocusClassName);
                        WinJS.Utilities.removeClass(document.body, MS.Entertainment.Framework.KeyboardInteractionListener.mouseHoverClassName);
                        this._keyboardStyleSet = false;
                        this._mouseStyleSet = false;
                        WinJS.Utilities.addClass(document.body, "win-visiblebutnotfocused");
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).visibleButNotFocused = true
                    }
                    this._documentLastHadFocus = documentHasFocus
                }, handleKeyDown: function handleKeyDown(evt) {
                    if (document.activeElement && (evt.key === "Spacebar" || evt.key === "Enter" || evt.keyCode === WinJS.Utilities.Key.invokeButton) && !evt.repeat)
                        WinJS.Utilities.addClass(document.activeElement, "win-itemcontainer-active")
                }, _handleKeyUp: function _handleKeyUp(evt) {
                    if ((evt.key === "Spacebar" || evt.key === "Enter" || evt.keyCode === WinJS.Utilities.Key.invokeButton) && !evt.repeat)
                        MS.Entertainment.Framework.KeyboardInteractionListener._cleanUpActiveItemContainers()
                }, _handlePointerDown: function _handlePointerDown() {
                    if (document.activeElement)
                        WinJS.Utilities.addClass(document.activeElement, "win-itemcontainer-active")
                }, _handlePointerUp: function _handlePointerUp() {
                    MS.Entertainment.Framework.KeyboardInteractionListener._cleanUpActiveItemContainers()
                }, _handlePointerOut: function _handlePointerOut() {
                    MS.Entertainment.Framework.KeyboardInteractionListener._cleanUpActiveItemContainers()
                }, _gestureEngagedHandler: function _gestureEngagedHandler() {
                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).gestureEngaged = true
                }, _gestureDisengagedHandler: function _gestureDisengagedHandler() {
                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).gestureEngaged = false
                }
        }, {
            keyboardFocusClassName: "showKeyboardFocus", mouseHoverClassName: "showPointerHover", showKeyboardFocus: false, instance: null, _cleanUpActiveItemContainers: function _cleanUpActiveItemContainers() {
                    WinJS.Utilities.query(".win-itemcontainer-active").forEach(function(item) {
                        WinJS.Utilities.removeClass(item, "win-itemcontainer-active")
                    })
                }, _isKeyboardNavigationEvent: function _isKeyboardNavigationEvent(evt) {
                    if (evt.keyCode === WinJS.Utilities.Key.tab)
                        return true;
                    else if (evt.keyCode === WinJS.Utilities.Key.leftArrow || evt.keyCode === WinJS.Utilities.Key.rightArrow || evt.keyCode === WinJS.Utilities.Key.downArrow || evt.keyCode === WinJS.Utilities.Key.upArrow || evt.keyCode === WinJS.Utilities.Key.home || evt.keyCode === WinJS.Utilities.Key.end || evt.keyCode === WinJS.Utilities.Key.pageUp || evt.keyCode === WinJS.Utilities.Key.pageDown || evt.keyCode === WinJS.Utilities.Key.escape || evt.keyCode === WinJS.Utilities.Key.backspace || evt.keyCode === WinJS.Utilities.Key.leftArrow || evt.keyCode === WinJS.Utilities.Key.lArrow || evt.keyCode === WinJS.Utilities.Key.lOtherArrow || evt.keyCode === WinJS.Utilities.Key.rightArrow || evt.keyCode === WinJS.Utilities.Key.rArrow || evt.keyCode === WinJS.Utilities.Key.rOtherArrow || evt.keyCode === WinJS.Utilities.Key.upArrow || evt.keyCode === WinJS.Utilities.Key.uArrow || evt.keyCode === WinJS.Utilities.Key.uOtherArrow || evt.keyCode === WinJS.Utilities.Key.downArrow || evt.keyCode === WinJS.Utilities.Key.dArrow || evt.keyCode === WinJS.Utilities.Key.dOtherArrow)
                        return !MS.Entertainment.Utilities.doesElementSupportKeyboardInput(document.activeElement);
                    return false
                }, init: function KeyboardInteractionListener_init() {
                    if (!MS.Entertainment.Framework.KeyboardInteractionListener.instance)
                        MS.Entertainment.Framework.KeyboardInteractionListener.instance = new MS.Entertainment.Framework.KeyboardInteractionListener
                }
        }), AccUtils: MS.Entertainment.UI.Framework.define(null, null, {
                idCounter: 0, createAriaLinkId: function createAriaLinkId(linkedElement) {
                        if (!linkedElement.id) {
                            linkedElement.id = "accid" + MS.Entertainment.Framework.AccUtils.idCounter;
                            MS.Entertainment.Framework.AccUtils.idCounter++
                        }
                    }, addAriaLink: function addAriaAttribute(element, linkedElementId, attribute) {
                        element.setAttribute(attribute, linkedElementId)
                    }, createAndAddAriaLink: function createAndAddAriaLink(element, linkedElement, attribute) {
                        MS.Entertainment.Framework.AccUtils.createAriaLinkId(linkedElement);
                        MS.Entertainment.Framework.AccUtils.addAriaLink(element, linkedElement.id, attribute)
                    }, checkAndSetAriaAttribute: WinJS.Utilities.markSupportedForProcessing(function checkAndSetAriaAttribute(value, targetElement, targetProperty) {
                        targetProperty = "" + targetProperty;
                        switch (targetProperty) {
                            case"label":
                                MS.Entertainment.Utilities.setAccessibilityText(targetElement, value);
                                break;
                            case"role":
                                value ? targetElement.setAttribute("role", value) : targetElement.removeAttribute("role");
                                break;
                            case"level":
                                value ? targetElement.setAttribute("aria-level", value) : targetElement.removeAttribute("aria-level");
                                break;
                            case"setsize":
                                value || value === 0 ? targetElement.setAttribute("aria-setsize", value) : targetElement.removeAttribute("aria-setsize");
                                break;
                            case"posinset":
                                value || value === 0 ? targetElement.setAttribute("aria-posinset", value) : targetElement.removeAttribute("aria-posinset");
                                break;
                            case"expanded":
                                typeof value === "boolean" ? targetElement.setAttribute("aria-expanded", value) : targetElement.removeAttribute("aria-expanded");
                                break;
                            case"selected":
                                typeof value === "boolean" ? targetElement.setAttribute("aria-selected", value) : targetElement.removeAttribute("aria-selected");
                                break;
                            case"controls":
                                value ? targetElement.setAttribute("aria-controls", value) : targetElement.removeAttribute("aria-controls");
                                break;
                            case"owns":
                                Array.isArray(value) && value.length ? targetElement.setAttribute("aria-owns", value.join(" ")) : targetElement.removeAttribute("aria-owns");
                                break;
                            default:
                                MS.Entertainment.Utilities.assert(false, "Attempted to bind to invalid ARIA attribute: " + targetProperty);
                                break
                        }
                    }), setAriaAttribute: MS.Entertainment.Utilities.weakElementBindingInitializer(function setAriaAttribute(text, targetElement, targetProperty) {
                        if (Array.isArray(targetProperty))
                            targetProperty = targetProperty[0];
                        MS.Entertainment.Framework.AccUtils.checkAndSetAriaAttribute(text, targetElement, targetProperty)
                    }), setAriaAttributeFromStringId: MS.Entertainment.Utilities.weakElementBindingInitializer(function setAriaAttributeFromStringId(stringId, targetElement, targetProperty) {
                        if (Array.isArray(targetProperty))
                            targetProperty = targetProperty[0];
                        MS.Entertainment.Framework.AccUtils.checkAndSetAriaAttribute(stringId && String.load(stringId), targetElement, targetProperty)
                    }), setAriaAttributeFromStaticStringId: WinJS.Utilities.markSupportedForProcessing(function setAriaAttributeFromStaticStringId(sourceObject, staticStringId, targetElement, targetProperty) {
                        var data = String.id[staticStringId[0]];
                        MS.Entertainment.Formatters.assert(String.id[data], "Formatters_formatStringId must be passed a valid string id, received: " + data);
                        if (String.id[data])
                            MS.Entertainment.Framework.AccUtils.checkAndSetAriaAttribute(String.load(data), targetElement, targetProperty)
                    }), setAriaSliderBounds: function setAriaSliderPosition(element, min, max) {
                        if (!element)
                            return;
                        element = element.domElement ? element.domElement : element;
                        if (element) {
                            MS.Entertainment.Utilities.assert(typeof min === "number", "Expected parameter min to be numeric in setAriaSliderBounds.");
                            MS.Entertainment.Utilities.assert(typeof max === "number", "Expected parameter max to be numeric in setAriaSliderBounds.");
                            element.setAttribute("aria-valuemin", min);
                            element.setAttribute("aria-valuemax", max)
                        }
                    }, setAriaSliderPosition: function setAriaSliderPosition(element, value, valueText) {
                        if (!element)
                            return;
                        element = element.domElement ? element.domElement : element;
                        if (element) {
                            MS.Entertainment.Utilities.assert(typeof value === "number", "Expected parameter value to be numeric in setAriaSliderPosition.");
                            element.setAttribute("aria-valuenow", value);
                            if (valueText)
                                element.setAttribute("aria-valuetext", valueText)
                        }
                    }
            })
    })
})()
