﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
var __extends = this.__extends || function(d, b) {
        for (var p in b)
            if (b.hasOwnProperty(p))
                d[p] = b[p];
        function __() {
            this.constructor = d
        }
        __.prototype = b.prototype;
        d.prototype = new __
    };
var MS;
(function(MS) {
    (function(Entertainment) {
        (function(Data) {
            var ObservableArrayVirtualListAdapter = (function(_super) {
                    __extends(ObservableArrayVirtualListAdapter, _super);
                    function ObservableArrayVirtualListAdapter() {
                        _super.call(this);
                        this._virtualList = null;
                        this._disposed = false;
                        this._initialized = false;
                        this._itemFactory = null;
                        this.pagingDelayMS = 250
                    }
                    ObservableArrayVirtualListAdapter.prototype.dispose = function() {
                        if (!this._disposed) {
                            this._clear();
                            this._disposed = true
                        }
                    };
                    ObservableArrayVirtualListAdapter.prototype.initialize = function(virtualList, itemFactory) {
                        MS.Entertainment.assert(virtualList, "virtualList argument expected");
                        MS.Entertainment.assert(itemFactory, "itemFactory argument expected");
                        MS.Entertainment.assert(!this._initialized, "initialize should only be called once!");
                        this._initialized = true;
                        this._itemFactory = itemFactory;
                        this._clear();
                        this._virtualList = virtualList;
                        this._pageInVirtualListData()
                    };
                    ObservableArrayVirtualListAdapter.prototype._pageInVirtualListData = function() {
                        var _this = this;
                        if (this._virtualList && !this._disposed)
                            this._virtualList.itemsFromIndex(this.length).done(function(result) {
                                if (result && result.items && _this._virtualList && !_this._disposed) {
                                    if (!_this.length)
                                        _this._virtualList.setNotificationHandler({
                                            beginNotifications: _this._beginNotifications.bind(_this), endNotifications: _this._endNotifications.bind(_this), invalidateAll: _this._invalidateAll.bind(_this), inserted: _this._inserted.bind(_this), removed: _this._removed.bind(_this), changed: _this._changed.bind(_this), moved: _this._moved.bind(_this)
                                        });
                                    for (var i = result.offset; i < result.items.length; i++)
                                        _this.push(_this._itemFactory(result.items[i].data));
                                    if (result.totalCount > _this.length)
                                        WinJS.Promise.timeout(_this.pagingDelayMS).done(function() {
                                            _this._pageInVirtualListData()
                                        })
                                }
                            }, function(error) {
                                MS.Entertainment.fail("itemsFromIndex failed!" + (error && error.message))
                            })
                    };
                    ObservableArrayVirtualListAdapter.prototype._clear = function() {
                        if (this._virtualList) {
                            this._virtualList.setNotificationHandler(null);
                            this._virtualList = null
                        }
                        if (this.length)
                            this.splice(0, this.length)
                    };
                    ObservableArrayVirtualListAdapter.prototype._beginNotifications = function(){};
                    ObservableArrayVirtualListAdapter.prototype._endNotifications = function(){};
                    ObservableArrayVirtualListAdapter.prototype._invalidateAll = function() {
                        if (!this._disposed) {
                            this.splice(0, this.length);
                            this._pageInVirtualListData()
                        }
                    };
                    ObservableArrayVirtualListAdapter.prototype._inserted = function(item, keyBefore, keyAfter, index) {
                        if (!this._disposed) {
                            MS.Entertainment.assert(index >= 0 && index <= this.length, "index out of bounds");
                            if (index >= 0 && index <= this.length)
                                this.splice(index, 0, this._itemFactory(item.data))
                        }
                    };
                    ObservableArrayVirtualListAdapter.prototype._removed = function(key, index) {
                        if (!this._disposed) {
                            MS.Entertainment.assert(index >= 0 && index < this.length, "index out of bounds");
                            if (index >= 0 && index < this.length)
                                this.splice(index, 1)
                        }
                    };
                    ObservableArrayVirtualListAdapter.prototype._changed = function(newValue, oldValue) {
                        if (!this._disposed) {
                            var index = this._virtualList.indexFromKey(oldValue.key);
                            MS.Entertainment.assert(index >= 0 && index < this.length, "index out of bounds");
                            if (index >= 0 && index < this.length) {
                                var item = this.item(index);
                                item.data = newValue.data
                            }
                        }
                    };
                    ObservableArrayVirtualListAdapter.prototype._moved = function(item, keyBefore, keyAfter, oldIndex, newIndex) {
                        if (!this._disposed) {
                            MS.Entertainment.assert(oldIndex >= 0 && oldIndex < this.length, "oldIndex out of bounds");
                            MS.Entertainment.assert(newIndex >= 0 && newIndex < this.length, "newIndex out of bounds");
                            if (oldIndex >= 0 && oldIndex < this.length && newIndex >= 0 && newIndex < this.length) {
                                this.splice(oldIndex, 1);
                                this.splice(newIndex, 0, item)
                            }
                        }
                    };
                    return ObservableArrayVirtualListAdapter
                })(MS.Entertainment.ObservableArray);
            Data.ObservableArrayVirtualListAdapter = ObservableArrayVirtualListAdapter
        })(Entertainment.Data || (Entertainment.Data = {}));
        var Data = Entertainment.Data
    })(MS.Entertainment || (MS.Entertainment = {}));
    var Entertainment = MS.Entertainment
})(MS || (MS = {}))
