﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Hydrator");
scriptValidator();
(function() {
    "use strict";
    WinJS.Namespace.define("MS.Entertainment.Hydrator", {
        _adjustVideoListItemFactory: function _adjustVideoListItemFactory(list, values) {
            if (!list || !list.itemFactory)
                return;
            var originalFactory = list.itemFactory;
            var itemFactory = function itemFactory(source) {
                    var item = originalFactory(source);
                    if (item && item.data && values)
                        for (var key in values)
                            item.data[key] = values[key];
                    return item
                };
            itemFactory.listItemFactory = true;
            list.setItemFactory(itemFactory);
            return list
        }, sanitizeVideoIds: function sanitizeVideoIds(data, hydrating) {
                if ("canonicalSeriesId" in data && MS.Entertainment.Utilities.isEmptyGuid(data.canonicalSeriesId))
                    data.canonicalSeriesId = null;
                if ("seriesZuneId" in data && (!MS.Entertainment.Utilities.isEmptyGuid(hydrating.seriesZuneId) || MS.Entertainment.Utilities.isEmptyGuid(data.seriesZuneId)))
                    data.seriesZuneId = null;
                if ("seriesId" in data && (!MS.Entertainment.Utilities.isEmptyGuid(hydrating.seriesId) || MS.Entertainment.Utilities.isEmptyGuid(data.seriesId)))
                    data.seriesId = null;
                if ("seriesServiceId" in data && (!MS.Entertainment.Utilities.isEmptyGuid(hydrating.seriesServiceId) || MS.Entertainment.Utilities.isEmptyGuid(data.seriesServiceId)))
                    data.seriesServiceId = null;
                if ("seriesIdType" in data && MS.Entertainment.Utilities.isEmptyGuid(data.seriesServiceId) && MS.Entertainment.Utilities.isEmptyGuid(data.seriesId))
                    data.seriesIdType = null;
                return MS.Entertainment.Hydrator.sanitizeIds(data, hydrating)
            }, _getOwnedVideoQualityFormats: function _getOwnedVideoQualityFormats(mediaItem) {
                if (!mediaItem.inCollection)
                    return mediaItem;
                return MS.Entertainment.ViewModels.SmartBuyStateEngine.getMarketplaceFileAccessAsync(mediaItem).then(function queryMediaStateAsync_complete(stateInfo) {
                        if (!stateInfo || !stateInfo.libraryDefinitions)
                            return mediaItem;
                        var foundFormats = [];
                        var edsResolutionFormat = MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat;
                        if (stateInfo.libraryDefinitions[edsResolutionFormat.hd] || stateInfo.libraryDefinitions[edsResolutionFormat.hd1080p]) {
                            foundFormats.push(edsResolutionFormat.hd);
                            mediaItem.hasHdLibraryDefinition = true
                        }
                        if (stateInfo.libraryDefinitions[edsResolutionFormat.sd] || stateInfo.libraryDefinitions[edsResolutionFormat.xd]) {
                            foundFormats.push(edsResolutionFormat.sd);
                            mediaItem.hasSdLibraryDefinition = true
                        }
                        mediaItem.libraryDefinitions = MS.Entertainment.Data.Factory.arrayJoin(foundFormats);
                        return mediaItem
                    }.bind(this))
            }, _getClosedCaptionFilesFromLibrary: function _getClosedCaptionFilesFromLibrary(mediaItem) {
                var result = mediaItem;
                if (mediaItem && mediaItem.inCollection && !mediaItem.closedCaptionFiles && mediaItem.hasZuneId)
                    result = Microsoft.Entertainment.ClosedCaptionDownloader.getDownloadedClosedCaptionListAsync(mediaItem.zuneId, MS.Entertainment.Utilities.EMPTY_GUID).then(function gotClosedCaptions(data) {
                        return MS.Entertainment.Data.augment(JSON.parse(data), MS.Entertainment.Data.Augmenter.Marketplace.EdsClosedCaptioningFiles).items
                    }, function failedClosedCaptions() {
                        return []
                    }).then(function setCloseCaptions(files) {
                        mediaItem.closedCaptionFiles = files;
                        return mediaItem
                    });
                return WinJS.Promise.as(result)
            }, _batchDetailQueries: function _batchDetailQueries(zuneIds, mediaItemType, batchSize) {
                var batchedDetailQueries = [];
                batchSize = batchSize || 5;
                for (var i = 0; i < zuneIds.length; i += batchSize) {
                    var batchDetailQuery;
                    switch (mediaItemType) {
                        case MS.Entertainment.Platform.PurchaseHelpers.MOVIE_TYPE:
                            batchDetailQuery = new MS.Entertainment.Data.Query.Video.EdsBatchDetailsMovie;
                            break;
                        case MS.Entertainment.Platform.PurchaseHelpers.TV_EPISODE_TYPE:
                            batchDetailQuery = new MS.Entertainment.Data.Query.Video.EdsBatchDetailsTVEpisode;
                            break;
                        case MS.Entertainment.Platform.PurchaseHelpers.TV_SEASON_TYPE:
                            batchDetailQuery = new MS.Entertainment.Data.Query.Video.EdsBatchDetailsTVSeason;
                            break;
                        default:
                            MS.Entertainment.Framework.fail("Unexpected mediaItemType: '{0}'. Only Movie, TVEpisode, and TVSeason media types are supported.".format(mediaItemType));
                            break
                    }
                    if (batchDetailQuery) {
                        var zuneIdsChunk = zuneIds.slice(i, i + batchSize);
                        batchDetailQuery.ids = zuneIdsChunk;
                        var performBatchDetailQuery = batchDetailQuery.execute().then(function(q) {
                                if (q.result.itemsArray && Array.isArray(q.result.itemsArray)) {
                                    q.result.itemsArray.forEach(function(item) {
                                        item.isPartOfBundle = true
                                    });
                                    return WinJS.Promise.wrap(q.result.itemsArray)
                                }
                                return null
                            });
                        batchedDetailQueries.push(performBatchDetailQuery)
                    }
                }
                return batchedDetailQueries
            }, video: function video(hydrating, options) {
                if (hydrating.videoType === Microsoft.Entertainment.Queries.VideoType.tvEpisode)
                    return MS.Entertainment.Hydrator.tvEpisode(hydrating, options);
                else if (hydrating.videoType === Microsoft.Entertainment.Queries.VideoType.movie)
                    return MS.Entertainment.Hydrator.movie(hydrating, options);
                else if (hydrating.videoType === Microsoft.Entertainment.Queries.VideoType.other)
                    return null;
                else {
                    MS.Entertainment.Hydrator.fail("Trying to hydrate an unsupported video type");
                    return WinJS.wrapError(new Error("Trying to hydrate an unsupported video type"))
                }
            }, movie: function movie(hydrating, options) {
                options = options || {};
                var result;
                var ignoreQueryError = function ignoreQueryError(error) {
                        if (!hydrating.inCollection)
                            return WinJS.Promise.wrapError(error);
                        var item = new MS.Entertainment.Data.Augmenter.Marketplace.EdsMovie;
                        item.releaseDate = null;
                        item.videoDefinition = null;
                        item.videoDefinitions = null;
                        item.zuneId = hydrating.zuneId;
                        item.libraryId = hydrating.libraryId;
                        return MS.Entertainment.Hydrator._getClosedCaptionFilesFromLibrary(item)
                    };
                if (!hydrating.hasServiceId)
                    result = null;
                else {
                    var movieQuery = MS.Entertainment.Hydrator.prepareDetailsQuery(new MS.Entertainment.Data.Query.Video.EdsDetailMovie, new MS.Entertainment.Framework.QueryWatcher("MS.Entertainment.Hydrator.movie"), hydrating);
                    result = movieQuery.execute().then(function queryCompleted(q) {
                        var item = q.result.item;
                        if (!item || !item.isValid)
                            return ignoreQueryError(null);
                        return MS.Entertainment.Hydrator._getClosedCaptionFilesFromLibrary(item)
                    }, ignoreQueryError).then(function gotClosedCaptionFiles(item) {
                        item.rightsHydrated = true;
                        return MS.Entertainment.Hydrator.sanitizeVideoIds(item, hydrating)
                    }).then(function haveSanitizeIds(item) {
                        return MS.Entertainment.Hydrator._getOwnedVideoQualityFormats(item)
                    }).then(function queryForPresaleItems(item) {
                        if (item && item.rights) {
                            item.isPresale = false;
                            item.rights.forEach(function forEach(right) {
                                if (!right)
                                    return;
                                if (right.distributionType === MS.Entertainment.Data.Augmenter.Marketplace.edsOfferDistributionType.presale)
                                    if (right.presaleFulfillmentDate) {
                                        var fulfillmentDate = new Date(right.presaleFulfillmentDate);
                                        if (fulfillmentDate > Date.now()) {
                                            item.isPresale = true;
                                            item.presaleFulfillmentDate = right.presaleFulfillmentDate;
                                            return true
                                        }
                                        else
                                            right.distributionType = MS.Entertainment.Data.Augmenter.Marketplace.edsOfferDistributionType.full
                                    }
                                    else {
                                        item.isPresale = true;
                                        return true
                                    }
                            })
                        }
                        return WinJS.Promise.wrap(item)
                    }).then(function getSecondsFromEndForPostroll(item) {
                        var isPostrollCreditsServiceRegistered = MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.postrollCreditsService);
                        if (isPostrollCreditsServiceRegistered) {
                            var postrollCreditsService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.postrollCreditsService);
                            var secondsFromEndForPostrollPromise = postrollCreditsService.secondsFromEndForPostroll(item).then(function setSecondsFromEndForPostroll(secondsFromEndForPostroll) {
                                    item.secondsFromEndForPostroll = secondsFromEndForPostroll;
                                    return WinJS.Promise.wrap(item)
                                });
                            return secondsFromEndForPostrollPromise
                        }
                        else
                            return WinJS.Promise.wrap(item)
                    }).then(function queryForBundledItems(item) {
                        var collectBundledItemsPromise = null;
                        if (item && item.rights)
                            item.rights.some(function forSome(right) {
                                if (!right)
                                    return;
                                if (right.isBundle && right.bundledItems) {
                                    item.isBundle = true;
                                    var zuneIdMap = [];
                                    for (var i = 0; i < right.bundledItems.length; i++) {
                                        var currentZuneId = WinJS.Utilities.getMember("ZuneId", right.bundledItems[i]);
                                        if (currentZuneId)
                                            zuneIdMap[currentZuneId] = i
                                    }
                                    var isMovie = function(element) {
                                            return element && element.MediaItemType && element.MediaItemType === MS.Entertainment.Platform.PurchaseHelpers.MOVIE_TYPE
                                        };
                                    var isTVEpisode = function(element) {
                                            return element && element.MediaItemType && element.MediaItemType === MS.Entertainment.Platform.PurchaseHelpers.TV_EPISODE_TYPE
                                        };
                                    var isTVSeason = function(element) {
                                            return element && element.MediaItemType && element.MediaItemType === MS.Entertainment.Platform.PurchaseHelpers.TV_SEASON_TYPE
                                        };
                                    var mapZuneIds = function(element) {
                                            return element && element.ZuneId
                                        };
                                    var movieZuneIds = right.bundledItems.filter(isMovie).map(mapZuneIds);
                                    var tvEpisodeZuneIds = right.bundledItems.filter(isTVEpisode).map(mapZuneIds);
                                    var tvSeasonZuneIds = right.bundledItems.filter(isTVSeason).map(mapZuneIds);
                                    var allBatchedDetailQueries = MS.Entertainment.Hydrator._batchDetailQueries(movieZuneIds, MS.Entertainment.Platform.PurchaseHelpers.MOVIE_TYPE);
                                    allBatchedDetailQueries = allBatchedDetailQueries.concat(MS.Entertainment.Hydrator._batchDetailQueries(tvEpisodeZuneIds, MS.Entertainment.Platform.PurchaseHelpers.TV_EPISODE_TYPE));
                                    allBatchedDetailQueries = allBatchedDetailQueries.concat(MS.Entertainment.Hydrator._batchDetailQueries(tvSeasonZuneIds, MS.Entertainment.Platform.PurchaseHelpers.TV_SEASON_TYPE));
                                    collectBundledItemsPromise = WinJS.Promise.join(allBatchedDetailQueries).then(function(bundledItemChunks) {
                                        var allBundledItems = [];
                                        if (bundledItemChunks && Array.isArray(bundledItemChunks))
                                            bundledItemChunks.forEach(function(bundledItemChunk) {
                                                if (bundledItemChunk && Array.isArray(bundledItemChunk))
                                                    bundledItemChunk.forEach(function(bundledItem) {
                                                        if (bundledItem && bundledItem.zuneId && zuneIdMap && bundledItem.zuneId in zuneIdMap && zuneIdMap[bundledItem.zuneId] > -1)
                                                            allBundledItems[zuneIdMap[bundledItem.zuneId]] = bundledItem;
                                                        else
                                                            MS.Entertainment.Framework.fail("Couldn't determine the ordering of the bundled item with zuneId: '{0}'.".format(bundledItem.zuneId))
                                                    })
                                            });
                                        return WinJS.Promise.wrap(allBundledItems)
                                    });
                                    return true
                                }
                            });
                        return WinJS.Promise.join({
                                bundledItems: WinJS.Promise.as(collectBundledItemsPromise), item: WinJS.Promise.wrap(item)
                            })
                    }).then(function onBundledItemsHydrated(result) {
                        var item = result.item;
                        if (item.isBundle) {
                            var bundledItems = result.bundledItems || [];
                            item.bundledItems = bundledItems;
                            item.allBundledItemsInCollection = bundledItems.every(function isBundledItemInCollection(bundledItem) {
                                return bundledItem && bundledItem.inCollection
                            })
                        }
                        return WinJS.Promise.wrap(item)
                    })
                }
                return WinJS.Promise.as(result)
            }, tvEpisode: function tvEpisode(hydrating, options) {
                options = options || {};
                var result;
                var ignoreQueryError = function ignoreQueryError(error) {
                        if (!hydrating.inCollection)
                            return WinJS.Promise.wrapError(error);
                        var item = new MS.Entertainment.Data.Augmenter.Marketplace.EdsTVEpisode;
                        item.releaseDate = null;
                        item.videoDefinition = null;
                        item.videoDefinitions = null;
                        item.zuneId = hydrating.zuneId;
                        item.libraryId = hydrating.libraryId;
                        return MS.Entertainment.Hydrator._getClosedCaptionFilesFromLibrary(item)
                    };
                if (!hydrating.hasServiceId)
                    result = null;
                else {
                    var tvEpisodeQuery = MS.Entertainment.Hydrator.prepareDetailsQuery(new MS.Entertainment.Data.Query.Video.EdsDetailTVEpisode, new MS.Entertainment.Framework.QueryWatcher("MS.Entertainment.Hydrator.tvEpisode"), hydrating);
                    result = tvEpisodeQuery.execute().then(function queryCompleted(q) {
                        var item = q.result.item;
                        if (!item || !item.isValid)
                            return ignoreQueryError(null);
                        return MS.Entertainment.Hydrator._getClosedCaptionFilesFromLibrary(item)
                    }, ignoreQueryError).then(function gotClosedCaptionFiles(item) {
                        if (item) {
                            item.rightsHydrated = true;
                            return MS.Entertainment.Hydrator.sanitizeVideoIds(item, hydrating)
                        }
                    }).then(function haveSanitizeIds(item) {
                        return MS.Entertainment.Hydrator._getOwnedVideoQualityFormats(item)
                    }).then(function getSecondsFromEndForPostroll(item) {
                        var isPostrollCreditsServiceRegistered = MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.postrollCreditsService);
                        if (isPostrollCreditsServiceRegistered) {
                            var postrollCreditsService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.postrollCreditsService);
                            var secondsFromEndForPostrollPromise = postrollCreditsService.secondsFromEndForPostroll(item).then(function setSecondsFromEndForPostroll(secondsFromEndForPostroll) {
                                    item.secondsFromEndForPostroll = secondsFromEndForPostroll;
                                    return WinJS.Promise.wrap(item)
                                });
                            return secondsFromEndForPostrollPromise
                        }
                        else
                            return WinJS.Promise.wrap(item)
                    })
                }
                return WinJS.Promise.as(result)
            }, tvSeries: function tvSeries(hydrating, options) {
                options = options || {};
                var result;
                var queryWatcher = new MS.Entertainment.Framework.QueryWatcher("MS.Entertainment.Hydrator.tvSeries");
                var ignoreQueryError = function ignoreQueryError(error) {
                        if (!hydrating.inCollection)
                            return WinJS.Promise.wrapError(error);
                        var item = new MS.Entertainment.Data.Augmenter.Marketplace.Video.EdsTVSeries;
                        item.releaseDate = null;
                        item.videoDefinition = null;
                        item.videoDefinitions = null;
                        return item
                    };
                var tvSeriesQuery = MS.Entertainment.Hydrator.prepareDetailsQuery(new MS.Entertainment.Data.Query.Video.EdsTvSeriesWithSeasons, queryWatcher, hydrating);
                result = tvSeriesQuery.execute().then(function queryCompleted(q) {
                    var item = q.result.item;
                    if (!item || !item.isValid)
                        return ignoreQueryError(null);
                    return MS.Entertainment.Hydrator._getClosedCaptionFilesFromLibrary(item)
                }, ignoreQueryError).then(function gotClosedCaptionFiles(item) {
                    item.rightsHydrated = true;
                    return MS.Entertainment.Hydrator.sanitizeVideoIds(item, hydrating)
                }).then(function haveSanitizeIds(item) {
                    var seasons = item.seasons;
                    if (MS.Entertainment.Data.List.isList(seasons))
                        MS.Entertainment.Hydrator._adjustVideoListItemFactory(seasons, {
                            seriesId: item.zuneId || hydrating.zuneId, seriesIdType: MS.Entertainment.Data.Query.edsIdType.zuneCatalog, seriesZuneId: item.zuneId || hydrating.zuneId, seriesLibraryId: item.libraryId || -1
                        });
                    else
                        item.seasonCount = 0;
                    return item
                }).then(function setSeasonData(item) {
                    if (item && item.libraryId > -1) {
                        var seasonsQuery = new MS.Entertainment.Data.Query.libraryTVSeasons;
                        seasonsQuery.tvSeriesId = item.libraryId;
                        seasonsQuery.sort = Microsoft.Entertainment.Queries.TVSeasonsSortBy.numberAscending;
                        seasonsQuery.isLive = !!options.listenForDBUpdates;
                        queryWatcher.registerQuery(seasonsQuery);
                        return seasonsQuery.execute().then(function librarySeasons(q) {
                                item.librarySeasons = q.result.items;
                                if (seasonsQuery.isLive)
                                    item.liveQuery = seasonsQuery;
                                return item
                            })
                    }
                    else
                        item.librarySeasons = [];
                    return item
                }).then(function mergeSeasonData(item) {
                    var seasonHydrationPromises = [];
                    if (WinJS.Utilities.getMember("seasons.itemsFromIndex", item) && WinJS.Utilities.getMember("librarySeasons.forEach", item)) {
                        var missingLibrarySeasons = [];
                        item.librarySeasons.forEach(function seasonIterator(librarySeason) {
                            var currentLibrarySeason = WinJS.Utilities.getMember("item.data", librarySeason);
                            if (currentLibrarySeason) {
                                var foundLibrarySeason = false;
                                item.seasons.itemsFromIndex(0).then(function scanSeasons(seasons) {
                                    for (var marketplaceIndex = 0; marketplaceIndex < seasons.items.length; marketplaceIndex++) {
                                        var currentMarketplaceSeason = seasons.items[marketplaceIndex].data;
                                        if (currentMarketplaceSeason && (currentLibrarySeason.seasonNumber === currentMarketplaceSeason.seasonNumber)) {
                                            foundLibrarySeason = true;
                                            break
                                        }
                                    }
                                }.bind(this));
                                if (!foundLibrarySeason) {
                                    seasonHydrationPromises.push(currentLibrarySeason.hydrate());
                                    missingLibrarySeasons.push(currentLibrarySeason)
                                }
                            }
                        }.bind(this));
                        if (missingLibrarySeasons && missingLibrarySeasons.length > 0)
                            item.seasons.insertRangeAtEnd(missingLibrarySeasons, {
                                suppressEvents: true, isSourceData: true
                            })
                    }
                    return WinJS.Promise.join(seasonHydrationPromises).then(function seasonsHydrated() {
                            return WinJS.Promise.wrap(item)
                        })
                });
                return WinJS.Promise.as(result)
            }, tvSeason: function tvSeason(hydrating, options) {
                options = options || {};
                var result;
                var queryWatcher = new MS.Entertainment.Framework.QueryWatcher("MS.Entertainment.Hydrator.tvSeason");
                if (!(hydrating.hasServiceId) && !((hydrating.isEditorialSeason || hydrating.isLibrarySeason) && hydrating.seriesId && hydrating.seasonNumber))
                    result = null;
                else {
                    var item;
                    var tvSeriesPromise;
                    if ((hydrating.isEditorialSeason || hydrating.isLibrarySeason) && hydrating.seriesId && hydrating.seasonNumber > -1) {
                        item = new MS.Entertainment.Data.Augmenter.Marketplace.EdsTVSeason;
                        var tvSeries = MS.Entertainment.Hydrator.prepareDetailsQuery(new MS.Entertainment.Data.Query.Video.EdsTvSeriesWithSeasons, queryWatcher, {
                                serviceId: hydrating.seriesId, serviceIdType: hydrating.seriesIdType, seasonNumber: hydrating.seasonNumber
                            });
                        tvSeriesPromise = tvSeries.execute().then(function tvSeriesQuerySuccess(q) {
                            var seasons = q.result && q.result.item && q.result.item.seasons;
                            if (MS.Entertainment.Data.List.isList(seasons))
                                return seasons.forEachAll(function(args) {
                                        if (args.item.data.seasonNumber === hydrating.seasonNumber) {
                                            item.canonicalId = args.item.data.canonicalId;
                                            item.serviceId = args.item.data.serviceId;
                                            item.serviceIdType = args.item.data.serviceIdType;
                                            item.seasonNumber = args.item.data.seasonNumber;
                                            args.stop = true
                                        }
                                    })
                        }, function ignoreQueryError(error) {
                            if (!hydrating.inCollection)
                                return WinJS.Promise.wrapError(error);
                            item.releaseDate = null;
                            item.videoDefinition = null;
                            item.videoDefinitions = null;
                            return item
                        })
                    }
                    result = MS.Entertainment.Utilities.redirectPromise(tvSeriesPromise, item).then(function gotSeasonFromZestSeries(item) {
                        var tvSeasonQuery = MS.Entertainment.Hydrator.prepareDetailsQuery(new MS.Entertainment.Data.Query.Video.EdsTvSeasonWithEpisodes, queryWatcher, item || hydrating);
                        return tvSeasonQuery.execute()
                    }).then(function queryCompleted(q) {
                        var item = q && q.result && q.result.item;
                        if (!item || !item.isValid)
                            return WinJS.Promise.wrapError(new Error("Failed to retrieve a valid tv episode media item from the query."));
                        item.rightsHydrated = true;
                        return MS.Entertainment.ViewModels.MediaItemModel.hydrateLibraryInfoAsync(hydrating).then(function libraryInfoHydrated() {
                                return MS.Entertainment.Hydrator._getClosedCaptionFilesFromLibrary(item)
                            })
                    }, function ignoreQueryError(error) {
                        if (!hydrating.inCollection)
                            return WinJS.Promise.wrapError(error);
                        item.releaseDate = null;
                        item.videoDefinition = null;
                        item.videoDefinitions = null;
                        item.episodesQueryFailed = true;
                        return item
                    }).then(function setLibraryEpisodeData(item) {
                        if (item && hydrating && hydrating.libraryId > -1 && hydrating.seriesLibraryId > -1) {
                            var episodesQuery = new MS.Entertainment.Data.Query.libraryVideoTV;
                            episodesQuery.seriesId = hydrating.seriesLibraryId;
                            episodesQuery.seasonId = hydrating.libraryId;
                            episodesQuery.sort = Microsoft.Entertainment.Queries.VideosSortBy.seriesTitleSeasonNumberEpisodeNumber;
                            episodesQuery.chunkSize = 1000;
                            episodesQuery.isLive = !!options.listenForDBUpdates;
                            queryWatcher.registerQuery(episodesQuery);
                            return episodesQuery.execute().then(function libraryEpisodes(q) {
                                    item.libraryEpisodes = q.result.items;
                                    if (episodesQuery.isLive)
                                        item.liveQuery = episodesQuery;
                                    return item
                                })
                        }
                        else
                            item.libraryEpisodes = [];
                        return item
                    }).then(function mergeLibraryEpisodes(item) {
                        var promise = null;
                        if (item && item.episodes && item.libraryEpisodes)
                            promise = item.episodes.toArrayAll().then(function onEpisodesLoaded(episodes) {
                                var episodeHydrationPromises = [];
                                var missingLibraryEpisodes = [];
                                item.libraryEpisodes.forEach(function episodeIterator(libraryEpisode) {
                                    var currentLibraryEpisode = WinJS.Utilities.getMember("item.data", libraryEpisode);
                                    if (currentLibraryEpisode) {
                                        var foundLibraryEpisode = false;
                                        for (var marketplaceIndex = 0; marketplaceIndex < episodes.length; marketplaceIndex++) {
                                            var currentMarketplaceEpisode = episodes[marketplaceIndex];
                                            if (currentMarketplaceEpisode && (currentLibraryEpisode.serviceId === currentMarketplaceEpisode.serviceId) || (currentLibraryEpisode.seasonNumber === currentMarketplaceEpisode.seasonNumber && currentLibraryEpisode.episodeNumber === currentMarketplaceEpisode.episodeNumber)) {
                                                foundLibraryEpisode = true;
                                                break
                                            }
                                        }
                                        if (!foundLibraryEpisode) {
                                            episodeHydrationPromises.push(currentLibraryEpisode.hydrate());
                                            missingLibraryEpisodes.push(currentLibraryEpisode)
                                        }
                                    }
                                }.bind(this));
                                if (missingLibraryEpisodes && missingLibraryEpisodes.length > 0)
                                    item.episodes.insertRangeAtEnd(missingLibraryEpisodes, {
                                        suppressEvents: true, isSourceData: true
                                    });
                                return WinJS.Promise.join(episodeHydrationPromises)
                            }.bind(this));
                        return WinJS.Promise.as(promise).then(function onMergeComplete() {
                                return WinJS.Promise.wrap(item)
                            })
                    }).then(function gotClosedCaptionFiles(item) {
                        return MS.Entertainment.Hydrator.sanitizeVideoIds(item, hydrating)
                    }).then(function haveSanitizeIds(item) {
                        var episodeHydratePromises = [];
                        var episodes = item.episodes;
                        var forEachAllPromise = WinJS.Promise.as();
                        if (episodes) {
                            item.episodeCount = episodes.count;
                            forEachAllPromise = episodes.forEachAll(function populateData(args) {
                                var episode = args && args.item && args.item.data;
                                if (!episode)
                                    return;
                                episodeHydratePromises.push(MS.Entertainment.ViewModels.MediaItemModel.hydrateLibraryInfoAsync(episode))
                            })
                        }
                        else
                            item.episodeCount = 0;
                        return WinJS.Promise.join({
                                episodes: forEachAllPromise.then(function hydrateEpisodes() {
                                    return WinJS.Promise.join(episodeHydratePromises)
                                }), item: WinJS.Promise.wrap(item)
                            })
                    }).then(function haveHydratedEpisodes(results) {
                        var item = results.item;
                        var episodes = results.episodes.sort(MS.Entertainment.Data.Comparer.createPropertyComparer("episodeNumber"));
                        var firstEpisode = null;
                        episodes.some(function(episode) {
                            if (episode && episode.episodeNumber > 0) {
                                firstEpisode = episode;
                                firstEpisode.hydrate();
                                return true
                            }
                        });
                        return WinJS.Promise.join({
                                firstEpisode: WinJS.Promise.as(firstEpisode), virtualList: MS.Entertainment.Data.VirtualList.wrapArray(episodes), item: item
                            })
                    }).then(function haveEpisodes(results) {
                        var item = results.item;
                        item.episodes = results.virtualList;
                        item.firstEpisode = results.firstEpisode;
                        return WinJS.Promise.wrap(item)
                    })
                }
                return WinJS.Promise.as(result)
            }, contributor: function movie(hydrating, options) {
                options = options || {};
                if (MS.Entertainment.Utilities.isVideoActorDetailsEnabled) {
                    var graceNoteService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.graceNoteService);
                    var result = graceNoteService.contributorSearch(hydrating.name, options.graceNoteImageSize).then(function appendContributorImage(result) {
                            if (result && result.items && result.items.length) {
                                var contributor = result.items[0];
                                if (contributor && contributor.image && contributor.name.toLowerCase() === hydrating.name.toLowerCase()) {
                                    hydrating.imageUri = contributor.image;
                                    hydrating.biography = contributor.biography;
                                    hydrating.biographyLanguage = contributor.biographyLanguage
                                }
                            }
                            if (!hydrating.imageUri)
                                hydrating.imageUri = MS.Entertainment.UI.ImagePaths.genericVideoActorPlaceholder;
                            return WinJS.Promise.wrap(hydrating)
                        }, function contributorSearchFailed() {
                            hydrating.imageUri = MS.Entertainment.UI.ImagePaths.genericVideoActorPlaceholder;
                            return WinJS.Promise.wrap(hydrating)
                        });
                    return WinJS.Promise.as(result)
                }
                else
                    return WinJS.Promise.wrap(hydrating)
            }
    })
})()
