﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    var MSE = WinJS.Namespace.define("MS.Entertainment", null);
    var VER_CLIENTTYPE_STR = "PC/Windows";
    var BROWSE_CHUNK_SIZE = 50;
    var BROWSE_TOP_CHUNK_SIZE = 100;
    var SEARCH_CHUNK_SIZE = 20;
    var EXTRAS_CHUNK_SIZE = 30;
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {errorCodeWrapperQuery: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MSE.derive(MSE.Data.ServiceWrapperQuery, function errorCodeWrapperQuery(errorCode) {
                    this.base();
                    if (errorCode < 0)
                        errorCode += 0xFFFFFFFF + 1;
                    var languageName = MS.Entertainment.Utilities.getResourceLanguage();
                    this.postData = {
                        contentType: MSE.Data.Query.errorCodeWrapperQuery.contentType, data: MSE.Data.Query.errorCodeWrapperQuery.postDataFormat.format(errorCode.toString(16), languageName)
                    };
                    this.requestType = MS.Entertainment.Data.ServiceWrapperQuery.RequestTypes.post
                }, {
                    createResourceURI: function() {
                        return MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_XboxSupport) + "/Services/ErrorCodeLookupService.asmx"
                    }, headers: {SOAPAction: "\"ErrorCodeLookup/LookupFullParam\""}, useCache: false, resultAugmentation: MS.Entertainment.Data.Augmenter.Common.ErrorCode
                }, {
                    postDataFormat: "<?xml version=\"1.0\" encoding=\"utf-8\"?><soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"><soap:Body><LookupFullParam xmlns=\"ErrorCodeLookup\"><errorCode>{0}</errorCode><searchMode>ExactOnly</searchMode><locale>{1}</locale></LookupFullParam></soap:Body></soap:Envelope>", contentType: "text/xml; charset=utf-8"
                })
        })});
    WinJS.Namespace.defineWithParent(MS.Entertainment.Data.Query, "Shared", {SetRichPresence: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MSE.derive(MSE.Data.ServiceWrapperQuery, function setRichPresence(queryData) {
                    this.base();
                    this.postData = {
                        contentType: MS.Entertainment.Data.Query.Shared.SetRichPresence.contentType, data: JSON.stringify(queryData)
                    };
                    this.requestType = MS.Entertainment.Data.ServiceWrapperQuery.RequestTypes.post;
                    this.serviceType = MS.Entertainment.Data.ServiceWrapperQuery.ServiceTypes.json
                }, {
                    getResourceEndpointID: function() {
                        return MS.Entertainment.Endpoint.id.seid_RichPresence
                    }, getResourceEndpoint: function() {
                            return MS.Entertainment.Endpoint.load(this.getResourceEndpointID())
                        }, createResourceURI: function() {
                            var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                            var suffix = MS.Entertainment.Data.Query.Shared.SetRichPresence.resourceSuffix.format(signedInUser.xuid);
                            return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                        }, requireResponseData: false, addAuthenticationHeader: true
                }, {
                    resourceSuffix: "users/xuid({0})/devices/current/titles/current", contentType: "application/json"
                })
        })});
    WinJS.Namespace.defineWithParent(MS.Entertainment.Data.Query, "Shared", {SetRecommendationFeeback: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MSE.derive(MSE.Data.ServiceWrapperQuery, function setRecommendationFeeback(queryData) {
                    this.base();
                    this.postData = {
                        contentType: MS.Entertainment.Data.Query.Shared.SetRecommendationFeeback.contentType, data: JSON.stringify(queryData)
                    };
                    this.addHeader("x-xbl-contract-version", MS.Entertainment.Data.Query.Shared.SetRecommendationFeeback.contractVersion);
                    this.addHeader("Accept", MS.Entertainment.Data.Query.Shared.SetRecommendationFeeback.contentType);
                    this.addHeader("Content-Type", MS.Entertainment.Data.Query.Shared.SetRecommendationFeeback.contentType);
                    this.requestType = MS.Entertainment.Data.ServiceWrapperQuery.RequestTypes.post;
                    this.serviceType = MS.Entertainment.Data.ServiceWrapperQuery.ServiceTypes.json
                }, {
                    getResourceEndpointID: function() {
                        return MS.Entertainment.Endpoint.id.seid_Feedback
                    }, getResourceEndpoint: function() {
                            return MS.Entertainment.Endpoint.load(this.getResourceEndpointID())
                        }, createResourceURI: function() {
                            var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                            var languageName = MS.Entertainment.Utilities.getResourceLanguage();
                            var suffix = MS.Entertainment.Data.Query.Shared.SetRecommendationFeeback.resourceSuffix.format(signedInUser.xuid, languageName);
                            return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                        }, requireResponseData: false, addAuthenticationHeader: true
                }, {
                    resourceSuffix: "xuid({0})/feedback?locale={1}", contractVersion: "1", contentType: "application/json", notInterestedFeedbackValue: "NotInterested"
                })
        })});
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {MovieHub: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MSE.derive(MSE.Data.Query.marketplaceWrapperQuery, null, {
                    createHeaders: function createHeaders() {
                        return MS.Entertainment.Data.Query.createEditorialHeaders()
                    }, getResourceEndpointID: function getResourceEndpointID() {
                            return MS.Entertainment.Endpoint.id.seid_MediaDiscovery
                        }, createResourceURI: function() {
                            var serviceConfig = (new Microsoft.Entertainment.Configuration.ConfigurationManager).service;
                            var uri = MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), serviceConfig.videoAppMovieHubSuffix);
                            return uri
                        }, pluralizers: [MSE.Data.Query.marketplaceWrapperQuery.channelPluralizer, MSE.Data.Query.marketplaceWrapperQuery.slotGroupPluralizer, MSE.Data.Query.marketplaceWrapperQuery.slotPluralizer], resultAugmentation: MSE.Data.Augmenter.Video.MovieHubContent, aggregateChunks: false, supportsContentNotifications: true, queryType: "MovieHub"
                })
        })});
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {MovieHubNewReleasesQuery: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MSE.derive(MS.Entertainment.Data.Query.MovieHub, null, {resultAugmentation: MS.Entertainment.Data.Augmenter.Video.MovieHubNewReleaseContent})
        })});
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {MovieHubFeaturedQuery: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MSE.derive(MS.Entertainment.Data.Query.MovieHub, null, {resultAugmentation: MS.Entertainment.Data.Augmenter.Video.MovieHubFeaturedContent})
        })});
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {TvHub: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MSE.derive(MSE.Data.Query.marketplaceWrapperQuery, null, {
                    createHeaders: function createHeaders() {
                        return MS.Entertainment.Data.Query.createEditorialHeaders()
                    }, getResourceEndpointID: function getResourceEndpointID() {
                            return MS.Entertainment.Endpoint.id.seid_MediaDiscovery
                        }, createResourceURI: function() {
                            var serviceConfig = (new Microsoft.Entertainment.Configuration.ConfigurationManager).service;
                            var uri = MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), serviceConfig.videoAppTvHubSuffix);
                            return uri
                        }, pluralizers: [MSE.Data.Query.marketplaceWrapperQuery.channelPluralizer, MSE.Data.Query.marketplaceWrapperQuery.slotGroupPluralizer, MSE.Data.Query.marketplaceWrapperQuery.slotPluralizer], queryType: "TvHub", resultAugmentation: MSE.Data.Augmenter.Video.TvHubContent, aggregateChunks: false
                })
        })});
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {
        marketplaceOrderBy: {
            downloadRank: "downloadRank", none: "", playRank: "playRank", releaseDate: "releaseDate", rentalRank: "rentalRank", salesRank: "salesRank", title: "title"
        }, marketplaceWrapperQuery: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MSE.derive(MSE.Data.ServiceWrapperQuery, null, {
                        getResourceEndpointID: function getResourceEndpointID() {
                            return MS.Entertainment.Endpoint.id.seid_RootCatalog
                        }, createParameters: function createParameters() {
                                var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                                if (configurationManager.service.enableTimeTravel)
                                    return {instant: (new Microsoft.Entertainment.Configuration.ConfigurationManager).service.timeTravelStartDate};
                                else
                                    return null
                            }
                    }, {
                        entriesPluralizer: "entry", rightsPluralizer: "rights/right", closedCaptionPluralizer: "closedCaptionFiles/closedCaptionFile", clientTypesPluralizer: "clientTypes/clientType", genresPluralizer: "genres/genre", peerGenresPluralizer: "peerGenres/genre", subGenresPluralizer: "subGenres/genre", categoriesPluralizer: "categories/category", imagesPluralizer: "entry/instances/imageInstance", editorialItemsPluralizer: "editorialItems/editorialItem", channelPluralizer: "Content/Channel", slotGroupPluralizer: "Channel/SlotGroup", slotPluralizer: "SlotGroup/Slot", featuresPluralizer: "features/feature", albumsPluralizer: "albums/album", paymentTypesPluralizer: "paymentTypes/paymentType"
                    })
            })
    });
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {BrowseDiscoveryFlexHub: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MSE.derive(MSE.Data.ServiceWrapperQuery, null, {
                    serviceType: MS.Entertainment.Data.ServiceWrapperQuery.ServiceTypes.json, target: "", createResourceURI: function() {
                            Trace.assert(this.target, "BrowseDiscoveryFlexHub: No target defined.");
                            var resourceUri = null;
                            if (MS.Entertainment.Utilities.verifyUrl(this.target))
                                resourceUri = window.encodeURI(this.target);
                            else {
                                Trace.assert(MS.Entertainment.Utilities.isMusicApp, "Only music flexhub queries supported.");
                                var serviceConfig = (new Microsoft.Entertainment.Configuration.ConfigurationManager).service;
                                var suffix = serviceConfig.musicAppSpotlightVersion.split("/");
                                suffix.push(window.encodeURI(this.target));
                                resourceUri = MS.Entertainment.Utilities.UriFactory.create(MS.Entertainment.Endpoint.id.seid_MediaDiscovery, suffix)
                            }
                            return resourceUri
                        }, createHeaders: function createHeaders() {
                            return MS.Entertainment.Data.Query.createEditorialHeaders()
                        }, pluralizers: [MSE.Data.Query.marketplaceWrapperQuery.entriesPluralizer, MSE.Data.Query.marketplaceWrapperQuery.rightsPluralizer, MSE.Data.Query.marketplaceWrapperQuery.clientTypesPluralizer, MSE.Data.Query.marketplaceWrapperQuery.editorialItemsPluralizer], resultAugmentation: MSE.Data.Augmenter.Spotlight.SpotlightContent
                })
        })});
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {prefixQuery: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MSE.derive(MSE.Data.Query.marketplaceWrapperQuery, null, {
                    keyword: "", createResourceURI: function() {
                            return this.getResourceEndpoint()
                        }, createParameters: function createParameters() {
                            return {
                                    prefix: this.keyword, includeTracks: true, includeAlbums: true, includeArtists: true, includeMovies: true, includeTVSeries: true, isActionable: true, clientType: VER_CLIENTTYPE_STR
                                }
                        }, pluralizers: [MSE.Data.Query.marketplaceWrapperQuery.entriesPluralizer], resultAugmentation: MSE.Data.Augmenter.Marketplace.PrefixResult
                })
        })});
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {HubsOverride: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MSE.derive(MSE.Data.ServiceWrapperQuery, null, {
                    getResourceEndpointID: function getResourceEndpointID() {
                        return MS.Entertainment.Endpoint.id.seid_WinPhoneXboxDotCom
                    }, getSpotlightFeedVersionSubPath: function getSpotlightFeedVersionSubPath() {
                            return String.empty
                        }, createResourceURI: function() {
                            return this.getResourceEndpoint() + this.getSpotlightFeedVersionSubPath() + this.appString
                        }, pluralizers: [MSE.Data.Query.marketplaceWrapperQuery.slotGroupPluralizer, MSE.Data.Query.marketplaceWrapperQuery.slotPluralizer], resultAugmentation: MSE.Data.Augmenter.Spotlight.EditorialOverrideContent, supportsContentNotifications: true
                })
        })})
})()
