﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    var MSE = WinJS.Namespace.define("MS.Entertainment", null);
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {
        SpotlightQueryType: {
            SpotlightMusic: "spotlight", SpotlightVideo: "spotlight"
        }, recommendationBrowseFields: [MS.Entertainment.Data.Query.edsFields.allTimeAverageRating, MS.Entertainment.Data.Query.edsFields.explanations, MS.Entertainment.Data.Query.edsFields.id, MS.Entertainment.Data.Query.edsFields.images, MS.Entertainment.Data.Query.edsFields.mediaGroup, MS.Entertainment.Data.Query.edsFields.mediaItemType, MS.Entertainment.Data.Query.edsFields.name, MS.Entertainment.Data.Query.edsFields.vuiDisplayName, MS.Entertainment.Data.Query.edsFields.zuneId], createEditorialHeaders: function createEditorialHeaders() {
                var architectureStringMap = {
                        0: "x86", 5: "arm", 9: "x64"
                    };
                var currentPackage = Windows.ApplicationModel.Package.current;
                var architecture = currentPackage.id.architecture;
                var mappedArchitecture = architectureStringMap[architecture];
                var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                if (configurationManager.service.targetedProgrammingArchitecture)
                    mappedArchitecture = configurationManager.service.targetedProgrammingArchitecture;
                var spotlightHeaders = {
                        "User-Agent": configurationManager.service.targetedProgrammingClientOs, "X-Application": MS.Entertainment.Utilities.isVideoApp ? "Video" : "Music", "X-Membership-Level": configurationManager.service.lastSignedInUserMembership || "Free", "X-Parent-Control": configurationManager.service.lastSignedInUserParentControl.toString(), "X-Client-Version": configurationManager.service.targetedProgrammingClientVersion, "X-Client-Architecture": mappedArchitecture
                    };
                if (MS.Entertainment.Utilities.isMusicApp)
                    spotlightHeaders["X-Music-Subscription"] = configurationManager.service.lastSignedInUserSubscription.toString();
                return spotlightHeaders
            }
    });
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {spotlightContentQuery: MSE.derive(MSE.Data.ServiceWrapperQuery, null, {
            spotlight: String.empty, getResourceEndpointID: function getResourceEndpointID() {
                    return MS.Entertainment.Endpoint.id.seid_CdfXboxLiveDotCom
                }, createHeaders: function createHeaders() {
                    return MS.Entertainment.Data.Query.createEditorialHeaders()
                }, createParameters: function createParameters() {
                    var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                    if (configurationManager.service.enableTimeTravel)
                        return {StartDate: configurationManager.service.timeTravelStartDate};
                    else
                        return null
                }, getSpotlightFeedVersionSubPath: function getSpotlightFeedVersionSubPath() {
                    return String.empty
                }, createResourceURI: function() {
                    return this.getResourceEndpoint() + this.getSpotlightFeedVersionSubPath() + this.spotlight
                }, pluralizers: ["Content/SlotGroup", "SlotGroup/Slot"], resultAugmentation: MSE.Data.Augmenter.Spotlight.SpotlightContent
        })});
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {MusicMediaDiscoverySpotlightQuery: MSE.derive(MSE.Data.Query.spotlightContentQuery, null, {
            serviceType: MS.Entertainment.Data.ServiceWrapperQuery.ServiceTypes.json, spotlight: MSE.Data.Query.SpotlightQueryType.SpotlightMusic, getResourceEndpointID: function getResourceEndpointID() {
                    return MS.Entertainment.Endpoint.id.seid_MediaDiscovery
                }, getSpotlightFeedVersionSubPath: function getSpotlightFeedVersionSubPath() {
                    var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                    return configurationManager.service.musicAppSpotlightVersion
                }, createResourceURI: function createResourceURI() {
                    return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpointID(), this.getSpotlightFeedVersionSubPath() + "spotlight")
                }
        })});
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {videoSpotlightQuery: MSE.derive(MSE.Data.Query.spotlightContentQuery, null, {
            spotlight: MSE.Data.Query.SpotlightQueryType.SpotlightVideo, getResourceEndpointID: function getResourceEndpointID() {
                    return MS.Entertainment.Endpoint.id.seid_MediaDiscovery
                }, getResourceUriSuffix: function getResourceUriSuffix() {
                    var serviceConfig = (new Microsoft.Entertainment.Configuration.ConfigurationManager).service;
                    return serviceConfig.videoAppSpotlightSuffix
                }, createResourceURI: function createResourceURI() {
                    var uri = MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), this.getResourceUriSuffix());
                    return uri
                }, pluralizers: [MSE.Data.Query.marketplaceWrapperQuery.slotGroupPluralizer, MSE.Data.Query.marketplaceWrapperQuery.slotPluralizer], resultAugmentation: MSE.Data.Augmenter.Spotlight.VideoSpotlightContent
        })});
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {
        movieSpotlightQuery: MSE.derive(MSE.Data.Query.videoSpotlightQuery, null, {getResourceUriSuffix: function getResourceUriSuffix() {
                var serviceConfig = (new Microsoft.Entertainment.Configuration.ConfigurationManager).service;
                return serviceConfig.videoAppMovieSpotlightSuffix
            }}), tvSpotlightQuery: MSE.derive(MSE.Data.Query.videoSpotlightQuery, null, {getResourceUriSuffix: function getResourceUriSuffix() {
                    var serviceConfig = (new Microsoft.Entertainment.Configuration.ConfigurationManager).service;
                    return serviceConfig.videoAppTvSpotlightSuffix
                }})
    });
    var xuidTokenTable = {};
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {EdsVideoRecommendations: MSE.derive(MSE.Data.Query.EDSDeviceQuery, null, {
            chunkSize: 20, requestFields: MS.Entertainment.Data.Query.recommendationBrowseFields, useApp2UserIfAvailable: true, includeContentRestrictionHeader: true, enabledImpressionGuid: true, queryType: "videoRecommendations", createResourceURI: function() {
                    var configurationManager;
                    var currentUserXuid;
                    var uri = this.getResourceEndpoint(MS.Entertainment.Data.Query.edsEndpointType.recommendations);
                    var contentRestrictionSuffixString = String.empty;
                    if (MS.Entertainment.Utilities.isVideoApp2) {
                        configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                        currentUserXuid = configurationManager.service.lastSignedInUserXuid || 0;
                        if (!xuidTokenTable[currentUserXuid])
                            xuidTokenTable[currentUserXuid] = MS.Entertainment.Utilities.getSessionUniqueInteger() + 1;
                        if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.contentRestrictionService))
                            contentRestrictionSuffixString = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.contentRestrictionService).getUniqueBrowseFilterRestrictionValue();
                        uri = MS.Entertainment.Utilities.UriFactory.appendQuery(uri, {random: xuidTokenTable[currentUserXuid] + contentRestrictionSuffixString})
                    }
                    return uri
                }, createParameters: function createParameters() {
                    var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                    var movieMarketplaceEnabled = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.moviesMarketplace);
                    var tvMarketplaceEnabled = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.tvMarketplace);
                    var types = [];
                    var typeArray = null;
                    if (movieMarketplaceEnabled && (!this.desiredMediaItemTypes || this.desiredMediaItemTypes === "all" || this.desiredMediaItemTypes === "movies"))
                        types.push(MS.Entertainment.Data.Query.edsMediaType.movie);
                    if (tvMarketplaceEnabled && (!this.desiredMediaItemTypes || this.desiredMediaItemTypes === "all" || this.desiredMediaItemTypes === "tv"))
                        types.push(MS.Entertainment.Data.Query.edsMediaType.tvSeries);
                    typeArray = MS.Entertainment.Data.Query.edsArray(types);
                    return {
                            desiredMediaItemTypes: typeArray, firstPartyOnly: true
                        }
                }, resultAugmentation: MSE.Data.Augmenter.Marketplace.EdsRecommendedVideoList, useCache: false
        })});
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {
        MusicUpdatePanel: MSE.deferredDerive(MS.Entertainment.Data.ServiceWrapperQuery, null, {
            chunked: false, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.IntroPanelResult, createResourceURI: function() {
                    return MS.Entertainment.Utilities.UriFactory.create(MS.Entertainment.Endpoint.id.seid_WinPhoneXboxDotCom, "x8/feeds/1.1/Upgrade-Music")
                }, pluralizers: ["BodyText/p"]
        }), VideoUpdatePanel: MS.Entertainment.deferredDerive(MS.Entertainment.Data.ServiceWrapperQuery, null, {
                chunked: false, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.IntroPanelResult, createResourceURI: function() {
                        return MS.Entertainment.Utilities.UriFactory.create(MS.Entertainment.Endpoint.id.seid_WinPhoneXboxDotCom, "x8/feeds/1.1/Upgrade-Video")
                    }, pluralizers: ["BodyText/p"]
            })
    });
    WinJS.Namespace.defineWithParent(MSE.Data, "Query", {MediaDiscoveryVideoFlexHub: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MSE.derive(MSE.Data.Query.marketplaceWrapperQuery, null, {
                    createHeaders: function createHeaders() {
                        return MS.Entertainment.Data.Query.createEditorialHeaders()
                    }, getResourceEndpointID: function getResourceEndpointID() {
                            return MS.Entertainment.Endpoint.id.seid_MediaDiscovery
                        }, createParameters: function createParameters() {
                            return {}
                        }, createResourceURI: function() {
                            var serviceConfig = (new Microsoft.Entertainment.Configuration.ConfigurationManager).service;
                            var uri = window.encodeURI(this.target || String.empty);
                            if (MS.Entertainment.Utilities.verifyUrl(uri))
                                uri = MS.Entertainment.Utilities.UriFactory.appendQuery(uri, {schema: "2.0"});
                            else
                                uri = MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), serviceConfig.videoAppFlexHubSuffix.format(uri));
                            return uri
                        }, pluralizers: [MSE.Data.Query.marketplaceWrapperQuery.channelPluralizer, MSE.Data.Query.marketplaceWrapperQuery.slotGroupPluralizer, MSE.Data.Query.marketplaceWrapperQuery.slotPluralizer], resultAugmentation: MSE.Data.Augmenter.Video.FlexHubContent
                })
        })})
})()
