﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
(function() {
    "use strict";
    scriptValidator("/Framework/utilities.js");
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Formatters");
    WinJS.Namespace.define("MS.Entertainment.Formatters", {DateTimeFormatters: MS.Entertainment.UI.Framework.define(function dateTimeFormatters(){}, {
            _year: null, _monthYear: null, _abbreviatedMonthYear: null, _abbreviatedMonth: null, _dayMonthYear: null, _shortDate: null, _decimalNumber: null, _groupedDecimalNumber: null, _percentNumber: null, year: {get: function() {
                        if (!this._year)
                            try {
                                this._year = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("year")
                            }
                            catch(e) {
                                var error = e && e.message;
                                MS.Entertainment.Formatters.fail("Could not instantiate new DateTimeFormatter(\"year\") object.  Error: " + error);
                                this._year = this._defaultFormatter
                            }
                        return this._year
                    }}, abbreviatedMonthYear: {get: function() {
                        if (!this._abbreviatedMonthYear)
                            try {
                                this._abbreviatedMonthYear = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("month.abbreviated year")
                            }
                            catch(e) {
                                var error = e && e.message;
                                MS.Entertainment.Formatters.fail("Could not instantiate new DateTimeFormatter(\"month.abbreviated year\") object.  Error: " + error);
                                this._abbreviatedMonthYear = this._defaultFormatter
                            }
                        return this._abbreviatedMonthYear
                    }}, abbreviatedMonth: {get: function() {
                        if (!this._abbreviatedMonth)
                            try {
                                this._abbreviatedMonth = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("month.abbreviated")
                            }
                            catch(e) {
                                var error = e && e.message;
                                MS.Entertainment.Formatters.fail("Could not instantiate new DateTimeFormatter(\"month.abbreviated\") object.  Error: " + error);
                                this._abbreviatedMonth = this._defaultFormatter
                            }
                        return this._abbreviatedMonth
                    }}, monthYear: {get: function() {
                        if (!this._monthYear)
                            try {
                                this._monthYear = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("month year")
                            }
                            catch(e) {
                                var error = e && e.message;
                                MS.Entertainment.Formatters.fail("Could not instantiate new DateTimeFormatter(\"month year\") object.  Error: " + error);
                                this._abbreviatedMonth = this._defaultFormatter
                            }
                        return this._monthYear
                    }}, shortDate: {get: function() {
                        if (!this._shortDate)
                            try {
                                this._shortDate = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("shortdate")
                            }
                            catch(e) {
                                var error = e && e.message;
                                MS.Entertainment.Formatters.fail("Could not instantiate new DateTimeFormatter(\"shortdate\") object.  Error: " + error);
                                this._shortDate = this._defaultFormatter
                            }
                        return this._shortDate
                    }}, dayMonthYear: {get: function() {
                        if (!this._dayMonthYear)
                            try {
                                this._dayMonthYear = new Windows.Globalization.DateTimeFormatting.DateTimeFormatter("day month year")
                            }
                            catch(e) {
                                var error = e && e.message;
                                MS.Entertainment.Formatters.fail("Could not instantiate new DateTimeFormatter(\"day month year\") object.  Error: " + error);
                                this._dayMonthYear = this._defaultFormatter
                            }
                        return this._dayMonthYear
                    }}, decimalNumber: {get: function() {
                        if (!this._decimalNumber) {
                            this._decimalNumber = new Windows.Globalization.NumberFormatting.DecimalFormatter;
                            this._decimalNumber.fractionDigits = 0;
                            this._decimalNumber.isGrouped = false;
                            this._decimalNumber.integerDigits = MS.Entertainment.Formatters.DateTimeFormatters.defaultDecimalDigits
                        }
                        return this._decimalNumber
                    }}, groupedDecimalNumber: {get: function() {
                        if (!this._groupedDecimalNumber) {
                            this._groupedDecimalNumber = new Windows.Globalization.NumberFormatting.DecimalFormatter;
                            this._groupedDecimalNumber.fractionDigits = 0;
                            this._groupedDecimalNumber.isGrouped = true;
                            this._groupedDecimalNumber.integerDigits = MS.Entertainment.Formatters.DateTimeFormatters.defaultDecimalDigits
                        }
                        return this._groupedDecimalNumber
                    }}, percentNumber: {get: function() {
                        if (!this._percentNumber) {
                            this._percentNumber = new Windows.Globalization.NumberFormatting.PercentFormatter;
                            this._percentNumber.fractionDigits = 0;
                            this._percentNumber.integerDigits = 0;
                            this._percentNumber.isDecimalPointAlwaysDisplayed = false;
                            this._percentNumber.isGrouped = true
                        }
                        return this._percentNumber
                    }}, _defaultFormatter: {get: function() {
                        return {format: function format() {
                                    return String.empty
                                }}
                    }}
        }, {
            create: function create() {
                return new MS.Entertainment.Formatters.DateTimeFormatters
            }, defaultDecimalDigits: 1
        })});
    MS.Entertainment.ServiceLocator.register(MS.Entertainment.Services.dateTimeFormatters, MS.Entertainment.Formatters.DateTimeFormatters.create);
    WinJS.Namespace.define("MS.Entertainment.Formatters", {
        MediaSortFormatter: MS.Entertainment.defineOptionalObservable(function mediaSortFormatter() {
            this.bind("totalCount", this._updateResult.bind(this))
        }, {
            _countStringId: null, _unknownStringId: null, _countOnly: null, _filter: null, initialize: function initialize(countStringId, unknownStringId, countOnly, filter) {
                    this._countStringId = countStringId;
                    this._unknownStringId = unknownStringId;
                    this._countOnly = countOnly;
                    this._filter = filter;
                    this._updateResult()
                }, _updateResult: function _updateResult() {
                    var countFormatter;
                    if (this.count >= 0) {
                        var mediaStringWithCount = MS.Entertainment.Utilities.Pluralization.getPluralizedString(this._countStringId, this.totalCount);
                        countFormatter = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber;
                        countFormatter.isGrouped = true;
                        this.result = String.load(this._filter ? String.id.IDS_N_FILTER_IN : String.id.IDS_N_SORTED_BY).format(countFormatter.format(this.totalCount), String.load(mediaStringWithCount));
                        countFormatter.isGrouped = false
                    }
                    else if (this._unknownStringId)
                        this.result = String.load(this._filter ? String.id.IDS_FILTER_IN : String.id.IDS_SORTED_BY).format(String.load(this._unknownStringId));
                    else if (this._countOnly) {
                        countFormatter = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber;
                        this.result = countFormatter.format(this.totalCount)
                    }
                    else
                        this.result = null
                }
        }, {
            totalCount: 0, result: null
        }), formatDestinationHelper: WinJS.Utilities.markSupportedForProcessing(function formatHelper(destination, destinationProperty, value) {
                if (destinationProperty.length > 1) {
                    var prop = destination[destinationProperty[0]];
                    for (var x = 1; x < destinationProperty.length - 1; x++)
                        prop = prop[destinationProperty[x]];
                    prop[destinationProperty[x]] = value
                }
                else
                    destination[destinationProperty] = value
            }), formatSourceHelper: WinJS.Utilities.markSupportedForProcessing(function formatSourceHelper(source, sourceProperty) {
                var sourceData;
                if (Array.isArray(sourceProperty)) {
                    sourceData = source[sourceProperty[0]];
                    for (var i = 1; i < sourceProperty.length; i++)
                        sourceData = sourceData[sourceProperty[i]]
                }
                else
                    sourceData = source[sourceProperty];
                return sourceData
            }), formatStringOrStringId: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function(value) {
                var sourceDataType = typeof value;
                var result = String.empty;
                if (sourceDataType === "string")
                    result = (String.id[value]) ? String.load(value) : value;
                else if (value)
                    MS.Entertainment.Formatters.fail("Unrecognized string type in formatter: " + sourceDataType);
                return result
            })), _formatStringStripHTML: function _formatStringStripHTML(value) {
                var sourceDataType = typeof value;
                var result = String.empty;
                if (sourceDataType === "string") {
                    if (value)
                        result = toStaticHTML(value.replace(/<(?:.|\n)*?>/gm, ""))
                }
                else if (value)
                    MS.Entertainment.Formatters.fail("Unrecognized string type in formatter: " + sourceDataType);
                return result
            }, _formatStringStripHTMLAndDecode: function _formatStringStripHTMLAndDecode(value) {
                var sourceDataType = typeof value;
                var result = String.empty;
                if (sourceDataType === "string") {
                    if (value)
                        result = MS.Entertainment.Utilities.unEscapeHTML(value)
                }
                else if (value)
                    MS.Entertainment.Formatters.fail("Unrecognized string type in formatter: " + sourceDataType);
                return result
            }, formatStringStripHTML: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function(value) {
                return MS.Entertainment.Formatters._formatStringStripHTML(value)
            })), formatStringStripHTMLAndDecode: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function(value) {
                return MS.Entertainment.Formatters._formatStringStripHTMLAndDecode(value)
            })), formatStringId: WinJS.Utilities.markSupportedForProcessing(function formatStringOrStringId(source, sourceProperty, destination, destinationProperty) {
                MS.Entertainment.Formatters.assert(sourceProperty[0] === "String", "Formatters_formatStringId must be passed a valid string id");
                var len = sourceProperty.length;
                var data = String;
                for (var i = 1; i < len && data; i++)
                    data = data[sourceProperty[i]];
                MS.Entertainment.Formatters.assert(String.id[data], "Formatters_formatStringId must be passed a valid string id");
                var result = (String.id[data] && String.load(data)) || String.empty;
                MS.Entertainment.Formatters.formatDestinationHelper(destination, destinationProperty, result)
            }), formatStringIdNoAssert: WinJS.Utilities.markSupportedForProcessing(function formatStringIdNoAssert(source, sourceProperty, destination, destinationProperty) {
                MS.Entertainment.Formatters.assert(sourceProperty[0] === "String", "Formatters_formatStringId must be passed a valid string id");
                var len = sourceProperty.length;
                var data = String;
                for (var i = 1; i < len && data; i++)
                    data = data[sourceProperty[i]];
                var result = (String.id[data] && String.load(data)) || String.empty;
                MS.Entertainment.Formatters.formatDestinationHelper(destination, destinationProperty, result)
            }), formatYearFromDateNonConverter: function formatYearFromDateNonConverter(sourceData) {
                var date = null;
                var year = null;
                if (sourceData) {
                    date = new Date(sourceData);
                    if (date) {
                        var formattedYear = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).year;
                        year = formattedYear.format(date)
                    }
                }
                return year || String.empty
            }, formatYearFromDate: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatReleaseYear(sourceData) {
                return MS.Entertainment.Formatters.formatYearFromDateNonConverter(sourceData)
            })), formatDatabaseDataFromReleaseDate: function formatDatabaseDataFromReleaseDate(sourceData) {
                var date = null;
                if (sourceData && sourceData.toISOString)
                    date = sourceData.toISOString().replace("T", " ").substring(0, 19);
                else
                    MS.Entertainment.Formatters.fail("Expected date to be provided as source data.  Cannot format releaseDate");
                return date || String.empty
            }, formatShortDate: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatYearMonthDate(sourceValue) {
                return MS.Entertainment.Formatters.formatShortDateNonConverter(sourceValue)
            })), formatShortDateNonConverter: function formatShortDateNonConverter(sourceValue) {
                if (sourceValue) {
                    var dateFormat = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).shortDate;
                    sourceValue = dateFormat.format(sourceValue)
                }
                return sourceValue || String.empty
            }, formatLongDate: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatYearMonthDate(sourceValue) {
                if (sourceValue) {
                    var dateFormat = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).dayMonthYear;
                    sourceValue = dateFormat.format(sourceValue)
                }
                return sourceValue || String.empty
            })), formatDurationFromDate: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatDurationFromDate(sourceData) {
                return MS.Entertainment.Formatters.formatDurationFromDateNonConverter(sourceData)
            })), formatDurationFromDateNonConverter: function formatDurationFromDateNonConverter(sourceData) {
                var date = sourceData;
                if (typeof sourceData !== "number")
                    date = new Date(sourceData);
                var duration = MS.Entertainment.Utilities.formatTimeString(date);
                return duration
            }, formatDurationFromDateNoSeconds: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatDurationFromDateNoSeconds(sourceData) {
                return MS.Entertainment.Formatters.formatDurationFromDateNoSecondsNonConverter(sourceData)
            })), formatDurationFromDateNoSecondsNonConverter: function formatDurationFromDateNoSecondsNonConverter(sourceData) {
                var duration = MS.Entertainment.Utilities.formatTimeString(sourceData, {excludeSeconds: true});
                return duration
            }, formatDurationGreaterThanZeroFromDate: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatDurationGreaterThanZeroFromDate(sourceData) {
                return MS.Entertainment.Formatters.formatDurationGreaterThanZeroFromDateNonConverter(sourceData)
            })), formatDurationGreaterThanZeroFromDateNonConverter: function formatDurationGreaterThanZeroFromDateNonConverter(sourceData) {
                var date = null;
                var forceEmptyString = false;
                if (typeof sourceData === "number")
                    if (sourceData > 0)
                        date = MS.Entertainment.Utilities._millisecondsToDate(sourceData);
                    else
                        forceEmptyString = true;
                else
                    date = new Date(sourceData);
                var duration = sourceData && !forceEmptyString ? MS.Entertainment.Utilities.formatTimeString(date) : String.empty;
                return duration
            }, milliSecondsFromTimeSpan: function milliSecondsFromTimeSpan(days, hours, minutes, seconds, milliseconds) {
                return ((((days ? days : 0) * 24 + (hours ? hours : 0)) * 60 + (minutes ? minutes : 0)) * 60 + (seconds ? seconds : 0)) * 1000 + (milliseconds ? milliseconds : 0)
            }, formatCount: function formatCount(count, countTextFunction) {
                if (count === undefined || count === -1)
                    return String.empty;
                return countTextFunction(count)
            }, countText: function countText(count, countFormatStringId) {
                var numberFormatter = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber;
                return MS.Entertainment.Formatters.countTextWithCustomFormatter(numberFormatter, count, countFormatStringId)
            }, countTextWithoutOrderFormat: function countTextWithoutOrderFormat(count, countFormatStringId) {
                var formatString = String.load(String.id.IDS_MUSIC_RELATED_PANEL_FORMAT);
                var formattedCount = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber.format(count);
                var countLabel = MS.Entertainment.Utilities.Pluralization.getPluralizedString(countFormatStringId, count);
                return formatString.format(formattedCount, countLabel)
            }, countTextWithCustomFormatter: function countTextWithCustomFormatter(numberFormatter, count, countFormatStringId) {
                var formattedCount = numberFormatter.format(count);
                return MS.Entertainment.Utilities.Pluralization.getPluralizedString(countFormatStringId, count).format(formattedCount)
            }, formatGenresList: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatGenresList(genres) {
                return MS.Entertainment.Formatters.formatGenresListNonConverter(genres)
            })), formatGenresListNonConverter: function formatGenresListNonConverter(genres) {
                if (!genres)
                    return String.empty;
                var genreNames = [];
                if (Array.isArray(genres)) {
                    genres.forEach(function forEachGenresList(genre) {
                        if (genre)
                            genreNames.push(genre.name)
                    });
                    return genreNames.join(String.load(String.id.IDS_DETAILS_INFO_SEPERATOR))
                }
                else if (typeof genres !== "string" && genres.forEach) {
                    MS.Entertainment.Formatters.fail("Supplied with virtual list. Don't know how to handle this. Don't think it's used any more");
                    return String.empty
                }
                else if (typeof genres === "string")
                    return genres
            }, formatStringOrUnknown: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatStringOrUnknown(result) {
                if (!result || result === "null")
                    result = String.load(String.id.IDS_UNKNOWN_VALUE);
                return result
            })), formatStringOrEmpty: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatStringOrEmpty(result) {
                if (!result || result === "null")
                    result = String.empty;
                return result
            })), formatStringOrEmptyOneTime: WinJS.Utilities.markSupportedForProcessing(function formatStringOrEmptyOneTime(source, sourceProperties, dest, destProperties) {
                var result = MS.Entertainment.Utilities.valueFromPropertyPathFragments(source, sourceProperties);
                if (!result || result === "null")
                    result = String.empty;
                MS.Entertainment.Utilities.setFromPropertyPathFragments(dest, destProperties, result)
            }), formatDecimalNumber: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatDecimalNumber(num) {
                if (num === undefined || num === null || typeof(num) !== "number")
                    return String.empty;
                var formattedNum;
                formattedNum = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber.format(num);
                return formattedNum
            })), formatDecimalNumberIncrement: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatDecimalNumberIncrement(num) {
                if (num === undefined || num === null || typeof(num) !== "number")
                    return String.empty;
                var formattedNum;
                formattedNum = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber.format(num + 1);
                return formattedNum
            })), formatDecimalNumberIncrementOneTime: WinJS.Utilities.markSupportedForProcessing(function formatDecimalNumberIncrementOneTime(source, sourceProperties, dest, destProperties) {
                var formattedNum = String.empty;
                var number = MS.Entertainment.Utilities.valueFromPropertyPathFragments(source, sourceProperties);
                if (typeof number === "number" && !isNaN(number))
                    formattedNum = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber.format(number + 1);
                MS.Entertainment.Utilities.setFromPropertyPathFragments(dest, destProperties, formattedNum)
            }), formatIcon: WinJS.Utilities.markSupportedForProcessing(function formatIcon(source, sourceProperty, destination, destinationProperty) {
                var icon = MS.Entertainment.Utilities.valueFromPropertyPathFragments(WinJS.Utilities.getMember(sourceProperty[0]), sourceProperty.slice(1));
                MS.Entertainment.Formatters.formatDestinationHelper(destination, destinationProperty, icon)
            }), setPropertyFromStatic: WinJS.Utilities.markSupportedForProcessing(function setPropertyFromStatic(source, sourceProperty, destination, destinationProperty) {
                var valueFromProperty = MS.Entertainment.Utilities.valueFromPropertyPathFragments(WinJS.Utilities.getMember(sourceProperty[0]), sourceProperty.slice(1));
                MS.Entertainment.Formatters.formatDestinationHelper(destination, destinationProperty, valueFromProperty)
            }), formatGenre: function formatGenre(sourceValue) {
                var result = String.empty;
                if (sourceValue.genre)
                    if (Array.isArray(sourceValue.genre)) {
                        var genreNames = [];
                        sourceValue.genre.forEach(function(genre) {
                            if (genre)
                                genreNames.push(genre.name)
                        });
                        result = genreNames.join(String.load(String.id.IDS_DETAILS_INFO_SEPERATOR))
                    }
                    else if (sourceValue.genre.name)
                        result = sourceValue.genre.name;
                    else
                        result = sourceValue.genre;
                else if (sourceValue.genreName)
                    result = sourceValue.genreName;
                return result
            }, formatModuleHeaderActionTitle: WinJS.Utilities.markSupportedForProcessing(function formatModuleHeaderActionTitle(source, sourceProperty, destination, destinationProperty) {
                var moduleTitleText = String.load(sourceProperty);
                if (moduleTitleText) {
                    var result = String.load(String.id.IDS_APP1_MODULE_VIEW_ALL_LABEL).format(moduleTitleText);
                    MS.Entertainment.Formatters.formatDestinationHelper(destination, destinationProperty, result)
                }
            }), canShowExpiredFlag: function canShowExpiredFlag() {
                var configManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                var isUsingXdlcForIngestion = MS.Entertainment.Utilities.isVideoApp && configManager.fulfillment.videoUsesXdlcInsteadOfZest;
                if (!isUsingXdlcForIngestion)
                    return true;
                if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.purchaseHistory)) {
                    var purchaseHistoryService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.purchaseHistory);
                    return !!(purchaseHistoryService && purchaseHistoryService.hasGrovelEverCompleted)
                }
                return false
            }, formatRentalExpirationFromSpanInt: function formatRentalExpirationFromSpanInt(milliSeconds, useLongRentalExpiredString) {
                var message;
                var seconds = Math.floor(milliSeconds / 1000);
                var minutes = Math.floor(seconds / 60);
                var minutesNormalized = minutes % 60;
                var hours = Math.floor(minutes / 60);
                var hoursNormalized = hours % 24;
                var days = Math.floor(hours / 24);
                if (milliSeconds < 0)
                    if (MS.Entertainment.Formatters.canShowExpiredFlag())
                        message = (useLongRentalExpiredString ? String.load(String.id.IDS_VIDEO_RENTAL_EXPIRED) : String.load(String.id.IDS_VIDEO_RENTAL_EXPIRED_ONLY));
                    else
                        message = String.empty;
                else if (milliSeconds >= MS.Entertainment.Formatters.milliSecondsFromTimeSpan(2))
                    if (hoursNormalized === 0)
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_DAYS).format(days);
                    else if (hoursNormalized === 1)
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_DAYS_HR).format(days);
                    else
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_DAYS_HRS).format(days, hoursNormalized);
                else if (milliSeconds >= MS.Entertainment.Formatters.milliSecondsFromTimeSpan(1, 1))
                    if (hoursNormalized === 1)
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_DAY_HR);
                    else
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_DAY_HRS).format(hoursNormalized);
                else if (milliSeconds >= MS.Entertainment.Formatters.milliSecondsFromTimeSpan(1))
                    message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_DAY);
                else if (hoursNormalized > 1)
                    if (minutesNormalized === 0)
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_HRS).format(hoursNormalized);
                    else if (minutesNormalized === 1)
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_HRS_MIN).format(hoursNormalized);
                    else
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_HRS_MINS).format(hoursNormalized, minutesNormalized);
                else if (hoursNormalized === 1)
                    if (minutesNormalized === 0)
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_HR);
                    else if (minutesNormalized === 1)
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_HR_MIN);
                    else
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_HR_MINS).format(minutesNormalized);
                else if (hoursNormalized === 0)
                    if (minutesNormalized === 0)
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_LESS_THAN_MIN);
                    else if (minutesNormalized === 1)
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_MIN);
                    else
                        message = String.load(String.id.IDS_VIDEO_RENTAL_EXPIRES_MINS).format(minutesNormalized);
                return message
            }, formatRentalExpirationFromSpan: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatRentalExpirationFromSpan(sourceData) {
                return MS.Entertainment.Formatters.formatRentalExpirationFromSpanInt(sourceData)
            })), formatSeason: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatSeason(value) {
                return MS.Entertainment.Formatters.formatCount(value, MS.Entertainment.Formatters.seasonCountText)
            })), seasonCountText: function seasonCountText(count) {
                return MS.Entertainment.Formatters.countText(count, String.id.IDS_TV_SEASONS_LABEL_PLURAL)
            }, formatNumberOfRatings: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatNumberOfRatings(number) {
                if (!number)
                    return String.empty;
                var groupedNumberFormatter = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).groupedDecimalNumber;
                return MS.Entertainment.Formatters.countTextWithCustomFormatter(groupedNumberFormatter, number, String.id.IDS_DETAILS_RATINGS_PLURAL)
            })), formatNumberOfReviews: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatNumberOfReviews(number) {
                if (!number)
                    return String.empty;
                var groupedNumber = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).groupedDecimalNumber.format(number);
                return String.load(String.id.IDS_VIDEO_REVIEW_COUNT).format(groupedNumber)
            })), formatPercentage: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatPercentage(number) {
                number = Number(number);
                if (!number)
                    return String.empty;
                number = number / 100;
                var percentNumber = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).percentNumber.format(number);
                percentNumber = percentNumber.replace(" ", "");
                return percentNumber
            })), formatTVEpisode: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatTVEpisode(episodeCount) {
                var formattedCount;
                if (episodeCount === undefined || episodeCount < 0)
                    return String.empty;
                formattedCount = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber.format(episodeCount);
                var result = String.empty;
                if (episodeCount === 0)
                    result = String.load(String.id.IDS_TV_NOEPISODES_LABEL).format(formattedCount);
                if (episodeCount === 1)
                    result = String.load(String.id.IDS_TV_EPISODE_LABEL).format(formattedCount);
                else
                    result = String.load(String.id.IDS_TV_EPISODES_LABEL).format(formattedCount);
                return result
            })), formatSeasonOwnedEpisodes: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatSeasonOwnedEpisodes(sourceValue) {
                var result = String.empty;
                if (sourceValue)
                    result = String.load(String.id.IDS_VIDEO2_TV_SEASON_EPISODES_OWNED).format(sourceValue);
                return result
            })), formatAddPeriod: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatAddPeriod(sourceValue) {
                var result = String.empty;
                if (sourceValue)
                    result = String.load(String.id.IDS_DETAILS_TRACK_NUM_FORMAT).format(sourceValue);
                return result
            })), formatTVSeasonNumberInt: function formatTVSeasonNumberInt(seasonNumber, isLowerCase) {
                var formattedNumber;
                if (seasonNumber === undefined)
                    return String.empty;
                var seasonExtraStringId = isLowerCase ? String.id.IDS_TV_SEASON_0_NAME_LC : String.id.IDS_TV_SEASON_0_NAME;
                var seasonNameStringId = isLowerCase ? String.id.IDS_TV_SEASON_NAME_LC : String.id.IDS_TV_SEASON_NAME;
                formattedNumber = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber.format(seasonNumber);
                var result = (seasonNumber <= 0) ? String.load(seasonExtraStringId) : String.load(seasonNameStringId).format(formattedNumber);
                return result
            }, formatTVSeasonNumber: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatTVSeasonNumber(sourceValue) {
                return MS.Entertainment.Formatters.formatTVSeasonNumberInt(sourceValue)
            })), formatTVSeasonEpisodeNumberInt: function formatTVSeasonEpisodeNumberInt(sourceValue) {
                var result = String.empty;
                var formattedEpisodeNumber;
                var formattedSeasonNumber;
                var numberFormatter;
                if (sourceValue === undefined || sourceValue === null)
                    return String.emtpy;
                if ((sourceValue.seriesTitle && sourceValue.seasonNumber !== undefined && sourceValue.episodeNumber !== undefined) || (sourceValue.contentType === "TVEpisode")) {
                    numberFormatter = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber;
                    var seasonNumber = sourceValue.seasonNumber;
                    if (sourceValue.season && sourceValue.season.number)
                        seasonNumber = sourceValue.season.number;
                    formattedSeasonNumber = numberFormatter.format(seasonNumber);
                    var seasonNumber = (seasonNumber <= 0) ? String.load(String.id.IDS_TV_SEASON_0_NAME) : String.load(String.id.IDS_TV_SEASON_NAME).format(formattedSeasonNumber);
                    var episodeNumber;
                    if (sourceValue.episodeNumber > 0) {
                        formattedEpisodeNumber = numberFormatter.format(sourceValue.episodeNumber);
                        episodeNumber = String.load(String.id.IDS_TV_NUMBERED_EPISODE_NAME).format(formattedEpisodeNumber)
                    }
                    else
                        episodeNumber = String.load(String.id.IDS_TV_NUMBERED_EPISODE_0_NAME);
                    if (seasonNumber !== 0 || episodeNumber !== 0)
                        result = String.load(String.id.IDS_COMMA_SEPARATOR).format(seasonNumber, episodeNumber);
                    else
                        result = episodeNumber
                }
                return result
            }, formatTVSeasonEpisodeNumber: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatTVSeasonEpisodeNumber(sourceValue) {
                return MS.Entertainment.Formatters.formatTVSeasonEpisodeNumberInt(sourceValue)
            })), formatEpisodeSeriesTitleSeasonEpisodeNumberFileSize: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatVideoDownloadMovieDescription(sourceValue) {
                var result = String.empty;
                if (sourceValue === undefined || sourceValue === null)
                    return String.emtpy;
                if (sourceValue.seriesTitle)
                    result = sourceValue.seriesTitle;
                var seasonEpisodeNumber = MS.Entertainment.Formatters.formatTVSeasonEpisodeNumberInt(sourceValue);
                if (seasonEpisodeNumber)
                    if (result.length > 0)
                        result = String.load(String.id.IDS_COMMA_SEPARATOR).format(result, seasonEpisodeNumber);
                    else
                        result = seasonEpisodeNumber;
                return result
            })), formatMovieGenreReleaseYear: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatMovieGenreReleaseYear(sourceValue) {
                var result = String.empty;
                if (sourceValue === undefined || sourceValue === null)
                    return String.emtpy;
                if (sourceValue.genreName)
                    result = sourceValue.genreName;
                if (sourceValue.releaseDate) {
                    var dateFormat = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).year;
                    var year = dateFormat.format(sourceValue.releaseDate);
                    if (result.length > 0)
                        result = String.load(String.id.IDS_COMMA_SEPARATOR).format(result, year);
                    else
                        result = year
                }
                return result
            })), formatMovieReleaseYearRatingDuration: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatMovieReleaseYearRatingDuration(sourceValue) {
                var result = String.empty;
                if (sourceValue) {
                    var parts = [];
                    if (sourceValue.releaseDate) {
                        var dateFormat = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).year;
                        var year = dateFormat.format(sourceValue.releaseDate);
                        parts.push(year)
                    }
                    if (sourceValue.rating)
                        parts.push(sourceValue.rating);
                    if (sourceValue.duration)
                        parts.push(MS.Entertainment.Formatters.formatDurationFromDateNonConverter(sourceValue.duration));
                    result = parts.join(String.load(String.id.IDS_DETAILS_INFO_SEPERATOR))
                }
                return result
            })), formatContentRatingsList: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatContentRatingsList(contentIncludes) {
                if (!contentIncludes)
                    return Stirng.empty;
                if (typeof contentIncludes !== "string") {
                    var contentFeatures = [];
                    contentIncludes.forEach(function forEachContentIncludes(contentIncludes) {
                        contentFeatures.push(contentIncludes.label)
                    });
                    return contentFeatures.join(String.load(String.id.IDS_DETAILS_INFO_SEPERATOR))
                }
                else
                    return contentIncludes
            })), formatEpisodeMetadata: function formatEpisodeMetadata(episode, options) {
                var values = [];
                var options = options || {};
                if (options.releaseDate) {
                    var releaseDate = null;
                    if (episode.releaseDate) {
                        var formatter = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).year;
                        if (MS.Entertainment.Platform.PlaybackHelpers.isTVEpisode(episode))
                            formatter = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).shortDate;
                        var date = new Date(episode.releaseDate);
                        releaseDate = formatter.format(date)
                    }
                    if (releaseDate)
                        values.push(releaseDate)
                }
                if (options.genres) {
                    var genres = null;
                    if (episode.genre)
                        if (Array.isArray(episode.genre)) {
                            var genreNames = [];
                            episode.genre.forEach(function(genre) {
                                if (genre)
                                    genreNames.push(genre.name)
                            });
                            genres = genreNames.join(String.load(String.id.IDS_DETAILS_INFO_SEPERATOR))
                        }
                        else
                            genres = episode.genre;
                    else if (episode.genreName)
                        genres = episode.genreName;
                    if (genres)
                        values.push(genres)
                }
                if (options.network)
                    if (episode.network)
                        values.push(episode.network);
                    else if (episode.networks && episode.networks.length > 0 && episode.networks[0].name)
                        values.push(episode.networks[0].name);
                if (options.languages) {
                    var languages = MS.Entertainment.ViewModels.VideoSmartBuyStateEngine.getLanguagesForRights(episode, MS.Entertainment.Utilities.defaultClientTypeFromApp);
                    if (languages && languages.length === 1)
                        values.push(String.load(String.id.IDS_DETAILS_AUDIO_LANGUAGE).format(MS.Entertainment.Utilities.getDisplayLanguageFromLanguageCode(languages[0])))
                }
                if (options.rating)
                    if (episode.rating)
                        values.push(episode.rating);
                if (options.videoDefinition)
                    if (episode.libraryDefinitions)
                        values.push(episode.libraryDefinitions);
                    else if (episode.videoDefinitions && episode.videoDefinitions !== String.load(String.id.IDS_UNKNOWN_VALUE))
                        values.push(episode.videoDefinitions);
                if (options.duration) {
                    var duration = null;
                    if (episode.duration && episode.duration !== -1)
                        duration = MS.Entertainment.Utilities.formatTimeString(episode.duration);
                    if (duration)
                        values.push(duration)
                }
                return values.join(String.load(String.id.IDS_DETAILS_INFO_SEPERATOR))
            }, formatIsDownloaded: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatCanPlayLocally(source) {
                var result = String.empty;
                if (!source && MS.Entertainment.Utilities.isApp1)
                    result = MS.Entertainment.UI.Icon.inlineStreaming;
                return result
            }))
    })
})()
