﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
(function() {
    "use strict";
    scriptValidator();
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Shell");
    WinJS.Namespace.define("MS.Entertainment.UI.Shell", {
        ImageLoader: MS.Entertainment.UI.Framework.define(function(imgLoadedCallback) {
            this._imgLoadedCallback = imgLoadedCallback;
            this._imgElement = document.createElement("img");
            this._imgElement.suppressUnload = true;
            this.state = this.states.initial
        }, {
            _imgLoadedCallback: null, _imgElement: null, _imgUrl: null, _loadingDefault: false, states: {
                    value: {
                        initial: -1, loading: 0, error: 1, loaded: 2, loadedDefault: 3, loadedFallback: 4
                    }, writable: false
                }, loadCatalogImage: function loadCatalogImage(defaultImageUrl, imageId, imageIdType, dimensions, forceImageResize, ignoreZeroLengths, requestedImage, imageChildId, imageContentType) {
                    var url = MS.Entertainment.UI.Shell.ImageLoader.makeCatalogImageUri(imageId, imageIdType, dimensions, forceImageResize, ignoreZeroLengths, requestedImage, imageChildId, imageContentType);
                    this.loadImage(url, defaultImageUrl)
                }, loadImage: function loadImage(imageUrl, defaultImageUrl, loadingImageUrl) {
                    this._loadingDefault = false;
                    this.loadImageInternal(imageUrl, defaultImageUrl, loadingImageUrl)
                }, loadImageInternal: function loadImageInternal(imageUrl, defaultImageUrl, loadingImageUrl) {
                    this.state = this.states.initial;
                    if (loadingImageUrl)
                        this._imgUrl = loadingImageUrl;
                    else
                        this._imgUrl = MS.Entertainment.UI.Shell.ImageLoader._commonLoading;
                    this._imgElement.setAttribute("src", this._imgUrl);
                    if (!imageUrl)
                        if (defaultImageUrl) {
                            this.state = this.states.loadedDefault;
                            this.imgUrl = defaultImageUrl
                        }
                        else {
                            this.state = this.states.loadedFallback;
                            this._loadingDefault = false;
                            this.imgUrl = MS.Entertainment.UI.Shell.ImageLoader._commonDefault
                        }
                    else if (imageUrl === MS.Entertainment.UI.Shell.ImageLoader.defaultImage) {
                        this.state = this.states.error;
                        this._loadingDefault = true;
                        this.loadImage(null, defaultImageUrl)
                    }
                    else if (MS.Entertainment.UI.Shell.ImageLoader.isWebUrl(imageUrl))
                        this._loadImageFromWeb(imageUrl, defaultImageUrl);
                    else if (MS.Entertainment.UI.Shell.ImageLoader.isBlobUrl(imageUrl))
                        this._loadImageFromWeb(imageUrl, defaultImageUrl);
                    else if (MS.Entertainment.UI.Shell.ImageLoader.isAppDataUrl(imageUrl))
                        this._loadImageFromWeb(imageUrl, defaultImageUrl);
                    else if (MS.Entertainment.UI.Shell.ImageLoader.isPackageUrl(imageUrl))
                        this._loadImageFromWeb(imageUrl, defaultImageUrl);
                    else
                        this._loadImageFromPath(imageUrl, defaultImageUrl)
                }, state: {
                    get: function getState() {
                        return this._state
                    }, set: function setState(value) {
                            if (this._state !== value)
                                this._state = value
                        }
                }, imgContainer: {get: function getImgContainer() {
                        return this._imgElement
                    }}, imgUrl: {
                    get: function getImgUrl() {
                        return this._imgUrl
                    }, set: function setImgUrl(value) {
                            if (this._imgUrl !== value) {
                                this._imgUrl = value;
                                this._imgElement.setAttribute("src", this._imgUrl);
                                this._fireImgLoaded(this._imgUrl)
                            }
                        }
                }, _fireImgLoaded: function(imageUrl) {
                    if (this._imgLoadedCallback)
                        this._imgLoadedCallback(imageUrl)
                }, _loadImageFromWeb: function(imageUrl, defaultImageUrl) {
                    var that = this;
                    var remoteImage = new Image;
                    remoteImage.addEventListener("load", function imageOnLoad() {
                        if (that._loadingDefault)
                            that.state = that.states.loadedDefault;
                        else
                            that.state = that.states.loaded;
                        that._loadingDefault = false;
                        that.imgUrl = remoteImage.src
                    });
                    remoteImage.addEventListener("error", function imageOnError() {
                        that.state = that.states.error;
                        that._loadingDefault = true;
                        that.loadImageInternal(null, defaultImageUrl)
                    });
                    this.state = this.states.loading;
                    remoteImage.src = imageUrl
                }, _loadImageFromPath: function(imagePath, defaultImageUrl, imageContainer) {
                    var that = this;
                    try {
                        Windows.Storage.StorageFile.getFileFromPathAsync(imagePath).then(function(loadedFile) {
                            var url = URL.createObjectURL(loadedFile, {oneTimeOnly: true});
                            if (that._loadingDefault)
                                that.state = that.states.loadedDefault;
                            else
                                that.state = that.states.loaded;
                            that._loadingDefault = false;
                            that.imgUrl = url
                        }, function() {
                            that.state = that.states.error;
                            that._loadingDefault = true;
                            that.loadImageInternal(null, defaultImageUrl)
                        });
                        this.state = this.states.loading
                    }
                    catch(e) {
                        that.loadImageInternal(null, defaultImageUrl)
                    }
                }
        }, {
            _commonDefault: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.UI.ImagePaths.imageNotFound
            }), _commonLoading: MS.Entertainment.UI.Framework.lazyDefine(function() {
                    return "ms-appx:///Images/loading_image." + MS.Entertainment.Utilities.getPackageImageFileExtension()
                }), _endpointCache: null, _initializeEndpointCache: function _initializeEndpointCache() {
                    if (!MS.Entertainment.UI.Shell.ImageLoader._endpointCache)
                        MS.Entertainment.UI.Shell.ImageLoader._endpointCache = {imageCatalog: MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_ImageCatalog)}
                }, defaultImage: -1, isBlobUrl: function(imageUrl) {
                    return (imageUrl && imageUrl.match && imageUrl.match(/^blob*:/i))
                }, isAppDataUrl: function(imageUrl) {
                    return (imageUrl && imageUrl.match && imageUrl.match(/^ms\-appdata*:\/\//i))
                }, isPackageUrl: function(imageUrl) {
                    return (imageUrl && imageUrl.match && imageUrl.match(/^ms\-appx*:\/\//i))
                }, isFileUrl: function isFileUrl(imageUrl) {
                    return (imageUrl && imageUrl.match && (imageUrl.match(/^file:\/\//i) || imageUrl.match(/^[a-z]:\\/i)))
                }, isWebUrl: function(imageUrl) {
                    return (imageUrl && imageUrl.match && imageUrl.match(/^http[s]*:\/\//i))
                }, isStreamingUrl: function(imageUrl) {
                    return MS.Entertainment.Utilities.isStreamingUrl(imageUrl)
                }, appendResizeParameters: function appendResizeParameters(url, width, height, contentType) {
                    if (url && MS.Entertainment.UI.Shell.ImageLoader.isWebUrl(url)) {
                        var parameters = {contenttype: contentType || MS.Entertainment.ImageContentType.jpeg};
                        var resize = false;
                        if (width > 0) {
                            resize = true;
                            parameters["width"] = width
                        }
                        if (height > 0) {
                            resize = true;
                            parameters["height"] = height
                        }
                        if (resize)
                            parameters["resize"] = true;
                        url = MS.Entertainment.Utilities.UriFactory.appendQuery(url, parameters)
                    }
                    return url
                }, MediaDefaultUrls: MS.Entertainment.UI.Framework.lazyDefine(function() {
                    return {
                            track: MS.Entertainment.UI.ImagePaths.genericMusicSmall, album: MS.Entertainment.Utilities.isApp1 ? MS.Entertainment.UI.ImagePaths.genericMusicSmall : MS.Entertainment.UI.ImagePaths.music2MissingArt320x320, artist: MS.Entertainment.UI.ImagePaths.genericMusicSmall, playlist: MS.Entertainment.Utilities.isApp1 ? MS.Entertainment.UI.ImagePaths.genericMusicSmall : MS.Entertainment.UI.ImagePaths.music2MissingArt158x158, playlistLarge: MS.Entertainment.Utilities.isApp1 ? MS.Entertainment.UI.ImagePaths.genericMusicSmall : MS.Entertainment.UI.ImagePaths.music2MissingArt320x320, movie: MS.Entertainment.UI.ImagePaths.genericVideoSmall, musicVideo: MS.Entertainment.UI.ImagePaths.genericVideoSmall, otherVideo: MS.Entertainment.UI.ImagePaths.genericVideoSmall, tvSeason: MS.Entertainment.UI.ImagePaths.genericVideoSmall, tvSeries: MS.Entertainment.UI.ImagePaths.genericVideoSmall, tvEpisode: MS.Entertainment.UI.ImagePaths.genericVideoSmall
                        }
                }), _defaultImageSize: {
                    x: 320, y: 320
                }, ImageService: {
                    Zune: "Zune", EDS: "EDS"
                }, _getAcceptableValuesArray: function _getAcceptableWidthsArray(imageService) {
                    var ImageLoader = MS.Entertainment.UI.Shell.ImageLoader;
                    return (imageService === ImageLoader.ImageService.Zune) ? ImageLoader._acceptableValuesZune : ImageLoader._acceptableValuesEDS
                }, _getAcceptableWideValuesArray: function _getAcceptableWideValuesArray(imageService) {
                    var ImageLoader = MS.Entertainment.UI.Shell.ImageLoader;
                    return (imageService === ImageLoader.ImageService.Zune) ? ImageLoader._acceptableWideValuesZune : ImageLoader._acceptableWideValuesEDS
                }, _getAcceptableSquareValuesArray: function _getAcceptableSquareValuesArray(imageService) {
                    var ImageLoader = MS.Entertainment.UI.Shell.ImageLoader;
                    return (imageService === ImageLoader.ImageService.Zune) ? ImageLoader._acceptableSquareValuesZune : ImageLoader._acceptableSquareValuesEDS
                }, _getAcceptableTallValuesArray: function _getAcceptableTallValuesArray(imageService) {
                    var ImageLoader = MS.Entertainment.UI.Shell.ImageLoader;
                    return (imageService === ImageLoader.ImageService.Zune) ? ImageLoader._acceptableTallValuesZune : ImageLoader._acceptableTallValuesEDS
                }, _getAcceptableWidthsArray: function _getAcceptableWidthsArray(imageService) {
                    var ImageLoader = MS.Entertainment.UI.Shell.ImageLoader;
                    return (imageService === ImageLoader.ImageService.Zune) ? ImageLoader._acceptableWidthsZune : ImageLoader._acceptableWidthsEDS
                }, _getAcceptableHeightsArray: function _getAcceptableHeightsArray(imageService) {
                    var ImageLoader = MS.Entertainment.UI.Shell.ImageLoader;
                    return (imageService === ImageLoader.ImageService.Zune) ? ImageLoader._acceptableValuesZune : ImageLoader._acceptableValuesEDS
                }, _acceptableValuesZune: [{
                        x: 1920, y: 1080
                    }, {
                        x: 1366, y: 768
                    }, {
                        x: 1280, y: 720
                    }, {
                        x: 1012, y: 693
                    }, {
                        x: 854, y: 480
                    }, {
                        x: 853, y: 480
                    }, {
                        x: 800, y: 800
                    }, {
                        x: 584, y: 800
                    }, {
                        x: 480, y: 480
                    }, {
                        x: 480, y: 270
                    }, {
                        x: 420, y: 320
                    }, {
                        x: 395, y: 270
                    }, {
                        x: 347, y: 195
                    }, {
                        x: 320, y: 320
                    }, {
                        x: 285, y: 195
                    }, {
                        x: 267, y: 150
                    }, {
                        x: 258, y: 258
                    }, {
                        x: 258, y: 194
                    }, {
                        x: 243, y: 137
                    }, {
                        x: 240, y: 240
                    }, {
                        x: 234, y: 320
                    }, {
                        x: 219, y: 150
                    }, {
                        x: 213, y: 120
                    }, {
                        x: 175, y: 120
                    }, {
                        x: 172, y: 258
                    }, {
                        x: 160, y: 160
                    }, {
                        x: 160, y: 120
                    }, {
                        x: 150, y: 150
                    }, {
                        x: 144, y: 108
                    }, {
                        x: 112, y: 84
                    }, {
                        x: 108, y: 108
                    }, {
                        x: 108, y: 72
                    }, {
                        x: 107, y: 160
                    }, {
                        x: 100, y: 100
                    }, {
                        x: 84, y: 84
                    }, {
                        x: 84, y: 56
                    }, {
                        x: 80, y: 60
                    }, {
                        x: 72, y: 72
                    }, {
                        x: 64, y: 64
                    }, {
                        x: 64, y: 48
                    }, {
                        x: 60, y: 60
                    }, {
                        x: 60, y: 45
                    }, {
                        x: 60, y: 40
                    }, {
                        x: 56, y: 56
                    }, {
                        x: 52, y: 52
                    }, {
                        x: 50, y: 50
                    }, {
                        x: 44, y: 44
                    }, {
                        x: 43, y: 64
                    }, {
                        x: 40, y: 40
                    }], _acceptableWideValuesZune: [{
                        x: 1920, y: 1080
                    }, {
                        x: 1366, y: 768
                    }, {
                        x: 1280, y: 720
                    }, {
                        x: 1012, y: 693
                    }, {
                        x: 854, y: 480
                    }, {
                        x: 853, y: 480
                    }, {
                        x: 480, y: 270
                    }, {
                        x: 420, y: 320
                    }, {
                        x: 395, y: 270
                    }, {
                        x: 347, y: 195
                    }, {
                        x: 285, y: 195
                    }, {
                        x: 267, y: 150
                    }, {
                        x: 258, y: 194
                    }, {
                        x: 243, y: 137
                    }, {
                        x: 219, y: 150
                    }, {
                        x: 213, y: 120
                    }, {
                        x: 175, y: 120
                    }, {
                        x: 160, y: 120
                    }, {
                        x: 144, y: 108
                    }, {
                        x: 112, y: 84
                    }, {
                        x: 108, y: 72
                    }, {
                        x: 84, y: 56
                    }, {
                        x: 80, y: 60
                    }, {
                        x: 64, y: 48
                    }, {
                        x: 60, y: 45
                    }, {
                        x: 60, y: 40
                    }], _acceptableSquareValuesZune: [{
                        x: 800, y: 800
                    }, {
                        x: 480, y: 480
                    }, {
                        x: 320, y: 320
                    }, {
                        x: 258, y: 258
                    }, {
                        x: 240, y: 240
                    }, {
                        x: 160, y: 160
                    }, {
                        x: 150, y: 150
                    }, {
                        x: 108, y: 108
                    }, {
                        x: 100, y: 100
                    }, {
                        x: 84, y: 84
                    }, {
                        x: 72, y: 72
                    }, {
                        x: 64, y: 64
                    }, {
                        x: 60, y: 60
                    }, {
                        x: 56, y: 56
                    }, {
                        x: 52, y: 52
                    }, {
                        x: 50, y: 50
                    }, {
                        x: 44, y: 44
                    }, {
                        x: 40, y: 40
                    }], _acceptableTallValuesZune: [{
                        x: 584, y: 800
                    }, {
                        x: 234, y: 320
                    }, {
                        x: 172, y: 258
                    }, {
                        x: 107, y: 160
                    }, {
                        x: 43, y: 64
                    }], _acceptableWidthsZune: [1920, 1366, 1280, 1012, 854, 853, 800, 584, 480, 420, 395, 347, 320, 285, 267, 258, 243, 240, 234, 219, 213, 175, 172, 160, 150, 144, 112, 108, 107, 100, 84, 80, 72, 64, 60, 56, 52, 50, 44, 43, 40], _acceptableHeightsZune: [1080, 800, 768, 720, 693, 480, 320, 270, 258, 240, 195, 194, 160, 150, 137, 120, 108, 100, 84, 72, 64, 60, 56, 52, 50, 48, 45, 44, 40], _acceptableValuesEDS: [{
                        x: 2560, y: 1440
                    }, {
                        x: 2471, y: 753
                    }, {
                        x: 2470, y: 2470
                    }, {
                        x: 2399, y: 576
                    }, {
                        x: 2398, y: 2398
                    }, {
                        x: 2398, y: 1642
                    }, {
                        x: 2398, y: 576
                    }, {
                        x: 2100, y: 2100
                    }, {
                        x: 2100, y: 753
                    }, {
                        x: 2048, y: 1536
                    }, {
                        x: 2021, y: 576
                    }, {
                        x: 2020, y: 2020
                    }, {
                        x: 2020, y: 576
                    }, {
                        x: 1920, y: 1080
                    }, {
                        x: 1919, y: 1200
                    }, {
                        x: 1919, y: 1079
                    }, {
                        x: 1831, y: 540
                    }, {
                        x: 1830, y: 1391
                    }, {
                        x: 1830, y: 540
                    }, {
                        x: 1795, y: 448
                    }, {
                        x: 1793, y: 1793
                    }, {
                        x: 1793, y: 448
                    }, {
                        x: 1593, y: 896
                    }, {
                        x: 1562, y: 1562
                    }, {
                        x: 1562, y: 540
                    }, {
                        x: 1561, y: 1561
                    }, {
                        x: 1561, y: 1069
                    }, {
                        x: 1561, y: 540
                    }, {
                        x: 1501, y: 448
                    }, {
                        x: 1499, y: 1028
                    }, {
                        x: 1440, y: 1440
                    }, {
                        x: 1440, y: 1291
                    }, {
                        x: 1440, y: 810
                    }, {
                        x: 1366, y: 768
                    }, {
                        x: 1351, y: 380
                    }, {
                        x: 1350, y: 1350
                    }, {
                        x: 1315, y: 1315
                    }, {
                        x: 1315, y: 448
                    }, {
                        x: 1280, y: 800
                    }, {
                        x: 1280, y: 720
                    }, {
                        x: 1277, y: 320
                    }, {
                        x: 1276, y: 1276
                    }, {
                        x: 1276, y: 320
                    }, {
                        x: 1206, y: 1206
                    }, {
                        x: 1191, y: 320
                    }, {
                        x: 1190, y: 1190
                    }, {
                        x: 1138, y: 640
                    }, {
                        x: 1080, y: 1080
                    }, {
                        x: 1080, y: 789
                    }, {
                        x: 1067, y: 320
                    }, {
                        x: 1066, y: 810
                    }, {
                        x: 1051, y: 1051
                    }, {
                        x: 1024, y: 768
                    }, {
                        x: 1021, y: 1021
                    }, {
                        x: 1021, y: 448
                    }, {
                        x: 1012, y: 693
                    }, {
                        x: 981, y: 320
                    }, {
                        x: 980, y: 980
                    }, {
                        x: 938, y: 938
                    }, {
                        x: 935, y: 320
                    }, {
                        x: 934, y: 934
                    }, {
                        x: 934, y: 320
                    }, {
                        x: 896, y: 896
                    }, {
                        x: 864, y: 864
                    }, {
                        x: 854, y: 480
                    }, {
                        x: 853, y: 480
                    }, {
                        x: 818, y: 818
                    }, {
                        x: 810, y: 1080
                    }, {
                        x: 800, y: 800
                    }, {
                        x: 796, y: 448
                    }, {
                        x: 789, y: 1080
                    }, {
                        x: 788, y: 1080
                    }, {
                        x: 770, y: 770
                    }, {
                        x: 770, y: 485
                    }, {
                        x: 756, y: 756
                    }, {
                        x: 754, y: 424
                    }, {
                        x: 725, y: 320
                    }, {
                        x: 724, y: 560
                    }, {
                        x: 724, y: 320
                    }, {
                        x: 720, y: 1280
                    }, {
                        x: 720, y: 1080
                    }, {
                        x: 720, y: 720
                    }, {
                        x: 672, y: 672
                    }, {
                        x: 672, y: 602
                    }, {
                        x: 666, y: 666
                    }, {
                        x: 648, y: 320
                    }, {
                        x: 640, y: 640
                    }, {
                        x: 640, y: 320
                    }, {
                        x: 598, y: 296
                    }, {
                        x: 594, y: 594
                    }, {
                        x: 592, y: 592
                    }, {
                        x: 588, y: 588
                    }, {
                        x: 584, y: 800
                    }, {
                        x: 580, y: 290
                    }, {
                        x: 576, y: 576
                    }, {
                        x: 562, y: 316
                    }, {
                        x: 556, y: 740
                    }, {
                        x: 553, y: 553
                    }, {
                        x: 540, y: 720
                    }, {
                        x: 526, y: 720
                    }, {
                        x: 518, y: 518
                    }, {
                        x: 496, y: 496
                    }, {
                        x: 480, y: 720
                    }, {
                        x: 480, y: 640
                    }, {
                        x: 480, y: 480
                    }, {
                        x: 480, y: 270
                    }, {
                        x: 470, y: 264
                    }, {
                        x: 467, y: 640
                    }, {
                        x: 464, y: 636
                    }, {
                        x: 464, y: 497
                    }, {
                        x: 464, y: 464
                    }, {
                        x: 448, y: 448
                    }, {
                        x: 441, y: 441
                    }, {
                        x: 430, y: 430
                    }, {
                        x: 426, y: 640
                    }, {
                        x: 424, y: 424
                    }, {
                        x: 423, y: 423
                    }, {
                        x: 420, y: 420
                    }, {
                        x: 420, y: 320
                    }, {
                        x: 420, y: 265
                    }, {
                        x: 420, y: 236
                    }, {
                        x: 420, y: 95
                    }, {
                        x: 402, y: 402
                    }, {
                        x: 400, y: 300
                    }, {
                        x: 395, y: 270
                    }, {
                        x: 380, y: 407
                    }, {
                        x: 380, y: 380
                    }, {
                        x: 380, y: 239
                    }, {
                        x: 380, y: 214
                    }, {
                        x: 380, y: 181
                    }, {
                        x: 370, y: 500
                    }, {
                        x: 370, y: 370
                    }, {
                        x: 370, y: 208
                    }, {
                        x: 367, y: 329
                    }, {
                        x: 366, y: 206
                    }, {
                        x: 361, y: 361
                    }, {
                        x: 347, y: 195
                    }, {
                        x: 343, y: 343
                    }, {
                        x: 336, y: 336
                    }, {
                        x: 329, y: 329
                    }, {
                        x: 327, y: 245
                    }, {
                        x: 320, y: 426
                    }, {
                        x: 320, y: 320
                    }, {
                        x: 320, y: 120
                    }, {
                        x: 318, y: 424
                    }, {
                        x: 316, y: 316
                    }, {
                        x: 313, y: 313
                    }, {
                        x: 313, y: 235
                    }, {
                        x: 310, y: 424
                    }, {
                        x: 307, y: 307
                    }, {
                        x: 300, y: 300
                    }, {
                        x: 296, y: 145
                    }, {
                        x: 290, y: 290
                    }, {
                        x: 290, y: 145
                    }, {
                        x: 288, y: 384
                    }, {
                        x: 288, y: 288
                    }, {
                        x: 285, y: 195
                    }, {
                        x: 282, y: 424
                    }, {
                        x: 280, y: 280
                    }, {
                        x: 278, y: 370
                    }, {
                        x: 275, y: 206
                    }, {
                        x: 270, y: 270
                    }, {
                        x: 267, y: 150
                    }, {
                        x: 265, y: 362
                    }, {
                        x: 265, y: 265
                    }, {
                        x: 262, y: 262
                    }, {
                        x: 262, y: 235
                    }, {
                        x: 260, y: 356
                    }, {
                        x: 258, y: 258
                    }, {
                        x: 258, y: 194
                    }, {
                        x: 253, y: 253
                    }, {
                        x: 248, y: 248
                    }, {
                        x: 245, y: 245
                    }, {
                        x: 243, y: 137
                    }, {
                        x: 240, y: 240
                    }, {
                        x: 237, y: 316
                    }, {
                        x: 235, y: 235
                    }, {
                        x: 234, y: 320
                    }, {
                        x: 234, y: 234
                    }, {
                        x: 230, y: 316
                    }, {
                        x: 224, y: 224
                    }, {
                        x: 219, y: 300
                    }, {
                        x: 219, y: 150
                    }, {
                        x: 215, y: 294
                    }, {
                        x: 215, y: 215
                    }, {
                        x: 213, y: 213
                    }, {
                        x: 213, y: 120
                    }, {
                        x: 210, y: 316
                    }, {
                        x: 210, y: 210
                    }, {
                        x: 210, y: 207
                    }, {
                        x: 210, y: 204
                    }, {
                        x: 210, y: 188
                    }, {
                        x: 208, y: 208
                    }, {
                        x: 207, y: 210
                    }, {
                        x: 200, y: 300
                    }, {
                        x: 200, y: 200
                    }, {
                        x: 181, y: 181
                    }, {
                        x: 181, y: 102
                    }, {
                        x: 180, y: 245
                    }, {
                        x: 180, y: 180
                    }, {
                        x: 178, y: 100
                    }, {
                        x: 177, y: 133
                    }, {
                        x: 175, y: 120
                    }, {
                        x: 173, y: 173
                    }, {
                        x: 172, y: 258
                    }, {
                        x: 170, y: 170
                    }, {
                        x: 164, y: 164
                    }, {
                        x: 164, y: 115
                    }, {
                        x: 160, y: 160
                    }, {
                        x: 160, y: 120
                    }, {
                        x: 156, y: 208
                    }, {
                        x: 154, y: 116
                    }, {
                        x: 152, y: 208
                    }, {
                        x: 151, y: 151
                    }, {
                        x: 150, y: 225
                    }, {
                        x: 150, y: 150
                    }, {
                        x: 150, y: 134
                    }, {
                        x: 150, y: 103
                    }, {
                        x: 150, y: 84
                    }, {
                        x: 148, y: 148
                    }, {
                        x: 147, y: 150
                    }, {
                        x: 145, y: 193
                    }, {
                        x: 144, y: 108
                    }, {
                        x: 138, y: 208
                    }, {
                        x: 136, y: 102
                    }, {
                        x: 135, y: 135
                    }, {
                        x: 132, y: 181
                    }, {
                        x: 132, y: 132
                    }, {
                        x: 131, y: 131
                    }, {
                        x: 128, y: 128
                    }, {
                        x: 124, y: 115
                    }, {
                        x: 120, y: 120
                    }, {
                        x: 120, y: 90
                    }, {
                        x: 112, y: 84
                    }, {
                        x: 110, y: 110
                    }, {
                        x: 108, y: 108
                    }, {
                        x: 108, y: 72
                    }, {
                        x: 107, y: 160
                    }, {
                        x: 105, y: 105
                    }, {
                        x: 100, y: 135
                    }, {
                        x: 100, y: 120
                    }, {
                        x: 100, y: 100
                    }, {
                        x: 99, y: 135
                    }, {
                        x: 99, y: 99
                    }, {
                        x: 95, y: 95
                    }, {
                        x: 86, y: 86
                    }, {
                        x: 85, y: 120
                    }, {
                        x: 85, y: 85
                    }, {
                        x: 84, y: 84
                    }, {
                        x: 84, y: 56
                    }, {
                        x: 80, y: 60
                    }, {
                        x: 75, y: 75
                    }, {
                        x: 72, y: 72
                    }, {
                        x: 68, y: 38
                    }, {
                        x: 64, y: 64
                    }, {
                        x: 64, y: 48
                    }, {
                        x: 60, y: 90
                    }, {
                        x: 60, y: 60
                    }, {
                        x: 60, y: 45
                    }, {
                        x: 60, y: 40
                    }, {
                        x: 56, y: 56
                    }, {
                        x: 52, y: 52
                    }, {
                        x: 50, y: 50
                    }, {
                        x: 46, y: 63
                    }, {
                        x: 46, y: 60
                    }, {
                        x: 44, y: 44
                    }, {
                        x: 43, y: 64
                    }, {
                        x: 40, y: 40
                    }, {
                        x: 38, y: 38
                    }, {
                        x: 34, y: 34
                    }], _acceptableWideValuesEDS: [{
                        x: 2560, y: 1440
                    }, {
                        x: 2471, y: 753
                    }, {
                        x: 2399, y: 576
                    }, {
                        x: 2398, y: 1642
                    }, {
                        x: 2398, y: 576
                    }, {
                        x: 2100, y: 753
                    }, {
                        x: 2048, y: 1536
                    }, {
                        x: 2021, y: 576
                    }, {
                        x: 2020, y: 576
                    }, {
                        x: 1920, y: 1080
                    }, {
                        x: 1919, y: 1200
                    }, {
                        x: 1919, y: 1079
                    }, {
                        x: 1831, y: 540
                    }, {
                        x: 1830, y: 1391
                    }, {
                        x: 1830, y: 540
                    }, {
                        x: 1795, y: 448
                    }, {
                        x: 1793, y: 448
                    }, {
                        x: 1593, y: 896
                    }, {
                        x: 1562, y: 540
                    }, {
                        x: 1561, y: 1069
                    }, {
                        x: 1561, y: 540
                    }, {
                        x: 1501, y: 448
                    }, {
                        x: 1499, y: 1028
                    }, {
                        x: 1440, y: 1291
                    }, {
                        x: 1440, y: 810
                    }, {
                        x: 1366, y: 768
                    }, {
                        x: 1351, y: 380
                    }, {
                        x: 1315, y: 448
                    }, {
                        x: 1280, y: 800
                    }, {
                        x: 1280, y: 720
                    }, {
                        x: 1277, y: 320
                    }, {
                        x: 1276, y: 320
                    }, {
                        x: 1191, y: 320
                    }, {
                        x: 1138, y: 640
                    }, {
                        x: 1080, y: 789
                    }, {
                        x: 1067, y: 320
                    }, {
                        x: 1066, y: 810
                    }, {
                        x: 1024, y: 768
                    }, {
                        x: 1021, y: 448
                    }, {
                        x: 1012, y: 693
                    }, {
                        x: 981, y: 320
                    }, {
                        x: 935, y: 320
                    }, {
                        x: 934, y: 320
                    }, {
                        x: 854, y: 480
                    }, {
                        x: 853, y: 480
                    }, {
                        x: 796, y: 448
                    }, {
                        x: 770, y: 485
                    }, {
                        x: 754, y: 424
                    }, {
                        x: 725, y: 320
                    }, {
                        x: 724, y: 560
                    }, {
                        x: 724, y: 320
                    }, {
                        x: 672, y: 602
                    }, {
                        x: 648, y: 320
                    }, {
                        x: 640, y: 320
                    }, {
                        x: 598, y: 296
                    }, {
                        x: 580, y: 290
                    }, {
                        x: 562, y: 316
                    }, {
                        x: 480, y: 270
                    }, {
                        x: 470, y: 264
                    }, {
                        x: 420, y: 320
                    }, {
                        x: 420, y: 265
                    }, {
                        x: 420, y: 236
                    }, {
                        x: 420, y: 95
                    }, {
                        x: 400, y: 300
                    }, {
                        x: 395, y: 270
                    }, {
                        x: 380, y: 239
                    }, {
                        x: 380, y: 214
                    }, {
                        x: 380, y: 181
                    }, {
                        x: 370, y: 208
                    }, {
                        x: 367, y: 329
                    }, {
                        x: 366, y: 206
                    }, {
                        x: 347, y: 195
                    }, {
                        x: 327, y: 245
                    }, {
                        x: 320, y: 120
                    }, {
                        x: 313, y: 235
                    }, {
                        x: 296, y: 145
                    }, {
                        x: 290, y: 145
                    }, {
                        x: 285, y: 195
                    }, {
                        x: 275, y: 206
                    }, {
                        x: 267, y: 150
                    }, {
                        x: 262, y: 235
                    }, {
                        x: 258, y: 194
                    }, {
                        x: 243, y: 137
                    }, {
                        x: 219, y: 150
                    }, {
                        x: 213, y: 120
                    }, {
                        x: 210, y: 207
                    }, {
                        x: 210, y: 204
                    }, {
                        x: 210, y: 188
                    }, {
                        x: 181, y: 102
                    }, {
                        x: 178, y: 100
                    }, {
                        x: 177, y: 133
                    }, {
                        x: 175, y: 120
                    }, {
                        x: 164, y: 115
                    }, {
                        x: 160, y: 120
                    }, {
                        x: 154, y: 116
                    }, {
                        x: 150, y: 134
                    }, {
                        x: 150, y: 103
                    }, {
                        x: 150, y: 84
                    }, {
                        x: 144, y: 108
                    }, {
                        x: 136, y: 102
                    }, {
                        x: 124, y: 115
                    }, {
                        x: 120, y: 90
                    }, {
                        x: 112, y: 84
                    }, {
                        x: 108, y: 72
                    }, {
                        x: 84, y: 56
                    }, {
                        x: 80, y: 60
                    }, {
                        x: 68, y: 38
                    }, {
                        x: 64, y: 48
                    }, {
                        x: 60, y: 45
                    }, {
                        x: 60, y: 40
                    }], _acceptableSquareValuesEDS: [{
                        x: 2470, y: 2470
                    }, {
                        x: 2398, y: 2398
                    }, {
                        x: 2100, y: 2100
                    }, {
                        x: 2020, y: 2020
                    }, {
                        x: 1793, y: 1793
                    }, {
                        x: 1562, y: 1562
                    }, {
                        x: 1561, y: 1561
                    }, {
                        x: 1440, y: 1440
                    }, {
                        x: 1350, y: 1350
                    }, {
                        x: 1315, y: 1315
                    }, {
                        x: 1276, y: 1276
                    }, {
                        x: 1206, y: 1206
                    }, {
                        x: 1190, y: 1190
                    }, {
                        x: 1080, y: 1080
                    }, {
                        x: 1051, y: 1051
                    }, {
                        x: 1021, y: 1021
                    }, {
                        x: 980, y: 980
                    }, {
                        x: 938, y: 938
                    }, {
                        x: 934, y: 934
                    }, {
                        x: 896, y: 896
                    }, {
                        x: 864, y: 864
                    }, {
                        x: 818, y: 818
                    }, {
                        x: 800, y: 800
                    }, {
                        x: 770, y: 770
                    }, {
                        x: 756, y: 756
                    }, {
                        x: 754, y: 424
                    }, {
                        x: 720, y: 720
                    }, {
                        x: 672, y: 672
                    }, {
                        x: 666, y: 666
                    }, {
                        x: 640, y: 640
                    }, {
                        x: 594, y: 594
                    }, {
                        x: 592, y: 592
                    }, {
                        x: 588, y: 588
                    }, {
                        x: 580, y: 290
                    }, {
                        x: 576, y: 576
                    }, {
                        x: 562, y: 316
                    }, {
                        x: 553, y: 553
                    }, {
                        x: 518, y: 518
                    }, {
                        x: 496, y: 496
                    }, {
                        x: 480, y: 480
                    }, {
                        x: 464, y: 464
                    }, {
                        x: 448, y: 448
                    }, {
                        x: 441, y: 441
                    }, {
                        x: 430, y: 430
                    }, {
                        x: 424, y: 424
                    }, {
                        x: 423, y: 423
                    }, {
                        x: 420, y: 420
                    }, {
                        x: 402, y: 402
                    }, {
                        x: 380, y: 380
                    }, {
                        x: 370, y: 370
                    }, {
                        x: 361, y: 361
                    }, {
                        x: 343, y: 343
                    }, {
                        x: 336, y: 336
                    }, {
                        x: 329, y: 329
                    }, {
                        x: 320, y: 320
                    }, {
                        x: 316, y: 316
                    }, {
                        x: 313, y: 313
                    }, {
                        x: 307, y: 307
                    }, {
                        x: 300, y: 300
                    }, {
                        x: 290, y: 290
                    }, {
                        x: 288, y: 288
                    }, {
                        x: 280, y: 280
                    }, {
                        x: 270, y: 270
                    }, {
                        x: 265, y: 265
                    }, {
                        x: 262, y: 262
                    }, {
                        x: 258, y: 258
                    }, {
                        x: 253, y: 253
                    }, {
                        x: 248, y: 248
                    }, {
                        x: 245, y: 245
                    }, {
                        x: 240, y: 240
                    }, {
                        x: 235, y: 235
                    }, {
                        x: 234, y: 234
                    }, {
                        x: 224, y: 224
                    }, {
                        x: 215, y: 215
                    }, {
                        x: 213, y: 213
                    }, {
                        x: 210, y: 210
                    }, {
                        x: 208, y: 208
                    }, {
                        x: 200, y: 200
                    }, {
                        x: 181, y: 181
                    }, {
                        x: 180, y: 180
                    }, {
                        x: 173, y: 173
                    }, {
                        x: 170, y: 170
                    }, {
                        x: 164, y: 164
                    }, {
                        x: 160, y: 160
                    }, {
                        x: 151, y: 151
                    }, {
                        x: 150, y: 150
                    }, {
                        x: 148, y: 148
                    }, {
                        x: 135, y: 135
                    }, {
                        x: 132, y: 132
                    }, {
                        x: 131, y: 131
                    }, {
                        x: 128, y: 128
                    }, {
                        x: 120, y: 120
                    }, {
                        x: 110, y: 110
                    }, {
                        x: 108, y: 108
                    }, {
                        x: 105, y: 105
                    }, {
                        x: 100, y: 100
                    }, {
                        x: 99, y: 99
                    }, {
                        x: 95, y: 95
                    }, {
                        x: 86, y: 86
                    }, {
                        x: 85, y: 85
                    }, {
                        x: 84, y: 84
                    }, {
                        x: 75, y: 75
                    }, {
                        x: 72, y: 72
                    }, {
                        x: 64, y: 64
                    }, {
                        x: 60, y: 60
                    }, {
                        x: 56, y: 56
                    }, {
                        x: 52, y: 52
                    }, {
                        x: 50, y: 50
                    }, {
                        x: 44, y: 44
                    }, {
                        x: 40, y: 40
                    }, {
                        x: 38, y: 38
                    }, {
                        x: 34, y: 34
                    }], _acceptableTallValuesEDS: [{
                        x: 720, y: 1280
                    }, {
                        x: 810, y: 1080
                    }, {
                        x: 789, y: 1080
                    }, {
                        x: 788, y: 1080
                    }, {
                        x: 720, y: 1080
                    }, {
                        x: 584, y: 800
                    }, {
                        x: 556, y: 740
                    }, {
                        x: 540, y: 720
                    }, {
                        x: 526, y: 720
                    }, {
                        x: 480, y: 720
                    }, {
                        x: 480, y: 640
                    }, {
                        x: 467, y: 640
                    }, {
                        x: 426, y: 640
                    }, {
                        x: 464, y: 636
                    }, {
                        x: 370, y: 500
                    }, {
                        x: 464, y: 497
                    }, {
                        x: 320, y: 426
                    }, {
                        x: 318, y: 424
                    }, {
                        x: 310, y: 424
                    }, {
                        x: 282, y: 424
                    }, {
                        x: 380, y: 407
                    }, {
                        x: 288, y: 384
                    }, {
                        x: 278, y: 370
                    }, {
                        x: 265, y: 362
                    }, {
                        x: 260, y: 356
                    }, {
                        x: 234, y: 320
                    }, {
                        x: 237, y: 316
                    }, {
                        x: 230, y: 316
                    }, {
                        x: 210, y: 316
                    }, {
                        x: 219, y: 300
                    }, {
                        x: 200, y: 300
                    }, {
                        x: 215, y: 294
                    }, {
                        x: 172, y: 258
                    }, {
                        x: 180, y: 245
                    }, {
                        x: 150, y: 225
                    }, {
                        x: 207, y: 210
                    }, {
                        x: 156, y: 208
                    }, {
                        x: 152, y: 208
                    }, {
                        x: 138, y: 208
                    }, {
                        x: 145, y: 193
                    }, {
                        x: 132, y: 181
                    }, {
                        x: 107, y: 160
                    }, {
                        x: 147, y: 150
                    }, {
                        x: 100, y: 135
                    }, {
                        x: 99, y: 135
                    }, {
                        x: 100, y: 120
                    }, {
                        x: 85, y: 120
                    }, {
                        x: 60, y: 90
                    }, {
                        x: 43, y: 64
                    }, {
                        x: 46, y: 63
                    }, {
                        x: 46, y: 60
                    }], _acceptableWidthsEDS: [2560, 2471, 2470, 2399, 2398, 2100, 2048, 2021, 2020, 1920, 1919, 1831, 1830, 1795, 1793, 1593, 1562, 1561, 1501, 1499, 1440, 1366, 1351, 1350, 1315, 1280, 1277, 1276, 1206, 1191, 1190, 1138, 1080, 1067, 1066, 1051, 1024, 1021, 1012, 981, 980, 938, 935, 934, 896, 864, 854, 853, 818, 810, 800, 796, 789, 788, 770, 756, 754, 725, 724, 720, 672, 666, 648, 640, 598, 594, 592, 588, 584, 580, 576, 562, 556, 553, 540, 526, 518, 496, 480, 470, 467, 464, 448, 441, 430, 426, 424, 423, 420, 402, 400, 395, 380, 370, 367, 366, 361, 347, 343, 336, 329, 327, 320, 318, 316, 313, 310, 307, 300, 296, 290, 288, 285, 282, 280, 278, 275, 270, 267, 265, 262, 260, 258, 253, 248, 245, 243, 240, 237, 235, 234, 230, 224, 219, 215, 213, 210, 208, 207, 200, 181, 180, 178, 177, 175, 173, 172, 170, 164, 160, 156, 154, 152, 151, 150, 148, 147, 145, 144, 138, 136, 135, 132, 131, 128, 124, 120, 112, 110, 108, 107, 105, 100, 99, 95, 86, 85, 84, 80, 75, 72, 68, 64, 60, 56, 52, 50, 46, 44, 43, 40, 38, 34], _acceptableHeightsEDS: [2470, 2398, 2100, 2020, 1793, 1642, 1562, 1561, 1536, 1440, 1391, 1350, 1315, 1291, 1280, 1276, 1206, 1200, 1190, 1080, 1079, 1069, 1051, 1028, 1021, 980, 938, 934, 896, 864, 818, 810, 800, 789, 770, 768, 756, 753, 740, 720, 693, 672, 666, 640, 636, 602, 594, 592, 588, 576, 560, 553, 540, 518, 500, 497, 496, 485, 480, 464, 448, 441, 430, 426, 424, 423, 420, 407, 402, 384, 380, 370, 362, 361, 356, 343, 336, 329, 320, 316, 313, 307, 300, 296, 294, 290, 288, 280, 270, 265, 264, 262, 258, 253, 248, 245, 240, 239, 236, 235, 234, 225, 224, 215, 214, 213, 210, 208, 207, 206, 204, 200, 195, 194, 193, 188, 181, 180, 173, 170, 164, 160, 151, 150, 148, 145, 137, 135, 134, 133, 132, 131, 128, 120, 116, 115, 110, 108, 105, 103, 102, 100, 99, 95, 90, 86, 85, 84, 75, 72, 64, 63, 60, 56, 52, 50, 48, 45, 44, 40, 38, 34], makeCatalogImageUri: function makeCatalogImageUri(imageId, imageIdType, dimensions, forceImageResize, ignoreZeroLengths, requestedImage, imageChildId, imageContentType) {
                    var url = null;
                    var idTypes = MS.Entertainment.Data.ImageIdType;
                    var verifiedRequestedImage;
                    var verifiedDimensions;
                    var verifiedImageIdType;
                    var paramArray = [];
                    if (!imageId)
                        throw"Must specify a valid catalog image guid";
                    if (imageContentType === null || imageContentType === undefined)
                        imageContentType = MS.Entertainment.ImageContentType.jpeg;
                    if (imageIdType === null || imageIdType === undefined)
                        verifiedImageIdType = MS.Entertainment.Data.ImageIdType.image;
                    else {
                        MS.Entertainment.Utilities.validateIsMemberOrThrow(imageIdType, MS.Entertainment.Data.ImageIdType);
                        verifiedImageIdType = imageIdType
                    }
                    if (!requestedImage)
                        verifiedRequestedImage = this._getStringFromRequestedImage(MS.Entertainment.ImageRequested.primaryImage);
                    else {
                        MS.Entertainment.Utilities.validateIsMemberOrThrow(requestedImage, MS.Entertainment.ImageRequested);
                        verifiedRequestedImage = this._getStringFromRequestedImage(requestedImage)
                    }
                    verifiedDimensions = {
                        x: 0, y: 0
                    };
                    if (dimensions) {
                        if (dimensions.hasOwnProperty("x") && dimensions.x)
                            verifiedDimensions.x = dimensions.x;
                        if (dimensions.hasOwnProperty("y") && dimensions.y)
                            verifiedDimensions.y = dimensions.y
                    }
                    try {
                        var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                        MS.Entertainment.UI.Shell.ImageLoader._initializeEndpointCache();
                        switch (verifiedImageIdType) {
                            case idTypes.movie:
                            case idTypes.editorialMovie:
                                if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.moviesMarketplace))
                                    url = "{0}/movie/{1}/{2}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, verifiedRequestedImage);
                                break;
                            case idTypes.artist:
                            case idTypes.editorialArtist:
                                if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace))
                                    url = "{0}/music/artist/{1}/{2}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, verifiedRequestedImage);
                                break;
                            case idTypes.album:
                            case idTypes.editorialAlbum:
                                if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace))
                                    url = "{0}/music/album/{1}/{2}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, verifiedRequestedImage);
                                break;
                            case idTypes.track:
                            case idTypes.editorialTrack:
                                if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace))
                                    url = "{0}/music/track/{1}/{2}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, verifiedRequestedImage);
                                break;
                            case idTypes.musicVideo:
                                if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicVideosMarketplace))
                                    url = "{0}/music/musicvideo/{1}/{2}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, verifiedRequestedImage);
                                break;
                            case idTypes.movieTrailer:
                                if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.movieTrailersMarketplace))
                                    url = "{0}/movieTrailer/{1}/{2}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, verifiedRequestedImage);
                                break;
                            case idTypes.playlist:
                                if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace))
                                    url = "{0}/music/playlist/{1}/{2}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, verifiedRequestedImage);
                                break;
                            case idTypes.podcast:
                                url = "{0}/podcast/{1}/{2}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, verifiedRequestedImage);
                                break;
                            case idTypes.network:
                                if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.tvMarketplace))
                                    url = "{0}/tv/network/{1}/{2}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, verifiedRequestedImage);
                                break;
                            case idTypes.series:
                            case idTypes.editorialSeries:
                                if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.tvMarketplace))
                                    url = "{0}/tv/series/{1}/{2}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, verifiedRequestedImage);
                                break;
                            case idTypes.episode:
                                if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.tvMarketplace))
                                    url = "{0}/tv/episode/{1}/{2}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, verifiedRequestedImage);
                                break;
                            case idTypes.application:
                                url = "{0}/apps/{1}/{2}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, verifiedRequestedImage);
                                break;
                            case idTypes.season:
                            case idTypes.editorialSeason:
                                if (typeof imageChildId === "undefined")
                                    throw"imageChildId must be specified for seasons";
                                if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.tvMarketplace))
                                    url = "{0}/tv/series/{1}/seasons/{2}/{3}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId, imageChildId, verifiedRequestedImage);
                                break;
                            case idTypes.image:
                                url = "{0}/image/{1}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageId);
                                break;
                            case idTypes.studio:
                                if (typeof imageChildId === "undefined")
                                    throw"imageChildId must be specified for studios";
                                url = "{0}/{1}/studio/{2}/{3}".format(MS.Entertainment.UI.Shell.ImageLoader._endpointCache.imageCatalog, imageChildId, imageId, verifiedRequestedImage);
                                break;
                            default:
                                throw"unknown imageidtype";
                        }
                    }
                    catch(error) {
                        return null
                    }
                    if (forceImageResize)
                        paramArray.push({
                            name: "resize", value: "true"
                        });
                    var scaleFactor = this._getCurrentScaleFactor();
                    var calcSize = this._calculateImageUriSize(verifiedDimensions, scaleFactor, MS.Entertainment.UI.Shell.ImageLoader.ImageService.Zune);
                    if (verifiedDimensions.x > 0 || !ignoreZeroLengths)
                        paramArray.push({
                            name: "width", value: calcSize.x.toString()
                        });
                    if (verifiedDimensions.y > 0 || !ignoreZeroLengths)
                        paramArray.push({
                            name: "height", value: calcSize.y.toString()
                        });
                    if (imageContentType)
                        paramArray.push({
                            name: "contenttype", value: imageContentType
                        });
                    if (url && paramArray.length > 0)
                        url = url + this._generateUrlParamSubstring(paramArray);
                    return url
                }, _calculateImageUriSize: function _calculateImageUriSize(size, scaleFactor, imageService) {
                    var calcSize = {
                            x: size.x, y: size.y
                        };
                    if (size.x === 0)
                        if (size.y === 0)
                            calcSize = this._findAcceptableWidthHeight(this._defaultImageSize.x * scaleFactor, this._defaultImageSize.y * scaleFactor, imageService);
                        else
                            calcSize = this._findMatchingWidth(size.y * scaleFactor, imageService);
                    else if (size.y === 0)
                        calcSize = this._findMatchingHeight(size.x * scaleFactor, imageService);
                    else {
                        var scaledX = size.x * scaleFactor;
                        var scaledY = size.y * scaleFactor;
                        calcSize = this._findAcceptableWidthHeight(scaledX, scaledY, imageService)
                    }
                    return calcSize
                }, _findAcceptableWidthHeight: function _findAcceptableWidthHeight(width, height, imageService) {
                    var ImageLoader = MS.Entertainment.UI.Shell.ImageLoader;
                    var calcSize = {
                            x: -1, y: 0
                        };
                    var index = 0;
                    var aspectAcceptableValues = [];
                    var searchSet = [];
                    if (width === height)
                        aspectAcceptableValues = ImageLoader._getAcceptableSquareValuesArray(imageService);
                    else if (width > height)
                        aspectAcceptableValues = ImageLoader._getAcceptableWideValuesArray(imageService);
                    else
                        aspectAcceptableValues = ImageLoader._getAcceptableTallValuesArray(imageService);
                    for (var index = 0; index < aspectAcceptableValues.length; index++)
                        if (calcSize.x > -1)
                            if (calcSize.x === aspectAcceptableValues[index].x)
                                searchSet.push(aspectAcceptableValues[index]);
                            else
                                break;
                        else if (aspectAcceptableValues[index].x <= width) {
                            if ((aspectAcceptableValues[index].x < width) && (index > 0))
                                index--;
                            calcSize.x = aspectAcceptableValues[index].x;
                            searchSet.push(aspectAcceptableValues[index])
                        }
                    for (var index = 0; index < searchSet.length; index++)
                        if (searchSet[index].y <= height) {
                            calcSize.y = searchSet[index].y;
                            break
                        }
                    if (calcSize.x === -1) {
                        var size = aspectAcceptableValues[aspectAcceptableValues.length - 1];
                        calcSize.x = size.x;
                        calcSize.y = size.y
                    }
                    if (calcSize.y === 0)
                        calcSize.y = searchSet[searchSet.length - 1].y;
                    return calcSize
                }, _findMatchingWidth: function _findMatchingWidth(height, imageService) {
                    var ImageLoader = MS.Entertainment.UI.Shell.ImageLoader;
                    var acceptableHeights = ImageLoader._getAcceptableHeightsArray(imageService);
                    var acceptableValues = ImageLoader._getAcceptableValuesArray(imageService);
                    var calcSize = {
                            x: 0, y: height
                        };
                    var index = 0;
                    while (index < acceptableHeights.length - 1 && height < acceptableHeights[index])
                        index = index + 1;
                    calcSize.y = acceptableHeights[index];
                    var i;
                    for (i = 0; i < acceptableValues.length; i++)
                        if (acceptableValues[i].y === calcSize.y) {
                            calcSize.x = acceptableValues[i].x;
                            break
                        }
                    return calcSize
                }, _findMatchingHeight: function _findMatchingHeight(width, imageService) {
                    var ImageLoader = MS.Entertainment.UI.Shell.ImageLoader;
                    var acceptableWidths = ImageLoader._getAcceptableWidthsArray(imageService);
                    var acceptableValues = ImageLoader._getAcceptableValuesArray(imageService);
                    var calcSize = {
                            x: width, y: 0
                        };
                    var index = 0;
                    while (index < acceptableWidths.length - 1 && width < acceptableWidths[index])
                        index = index + 1;
                    calcSize.x = acceptableWidths[index];
                    var i;
                    for (i = 0; i < acceptableValues.length; i++)
                        if (acceptableValues[i].x === calcSize.x) {
                            calcSize.y = acceptableValues[i].y;
                            break
                        }
                    return calcSize
                }, _generateUrlParamSubstring: function _generateUrlParamSubstring(params) {
                    var paramString = "";
                    for (var i = 0; i < params.length; i++) {
                        if (i === 0)
                            paramString = paramString + "?";
                        else
                            paramString = paramString + "&";
                        paramString = paramString + params[i].name.toString();
                        paramString = paramString + "=";
                        paramString = paramString + params[i].value.toString()
                    }
                    return paramString
                }, _getStringFromRequestedImage: function _getStringFromRequestedImage(requestedImage) {
                    for (var k in MS.Entertainment.ImageRequested)
                        if (MS.Entertainment.ImageRequested[k] === requestedImage)
                            return k;
                    MS.Entertainment.UI.Shell.assert(false, "Invalid value for ImageRequested");
                    return null
                }, imageManager: {get: function getImageManager() {
                        if (!this._imageManager)
                            this._imageManager = new Microsoft.Entertainment.ImageManager;
                        return this._imageManager
                    }}, cacheImage: function cacheUrl(url, defaultImage) {
                    return this.imageManager.retrieveImageFromUrlAsync(Microsoft.Entertainment.NetworkUsage.normal, url, String.empty, defaultImage)
                }, calculateScaledSize: function calculateScaledSize(width, height) {
                    var size = {
                            x: 0, y: 0
                        };
                    if (width)
                        size.x = width;
                    if (height)
                        size.y = height;
                    MS.Entertainment.UI.Shell.assert(size.x || (MS.Entertainment.UI.Shell.ImageLoader._acceptableWidthsEDS.indexOf(size.x) > -1), "ImageLoader: pixel width is not an acceptable value");
                    MS.Entertainment.UI.Shell.assert(size.y || (MS.Entertainment.UI.Shell.ImageLoader._acceptableHeightsEDS.indexOf(size.y) > -1), "ImageLoader: pixel height is not an acceptable value");
                    var scaleFactor = this._getCurrentScaleFactor();
                    return this._calculateImageUriSize(size, scaleFactor)
                }, _getCurrentScaleFactor: function _getCurrentScaleFactor() {
                    var scaleFactor = 1.0;
                    switch (MS.Entertainment.Utilities.getDisplayProperties().resolutionScale) {
                        case Windows.Graphics.Display.ResolutionScale.scale180Percent:
                            scaleFactor = 1.4;
                            break;
                        case Windows.Graphics.Display.ResolutionScale.scale140Percent:
                        case Windows.Graphics.Display.ResolutionScale.scale100Percent:
                            scaleFactor = 1.0;
                            break;
                        default:
                            MS.Entertainment.UI.Shell.assert(false, "Unknown scale size requested");
                            scaleFactor = 1.0;
                            break
                    }
                    return scaleFactor
                }, getServiceImageRequestSize: function getServiceImageRequestSize(initialSize) {
                    var scaleFactor = this._getCurrentScaleFactor();
                    return this._calculateImageUriSize(initialSize, scaleFactor)
                }, getServiceImageUrl: function getServiceImageUrl(mediaItem, width, height, requestedImage, imageIdType, defaultImageUri, imageContentType) {
                    var resultPromise;
                    var imageId = null;
                    var data = null;
                    var convertedItem = null;
                    var verifiedRequestedImage;
                    var asyncUri;
                    if (!mediaItem)
                        return WinJS.Promise.wrap();
                    if (width === "screen")
                        width = MS.Entertainment.Utilities.getLandscapeScreenSize().width;
                    if (height === "screen")
                        width = MS.Entertainment.Utilities.getLandscapeScreenSize().height;
                    if (!requestedImage)
                        verifiedRequestedImage = MS.Entertainment.ImageRequested.primaryImage;
                    else {
                        MS.Entertainment.Utilities.validateIsMemberOrThrow(requestedImage, MS.Entertainment.ImageRequested);
                        verifiedRequestedImage = requestedImage
                    }
                    if (mediaItem.serviceType === MS.Entertainment.Data.Augmenter.ServiceTypes.editorialItem) {
                        convertedItem = MS.Entertainment.Utilities.convertEditorialItem(mediaItem);
                        mediaItem.mediaType = convertedItem.mediaType;
                        mediaItem.videoType = convertedItem.videoType;
                        imageIdType = imageIdType || convertedItem.serviceImageType;
                        if (!mediaItem.mediaType && MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Hub)
                            mediaItem.mediaType = Microsoft.Entertainment.Queries.ObjectType.editorial
                    }
                    if (mediaItem.serviceId && mediaItem.serviceId !== MS.Entertainment.Utilities.EMPTY_GUID)
                        imageId = mediaItem.serviceId;
                    var mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.defaultImage;
                    height = height || MS.Entertainment.UI.Shell.ImageLoader.DefaultThumbnailSizes.defaultWidth;
                    imageIdType = imageIdType || mediaItem.serviceImageType;
                    switch (mediaItem.mediaType || mediaItem.serviceType) {
                        case Microsoft.Entertainment.Queries.ObjectType.video:
                            if (mediaItem.videoType === Microsoft.Entertainment.Queries.VideoType.musicVideo) {
                                mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.musicVideo;
                                imageIdType = imageIdType || MS.Entertainment.Data.ImageIdType.image;
                                width = width || 420
                            }
                            else if (mediaItem.videoType === Microsoft.Entertainment.Queries.VideoType.tvEpisode) {
                                mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.tvEpisode;
                                imageIdType = imageIdType || MS.Entertainment.Data.ImageIdType.episode;
                                width = width || height
                            }
                            else {
                                mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.movie;
                                imageIdType = imageIdType || MS.Entertainment.Data.ImageIdType.movie;
                                width = width || MS.Entertainment.UI.Shell.ImageLoader.DefaultThumbnailSizes.video
                            }
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeries:
                            mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.tvSeries;
                            imageIdType = imageIdType || MS.Entertainment.Data.ImageIdType.series;
                            width = width || height;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeason:
                            mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.tvSeason;
                            imageIdType = imageIdType || MS.Entertainment.Data.ImageIdType.season;
                            imageId = mediaItem.seriesId || (convertedItem ? convertedItem.seriesId : null);
                            data = mediaItem.seasonNumber;
                            if ((data === null || data === undefined) && convertedItem)
                                data = convertedItem.seasonNumber;
                            width = width || height;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                            mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.artist;
                            imageIdType = imageIdType || MS.Entertainment.Data.ImageIdType.artist;
                            width = width || 420;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.smartDJ:
                            mediaItemDefaultImage = "";
                            imageIdType = imageIdType || MS.Entertainment.Data.ImageIdType.artist;
                            width = width || 420;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                            mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.album;
                            imageIdType = imageIdType || MS.Entertainment.Data.ImageIdType.album;
                            width = width || height;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                            if (mediaItem.hasAlbumServiceId) {
                                mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.album;
                                imageIdType = imageIdType || MS.Entertainment.Data.ImageIdType.album;
                                imageId = mediaItem.albumServiceId
                            }
                            else {
                                mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.track;
                                imageIdType = imageIdType || MS.Entertainment.Data.ImageIdType.track;
                                verifiedRequestedImage = requestedImage || MS.Entertainment.ImageRequested.albumImage
                            }
                            width = width || height;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.game:
                            mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.game;
                            if (mediaItem.hexTitleId) {
                                imageId = parseInt(mediaItem.hexTitleId);
                                data = parseInt(mediaItem.hexTitleId);
                                imageIdType = imageIdType || MS.Entertainment.Data.ImageIdType.xboxGame
                            }
                            width = width || 234;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.playlist:
                            mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.playlist;
                            imageIdType = imageIdType || MS.Entertainment.Data.ImageIdType.playlist;
                            width = width || height;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.editorial:
                            imageIdType = MS.Entertainment.Data.ImageIdType.image;
                            width = width || height;
                            break;
                        case MS.Entertainment.Data.Augmenter.ServiceTypes.wmis:
                        case MS.Entertainment.Data.Augmenter.ServiceTypes.audioAd:
                            mediaItemDefaultImage = MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.album;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.folder:
                            width = width || 234;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.genre:
                            width = width || 240;
                            break;
                        default:
                            MS.Entertainment.UI.Shell.assert(false, "Unknown media type or service type had image requested, MediaType: " + mediaItem.mediaType + " ServiceType: " + mediaItem.serviceType + " ImageIdType: " + imageIdType);
                            width = 240;
                            break
                    }
                    if (imageIdType === MS.Entertainment.Data.ImageIdType.image && mediaItem.imageId && mediaItem.imageId !== MS.Entertainment.Utilities.EMPTY_GUID)
                        imageId = mediaItem.imageId;
                    if (typeof defaultImageUri !== "string")
                        defaultImageUri = mediaItemDefaultImage;
                    var calculatedSize = MS.Entertainment.UI.Shell.ImageLoader.calculateScaledSize(width, height);
                    if (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.person && MS.Entertainment.Utilities.isValidServiceId(mediaItem.zuneId))
                        asyncUri = MS.Entertainment.UI.Shell.ImageLoader.getNewImageServiceUri(mediaItem, calculatedSize.x, calculatedSize.y);
                    else if (mediaItem.imageResizeUri)
                        asyncUri = MS.Entertainment.UI.Shell.ImageLoader.appendResizeParameters(mediaItem.imageResizeUri, calculatedSize.x, calculatedSize.y);
                    else if (mediaItem.imageUri)
                        asyncUri = mediaItem.imageUri;
                    else if (mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.track || mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.album)
                        asyncUri = MS.Entertainment.UI.Shell.ImageLoader.getNewImageServiceUri(mediaItem, calculatedSize.x, calculatedSize.y);
                    else if (imageIdType !== null && imageIdType !== undefined && imageId && imageId !== MS.Entertainment.Utilities.EMPTY_GUID) {
                        var sizeUnscaled = {
                                x: width, y: height
                            };
                        asyncUri = this.makeCatalogImageUri(imageId, imageIdType, sizeUnscaled, true, false, verifiedRequestedImage, data, imageContentType)
                    }
                    if (asyncUri)
                        if (MS.Entertainment.UI.Shell.ImageLoader.isPackageUrl(asyncUri))
                            resultPromise = WinJS.Promise.wrap(asyncUri);
                        else
                            resultPromise = this.imageManager.retrieveImageFromUrlAsync(Microsoft.Entertainment.NetworkUsage.normal, asyncUri, mediaItem.filePath || String.empty, defaultImageUri);
                    else if (mediaItem.filePath && !MS.Entertainment.UI.Shell.ImageLoader.isWebUrl(mediaItem.filePath) && !MS.Entertainment.UI.Shell.ImageLoader.isBlobUrl(mediaItem.filePath) && !MS.Entertainment.UI.Shell.ImageLoader.isStreamingUrl(mediaItem.filePath))
                        resultPromise = this.imageManager.retrieveImageFromUrlAsync(Microsoft.Entertainment.NetworkUsage.normal, String.empty, mediaItem.filePath, defaultImageUri);
                    else if (mediaItem.imageUri)
                        if (mediaItem.filePath && MS.Entertainment.UI.Shell.ImageLoader.isBlobUrl(mediaItem.filePath) && MS.Entertainment.UI.Shell.ImageLoader.isBlobUrl(mediaItem.imageUri))
                            resultPromise = WinJS.Promise.wrap(mediaItem.imageUri);
                        else
                            resultPromise = this.imageManager.retrieveImageFromUrlAsync(Microsoft.Entertainment.NetworkUsage.normal, mediaItem.imageUri, mediaItem.filePath || String.empty, defaultImageUri);
                    else
                        resultPromise = WinJS.Promise.wrap(defaultImageUri);
                    return resultPromise
                }, getNewImageServiceUri: function getNewImageServiceUri(mediaItem, width, height, sources) {
                    if (!mediaItem || !MS.Entertainment.Utilities.isValidServiceId(mediaItem.zuneId))
                        return String.empty;
                    var uri = "{0}/catalog/music.{1}/image".format(MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_MusicImageService), mediaItem.zuneId);
                    var params = {locale: MS.Entertainment.Endpoint.getCulture()};
                    if (width > 0 && height > 0) {
                        params.w = width;
                        params.h = height
                    }
                    if (sources)
                        params.sources = sources;
                    return MS.Entertainment.Utilities.UriFactory.appendQuery(uri, params)
                }, getMediaItemDefaultImageUrl: function getMediaItemDefaultImageUrl(item) {
                    if (!item)
                        return MS.Entertainment.UI.Shell.ImageLoader._commonDefault;
                    var mediaType = item.mediaType;
                    var videoType = MS.Entertainment.Platform.PlaybackHelpers && MS.Entertainment.Platform.PlaybackHelpers.isVideo(item) ? item.videoType : Microsoft.Entertainment.Queries.VideoType.other;
                    if (item.serviceType === MS.Entertainment.Data.Augmenter.ServiceTypes.editorialItem) {
                        var convertedItem = MS.Entertainment.Utilities.convertEditorialItem(item);
                        mediaType = convertedItem.mediaType;
                        videoType = convertedItem.videoType
                    }
                    switch (mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.video:
                            if (videoType === Microsoft.Entertainment.Queries.VideoType.musicVideo)
                                return MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.musicVideo;
                            else if (videoType === Microsoft.Entertainment.Queries.VideoType.movie)
                                return MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.movie;
                            return MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.otherVideo;
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeries:
                            return MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.tvSeries;
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeason:
                            return MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.tvSeason;
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                            return MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.artist;
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                            return MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.album;
                        case Microsoft.Entertainment.Queries.ObjectType.game:
                            return MS.Entertainment.UI.Shell.ImageLoader.MediaDefaultUrls.game;
                        default:
                            return MS.Entertainment.UI.Shell.ImageLoader._commonDefault
                    }
                }, registerMediaItemImageUrlCallback: function registerMediaItemImageUrlCallback(mediaItem, callback, width, height) {
                    if (!mediaItem.mediaItemImageUrlCallback) {
                        var observable = WinJS.Binding.as(mediaItem);
                        var pendingImageUpdate = true;
                        var mediaItemImageUrlCallback = function calculateNewUrl() {
                                if (!pendingImageUpdate) {
                                    pendingImageUpdate = true;
                                    return MS.Entertainment.UI.Shell.ImageLoader.getServiceImageUrl(mediaItem, width, height).then(function deliverPrimaryUrl(url) {
                                            callback(url);
                                            pendingImageUpdate = false
                                        })
                                }
                            };
                        observable.bind("serviceType", mediaItemImageUrlCallback);
                        observable.bind("mediaType", mediaItemImageUrlCallback);
                        observable.bind("videoType", mediaItemImageUrlCallback);
                        observable.bind("serviceId", mediaItemImageUrlCallback);
                        observable.bind("imageId", mediaItemImageUrlCallback);
                        observable.bind("filePath", mediaItemImageUrlCallback);
                        observable.bind("imageUri", mediaItemImageUrlCallback);
                        pendingImageUpdate = false;
                        WinJS.Promise.timeout().then(function() {
                            mediaItemImageUrlCallback()
                        });
                        return {
                                observable: observable, mediaItemImageUrlCallback: mediaItemImageUrlCallback
                            }
                    }
                }, unregisterMediaItemImageUrlCallback: function unregisterMediaItemImageUrlCallback(token) {
                    var observable = token.observable;
                    var mediaItemImageUrlCallback = token.mediaItemImageUrlCallback;
                    observable.unbind("serviceType", mediaItemImageUrlCallback);
                    observable.unbind("mediaType", mediaItemImageUrlCallback);
                    observable.unbind("videoType", mediaItemImageUrlCallback);
                    observable.unbind("serviceId", mediaItemImageUrlCallback);
                    observable.unbind("imageId", mediaItemImageUrlCallback);
                    observable.unbind("filePath", mediaItemImageUrlCallback);
                    observable.unbind("imageUri", mediaItemImageUrlCallback)
                }, DefaultThumbnailSizes: {
                    defaultWidth: 320, video: 234
                }
        }), ImageControl: MS.Entertainment.UI.Framework.deriveUserControl("MS.Entertainment.UI.Framework.ImageControl", null, function(element, options){}, {
                _cachedUrl: null, addLoadedClassToParent: false, loadImage: function loadImage(target) {
                        if (String.isString(target) && (MS.Entertainment.UI.Shell.ImageLoader.isWebUrl(target) || MS.Entertainment.UI.Shell.ImageLoader.isFileUrl(target)))
                            return MS.Entertainment.UI.Shell.ImageLoader.cacheImage(target, String.empty);
                        else if (String.isString(target))
                            return WinJS.Promise.as(target);
                        else if (target.overrideItemUseUrl)
                            return MS.Entertainment.UI.Shell.ImageLoader.cacheImage(target.imagePrimaryUrl, String.empty);
                        else if (target.mediaType || target.serviceType && target.serviceType === MS.Entertainment.Data.Augmenter.ServiceTypes.editorialItem || target.serviceType && target.serviceType === MS.Entertainment.Data.Augmenter.ServiceTypes.wmis)
                            return this._loadImageFromTarget(target);
                        else if (target.imageUriOverride)
                            return WinJS.Promise.as(target.imageUriOverride);
                        else
                            return WinJS.Promise.as(target)
                    }, _handleLoaded: function _handleLoaded(args) {
                        if (this._unloaded)
                            return;
                        var naturalWidth = 0;
                        var naturalHeight = 0;
                        if (args && args.target) {
                            naturalWidth = args.target.naturalWidth;
                            naturalHeight = args.target.naturalHeight
                        }
                        if (naturalWidth > naturalHeight) {
                            WinJS.Utilities.addClass(this.domElement, "wide");
                            WinJS.Utilities.removeClass(this.domElement, "tall")
                        }
                        else if (naturalWidth < naturalHeight) {
                            WinJS.Utilities.addClass(this.domElement, "tall");
                            WinJS.Utilities.removeClass(this.domElement, "wide")
                        }
                        else {
                            WinJS.Utilities.removeClass(this.domElement, "tall");
                            WinJS.Utilities.removeClass(this.domElement, "wide")
                        }
                        this.applyCSSClassesOnLoad(naturalWidth, naturalHeight);
                        if (this.addLoadedClassToParent && this.domElement.parentElement)
                            WinJS.Utilities.addClass(this.domElement.parentElement, "imageControl_loaded");
                        MS.Entertainment.UI.Framework.ImageControl.prototype._handleLoaded.apply(this, arguments)
                    }, applyCSSClassesOnLoad: function applyCSSClassesOnLoad(naturalWidth, naturalHeight){}, _loadImageFromTarget: function _loadImageFromTarget(target) {
                        MS.Entertainment.UI.Controls.assert(this.desiredImageSize && (this.desiredImageSize.width || this.desiredImageSize.height), "We need a desired size (along at least one axis to ensure the service requests the correct image");
                        if (target && target.imageUri && target.imageUri.lastIndexOf("blob:", 0) > -1) {
                            this.target = target.imageUri;
                            return WinJS.Promise.as(this.target)
                        }
                        else
                            return MS.Entertainment.UI.Shell.ImageLoader.getServiceImageUrl(this.target, this.desiredImageSize.width, this.desiredImageSize.height, null, this.imageIdType, this.defaultImagePath)
                    }, _setImgSrc: function _setImgSrc(url) {
                        var imageManager;
                        if (MS.Entertainment.UI.Shell.ImageLoader.isAppDataUrl(url) && url !== this._cachedUrl) {
                            imageManager = new Microsoft.Entertainment.ImageManager;
                            if (this._cachedUrl)
                                imageManager.unregisterImageUrl(this._cachedUrl);
                            imageManager.registerImageUrl(url);
                            this._cachedUrl = url
                        }
                        MS.Entertainment.UI.Framework.ImageControl.prototype._setImgSrc.apply(this, arguments)
                    }, unload: function unload() {
                        var imageManager;
                        if (this._cachedUrl) {
                            imageManager = new Microsoft.Entertainment.ImageManager;
                            imageManager.unregisterImageUrl(this._cachedUrl);
                            this._cachedUrl = null
                        }
                        MS.Entertainment.UI.Framework.ImageControl.prototype.unload.apply(this, arguments)
                    }, hide: function hide(element) {
                        MS.Entertainment.Utilities.hideElement(element)
                    }, show: function show(element) {
                        MS.Entertainment.Utilities.showElement(element)
                    }
            })
    })
})()
