﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator("/Framework/debug.js");
(function() {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Utilities");
    WinJS.Namespace.define("MS.Entertainment.Utilities.Telemetry", {
        _adsClicked: [], _previewsPlayed: [], _enableLogging: false, _totalTimeCounterStart: 0, _somethingPlaying: false, openDataPoint: function openDataPoint() {
                var platLog = Microsoft.Entertainment.Platform.Logging;
                var dataPoint = new platLog.DataPoint(platLog.LoggingLevel.telemetry);
                if (MS.Entertainment.Utilities.Telemetry._enableLogging && WebUnitExtensions.TelemetryDataPointWrapper)
                    dataPoint = new WebUnitExtensions.TelemetryDataPointWrapper(dataPoint);
                return dataPoint
            }, closeDataPoint: function closeDataPoint(dataPoint) {
                if (MS.Entertainment.UI.Application && MS.Entertainment.UI.Application.Helpers && MS.Entertainment.UI.Application.Helpers.activationTelemetryData)
                    MS.Entertainment.UI.Application.Helpers.activationTelemetryData.push(dataPoint);
                else
                    dataPoint.write()
            }, appendUIPath: function appendUIPath(dataPoint) {
                var currentLocation = MS.Entertainment.Utilities.getCurrentUserLocation();
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.UIPath, currentLocation)
            }, appendLastUIPath: function appendLastUIPath(dataPoint) {
                var service = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                if (service.canNavigateBack) {
                    var backStack = service._backStack;
                    var lastPage = backStack._stack[backStack._stack.length - 1];
                    var lastLocation = "None";
                    if (lastPage)
                        lastLocation = lastPage.page.iaNode.moniker;
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.LastUIPath, lastLocation)
                }
            }, isCurrentPageSearchPage: function isCurrentPageSearchPage() {
                var currentPage;
                if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.navigation))
                    currentPage = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).currentPage;
                if (currentPage && currentPage.overrideFragmentUrl)
                    if (MS.Entertainment.Utilities.isMusicApp || MS.Entertainment.Utilities.isVideoApp)
                        return currentPage.iaNode && currentPage.iaNode.moniker === MS.Entertainment.UI.Monikers.searchPage;
                    else
                        return currentPage.overrideFragmentUrl.match(/SearchNavStub.html$/i);
                return false
            }, _anonymousUserId: null, _getUserIdValue: function _getUserIdValue() {
                if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.signIn) && MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn).isSignedIn)
                    if (MS.Entertainment.Utilities.isNYCApp)
                        return MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser).__userPuid;
                    else
                        return MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser).xuid;
                else {
                    if (!this._anonymousUserId)
                        this._anonymousUserId = Microsoft.Entertainment.Marketplace.Marketplace.generateGuid();
                    return this._anonymousUserId
                }
            }, _getUsersValue: function _getUsersValue() {
                var userId;
                if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.signIn) && MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn).isSignedIn)
                    if (MS.Entertainment.Utilities.isNYCApp)
                        userId = "p:" + MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser).__userPuid;
                    else
                        userId = "x:" + MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser).xuid;
                else {
                    if (!this._anonymousUserId)
                        this._anonymousUserId = Microsoft.Entertainment.Marketplace.Marketplace.generateGuid();
                    userId = "a:" + this._anonymousUserId
                }
                return userId
            }, licenseRightFromMedia: function licenseRightFromMedia(media, offerId) {
                var licenseRight = false;
                var i;
                if ((media.mediaType === Microsoft.Entertainment.Queries.ObjectType.video || media.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeason) && offerId)
                    if (media.rights)
                        for (i = 0; i < media.rights.length; i++)
                            if (media.rights[i].offerId) {
                                var offerMatch = new RegExp(offerId + "$", "i");
                                if (media.rights[i].offerId.match(offerMatch)) {
                                    licenseRight = media.rights[i].licenseRight;
                                    break
                                }
                            }
                return licenseRight
            }, priceFromMedia: function priceFromMedia(media, offerId) {
                var price = String.empty;
                var i,
                    j;
                if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.video && offerId)
                    if (media.rights)
                        for (i = 0; i < media.rights.length; i++) {
                            var offerMatch = new RegExp(offerId + "$", "i");
                            if (media.rights[i] && ((media.rights[i].offerId && media.rights[i].offerId.match(offerMatch) && media.rights[i].price) || MS.Entertainment.Utilities.isRightFree(media.rights[i]))) {
                                price = media.rights[i].price + " " + media.rights[i].priceCurrencyCode;
                                break
                            }
                        }
                if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.game) {
                    var gold = MS.Entertainment.Data.Factory.Marketplace.xboxSubscriptionLevel.gold.toLowerCase();
                    if (media.offers)
                        for (i = 0; i < media.offers.length && !price; i++) {
                            var offer = media.offers[i];
                            if (offer.SubscriptionLevels)
                                for (j = 0; j < offer.SubscriptionLevels.length; j++)
                                    if (offer.SubscriptionLevels[j].toLowerCase() === gold && offer.PaymentType === 1) {
                                        price = offer.Price;
                                        break
                                    }
                        }
                }
                if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.album || media.mediaType === Microsoft.Entertainment.Queries.ObjectType.track)
                    if (media.rights && offerId)
                        for (i = 0; i < media.rights.length; i++) {
                            var offerMatch = new RegExp(offerId + "$", "i");
                            if (media.rights[i] && ((media.rights[i].offerId && media.rights[i].offerId.match(offerMatch) && media.rights[i].price) || MS.Entertainment.Utilities.isRightFree(media.rights[i]))) {
                                price = media.rights[i].price + " " + media.rights[i].priceCurrencyCode;
                                break
                            }
                        }
                if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeason)
                    if (media.rights)
                        if (media.rights[0])
                            for (i = 0; i < media.rights.length; i++) {
                                var offerMatch = new RegExp(offerId + "$", "i");
                                if (media.rights[i] && ((media.rights[i].offerId && media.rights[i].offerId.match(offerMatch) && media.rights[i].price) || MS.Entertainment.Utilities.isRightFree(media.rights[i]))) {
                                    price = media.rights[i].price + " " + media.rights[i].priceCurrencyCode;
                                    break
                                }
                            }
                        else if (media.rights.right)
                            for (i = 0; i < media.rights.right.length; i++) {
                                var offerMatch = new RegExp(offerId + "$", "i");
                                if (media.rights[i] && ((media.rights.right[i].offerId && media.rights.right[i].offerId.match(offerMatch) && media.rights.right[i].price) || MS.Entertainment.Utilities.isRightFree(media.rights.right[i]))) {
                                    price = media.rights.right[i].price + " " + media.rights.right[i].priceCurrencyCode;
                                    break
                                }
                            }
                return price
            }, mediaTypeNameForTelemetry: function mediaTypeNameForTelemetry(media) {
                var name;
                var mediaType = media && media.mediaType;
                var defaultPlatformType = (media && media.defaultPlatformType) || String.empty;
                var downloadType = media && media.downloadType;
                switch (mediaType) {
                    case Microsoft.Entertainment.Queries.ObjectType.video:
                        if (media.videoType === Microsoft.Entertainment.Queries.VideoType.musicVideo)
                            name = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.MusicVideo;
                        else if (media.videoType === Microsoft.Entertainment.Queries.VideoType.tvEpisode)
                            name = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.TVEpisode;
                        else
                            name = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Movie;
                        break;
                    case Microsoft.Entertainment.Queries.ObjectType.tvSeries:
                        name = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.TVSeries;
                        break;
                    case Microsoft.Entertainment.Queries.ObjectType.tvSeason:
                        name = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.TVSeason;
                        break;
                    case Microsoft.Entertainment.Queries.ObjectType.person:
                        name = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Artist;
                        break;
                    case Microsoft.Entertainment.Queries.ObjectType.album:
                        name = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Album;
                        break;
                    case Microsoft.Entertainment.Queries.ObjectType.track:
                        name = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Song;
                        break;
                    case Microsoft.Entertainment.Queries.ObjectType.playlist:
                        name = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Playlist;
                        break;
                    case Microsoft.Entertainment.Queries.ObjectType.game:
                        if (defaultPlatformType)
                            name = defaultPlatformType + " " + MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Game;
                        else
                            name = downloadType || MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Game;
                        break;
                    default:
                        name = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.unknown;
                        break
                }
                return name
            }, hcrMedia: null, logTelemetryEvent: function logTelemetryEvent(event) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(event);
                for (var i = 1; i < arguments.length - 1; i += 2) {
                    var key = arguments[i];
                    var value = arguments[i + 1];
                    if (key && value !== undefined)
                        dataPoint.appendParameter(key, value)
                }
                this.closeDataPoint(dataPoint)
            }, logTelemetryEventWithParameterArray: function logTelemetryEventWithParameterArray(event, parameterArray) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(event);
                for (var i = 0; i < parameterArray.length; i++)
                    if (parameterArray[i].parameterName && parameterArray[i].parameterValue !== undefined && parameterArray[i].parameterValue !== null)
                        dataPoint.appendParameter(parameterArray[i].parameterName, parameterArray[i].parameterValue);
                    else
                        MS.Entertainment.Utilities.fail("logTelemetryEventWithParameterArray called with invalid parameterArray element. Event: " + event);
                this.closeDataPoint(dataPoint)
            }, logTelemetryEventWithUIPath: function logTelemetryEventWithUIPath(event, parameterArray) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(event);
                this.appendUIPath(dataPoint);
                if (parameterArray)
                    for (var i = 0; i < parameterArray.length; i++)
                        if (parameterArray[i].parameterName && parameterArray[i].parameterValue !== undefined && parameterArray[i].parameterValue !== null)
                            dataPoint.appendParameter(parameterArray[i].parameterName, parameterArray[i].parameterValue);
                        else
                            MS.Entertainment.Utilities.fail("logTelemetryEventWithUIPath called with invalid parameterArray element. Event: " + event);
                this.closeDataPoint(dataPoint)
            }, logSearchEnter: function logSearchEnter(hcrMedia) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.SearchEnter);
                this.appendUIPath(dataPoint);
                this.appendLastUIPath(dataPoint);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.EDSSearch, true);
                var typeName = "";
                var mediaName = "";
                if (hcrMedia) {
                    mediaName = hcrMedia.name;
                    typeName = this.mediaTypeNameForTelemetry(hcrMedia)
                }
                MS.Entertainment.Utilities.Telemetry.hcrMedia = hcrMedia;
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.SearchHCRMediaName, mediaName);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.SearchHCRMediaType, typeName);
                this.closeDataPoint(dataPoint)
            }, logSearchExit: function logSearchExit(media, hcr, destination, url) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.SearchExit);
                this.appendUIPath(dataPoint);
                if (media) {
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaName, media.name);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaType, this.mediaTypeNameForTelemetry(media));
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.InCollection, media.inCollection)
                }
                var isHCR = !!(media && media.isEqual && media.isEqual(MS.Entertainment.Utilities.Telemetry.hcrMedia));
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.HCR, isHCR);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.SearchDestination, destination);
                if (url)
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ExternalLink, url);
                this.closeDataPoint(dataPoint)
            }, logSearchWordWheelEnter: function logSearchWordWheel(title, mediaType, videoType) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.SearchWordWheelEnter);
                this.appendUIPath(dataPoint);
                this.appendLastUIPath(dataPoint);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.SelectedItemTitle, title);
                var typeName = MS.Entertainment.Utilities.getMediaTypeName(mediaType, videoType);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.SelectedItemMediaType, typeName);
                this.closeDataPoint(dataPoint)
            }, logPlayPreview: function logPlayPreview(media, fromAd) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PreviewPlayed);
                this.appendUIPath(dataPoint);
                var typeName = this.mediaTypeNameForTelemetry(media);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaName, media.name);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaType, typeName);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.FromAd, fromAd);
                if (fromAd)
                    this._adsClicked.push(media.serviceId);
                this._previewsPlayed.push(media.serviceId);
                this.closeDataPoint(dataPoint)
            }, logSubscriptionSignupPrePurchase: function logSubscriptionSignupPrePurchase(subscription) {
                if (!subscription)
                    return;
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.SubscriptionSignupPrePurchase);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.OfferId, subscription.offerId);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Title, subscription.title);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Price, subscription.price);
                this.closeDataPoint(dataPoint)
            }, logSubscriptionSignupPurchaseMade: function logSubscriptionSignupPurchaseMade(subscription) {
                if (!subscription)
                    return;
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.SubscriptionSignupPurchaseMade);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.OfferId, subscription.offerId);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Title, subscription.title);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Price, subscription.price);
                this.closeDataPoint(dataPoint)
            }, logPurchaseMade: function logPurchaseMade(media, offerId) {
                this.logPurchaseHappened(media, offerId);
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PurchaseMade);
                this.appendUIPath(dataPoint);
                var typeName = this.mediaTypeNameForTelemetry(media);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaType, typeName);
                if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeason) {
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaName, media.seriesTitle + " " + media.name);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.SMID, media.seasonServiceId);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.CorrelationID, MS.Entertainment.Utilities.Telemetry.getCorrelationId(media.seasonServiceId))
                }
                else {
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaName, media.name);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.SMID, media.serviceId);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.CorrelationID, MS.Entertainment.Utilities.Telemetry.getCorrelationId(media.serviceId))
                }
                var price = this.priceFromMedia(media, offerId);
                if (price)
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Price, price);
                if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.video || media.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeason) {
                    var licenseRight = this.licenseRightFromMedia(media, offerId);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.licenseRight, licenseRight)
                }
                if (MS.Entertainment.Utilities.isVideoApp1) {
                    var videoPostRollABTestName = MS.Entertainment.ViewModels.VideoPostRollHelpers.abTestName;
                    var videoPostRollABGroupName = MS.Entertainment.ViewModels.VideoPostRollHelpers.abGroupName;
                    if (videoPostRollABTestName && videoPostRollABGroupName) {
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ABTestName, videoPostRollABTestName);
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ABGroupName, videoPostRollABGroupName)
                    }
                }
                var fromAd = false;
                var fromPreview = false;
                var i;
                for (i = 0; i < this._adsClicked.length; i++)
                    if (this._adsClicked[i] === media.serviceId) {
                        fromAd = true;
                        break
                    }
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.FromAd, fromAd);
                for (i = 0; i < this._previewsPlayed.length; i++)
                    if (this._previewsPlayed[i] === media.serviceId) {
                        fromPreview = true;
                        break
                    }
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.FromPreview, fromPreview);
                this.closeDataPoint(dataPoint)
            }, logShare: function logShare(sharePackage) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.ShareCharm);
                if (sharePackage && sharePackage.package) {
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ShareTitle, sharePackage.package.title);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ShareUri, sharePackage.package.uri);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaId, sharePackage.package.mediaId);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaType, this.mediaTypeNameForTelemetry(sharePackage.package));
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaName, sharePackage.package.mediaTitle)
                }
                this.appendUIPath(dataPoint);
                this.closeDataPoint(dataPoint)
            }, logAdClicked: function logAdClicked(adInfo) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.X8AdClicked);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AdInfo, adInfo);
                this.appendUIPath(dataPoint);
                this.closeDataPoint(dataPoint)
            }, _logPlaybackControl: function _logPlaybackControl(controlUsed, whereUsed) {
                var dataPoint = this.openDataPoint();
                this.appendUIPath(dataPoint);
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlaybackControl);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ControlUsed, controlUsed);
                var whereUsedMapped = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Unknown;
                if (whereUsed.indexOf("nowPlayingTransportControls") !== -1)
                    whereUsedMapped = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.NowPlaying;
                else if (whereUsed.indexOf("globalTransportControls") !== -1)
                    whereUsedMapped = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.GlobalAppbar;
                else if (whereUsed.indexOf("appBarTransportControls") !== -1)
                    whereUsedMapped = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.AppBar;
                else if (whereUsed.indexOf("homeHubNowPlayingTile") !== -1)
                    whereUsedMapped = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.HomeNPTile;
                else if (whereUsed.indexOf("snappedTransportControls") !== -1)
                    whereUsedMapped = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.SnappedAppBar;
                else if (whereUsed.indexOf("mediaKey") !== -1)
                    whereUsedMapped = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.MediaKey;
                else if (whereUsed.indexOf("voiceCommand") !== -1)
                    whereUsedMapped = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.VoiceCommand;
                else if (whereUsed.indexOf("controls-") !== -1 || whereUsed.indexOf("adControls-") !== -1 || whereUsed.indexOf("currentItem-") !== -1 || whereUsed.indexOf("commonOverlays-transportControls") !== -1)
                    whereUsedMapped = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.MusicTransportControls;
                else
                    MS.Entertainment.Utilities.assert(false, "unrecognised location for playback control telemetry");
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.WhereUsed, whereUsedMapped);
                this.closeDataPoint(dataPoint)
            }, logPlayClicked: function logPlayClicked(whereUsed) {
                this._logPlaybackControl(MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Play, whereUsed)
            }, logPauseClicked: function logPauseClicked(whereUsed) {
                this._logPlaybackControl(MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Pause, whereUsed)
            }, logStopClicked: function logStopClicked(whereUsed) {
                this._logPlaybackControl(MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Stop, whereUsed)
            }, logPreviousClicked: function logPreviousClicked(whereUsed) {
                this._logPlaybackControl(MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Previous, whereUsed)
            }, logNextClicked: function logNextClicked(whereUsed) {
                this._logPlaybackControl(MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Next, whereUsed)
            }, logPlaybackAttempted: function logPlaybackAttempted(mediaInstance) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlaybackAttempted);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.PlaybackSource, MS.Entertainment.Platform.PlaybackHelpers.lastPlaybackSource);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.NetworkStatus, this._getNetworkStatus());
                this._appendPlaybackCommonDataToDataPoint(mediaInstance, dataPoint);
                this.closeDataPoint(dataPoint)
            }, logGetPreferredStreamSucceeded: function logGetPreferredStreamSucceeded(retries) {
                this.logGetPreferredStreamResult(true, null, retries)
            }, logGetPreferredStreamFailed: function logGetPreferredStreamFailed(reason, retries) {
                this.logGetPreferredStreamResult(false, reason, retries)
            }, logGetPreferredStreamResult: function logGetPreferredStreamResult(success, reason, retries) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.GetPreferredStreamResult);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Success, success);
                if (reason)
                    if (reason.number)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Reason, reason.number);
                    else
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Reason, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Unknown);
                else
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Reason, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Unknown);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Retries, retries ? retries : 0);
                this.closeDataPoint(dataPoint)
            }, logSemanticZoom: function logSemanticZoom(initMethod) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.SemanticZoom);
                this.appendUIPath(dataPoint);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MethodInitiated, initMethod);
                this.closeDataPoint(dataPoint)
            }, logVolumeSelected: function logVolumeSelected(value) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.VolumeSelected);
                this.appendUIPath(dataPoint);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.VolumeValue, value);
                this.closeDataPoint(dataPoint)
            }, logMuteStateSelected: function logMuteStateSelected(state) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.MuteStateSelected);
                this.appendUIPath(dataPoint);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MuteState, state);
                this.closeDataPoint(dataPoint)
            }, _correlationIds: {}, getCorrelationId: function getCorrelationId(serviceId) {
                var correlationId = String.empty;
                if (!MS.Entertainment.Utilities.isEmptyGuid(serviceId))
                    correlationId = MS.Entertainment.Utilities.Telemetry._correlationIds[serviceId] || String.empty;
                return correlationId
            }, setCorrelationId: function setCorrelationId(serviceId) {
                if (!MS.Entertainment.Utilities.isEmptyGuid(serviceId))
                    MS.Entertainment.Utilities.Telemetry._correlationIds[serviceId] = Microsoft.Entertainment.Marketplace.Marketplace.generateGuid()
            }, _getNetworkStatus: function _getNetworkStatus() {
                var isOnline = MS.Entertainment.UI.NetworkStatusService.isOnline() ? "online" : "offline";
                var networkStatusStringArray = ["unknown", "unrestricted", "throttled", "onDemand", "localOnly", "none"];
                return isOnline + ":" + networkStatusStringArray[MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).networkStatus]
            }, _getFileExtensionFromMediaInstance: function getFileExtensionFromMediaInstance(mediaInstance)
            {
                var extension = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.unknown;
                MS.Entertainment.Platform.Playback.assert(mediaInstance, "tried to determine path for undefined mediaInstance");
                try {
                    if (mediaInstance) {
                        var path = String.empty;
                        if (mediaInstance.isLocal) {
                            if (mediaInstance._mediaItem && mediaInstance._mediaItem.data)
                                if (mediaInstance._mediaItem.data.filePath)
                                    path = mediaInstance._mediaItem.data.filePath;
                                else if (mediaInstance._mediaItem.data.fileItem && mediaInstance._mediaItem.data.fileItem.path)
                                    path = mediaInstance._mediaItem.data.fileItem.path
                        }
                        else if (mediaInstance.source)
                            path = mediaInstance.source;
                        if (path.length > 0)
                            extension = Windows.Foundation.Uri(path).extension
                    }
                }
                catch(e) {
                    MS.Entertainment.Platform.Playback.assert(false, "failed to determine extension for path")
                }
                if (!extension)
                    extension = "no extension";
                return extension
            }, _getUrlFromMediaInstance: function _getUrlFromMediaInstance(mediaInstance) {
                var url = "undetermined";
                MS.Entertainment.Platform.Playback.assert(mediaInstance, "tried to determine url for undefined mediaInstance");
                if (mediaInstance)
                    if (!mediaInstance.isLocal && mediaInstance.source)
                        url = mediaInstance.source;
                    else if (mediaInstance.isLocal && mediaInstance._mediaItem && mediaInstance._mediaItem.data && mediaInstance._mediaItem.data.filePath && Microsoft.Entertainment.Configuration.ConfigurationManager().telemetry.flightId)
                        url = mediaInstance._mediaItem.data.filePath;
                return url
            }, _getIsDrmFromMediaInstance: function _getDrmStateFromMediaInstance(mediaInstance) {
                var isDRM = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.unknown;
                if (mediaInstance)
                    switch (mediaInstance.protectionState) {
                        case MS.Entertainment.Platform.Playback.ProtectionState.drmProtected:
                            isDRM = "true";
                            break;
                        case MS.Entertainment.Platform.Playback.ProtectionState.unprotected:
                            isDRM = "false";
                            break;
                        case MS.Entertainment.Platform.Playback.ProtectionState.unknown:
                            isDRM = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.unknown;
                            break;
                        default:
                            MS.Entertainment.Platform.Playback.assert(false, "unknown protection state in logging");
                            break
                    }
                return isDRM
            }, _getIdFromMediaInstance: function _getIdFromMediaInstance(mediaInstance) {
                var id = 0;
                if (mediaInstance)
                    id = (mediaInstance.mediaInstanceId ? mediaInstance.mediaInstanceId : mediaInstance.libraryId);
                return id
            }, _getSmidFromMediaInstance: function _getSmidFromMediaInstance(mediaInstance) {
                var id = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.unknown;
                if (mediaInstance && mediaInstance._mediaItem && mediaInstance._mediaItem.data && mediaInstance._mediaItem.data.zuneId)
                    id = mediaInstance._mediaItem.data.zuneId;
                return id
            }, _getParentSmidFromMediaInstance: function _getParentSmidFromMediaInstance(mediaInstance) {
                return WinJS.Utilities.getMember("_mediaItem.data.seasonServiceId", mediaInstance) || WinJS.Utilities.getMember("_mediaItem.data.canonicalSeasonId", mediaInstance) || MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.unknown
            }, _getPlaybackSource: function _getPlaybackSource(mediaInstance) {
                var playbackSource = MS.Entertainment.Platform.PlaybackHelpers.lastPlaybackSource;
                if (mediaInstance && mediaInstance._mediaItem && mediaInstance._mediaItem.data && mediaInstance._mediaItem.data.acquisitionData) {
                    if (!playbackSource)
                        playbackSource = MS.Entertainment.Utilities.getAcquisitionContextFromEnum(mediaInstance._mediaItem.data.acquisitionData.acquisitionContext);
                    if (mediaInstance._mediaItem.data.acquisitionData.acquisitionContextType)
                        playbackSource = (playbackSource ? playbackSource + ": " : String.empty) + MS.Entertainment.Utilities.getAcquisitionContextTypeFromEnum(mediaInstance._mediaItem.data.acquisitionData.acquisitionContextType)
                }
                return playbackSource
            }, _appendPlaybackCommonDataToDataPoint: function _appendPlaybackCommonDataToDataPoint(mediaInstance, dataPoint) {
                try {
                    if (mediaInstance) {
                        var media = mediaInstance;
                        if (mediaInstance && mediaInstance._mediaItem && mediaInstance._mediaItem.data)
                            media = mediaInstance._mediaItem.data;
                        var typeName = this.mediaTypeNameForTelemetry(media);
                        var isSubscription = MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.signedInUser) ? MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser).isSubscription : "false";
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaType, typeName);
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.IsSubscription, isSubscription);
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.IsDrm, this._getIsDrmFromMediaInstance(mediaInstance));
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.IsStream, mediaInstance.isLocal ? "false" : "true");
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Id, this._getIdFromMediaInstance(mediaInstance));
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Smid, this._getSmidFromMediaInstance(mediaInstance));
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ParentSmid, this._getParentSmidFromMediaInstance(mediaInstance));
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Extension, this._getFileExtensionFromMediaInstance(mediaInstance));
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Url, this._getUrlFromMediaInstance(mediaInstance));
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.PlaybackSource, this._getPlaybackSource(mediaInstance));
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.IsPreview, mediaInstance.isPreview ? "true" : "false");
                        var correlationId = MS.Entertainment.Utilities.Telemetry.getCorrelationId(media.serviceId) || MS.Entertainment.Utilities.Telemetry.getCorrelationId(this._getParentSmidFromMediaInstance(mediaInstance));
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.CorrelationID, correlationId)
                    }
                    else
                        MS.Entertainment.Utilities.assert(mediaInstance, "MediaInstance is null")
                }
                catch(ex) {
                    MS.Entertainment.Utilities.fail("Unable to append parameter to data point")
                }
            }, logPlaybackHappened: function logPlaybackHappened(mediaInstance) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlaybackHappened);
                this._appendPlaybackCommonDataToDataPoint(mediaInstance, dataPoint);
                var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                var playLocation = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.unknown;
                if (uiStateService.isAppVisible) {
                    if (uiStateService.isSnapped)
                        playLocation = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.snapped;
                    else if (uiStateService.nowPlayingVisible || uiStateService.nowPlayingInset)
                        playLocation = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.nowplaying
                }
                else
                    playLocation = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.background;
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.PlayLocation, playLocation);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.NetworkStatus, this._getNetworkStatus());
                this.closeDataPoint(dataPoint)
            }, logPlaybackError: function logPlaybackError(mediaInstance, error, context) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlaybackError);
                var isSubscription = MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.signedInUser) ? MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser).isSubscription : "false";
                var mediaType = this.mediaTypeNameForTelemetry(mediaInstance._mediaItem.data);
                var errorCode = error && error.code ? error.code : 0;
                var msExtendedCode = error && error.msExtendedCode ? error.msExtendedCode : errorCode;
                var networkStatus = this._getNetworkStatus();
                var errorContext = context || String.empty;
                if (mediaInstance) {
                    this._appendPlaybackCommonDataToDataPoint(mediaInstance, dataPoint);
                    if (mediaInstance._errorDescriptor && mediaInstance._errorDescriptor.context && mediaInstance._errorDescriptor.context !== errorContext)
                        errorContext += (errorContext ? "; " : String.empty) + mediaInstance._errorDescriptor.context
                }
                if (error && error.context && (errorContext !== error.context))
                    errorContext += (errorContext ? "; " : String.empty) + error.context;
                var stackTraceLimitDefault = Error.stackTraceLimit;
                Error.stackTraceLimit = Infinity;
                var stackTrace = MS.Entertainment.UI.Debug.getStackTrace();
                Error.stackTraceLimit = stackTraceLimitDefault;
                stackTrace = stackTrace.replace(/.*at.*(winjs|Function\scode|CoreFX).*\)\n/g, String.empty).replace(/\(.+?.*\//g, "(").replace(/.*at\s/g, String.empty);
                var stackTraceArray = stackTrace.split("\n");
                stackTraceArray.reverse();
                stackTrace = stackTraceArray.join("\n").substring(0, 2048);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ErrorCode, MS.Entertainment.Platform.Playback.errorToString(errorCode));
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ExtendedCode, MS.Entertainment.Platform.Playback.errorToString(msExtendedCode));
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ErrorContext, errorContext);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaType, mediaType);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.IsSubscription, isSubscription);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.NetworkStatus, networkStatus);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.StackTrace, stackTrace);
                MS.Entertainment.Platform.Playback.Etw.traceString("Telemetry::Playback::PlaybackError:" + " ErrorCode=" + MS.Entertainment.Platform.Playback.errorToString(errorCode) + "| msExtendedCode=" + MS.Entertainment.Platform.Playback.errorToString(msExtendedCode) + "| ErrorContext=" + errorContext + "| MediaType=" + mediaType + "| isSubscription=" + isSubscription + "| NetworkStatus=" + networkStatus + "| StackTrace=" + stackTrace);
                this._getMediaInfo(mediaInstance).then(function onSuccess(mediaDetail) {
                    var audioSubtype = String.empty;
                    var videoSubtype = String.empty;
                    var mimeType = String.empty;
                    var sourceCharacter = String.empty;
                    if (mediaDetail) {
                        audioSubtype = mediaDetail.audioSubtype;
                        videoSubtype = mediaDetail.videoSubtype;
                        mimeType = mediaDetail.mimeType;
                        sourceCharacter = mediaDetail.sourceCharacter
                    }
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MFSRAudioSubtype, audioSubtype);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MFSRVideoSubtype, videoSubtype);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MFSRMimeType, mimeType);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MFSRSourceCharacter, sourceCharacter);
                    this.closeDataPoint(dataPoint);
                    if (mediaInstance.source && mediaInstance.source.match && mediaInstance.source.match(/^blob*:/i))
                        URL.revokeObjectURL(mediaInstance.source);
                    MS.Entertainment.Platform.Playback.Etw.traceString("Telemetry::Playback::PlaybackError: data from MF Source Reader" + " audioSubtype=" + audioSubtype + "| videoSubtype=" + videoSubtype + "| mimeType=" + mimeType + "| sourceCharacter=" + sourceCharacter)
                }.bind(this), function onError(error) {
                    var mfSRError = String.empty;
                    if (error)
                        mfSRError = error.number + ": " + error.message;
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MFSRError, mfSRError);
                    this.closeDataPoint(dataPoint);
                    if (mediaInstance.source && mediaInstance.source.match && mediaInstance.source.match(/^blob*:/i))
                        URL.revokeObjectURL(mediaInstance.source);
                    MS.Entertainment.Platform.Playback.Etw.traceString("Telemetry::Playback::PlaybackError: error from MF Source Reader" + " error=" + mfSRError)
                }.bind(this))
            }, _getMediaInfo: function _getMediaInfo(mediaInstance) {
                var completion = null;
                var error = null;
                var promise = new WinJS.Promise(function(c, e, p) {
                        completion = c;
                        error = e
                    });
                if (!mediaInstance || !mediaInstance.isLocal || !mediaInstance.source || !MS.Entertainment.Utilities.isWindowsBlue) {
                    completion(null);
                    return promise
                }
                var srcURL = mediaInstance.source;
                var xhr = new XMLHttpRequest;
                xhr = new XMLHttpRequest;
                xhr.open("GET", srcURL, true);
                xhr.responseType = "blob";
                xhr.onreadystatechange = function onXhrInfoReadyStateChange() {
                    if (xhr.readyState == 4)
                        if (xhr.status === 200) {
                            var blob = xhr.response;
                            var blobStream = blob.msDetachStream();
                            try {
                                var mediaInfo = new Microsoft.Entertainment.MediaInfo.MediaInfo;
                                return mediaInfo.getMediaDetail(blobStream).then(function onSuccess(mediaDetail) {
                                        completion(mediaDetail)
                                    }, function onError(e) {
                                        error(e)
                                    })
                            }
                            catch(ex) {
                                completion(null)
                            }
                        }
                        else {
                            MS.Entertainment.Platform.Playback.Etw.traceString("telemetryUtility::_getMediaInfo() 1 failed to load stream for " + srcURL + ".  XMLHttpRequest status = " + xhr.status);
                            completion(null)
                        }
                };
                xhr.onerror = function onXhrError(error) {
                    MS.Entertainment.Platform.Playback.Etw.traceString("telemetryUtility::_getMediaInfo() 2 failed to load stream for " + srcURL + ".  XMLHttpRequest status = " + xhr.status);
                    completion(null)
                };
                try {
                    xhr.send();
                    return promise
                }
                catch(ex) {
                    completion(null);
                    return promise
                }
            }, logPlayerBufferState: function logPlayerBufferState(buffered, currentPosition) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlayerBufferState);
                var startPositions = String.empty,
                    endPositions = String.empty;
                for (var i = 0; i < buffered.length; i++) {
                    startPositions += (buffered.start(i) + ";");
                    endPositions += (buffered.end(i) + ";")
                }
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.EventTime, Date.now());
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.BufferStart, startPositions);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.BufferEnd, endPositions);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.CurrentPosition, currentPosition);
                this.closeDataPoint(dataPoint);
                MS.Entertainment.Platform.Playback.Etw.traceString("Telemetry::Playback::PlayerBufferState: BufferStart=" + startPositions + " BufferEnd=" + endPositions + " CurrentPosition=" + currentPosition)
            }, logPlayToReceiverError: function logPlayToReceiverError(exception, message) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlayToReceiverError);
                var errorCode = "0x" + (0xFFFFFFFF + exception.number + 1).toString(16);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ErrorCode, errorCode);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Message, message);
                this.closeDataPoint(dataPoint)
            }, logCommandClicked: function logCommandClicked(commandAction) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.CommandUsage);
                this.appendUIPath(dataPoint);
                MS.Entertainment.Utilities.assert(!!commandAction.automationId, "commandAction.automationId is null or undefined, commandAction.id = " + commandAction.id);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.commandId, commandAction.automationId);
                if (commandAction.title)
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.commandLabel, commandAction.title);
                if (commandAction.parameter) {
                    if (commandAction.parameter.abTests)
                        commandAction.parameter.abTests.forEach(function(abTest) {
                            if (abTest.abTestName)
                                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ABTestName, abTest.abTestName);
                            if (abTest.abGroupName)
                                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ABTestName, abTest.abGroupName)
                        });
                    if (commandAction.parameter.mediaItem) {
                        if (commandAction.parameter.mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.game) {
                            dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.mediaType, commandAction.parameter.mediaItem.mediaType);
                            dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.serviceId, commandAction.parameter.mediaItem.serviceId);
                            dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.titleType, commandAction.parameter.mediaItem.titleType);
                            dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.titleId, commandAction.parameter.mediaItem.titleId);
                            dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.defaultPlatformType, commandAction.parameter.mediaItem.defaultPlatformType)
                        }
                        if (commandAction.parameter.mediaItem.name)
                            dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.mediaName, commandAction.parameter.mediaItem.name);
                        if (commandAction.parameter.mediaItem.queryId)
                            dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.queryId, commandAction.parameter.mediaItem.queryId);
                        if (commandAction.parameter.mediaItem.acquisitionData) {
                            if (commandAction.parameter.mediaItem.acquisitionData.acquisitionContext)
                                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContext, MS.Entertainment.Utilities.getAcquisitionContextFromEnum(commandAction.parameter.mediaItem.acquisitionData.acquisitionContext));
                            if (commandAction.parameter.mediaItem.acquisitionData.acquisitionContextType)
                                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContextType, MS.Entertainment.Utilities.getAcquisitionContextTypeFromEnum(commandAction.parameter.mediaItem.acquisitionData.acquisitionContextType));
                            if (commandAction.parameter.mediaItem.acquisitionData.acquisitionContextID)
                                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContextId, commandAction.parameter.mediaItem.acquisitionData.acquisitionContextID)
                        }
                    }
                    else if (commandAction.parameter.hub || commandAction.parameter.page) {
                        if (commandAction.parameter.hub)
                            dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.hub, commandAction.parameter.hub);
                        if (commandAction.parameter.page)
                            dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.page, commandAction.parameter.page);
                        if (commandAction.parameter.queryId)
                            dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.queryId, commandAction.parameter.queryId)
                    }
                    if (commandAction.parameter.actionType)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.actionType, commandAction.parameter.actionType);
                    if (commandAction.parameter.executeLocation)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.executeLocation, commandAction.parameter.executeLocation)
                }
                else if (commandAction.userModel)
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.secondaryGamerTag, commandAction.userModel.gamerTag);
                this.closeDataPoint(dataPoint)
            }, logTransportControlsCommand: function logTransportControlsCommand(automationId, title) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.CommandUsage);
                this.appendUIPath(dataPoint);
                MS.Entertainment.Utilities.assert(!!automationId, "automationId is null or undefined, title = " + title);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.commandId, automationId);
                if (title)
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.commandLabel, title);
                this.closeDataPoint(dataPoint)
            }, logPopoverShown: function logPopoverShown(commandId, mediaItem) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PopoverDisplay);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.UIPart, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Popover);
                this.appendUIPath(dataPoint);
                if (commandId)
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.commandId, commandId);
                if (mediaItem) {
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.mediaType, mediaItem.mediaType);
                    if (mediaItem.serviceId)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.serviceId, mediaItem.serviceId);
                    if (mediaItem.seriesId)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.seriesId, mediaItem.seriesId);
                    if (mediaItem.titleId)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.titleId, mediaItem.titleId);
                    if (mediaItem.queryId)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.queryId, mediaItem.queryId);
                    if (mediaItem.name)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.name, mediaItem.name);
                    if (mediaItem.itemTypeQueryString)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.itemTypeQueryString, mediaItem.itemTypeQueryString)
                }
                this.closeDataPoint(dataPoint)
            }, logModifierClicked: function logModifierClicked(id, label) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.ModifierUsage);
                this.appendUIPath(dataPoint);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Label, label);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Id, id);
                this.closeDataPoint(dataPoint)
            }, logNotification: function logNotification(notification, isNewNotification) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.NotificationUsage);
                this.appendUIPath(dataPoint);
                MS.Entertainment.Utilities.assert(notification, "notification is null or undefined");
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.NotificationTitle, notification.title);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.NotificationCategory, notification.category.name);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.IsNewNotification, isNewNotification ? "Yes" : "No");
                this.closeDataPoint(dataPoint)
            }, logPageAction: function logPageAction(pageContentDataObject, currentLocation, destination, parameterArray) {
                var telemetryParameterNames = MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames;
                var telemetryValueNames = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues;
                if (!currentLocation) {
                    MS.Entertainment.Utilities.fail("Page Action call missing currentLocation");
                    return
                }
                if (!pageContentDataObject) {
                    MS.Entertainment.Utilities.fail("Page Action call missing pageContentDataObject");
                    return
                }
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PageAction);
                dataPoint.appendParameter(telemetryParameterNames.UserId, this._getUserIdValue());
                dataPoint.appendParameter(telemetryParameterNames.Users, this._getUsersValue());
                dataPoint.appendParameter(telemetryParameterNames.ActionTypeId, MS.Entertainment.Utilities.Telemetry.InputActionTypes.Click);
                dataPoint.appendParameter(telemetryParameterNames.ActionInputMethodId, MS.Entertainment.Utilities.Telemetry.InputMethodTypes.Click);
                if (currentLocation.uri) {
                    dataPoint.appendParameter(telemetryParameterNames.PageURI, currentLocation.uri);
                    var splitUri = currentLocation.uri.split("/");
                    if (splitUri)
                        dataPoint.appendParameter(telemetryParameterNames.PageName, splitUri[splitUri.length - 1]);
                    else
                        dataPoint.appendParameter(telemetryParameterNames.PageName, currentLocation.uri)
                }
                else {
                    dataPoint.appendParameter(telemetryParameterNames.PageURI, telemetryValueNames.Unknown);
                    dataPoint.appendParameter(telemetryParameterNames.PageName, telemetryValueNames.Unknown)
                }
                dataPoint.appendParameter(telemetryParameterNames.PageTypeId, currentLocation.pageTypeId);
                dataPoint.appendParameter(telemetryParameterNames.TemplateId, String.empty);
                if (destination) {
                    dataPoint.appendParameter(telemetryParameterNames.DestPageURI, destination.uri);
                    dataPoint.appendParameter(telemetryParameterNames.DestPageTypeId, destination.pageTypeId)
                }
                else {
                    dataPoint.appendParameter(telemetryParameterNames.DestPageURI, telemetryValueNames.None);
                    dataPoint.appendParameter(telemetryParameterNames.DestPageTypeId, telemetryValueNames.None)
                }
                dataPoint.appendParameter(telemetryParameterNames.CampaignId, String.empty);
                dataPoint.appendParameter(telemetryParameterNames.GroupId, String.empty);
                var jsonString = null;
                try {
                    jsonString = JSON.stringify(this._buildPageContentSlotForElement(pageContentDataObject))
                }
                catch(e) {
                    MS.Entertainment.Utilities.fail("failed to stringify page content: " + e);
                    jsonString = String.empty
                }
                dataPoint.appendParameter(telemetryParameterNames.ContentJsonVersion, 1.1);
                dataPoint.appendParameter(telemetryParameterNames.Content, jsonString);
                if (parameterArray && parameterArray.length)
                    for (var i = 0; i < parameterArray.length; i++)
                        if (parameterArray[i].parameterName && parameterArray[i].parameterValue !== undefined && parameterArray[i].parameterValue !== null)
                            dataPoint.appendParameter(parameterArray[i].parameterName, parameterArray[i].parameterValue);
                        else
                            MS.Entertainment.Utilities.fail("logPageAction called with invalid parameterArray element.  parameterName: " + parameterArray[i].parameterName);
                this.closeDataPoint(dataPoint)
            }, logPageView: function logPageView(rootElement, newPageInformation, refererPageInformation) {
                var telemetryParameterNames = MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames;
                var telemetryValueNames = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues;
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PageView);
                dataPoint.appendParameter(telemetryParameterNames.UserId, this._getUserIdValue());
                dataPoint.appendParameter(telemetryParameterNames.Users, this._getUsersValue());
                var refererUri = telemetryValueNames.Unknown;
                var refererTypeId = telemetryValueNames.Unknown;
                if (refererPageInformation) {
                    if (refererPageInformation.uri)
                        refererUri = refererPageInformation.uri;
                    if (refererPageInformation.pageTypeId)
                        refererTypeId = refererPageInformation.pageTypeId
                }
                var newPageUri = telemetryValueNames.Unknown;
                var newPageName = telemetryValueNames.Unknown;
                var newPageType = telemetryValueNames.Unknown;
                if (newPageInformation) {
                    if (newPageInformation.uri) {
                        newPageUri = newPageInformation.uri;
                        var splitUri = newPageInformation.uri.split("/");
                        if (splitUri)
                            newPageName = splitUri[splitUri.length - 1];
                        else
                            newPageName = newPageInformation.uri
                    }
                    if (newPageInformation.pageTypeId)
                        newPageType = newPageInformation.pageTypeId
                }
                dataPoint.appendParameter(telemetryParameterNames.PageURI, newPageUri);
                dataPoint.appendParameter(telemetryParameterNames.PageName, newPageName);
                dataPoint.appendParameter(telemetryParameterNames.RefererPageURI, refererUri);
                dataPoint.appendParameter(telemetryParameterNames.PageTypeId, newPageType);
                dataPoint.appendParameter(telemetryParameterNames.RefererPageTypeId, refererTypeId);
                dataPoint.appendParameter(telemetryParameterNames.PageTags, this._buildPageTagInformation(rootElement));
                dataPoint.appendParameter(telemetryParameterNames.PageParameters, String.empty);
                dataPoint.appendParameter(telemetryParameterNames.ProductCatalogId, String.empty);
                dataPoint.appendParameter(telemetryParameterNames.ProductId, String.empty);
                dataPoint.appendParameter(telemetryParameterNames.TemplateId, String.empty);
                dataPoint.appendParameter(telemetryParameterNames.ResolutionWidth, MS.Entertainment.Utilities.getWindowWidth());
                dataPoint.appendParameter(telemetryParameterNames.ResolutionHeight, MS.Entertainment.Utilities.getWindowHeight());
                var screenState = MS.Entertainment.Utilities.Telemetry.ScreenState.FullScreen;
                if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.uiState) && MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isSnapped)
                    screenState = MS.Entertainment.Utilities.Telemetry.ScreenState.Snapped;
                dataPoint.appendParameter(telemetryParameterNames.ScreenState, screenState);
                dataPoint.appendParameter(telemetryParameterNames.ColorDepth, 24000);
                dataPoint.appendParameter(telemetryParameterNames.SkinId, String.empty);
                var authedParameter = MS.Entertainment.Utilities.Telemetry.SignInState.Anonymous;
                if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.signIn) && MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn).isSignedIn)
                    authedParameter = MS.Entertainment.Utilities.Telemetry.SignInState.Authenticated;
                dataPoint.appendParameter(telemetryParameterNames.AuthState, authedParameter);
                dataPoint.appendParameter(telemetryParameterNames.CampaignId, String.empty);
                dataPoint.appendParameter(telemetryParameterNames.GroupId, String.empty);
                dataPoint.appendParameter(telemetryParameterNames.ContentJsonVersion, 1.1);
                dataPoint.appendParameter(telemetryParameterNames.Content, this._buildPageContentInformation(rootElement));
                this.closeDataPoint(dataPoint)
            }, _buildPageTagInformation: function _buildPageTagInformation(rootElement) {
                var rootToUse = rootElement ? rootElement : document.querySelector(".pageContainer.currentPage");
                var attributeElements = null;
                var resultString = String.empty;
                var result = null;
                if (rootToUse)
                    attributeElements = rootToUse.querySelectorAll("[data-ent-pagetag-name]");
                if (attributeElements && attributeElements.length)
                    for (var i = 0; i < attributeElements.length; i++) {
                        var pageTagName = null;
                        var pageTagValuePath = null;
                        var pageTagValue = null;
                        var pageTagItem = null;
                        var currentElement = attributeElements[i];
                        if (currentElement.hasAttribute("data-ent-pagetag-name"))
                            pageTagName = currentElement.getAttribute("data-ent-pagetag-name");
                        if (currentElement.hasAttribute("data-ent-pagetag-value"))
                            pageTagValuePath = currentElement.getAttribute("data-ent-pagetag-value");
                        MS.Entertainment.Utilities.assert(pageTagName, "element has data-ent-pagetag-name, but was empty");
                        MS.Entertainment.Utilities.assert(pageTagValuePath, "element has data-ent-pagetag-name, but has no or empty value");
                        if (pageTagValuePath)
                            pageTagValue = WinJS.Utilities.getMember(pageTagValuePath, currentElement.winControl || currentElement);
                        if (pageTagName && pageTagValue) {
                            result = result || {};
                            result[pageTagName] = pageTagValue
                        }
                    }
                if (result)
                    try {
                        resultString = JSON.stringify(result)
                    }
                    catch(e) {
                        MS.Entertainment.Utilities.fail("failed to stringify page tag: " + e)
                    }
                return resultString
            }, _buildPageContentInformation: function _buildPageContentInformation(rootElement) {
                var telemetryValueNames = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues;
                var rootToUse = rootElement ? rootElement : document.querySelector(".pageContainer.currentPage");
                if (!rootToUse)
                    return String.empty;
                var attributedElements = rootToUse.querySelectorAll("[data-ent-pagecontent]");
                var pageContentDataArray = this._buildPageContentArrayForList(attributedElements);
                var jsonString = String.empty;
                var pageContentObject = {SVer: "1.00"};
                pageContentObject.Slots = pageContentDataArray;
                try {
                    jsonString = JSON.stringify(pageContentObject)
                }
                catch(e) {
                    MS.Entertainment.Utilities.fail("failed to stringify page content: " + e)
                }
                return jsonString
            }, _buildPageContentArrayForList: function _buildPageContentObjectForElement(list) {
                var currentElement = null;
                var attributeValue = null;
                var pageContentTelemetryObject = null;
                var pageContentObjectProperty = null;
                var contentArray = [];
                for (var i = 0; i < list.length; i++) {
                    if (!list[i].hasAttribute("data-ent-pagecontent")) {
                        MS.Entertainment.Utilities.fail("element has invalid data-ent-pagecontent value");
                        continue
                    }
                    var attributeValue = list[i].getAttribute("data-ent-pagecontent");
                    if (!attributeValue) {
                        MS.Entertainment.Utilities.fail("element has data-ent-pagecontent, with no value");
                        continue
                    }
                    if (list[i].winControl)
                        pageContentObjectProperty = list[i].winControl[attributeValue];
                    else if (list[i][attributeValue])
                        pageContentObjectProperty = list[i][attributeValue];
                    else
                        pageContentObjectProperty = null;
                    if (pageContentObjectProperty && pageContentObjectProperty.mediaItem)
                        pageContentObjectProperty = pageContentObjectProperty.mediaItem;
                    pageContentTelemetryObject = this._buildPageContentSlotForElement({
                        slot: i, domElement: list[i], itemPropertyBag: pageContentObjectProperty
                    });
                    if (pageContentTelemetryObject)
                        contentArray.push(pageContentTelemetryObject)
                }
                return contentArray
            }, _buildPageContentSlotForElement: function _buildPageContentSlotForElement(pageContentObject) {
                var telemetryValueNames = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues;
                var contentCatalogIds = MS.Entertainment.Utilities.Telemetry.ContentCatalogId;
                var contentSourceIds = MS.Entertainment.Utilities.Telemetry.ContentSourceId;
                var contentTypeIds = MS.Entertainment.Utilities.Telemetry.ContentTypeId;
                var pageContentTelemetryObject = {};
                if (!pageContentObject)
                    return pageContentTelemetryObject;
                if (pageContentObject.slot >= 0)
                    pageContentTelemetryObject.Slot = "Slot-" + pageContentObject.slot.toString();
                else
                    pageContentTelemetryObject.Slot = "Slot-" + telemetryValueNames.Unknown;
                if (pageContentObject.itemPropertyBag) {
                    pageContentTelemetryObject.ContentId = pageContentObject.itemPropertyBag.serviceId;
                    pageContentTelemetryObject.ContentCatalogId = pageContentObject.itemPropertyBag.catalogId;
                    pageContentTelemetryObject.ContentSourceId = pageContentObject.itemPropertyBag.sourceId;
                    pageContentTelemetryObject.ContentTypeId = pageContentObject.itemPropertyBag.typeId || contentTypeIds.Editorial
                }
                else if (pageContentObject.domElement && pageContentObject.domElement.hasAttribute("data-win-automationid")) {
                    pageContentTelemetryObject.ContentId = pageContentObject.domElement.getAttribute("data-win-automationid");
                    pageContentTelemetryObject.ContentCatalogId = contentCatalogIds.App;
                    pageContentTelemetryObject.ContentSourceId = contentSourceIds.App;
                    pageContentTelemetryObject.ContentTypeId = contentTypeIds.App
                }
                else {
                    pageContentTelemetryObject.ContentId = pageContentObject.contentId;
                    pageContentTelemetryObject.ContentCatalogId = pageContentObject.contentCatalogId;
                    pageContentTelemetryObject.ContentSourceId = pageContentObject.contentSourceId;
                    pageContentTelemetryObject.ContentTypeId = pageContentObject.contentTypeId
                }
                pageContentTelemetryObject.ContentName = pageContentTelemetryObject.ContentId || String.empty;
                pageContentTelemetryObject.ContentTags = String.empty;
                pageContentTelemetryObject.ProductCatalogId = String.empty;
                pageContentTelemetryObject.ProductId = String.empty;
                if (pageContentObject.areaName)
                    pageContentTelemetryObject.AreaId = pageContentObject.areaName;
                else if (pageContentObject.domElement)
                    pageContentTelemetryObject.AreaId = this._findAreaFromContentItem(pageContentObject.domElement);
                else
                    pageContentTelemetryObject.AreaId = telemetryValueNames.Unknown;
                pageContentTelemetryObject.CampaignId = String.empty;
                pageContentTelemetryObject.GroupId = String.empty;
                return pageContentTelemetryObject
            }, _findAreaFromContentItem: function _findAreaFromContentItem(element) {
                var currentElement = element;
                var areaName = String.empty;
                while (currentElement && !areaName) {
                    areaName = currentElement.getAttribute("data-ent-areaname") || String.empty;
                    currentElement = currentElement.parentElement
                }
                return areaName
            }, mediaUsageContentTypeName: function mediaUsageContentTypeName(media) {
                var name;
                name = media.edsMediaItemTypeString;
                if (!name) {
                    var mediaType = media.mediaType;
                    switch (mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.video:
                            if (media.videoType === Microsoft.Entertainment.Queries.VideoType.tvEpisode)
                                name = MS.Entertainment.Data.Query.edsMediaType.tvEpisode;
                            else if (media.videoType === Microsoft.Entertainment.Queries.VideoType.musicVideo)
                                name = MS.Entertainment.Data.Query.edsMediaType.musicVideo;
                            else if (media.videoType === Microsoft.Entertainment.Queries.VideoType.movie)
                                name = MS.Entertainment.Data.Query.edsMediaType.movie;
                            else
                                name = String.empty;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeries:
                            name = MS.Entertainment.Data.Query.edsMediaType.tvSeries;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeason:
                            name = MS.Entertainment.Data.Query.edsMediaType.tvSeason;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                            name = MS.Entertainment.Data.Query.edsMediaType.musicArtist;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                            name = MS.Entertainment.Data.Query.edsMediaType.album;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                            name = MS.Entertainment.Data.Query.edsMediaType.track;
                            break;
                        default:
                            name = String.empty;
                            break
                    }
                }
                return name
            }, logPublicationUsageHappened: function logPublicationUsageHappened(pubControl, eventType, isStartEvent) {
                if (!pubControl)
                    return;
                var position = pubControl.position;
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PublicationUsage);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.EventType, eventType);
                if (position)
                    position = (position | 0);
                if (position < 101 && position >= 0)
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Position, position);
                else {
                    MS.Entertainment.Utilities.fail("Publication position greater than 100% or less than 0.00%, position is " + position);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Position, String.empty)
                }
                if (!pubControl.identifier)
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ProviderMediaId, pubControl.identifier);
                var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                if (uiStateService.isAppVisible)
                    if (uiStateService.isSnapped)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ScreenSize, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Docked);
                    else
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ScreenSize, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Full);
                else
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ScreenSize, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Minimized);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.IsDrm, pubControl.isDrm);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaType, pubControl.mediaType);
                this.closeDataPoint(dataPoint)
            }, logMediaUsageHappened: function logMediaUsageHappened(playbackControl, position, eventType, isStartEvent) {
                if (!playbackControl)
                    return;
                var mediaInstance = playbackControl.currentMedia;
                if (playbackControl._player && playbackControl._player._currentMedia)
                    mediaInstance = playbackControl._player._currentMedia;
                var media = null;
                if (mediaInstance && mediaInstance._mediaItem && mediaInstance._mediaItem.data)
                    media = mediaInstance._mediaItem.data;
                if (!media)
                    return;
                var contentType = this.mediaUsageContentTypeName(media);
                if (!contentType)
                    return;
                if (contentType === MS.Entertainment.Data.Query.edsMediaType.track && mediaInstance) {
                    if (mediaInstance.isAudioAd)
                        contentType = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.MusicAd;
                    else if (mediaInstance.isPreview)
                        contentType = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.MusicPreview
                }
                else if (contentType === MS.Entertainment.Data.Query.edsMediaType.movie && mediaInstance && mediaInstance.isPreview)
                    contentType = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.VideoPreview;
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.MediaUsageEvent);
                dataPoint.appendDestination(Microsoft.Entertainment.Platform.Logging.TelemetryListener.mediaUsage);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.EventType, eventType);
                if (position)
                    position = (position | 0);
                if (position !== this.badPositionMs)
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Position, position);
                else {
                    MS.Entertainment.Utilities.assert(false, "playback position is 604800000(7days), duration is " + playbackControl.duration);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Position, String.empty)
                }
                if (!MS.Entertainment.Utilities.isEmptyGuid(media.canonicalId))
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.BingMediaId, media.canonicalId);
                if (!MS.Entertainment.Utilities.isEmptyGuid(media.zuneId))
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ProviderMediaId, media.zuneId);
                if (MS.Entertainment.Utilities.isEmptyGuid(media.canonicalId) && MS.Entertainment.Utilities.isEmptyGuid(media.zuneId)) {
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.BingMediaId, String.empty);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ProviderMediaId, String.empty)
                }
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ContentType, contentType);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.IsStream, mediaInstance && mediaInstance.isLocal ? "false" : "true");
                var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                if (uiStateService.isAppVisible)
                    if (uiStateService.isSnapped)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ScreenSize, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Docked);
                    else
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ScreenSize, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Full);
                else
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ScreenSize, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Minimized);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.IsDrm, this._getIsDrmFromMediaInstance(mediaInstance));
                var playbackDuration = 0;
                if (isStartEvent) {
                    var now = Date.now();
                    this._totalTimeCounterStart = now
                }
                else if (this._totalTimeCounterStart !== 0) {
                    playbackDuration = Date.now() - this._totalTimeCounterStart;
                    this._totalTimeCounterStart = 0
                }
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.PlaybackDurationMs, playbackDuration);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ProviderMediaInstanceId, mediaInstance && mediaInstance.mediaInstanceId ? mediaInstance.mediaInstanceId : String.empty);
                if (playbackControl.duration && playbackControl.duration > 0)
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaLengthMs, playbackControl.duration);
                else
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.MediaLengthMs, mediaInstance && mediaInstance.duration ? mediaInstance.duration : String.empty);
                var acquisitionData = media.acquisitionData ? media.acquisitionData : (mediaInstance && mediaInstance.acquisitionData);
                if (acquisitionData) {
                    MS.Entertainment.Utilities.assert(acquisitionData.acquisitionContext, "Expected acquisition context to be defined for media usage event");
                    if (acquisitionData.acquisitionContext)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContext, MS.Entertainment.Utilities.getAcquisitionContextFromEnum(acquisitionData.acquisitionContext));
                    MS.Entertainment.Utilities.assert(acquisitionData.acquisitionContextType, "Expected acquisition context type to be defined for media usage event");
                    if (acquisitionData.acquisitionContextType)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContextType, MS.Entertainment.Utilities.getAcquisitionContextTypeFromEnum(acquisitionData.acquisitionContextType));
                    if (!MS.Entertainment.Utilities.isEmptyGuid(acquisitionData.acquisitionContextID)) {
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContextId, acquisitionData.acquisitionContextID);
                        if (acquisitionData.acquisitionContextType === Microsoft.Entertainment.Platform.AcquisitionContextType.playlist)
                            dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AdditionalContext, acquisitionData.acquisitionAdditionalContext || media.playlistName)
                    }
                    else if (acquisitionData.acquisitionContextType === Microsoft.Entertainment.Platform.AcquisitionContextType.track && !MS.Entertainment.Utilities.isEmptyGuid(media.canonicalId))
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContextId, (media.track && media.track.canonicalId) || media.canonicalId);
                    else if (acquisitionData.acquisitionContextType === Microsoft.Entertainment.Platform.AcquisitionContextType.album && media.album && !MS.Entertainment.Utilities.isEmptyGuid(media.album.canonicalId))
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContextId, media.album.canonicalId);
                    else if (acquisitionData.acquisitionContextType === Microsoft.Entertainment.Platform.AcquisitionContextType.artist && media.artist && !MS.Entertainment.Utilities.isEmptyGuid(media.artist.canonicalId))
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContextId, media.artist.canonicalId);
                    else if (acquisitionData.acquisitionContextType === Microsoft.Entertainment.Platform.AcquisitionContextType.playlist && MS.Entertainment.Utilities.isValidServiceId(media.playlistCloudSyncId)) {
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContextId, media.playlistCloudSyncId);
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AdditionalContext, media.playlistName)
                    }
                    else if (acquisitionData.acquisitionContextType === Microsoft.Entertainment.Platform.AcquisitionContextType.musicVideo && !MS.Entertainment.Utilities.isEmptyGuid(media.canonicalId))
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContextId, (media.musicVideo && media.musicVideo.canonicalId) || media.canonicalId)
                }
                else if (mediaInstance && mediaInstance.isAudioAd) {
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContext, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.MusicAd);
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionContextType, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.MusicAd)
                }
                this.closeDataPoint(dataPoint)
            }, logPlayHappened: function logPlayHappened(playbackControl, position) {
                if (this._somethingPlaying)
                    this.logMediaUsageHappened(playbackControl, position, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Resume, true);
                else
                    this.logMediaUsageHappened(playbackControl, position, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Play, true);
                this._somethingPlaying = true
            }, logPauseHappened: function logPauseHappened(playbackControl, position) {
                if (this._somethingPlaying)
                    this.logMediaUsageHappened(playbackControl, position, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Pause, false)
            }, logEndHappened: function logEndHappened(playbackControl, position) {
                if (this._somethingPlaying)
                    this.logMediaUsageHappened(playbackControl, position, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.End, false);
                this._somethingPlaying = false
            }, logSkipHappened: function logSkipHappened(playbackControl, position) {
                if (this._somethingPlaying)
                    this.logMediaUsageHappened(playbackControl, position, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Skip, false);
                this._somethingPlaying = false
            }, logPurchaseHappened: function logPurchaseHappened(media, offerId) {
                if (!media)
                    return;
                var serviceIdToUse = null;
                if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeason)
                    serviceIdToUse = media.seasonServiceId;
                else
                    serviceIdToUse = media.canonicalId;
                if (MS.Entertainment.Utilities.isEmptyGuid(media.zuneId) && MS.Entertainment.Utilities.isEmptyGuid(serviceIdToUse))
                    return;
                var contentType = this.mediaUsageContentTypeName(media);
                if (!contentType)
                    return;
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.MediaUsageEvent);
                dataPoint.appendDestination(Microsoft.Entertainment.Platform.Logging.TelemetryListener.mediaUsage);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.EventType, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Purchase);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Position, 0);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ContentType, contentType);
                if (!MS.Entertainment.Utilities.isEmptyGuid(serviceIdToUse))
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.BingMediaId, serviceIdToUse);
                if (!MS.Entertainment.Utilities.isEmptyGuid(media.zuneId))
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ProviderMediaId, media.zuneId);
                var acquisitionType = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Purchase;
                if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.video) {
                    var licenseRight = this.licenseRightFromMedia(media, offerId);
                    if (licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Rent || licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.RentStream)
                        acquisitionType = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Rental
                }
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.AcquisitionType, acquisitionType);
                var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                if (uiStateService.isAppVisible)
                    if (uiStateService.isSnapped)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ScreenSize, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Docked);
                    else
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ScreenSize, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Full);
                else
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ScreenSize, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Minimized);
                this.closeDataPoint(dataPoint)
            }, logDownloadHappened: function logDownloadHappened(media) {
                if (!media)
                    return;
                if (MS.Entertainment.Utilities.isEmptyGuid(media.zuneId) && MS.Entertainment.Utilities.isEmptyGuid(media.canonicalId))
                    return;
                var contentType = this.mediaUsageContentTypeName(media);
                if (!contentType)
                    return;
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.MediaUsageEvent);
                dataPoint.appendDestination(Microsoft.Entertainment.Platform.Logging.TelemetryListener.mediaUsage);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.EventType, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Download);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Position, 0);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ContentType, contentType);
                if (!MS.Entertainment.Utilities.isEmptyGuid(media.canonicalId))
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.BingMediaId, media.canonicalId);
                if (!MS.Entertainment.Utilities.isEmptyGuid(media.zuneId))
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ProviderMediaId, media.zuneId);
                var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                if (uiStateService.isAppVisible)
                    if (uiStateService.isSnapped)
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ScreenSize, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Docked);
                    else
                        dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ScreenSize, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Full);
                else
                    dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ScreenSize, MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Minimized);
                this.closeDataPoint(dataPoint)
            }, logPlaybackErrorConvertedToGeneric: function logPlaybackErrorConvertedToGeneric(originalCode, originalExtendedCode, context) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlaybackErrorConvertedToGeneric);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ErrorCode, (originalCode ? originalCode : 0));
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ExtendedCode, (originalExtendedCode ? originalExtendedCode : 0));
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Context, (context ? context : ""));
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.PlaybackSource, MS.Entertainment.Platform.PlaybackHelpers.lastPlaybackSource);
                this.closeDataPoint(dataPoint)
            }, logRootLicenseAcquisition: function logRootLicenseAcquisition(status, errorCode) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.RootLicenseAcquisition);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Status, status);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ErrorCode, (errorCode ? errorCode : 0));
                this.closeDataPoint(dataPoint)
            }, logReactiveLicenseAcquisitionProgress: function logReactiveLicenseAcquisitionProgress(mediaInstance, kid, state) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.ReactiveLicenseAcquisition);
                this._appendPlaybackCommonDataToDataPoint(mediaInstance, dataPoint);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.State, state);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Kid, kid ? kid : MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Unknown);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Miid, mediaInstance && mediaInstance.mediaInstanceId ? mediaInstance.mediaInstanceId : MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Unknown);
                this.closeDataPoint(dataPoint)
            }, logWelcomePanelDismissed: function logWelcomePanelDismissed() {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.WelcomePanelDismissed);
                this.appendUIPath(dataPoint);
                this.closeDataPoint(dataPoint)
            }, logWelcomePanelStartButton: function logWelcomePanelStartButton() {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.WelcomePanelStartButtonClicked);
                this.appendUIPath(dataPoint);
                this.closeDataPoint(dataPoint)
            }, logSignInWelcomePanel: function logSignInWelcomePanel() {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.SignInWelcomePanel);
                this.appendUIPath(dataPoint);
                this.closeDataPoint(dataPoint)
            }, logErrorInMediaTagRemoval: function logErrorInMediaTagRemoval(error) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.ErrorInMediaTagRemoval);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.Error, error);
                this.closeDataPoint(dataPoint)
            }, logVideoPostRollAction: function logVideoPostRollAction(abTestName, abGroupName, action) {
                var dataPoint = this.openDataPoint();
                dataPoint.appendEventName(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.VideoPostRollAction);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ABTestName, abTestName);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ABGroupName, abGroupName);
                dataPoint.appendParameter(MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.VideoPostRollAction, action);
                this.closeDataPoint(dataPoint)
            }, badPositionMs: 604800000, Events: {CloudMatchStateChangeMethod: "CloudMatchEnabledStateChangeMethod"}, StateChangeMethodValues: {
                dialog: "dialog", toggle: "toggle", notification: "notification"
            }, InputActionTypes: {Click: "1"}, InputMethodTypes: {Click: "8"}, ContentCatalogId: {
                App: "1", Compass: "2", XboxCatalog: "3", EDS: "4"
            }, ContentSourceId: {
                UAD: "1", Compass: "2", App: "3", Recommendation: "4", SDS: "5"
            }, ContentTypeId: {
                Advertisement: "1", Editorial: "2", App: "3", Webpage: "4", Weblink: "5", TextEntry: "6"
            }, PageTypeId: {
                Dash: "1", Webblend: "2", WebPage: "3", Popup: "4", Charm: "5", Menu: "6", AppBar: "7", SmartDJ: "8", Featured: "9", TopMusic: "10", Details: "11"
            }, ScreenState: {
                FullScreen: "FullScreen", Snapped: "Snapped"
            }, SignInState: {
                Authenticated: "Authenticated", Anonymous: "Anonymous"
            }, TelemetryEvents: {
                AccountCreation: "AccountCreation", AuthHeaderResult: "AuthHeaderResult", CommandUsage: "CommandUsage", ErrorInMediaTagRemoval: "ErrorInMediaTagRemoval", GetPreferredStreamResult: "GetPreferredStreamResult", HydratorWrongMediaType: "HydratorWrongMediaType", ImmersiveDetailsMetaDataUpdated: "ImmersiveDetailsMetaDataUpdated", MediaUsageEvent: "MediaUsageEvent", MuteStateSelected: "MuteStateSelected", ModifierUsage: "ModifierUsage", NetworkService: "NetworkService", NotificationUsage: "NotificationUsage", PageAction: "PageAction", PageView: "PageView", PlaybackAttempted: "PlaybackAttempted", PlaybackControl: "PlaybackControl", PlaybackError: "PlaybackError", PlaybackErrorConvertedToGeneric: "PlaybackErrorConvertedToGeneric", PlaybackHappened: "PlaybackHappened", PlayerBufferState: "PlayerBufferState", PlayToReceiverError: "PlayToReceiverError", PlayToReceiverPlaybackError: "PlayToReceiverPlaybackError", PlayToReceiverSessionCompleted: "PlayToReceiverSessionCompleted", PlayToReceiverSessionRequested: "PlayToReceiverSessionRequested", PlayToSenderAttempted: "PlayToSenderAttempted", PlayToSenderError: "PlayToSenderError", PlayToSenderPlaybackHappened: "PlayToSenderPlaybackHappened", PopoverDisplay: "PopoverDisplay", PreviewPlayed: "PreviewPlayed", PublicationUsage: "PublicationUsage", PurchaseMade: "PurchaseMade", PurchaseResponse: "PurchaseResponse", PurchaseReviewStart: "PurchaseReviewStart", RecentUpsellPanelShown: "RecentUpsellPanelShown", ReactiveLicenseAcquisition: "ReactiveLicenseAcquisition", RootLicenseAcquisition: "RootLicenseAcquisition", SearchEnter: "SearchEnter", SearchExit: "SearchExit", SearchGlyphClick: "SearchGlyphClick", SearchPanelLaunch: "SearchPanelLaunch", SearchPanelLaunchFailure: "SearchPanelLaunchFailure", SearchQuerySubmitted: "SearchQuerySubmitted", SearchWordWheelEnter: "SearchWordWheelEnter", SemanticZoom: "SemanticZoom", SendASmile: "Send A Smile", Settings: "Settings", ShareCharm: "ShareCharm", ShareEDSWebQueryComplete: "ShareEDSWebQueryInitiated", ShareEDSWebQueryInitiated: "ShareEDSWebQueryInitiated", SignIn: "SignIn", SignInWelcomePanel: "SignInWelcomePanel", SignOut: "SignOut", SubscriptionSignupPrePurchase: "SubscriptionSignupPrePurchase", SubscriptionSignupPurchaseMade: "SubscriptionSignupPurchaseMade", UIFailure: "UIFailure", UpsellDialogDisplayed: "UpsellDialogDisplayed", UserEngagementCollectionMessageClicked: "UserEngagementCollectionMessageClicked", UserEngagementExternalLinkClicked: "UserEngagementExternalLinkClicked", UserEngagementMessageClick: "UserEngagementMessageClick", UserEngagementMessageDisplayed: "UserEngagementMessageDisplayed", VideoPostRollAction: "VideoPostRollAction", VideoPrePurchaseBlocked: "VideoPrePurchaseBlocked", VolumeSelected: "VolumeSelected", WebBlendError: "WebBlendError", WebBlendLaunch: "WebBlendLaunch", WelcomePageEntered: "WelcomePageEntered", WelcomePanelDismissed: "WelcomePanelDismissed", WelcomePanelStartButtonClicked: "WelcomePanelStartButtonClicked", WhatsNewDismissed: "WhatsNewDismissed", X8AdClicked: "X8AdClicked", X8NewPage: "X8NewPage", X8UsageTime: "X8UsageTime"
            }, TelemetryParameterNames: {
                ABGroupName: "ABGroupName", ABTestName: "ABTestName", AcceptTermsOfService: "AcceptTermsOfService", AcquisitionContext: "AcquisitionContext", AcquisitionContextId: "AcquisitionContextId", AcquisitionContextType: "AcquisitionContextType", AcquisitionType: "AcquisitionType", ActionInputMethodId: "ActionInputMethodId", actionType: "actionType", ActionTypeId: "ActionTypeId", AdInfo: "AdInfo", AdditionalContext: "AdditionalContext", AppMode: "AppMode", ArtistServiceId: "ArtistServiceId", AuthHeaderResultText: "AuthHeaderResultText", AuthState: "AuthState", AutomationId: "AutomationId", BingMediaId: "BingMediaId", BufferEnd: "BufferEnd", BufferStart: "BufferStart", CampaignId: "CampaignId", CanonicalId: "CanonicalId", CanSignOut: "CanSignOut", ColorDepth: "ColorDepth", commandId: "commandId", commandLabel: "commandLabel", CommentText: "CommentText", Content: "Content", ContentIssueType: "ContentIssueType", ContentJsonVersion: "ContentJsonVersion", ContentType: "ContentType", Context: "Context", ControlUsed: "ControlUsed", CorrelationID: "CorrelationID", CreateAccount: "CreateAccount", CurrentPage: "currentPage", CurrentPosition: "CurrentPosition", defaultPlatformType: "defaultPlatformType", DeliveryFormat: "DeliveryFormat", DestPageTypeId: "DestPageTypeId", DestPageURI: "DestPageURI", DialogType: "DialogType", DisplayPrice: "DisplayPrice", DistributionRight: "DistributionRight", DMRFriendlyName: "DMRFriendlyName", Duration: "Duration", EdsMediaType: "EdsMediaType", EDSSearch: "EDSSearch", Error: "Error", ErrorCode: "ErrorCode", ErrorContext: "ErrorContext", ErrorId: "ErrorId", ErrorTimestamp: "ErrorTimestamp", EventTime: "EventTime", EventType: "EventType", ExpectedMediaType: "ExpectedEdsMediaType", executeLocation: "executeLocation", ExtendedCode: "ExtendedCode", Extension: "Extension", ExternalLink: "ExternalLink", FeedbackType: "FeedbackType", ForceSignOut: "ForceSignOut", FreePlaysLeft: "FreePlaysLeft", FromAd: "FromAd", FromCollection: "FromCollection", FromPreview: "FromPreview", GroupId: "GroupId", HasLocalContent: "HasLocalContent", HasPurchasedCount: "HasPurchasedCount", HCR: "HCR", hub: "hub", Id: "Id", InCollection: "InCollection", IsDrm: "IsDrm", IsLibraryQuery: "IsLibraryQuery", IsNewNotification: "IsNewNotification", IsNextMediaInstance: "IsNextMediaInstance", IsPreview: "IsPreview", IsStream: "IsStream", IsSubscription: "IsSubscription", IssueType: "IssueType", ItemCount: "ItemCount", itemTypeQueryString: "itemTypeQueryString", Kid: "Kid", Label: "Label", LastUIPath: "LastUIPath", LiveMembershipLevel: "LiveMembershipLevel", licenseRight: "licenseRight", LicenseRight: "LicenseRight", Location: "Location", MediaId: "MediaId", MediaItem: "MediaItem", MediaItemInstanceId: "MediaItemInstanceId", MediaLengthMs: "MediaLengthMs", MediaName: "MediaName", mediaName: "mediaName", MediaType: "MediaType", mediaType: "mediaType", Message: "Message", MessageText: "MessageText", MethodInitiated: "MethodInitiated", Miid: "Miid", MusicPlaybackBackground: "MusicPlaybackBackground", MusicPlaybackSnapped: "MusicPlaybackSnapped", MusicPlaybackUnsnapped: "MusicPlaybackUnsnapped", MuteState: "MuteState", name: "name", NetworkStatus: "NetworkStatus", NoConnectedAccount: "NoConnectedAccount", NotificationCategory: "NotificationCategory", NotificationTitle: "NotificationTitle", NotSnapped: "NotSnapped", OfferId: "OfferId", page: "page", PageName: "PageName", PageParameters: "PageParameters", PageTags: "PageTags", PageTypeId: "PageTypeId", PageURI: "PageURI", ParentProductGuid: "ParentProductGuid", ParentSmid: "ParentSmid", PivotSelected: "PivotSelected", PlayabilityCloudCounts: "PlayabilityCloudCounts", PlayabilityLocalCounts: "PlayabilityLocalCounts", PlaybackDurationMs: "PlaybackDurationMs", PlaybackSource: "PlaybackSource", PlayLocation: "PlayLocation", PlayToType: "PlayToType", MFSRAudioSubtype: "MFSRAudioSubtype", MFSRVideoSubtype: "MFSRVideoSubtype", MFSRMimeType: "MFSRMimeType", MFSRSourceCharacter: "MFSRSourceCharacter", MFSRError: "MFSRError", Position: "Position", Price: "Price", ProductCatalogId: "ProductCatalogId", ProductGuid: "ProductGuid", ProductId: "ProductId", ProtectionState: "ProtectionState", ProviderMediaId: "ProviderMediaId", ProviderMediaInstanceId: "ProviderMediaInstanceId", PurchaseHistoryServiceIsGroveling: "PurchaseHistoryServiceIsGroveling", queryId: "queryId", Reason: "Reason", RefererPageTypeId: "RefererPageTypeId", RefererPageURI: "RefererPageURI", RefreshSignInResult: "RefreshSignInResult", ResolutionHeight: "ResolutionHeight", ResolutionWidth: "ResolutionWidth", Result: "Result", Retries: "Retries", ScreenSize: "ScreenSize", ScreenState: "ScreenState", SearchDestination: "SearchDestination", SearchHCRMediaName: "SearchHCRMediaName", SearchHCRMediaType: "SearchHCRMediaType", secondaryGamerTag: "secondaryGamerTag", SelectedItemMediaType: "SelectedItemMediaType", SelectedItemTitle: "SelectedItemTitle", seriesId: "seriesId", ServiceId: "ServiceId", serviceId: "serviceId", SettingsAccountUsage: "SettingsAccountUsage", SettingsHelpUsage: "SettingsHelpUsage", ShareTitle: "ShareTitle", ShareUri: "ShareUri", ShareUrl: "ShareUrl", SignedOfferGuid: "SignedOfferGuid", SignInDuration: "SignInDuration", SignInResult: "SignInResult", SignOutResult: "SignOutResult", SkinId: "SkinId", SMID: "SMID", Smid: "Smid", SnappedMode: "SnappedMode", StackTrace: "StackTrace", State: "State", Status: "Status", Success: "Success", SystemErrorCode: "SystemErrorCode", TaskId: "taskId", TemplateId: "TemplateId", TimeSpent: "TimeSpent", Title: "Title", titleId: "titleId", titleType: "titleType", TotalAudioAdImpressions: "TotalAudioAdImpressions", TotalVideoAdImpressions: "TotalVideoAdImpressions", TrackCount: "TrackCount", TracksLeft: "TracksLeft", UIPart: "UIPart", UIPath: "UIPath", UnsupportedRegion: "UnsupportedRegion", Url: "Url", UserId: "UserId", Users: "Users", VideoPlaybackBackground: "VideoPlaybackBackground", VideoPlaybackSnapped: "VideoPlaybackSnapped", VideoPlaybackUnsnapped: "VideoPlaybackUnsnapped", VideoPostRollAction: "VideoPostRollAction", VolumeValue: "VolumeValue", WhatsNewShownDuration: "WhatsNewOpenedDuration", WhereUsed: "WhereUsed", XCorrelationId: "XCorrelationId", ZuneId: "ZuneId", ZunePass: "ZunePass"
            }, TelemetryParameterValues: {
                Album: "Album", AppBar: "AppBar", Artist: "Artist", background: "background", BillingContactInfoClick: "BillingContactInfoClick", BillingHistoryClick: "BillingHistoryClick", ContactPreferencesClick: "ContactPreferencesClick", Docked: "Docked", Download: "Download", End: "End", Full: "Full", Game: "Game", GlobalAppbar: "GlobalAppbar", HomeNPTile: "HomeNPTile", Immersive: "Immersive", LoadXboxAccountSummaryPage: "LoadXboxAccountSummaryPage", ManageMusicDevices: "ManageMusicDevices", ManagePaymentOptionsClick: "ManagePaymentOptionsClick", MediaKey: "MediaKey", Minimized: "Minimized", Movie: "Movie", MusicAd: "MusicAd", MusicHelpClicked: "MusicHelpClicked", MusicPassSubscribeClick: "MusicPassSubscribeClick", MusicPreview: "MusicPreview", MusicTransportControls: "MusicTransportControls", MusicVideo: "MusicVideo", Next: "Next", None: "None", nowplaying: "nowplaying", NowPlaying: "NowPlaying", NycHelpClicked: "NycHelpClicked", Overlay: "Overlay", Pause: "Pause", Play: "Play", Playlist: "Playlist", PlayToByRef: "PlayToByRef", PlayToUGC: "PlayToUGC", Popover: "Popover", Previous: "Previous", Purchase: "Purchase", PrivacyClick: "PrivacyClick", RedeemCodeClick: "RedeemCodeClick", Rental: "Rental", Resume: "Resume", Skip: "Skip", Song: "Song", snapped: "snapped", SnappedAppBar: "SnappedAppBar", Stop: "Stop", TVEpisode: "TV Episode", TVSeason: "TV Season", TVSeries: "TV Series", unknown: "unknown", Unknown: "Unknown", UnknownHelpClicked: "UnknownHelpClicked", UserAccountLinkClicked: "UserAccountLinkClicked", VideoHelpClicked: "VideoHelpClicked", VideoPreview: "VideoPreview", VoiceCommand: "VoiceCommand"
            }
    })
})()
