﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator("/Framework/debug.js");
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment");
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI");
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Framework");
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Utilities");
(function() {
    "use strict";
    Object.defineProperty(Array.prototype, 'unique', {
        value: function() {
            var isUnique = function isUnique(element, index, array) {
                    return array.indexOf(element) === index
                };
            return this.filter(isUnique)
        }, enumerable: false
    });
    Object.defineProperty(Array.prototype, 'spliceArray', {
        value: function(index, deleteCount, items) {
            var args = [index, deleteCount].concat(items);
            return this.splice.apply(this, args)
        }, enumerable: false
    });
    (function() {
        if (String['load'])
            throw"String function already has a load function defined";
        else if (String.prototype['format'])
            throw"String prototype already has a format function defined";
        if (!String.empty)
            Object.defineProperty(String, "empty", {
                value: "", writable: false, enumerable: true
            });
        if (!String.isString)
            Object.defineProperty(String, "isString", {
                value: function(value) {
                    return typeof value === "string"
                }, writable: false, enumerable: true
            });
        if (!String.nbsp)
            Object.defineProperty(String, "nbsp", {
                value: "\u00A0", writable: false, enumerable: true
            });
        if (!String.ltrm)
            Object.defineProperty(String, "ltrm", {
                value: "\u200E", writable: false, enumerable: true
            });
        if (!String.ltrmCode)
            Object.defineProperty(String, "ltrmCode", {
                value: 0x200E, writable: false, enumerable: true
            });
        if (!String.rtlm)
            Object.defineProperty(String, "rtlm", {
                value: "\u200F", writable: false, enumerable: true
            });
        if (!String.rtlmCode)
            Object.defineProperty(String, "rtlmCode", {
                value: 0x200F, writable: false, enumerable: true
            });
        var resourceLoader;
        try {
            resourceLoader = new Windows.ApplicationModel.Resources.ResourceLoader
        }
        catch(err) {
            MS.Entertainment.fail("Windows.ApplicationModel.Resources.ResourceLoader failed with error: " + err)
        }
        String.idOverride = [];
        String.load = function loadString(stringId, skipIdVision) {
            if (!resourceLoader)
                return String.empty;
            if (String.idOverride[stringId])
                stringId = String.idOverride[stringId];
            MS.Entertainment.assert(stringId, "Can't find stringId: '" + stringId + "'");
            var loadedString = resourceLoader.getString(stringId);
            MS.Entertainment.assert(loadedString, "String lookup failed: '" + stringId + "'");
            return loadedString
        }
    })();
    String.prototype.format = function formatString() {
        var argumentList = arguments;
        var matchNum = this.match(/{(\d+)}/g);
        if (!matchNum) {
            MS.Entertainment.assert(false, "string value, '" + this.toString() + "', has no placeholders to allow for replacement");
            return this.toString()
        }
        if (matchNum.length !== argumentList.length) {
            MS.Entertainment.assert(false, "Incorrect number of arguments passed into format function. String: {0} Expected: {1}; Received: {2}".format(this.toString(), matchNum.length, argumentList.length));
            return String.empty
        }
        return this.replace(/{(\d+)}/g, function(placeholder, index) {
                return argumentList[index] !== undefined ? argumentList[index] : placeholder
            })
    };
    String.prototype.editDistanceFrom = function editDistanceFrom(targetString) {
        if (!targetString)
            return -1;
        var editMatrix = new Array(this.length + 1);
        for (var i = 0; i <= this.length; i++)
            editMatrix[i] = new Array(targetString.length + 1);
        for (var i = 0; i <= this.length; i++)
            editMatrix[i][0] = i;
        for (var j = 0; j <= targetString.length; j++)
            editMatrix[0][j] = j;
        for (var j = 0; j < targetString.length; j++)
            for (var i = 0; i < this.length; i++)
                if (this[i] === targetString[j])
                    editMatrix[i + 1][j + 1] = editMatrix[i][j];
                else {
                    var deleteDistance = editMatrix[i][j + 1] + 1;
                    var insertDistance = editMatrix[i + 1][j] + 1;
                    var substituteDistance = editMatrix[i][j] + 1;
                    editMatrix[i + 1][j + 1] = Math.min(deleteDistance, insertDistance, substituteDistance)
                }
        return editMatrix[this.length][targetString.length]
    };
    (function() {
        if (WinJS && WinJS.Promise && !WinJS.Promise.isCanceledError)
            Object.defineProperty(WinJS.Promise, "isCanceledError", {
                value: function isCanceled(error) {
                    return (error && error.name === "Canceled")
                }, writable: false, enumerable: true
            })
    })();
    (function() {
        var secondMS = 1000;
        var minuteMS = 60 * secondMS;
        var hourMS = 60 * minuteMS;
        var dayMS = 24 * hourMS;
        Date.minValue = new Date(0);
        Date.getTimeUTC = function toUTC(date) {
            return Date.UTC(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours(), date.getMinutes(), date.getSeconds(), date.getMilliseconds())
        },
        Date.subtract = function subtract(date1, date2) {
            var delta = Date.getTimeUTC(date1) - Date.getTimeUTC(date2);
            var result = {
                    days: 0, hours: 0, minutes: 0, seconds: 0, milliseconds: 0
                };
            result.days = Math.floor(delta / dayMS);
            delta = delta - (result.days * dayMS);
            result.hours = Math.floor(delta / hourMS);
            delta = delta - (result.hours * hourMS);
            result.minutes = Math.floor(delta / minuteMS);
            delta = delta - (result.minutes * minuteMS);
            result.seconds = Math.floor(delta / secondMS);
            delta = delta - (result.seconds * secondMS);
            result.milliseconds = delta;
            return result
        };
        function getNewConstructorFunction() {
            return function() {
                    if (this === window)
                        throw new Error("Need to use 'new' to invoke the constructor");
                    this.base()
                }
        }
        WinJS.Namespace.define("MS.Entertainment", {
            derive: function derive(baseClass, constructor, instanceMembers, staticMembers) {
                return MS.Entertainment.deferredDerive(baseClass, constructor, instanceMembers, staticMembers).get()
            }, deferredDerive: function deferredDerive(baseClass, constructor, instanceMembers, staticMembers) {
                    var value;
                    return {
                            get: function entertainmentDerive_getter() {
                                if (value)
                                    return value;
                                if (typeof baseClass === "string") {
                                    var baseClassName = baseClass;
                                    baseClass = WinJS.Utilities.getMember(baseClass);
                                    MS.Entertainment.Utilities.assert(baseClass, "Could not resolve baseClass named " + baseClassName)
                                }
                                MS.Entertainment.Utilities.assert(baseClass, "Base class was not valid. Can't derive without something to derive from");
                                if (!constructor)
                                    constructor = getNewConstructorFunction();
                                if (instanceMembers.base)
                                    throw"Deriving requires that there is no existing 'base' property";
                                var localInstanceMembers = MS.Entertainment.UI.Framework.markFunctionsOfObjectAsSupportedForProcessing(instanceMembers);
                                var helperConstructor = function(){};
                                if (baseClass.prototype._initObservable && !baseClass.prototype._preventObservableInheritance) {
                                    localInstanceMembers = WinJS.Binding.expandProperties(instanceMembers);
                                    helperConstructor = function() {
                                        MS.Entertainment.assert(this._backingData, "We thought this was observable. It's not, reconsider...");
                                        for (var member in instanceMembers)
                                            if (instanceMembers.hasOwnProperty(member))
                                                this._backingData[member] = instanceMembers[member]
                                    }
                                }
                                var baseConstructors = [function() {
                                            baseClass.prototype.constructor.apply(this, arguments);
                                            helperConstructor.call(this)
                                        }];
                                if (baseClass.prototype._baseConstructors)
                                    baseConstructors = baseClass.prototype._baseConstructors.concat(baseConstructors);
                                localInstanceMembers._baseConstructors = baseConstructors;
                                localInstanceMembers.base = function() {
                                    if (this._baseConstructors === baseConstructors)
                                        this._baseConstructors = this._baseConstructors.concat();
                                    var baseConstructor = this._baseConstructors.pop();
                                    if (baseConstructor)
                                        baseConstructor.apply(this, arguments)
                                };
                                value = WinJS.Class.derive(baseClass, constructor, localInstanceMembers, MS.Entertainment.UI.Framework.markFunctionsOfObjectAsSupportedForProcessing(staticMembers));
                                return value
                            }, configurable: true
                        }
                }
        })
    })();
    WinJS.Namespace.define("MS.Entertainment", {defineObservable: function(constructor, instanceMembers, staticMembers) {
            return MS.Entertainment.UI.Framework.derive(WinJS.Binding.define(instanceMembers), function defineObservableConstructor() {
                    this._initObservable(Object.create(MS.Entertainment.UI.Framework.markFunctionsOfObjectAsSupportedForProcessing(instanceMembers)));
                    constructor.apply(this, arguments)
                }, null, MS.Entertainment.UI.Framework.markFunctionsOfObjectAsSupportedForProcessing(staticMembers))
        }});
    WinJS.Namespace.define("MS.Entertainment", {defineOptionalObservable: function(constructor, instanceMembers, observableMembers, staticMembers) {
            return MS.Entertainment.UI.Framework.derive(WinJS.Binding.define(observableMembers), function defineOptionalObservableConstructor() {
                    if (this === window)
                        throw new Error("Need to use 'new' to invoke the constructor");
                    if (observableMembers)
                        this._initObservable(Object.create(observableMembers));
                    constructor.apply(this, arguments)
                }, instanceMembers, MS.Entertainment.UI.Framework.markFunctionsOfObjectAsSupportedForProcessing(staticMembers))
        }});
    WinJS.Namespace.define("MS.Entertainment", {
        _appMode: null, appMode: {
                get: function() {
                    if (this._appMode === null)
                        this._appMode = (new Microsoft.Entertainment.Application.ApplicationStatics).appMode;
                    return this._appMode
                }, set: function(value) {
                        this._appMode = value
                    }
            }
    });
    WinJS.Namespace.define("MS.Entertainment.Utilities", {EventSource: MS.Entertainment.UI.Framework.define(function eventSource() {
            this._eventhandlers = [];
            this._eventSource = new MS.Entertainment.UI.Framework.ObservableBase
        }, {
            _eventSource: null, _eventhandlers: null, addEventHandlers: function addEventHandlers(events) {
                    MS.Entertainment.UI.assert(!Array.isArray(events), "Events passed in cannot be an array");
                    var eventHandlers;
                    eventHandlers = MS.Entertainment.Utilities.addEventHandlers(this._eventSource, events);
                    this._eventhandlers.push(eventHandlers);
                    return eventHandlers
                }, dispose: function dispose() {
                    for (var i = 0; i < this._eventhandlers.length; i++)
                        this._eventhandlers[i].cancel();
                    this._eventhandlers = []
                }, dispatchEvent: function dispatchEvent(eventName, parameters) {
                    this._eventSource.dispatchEvent(eventName, parameters)
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.Utilities.DRM", {individualizationAsync: function() {
            if (MS.Entertainment.Platform.Playback.drmIndividualizationPromise)
                return MS.Entertainment.Platform.Playback.drmIndividualizationPromise;
            var drmIndivSignal = new MS.Entertainment.UI.Framework.Signal;
            var versionCheckPromise = drmIndivSignal.promise;
            MS.Entertainment.Platform.Playback.individualizationWorker = new Worker("/Components/Playback/IndivWorker.js");
            MS.Entertainment.Platform.Playback.individualizationWorker.onmessage = function(event) {
                drmIndivSignal.complete(event.data)
            };
            MS.Entertainment.Platform.Playback.drmIndividualizationPromise = versionCheckPromise.then(function checkComplete(individualizedAlready) {
                if (individualizedAlready)
                    return true;
                try {
                    var individualizationRequest = new Microsoft.Media.PlayReadyClient.PlayReadyIndividualizationServiceRequest;
                    return individualizationRequest.beginServiceRequest().then(function individualizationRequest_success() {
                            return true
                        }, function individualizationRequest_failure() {
                            return false
                        })
                }
                catch(ex) {
                    return false
                }
            }).then(function(isIndividualized) {
                MS.Entertainment.Utilities.isDrmIndividualized = isIndividualized;
                return isIndividualized
            });
            return MS.Entertainment.Platform.Playback.drmIndividualizationPromise
        }});
    WinJS.Namespace.define("MS.Entertainment", {ProxyHelpers: MS.Entertainment.UI.Framework.define(null, {}, {
            proxyObservables: function(outerObject, innerObject, observables) {
                observables.forEach(function(observable) {
                    var externalSet = function(v) {
                            var oldValue = outerObject[observable];
                            outerObject.setProperty(observable, v);
                            if (outerObject.dispatchEvent)
                                outerObject.dispatchEvent(observable + "Changed", {
                                    newValue: v, oldValue: oldValue
                                })
                        };
                    innerObject.bind(observable, externalSet);
                    Object.defineProperty(outerObject, observable, {
                        set: function(v) {
                            innerObject[observable] = v
                        }, get: function() {
                                return innerObject[observable]
                            }, enumerable: true, configurable: false
                    })
                })
            }, proxyProperties: function(outerObject, innerObject, properties) {
                    properties.forEach(function(property) {
                        Object.defineProperty(outerObject, property, {
                            set: function(v) {
                                innerObject[property] = v
                            }, get: function() {
                                    return innerObject[property]
                                }, enumerable: true, configurable: false
                        })
                    })
                }, delegateFunctions: function(outerObject, innerObject, methods) {
                    methods.forEach(function(method) {
                        Object.defineProperty(outerObject, method, {
                            value: function() {
                                return innerObject[method].apply(innerObject, arguments)
                            }, enumerable: true, configurable: false
                        })
                    })
                }
        })});
    (function() {
        WinJS.Namespace.define("MS.Entertainment.Utilities", {
            bindWorker: MS.Entertainment.UI.Framework.bindWorker, weakElementBindingInitializer: MS.Entertainment.UI.Framework.weakElementBindingInitializer, checkIfInDom: MS.Entertainment.UI.Framework.checkIfInDom, forceFullLayout: MS.Entertainment.UI.Framework.forceFullLayout, thawControlsInSubtree: MS.Entertainment.UI.Framework.thawControlsInSubtree, freezeControlsInSubtree: MS.Entertainment.UI.Framework.freezeControlsInSubtree, empty: MS.Entertainment.UI.Framework.empty, addEventHandlers: MS.Entertainment.UI.Framework.addEventHandlers, removeEvents: MS.Entertainment.UI.Framework.removeEvents
        });
        WinJS.Namespace.define("MS.Entertainment.Utilities", {
            EventInvoker: WinJS.Class.mix(function() {
                this._initObservable()
            }, WinJS.Utilities.eventMixin, WinJS.Binding.mixin), assertError: function assertError(message, error) {
                    if (!error)
                        return;
                    MS.Entertainment.Utilities.assert(WinJS.Promise.isCanceledError(error), "{0} [Error Number: {1}] [Error Code: {2}] [Error Async Type: {3}] [Error Async Causality: {4}] [Error Message: {5}] [Error Stack: {6}]".format(message, error.number, error.errorCode, error.asyncOpType, error.asyncOpCausalityId, error.message, error.stack))
                }, attachResizeEvent: function attachEvent(object, callback) {
                    if (object.attachEvent)
                        object.attachEvent("onresize", callback);
                    else
                        object.addEventListener("mselementresize", callback);
                    return {cancel: function() {
                                MS.Entertainment.Utilities.detachResizeEvent(object, callback)
                            }}
                }, detachResizeEvent: function detachEvent(object, callback) {
                    if (object.detachEvent)
                        object.detachEvent("onresize", callback);
                    else
                        object.removeEventListener("mselementresize", callback)
                }, attachPropertyChangeEvent: function attachPropertyChangeEvent(object, callback) {
                    if (object.attachEvent)
                        object.attachEvent("onpropertychange", callback);
                    else {
                        var callbackWrapper = function callbackWrapper(evt) {
                                var element = null;
                                var propertyName = null;
                                if (evt && evt[0]) {
                                    element = evt[0].target;
                                    propertyName = evt[0].attributeName
                                }
                                callback({
                                    srcElement: element, propertyName: propertyName
                                })
                            };
                        return new MutationObserver(callbackWrapper).observe(object, {
                                attributes: true, attributeFilter: ["aria-selected", "aria-pressed"]
                            })
                    }
                }, detachPropertyChangeEvent: function detachPropertyChangeEvent(object, callback) {
                    if (object.detachEvent)
                        object.detachEvent("onpropertychange", callback)
                }, waitForDomEvent: function waitForDomEvent(eventName, element, capture) {
                    return MS.Entertainment.Utilities.waitForDomEventMultipleTimes(eventName, element, 1, capture)
                }, waitForDomEventMultipleTimes: function waitForDomEventMultipleTimes(eventName, element, numEvents, capture) {
                    if (!element || !element.addEventListener || !eventName)
                        throw new Error("Invalid args: eventName: {0}, element: {1}".format(eventName, element && JSON.stringify(element)));
                    numEvents = numEvents || 1;
                    var signal = new MS.Entertainment.UI.Framework.Signal;
                    var onEvent = function onEvent(event) {
                            numEvents--;
                            if (numEvents === 0) {
                                element.removeEventListener(eventName, onEvent, capture);
                                signal.complete(event)
                            }
                        };
                    element.addEventListener(eventName, onEvent, capture);
                    return signal.promise.then(function onComplete(result) {
                            return result
                        }, function onError(err) {
                            element.removeEventListener(eventName, onEvent, capture);
                            return err
                        })
                }, validateIsMemberOrThrow: function validateIsMemberOrThrow(value, object) {
                    var member = "";
                    for (member in object)
                        if (object.hasOwnProperty(member) && value === object[member])
                            return;
                    throw"validateIsMemberOrThrow: Value not found in object.";
                }, loadHtmlPage: function loadHtmlPage(filePath, targetElement, dataContext) {
                    MS.Entertainment.Utilities.assert(targetElement, "No destination element was passed");
                    var container;
                    var load;
                    return WinJS.UI.Fragments.renderCopy(filePath).then(function placeHtmlPageInDom(fragment) {
                            var loadName;
                            var loadTags;
                            if (typeof targetElement === "string")
                                container = document.getElementById(targetElement);
                            else
                                container = targetElement;
                            MS.Entertainment.Utilities.assert(container, "Page container supplied couldn't be found");
                            MS.Entertainment.Utilities.empty(container);
                            loadTags = fragment.querySelectorAll("[data-ent-fragmentLoad]");
                            if (loadTags && loadTags.length > 0)
                                for (var i = 0, l = loadTags.length; i < l; i++)
                                    if (loadTags[i].nodeName !== "BODY") {
                                        loadName = loadTags[i].getAttribute("data-ent-fragmentLoad");
                                        break
                                    }
                            if (loadName) {
                                load = WinJS.Utilities.getMember(loadName);
                                MS.Entertainment.Utilities.assert(load, "Load function was not located: " + loadName)
                            }
                            container.appendChild(fragment)
                        }).then(function waitOnData() {
                            return dataContext
                        }).then(function(data) {
                            dataContext = data;
                            return WinJS.UI.processAll(container)
                        }).then(function() {
                            return WinJS.Binding.processAll(container, dataContext)
                        }).then(function callCallbacks() {
                            if (load)
                                load(container.children[0])
                        })
                }, documentAlreadyProcessed: false, systemAppListenersAttached: false, processAllOnDocumentLoaded: function processAllOnDocumentLoaded(element, dataContext) {
                    if (MS.Entertainment.Utilities.documentAlreadyProcessed)
                        return WinJS.Promise.timeout();
                    MS.Entertainment.Utilities.documentAlreadyProcessed = true;
                    return WinJS.Utilities.ready().then(function processAllAfterDocumentReady() {
                            return WinJS.UI.processAll(element)
                        }).then(function bindingProcessAllAfterControlProcessAll() {
                            return WinJS.Binding.processAll(element, dataContext)
                        })
                }, isAboveMemoryThreshold: function isAboveMemoryThreshold(thresholdPercentage) {
                    MS.Entertainment.Utilities.assert(thresholdPercentage > 0.0 && thresholdPercentage <= 1.0, "thresholdPercentage must be between 0.0 and 1.0");
                    var isAboveThreshold = false;
                    var resourceLimitsObject = WinJS.Utilities.getMember("App2.ApplicationModel.ApplicationResourceLimits");
                    if (!resourceLimitsObject || thresholdPercentage <= 0.0 || thresholdPercentage > 1.0)
                        return false;
                    try {
                        var currentMemoryUsageInKB = Math.max((resourceLimitsObject.currentMemoryUsage / 1024) | 0, 1);
                        var currentPercentage = (currentMemoryUsageInKB / App2.ApplicationModel.MemoryThresholdInKB);
                        isAboveThreshold = (thresholdPercentage < currentPercentage && !isNaN(currentPercentage))
                    }
                    catch(e) {
                        MS.Entertainment.Utilities.fail("isAboveMemoryThreshold failed: " + e)
                    }
                    return isAboveThreshold
                }, enterRestrictedBackground: function enterRestrictedBackground() {
                    var navigatonService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                    navigatonService.clearAllNavigationState();
                    WinJS.Promise.timeout(MS.Entertainment.UI.Framework.autoCleanupTimeout + 100).then(function() {
                        MS.Entertainment.UI.Framework.clearCaches();
                        MS.Entertainment.ServiceLocator.cleanupNonCriticalServices();
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isInRestrictedBackground = true
                    })
                }, leaveRestrictedBackground: function leaveRestrictedBackground() {
                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isInRestrictedBackground = false;
                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateToDefaultPage()
                }, multiStateToggleClassName: MS.Entertainment.Utilities.weakElementBindingInitializer(function multiStateToggleClassNameHelper(sourceValue, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    var maxStates = 5;
                    for (var i = 1; i < maxStates; i++)
                        WinJS.Utilities.removeClass(targetElement, cssClassName + "_" + i);
                    if (sourceValue)
                        WinJS.Utilities.addClass(targetElement, cssClassName + "_" + sourceValue)
                }), toggleTabIndex: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleTabIndex(sourceValue, targetElement) {
                    if (sourceValue)
                        targetElement.setAttribute("tabIndex", 0);
                    else
                        targetElement.removeAttribute("tabIndex")
                }), toggleTabIndexNegate: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleTabIndexNegate(sourceValue, targetElement) {
                    if (!sourceValue)
                        targetElement.setAttribute("tabIndex", 0);
                    else
                        targetElement.removeAttribute("tabIndex")
                }), toggleClassNameOnObservableArrayLength: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameOnObservableArrayLength(sourceValue, targetElement, cssClassName) {
                    if (!(sourceValue instanceof MS.Entertainment.ObservableArray))
                        return;
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    var handleArrayChanges = function handleArrayChanges(notification) {
                            if (sourceValue.length === 0)
                                WinJS.Utilities.addClass(targetElement, cssClassName);
                            else
                                WinJS.Utilities.removeClass(targetElement, cssClassName)
                        };
                    sourceValue.addChangeListener(handleArrayChanges);
                    handleArrayChanges()
                }), toggleClassNameOnObservableArrayLengthNegate: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameOnObservableArrayLengthNegate(sourceValue, targetElement, cssClassName) {
                    if (!(sourceValue instanceof MS.Entertainment.ObservableArray))
                        return;
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    var handleArrayChanges = function handleArrayChanges(notification) {
                            if (sourceValue.length === 0)
                                WinJS.Utilities.removeClass(targetElement, cssClassName);
                            else
                                WinJS.Utilities.addClass(targetElement, cssClassName)
                        };
                    sourceValue.addChangeListener(handleArrayChanges);
                    handleArrayChanges()
                }), toggleClassNameOnObservableArrayLengthMoreThanOne: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameOnObservableArrayLengthMoreThanOne(sourceValue, targetElement, cssClassName) {
                    if (!(sourceValue instanceof MS.Entertainment.ObservableArray))
                        return;
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    var handleArrayChanges = function handleArrayChanges(notification) {
                            if (sourceValue.length > 1)
                                WinJS.Utilities.addClass(targetElement, cssClassName);
                            else
                                WinJS.Utilities.removeClass(targetElement, cssClassName)
                        };
                    sourceValue.addChangeListener(handleArrayChanges);
                    handleArrayChanges()
                }), toggleClassNameOnObservableArrayLengthMoreThanOneNegate: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameOnObservableArrayLengthMoreThanOneNegate(sourceValue, targetElement, cssClassName) {
                    if (!(sourceValue instanceof MS.Entertainment.ObservableArray))
                        return;
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    var handleArrayChanges = function handleArrayChanges(notification) {
                            if (sourceValue.length > 1)
                                WinJS.Utilities.removeClass(targetElement, cssClassName);
                            else
                                WinJS.Utilities.addClass(targetElement, cssClassName)
                        };
                    sourceValue.addChangeListener(handleArrayChanges);
                    handleArrayChanges()
                }), setAttributeValue: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassName(sourceValue, targetElement, attributeName) {
                    if (!attributeName || !targetElement)
                        return;
                    if (Array.isArray(attributeName))
                        attributeName = attributeName[0];
                    switch (attributeName) {
                        case"dataWinAutomationId":
                            attributeName = "data-win-automationId";
                            break;
                        default:
                            break
                    }
                    if (sourceValue)
                        targetElement.setAttribute(attributeName, sourceValue);
                    else if (targetElement.hasAttribute(attributeName))
                        targetElement.removeAttribute(attributeName)
                }), toggleClassName: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassName(sourceValue, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    if (sourceValue)
                        WinJS.Utilities.addClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.removeClass(targetElement, cssClassName)
                }), toggleClassNameNegate: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameNegate(sourceValue, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    if (!sourceValue)
                        WinJS.Utilities.addClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.removeClass(targetElement, cssClassName)
                }), getConfigValueOneTime: WinJS.Utilities.markSupportedForProcessing(function getConfigValueOneTime(source, sourceProperties, dest, destProperties) {
                    var configManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                    var configurationValue = MS.Entertainment.Utilities.valueFromPropertyPathFragments(configManager, sourceProperties);
                    MS.Entertainment.Utilities.setFromPropertyPathFragments(dest, destProperties, configurationValue)
                }), formatDirection: MS.Entertainment.Utilities.weakElementBindingInitializer(function formatDirection(sourceValue, targetElement) {
                    if (sourceValue) {
                        var textDirection = MS.Entertainment.Utilities.detectStringDirection(sourceValue);
                        if (textDirection && textDirection !== MS.Entertainment.Utilities.getTextDirection())
                            switch (textDirection) {
                                case MS.Entertainment.UI.Framework.TextDirections.LeftToRight:
                                    WinJS.Utilities.addClass(targetElement, "ui-direction-ltr");
                                    break;
                                case MS.Entertainment.UI.Framework.TextDirections.RightToLeft:
                                    WinJS.Utilities.addClass(targetElement, "ui-direction-rtl");
                                    break
                            }
                        else
                            WinJS.Utilities.removeClass(targetElement, "ui-direction-ltr ui-direction-rtl")
                    }
                }), toggleClassNameViewStateNotReady: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameViewStateNotReady(sourceValue, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    if (sourceValue !== MS.Entertainment.UI.ViewState.ready)
                        WinJS.Utilities.addClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.removeClass(targetElement, cssClassName)
                }), toggleClassNameViewStateNotLoading: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameViewStateNotLoading(sourceValue, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    if (sourceValue !== MS.Entertainment.UI.ViewState.loading)
                        WinJS.Utilities.addClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.removeClass(targetElement, cssClassName)
                }), toggleClassNameOnFeatureDisabledAndNoSubscription: WinJS.Utilities.markSupportedForProcessing(function toggleClassNameOnFeatureDisabledAndNoSubscription(source, sourceProperty, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    var featureItem;
                    if (sourceProperty && sourceProperty[0])
                        featureItem = MS.Entertainment.Utilities.valueFromPropertyPathFragments(WinJS.Utilities.getMember(sourceProperty[0]), sourceProperty.slice(1));
                    MS.Entertainment.Utilities.assert(!!featureItem, "Feature enablement feature item not defined. featureItem: " + featureItem + ", sourceProperty: " + sourceProperty + ", targetElement: " + targetElement);
                    var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                    if (!MS.Entertainment.Utilities.currentOrLastUserSubscriptionEnabled() && featureItem && !featureEnablement.isEnabled(featureItem))
                        WinJS.Utilities.addClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.removeClass(targetElement, cssClassName)
                }), toggleClassNameOnFeatureDisabled: WinJS.Utilities.markSupportedForProcessing(function toggleClassNameOnFeatureDisabled(source, sourceProperty, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    var featureItem;
                    if (sourceProperty && sourceProperty[0])
                        featureItem = MS.Entertainment.Utilities.valueFromPropertyPathFragments(WinJS.Utilities.getMember(sourceProperty[0]), sourceProperty.slice(1));
                    MS.Entertainment.Utilities.assert(!!featureItem, "Feature enablement feature item not defined. featureItem: " + featureItem + ", sourceProperty: " + sourceProperty + ", targetElement: " + targetElement);
                    var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                    if (featureItem && !featureEnablement.isEnabled(featureItem))
                        WinJS.Utilities.addClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.removeClass(targetElement, cssClassName)
                }), toggleClassNameOnEmptyGuid: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameOnEmptyGuid(smid, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    if (!smid || smid === MS.Entertainment.Utilities.EMPTY_GUID)
                        WinJS.Utilities.addClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.removeClass(targetElement, cssClassName)
                }), toggleClassNameOnNonNegativeNumber: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameOnNonNegativeNumber(sourceValue, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    if (sourceValue && (typeof(sourceValue) === "number") && sourceValue >= 0)
                        WinJS.Utilities.addClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.removeClass(targetElement, cssClassName)
                }), toggleClassNameOnNegativeNumber: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameOnNegativeNumber(sourceValue, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    var sourceValueIsNumber = (sourceValue || sourceValue === 0) && (typeof(sourceValue) === "number");
                    if ((sourceValueIsNumber && sourceValue < 0) || !sourceValueIsNumber)
                        WinJS.Utilities.addClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.removeClass(targetElement, cssClassName)
                }), toggleClassNameOnNonPositiveNumber: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameOnNegativeNumber(sourceValue, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    if (sourceValue === undefined || sourceValue === null || typeof(sourceValue) !== "number" || sourceValue < 0)
                        WinJS.Utilities.addClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.removeClass(targetElement, cssClassName)
                }), toggleClassNameOnEmptyString: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameOnEmptyString(sourceValue, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    if (sourceValue !== null && sourceValue !== undefined)
                        sourceValue += String.empty;
                    if (sourceValue && sourceValue.length > 0)
                        WinJS.Utilities.removeClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.addClass(targetElement, cssClassName)
                }), toggleClassNameOnEmptyOrBlankString: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameOnEmptyOrBlankString(sourceValue, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    if (sourceValue !== null && sourceValue !== undefined)
                        sourceValue += String.empty;
                    if (sourceValue && sourceValue.trim().length > 0)
                        WinJS.Utilities.removeClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.addClass(targetElement, cssClassName)
                }), toggleClassNameOnNonEmptyString: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameOnNonEmptyString(sourceValue, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    if (sourceValue !== null && sourceValue !== undefined)
                        sourceValue += String.empty;
                    if (sourceValue && sourceValue.length > 0)
                        WinJS.Utilities.addClass(targetElement, cssClassName);
                    else
                        WinJS.Utilities.removeClass(targetElement, cssClassName)
                }), toggleTextContentOrEmptyString: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleTextContentOrEmptyString(toggleValue, targetElement, stringID) {
                    if (toggleValue && stringID && stringID[0])
                        targetElement.innerText = String.load(stringID[0]);
                    else
                        targetElement.innerText = String.empty
                }), appendClassName: MS.Entertainment.Utilities.weakElementBindingInitializer(function appendClassName(sourceValue, targetElement, targetProperty) {
                    if (!sourceValue)
                        return;
                    var sourceValueArray = sourceValue.split(" ");
                    for (var i = 0; i < sourceValueArray.length; i++)
                        if (sourceValueArray[i] && !WinJS.Utilities.hasClass(targetElement, sourceValueArray[i]))
                            WinJS.Utilities.addClass(targetElement, sourceValueArray[i])
                }), removeClassName: MS.Entertainment.Utilities.weakElementBindingInitializer(function removeClassName(sourceValue, targetElement, targetProperty) {
                    if (sourceValue && WinJS.Utilities.hasClass(targetElement, sourceValue))
                        WinJS.Utilities.removeClass(targetElement, sourceValue)
                }), limitClassName: MS.Entertainment.Utilities.weakElementBindingInitializer(function limitClassName(sourceValue, targetElement, targetProperty) {
                    if (!sourceValue || !targetElement)
                        return;
                    for (var i = 0; i < targetElement.classList.length; ) {
                        var className = targetElement.classList[i];
                        if (className && className.indexOf(targetProperty) === 0)
                            WinJS.Utilities.removeClass(targetElement, className);
                        else
                            i++
                    }
                    var sourceValueArray = sourceValue.split(" ");
                    for (var i = 0; i < sourceValueArray.length; i++) {
                        var className = sourceValueArray[i];
                        if (className && !WinJS.Utilities.hasClass(targetElement, className))
                            WinJS.Utilities.addClass(targetElement, className)
                    }
                }), toggleMediaTypeClassName: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleMediaTypeClassName(sourceValue, targetElement, cssClassName) {
                    if (!targetElement)
                        return;
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    var i;
                    var cssPrefix = cssClassName.toLowerCase() + "-";
                    var classList = targetElement.classList || [];
                    var mediaClasses = [];
                    for (i = 0; i < classList.length; ++i) {
                        var className = classList[i];
                        if (className.indexOf(cssPrefix) === 0)
                            mediaClasses.push(className)
                    }
                    for (i = 0; i < mediaClasses.length; ++i)
                        WinJS.Utilities.removeClass(targetElement, mediaClasses[i]);
                    if (sourceValue && sourceValue.mediaType) {
                        var mediaTypeString = MS.Entertainment.Data.Factory.Marketplace.edsMediaTypeFromDatabaseType(sourceValue.mediaType);
                        if (mediaTypeString)
                            WinJS.Utilities.addClass(targetElement, (cssPrefix + mediaTypeString).toLowerCase())
                    }
                }), appendMediaTypeClassName: MS.Entertainment.Utilities.weakElementBindingInitializer(function appendMediaTypeClassName(sourceValue, targetElement, cssClassName) {
                    if (Array.isArray(cssClassName))
                        cssClassName = cssClassName[0];
                    if (sourceValue && sourceValue.mediaType) {
                        var mediaTypeString = MS.Entertainment.Data.Factory.Marketplace.edsMediaTypeFromDatabaseType(sourceValue.mediaType);
                        if (mediaTypeString)
                            WinJS.Utilities.addClass(targetElement, (cssClassName + "-" + mediaTypeString).toLowerCase())
                    }
                }), toggleBoolean: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function toggleBoolean(sourceValue) {
                    return !sourceValue
                })), toggleHideOnElement: WinJS.Utilities.markSupportedForProcessing(function toggleHideOnElement(element, switchToHidden) {
                    if (switchToHidden)
                        WinJS.Utilities.addClass(element, "removeFromDisplay");
                    else
                        WinJS.Utilities.removeClass(element, "removeFromDisplay")
                }), useShowHideAnimationsOnValueChange: MS.Entertainment.Utilities.weakElementBindingInitializer(function useShowHideAnimationsOnValueChange(sourceValue, targetElement, elementProperty) {
                    if (sourceValue === targetElement[elementProperty])
                        return;
                    var hidePromise = null;
                    if (!WinJS.Utilities.hasClass(targetElement, "hideFromDisplay"))
                        hidePromise = MS.Entertainment.Utilities.hideElement(targetElement);
                    else
                        hidePromise = WinJS.Promise.wrap();
                    hidePromise.then(function elementHidden() {
                        targetElement[elementProperty] = sourceValue;
                        MS.Entertainment.Utilities.showElement(targetElement)
                    })
                }), forceShowAnimationOnLoad: MS.Entertainment.Utilities.weakElementBindingInitializer(function forceShowAnimationOnLoad(sourceValue, targetElement, elementProperty) {
                    if (sourceValue && !(Array.isArray(sourceValue) && sourceValue.length === 0))
                        MS.Entertainment.Utilities.showElement(targetElement)
                }), toggleClassOnElement: WinJS.Utilities.markSupportedForProcessing(function toggleHideOnElement(element, switchOn, className) {
                    if (Array.isArray(className))
                        className = className[0];
                    if (switchOn)
                        WinJS.Utilities.addClass(element, className);
                    else
                        WinJS.Utilities.removeClass(element, className)
                }), toggleVoiceTarget: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassName(sourceValue, targetElement) {
                    if (!MS.Entertainment.Utilities.isApp2)
                        return;
                    if (sourceValue)
                        WinJS.Utilities.addClass(targetElement, MS.Entertainment.UI.App2.VoiceStatics.voiceTextCssStyle);
                    else
                        WinJS.Utilities.removeClass(targetElement, MS.Entertainment.UI.App2.VoiceStatics.voiceTextCssStyle)
                }), toggleVoiceTargetNegate: MS.Entertainment.Utilities.weakElementBindingInitializer(function toggleClassNameNegate(sourceValue, targetElement) {
                    if (!MS.Entertainment.Utilities.isApp2)
                        return;
                    if (!sourceValue)
                        WinJS.Utilities.addClass(targetElement, MS.Entertainment.UI.App2.VoiceStatics.voiceTextCssStyle);
                    else
                        WinJS.Utilities.removeClass(targetElement, MS.Entertainment.UI.App2.VoiceStatics.voiceTextCssStyle)
                }), createActionFromBind: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function createActionFromBind(value) {
                    var action = value;
                    if (String.isString(action))
                        action = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).getAction(value);
                    return action
                })), bindAndMarkSupportedForProcessing: function(fn, thisPointer) {
                    var boundFunction = fn.bind(thisPointer);
                    return WinJS.Utilities.markSupportedForProcessing(boundFunction)
                }, findParentElementByClassName: function findParentElementByClassName(childElement, className) {
                    var parent = childElement.parentElement;
                    while (parent) {
                        if (WinJS.Utilities.hasClass(parent, className))
                            return parent;
                        parent = parent.parentElement
                    }
                }, isNumeric: function Utilities_isNumeric(sourceString) {
                    if (sourceString === null || sourceString === undefined || typeof sourceString === "object")
                        return false;
                    var validChars = "0123456789.";
                    for (var i = sourceString.length - 1; i >= 0; i--)
                        if (validChars.indexOf(sourceString.charAt(i)) === -1)
                            return false;
                    return true
                }, simpleEscapeHTML: function simpleEscapeHtml(text) {
                    if (text.indexOf("<") !== -1)
                        text = text.replace(/</g, "&lt;");
                    if (text.indexOf(">") !== -1)
                        text = text.replace(/>/g, "&gt;");
                    return text
                }, escapeHTML: (function escapeHTML(text) {
                    var element = null;
                    return function escapeHTML(text) {
                            if (!element)
                                element = document.createElement("div");
                            element.textContent = text;
                            return element.innerHTML
                        }
                })(), unEscapeHTML: (function unEscapeHTML(html) {
                    var element = null;
                    return function unEscapeHTML(html) {
                            if (!element)
                                element = document.createElement("div");
                            element.innerHTML = html;
                            return element.textContent
                        }
                })(), getChildControl: function Utilities_getChildControl(dom, name) {
                    var element;
                    try {
                        element = dom.querySelector("[data-ent-id='" + name + "']");
                        if (element && (element.winControl !== null && element.winControl !== undefined))
                            element = element.winControl
                    }
                    catch(error) {
                        MS.Entertainment.fail("Exception in getChildControl. error: " + error + " name: " + name);
                        element = null
                    }
                    return element
                }, _millisecondsToDate: function millisecondsToDate(milliseconds) {
                    var seconds = Math.floor(milliseconds / 1000);
                    milliseconds = milliseconds % 1000;
                    var minutes = Math.floor(seconds / 60);
                    seconds = seconds % 60;
                    var hours = Math.floor(minutes / 60);
                    minutes = minutes % 60;
                    return new Date(null, null, null, hours, minutes, seconds, milliseconds)
                }, millisecondsToTimeCode: function millisecondsToTimeCode(duration) {
                    var durationType = typeof duration;
                    if (!MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.dateTimeFormatters))
                        return String.empty;
                    else if (durationType === "number")
                        if (duration < 0)
                            return String.empty;
                        else
                            duration = MS.Entertainment.Utilities._millisecondsToDate(duration);
                    else if (durationType === "string")
                        duration = MS.Entertainment.Utilities._millisecondsToDate(parseInt(duration));
                    else if (!duration)
                        return String.empty;
                    var formatter;
                    var hoursString;
                    var minutesString;
                    var secondsString;
                    var hours = duration.getHours();
                    var minutes = duration.getMinutes();
                    var seconds = duration.getSeconds();
                    if (isNaN(hours))
                        hours = 0;
                    if (isNaN(minutes))
                        minutes = 0;
                    if (isNaN(seconds))
                        seconds = 0;
                    formatter = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber;
                    var timeString;
                    if (hours > 0) {
                        hoursString = formatter.format(hours);
                        formatter.integerDigits = 2;
                        minutesString = formatter.format(minutes);
                        secondsString = formatter.format(seconds);
                        timeString = String.load(String.id.IDS_HHMMSS_DURATION).format(hoursString, minutesString, secondsString)
                    }
                    else {
                        minutesString = formatter.format(minutes);
                        formatter.integerDigits = 2;
                        secondsString = formatter.format(seconds);
                        timeString = String.load(String.id.IDS_MMSS_DURATION).format(minutesString, secondsString)
                    }
                    formatter.integerDigits = MS.Entertainment.Formatters.DateTimeFormatters.defaultDecimalDigits;
                    return timeString
                }, formatTimeString: function formatTimeString(duration, options) {
                    if (typeof duration === "number" && duration < 0)
                        return String.empty;
                    var formatter;
                    var hoursString;
                    var minutesString;
                    var secondsString;
                    formatter = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber;
                    var timeString = String.empty;
                    var hours = 0;
                    var minutes = 0;
                    var seconds = 0;
                    options = options || {};
                    var excludeSeconds = options.excludeSeconds || false;
                    if (typeof duration === "number") {
                        hours = Math.floor(duration / (1000 * 60 * 60));
                        duration = duration - (hours * 1000 * 60 * 60);
                        minutes = Math.floor(duration / (1000 * 60));
                        duration = duration - (minutes * 1000 * 60);
                        seconds = Math.floor(duration / 1000)
                    }
                    else if (duration) {
                        hours = duration.getHours();
                        minutes = duration.getMinutes();
                        seconds = duration.getSeconds()
                    }
                    if (hours > 0) {
                        hoursString = formatter.format(hours);
                        minutesString = formatter.format(minutes);
                        timeString = String.load(String.id.IDS_DETAILS_DURATION_HOURS_FORMAT).format(hoursString, minutesString)
                    }
                    else {
                        minutesString = formatter.format(minutes);
                        secondsString = formatter.format(seconds);
                        if (!excludeSeconds)
                            timeString = String.load(String.id.IDS_DETAILS_DURATION_MINSEC_FORMAT).format(minutesString, secondsString);
                        else
                            timeString = String.load(String.id.IDS_DETAILS_DURATION_MINS_FORMAT).format(minutesString)
                    }
                    return timeString
                }, getTotalMinutesFromDate: function getTotalMinutesFromDate(date) {
                    var minutes = 0;
                    if (date)
                        minutes = date.getHours() * 60 + date.getMinutes();
                    return minutes
                }, millisecondsFromTimeSpan: function millisecondsFromTimeSpan(timeSpan) {
                    var milliSeconds = 0;
                    if (timeSpan && timeSpan instanceof Date) {
                        var zero = new Date(0, 0, 0, 0, 0, 0, 0);
                        milliSeconds = timeSpan.getTime() - zero.getTime()
                    }
                    return milliSeconds
                }, secondsFromTimeSpan: function secondsFromTimeSpan(timeSpan) {
                    var seconds = MS.Entertainment.Utilities.millisecondsFromTimeSpan(timeSpan) / 1000;
                    return seconds
                }, ensureDurationIsNumber: function ensureDurationIsNumber(duration) {
                    var ms = 0;
                    if (typeof duration === "number")
                        ms = duration;
                    else if (duration.getHours) {
                        ms += duration.getHours() * 60 * 60 * 1000;
                        ms += duration.getMinutes() * 60 * 1000;
                        ms += duration.getSeconds() * 1000;
                        ms += duration.getMilliseconds()
                    }
                    else
                        MS.Entertainment.Utilities.fail("Expected number or date");
                    return ms
                }, handleSearchInput: function handleSearchInput(input) {
                    if (!input)
                        return null;
                    var result = String.empty;
                    var trimmed = input.trim();
                    if (trimmed)
                        for (var i = 0; i < trimmed.length; i++) {
                            var ch = trimmed[i];
                            switch (ch) {
                                case"%":
                                case";":
                                case"(":
                                case")":
                                case"{":
                                case"}":
                                case"!":
                                case"\r":
                                case"\t":
                                case"\n":
                                case">":
                                case"<":
                                case"+":
                                case"&":
                                case"-":
                                    result = result + " ";
                                    break;
                                default:
                                    result = result + ch;
                                    break
                            }
                        }
                    return result.trim()
                }, processServiceLinks: function processServiceLinks(text, linkMappings) {
                    var ignoreTags = ["<I>", "<i>", "<B>", "<b>", "</I>", "</i>", "</B>", "</b>"];
                    var isIgnorableTag = function isIgnoreableTag(tagStart, isClosingTag) {
                            var ignore = false;
                            for (var i = 0; i < ignoreTags.length; i++)
                                if (tagStart.lastIndexOf(ignoreTags[i], 0) === 0) {
                                    ignore = true;
                                    break
                                }
                            return ignore
                        };
                    var skipLinkClose = false;
                    return MS.Entertainment.Utilities.processServiceTextBlob(text, function handleTag(tagStart) {
                            tagStart = tagStart || String.empty;
                            if (isIgnorableTag(tagStart))
                                return String.empty;
                            var result = String.empty;
                            var match = /^\<link type\=\"([a-zA-Z0-9]+)\" id\=\"([a-zA-Z0-9\-]+)\">/.exec(tagStart);
                            if (match) {
                                if (match[1] && match[2]) {
                                    var stringFormat = linkMappings[match[1]];
                                    if (stringFormat)
                                        result = stringFormat.format(match[2])
                                }
                                skipLinkClose = (result === String.empty)
                            }
                            else if (tagStart.indexOf("</link>") === 0 && !skipLinkClose)
                                result = linkMappings["$close"];
                            return result
                        })
                }, processServiceTextBlob: function processServiceTextBlob(textBlob, tagCallback) {
                    var buff = "";
                    if (textBlob) {
                        var inTag = false;
                        for (var i = 0; i < textBlob.length; i++) {
                            var ch = textBlob[i];
                            if (ch === "<")
                                inTag = true;
                            if (!inTag)
                                if (ch === "\n")
                                    buff = buff + "<br><br>";
                                else if (ch === ">")
                                    buff = buff + "&gt;";
                                else
                                    buff = buff + ch;
                            else if (ch === "<" && tagCallback)
                                buff = buff + tagCallback(textBlob.substr(i));
                            if (ch === ">")
                                inTag = false
                        }
                    }
                    return buff
                }, getMediaTypeName: function getMediaTypeName(mediaType, videoType, gameType, downloadTypeText) {
                    var name;
                    switch (mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.video:
                            if (videoType === Microsoft.Entertainment.Queries.VideoType.tvEpisode)
                                name = String.load(String.id.IDS_MEDIATYPE_TVEPISODE);
                            else
                                name = String.load(String.id.IDS_MEDIATYPE_MOVIE);
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeries:
                            name = String.load(String.id.IDS_MEDIATYPE_TVSERIES);
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeason:
                            name = String.load(String.id.IDS_MEDIATYPE_TVSEASON);
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                            name = String.load(String.id.IDS_MEDIATYPE_ARTIST);
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                            name = String.load(String.id.IDS_MEDIATYPE_ALBUM);
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                            name = String.load(String.id.IDS_MEDIATYPE_SONG);
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.playlist:
                            name = String.load(String.id.IDS_MEDIATYPE_PLAYLIST);
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.game:
                            if (gameType === MS.Entertainment.Data.Augmenter.GamePlatform.Modern)
                                name = String.load(String.id.IDS_MEDIATYPE_GAME_WINDOWS);
                            else {
                                name = downloadTypeText;
                                if (!name)
                                    name = String.load(String.id.IDS_MEDIATYPE_GAME)
                            }
                            break;
                        default:
                            name = String.load(String.id.IDS_UNKNOWN_VALUE);
                            break
                    }
                    return name
                }, getMediaTypeNameFromMedia: function getMediaTypeNameFromMedia(media) {
                    return MS.Entertainment.Utilities.getMediaTypeName(media.mediaType, media.videoType, media.defaultPlatformType, media.downloadTypeText)
                }, isEDSAuthRequired: function isEDSAuthRequired() {
                    var isAuthRequired = false;
                    var edsUri = MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_EDSSearch);
                    isAuthRequired = edsUri.toLowerCase().substr(0, 8) === "https://";
                    return isAuthRequired
                }, isTouchDevicePresent: function isTouchDevicePresent() {
                    var touchCapabilities = new Windows.Devices.Input.TouchCapabilities;
                    return touchCapabilities && touchCapabilities.touchPresent
                }, EMPTY_GUID: "00000000-0000-0000-0000-000000000000", isEmptyGuid: function isEmptyGuid(guid) {
                    return !guid || guid === MS.Entertainment.Utilities.EMPTY_GUID
                }, isValidGuid: function isValidGuid(guid) {
                    return guid && typeof guid === "string" && guid.match(/^(\{){0,1}[0-9a-fA-F]{8}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{12}(\}){0,1}$/)
                }, trimGuid: function trimGuid(guid) {
                    var trimmed = guid.replace('{', '');
                    trimmed = trimmed.replace('}', '');
                    return trimmed
                }, isValidServiceId: function isValidServiceId(guid) {
                    return !MS.Entertainment.Utilities.isEmptyGuid(guid) && MS.Entertainment.Utilities.isValidGuid(guid)
                }, isValidLibraryId: function isValidLibraryId(id) {
                    return id > 0
                }, invalidateLibraryId: -1, _executeQueryAndGetMediaItemAsync: function _executeQueryAndGetMediaItemAsync(query) {
                    var media;
                    return query.getItems().then(function queryGetItems_success(virtualList) {
                            if (virtualList.count !== 1)
                                return WinJS.Promise.wrap();
                            return virtualList.itemsFromIndex(0, 0, 0)
                        }).then(function itemsFromIndex_complete(args) {
                            if (args) {
                                media = args.items[0].data;
                                MS.Entertainment.ViewModels.MediaItemModel.augment(media);
                                if (media.hydrate)
                                    return media.hydrate()
                            }
                            return WinJS.Promise.wrap()
                        }).then(function hydrate_complete() {
                            return WinJS.Promise.wrap(media)
                        }, function _executeQueryAndGetMediaItemAsync_error() {
                            return WinJS.Promise.wrap()
                        })
                }, createMediaItemByLibraryInfo: function createMediaItemByLibraryInfo(libraryId, mediaType) {
                    var query;
                    switch (mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.playlist:
                            return MS.Entertainment.UI.Actions.Playlists.queryPlaylistById(libraryId);
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                            var query = new MS.Entertainment.Data.Query.libraryTracks;
                            query.trackId = libraryId;
                            query.acquisitionData = new MS.Entertainment.Utilities.AcquisitionData(Microsoft.Entertainment.Platform.AcquisitionContext.collection, Microsoft.Entertainment.Platform.AcquisitionContextType.track);
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                            var query = new MS.Entertainment.Data.Query.libraryAlbums;
                            query.albumId = libraryId;
                            query.acquisitionData = new MS.Entertainment.Utilities.AcquisitionData(Microsoft.Entertainment.Platform.AcquisitionContext.collection, Microsoft.Entertainment.Platform.AcquisitionContextType.album);
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                            var query = new MS.Entertainment.Data.Query.libraryArtists;
                            query.personId = libraryId;
                            query.acquisitionData = new MS.Entertainment.Utilities.AcquisitionData(Microsoft.Entertainment.Platform.AcquisitionContext.collection, Microsoft.Entertainment.Platform.AcquisitionContextType.artist);
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.video:
                            var query = new MS.Entertainment.Data.Query.libraryVideos;
                            query.objectIds = [libraryId];
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeason:
                            var query = new MS.Entertainment.Data.Query.libraryTVSeasons;
                            query.tvSeasonId = libraryId;
                            break;
                        default:
                            return WinJS.Promise.wrap()
                    }
                    return MS.Entertainment.Utilities._executeQueryAndGetMediaItemAsync(query)
                }, convertToHexString: function convertToHexString(value) {
                    if (!value)
                        value = 0;
                    else if (value < 0)
                        value += 0xFFFFFFFF + 1;
                    return "0x" + value.toString(16)
                }, getSpotlightItemAugmentation: function getSpotlightItemAugmentation(spotlightType) {
                    var augmentation = null;
                    switch (spotlightType) {
                        case MS.Entertainment.Data.Augmenter.Spotlight.MediaType.Movie:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Video.SpotlightMovie;
                            break;
                        case MS.Entertainment.Data.Augmenter.Spotlight.MediaType.Artist:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Music.SpotlightArtist;
                            break;
                        case MS.Entertainment.Data.Augmenter.Spotlight.MediaType.Album:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Music.SpotlightAlbum;
                            break;
                        case MS.Entertainment.Data.Augmenter.Spotlight.MediaType.Series:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Video.SpotlightTVSeries;
                            break;
                        case MS.Entertainment.Data.Augmenter.Spotlight.MediaType.Season:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Video.SpotlightTVSeason;
                            break;
                        case MS.Entertainment.Data.Augmenter.Spotlight.MediaType.Episode:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Video.SpotlightTVEpisode;
                            break;
                        case MS.Entertainment.Data.Augmenter.Spotlight.MediaType.None:
                        case MS.Entertainment.Data.Augmenter.Spotlight.MediaType.Web:
                        case MS.Entertainment.Data.Augmenter.Spotlight.MediaType.WebBlend:
                        case MS.Entertainment.Data.Augmenter.Spotlight.MediaType.FlexHub:
                        case MS.Entertainment.Data.Augmenter.Spotlight.MediaType.ZuneFlexHub:
                            augmentation = MS.Entertainment.Data.Augmenter.Spotlight.SpotlightItem;
                            break;
                        default:
                            var shipAssertProvider = new Microsoft.Entertainment.Infrastructure.ShipAssertProvider;
                            shipAssertProvider.shipAssert("MS.Entertainment.UI.Controls", "getSpotlightItemAugmentation()", MS.Entertainment.UI.Debug.getStackTrace(), "Message: " + "Unsupported spotlight media type.  Could not find augmentation for type: " + spotlightType, "");
                            break
                    }
                    {};
                    return augmentation
                }, getEditorialItemAugmentation: function getEditorialItemAugmentation(editorialType) {
                    var augmentation = null;
                    switch (editorialType) {
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Movie:
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.MovieTrailer:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Video.EditorialMovie;
                            break;
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.FlexHub:
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Hub:
                            augmentation = MS.Entertainment.Utilities.isMusicApp2 ? MS.Entertainment.Data.Augmenter.Marketplace.EditorialMusicHub : MS.Entertainment.Data.Augmenter.Marketplace.EditorialHub;
                            break;
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Series:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Video.EditorialTVSeries;
                            break;
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Season:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Video.EditorialTVSeason;
                            break;
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Episode:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Video.EditorialEpisode;
                            break;
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Album:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Music.EditorialAlbum;
                            break;
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Artist:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Music.EditorialArtist;
                            break;
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Track:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Music.EditorialTrack;
                            break;
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.MusicVideo:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.Music.EditorialMusicVideo;
                            break;
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Playlist:
                            break;
                        default:
                            MS.Entertainment.UI.Controls.fail("Unsupported editorial media type.  Could not find augmentation for type: " + editorialType);
                            break
                    }
                    {};
                    return augmentation
                }, getPinnedItemAugmentation: function getPinnedItemAugmentation(pinnedType) {
                    var augmentation = null;
                    switch (pinnedType) {
                        case MS.Entertainment.Data.Query.edsMediaType.musicPlaylist:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.PinnedPlaylist;
                            break;
                        case MS.Entertainment.Data.Query.edsMediaType.album:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.PinnedAlbum;
                            break;
                        case MS.Entertainment.Data.Query.edsMediaType.musicArtist:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.PinnedArtist;
                            break;
                        case MS.Entertainment.Data.Query.edsMediaType.movie:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.PinnedMovie;
                            break;
                        case MS.Entertainment.Data.Query.edsMediaType.tvSeason:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.PinnedTvSeason;
                            break;
                        case MS.Entertainment.Data.Query.edsMediaType.tvSeries:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.PinnedTvSeries;
                            break;
                        default:
                            break
                    }
                    {};
                    return augmentation
                }, getRecentItemAugmentation: function getRecentItemAugmentation(recentType, acquisitionContext) {
                    var augmentation = null;
                    switch (recentType) {
                        case MS.Entertainment.Data.Query.edsMediaType.musicPlaylist:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.RecentPlaylist;
                            break;
                        case MS.Entertainment.Data.Query.edsMediaType.album:
                            augmentation = MS.Entertainment.Data.Augmenter.Marketplace.RecentAlbum;
                            break;
                        case MS.Entertainment.Data.Query.edsMediaType.musicArtist:
                            switch (acquisitionContext) {
                                case MS.Entertainment.Utilities.AcquisitionContext.SmartDJ:
                                    augmentation = MS.Entertainment.Data.Augmenter.Marketplace.RecentSmartDJ;
                                    break;
                                default:
                                    augmentation = MS.Entertainment.Data.Augmenter.Marketplace.RecentArtist;
                                    break
                            }
                            break;
                        default:
                            break
                    }
                    {};
                    return augmentation
                }, isSupportedVideoEditorialType: function isSupportedVideoEditorialType(item, editorialType) {
                    var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                    var showMovieTrailers = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.videoShowMovieTrailers);
                    var featuredSetsEnabled = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.videoFeaturedSetsEnabled);
                    var config = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                    var showEditorialEpisodes = config.video.supportsEditorialTVEpisodes;
                    switch (editorialType) {
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.MovieTrailer:
                            return showMovieTrailers;
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Episode:
                            return showEditorialEpisodes;
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Season:
                            return MS.Entertainment.Utilities.isValidServiceId(item.seriesId);
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.FlexHub:
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Hub:
                            return featuredSetsEnabled;
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Series:
                        case MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.Movie:
                            return true;
                        default:
                            break
                    }
                    return false
                }, convertEditorialItem: function convertEditorialItem(mediaItem) {
                    var newMediaItem = mediaItem;
                    var augmenter;
                    if (mediaItem && !mediaItem.mediaType)
                        augmenter = MS.Entertainment.Utilities.getEditorialItemAugmentation(mediaItem && mediaItem.type);
                    if (augmenter) {
                        var oldItem = WinJS.Binding.unwrap(mediaItem);
                        newMediaItem = MS.Entertainment.Data.augment(MS.Entertainment.Data.deflate(oldItem), augmenter);
                        if (oldItem.contentNotifications)
                            newMediaItem.contentNotifications = oldItem.contentNotifications
                    }
                    return newMediaItem
                }, uninitializeChildControls: function disposeChildControls(dom) {
                    var element = dom.querySelector("[data-win-control], .ent-control");
                    var controls = dom.querySelectorAll("[data-win-control], .ent-control");
                    if (controls && controls.length > 0)
                        for (var i = 0, l = controls.length; i < l; i++)
                            if (controls[i] && controls[i].winControl && controls[i].winControl.uninitialize)
                                controls[i].winControl.uninitialize()
                }, searchArray: function searchArray(array, predicate) {
                    var index = 0;
                    var candidate = null;
                    for (index = 0; index < array.length; index++) {
                        candidate = array[index];
                        if (predicate(candidate))
                            return candidate
                    }
                    return null
                }, getPropertyPathFragments: function getPropertyPathFragments(propertyPath) {
                    var propertyPathFragments = (propertyPath) ? propertyPath.split(/(?:\]\.)|\.|\[|\]/) : [];
                    if (propertyPathFragments[propertyPathFragments.length - 1] === String.empty)
                        propertyPathFragments.pop();
                    if (propertyPathFragments[0] === String.empty)
                        propertyPathFragments.splice(0, 1);
                    return propertyPathFragments
                }, valueFromPropertyPath: function valueFromPropertyPath(data, propertyPath) {
                    return MS.Entertainment.Utilities.valueFromPropertyPathFragments(data, MS.Entertainment.Utilities.getPropertyPathFragments(propertyPath))
                }, valueFromPropertyPathFragments: function valueFromPropertyPathFragments(data, propertyPath, max) {
                    var max = isNaN(max) ? propertyPath.length : max;
                    for (var i = 0; i < max && data; i++)
                        try {
                            data = data[propertyPath[i]]
                        }
                        catch(e) {
                            MS.Entertainment.fail("Exception in valueFromPropertyPathFragments: " + e.toString() + " Property: " + propertyPath.toString())
                        }
                    return data
                }, hasPropertyPath: function hasProperty(data, propertyPath) {
                    return MS.Entertainment.Utilities.hasPropertyPathFragments(data, MS.Entertainment.Utilities.getPropertyPathFragments(propertyPath))
                }, hasPropertyPathFragments: function hasPropertyPathFragments(data, propertyPath) {
                    var len = propertyPath.length - 1;
                    for (var i = 0; i < len && data; i++)
                        data = data[propertyPath[i]];
                    return data && (len < 0 || (typeof data === "object" && propertyPath[len] in data))
                }, setFromPropertyPath: function setFromPropertyPath(data, propertyPath, value) {
                    return MS.Entertainment.Utilities.setFromPropertyPathFragments(data, MS.Entertainment.Utilities.getPropertyPathFragments(propertyPath), value)
                }, setFromPropertyPathFragments: function setFromPropertyPathFragments(data, propertyPath, value) {
                    for (var i = 0, len = propertyPath.length - 1; i < len && data; i++) {
                        if (!(propertyPath[i] in data))
                            data[propertyPath[i]] = {};
                        data = data[propertyPath[i]]
                    }
                    data[propertyPath[len]] = value;
                    return data
                }, random: function random(min, max) {
                    return Math.floor(Math.random() * (max - min + 1)) + min
                }, getRelativeOffset: function getRelativeOffset(element, parent) {
                    MS.Entertainment.Utilities.assert(element, "No element provided");
                    MS.Entertainment.Utilities.assert(parent, "No parent provided");
                    if (!element || !parent)
                        return {
                                top: 0, left: 0
                            };
                    var left = 0;
                    var top = 0;
                    var e = element;
                    while (e && parent && (e.offsetParent !== parent.offsetParent)) {
                        left += e.offsetLeft;
                        top += e.offsetTop;
                        e = e.offsetParent;
                        if (!e)
                            return {
                                    left: left, top: top
                                }
                    }
                    left += (e.offsetLeft - parent.offsetLeft);
                    top += (e.offsetTop - parent.offsetTop);
                    return {
                            left: left, top: top
                        }
                }, onRootPage: function onRootPage() {
                    var currentPage = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).currentPage;
                    return currentPage.iaNode.moniker === MS.Entertainment.UI.Monikers.root
                }, isElementInViewportOfParent: function isElementInViewportOfParent(item, parent) {
                    return MS.Entertainment.Utilities.isElementInHorizontalViewportOfParent(item, parent) && MS.Entertainment.Utilities.isElementInVerticalViewportOfParent(item, parent)
                }, isElementInHorizontalViewportOfParent: function isElementInHorizontalViewportOfParent(item, parent, adjustmentFactor) {
                    adjustmentFactor = adjustmentFactor || 0;
                    var relativePosition = MS.Entertainment.Utilities.getRelativeOffset(item, parent);
                    if (relativePosition && item) {
                        var leftEdge = (relativePosition.left - adjustmentFactor) >= parent.scrollLeft;
                        var rightEdge = Math.min(parent.scrollWidth, relativePosition.left + item.offsetWidth + adjustmentFactor) <= (parent.offsetWidth + parent.scrollLeft);
                        return leftEdge && rightEdge
                    }
                    else
                        return false
                }, isElementInVerticalViewportOfParent: function isElementInVerticalViewportOfParent(item, parent) {
                    var relativePosition = MS.Entertainment.Utilities.getRelativeOffset(item, parent);
                    if (relativePosition) {
                        var topEdge = relativePosition.top >= (parent.scrollTop);
                        var bottomEdge = (relativePosition.top + item.offsetHeight) <= (parent.offsetHeight + parent.scrollTop);
                        return topEdge && bottomEdge
                    }
                    else
                        return false
                }, getEdgeWithinViewport: function getEdgeWithinViewport(desiredEdgePosition, size, viewportSize) {
                    var edgePosition = Math.min(desiredEdgePosition, viewportSize - size);
                    return Math.max(edgePosition, 0)
                }, parseVersionString: function parseVersionString(inputString) {
                    MS.Entertainment.Utilities.assert(inputString, "Empty input string");
                    var versionArray = inputString.split(".");
                    MS.Entertainment.Utilities.assert(versionArray.length === 4, "Version string should have four parts");
                    return {
                            major: parseInt(versionArray[0]), minor: parseInt(versionArray[1]), build: parseInt(versionArray[2]), revision: parseInt(versionArray[3])
                        }
                }, compareVersions: function compareVersions(ver1, ver2) {
                    var properties = ["major", "minor", "build", "revision"];
                    MS.Entertainment.Utilities.assert((ver1.major >= 0) && (ver1.minor >= 0) && (ver1.build >= 0) && (ver1.revision >= 0), " Input version1 is invalid");
                    MS.Entertainment.Utilities.assert((ver2.major >= 0) && (ver2.minor >= 0) && (ver2.build >= 0) && (ver2.revision >= 0), " Input version2 is invalid");
                    for (var i = 0; i < properties.length; i++)
                        if (ver1[properties[i]] < ver2[properties[i]])
                            return -1;
                        else if (ver1[properties[i]] > ver2[properties[i]])
                            return 1;
                    return 0
                }, getClientVersionString: function getClientVersionString() {
                    var ver = Windows.ApplicationModel.Package.current.id.version;
                    return [ver.major, ver.minor, ver.build, ver.revision].join(".")
                }, getVersionString: function getVersionString() {
                    return WinJS.xhr({url: "version.json"}).then(function(data) {
                            var versionInformation = JSON.parse(data.responseText);
                            return versionInformation.version
                        })
                }, navigateToUrl: function navigateToUrl(url, automationId) {
                    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                    var externalNavigationAction = actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.externalNavigate);
                    externalNavigationAction.parameter = url;
                    externalNavigationAction.automationId = automationId;
                    externalNavigationAction.execute()
                }, launchStoreUpdatePage: function launchStoreUpdatePage() {
                    var launchInfo = "ms-windows-store:Updates";
                    var appAction = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.launchApp);
                    appAction.automationId = MS.Entertainment.UI.AutomationIds.launchAppStoreUpgrade;
                    appAction.parameter = {
                        uri: launchInfo, appendSource: true, appendGamerTag: false
                    };
                    appAction.execute()
                }, doesElementSupportKeyboardInput: function doesElementSupportKeyboardInput(element) {
                    var result = false;
                    if (!element)
                        return result;
                    if (element.tagName === "TEXTAREA" || element.tagName === "SELECT")
                        result = true;
                    else if (element.tagName === "INPUT")
                        switch (element.getAttribute("type")) {
                            case"text":
                            case"password":
                            case"number":
                            case"email":
                            case"tel":
                            case"url":
                            case"search":
                                result = true;
                                break
                        }
                    return result
                }, suppressSubtreeKeyboardFocus: function suppressSubtreeKeyboardFocus(element) {
                    var suppressed = [];
                    if (element) {
                        var elements = element.querySelectorAll("*[tabindex]");
                        if (elements)
                            for (var i = 0; i < elements.length; i++) {
                                var element = elements[i];
                                var tabIndex = element.getAttribute("tabindex");
                                if (tabIndex && tabIndex !== -1) {
                                    suppressed.push({
                                        element: element, tabIndex: tabIndex
                                    });
                                    element.setAttribute("tabindex", -1)
                                }
                            }
                    }
                    return suppressed
                }, restoreSubtreeKeyboardFocus: function restoreSubtreeKeyboardFocus(suppressedList) {
                    if (suppressedList)
                        for (var i = 0; i < suppressedList.length; i++) {
                            var suppressedItem = suppressedList[i];
                            suppressedItem.element.setAttribute("tabindex", suppressedItem.tabIndex)
                        }
                }, isHighResolution: function isHighResolution() {
                    var screenHeight = MS.Entertainment.Utilities.getWindowHeight();
                    var isHighResolution = MS.Entertainment.Utilities.isVideoApp ? (screenHeight >= MS.Entertainment.Utilities.VIEW1_VIDEO_VERTICAL_HEIGHT_LIMIT) : (screenHeight >= MS.Entertainment.Utilities.VIEW1_VERTICAL_HEIGHT_LIMIT);
                    return isHighResolution
                }, getRowCountForResolution: function getRowCountForResolution() {
                    var screenHeight = MS.Entertainment.Utilities.getWindowHeight();
                    var standardResolutionRows = MS.Entertainment.Utilities.isVideoApp ? MS.Entertainment.Utilities.STANDARD_RESOLUTION_VIDEO_ROWS : MS.Entertainment.Utilities.STANDARD_RESOLUTION_ROWS;
                    var highResolutionRows = MS.Entertainment.Utilities.isVideoApp ? MS.Entertainment.Utilities.HIGH_RESOLUTION_VIDEO_ROWS : MS.Entertainment.Utilities.HIGH_RESOLUTION_ROWS;
                    var highResolutionHeight = MS.Entertainment.Utilities.isVideoApp ? MS.Entertainment.Utilities.VIEW1_VIDEO_VERTICAL_HEIGHT_LIMIT : MS.Entertainment.Utilities.VIEW1_VERTICAL_HEIGHT_LIMIT;
                    var rowCount = standardResolutionRows;
                    if (screenHeight >= highResolutionHeight)
                        rowCount = highResolutionRows;
                    return rowCount
                }, getLandscapeScreenSize: function getLandscapeScreenSize() {
                    var size = {
                            width: 0, height: 0
                        };
                    if (window.screen)
                        if (window.screen.msOrientation === "portrait-primary" || window.screen.msOrientation === "portrait-secondary") {
                            size.width = window.screen.height;
                            size.height = window.screen.width
                        }
                        else {
                            size.width = window.screen.width;
                            size.height = window.screen.height
                        }
                    return size
                }, getWindowHeight: function getWindowHeight() {
                    var heightProperty = "outerHeight";
                    if (!document.attachEvent)
                        heightProperty = "innerHeight";
                    return window[heightProperty]
                }, getWindowWidth: function getWindowWidth() {
                    var widthProperty = "outerWidth";
                    if (!document.attachEvent)
                        widthProperty = "innerWidth";
                    return window[widthProperty]
                }, getDisplayProperties: function getDisplayProperties() {
                    var hasDisplayInformation = MS.Entertainment.Utilities.valueFromPropertyPathFragments(Windows, ["Graphics", "Display", "DisplayInformation"]);
                    if (hasDisplayInformation)
                        return Windows.Graphics.Display.DisplayInformation.getForCurrentView();
                    return Windows.Graphics.Display.DisplayProperties
                }, isMusicApp1: {get: function getIsMusicApp1() {
                        return MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.music
                    }}, isMusicApp2: {get: function getIsMusicApp2() {
                        return MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.music2
                    }}, isMusicApp: {get: function getIsMusicApp() {
                        return MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.music || MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.music2
                    }}, isVideoApp1: {get: function getIsVideoApp1() {
                        return MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.video
                    }}, isVideoApp2: {get: function getIsVideoApp2() {
                        return MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.video2
                    }}, isVideoApp: {get: function getIsVideoApp() {
                        return MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.video || MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.video2
                    }}, isNYCApp: {get: function getIsNYCApp() {
                        return MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.nyc
                    }}, isTestApp: {get: function getIsTestApp() {
                        return MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.test
                    }}, isApp1: {get: function getIsApp1() {
                        return MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.music || MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.video || MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.games || MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.companion
                    }}, isApp2: {get: function getIsApp2() {
                        return MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.music2 || MS.Entertainment.appMode === Microsoft.Entertainment.Application.AppMode.video2
                    }}, isWindowsBlue: {get: function getIsWindowsBlue() {
                        return !!Windows.UI.ViewManagement.ViewSizePreference
                    }}, isMain: {get: function getIsMain() {
                        var mainString = "main.html";
                        var href = window.location.href;
                        return href.indexOf(mainString, href.length - mainString.length) >= 0
                    }}, titleId: {get: function getTitleId() {
                        if (MS.Entertainment.Utilities.isVideoApp2)
                            return "0x3D705025";
                        else if (MS.Entertainment.Utilities.isMusicApp2)
                            return "0x18FFC9F4";
                        else
                            return null
                    }}, shortTitleId: {get: function getTitleId() {
                        if (MS.Entertainment.Utilities.isVideoApp2)
                            return "3D705025";
                        else if (MS.Entertainment.Utilities.isMusicApp2)
                            return "18FFC9F4";
                        else
                            return null
                    }}, supportsProtectedContent: {get: function getSupportsProtectedContent() {
                        try {
                            return Microsoft.Media.PlayReadyClient.PlayReadyStatics.playReadyCertificateSecurityLevel === 2000
                        }
                        catch(e) {
                            return false
                        }
                    }}, _isDrmIndividualized: false, isDrmIndividualized: {
                    get: function get_isDrmIndividualized() {
                        return MS.Entertainment.Utilities._isDrmIndividualized
                    }, set: function set_isDrmIndividualized(newValue) {
                            MS.Entertainment.Utilities._isDrmIndividualized = newValue
                        }
                }, clientSupportsFreeJustPlays: {get: function getClientSupportsFreeJustPlays() {
                        var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                        return configurationManager.service.enableFreeJustPlays
                    }}, showCastAndCrewDetails: {get: function getShowCastAndCrewDetails() {
                        var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                        return configurationManager.video.showCastAndCrewDetails
                    }}, isVideoActorDetailsEnabled: {get: function getIsVideoActorDetailsEnabled() {
                        var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                        return featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.videoActorDetails)
                    }}, isRightFree: function isRightFree(right) {
                    var retVal = false;
                    if (right.acceptablePaymentInstrumentTypes)
                        for (var i = 0; i < right.acceptablePaymentInstrumentTypes.length; i++)
                            if (right.acceptablePaymentInstrumentTypes[i] === MS.Entertainment.Data.Augmenter.Marketplace.edsAcceptedPaymentTypes.none)
                                retVal = true;
                    return retVal
                }, useModalNowPlaying: {get: function getUseModalNowPlaying() {
                        return MS.Entertainment.Utilities.isVideoApp
                    }}, noMarketplaceRootClassName: {get: function noMarketplaceRootClassName() {
                        return "noMarketplace"
                    }}, raisePanelReadyTest: function raisePanelReadyTest(element, failed, model) {
                    if (!element)
                        return;
                    var domEvent = document.createEvent("Event");
                    domEvent.initEvent("PanelReadyTest", true, true);
                    domEvent.failed = failed;
                    domEvent.model = model;
                    element.dispatchEvent(domEvent)
                }, raisePanelReady: function raisePanelReady(element, failed, model) {
                    if (!element)
                        return;
                    var domEvent = document.createEvent("Event");
                    domEvent.initEvent("PanelReady", true, true);
                    domEvent.failed = failed;
                    domEvent.model = model;
                    element.dispatchEvent(domEvent)
                }, raiseDomEvent: function raiseDomEvent(element, eventName) {
                    if (!element || !eventName)
                        return;
                    var domEvent = document.createEvent("Event");
                    domEvent.initEvent(eventName, true, true);
                    element.dispatchEvent(domEvent)
                }, raisePanelReset: function raisePanelReset(element) {
                    var domEvent = document.createEvent("Event");
                    domEvent.initEvent("PanelReset", true, true);
                    element.dispatchEvent(domEvent)
                }, resetPerfTrackStartTime: function resetPerfTrackStartTime() {
                    window.PerfTrack = {LaunchInformation: {getStartTime: (function _perfTrackLaunchInformation_getStartTimeClosure() {
                                var startTime = new Date;
                                return function _perfTrackLaunchInformation_getStartTimeWorker() {
                                        return startTime
                                    }
                            })()}}
                }, cloneActivationArguments: function cloneActivationArguments(args) {
                    var copy = {};
                    for (var name in args)
                        switch (name) {
                            case"activatedOperation":
                            case"prelaunchActivated":
                            case"splashScreen":
                            case"detail":
                            case"linguisticDetails":
                                continue;
                            default:
                                copy[name] = args[name];
                                break
                        }
                    return copy
                }, getDirectionFromGlobalKeyInput: function getDirectionFromGlobalKeyInput(e) {
                    MS.Entertainment.Utilities.assert(e, "Need an event to decide which direction to go in");
                    if (!e)
                        return;
                    var direction;
                    switch (e.keyCode) {
                        case WinJS.Utilities.Key.rGlobal:
                            direction = MS.Entertainment.Utilities.GlobalKeyDirection.right;
                            break;
                        case WinJS.Utilities.Key.lGlobal:
                            direction = MS.Entertainment.Utilities.GlobalKeyDirection.left;
                            break;
                        case WinJS.Utilities.Key.pageUp:
                            if (e.altKey)
                                direction = MS.Entertainment.Utilities.GlobalKeyDirection.right;
                            break;
                        case WinJS.Utilities.Key.pageDown:
                            if (e.altKey)
                                direction = MS.Entertainment.Utilities.GlobalKeyDirection.left;
                            break
                    }
                    return direction
                }, GlobalKeyDirection: {
                    left: "left", right: "right"
                }, isInvocationEvent: function isInvocationEvent(eventArgs) {
                    var keyCode = eventArgs && eventArgs.keyCode;
                    if (!keyCode)
                        return true;
                    var isInvocationEvent = false;
                    switch (keyCode) {
                        case WinJS.Utilities.Key.enter:
                        case WinJS.Utilities.Key.space:
                        case WinJS.Utilities.Key.invokeButton:
                            isInvocationEvent = true;
                            break
                    }
                    return isInvocationEvent
                }, defaultClientTypeFromApp: {get: function defaultClientTypeFromApp() {
                        var tuner = MS.Entertainment.Data.Augmenter.Marketplace.Tuner.Windows;
                        if (MS.Entertainment.Utilities.isApp2 && !MS.Entertainment.isAppModeOverride)
                            tuner = MS.Entertainment.Data.Augmenter.Marketplace.Tuner.XboxOne;
                        return tuner
                    }}, _defaultEdsTargetDeviceTypeFromApp: null, defaultEdsTargetDeviceTypeFromApp: {get: function defaultEdsDeviceTypeFromApp() {
                        if (!MS.Entertainment.Utilities._defaultEdsTargetDeviceTypeFromApp) {
                            var configuration = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                            MS.Entertainment.Utilities._defaultEdsTargetDeviceTypeFromApp = configuration.service.defaultTargetDeviceType
                        }
                        return MS.Entertainment.Utilities._defaultEdsTargetDeviceTypeFromApp
                    }}, _defaultEdsHeaderDeviceTypeFromApp: null, defaultEdsHeaderDeviceTypeFromApp: {get: function defaultEdsDeviceTypeFromApp() {
                        if (!MS.Entertainment.Utilities._defaultEdsHeaderDeviceTypeFromApp) {
                            var configuration = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                            MS.Entertainment.Utilities._defaultEdsHeaderDeviceTypeFromApp = configuration.service.defaultHeaderDeviceType
                        }
                        return MS.Entertainment.Utilities._defaultEdsHeaderDeviceTypeFromApp
                    }}, rightSupportsTuner: function rightSupportsTuner(right, tuner) {
                    var foundTuner = false;
                    if (right.clientTypes && right.clientTypes.length > 0)
                        for (var k = 0; k < right.clientTypes.length; k++)
                            if (tuner === right.clientTypes[k] || right.clientTypes[k] === MS.Entertainment.Data.Augmenter.Marketplace.Tuner.All) {
                                foundTuner = true;
                                break
                            }
                    return foundTuner
                }, optionsAppTypeTabIndexHelper: {get: function optionsAppTypeTabIndexHelper() {
                        return MS.Entertainment.Utilities.isApp2 ? 0 : -1
                    }}, bindingAppTypeTabIndexHelper: WinJS.Utilities.markSupportedForProcessing(function bindingAppTypeTabIndexHelper(source, sourceProperties, destination, destinationProperties) {
                    destination.tabIndex = MS.Entertainment.Utilities.optionsAppTypeTabIndexHelper
                }), toggleClass: function toggleClass(element, className, classEnabled) {
                    if (classEnabled)
                        WinJS.Utilities.addClass(element, className);
                    else
                        WinJS.Utilities.removeClass(element, className)
                }, SNAPPED_MODE_WIDTH: 480, STANDARD_RESOLUTION_VIDEO_ROWS: 2, HIGH_RESOLUTION_VIDEO_ROWS: {get: function() {
                        if (MS.Entertainment.Utilities.isApp2)
                            return 2;
                        else
                            return 3
                    }}, STANDARD_RESOLUTION_ROWS: 3, HIGH_RESOLUTION_ROWS: {get: function() {
                        if (MS.Entertainment.Utilities.isApp2)
                            return 3;
                        else
                            return 4
                    }}, VIEW1_VERTICAL_HEIGHT_LIMIT: 900, VIEW1_VIDEO_VERTICAL_HEIGHT_LIMIT: 1050, popOverDefaultSize: {get: function() {
                        if (MS.Entertainment.Utilities.isVideoApp && !MS.Entertainment.Utilities.getRetailExperienceState())
                            return {
                                    width: 470, height: 550
                                };
                        else
                            return {
                                    width: 730, height: 530
                                }
                    }}, cardItemSizeWithMargin: {get: function() {
                        if (MS.Entertainment.Utilities.isApp2)
                            return {
                                    width: 476, height: 240
                                };
                        else
                            return {
                                    width: 300, height: 170
                                }
                    }}, tileImageSize: {get: function() {
                        return {
                                width: 316, height: 316
                            }
                    }}, immersiveBackgroundArtSize: {get: function() {
                        if (MS.Entertainment.Utilities.isApp2)
                            return {
                                    width: 1080, height: 1080
                                };
                        else
                            return {
                                    width: 1366, height: 768
                                }
                    }}, playlistPinImageSize: {get: function() {
                        return {
                                width: 240, height: 240
                            }
                    }}
        })
    })();
    WinJS.Namespace.define("MS.Entertainment.Utilities", {FlipViewUnloadHelper: MS.Entertainment.UI.Framework.defineUserControl(null, function FlipViewUnloadHelper(element) {
            if (element && element.parentElement)
                element.parentElement.unloadTimeout = 3000
        })});
    WinJS.Namespace.define("MS.Entertainment.Utilities", {User: MS.Entertainment.UI.Framework.derive("MS.Entertainment.UI.Framework.ObservableBase", function user_constructor(userXuidOrNativeUser, gamerTag) {
            if (isNaN(userXuidOrNativeUser))
                this.nativeUserModel = [userXuidOrNativeUser || null];
            else
                this.nativeUserModel = this._createNativeUserModel(userXuidOrNativeUser, gamerTag)
        }, {
            nativeUserModel: null, getIsValid: function getIsValid() {
                    return Array.isArray(this.nativeUserModel) && this.nativeUserModel[0]
                }, _createNativeUserModel: function _createNativeUserModel(userXuid, gamerTag) {
                    return [MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.xboxLive).createUser(userXuid, gamerTag)]
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.Utilities.BindingAgnostic", {
        setProperty: function setProperty(target, property, value) {
            if (!(property in target) && target._backingData)
                target.addProperty(property, value);
            else
                target[property] = value
        }, setProperties: function setProperties(target, source) {
                var key;
                if (source)
                    for (key in source)
                        if (key.indexOf("_") !== 0 && key !== "instance" && key !== "constructor")
                            MS.Entertainment.Utilities.BindingAgnostic.setProperty(target, key, source[key])
            }
    });
    WinJS.Namespace.define("MS.Entertainment.Utilities", {
        showElementNoAnimation: WinJS.Utilities.markSupportedForProcessing(function showElementNoAnimation(element) {
            WinJS.Utilities.removeClass(element, "hideFromDisplay")
        }), hideElementNoAnimation: WinJS.Utilities.markSupportedForProcessing(function hideElementNoAnimation(element) {
                WinJS.Utilities.addClass(element, "hideFromDisplay")
            }), showElement: function showElement(element, timeoutMS, overrideOldSelector, overrideNewSelector) {
                if (!element)
                    return WinJS.Promise.as();
                var result;
                element.hideFromDisplay = false;
                WinJS.Utilities.removeClass(element, "hideFromDisplay");
                result = MS.Entertainment.UI.Framework.beginShowAnimations(element, overrideOldSelector, overrideNewSelector);
                return WinJS.Promise.as(result)
            }, hideElement: function hideElement(element, timeoutMS, overrideOldSelector, overrideNewSelector) {
                var hidePromise = null;
                var timeoutPromise = null;
                if (!element)
                    return WinJS.Promise.wrap();
                element.hideFromDisplay = true;
                if (timeoutMS)
                    timeoutPromise = WinJS.Promise.timeout(timeoutMS).then(function cancelHide() {
                        if (hidePromise) {
                            hidePromise.cancel();
                            hidePromise = null
                        }
                    });
                hidePromise = MS.Entertainment.UI.Framework.beginHideAnimations(element, overrideOldSelector, overrideNewSelector).then(function hideAnimationsComplete() {
                    if (timeoutPromise) {
                        timeoutPromise.cancel();
                        timeoutPromise = null
                    }
                    if (element.hideFromDisplay)
                        WinJS.Utilities.addClass(element, "hideFromDisplay")
                });
                return hidePromise
            }, displayElement: function displayElement(element, timeoutMS, overrideOldSelector, overrideNewSelector) {
                if (!element)
                    return WinJS.Promise.as();
                element.removeFromDisplay = false;
                WinJS.Utilities.removeClass(element, "removeFromDisplay");
                return MS.Entertainment.UI.Framework.beginShowAnimations(element, overrideOldSelector, overrideNewSelector)
            }, collapseElement: function collapseElement(element, timeoutMS, overrideOldSelector, overrideNewSelector) {
                var removePromise = null;
                var timeoutPromise = null;
                if (!element)
                    return WinJS.Promise.wrap();
                element.removeFromDisplay = true;
                if (timeoutMS)
                    timeoutPromise = WinJS.Promise.timeout(timeoutMS).then(function cancelHide() {
                        if (removePromise) {
                            removePromise.cancel();
                            removePromise = null
                        }
                    });
                removePromise = MS.Entertainment.UI.Framework.beginHideAnimations(element, overrideOldSelector, overrideNewSelector).then(function hideAnimationsComplete() {
                    if (timeoutPromise) {
                        timeoutPromise.cancel();
                        timeoutPromise = null
                    }
                    if (element.removeFromDisplay)
                        WinJS.Utilities.addClass(element, "removeFromDisplay")
                });
                return removePromise
            }, toggleDisplayCollapseElement: function toggleDisplayCollapseElement(element, makeVisible) {
                var timeout = 500;
                var domElement = element;
                if (element && element.domElement)
                    domElement = element.domElement;
                if (!domElement)
                    return WinJS.Promise.as();
                if (makeVisible)
                    return MS.Entertainment.Utilities.displayElement(domElement);
                else
                    return MS.Entertainment.Utilities.collapseElement(domElement, timeout).then(null, function onTimeout() {
                            WinJS.Utilities.addClass(domElement, "removeFromDisplay")
                        })
            }, enterElement: function enterElement(element) {
                if (!element)
                    return WinJS.Promise.as();
                var result;
                element.hideFromDisplay = false;
                WinJS.Utilities.removeClass(element, "hideFromDisplay");
                result = MS.Entertainment.UI.Framework.beginEnterAnimations(element);
                return WinJS.Promise.as(result)
            }, exitElement: function exitElement(element) {
                if (!element)
                    return WinJS.Promise.as();
                element.hideFromDisplay = true;
                return MS.Entertainment.UI.Framework.beginExitAnimations(element).then(function hideAnimationsComplete() {
                        if (element.hideFromDisplay)
                            WinJS.Utilities.addClass(element, "hideFromDisplay")
                    })
            }, appendChild: function appendChild(parent, child, hidden) {
                parent.appendChild(child);
                if (!hidden)
                    return MS.Entertainment.Utilities.showElement(child);
                else {
                    WinJS.Utilities.addClass(child, "hideFromDisplay");
                    return WinJS.Promise.wrap()
                }
            }, insertBefore: function insertBefore(parent, newChild, refChild, hidden) {
                parent.insertBefore(newChild, refChild);
                if (!hidden)
                    return MS.Entertainment.Utilities.showElement(newChild);
                else {
                    WinJS.Utilities.addClass(newChild, "hideFromDisplay");
                    return WinJS.Promise.wrap()
                }
            }, removeChild: function removeChild(parent, child, skipHideElement) {
                var remove = function remove() {
                        if (child.parentElement) {
                            MS.Entertainment.UI.Framework.assert(child.parentElement === parent, "Supplied parent was not childs direct parent");
                            child.parentElement.removeChild(child)
                        }
                    };
                if (skipHideElement) {
                    remove();
                    return WinJS.Promise.wrap()
                }
                else
                    return MS.Entertainment.Utilities.hideElement(child).then(function hideElementCompletes() {
                            remove()
                        })
            }, replaceChild: function replaceChild(parent, newChild, oldChild, hidden) {
                return MS.Entertainment.UI.Framework.beginHideAnimations(oldChild).then(function beginHideAnimationsCompletes() {
                        parent.replaceChild(newChild, oldChild);
                        if (!hidden)
                            MS.Entertainment.Utilities.showElement(newChild);
                        else {
                            WinJS.Utilities.addClass(newChild, "hideFromDisplay");
                            return WinJS.Promise.wrap()
                        }
                    })
            }, isElementVisible: function isElementVisible(element) {
                var style = window.getComputedStyle(element);
                return !(style.display === "none" || style.visibility === "hidden" || style.opacity === 0.0)
            }, safeAddClass: function safeAddClass(element, className) {
                if (element)
                    WinJS.Utilities.addClass(element, className)
            }, safeRemoveClass: function safeRemoveClass(element, className) {
                if (element)
                    WinJS.Utilities.removeClass(element, className)
            }, addEvents: function addEvents(element, events, capture) {
                element = (element && element.domElement) ? element.domElement : element;
                return MS.Entertainment.Utilities.addEventHandlers(element, events, capture)
            }, waitForStartedTransitionsToComplete: MS.Entertainment.UI.Framework.waitForStartedTransitionsToComplete, redirectPromise: function redirectPromise(promise, value) {
                return WinJS.Promise.as(promise).then(function redirectPromise() {
                        return value
                    })
            }, _schedulePromise: function _schedulePromise(promiseFunction, promiseValue, jobName) {
                var result;
                if (WinJS.Utilities.Scheduler)
                    result = WinJS.Utilities.Scheduler[promiseFunction](promiseValue, jobName);
                else {
                    jobName = jobName || String.empty;
                    msWriteProfilerMark("ent:_schedulePromise:" + promiseFunction + ":" + jobName + ",StartTM");
                    result = WinJS.Promise.timeout(1).then(function() {
                        msWriteProfilerMark("ent:_schedulePromise:" + promiseFunction + ":" + jobName + ",StopTM");
                        return promiseValue
                    })
                }
                return result
            }, schedulePromiseHigh: function schedulePromiseHigh(promiseValue, jobName) {
                return MS.Entertainment.Utilities._schedulePromise("schedulePromiseHigh", promiseValue, jobName)
            }, schedulePromiseAboveNormal: function schedulePromiseAboveNormal(promiseValue, jobName) {
                return MS.Entertainment.Utilities._schedulePromise("schedulePromiseAboveNormal", promiseValue, jobName)
            }, schedulePromiseNormal: function schedulePromiseNormal(promiseValue, jobName) {
                return MS.Entertainment.Utilities._schedulePromise("schedulePromiseNormal", promiseValue, jobName)
            }, schedulePromiseBelowNormal: function schedulePromiseNormal(promiseValue, jobName) {
                return MS.Entertainment.Utilities._schedulePromise("schedulePromiseBelowNormal", promiseValue, jobName)
            }, schedulePromiseIdle: function schedulePromiseIdle(promiseValue, jobName) {
                return MS.Entertainment.Utilities._schedulePromise("schedulePromiseIdle", promiseValue, jobName)
            }, setAccessibilityTextFromStringId: function setAccessibilityTextFromStringId(element, stringId) {
                MS.Entertainment.Utilities.setAccessibilityText(element, String.load(stringId))
            }, setAccessibilityText: function setAccessibilityText(element, text) {
                element = (element && element.domElement) ? element.domElement : element;
                if (element)
                    if (text)
                        element.setAttribute("aria-label", text);
                    else
                        element.removeAttribute("aria-label")
            }, setAccessibilityTextFromElement: function setAccessibilityTextFromElement(destinationElement, sourceElement) {
                destinationElement = (destinationElement && destinationElement.domElement) ? destinationElement.domElement : destinationElement;
                sourceElement = (sourceElement && sourceElement.domElement) ? sourceElement.domElement : sourceElement;
                if (destinationElement) {
                    MS.Entertainment.Utilities.assert(!sourceElement || sourceElement.id, "A source element was provided, but the element did not have an id. For aria-labelledby to work an id is required.");
                    if (sourceElement && sourceElement.id)
                        destinationElement.setAttribute("aria-labelledby", sourceElement.id);
                    else
                        destinationElement.removeAttribute("aria-labelledby")
                }
            }, copyAugmentedProperties: function copyAugmentedProperties(from, to) {
                var originalFrom = from;
                var property;
                var propertyDescriptor;
                function copyAugmentedPropertiesEx(from, to) {
                    if (from && to) {
                        from = MS.Entertainment.Data.augmentationShape(WinJS.Binding.unwrap(from));
                        Object.getOwnPropertyNames(from).forEach(function(property) {
                            if (property.indexOf("_") !== 0 && property !== "instance")
                                MS.Entertainment.Utilities.BindingAgnostic.setProperty(to, property, originalFrom[property])
                        });
                        to.hydrated = originalFrom.hydrated
                    }
                }
                copyAugmentedPropertiesEx(from, to)
            }, uniteObjects: function uniteObjects(lessSignificant, moreSignificant) {
                if (lessSignificant && moreSignificant) {
                    var combinedItem = {};
                    for (var property in lessSignificant)
                        combinedItem[property] = lessSignificant[property];
                    for (property in moreSignificant)
                        combinedItem[property] = moreSignificant[property];
                    return combinedItem
                }
                else if (lessSignificant)
                    return lessSignificant;
                else if (moreSignificant)
                    return moreSignificant;
                else
                    return {}
            }, doNothing: WinJS.Utilities.markSupportedForProcessing(function doNothing(){}), getAccountLocale: function getAccountLocale() {
                var locale;
                var signedInUser = null;
                if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.signedInUser))
                    signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                if (signedInUser && signedInUser.locale)
                    locale = signedInUser.locale;
                else {
                    var config = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                    if (config.service.lastSignedInUserAccountLocale)
                        locale = config.service.lastSignedInUserAccountLocale;
                    else
                        locale = MS.Entertainment.Utilities.getResourceLanguage()
                }
                return locale
            }, appRegionMatchesMachineRegion: function doesAccountRegionMatchMachineRegion() {
                var appRegion = (new Microsoft.Entertainment.Util.GlobalizationManager).getRegion();
                var machineRegion = (new Windows.Globalization.GeographicRegion).codeTwoLetter;
                return (appRegion === machineRegion)
            }, getResourceLanguage: function getResourceLanguage() {
                var resourceLanguage;
                try {
                    var resourceContext = new Windows.ApplicationModel.Resources.Core.ResourceContext;
                    resourceLanguage = resourceContext.languages[0]
                }
                catch(e) {
                    MS.Entertainment.UI.Components.Shell.assert(false, "Unexpected error retrieving resource locale")
                }
                if (!resourceLanguage)
                    resourceLanguage = "en-US";
                return resourceLanguage
            }, getDisplayLanguageFromLanguageCode: function getDisplayLanguageFromLanguageCode(languageCode) {
                var index = languageCode.lastIndexOf("-");
                var language = (index > 0) ? languageCode.substr(0, index) : languageCode;
                var languageObject = Windows.Globalization.Language(language);
                return languageObject.displayName ? languageObject.displayName : String.empty
            }, getValueFromCsvList: function getValueFromCsvList(nameValuePairList, name) {
                var value;
                var regularExpression = "(^|,)\\s*" + name + "\\s*=\\s*([^=,\\s]+)";
                var result = new RegExp(regularExpression).exec(nameValuePairList);
                if (result)
                    value = result[2];
                return value
            }, getRegionCodeFromLocale: function getRegionCodeFromLocale(localeName) {
                var regionCode = String.empty;
                var lastDashIndex = localeName.lastIndexOf("-");
                if (lastDashIndex >= 0)
                    regionCode = localeName.slice(lastDashIndex + 1);
                return regionCode.toUpperCase()
            }, getLanguageCodeFromLocale: function getLanguageCodeFromLocale(localeName) {
                var languageCode;
                var lastDashIndex = localeName.lastIndexOf("-");
                if (lastDashIndex >= 0)
                    languageCode = localeName.slice(0, lastDashIndex);
                else
                    languageCode = localeName;
                return languageCode
            }, instantiateControl: function instantiateControl(control, options, host) {
                options = options || {};
                var UserControlConstructor = control;
                var controlAttribute;
                if (typeof UserControlConstructor === "string") {
                    UserControlConstructor = WinJS.Utilities.getMember(control);
                    controlAttribute = control
                }
                else
                    controlAttribute = "MS.Entertainment.UI.Framework.UserControl";
                if (!options.hasOwnProperty("_skipDefer"))
                    options._skipDefer = true;
                MS.Entertainment.UI.Controls.assert(typeof UserControlConstructor === "function", "Overlay: " + control + " is not a function");
                host.setAttribute("data-win-control", controlAttribute);
                return new UserControlConstructor(host, options)
            }
    });
    WinJS.Namespace.define("MS.Entertainment.Utilities.DateFormatters", {
        formatDay: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatDay(sourceValue) {
            if (sourceValue)
                return sourceValue.getDate();
            else
                return String.empty
        })), formatMonthAbbreviation: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatMonthAbbreviation(sourceValue) {
                if (sourceValue) {
                    var superShortMonthPattern = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).abbreviatedMonth;
                    return superShortMonthPattern.format(sourceValue)
                }
                else
                    return String.empty
            })), formatMonthAbbreviationYear: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatMonthAbbreviationYear(sourceValue) {
                if (sourceValue) {
                    var superShortMonthPattern = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).abbreviatedMonthYear;
                    return superShortMonthPattern.format(sourceValue)
                }
                else
                    return String.empty
            })), formatYearMonthDayLong: WinJS.Binding.converter(WinJS.Utilities.markSupportedForProcessing(function formatYearMonthDayLong(sourceValue) {
                if (sourceValue) {
                    var longdate = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).dayMonthYear;
                    return longdate.format(sourceValue)
                }
                else
                    return String.empty
            }))
    });
    WinJS.Namespace.define("MS.Entertainment.globalControls", {
        GlobalControl: {
            appBar: "BottomAppBar", appInfoNotification: "AppInfoNotification", appCriticalNotification: "AppCriticalNotification", bottomProgressBar: "BottomProgressBar", backButton: "BackButton", backButtonBling: "BackButtonBling", immersiveDetails: "ImmersiveDetails", mainHeader: "MainHeader", navigationWrapper: "NavigationWrapper", xboxControls: "xboxControls", musicTransportControls: "MusicTransportControls", profilePicker: "ProfilePicker", profilePickerContainer: "ProfilePickerContainer"
        }, getControl: function getControl(identifier) {
                return MS.Entertainment.Utilities.getChildControl(document.body, identifier)
            }
    });
    (function() {
        var currentOverlayContainers = [];
        function sortOnTabIndex(a, b) {
            if (a.tabIndex < b.tabIndex) {
                if (a.tabIndex === -1)
                    return 1;
                return -1
            }
            else if (a.tabIndex > b.tabIndex)
                return 1;
            else
                return 0
        }
        function excludeNonFocusableElements(element) {
            if (!element.getAttribute)
                return false;
            var disabledAttribute = element.getAttribute("disabled");
            var currentStyle = element.currentStyle;
            var isVisible = currentStyle && currentStyle.visibility !== "hidden" && currentStyle.display !== "none" && disabledAttribute !== "disabled" && disabledAttribute !== "";
            if (!isVisible)
                return false;
            var tagName = element.tagName;
            var isFocusableElementType = /INPUT|SELECT|TEXTAREA|BUTTON|IFRAME/.test(tagName);
            var isFocusableLink = (tagName === "A" && element.href);
            var actuallyHasTabStop = false;
            var tabIndexAttribute = element.getAttribute("tabindex");
            if (tabIndexAttribute && !isNaN(tabIndexAttribute))
                actuallyHasTabStop = (tabIndexAttribute !== "-1");
            else if ((isFocusableElementType || isFocusableLink) && element.tabIndex > -1)
                actuallyHasTabStop = true;
            var isKeyboardNavigationManagerFocusable = WinJS.Utilities.hasClass(element, "win-focusable");
            return (isFocusableElementType && actuallyHasTabStop) || (isFocusableLink && actuallyHasTabStop) || actuallyHasTabStop || isKeyboardNavigationManagerFocusable
        }
        WinJS.Namespace.define("MS.Entertainment.UI.Framework", {
            TabConstrainer: MS.Entertainment.UI.Framework.define(function(element, options) {
                this.domElement = element;
                if (options)
                    WinJS.UI.setOptions(this, options);
                var firstDiv = document.createElement("div");
                firstDiv.className = MS.Entertainment.UI.Framework.TabConstrainer.firstDivClass;
                firstDiv.setAttribute("role", "menuitem");
                firstDiv.setAttribute("aria-hidden", "true");
                if (this.enabled)
                    firstDiv.setAttribute("tabindex", 0);
                firstDiv.addEventListener("focus", this._focusLandsOnFirstElement.bind(this), false);
                element.insertAdjacentElement("AfterBegin", firstDiv);
                var lastDiv = document.createElement("div");
                lastDiv.className = MS.Entertainment.UI.Framework.TabConstrainer.lastDivClass;
                lastDiv.setAttribute("role", "menuitem");
                lastDiv.setAttribute("aria-hidden", "true");
                if (this.enabled)
                    lastDiv.setAttribute("tabindex", this.assumeTabIndexIsZero ? 0 : 10000);
                lastDiv.addEventListener("focus", this._focusLandsOnLastElement.bind(this), false);
                element.appendChild(lastDiv);
                if (!this.assumeTabIndexIsZero)
                    this._fixUpLastDivTabIndexes()
            }, {
                assumeTabIndexIsZero: false, _enabled: true, enabled: {
                        get: function tabConstrainer_get() {
                            return this._enabled
                        }, set: function tabConstrainer_set(value) {
                                if (this._enabled === value)
                                    return;
                                this._enabled = value;
                                if (this.enabled) {
                                    this.domElement.firstElementChild.setAttribute("tabindex", 0);
                                    this.domElement.lastElementChild.setAttribute("tabindex", this.assumeTabIndexIsZero ? 0 : 10000);
                                    if (!this.assumeTabIndexIsZero)
                                        this._fixUpLastDivTabIndexes()
                                }
                                else {
                                    this.domElement.firstElementChild.removeAttribute("tabindex");
                                    this.domElement.lastElementChild.removeAttribute("tabindex")
                                }
                            }
                    }, _focusLandsOnLastElement: function _focusLandsOnLastElement() {
                        if (!this.enabled)
                            return;
                        if (!WinJS.Utilities.hasClass(document.activeElement, MS.Entertainment.UI.Framework.TabConstrainer.lastDivClass))
                            return;
                        var appBar = MS.Entertainment.globalControls.getControl(MS.Entertainment.globalControls.GlobalControl.appBar);
                        if (appBar && this.includeAppBarOnLastTab && appBar.visible)
                            appBar.focusAppBar(this.domElement);
                        else
                            MS.Entertainment.UI.Framework.focusFirstInSubTree(this.domElement, this.excludeEndPointElements)
                    }, _focusLandsOnFirstElement: function _focusLandsOnFirstElement() {
                        if (!this.enabled)
                            return;
                        if (document.activeElement)
                            if (!WinJS.Utilities.hasClass(document.activeElement, MS.Entertainment.UI.Framework.TabConstrainer.firstDivClass))
                                return;
                        if (this.domElement)
                            MS.Entertainment.UI.Framework.focusLastInSubTree(this.domElement)
                    }, _fixUpLastDivTabIndexes: function _fixUpLastDivTabIndexes() {
                        var otherLastItems = document.querySelectorAll("." + MS.Entertainment.UI.Framework.TabConstrainer.lastDivClass);
                        var nextTabIndex = 10001;
                        if (otherLastItems && otherLastItems.length > 0)
                            Array.prototype.forEach.call(otherLastItems, function(item) {
                                if (this.domElement.lastElementChild === item) {
                                    item.tabIndex = 10000;
                                    return
                                }
                                if (item.parentElement && item.parentElement.winControl && item.parentElement.winControl.enabled) {
                                    item.tabIndex = nextTabIndex;
                                    nextTabIndex = item.tabIndex + 10000
                                }
                            }.bind(this))
                    }, excludeEndPointElements: true, includeAppBarOnLastTab: false
            }, {
                firstDivClass: "ent-firstdiv", lastDivClass: "ent-lastdiv"
            }), focusLastInSubTree: function focusLastInSubTree(element) {
                    if (MS.Entertainment.UI.Framework.focusedItemInContainer(element))
                        return;
                    var children = element.getElementsByTagName("*");
                    var sorted = Array.prototype.filter.call(children, excludeNonFocusableElements).sort(sortOnTabIndex);
                    for (var i = (sorted.length - 1); i > -1; i--) {
                        var elementToFocus = sorted[i];
                        if (WinJS.Utilities.hasClass(elementToFocus, MS.Entertainment.UI.Framework.TabConstrainer.firstDivClass) || WinJS.Utilities.hasClass(elementToFocus, MS.Entertainment.UI.Framework.TabConstrainer.lastDivClass))
                            continue;
                        elementToFocus.focus();
                        if (elementToFocus === document.activeElement || MS.Entertainment.UI.Framework.focusedItemInContainer(element))
                            break
                    }
                }, focusFirstInSubTree: function focusFirstInSubTree(element, excludeEndpointElements, force) {
                    if (!MS.Entertainment.UI.Framework.canMoveFocus(element))
                        return false;
                    if (!force && MS.Entertainment.UI.Framework.focusedItemInContainer(element))
                        return true;
                    var successfullyFocused = false;
                    var firstDiv = null;
                    var children = element.getElementsByTagName("*");
                    var sorted = Array.prototype.filter.call(children, excludeNonFocusableElements).sort(sortOnTabIndex);
                    for (var i = 0; i < sorted.length; i++) {
                        var elementToFocus = sorted[i];
                        if (excludeEndpointElements)
                            if (WinJS.Utilities.hasClass(elementToFocus, MS.Entertainment.UI.Framework.TabConstrainer.firstDivClass)) {
                                firstDiv = elementToFocus;
                                continue
                            }
                            else if (WinJS.Utilities.hasClass(elementToFocus, MS.Entertainment.UI.Framework.TabConstrainer.lastDivClass))
                                continue;
                        elementToFocus.focus();
                        if (elementToFocus === document.activeElement || MS.Entertainment.UI.Framework.focusedItemInContainer(element)) {
                            successfullyFocused = true;
                            break
                        }
                    }
                    if (!successfullyFocused)
                        if (firstDiv) {
                            firstDiv.focus();
                            successfullyFocused = (firstDiv === document.activeElement)
                        }
                    return successfullyFocused
                }, tryAndFocusElementInSubTreeWithTimer: function tryAndFocusElementInSubTreeWithTimer(tree, timeout) {
                    MS.Entertainment.UI.Framework.assert(timeout > -1, "Invalid timeout provided to try and set focus");
                    MS.Entertainment.UI.Framework.assert(tree, "Need a valid tree to try and set focus");
                    if (timeout < 0)
                        return WinJS.Promise.as();
                    if (!tree)
                        return WinJS.Promise.as();
                    return WinJS.Promise.timeout(timeout).then(function trySetFocusOnTimeoutComplete() {
                            if (MS.Entertainment.UI.Framework.focusedItemInContainer(tree))
                                return;
                            MS.Entertainment.UI.Framework.focusFirstInSubTree(tree)
                        })
                }, focusedItemInContainer: function focusedItemInContainer(container) {
                    if (document.activeElement)
                        if (container.contains(document.activeElement) && !(WinJS.Utilities.hasClass(document.activeElement, MS.Entertainment.UI.Framework.TabConstrainer.firstDivClass) || WinJS.Utilities.hasClass(document.activeElement, MS.Entertainment.UI.Framework.TabConstrainer.lastDivClass)))
                            return true;
                    return false
                }, focusFirstFocusableAncestor: function focusFirstFocusableAncestor(element, skipScroll) {
                    var ancestor = element;
                    do {
                        if (ancestor.domElement)
                            ancestor = ancestor.domElement;
                        if (excludeNonFocusableElements(ancestor)) {
                            MS.Entertainment.UI.Framework.focusElement(ancestor, skipScroll);
                            break
                        }
                        ancestor = ancestor.parentNode
                    } while (ancestor)
                }, focusElement: function focusElement(elementToFocus, skipScroll, skipInputElementCheck) {
                    if (!MS.Entertainment.UI.Framework.canMoveFocus(elementToFocus, skipInputElementCheck))
                        return;
                    if (skipScroll)
                        try {
                            elementToFocus.setActive()
                        }
                        catch(error) {
                            MS.Entertainment.Utilities.fail("Failed to set active element. Classes: " + elementToFocus.className + ", Error: " + (error && error.message))
                        }
                    else
                        elementToFocus.focus()
                }, focusedElementHasClass: function focusedElementHasClass(classList) {
                    var hasClass = false;
                    if (document.activeElement && classList) {
                        if (!Array.isArray(classList))
                            classList = [classList];
                        for (var i = 0, l = classList.length; i < l; i++)
                            if (WinJS.Utilities.hasClass(document.activeElement, classList[i])) {
                                hasClass = true;
                                break
                            }
                    }
                    return hasClass
                }, currentContentContainer: null, currentOverlayContainer: {get: function currentOverlayContainer_get() {
                        var currentIndex = currentOverlayContainers.length - 1;
                        return currentOverlayContainers[currentIndex]
                    }}, currentFocusContainer: {get: function currentFocusContainer_get() {
                        var focusableOverlay = MS.Entertainment.UI.Framework.currentOverlayContainer;
                        if (focusableOverlay && focusableOverlay.currentStyle && focusableOverlay.currentStyle.display === "none")
                            focusableOverlay = null;
                        return focusableOverlay || MS.Entertainment.UI.Framework.currentContentContainer
                    }}, addOverlayContainer: function addOverlayContainer(container) {
                    if (MS.Entertainment.UI.Framework.currentOverlayContainer && MS.Entertainment.UI.Framework.currentOverlayContainer.setAttribute)
                        MS.Entertainment.UI.Framework.currentOverlayContainer.setAttribute("aria-hidden", "true");
                    currentOverlayContainers.push(container)
                }, removeOverlayContainer: function removeOverlayContainer(container) {
                    var indexOfItemToRemove = currentOverlayContainers.indexOf(container);
                    if (indexOfItemToRemove < 0)
                        return;
                    currentOverlayContainers.splice(indexOfItemToRemove, 1);
                    if (MS.Entertainment.UI.Framework.currentOverlayContainer && MS.Entertainment.UI.Framework.currentOverlayContainer.setAttribute)
                        MS.Entertainment.UI.Framework.currentOverlayContainer.setAttribute("aria-hidden", "false")
                }, canMoveFocus: function canMoveFocus(targetContainer, skipInputElementCheck) {
                    if (!targetContainer)
                        return false;
                    var focusContainer = MS.Entertainment.UI.Framework.currentFocusContainer;
                    var bottomAppBar = MS.Entertainment.globalControls.getControl(MS.Entertainment.globalControls.GlobalControl.appBar);
                    var targetingAppBar = false;
                    if (bottomAppBar && bottomAppBar.domElement)
                        targetingAppBar = bottomAppBar.domElement.contains(targetContainer);
                    if (focusContainer && !focusContainer.contains(targetContainer) && !targetingAppBar)
                        return false;
                    if (!skipInputElementCheck && MS.Entertainment.Utilities.isApp1 && document.activeElement && MS.Entertainment.Utilities.doesElementSupportKeyboardInput(document.activeElement))
                        return false;
                    return true
                }, setFocusRoot: function setFocusRoot(prospectiveFocusRoot) {
                    var currentContentContainer = MS.Entertainment.UI.Framework.currentContentContainer || document.querySelector(".pageContainer.currentPage");
                    if (!prospectiveFocusRoot || !MS.Entertainment.UI.Framework.checkIfInDom(prospectiveFocusRoot) || (currentContentContainer !== prospectiveFocusRoot) || !currentContentContainer.contains(prospectiveFocusRoot))
                        prospectiveFocusRoot = currentContentContainer;
                    var overlay = MS.Entertainment.UI.Framework.currentOverlayContainer;
                    if (overlay && !overlay.contains(prospectiveFocusRoot))
                        prospectiveFocusRoot = overlay;
                    MS.Entertainment.UI.Framework.assert(prospectiveFocusRoot, "Somehow, we ended up without a focus root to set when we expected one");
                    if (WinJS.UI.AutomaticFocus && prospectiveFocusRoot)
                        WinJS.UI.AutomaticFocus.focusRoot = prospectiveFocusRoot
                }, resetFocusToTopMostContent: function resetFocusToTopMostContent() {
                    var focusTarget = MS.Entertainment.UI.Framework.currentContentContainer;
                    if (MS.Entertainment.UI.Framework.currentOverlayContainer)
                        focusTarget = MS.Entertainment.UI.Framework.currentOverlayContainer;
                    var currentFocusedElementDetails = "No Focused element";
                    if (document.activeElement)
                        currentFocusedElementDetails = "ClassName: " + document.activeElement.className + ", " + "ID: " + document.activeElement.id;
                    MS.Entertainment.UI.Framework.fail("User manually reset focus. Focus was in this location: " + currentFocusedElementDetails);
                    MS.Entertainment.UI.Framework.focusFirstInSubTree(focusTarget)
                }, scrollViewerFocusHelper: function scrollViewerFocusHelper(scrollViewer) {
                    if (!scrollViewer)
                        return;
                    if (!WinJS.Utilities.hasClass(scrollViewer, "win-scrollviewer-scrollmode-list"))
                        return;
                    var scrollContainer = scrollViewer.querySelector(".win-scrollviewer-contentelement");
                    if (!scrollContainer)
                        return;
                    scrollContainer.addEventListener("focus", function(evt) {
                        if (evt.target)
                            MS.Entertainment.UI.Framework.focusFirstInSubTree(evt.target, true, true)
                    })
                }
        })
    })();
    WinJS.Namespace.define("MS.Entertainment.Utilities", {VirtualizedDataSource: MS.Entertainment.UI.Framework.derive(WinJS.UI.VirtualizedDataSource, function(listDataAdapter, options) {
            if (listDataAdapter && listDataAdapter.createListAdaptor)
                listDataAdapter = listDataAdapter.createListAdaptor();
            this._baseDataSourceConstructor(listDataAdapter, options)
        })});
    WinJS.Namespace.define("MS.Entertainment.Utilities", {
        getRetailExperienceState: function getRetailExperienceState() {
            return (new Microsoft.Entertainment.Configuration.ConfigurationManager).shell.retailExperience
        }, setRetailExperienceState: function setRetailExperienceState(isEnabled) {
                var configuration = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                configuration.shell.retailExperience = isEnabled;
                configuration.fue.showLXFUE = !isEnabled;
                WinJS.Promise.timeout(10 * 1000).done(function retailExperienceInformCustomer() {
                    MSApp.terminateApp(new Error("Retail experience mode changed"))
                })
            }, resetAppStateForRetailExperienceIfNeeded: function resetAppStateForRetailExperienceIfNeeded() {
                if (!MS.Entertainment.Utilities.getRetailExperienceState())
                    return false;
                var previousExecutionDate = Windows.Storage.ApplicationData.current.localSettings[MS.Entertainment.Utilities.lastRunDemoModeDateKey];
                var currentExecutionDate = (new Date).toLocaleDateString();
                var wasReset = false;
                if (previousExecutionDate && (previousExecutionDate !== currentExecutionDate)) {
                    var configManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                    configManager.groveler.fullGrovelCompleted = false;
                    wasReset = true
                }
                Windows.Storage.ApplicationData.current.localSettings[MS.Entertainment.Utilities.lastRunDemoModeDateKey] = currentExecutionDate;
                return wasReset
            }, loadDemoScripts: function loadDemoScripts() {
                var loadPromise;
                if (MS.Entertainment.Utilities.isVideoApp1 && MS.Entertainment.Utilities.getRetailExperienceState())
                    loadPromise = MS.Entertainment.UI.Framework.loadScripts("/components/video/DemoDashboardViewModel.js", "/components/inlinedetails/VideoInlineDetailsDemo.js");
                return WinJS.Promise.as(loadPromise)
            }, removeAllPlaylistsIfDemoMode: function removeAllPlayListsIfDemoMode() {
                if (!MS.Entertainment.Utilities.getRetailExperienceState() || !MS.Entertainment.Utilities.isMusicApp)
                    return;
                var playlistsToDelete = [];
                var query = new MS.Entertainment.Data.Query.libraryPlaylists;
                query.sort = Microsoft.Entertainment.Queries.PlaylistsSortBy.dateEditedDescending;
                query.getItems().then(function demoModePlaylistQuery(playlists) {
                    return playlists.forEachAll(function demoModePlaylistForEach(playlist) {
                            var id = WinJS.Utilities.getMember("item.data.libraryId", playlist);
                            if (id < 1)
                                return;
                            playlistsToDelete.push(id)
                        })
                }).then(function demoModePlaylistDeletePlaylists() {
                    var mediaStore = new Microsoft.Entertainment.Platform.MediaStore;
                    var playlistProvider = mediaStore.playlistProvider;
                    return playlistProvider.deletePlaylistAsync(playlistsToDelete, Microsoft.Entertainment.Platform.MediaAvailability.available)
                }).done(null, function demoModePlaylistFailure(e) {
                    MS.Entertainment.Utilities.fail("Failure deleting playlists in demo mode: " + (e && e.message))
                })
            }, retailExperienceArgument: "RetailExperience", lastRunDemoModeDateKey: "RetailDemoMode"
    });
    WinJS.Namespace.define("MS.Entertainment.Utilities", {getCurrentUserLocation: function getCurrentUserLocation() {
            var classicNavServiceCurrentPage = String.empty;
            if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.navigation))
                classicNavServiceCurrentPage = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).getUserLocation();
            if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.winJSNavigation) && (classicNavServiceCurrentPage.indexOf("homeHub") === 0 || !classicNavServiceCurrentPage)) {
                var navigationJournal = MS.Entertainment.UI.Framework.Navigation.getJournal();
                return navigationJournal ? navigationJournal.location : String.empty
            }
            else if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.navigation))
                return classicNavServiceCurrentPage;
            else
                return String.empty
        }});
    WinJS.Namespace.define("MS.Entertainment.Utilities", {
        wasNavigatedToDashboard: false, assertIfCalledBeforeCriticalWorkCompleted: function assertIfCalledBeforeCriticalWorkCompleted() {
                var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                var isFirstLocationLoaded = MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.winJSNavigation) && MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.winJSNavigation).isFirstLocationLoaded;
                var wasNavigatedToDashboard = !MS.Entertainment.Utilities.wasNavigatedToDashboard || uiStateService.isHubStripVisible;
                MS.Entertainment.UI.Framework.assert(isFirstLocationLoaded || wasNavigatedToDashboard, "Function was called before all start-up critical work has been completed")
            }
    });
    WinJS.Namespace.define("MS.Entertainment.Utilities", {
        updateHtmlDirectionAttribute: MS.Entertainment.UI.Framework.updateHtmlDirectionAttribute, getTextDirection: MS.Entertainment.UI.Framework.getTextDirection, TextDirections: MS.Entertainment.UI.Framework.TextDirections, TriggerSequence: function(keyArray, continuityTimeout, callback) {
                var state = 0;
                var lastTime = null;
                this.inSequence = function() {
                    return state > 0
                };
                this.keyIn = function(key) {
                    if (lastTime === null || ((new Date) - lastTime) > continuityTimeout)
                        state = 0;
                    lastTime = new Date;
                    if (keyArray[state] === key)
                        state++;
                    else
                        lastTime = null;
                    if (lastTime !== null && state === keyArray.length) {
                        callback();
                        lastTime = null;
                        return true
                    }
                    return false
                }
            }, trimCharacterDirection: function trimCharacterDirection(string) {
                if (string) {
                    var start = 0;
                    var end = string.length;
                    var firstChar = string.charCodeAt(0);
                    var lastChar = string.charCodeAt(end - 1);
                    if (firstChar === String.rtlmCode || firstChar === String.ltrmCode)
                        start++;
                    if (lastChar === String.rtlmCode || lastChar === String.ltrmCode)
                        end--;
                    if (start !== 0 || end !== string.length)
                        string = string.substr(start, end)
                }
                return string
            }, detectCharacterDirection: function detectCharacterDirection(ch) {
                if (ch < 0x249C) {
                    if (ch < 0x200E) {
                        if (ch >= 0x0041 && ch <= 0x005A || ch >= 0x0061 && ch <= 0x007A || ch >= 0x00C0 && ch <= 0x02B8 || ch >= 0x0370 && ch <= 0x0589 || ch >= 0x0900 && ch <= 0x1FFC)
                            return MS.Entertainment.Utilities.TextDirections.LeftToRight;
                        else if (ch >= 0x0591 && ch <= 0x085E)
                            return MS.Entertainment.Utilities.TextDirections.RightToLeft
                    }
                    else if (ch >= 0x2090 && ch <= 0x209C || ch >= 0x210A && ch <= 0x2188 || ch >= 0x2336 && ch <= 0x237A || ch === String.ltrmCode)
                        return MS.Entertainment.Utilities.TextDirections.LeftToRight;
                    else if (ch === String.rtlmCode)
                        return MS.Entertainment.Utilities.TextDirections.RightToLeft
                }
                else if (ch < 0xFD50) {
                    if (ch <= 0x24E9 || (ch >= 0x2800 && ch <= 0x28FF) || (ch >= 0x2C00 && ch <= 0x2DFF) || (ch >= 0x3021 && ch <= 0x4DB5) || (ch >= 0x4E00 && ch <= 0xFB17))
                        return MS.Entertainment.Utilities.TextDirections.LeftToRight;
                    else if (ch >= 0xFB1D && ch <= 0xFD3D)
                        return MS.Entertainment.Utilities.TextDirections.RightToLeft
                }
                else if (ch <= 0xFDFC || (ch >= 0xFE70 && ch <= 0xFEFC))
                    return MS.Entertainment.Utilities.TextDirections.RightToLeft;
                else if ((ch >= 0xFF21 && ch <= 0xFF3A) || (ch >= 0xFF41 && ch <= 0xFF5A) || (ch >= 0xFF65 && ch <= 0xFFDC))
                    return MS.Entertainment.Utilities.TextDirections.LeftToRight;
                return null
            }, detectStringDirection: function detectStringDirection(value, min, max) {
                var char;
                var characterDirection;
                max = isNaN(max) ? value.length : max;
                for (var i = isNaN(min) ? 0 : min; i < max; i++) {
                    char = value.charCodeAt(i);
                    if (char !== 0x0020 && char !== 0x0009 && char !== 0x000A && char !== 0x000D) {
                        characterDirection = MS.Entertainment.Utilities.detectCharacterDirection(char);
                        if (characterDirection)
                            return characterDirection
                    }
                }
                return MS.Entertainment.Utilities.getTextDirection()
            }, needsNormalizedBaseline: function needsNormalizedBaseline(value, min, max) {
                var char;
                max = isNaN(max) ? value.length : max;
                min = isNaN(min) ? 0 : min;
                for (var i = min; i < max; i++) {
                    char = value.charCodeAt(i);
                    if ((char < 0xAC00 || char > 0xD7AF) && (char !== 0x0020))
                        return false
                }
                return (max - min) ? true : false
            }, verifyUrl: function verifyUrl(value, requireHttps) {
                var requiredHttpsExpression = /^https:\/\//i;
                var anyHttpExpression = /^http[s]?:\/\//i;
                if (requireHttps)
                    return requiredHttpsExpression.test(value);
                else
                    return anyHttpExpression.test(value)
            }, sanitizeString: function sanitizeString(dirtyString) {
                return dirtyString ? dirtyString.toLowerCase().replace(/[^\w\s]/g, String.empty) : String.empty
            }, isStreamingUrl: function(url) {
                return (url && url.match && url.match(/^zest:/i))
            }
    });
    WinJS.Namespace.define("MS.Entertainment.Utilities", {
        _cachedConfigurationXuid: null, _cachedConfigurationValues: null, _loadUserConfigurationValues: function _loadUserConfigurationValues(xuid) {
                var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                if (!xuid)
                    MS.Entertainment.fail("loadUserConfigurationValues: xuid not defined");
                try {
                    var setting = Windows.Storage.ApplicationData.current.localSettings.values[xuid];
                    var values = {};
                    if (setting)
                        try {
                            values = JSON.parse(setting)
                        }
                        catch(error) {
                            MS.Entertainment.UI.fail("Failed to parse user configuration values: " + error && error.message);
                            Windows.Storage.ApplicationData.current.localSettings.values[xuid] = null
                        }
                    MS.Entertainment.Utilities._cachedConfigurationXuid = xuid;
                    MS.Entertainment.Utilities._cachedConfigurationValues = values
                }
                catch(error) {}
            }, getUserConfigurationValue: function getUserConfigurationValue(configurationKey) {
                var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                var xuid = configurationManager.service.lastSignedInUserXuid;
                if (!xuid)
                    xuid = "UnauthenticatedUser";
                if (MS.Entertainment.Utilities.isApp2 && xuid !== MS.Entertainment.Utilities._cachedConfigurationXuid)
                    MS.Entertainment.Utilities._loadUserConfigurationValues(xuid);
                return MS.Entertainment.Utilities._cachedConfigurationValues && MS.Entertainment.Utilities._cachedConfigurationValues[configurationKey]
            }, setUserConfigurationValue: function setUserConfigurationValue(configurationKey, value) {
                var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                var xuid = configurationManager.service.lastSignedInUserXuid;
                if (!xuid)
                    xuid = "UnauthenticatedUser";
                if (MS.Entertainment.Utilities.isApp2 && xuid !== MS.Entertainment.Utilities._cachedConfigurationXuid)
                    MS.Entertainment.Utilities._loadUserConfigurationValues(xuid);
                if (MS.Entertainment.Utilities.isApp2)
                    try {
                        var values = MS.Entertainment.Utilities._cachedConfigurationValues || {};
                        var oldValue = values[configurationKey];
                        if (value === undefined)
                            delete values[configurationKey];
                        else
                            values[configurationKey] = value;
                        if (value !== oldValue)
                            Windows.Storage.ApplicationData.current.localSettings.values[xuid] = JSON.stringify(values);
                        MS.Entertainment.Utilities._cachedConfigurationValues = values
                    }
                    catch(error) {}
            }, deleteUserConfigurationValue: function deleteUserConfigurationValue(configurationKey) {
                MS.Entertainment.Utilities.setUserConfigurationValue(configurationKey, undefined)
            }, UserConfigurationValues: {
                albumArtistSyncScreenDismissed: "albumArtistSyncScreenDismissed", cachedGenre1: "cachedGenre1", cachedGenre2: "cachedGenre2", collectionDialogXuids: "collectionDialogXuids", homeEducationXuids: "homeEducationXuids", minimumListensForEducationPanelsReachedXuids: "minimumListensForEducationPanelsReachedXuids", musicVideosOpt: "musicVideosOpt", notInterestedRecommendations: "notInterestedRecommendations", nowPlayingEducationXuids: "nowPlayingEducationXuids", playlistDialogXuids: "playlistDialogXuids", playlistSyncScreenDismissed: "playlistSyncScreenDismissed", previousApplicationState: "previousApplicationState", welcomeDialogXuids: "welcomeDialogXuids", welcomeDialogOverride: "welcomeDialogOverride"
            }
    });
    WinJS.Namespace.define("MS.Entertainment.Utilities", {
        getUserRoamingSetting: function getUserRoamingSetting(configurationKey) {
            if (MS.Entertainment.Utilities.isApp2)
                return MS.Entertainment.Utilities.getUserConfigurationValue(configurationKey);
            else
                return Windows.Storage.ApplicationData.current.roamingSettings.values[configurationKey]
        }, setUserRoamingSetting: function setUserRoamingSetting(configurationKey, value) {
                if (MS.Entertainment.Utilities.isApp2)
                    MS.Entertainment.Utilities.setUserConfigurationValue(configurationKey, value);
                else
                    Windows.Storage.ApplicationData.current.roamingSettings.values[configurationKey] = value
            }, deleteUserRoamingSetting: function deleteUserRoamingSetting(configurationKey) {
                if (MS.Entertainment.Utilities.isApp2)
                    MS.Entertainment.Utilities.deleteUserConfigurationValue(configurationKey);
                else
                    Windows.Storage.ApplicationData.current.roamingSettings.values.remove(configurationKey)
            }
    });
    (function() {
        var uniqueInteger = -1;
        WinJS.Namespace.define("MS.Entertainment.Utilities", {getSessionUniqueInteger: function getSessionUniqueInteger() {
                uniqueInteger++;
                return uniqueInteger
            }})
    })();
    (function() {
        WinJS.Namespace.define("MS.Entertainment.UI.Framework", {
            beginShowAnimations: function beginShowAnimations(element, overrideOldSelector, overrideNewSelector) {
                return MS.Entertainment.UI.Framework._beginAnimations(element, overrideOldSelector || "data-ent-hideanimation", overrideNewSelector || "data-ent-showanimation")
            }, beginHideAnimations: function beginHideAnimations(element, overrideOldSelector, overrideNewSelector) {
                    return MS.Entertainment.UI.Framework._beginAnimations(element, overrideOldSelector || "data-ent-showanimation", overrideNewSelector || "data-ent-hideanimation")
                }, clearHideAnimations: function clearHideAnimations(element) {
                    return MS.Entertainment.UI.Framework._clearAnimations(element, "data-ent-hideanimation")
                }, clearShowAnimations: function clearShowAnimations(element) {
                    return MS.Entertainment.UI.Framework._clearAnimations(element, "data-ent-showanimation")
                }, beginEnterAnimations: function beginShowAnimations(element) {
                    return MS.Entertainment.UI.Framework._beginAnimations(element, "data-ent-exitanimation", "data-ent-enteranimation")
                }, beginExitAnimations: function beginHideAnimations(element) {
                    return MS.Entertainment.UI.Framework._beginAnimations(element, "data-ent-enteranimation", "data-ent-exitanimation")
                }, _clearAnimations: function _clearAnimations(element, oldSelector) {
                    if (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).animationsEnabled)
                        MS.Entertainment.UI.Framework.applyWithSelector(element, "[" + oldSelector + "]", function(subElement) {
                            var animationClasses = subElement.getAttribute(oldSelector);
                            if (animationClasses)
                                animationClasses.split(" ").forEach(function(animationClass) {
                                    WinJS.Utilities.removeClass(subElement, animationClass)
                                })
                        }, true)
                }, _beginAnimations: function _beginAnimations(element, oldSelector, newSelector) {
                    var hasAnimations = false;
                    var promises = [];
                    var animationsEnabled = false;
                    if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.uiState))
                        animationsEnabled = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).animationsEnabled;
                    if (animationsEnabled) {
                        MS.Entertainment.UI.Framework._clearAnimations(element, oldSelector);
                        MS.Entertainment.UI.Framework.applyWithSelector(element, "[" + newSelector + "]", function(subElement) {
                            if (MS.Entertainment.Utilities.isElementVisible(subElement)) {
                                var animationClasses = subElement.getAttribute(newSelector);
                                if (animationClasses) {
                                    var complete;
                                    var promise = new WinJS.Promise(function(c, e, p) {
                                            complete = c
                                        });
                                    promises.push(promise);
                                    var processAnimationAndComplete = function processAnimationAndComplete(animationName) {
                                            var index = animationNames.indexOf(animationName);
                                            if (index > -1) {
                                                animationNames.splice(index, 1);
                                                if (animationNames.length === 0) {
                                                    subElement.removeEventListener("animationend", animationComplete);
                                                    complete()
                                                }
                                                return true
                                            }
                                            else
                                                return false
                                        };
                                    var animationNames = [];
                                    var animationComplete = function animationComplete(event) {
                                            if (event.srcElement !== subElement)
                                                return;
                                            if (!processAnimationAndComplete(event.animationName))
                                                if (subElement.currentStyle && subElement.currentStyle.animationName) {
                                                    animationNames = subElement.currentStyle.animationName.split(",").map(function(animationName) {
                                                        return animationName.trim()
                                                    });
                                                    if (!processAnimationAndComplete(event.animationName));
                                                    hasAnimations = true
                                                }
                                        };
                                    subElement.addEventListener("animationend", animationComplete);
                                    animationClasses.split(" ").forEach(function(animationClass) {
                                        WinJS.Utilities.addClass(subElement, animationClass)
                                    });
                                    if (!hasAnimations) {
                                        animationNames = subElement.currentStyle.msAnimationName.split(",").map(function(animationName) {
                                            return animationName.trim()
                                        });
                                        if (animationNames.length <= 0 || (animationNames.length === 1 && animationNames[0] === "none")) {
                                            subElement.removeEventListener("animationend", animationComplete);
                                            complete()
                                        }
                                        else
                                            hasAnimations = true
                                    }
                                    if (!hasAnimations) {
                                        var transitionProperties = subElement.currentStyle.msTransitionProperty.split(",").map(function(transitionProperty) {
                                                return transitionProperty.trim()
                                            });
                                        if (transitionProperties.length > 0) {
                                            var transitionComplete = function transitionComplete(event) {
                                                    var index = transitionProperties.indexOf(event.propertyName);
                                                    if (index > -1) {
                                                        transitionProperties.splice(index, 1);
                                                        if (transitionProperties.length === 0) {
                                                            subElement.removeEventListener("transitionend", transitionComplete);
                                                            complete()
                                                        }
                                                    }
                                                };
                                            subElement.addEventListener("transitionend", transitionComplete);
                                            hasAnimations = true
                                        }
                                    }
                                }
                            }
                        }, true)
                    }
                    if (promises.length)
                        return WinJS.Promise.join(promises);
                    else
                        return WinJS.Promise.wrap()
                }
        })
    })();
    (function() {
        var lazyDefine = MS.Entertainment.UI.Framework.lazyDefine;
        var imageExtension = "png";
        WinJS.Namespace.define("MS.Entertainment.Utilities", {getPackageImageFileExtension: function() {
                return imageExtension
            }});
        WinJS.Namespace.define("MS.Entertainment.UI.ImagePaths", {
            transparent: lazyDefine(function() {
                return "ms-appx:///Images/transparent." + imageExtension
            }), imageNotFound: lazyDefine(function() {
                    return "ms-appx:///Images/img_not_found." + imageExtension
                }), genericVideoActorPlaceholder: lazyDefine(function() {
                    return "ms-appx:///Images/GenericIcons/actor_placeholder." + imageExtension
                }), genericVideoL3Hero: lazyDefine(function() {
                    return "ms-appx:///Images/GenericIcons/ico_GenericVideo_L3_Hero." + imageExtension
                }), genericVideoSmall: lazyDefine(function() {
                    return "ms-appx:///Images/GenericIcons/ico_GenericVideo_Small." + imageExtension
                }), genericVideoMedium: lazyDefine(function() {
                    return "ms-appx:///Images/GenericIcons/ico_74x_GenericVideo_Medium." + imageExtension
                }), videoDemoMoviePopover: lazyDefine(function() {
                    return "ms-appdata:///local/Images/movie_popover." + imageExtension
                }), videoDemoTvPopover: lazyDefine(function() {
                    return "ms-appdata:///local/Images/tv_popover." + imageExtension
                })
        })
    })();
    WinJS.Namespace.define("MS.Entertainment.Utilities", {
        getTryAgainDynamicText: function getTryAgainDynamicText() {
            var stringId = String.id.IDS_DETAILS_TRY_AGAIN_LABEL;
            if (MS.Entertainment.Utilities.isVideoApp)
                if (MS.Entertainment.UI.Controls.ChooseDownloadQualityOverlay.storedDownloadPreference === MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.hd)
                    stringId = String.id.IDS_DETAILS_TRY_AGAIN_LABEL_HD;
                else if (MS.Entertainment.UI.Controls.ChooseDownloadQualityOverlay.storedDownloadPreference === MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.sd)
                    stringId = String.id.IDS_DETAILS_TRY_AGAIN_LABEL_SD;
            return stringId
        }, getDownloadDynamicText: function getDownloadDynamicText() {
                var stringId = String.id.IDS_DETAILS_DOWNLOAD_LABEL;
                if (MS.Entertainment.Utilities.isVideoApp)
                    if (MS.Entertainment.UI.Controls.ChooseDownloadQualityOverlay.storedDownloadPreference === MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.hd)
                        stringId = String.id.IDS_DETAILS_DOWNLOAD_LABEL_HD;
                    else if (MS.Entertainment.UI.Controls.ChooseDownloadQualityOverlay.storedDownloadPreference === MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.sd)
                        stringId = String.id.IDS_DETAILS_DOWNLOAD_LABEL_SD;
                return stringId
            }, getDownloadSeasonDynamicText: function getDownloadSeasonDynamicText() {
                var stringId = String.id.IDS_TV_DOWNLOAD_SEASON_BUTTON;
                if (MS.Entertainment.Utilities.isVideoApp)
                    if (MS.Entertainment.UI.Controls.ChooseDownloadQualityOverlay.storedDownloadPreference === MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.hd)
                        stringId = String.id.IDS_TV_DOWNLOAD_SEASON_BUTTON_HD;
                    else if (MS.Entertainment.UI.Controls.ChooseDownloadQualityOverlay.storedDownloadPreference === MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.sd)
                        stringId = String.id.IDS_TV_DOWNLOAD_SEASON_BUTTON_SD;
                return stringId
            }, convertMediaItemToTvSeries: function convertMediaItemToTvSeries(mediaItem) {
                MS.Entertainment.Framework.assert(mediaItem, "convertMediaItemToTvSeries requires a mediaItem");
                var seriesData = null;
                if (mediaItem.seriesId) {
                    seriesData = {
                        libraryId: mediaItem.seriesLibraryId, Name: mediaItem.seriesTitle
                    };
                    if (mediaItem.seriesIdType && mediaItem.seriesIdType === MS.Entertainment.Data.Query.edsIdType.zuneCatalog)
                        seriesData.ZuneId = mediaItem.seriesId;
                    else
                        seriesData.ID = mediaItem.seriesId
                }
                else if (mediaItem.canonicalSeriesId)
                    seriesData = {
                        ID: mediaItem.canonicalSeriesId, title: {$value: mediaItem.canonicalSeriesTitle}
                    };
                else
                    MS.Entertainment.UI.Framework.fail("convertMediaItemToTvSeries encountered an unsupported media type");
                var newMediaItem = MS.Entertainment.Data.augment(seriesData, MS.Entertainment.Data.Augmenter.Marketplace.EdsTVSeries);
                return MS.Entertainment.ViewModels.MediaItemModel.augment(newMediaItem)
            }, getLegacyVideoRowCountForResolution: function getLegacyVideoRowCountForResolution() {
                var standardResolutionRows = MS.Entertainment.Utilities.STANDARD_RESOLUTION_ROWS;
                var highResolutionRows = MS.Entertainment.Utilities.HIGH_RESOLUTION_ROWS;
                var rowCount = standardResolutionRows;
                if (MS.Entertainment.Utilities.isHighResolution())
                    rowCount = highResolutionRows;
                return rowCount
            }, checkRecommendationsItemForNotInterested: function checkRecommendationsItemForNotInterested(item) {
                var notInterestedRecommendations = MS.Entertainment.Utilities.getUserConfigurationValue(MS.Entertainment.Utilities.UserConfigurationValues.notInterestedRecommendations);
                if (notInterestedRecommendations)
                    for (var i = 0; i < notInterestedRecommendations.length; i++)
                        if (notInterestedRecommendations[i] === item.serviceId)
                            return true;
                return false
            }, addNotInterestedRecommendationsItem: function addNotInterestedRecommendationsItem(item) {
                var notInterestedRecommendations = MS.Entertainment.Utilities.getUserConfigurationValue(MS.Entertainment.Utilities.UserConfigurationValues.notInterestedRecommendations);
                if (!notInterestedRecommendations)
                    notInterestedRecommendations = [];
                notInterestedRecommendations.push(item.serviceId);
                MS.Entertainment.Utilities.setUserConfigurationValue(MS.Entertainment.Utilities.UserConfigurationValues.notInterestedRecommendations, notInterestedRecommendations)
            }, postNotInterestedRecommendationsItem: function postNotInterestedRecommendationsItem(item) {
                if (item) {
                    var queryData = {Feedback: {
                                FeedbackValue: MS.Entertainment.Data.Query.Shared.SetRecommendationFeeback.notInterestedFeedbackValue, MediaId: item.serviceId, MediaType: item.edsMediaItemType
                            }};
                    var query = new MS.Entertainment.Data.Query.Shared.SetRecommendationFeeback(queryData);
                    return query.execute().then(null, function error(){})
                }
                else
                    return WinJS.Promise.as()
            }, getTvInlineDetailsPopoverControl: function getTvInlineDetailsPopoverControl() {
                var popoverControl = "MS.Entertainment.Pages.TvSeriesInlineDetails";
                if (MS.Entertainment.Utilities.isApp2)
                    popoverControl = "MS.Entertainment.Pages.TvSeriesInlineDetailsFullScreen";
                return popoverControl
            }, relatedMovieGridItemSizeSmall: {
                width: 200, height: 265
            }, relatedMovieGridItemSize: {
                width: 270, height: 360
            }, relatedMovieGridItemSize2: {
                width: 238, height: 324
            }, relatedTvGridItemSize2: {
                width: 324, height: 324
            }, seasonGridItemSizeSmall: {
                width: 225, height: 265
            }, seasonGridItemSize: {
                width: 320, height: 360
            }, seasonGridItemSize2: {
                width: 275, height: 360
            }, movieTileSize: {
                width: 185, height: 255
            }, movieLargeTileSize: {
                width: 240, height: 326
            }, personalTileSize: {
                width: 250, height: 255
            }, personalTileSizeL1Small: {
                width: 165, height: 170
            }, tvTileSize: {
                width: 250, height: 255
            }, tvLargeTileSize: {
                width: 326, height: 326
            }, tvHeroSize: {get: function() {
                    if (MS.Entertainment.Utilities.isVideoApp1)
                        return {
                                width: 550, height: 550
                            };
                    else
                        return {
                                width: 480, height: 480
                            }
                }}
    });
    WinJS.Namespace.define("MS.Entertainment.UI.Icon", {
        achievements: "\uE288", actionLinkArrow: "\uE26B", actionLinkArrowRTL: "\uE26C", addToCollection: "\uE2B3", addToNowPlaying: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return MS.Entertainment.UI.Icon.addToNowPlaying2;
                    else
                        return MS.Entertainment.UI.Icon.addToNowPlaying1
                }}, addToNowPlaying1: "\uE2AE", addToNowPlaying2: "\uE2E0", addToPlaylist: "\uE2B1", arrowNext: "\uE2EF", arrowPrevious: "\uE2EE", avatarItem: "\uE27B", book: "\uE2EA", bookStore: "\uE2F8", bullet: "\u16EB", invokeButton: "\uE3CE", chevronDown: "\uE3AF", chevronDownLight: "\E09D", chevronUp: "\uE3B0", cloud: "\uE2C1", cloudNotAvailable: "\uE2BF", cloudSyncing: "\uE2DD", close: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE409";
                    else
                        return "\u2715"
                }}, closedCaption: "\uE190", colorable: "\uE292", devices: "\uE2ED", details: "\uE28F", downloading: "\uE2BC", downloadingArrowPart: "\uE2BE", downloadingBasePart: "\uE2BD", downChevron: "\uE0A1", feedback: "\uE2E4", flexhub: "\uE2DE", friend: "\uE287", friendPending: "\uE2DC", friendRespond: "\uE289", friendRemove: "\uE28A", friendAdd: "\uE28B", genericVideo: "\uE29D", genericGame: "\uE29E", genericMusic: "\uE29F", game: "\uE279", gameAddon: "\uE277", gameCompare: "\uE286", gameDemo: "\uE276", gameJoin: "\uE278", home: "\uE274", info: "\uE2B6", inlineAchievements: "\uE2A1", inlineAwarded: "\uE2A2", inlineBeacon: "\uE2A3", inlineColorable: "\uE2A4", inlineFriend: "\uE2A8", inlineGame: "\uE2A5", inlineGamerScore: "\uE2A9", inlineError: "\uE2A6", inlineNotification: "\uE2E2", inlineLink: "\uE2A7", inlineLock: "\uE2A0", inlineNowPlaying: "\uE2B4", inlinePoints: "\uE2AA", inlineStreaming: "\uE2AB", magazine: "\uE2F7", magazineStore: "\uE2F9", modifierAvatarItem: "\uE295", modifierFriend: "\uE296", modifierXbox: "\uE297", modifierPlaylist: "\uE298", modifierAddRing: "\uE299", modifierAddRingPressed: "\uE29A", modifierRemoveRing: "\uE29B", modifierRemoveRingPressed: "\uE29C", moveToBottom: "\uE2FB", moveToTop: "\uE2FA", movies: "\uE410", musicInCollection: "\uE300", menu: "\uE27C", next: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE402";
                    else
                        return WinJS.UI.AppBarIcon.next
                }}, noSelection: "\uE294", notification: "\uE2E5", nowPlayingNext: "\uE097", nowPlayingPrev: "\uE096", playlist: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE43E";
                    else
                        return "\uE28F"
                }}, photo: "\uE282", previous: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE403";
                    else
                        return WinJS.UI.AppBarIcon.previous
                }}, profile: "\uE285", related: "\uE280", repeat: "\uE290", repeatOnce: "\uE291", radio: "\uE2FD", readingCollection: "\uE2F5", readingSubscription: "\uE2EB", screenFull: "\uE272", fullScreen: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE45A";
                    else
                        return MS.Entertainment.UI.Icon.screenFull
                }}, screenNormal: "\uE273", semanticZoom: "\uE2E7", shop: "\uE275", smartDj: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return MS.Entertainment.UI.Icon.smartDjNoRing;
                    else
                        return MS.Entertainment.UI.Icon.smartDjWithRing
                }}, smartDjPressed: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return MS.Entertainment.UI.Icon.smartDjNoRing;
                    else
                        return MS.Entertainment.UI.Icon.smartDjPressedWithRing
                }}, smartDjPressedWithRing: "\uE283", smartDjWithRing: "\uE284", smartDjAppbar: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return MS.Entertainment.UI.Icon.smartDjNoRing;
                    else
                        return MS.Entertainment.UI.Icon.smartDjAppbarWithRing
                }}, smartDjAppbarPressed: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return MS.Entertainment.UI.Icon.smartDjNoRing;
                    else
                        return MS.Entertainment.UI.Icon.smartDjAppbarPressedWithRing
                }}, smartDjAppbarWithRing: "\uE2BB", smartDjAppbarPressedWithRing: "\uE2BA", smartDjNoRing: "\uE2DF", streaming: "\uE2E3", smiley: "\uE293", subtitle: "\uE44D", switchDefinition: "\uE3C7", switchLanguage: "\uE413", tvMonitor: "\uE39D", themes: "\uE281", upChevron: "\uE0A0", video11: "\uE33E", sendToXbox: "\uE2AC", sendToXboxAdorner: "\uE2AD", sendToXboxAdornerAppbar: "\uE2B9", takeFromXbox: "\uE2AF", takeFromXboxAdorner: "\uE2B0", takeFromXboxAdornerAppbar: "\uE2B8", xbox: "\uE28C", xboxConnect: "\uE28D", xboxConnected: "\uE28E", xboxXenonLogo: "\uE3E3", videoCollection: "\uE2EA", joystickUp: "\uE2C2", joystickRight: "\uE2C3", joystickDown: "\uE2C4", joystickLeft: "\uE2C5", triggerRight: "\uE2C6", bumperRight: "\uE2C7", bumperLeft: "\uE2C8", triggerLeft: "\uE2C9", clear: "\uE2CA", webHub: "\uE2CB", swap: "\uE2CC", smartGlassGem: "\uE2B7", smartGlassConsole: "\uE2CD", smartGlassSlate: "\uE2CE", buttonBottomRight: "\uE2CF", buttonBottomLeft: "\uE2D0", buttonTopRight: "\uE2D1", buttonTopLeft: "\uE2D2", dpadLeft: "\uE2D3", dpadDown: "\uE2D4", dpadUp: "\uE2D5", dpadRight: "\uE2D6", fileFolder: "\uE3F5", keyboard: "\uE2D7", touchGuide: "\uE2D8", onscreenCursor: "\uE2D9", xenon: "\uE2DA", addTo: "\uE444", explore: "\uE42F", folder: "\uE188", help: "\uE405", mediaMusic: "\uE386", menuButton: "\uE3EC", moreActions: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE34B";
                    else
                        return WinJS.UI.AppBarIcon.more
                }}, music: "\uE41A", nowPlaying: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE335";
                    else
                        return MS.Entertainment.UI.Icon.inlineNowPlaying
                }}, pause: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE40C";
                    else
                        return "\uE103"
                }}, play: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE40D";
                    else
                        return "\uE102"
                }}, player: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE301";
                    else
                        return WinJS.UI.AppBarIcon.contact
                }}, playerSignIn: "\uE2FC", playlistMusic: "\uE2E9", playlistImport: "\uE3CA", restart: "\uE3C0", toggleMenu: "\uE332", shuffleOn: "\uE3C9", store: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE3A0";
                    else
                        return WinJS.UI.AppBarIcon.shop
                }}, refresh: "\uE3C3", "delete": {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE359";
                    else
                        return WinJS.UI.AppBarIcon.delete
                }}, search: {get: function() {
                    if (MS.Entertainment.Utilities.isApp2)
                        return "\uE3F7";
                    else
                        return WinJS.UI.AppBarIcon.find
                }}, watchlist: "\uE2E9", fixupIcons: function fixupIcons() {
                if (WinJS.UI.AppBarIcon.previous.indexOf("ms") !== -1)
                    (function() {
                        delete WinJS.UI.AppBarIcon;
                        WinJS.Namespace.define("WinJS.UI.AppBarIcon", {
                            previous: "", next: "", play: "", pause: "", edit: "", save: "", clear: "", "delete": "", remove: "", add: "", cancel: "", accept: "", more: "", redo: "", undo: "", home: "", up: "", forward: "", right: "", back: "", left: "", favorite: "", camera: "", settings: "", video: "", sync: "", download: "", mail: "", find: "", help: "", upload: "", emoji: "", twopage: "", leavechat: "", mailforward: "", clock: "", send: "", crop: "", rotatecamera: "", people: "", closepane: "", openpane: "", world: "", flag: "", previewlink: "", globe: "", trim: "", attachcamera: "", zoomin: "", bookmarks: "", document: "", protecteddocument: "", page: "", bullets: "", comment: "", mail2: "", contactinfo: "", hangup: "", viewall: "", mappin: "", phone: "", videochat: "", "switch": "", contact: "", rename: "", pin: "", musicinfo: "", go: "", keyboard: "", dockleft: "", dockright: "", dockbottom: "", remote: "", refresh: "", rotate: "", shuffle: "", list: "", shop: "", selectall: "", orientation: "", "import": "", importall: "", browsephotos: "", webcam: "", pictures: "", savelocal: "", caption: "", stop: "", showresults: "", volume: "", repair: "", message: "", page2: "", calendarday: "", calendarweek: "", calendar: "", characters: "", mailreplyall: "", read: "", link: "", accounts: "", showbcc: "", hidebcc: "", cut: "", attach: "", paste: "", filter: "", copy: "", emoji2: "", important: "", mailreply: "", slideshow: "", sort: "", manage: "", allapps: "", disconnectdrive: "", mapdrive: "", newwindow: "", openwith: "", contactpresence: "", priority: "", uploadskydrive: "", gototoday: "", font: "", fontcolor: "", contact2: "", folder: "", audio: "", placeholder: "", view: "", setlockscreen: "", settile: "", cc: "", stopslideshow: "", permissions: "", highlight: "", disableupdates: "", unfavorite: "", unpin: "", openlocal: "", mute: "", italic: "", underline: "", bold: "", movetofolder: "", likedislike: "", dislike: "", like: "", alignright: "", aligncenter: "", alignleft: "", zoom: "", zoomout: "", openfile: "", otheruser: "", admin: "", street: "", map: "", clearselection: "", fontdecrease: "", fontincrease: "", fontsize: "", cellphone: "", reshare: "", tag: "", repeatone: "", repeatall: "", outlinestar: "", solidstar: "", calculator: "", directions: "", target: "", library: "", phonebook: "", memo: "", microphone: "", postupdate: "", backtowindow: "", fullscreen: "", newfolder: "", calendarreply: "", unsyncfolder: "", reporthacked: "", syncfolder: "", blockcontact: "", switchapps: "", addfriend: "", touchpointer: "", gotostart: "", zerobars: "", onebar: "", twobars: "", threebars: "", fourbars: ""
                        })
                    })()
            }
    });
    WinJS.Namespace.define("MS.Entertainment.UI", {FWLink: {
            musicLibraries: "http://go.microsoft.com/fwlink/?LinkId=231829", videoLibraries: "http://go.microsoft.com/fwlink/?LinkId=267525", learnMore: "http://go.microsoft.com/fwlink/?LinkId=267489", cloudGrovelLearnMore: "http://go.microsoft.com/fwlink/?LinkId=280203", cloudGrovelPartialMatchLearnMore: "http://go.microsoft.com/fwlink/?LinkId=286257", advertisementReason: "http://go.microsoft.com/fwlink/?LinkID=282137", musicIntro: "http://go.microsoft.com/fwlink/?LinkID=285375"
        }});
    WinJS.Namespace.define("MS.Entertainment.Utilities", {
        _sounds: {
            initialized: false, backButtonClick: null, disabledButtonClick: null, elementFocus1: null, elementFocus2: null, elementFocus3: null, elementFocus4: null, overlayIn: null, overlayOut: null, pageTransitionLeft: null, pageTransitionRight: null, selectButtonClick: null, selectButtonPressingKinect: null, virtualKeyboardFocus: null, virtualKeyboardSelect: null
        }, initializeSounds: function initializeSounds() {
                if (MS.Entertainment.Utilities.isApp2 && App2.SoundClips.Audio && !MS.Entertainment.Utilities._sounds.initialized) {
                    MS.Entertainment.Utilities._sounds.backButtonClick = App2.SoundClips.BackButtonClick;
                    MS.Entertainment.Utilities._sounds.disabledButtonClick = App2.SoundClips.DisabledButtonClick;
                    MS.Entertainment.Utilities._sounds.elementFocus1 = App2.SoundClips.ElementFocus1;
                    MS.Entertainment.Utilities._sounds.elementFocus2 = App2.SoundClips.ElementFocus2;
                    MS.Entertainment.Utilities._sounds.elementFocus3 = App2.SoundClips.ElementFocus3;
                    MS.Entertainment.Utilities._sounds.elementFocus4 = App2.SoundClips.ElementFocus4;
                    MS.Entertainment.Utilities._sounds.overlayIn = App2.SoundClips.OverlayIn;
                    MS.Entertainment.Utilities._sounds.overlayOut = App2.SoundClips.OverlayOut;
                    MS.Entertainment.Utilities._sounds.pageTransitionLeft = App2.SoundClips.PageTransitionLeft;
                    MS.Entertainment.Utilities._sounds.pageTransitionRight = App2.SoundClips.PageTransitionRight;
                    MS.Entertainment.Utilities._sounds.selectButtonClick = App2.SoundClips.SelectButtonClick;
                    MS.Entertainment.Utilities._sounds.initialized = true
                }
            }, playBackButtonClick: function playBackButtonClick() {
                MS.Entertainment.Utilities.initializeSounds();
                if (MS.Entertainment.Utilities._sounds.backButtonClick)
                    MS.Entertainment.Utilities._sounds.backButtonClick.play()
            }, playDisabledButtonClick: function playDisabledButtonClick() {
                MS.Entertainment.Utilities.initializeSounds();
                if (MS.Entertainment.Utilities._sounds.disabledButtonClick)
                    MS.Entertainment.Utilities._sounds.disabledButtonClick.play()
            }, playElementFocus1: function playElementFocus1() {
                MS.Entertainment.Utilities.initializeSounds();
                if (MS.Entertainment.Utilities._sounds.elementFocus1)
                    MS.Entertainment.Utilities._sounds.elementFocus1.play()
            }, playElementFocus2: function playElementFocus2() {
                MS.Entertainment.Utilities.initializeSounds();
                if (MS.Entertainment.Utilities._sounds.elementFocus2)
                    MS.Entertainment.Utilities._sounds.elementFocus2.play()
            }, playElementFocus3: function playElementFocus3() {
                MS.Entertainment.Utilities.initializeSounds();
                if (MS.Entertainment.Utilities._sounds.elementFocus3)
                    MS.Entertainment.Utilities._sounds.elementFocus3.play()
            }, playElementFocus4: function playElementFocus4() {
                MS.Entertainment.Utilities.initializeSounds();
                if (MS.Entertainment.Utilities._sounds.elementFocus4)
                    MS.Entertainment.Utilities._sounds.elementFocus4.play()
            }, playOverlayIn: function playOverlayIn() {
                MS.Entertainment.Utilities.initializeSounds();
                if (MS.Entertainment.Utilities._sounds.overlayIn)
                    MS.Entertainment.Utilities._sounds.overlayIn.play()
            }, playOverlayOut: function playOverlayOut() {
                MS.Entertainment.Utilities.initializeSounds();
                if (MS.Entertainment.Utilities._sounds.overlayOut)
                    MS.Entertainment.Utilities._sounds.overlayOut.play()
            }, playPageTransitionLeft: function playPageTransitionLeft() {
                MS.Entertainment.Utilities.initializeSounds();
                if (MS.Entertainment.Utilities._sounds.pageTransitionLeft)
                    MS.Entertainment.Utilities._sounds.pageTransitionLeft.play()
            }, playPageTransitionRight: function playPageTransitionRight() {
                MS.Entertainment.Utilities.initializeSounds();
                if (MS.Entertainment.Utilities._sounds.pageTransitionRight)
                    MS.Entertainment.Utilities._sounds.pageTransitionRight.play()
            }, playSelectButtonClick: function playSelectButtonClick() {
                MS.Entertainment.Utilities.initializeSounds();
                if (MS.Entertainment.Utilities._sounds.selectButtonClick)
                    MS.Entertainment.Utilities._sounds.selectButtonClick.play()
            }, playRandomFocusSound: function playRandomFocusSound() {
                MS.Entertainment.Utilities.initializeSounds();
                var indexOfFocusSoundToPlay = Math.ceil(Math.random() * 4);
                switch (indexOfFocusSoundToPlay) {
                    case 1:
                        MS.Entertainment.Utilities.playElementFocus1();
                        break;
                    case 2:
                        MS.Entertainment.Utilities.playElementFocus2();
                        break;
                    case 3:
                        MS.Entertainment.Utilities.playElementFocus3();
                        break;
                    case 4:
                        MS.Entertainment.Utilities.playElementFocus4();
                        break;
                    default:
                        MS.Entertainment.Utilities.playElementFocus1();
                        break
                }
            }
    });
    WinJS.Namespace.define("MS.Entertainment.Utilities", {stringify: function stringify(obj) {
            try {
                return JSON.stringify(obj)
            }
            catch(ex) {
                return obj.toString()
            }
        }})
})()
