﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.ViewModels");
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {
        DelayLoadAction: MS.Entertainment.UI.Framework.define(function DelayLoadAction(actionId) {
            this.id = actionId
        }, {
            id: null, action: {get: function() {
                        var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                        return actionService.isRegistered(this.id) ? actionService.getAction(this.id) : null
                    }}
        }), SmartAppbarActionHost: MS.Entertainment.UI.Framework.define(function SmartAppbarActionHost() {
                this._selectionHandlers = []
            }, {
                _mediaItem: null, _selectedItemBindings: null, _mediaContext: null, _selectedItemStateEngine: null, _shareOperation: null, _selectionHandlers: null, dispose: function dispose() {
                        var appBarControl = MS.Entertainment.globalControls.getControl(MS.Entertainment.globalControls.GlobalControl.appBar);
                        if (appBarControl && appBarControl.hide)
                            appBarControl.hide(true);
                        this._releaseMediaContext();
                        this._releaseMediaResources()
                    }, mediaItem: {
                        get: function mediaItem_get() {
                            return this._mediaItem
                        }, set: function mediaItem_set(value) {
                                if (this._mediaItem !== value) {
                                    this._mediaItem = value;
                                    this._updateState()
                                }
                            }
                    }, addSelectionHandlers: function addSelectionHandlers(handlers) {
                        this._selectionHandlers = this._selectionHandlers || [];
                        this._selectionHandlers.push(handlers)
                    }, _releaseMediaResources: function _releaseMediaResources() {
                        if (this._shareOperation) {
                            this._shareOperation.cancel();
                            this._shareOperation = null
                        }
                        if (this._selectedItemStateEngine) {
                            this._selectedItemStateEngine.unload();
                            this._selectedItemStateEngine = null
                        }
                        if (this._selectedItemBindings) {
                            this._selectedItemBindings.cancel();
                            this._selectedItemBindings = null
                        }
                    }, _releaseMediaContext: function _releaseMediaContext() {
                        if (this._mediaContext) {
                            this._mediaContext.clearContext();
                            this._mediaContext = null
                        }
                    }, _updateState: function _updateState() {
                        this._releaseMediaResources();
                        if (!this._mediaItem)
                            this._releaseMediaContext();
                        else {
                            var appBarService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar);
                            var sender = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.shareSender);
                            if (this._mediaContext)
                                this._mediaContext.updateMediaItem(this._mediaItem);
                            else
                                this._mediaContext = appBarService.pushMediaContext(this._mediaItem, this._selectionHandlers, [], {executeLocation: MS.Entertainment.UI.Actions.ExecutionLocation.selection});
                            if (this._mediaItem.hydrate)
                                this._mediaItem.hydrate();
                            MS.Entertainment.ViewModels.MediaItemModel.hydrateLibraryInfoAsync(this._mediaItem);
                            if (!this._selectedItemStateEngine)
                                this._selectedItemStateEngine = MS.Entertainment.ViewModels.SmartBuyStateEngine.getSelectionStateFromMediaItem(this._mediaItem);
                            if (!this._selectedItemBindings)
                                this._selectedItemBindings = WinJS.Binding.bind(this._selectedItemStateEngine, {currentAppbarActions: this._updateAppBarActions.bind(this)});
                            this._shareOperation = sender.pendingShare(this._mediaItem)
                        }
                    }, _updateAppBarActions: function _updateAppBarActions(newActions) {
                        if (this._mediaContext)
                            this._mediaContext.setToolbarActions(newActions);
                        if (newActions.length > 0) {
                            var appBarControl = MS.Entertainment.globalControls.getControl(MS.Entertainment.globalControls.GlobalControl.appBar);
                            if (appBarControl)
                                appBarControl.show()
                        }
                    }
            }), SmartAppbarActions: (function SmartAppbarActions() {
                var _actionService;
                return {
                        checkForServiceBlockAsync: function checkForServiceBlockAsync(arrayMediaItems) {
                            var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                            arrayMediaItems = arrayMediaItems || [];
                            var firstMediaItem = arrayMediaItems[0] || {};
                            var isFirstItemTrackOrAlbum = (firstMediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.track || firstMediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.album);
                            if (!uiStateService.servicesEnabled && isFirstItemTrackOrAlbum && !firstMediaItem.canPlayLocally)
                                return MS.Entertainment.UI.Shell.showAppUpdateDialog().then(function dialogDismissed() {
                                        return WinJS.Promise.wrapError("Can't play due to medium block")
                                    });
                            else
                                return WinJS.Promise.wrap(arrayMediaItems)
                        }, createPlaySelectionAppbarAction: function createPlaySelectionAppbarAction(actionId) {
                                var playPreviewOnly = (actionId === MS.Entertainment.UI.AppBarActions.previewSelection);
                                var continuous = (actionId === MS.Entertainment.UI.AppBarActions.playSelectionContinuous || actionId === MS.Entertainment.UI.AppBarActions.playContinuous);
                                var playSelectionAction = new MS.Entertainment.UI.ToolbarAction;
                                playSelectionAction.id = actionId;
                                playSelectionAction.automationId = MS.Entertainment.UI.AutomationIds.appBarPlaySelection;
                                var titleId = String.id.IDS_MUSIC_PLAY_SELECTION_ACTION;
                                if (playPreviewOnly)
                                    titleId = String.id.IDS_MUSIC_PREVIEW_SELECTION_ACTION;
                                else if (actionId === MS.Entertainment.UI.AppBarActions.playContinuous)
                                    titleId = String.id.IDS_PLAY_BUTTON;
                                playSelectionAction.title = String.load(titleId);
                                playSelectionAction.icon = MS.Entertainment.UI.Icon.play;
                                playSelectionAction.shouldAutomaticallySendTelemetry = false;
                                if (playSelectionAction.isVoiceEnabled) {
                                    MS.Entertainment.ViewModels.assert(!(playPreviewOnly && MS.Entertainment.Utilities.isMusicApp2), "No VUI strings are available for preview; defaulting to Play strings.");
                                    playSelectionAction.title = String.load(String.id.IDS_PLAY_MUSIC_BUTTON_VUI_GUI);
                                    playSelectionAction.voicePhraseStringId = String.id.IDS_PLAY_MUSIC_BUTTON_VUI_ALM;
                                    playSelectionAction.voicePhoneticPhraseStringId = String.id.IDS_PLAY_MUSIC_BUTTON_VUI_PRON
                                }
                                playSelectionAction.executed = function executed() {
                                    if (!_actionService)
                                        _actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                    var action = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia);
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var mediaItems = mediaContext.mediaItem;
                                    var collectionFilter = mediaContext.collectionFilter;
                                    var options = mediaContext.options || {};
                                    var offset = null;
                                    var key = null;
                                    if (continuous && options && options.containingMedia) {
                                        mediaItems = options.containingMedia.playbackItemSource;
                                        offset = options.containingMedia.playbackOffset;
                                        key = options.containingMedia.playbackKey
                                    }
                                    action.automationId = MS.Entertainment.UI.AutomationIds.appBarPlaySelection;
                                    action.parameter = {
                                        mediaItem: mediaItems && mediaItems.copy ? mediaItems.copy() : mediaItems, playPreviewOnly: playPreviewOnly, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection, collectionFilter: collectionFilter, offset: offset
                                    };
                                    return MS.Entertainment.Data.List.listToArray(mediaItems).then(MS.Entertainment.ViewModels.SmartAppbarActions.checkForServiceBlockAsync).then(function preRollAdIfNeeded(arrayMediaItems) {
                                            var firstMediaItem = arrayMediaItems[0];
                                            var adService;
                                            if (collectionFilter !== Microsoft.Entertainment.Platform.MediaAvailability.availableOffline && MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.adService))
                                                adService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.adService);
                                            mediaItems = arrayMediaItems;
                                            return !playPreviewOnly && adService ? adService.playVideoAdIfRequired(firstMediaItem, collectionFilter) : WinJS.Promise.as(true)
                                        }, function handleServiceBlocked() {
                                            return false
                                        }).then(function playVideoAdIfRequired_complete(adPlayedIfNeeded) {
                                            return WinJS.Promise.wrap({
                                                    arrayMediaItems: mediaItems, allowPlayback: adPlayedIfNeeded
                                                })
                                        }, function playVideoAdIfRequired_failed(error) {
                                            MS.Entertainment.ViewModels.fail("playVideoAdIfRequired_failed: " + (error && error.message));
                                            return WinJS.Promise.wrap({
                                                    arrayMediaItems: mediaItems, allowPlayback: true
                                                })
                                        }).then(function playSelectedItems(playData) {
                                            var arrayMediaItems = playData.arrayMediaItems;
                                            var allowPlayback = playData.allowPlayback;
                                            if (!allowPlayback)
                                                return;
                                            if (action.parameter)
                                                action.parameter.mediaItem = arrayMediaItems;
                                            if (arrayMediaItems[0].mediaType === Microsoft.Entertainment.Queries.ObjectType.track)
                                                action.execute();
                                            else {
                                                var playContainerAction;
                                                var firstItem = arrayMediaItems.shift();
                                                var isArtistMediaItemArray = ((firstItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.person) && (firstItem.personType === Microsoft.Entertainment.Queries.PersonType.artist));
                                                if (isArtistMediaItemArray) {
                                                    playContainerAction = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.playArtist);
                                                    playContainerAction.automationId = MS.Entertainment.UI.AutomationIds.appBarPlaySelection;
                                                    playContainerAction.parameter = {
                                                        showImmersive: false, showAppBar: true, playCollectionItemsOnly: true, mediaItem: firstItem, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection, collectionFilter: collectionFilter, offset: offset, key: key
                                                    }
                                                }
                                                else {
                                                    playContainerAction = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia);
                                                    playContainerAction.automationId = MS.Entertainment.UI.AutomationIds.appBarPlaySelection;
                                                    playContainerAction.parameter = {
                                                        mediaItem: firstItem, showAppBar: true, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, playPreviewOnly: playPreviewOnly, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection, collectionFilter: collectionFilter, offset: offset, key: key
                                                    }
                                                }
                                                playContainerAction.execute();
                                                var sessionManager = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                                                var playbackSession = sessionManager.nowPlayingSession;
                                                var onPlaybackStarted = function playContainerItems(currentPlaybackState) {
                                                        if (currentPlaybackState === MS.Entertainment.Platform.Playback.TransportState.starting || currentPlaybackState === MS.Entertainment.Platform.Playback.TransportState.playing) {
                                                            playbackSession.unbind("currentTransportState", onPlaybackStarted);
                                                            if (isArtistMediaItemArray) {
                                                                playContainerAction = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.addArtistToNowPlaying);
                                                                playContainerAction.automationId = MS.Entertainment.UI.AutomationIds.appBarAddLocalArtistToNowPlaying;
                                                                playContainerAction.parameter = {
                                                                    mediaItem: arrayMediaItems, playCollectionItemsOnly: true, suppressNotification: true, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection, collectionFilter: collectionFilter
                                                                }
                                                            }
                                                            else {
                                                                playContainerAction = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.addToNowPlaying);
                                                                playContainerAction.automationId = MS.Entertainment.UI.AutomationIds.appBarPlaySelection;
                                                                playContainerAction.parameter = {
                                                                    mediaItem: arrayMediaItems, suppressNotification: true, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection, collectionFilter: collectionFilter
                                                                }
                                                            }
                                                            playContainerAction.execute()
                                                        }
                                                        else if (playbackSession.errorDescriptor)
                                                            playbackSession.unbind("currentTransportState", onPlaybackStarted)
                                                    };
                                                if (arrayMediaItems.length > 0)
                                                    playbackSession.bind("currentTransportState", onPlaybackStarted)
                                            }
                                            if (mediaContext)
                                                mediaContext.dispatchEvent(actionId)
                                        })
                                };
                                playSelectionAction.canExecute = function canExecute(parameter) {
                                    return true
                                };
                                return {action: playSelectionAction}
                            }, createAddToNowPlayingAppbarAction: function createAddToNowPlayingAppbarAction(actionId) {
                                var addToNowPlayingAction = new MS.Entertainment.UI.ToolbarAction;
                                if (actionId === MS.Entertainment.UI.AppBarActions.addLocalArtistToNowPlaying) {
                                    addToNowPlayingAction.id = MS.Entertainment.UI.AppBarActions.addLocalArtistToNowPlaying;
                                    addToNowPlayingAction.automationId = MS.Entertainment.UI.AutomationIds.appBarAddLocalArtistToNowPlaying
                                }
                                else if (actionId === MS.Entertainment.UI.AppBarActions.addMarketplaceArtistToNowPlaying) {
                                    addToNowPlayingAction.id = MS.Entertainment.UI.AppBarActions.addMarketplaceArtistToNowPlaying;
                                    addToNowPlayingAction.automationId = MS.Entertainment.UI.AutomationIds.appBarAddMarketplaceArtistToNowPlaying
                                }
                                else {
                                    addToNowPlayingAction.id = MS.Entertainment.UI.AppBarActions.addToNowPlaying;
                                    addToNowPlayingAction.automationId = MS.Entertainment.UI.AutomationIds.appBarAddToNowPlaying
                                }
                                addToNowPlayingAction.title = String.load(String.id.IDS_MUSIC_ADD_TO_LIST_NOW_PLAYING_ACTION);
                                addToNowPlayingAction.icon = MS.Entertainment.UI.Icon.addToNowPlaying;
                                addToNowPlayingAction.shouldAutomaticallySendTelemetry = false;
                                if (addToNowPlayingAction.isVoiceEnabled) {
                                    addToNowPlayingAction.title = String.load(String.id.IDS_MUSIC2_ADD_TO_NOW_PLAYING_BUTTON_VUI_GUI);
                                    addToNowPlayingAction.voicePhraseStringId = String.id.IDS_MUSIC2_ADD_TO_NOW_PLAYING_BUTTON_VUI_ALM;
                                    addToNowPlayingAction.voicePhoneticPhraseStringId = String.id.IDS_MUSIC2_ADD_TO_NOW_PLAYING_BUTTON_VUI_PRON
                                }
                                addToNowPlayingAction.executed = function executed(parameter) {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var mediaItems = (parameter && parameter.mediaItem) || mediaContext.mediaItem;
                                    var options = mediaContext.options || {};
                                    var shouldUseTopSongsForArtistAddTo = parameter.shouldUseTopSongsForArtistAddTo;
                                    return MS.Entertainment.Data.List.listToArray(mediaItems).then(MS.Entertainment.ViewModels.SmartAppbarActions.checkForServiceBlockAsync).then(function gotArray(arrayMediaItems) {
                                            if (!_actionService)
                                                _actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                            var action;
                                            if (!arrayMediaItems || !arrayMediaItems.length || !arrayMediaItems[0])
                                                return;
                                            var playLocalCollectionTracksOnly;
                                            if (addToNowPlayingAction.id === MS.Entertainment.UI.AppBarActions.addLocalArtistToNowPlaying)
                                                playLocalCollectionTracksOnly = true;
                                            else if (addToNowPlayingAction.id === MS.Entertainment.UI.AppBarActions.addMarketplaceArtistToNowPlaying)
                                                playLocalCollectionTracksOnly = false;
                                            else
                                                playLocalCollectionTracksOnly = arrayMediaItems[0].fromCollection;
                                            if (arrayMediaItems[0].mediaType === Microsoft.Entertainment.Queries.ObjectType.person) {
                                                action = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.addArtistToNowPlaying);
                                                action.automationId = playLocalCollectionTracksOnly ? MS.Entertainment.UI.AutomationIds.appBarAddLocalArtistToNowPlaying : MS.Entertainment.UI.AutomationIds.appBarAddMarketplaceArtistToNowPlaying;
                                                action.parameter = {
                                                    playCollectionItemsOnly: playLocalCollectionTracksOnly, mediaItem: arrayMediaItems, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection, shouldUseTopSongsForArtistAddTo: shouldUseTopSongsForArtistAddTo
                                                }
                                            }
                                            else {
                                                action = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.addToNowPlaying);
                                                action.automationId = addToNowPlayingAction.automationId;
                                                action.parameter = {
                                                    mediaItem: arrayMediaItems, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                                }
                                            }
                                            action.execute();
                                            if (mediaContext)
                                                mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.addToNowPlaying)
                                        }.bind(this), function handleServiceBlocked() {
                                            return
                                        })
                                };
                                addToNowPlayingAction.canExecute = function canExecute(parameter) {
                                    return true
                                };
                                return {action: addToNowPlayingAction}
                            }, createTrackPlayPreviewAppbarAction: function createTrackPlayPreviewAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia, {
                                        mediaItem: null, showAppBar: true, playPreviewOnly: true
                                    }, MS.Entertainment.UI.AppBarActions.previewTrack, MS.Entertainment.UI.AutomationIds.appBarPreviewTrack, String.id.IDS_DETAILS_PREVIEW_SONG_LABEL, MS.Entertainment.UI.Icon.play, {
                                        guiStringId: String.id.IDS_MUSIC2_PREVIEW_SONG_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_MUSIC2_PREVIEW_SONG_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_MUSIC2_PREVIEW_SONG_BUTTON_VUI_PRON
                                    })
                            }, createTrackPlayMusicVideoAppbarAction: function createTrackPlayMusicVideoAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia, {
                                        mediaItem: null, showAppBar: true, playMusicVideo: true
                                    }, MS.Entertainment.UI.AppBarActions.playMusicVideoFromTrack, MS.Entertainment.UI.AutomationIds.playMusicVideoFromTrack, String.id.IDS_WATCH_VIDEO_BUTTON_VUI_GUI, MS.Entertainment.UI.Icon.video11, {
                                        guiStringId: String.id.IDS_WATCH_VIDEO_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_WATCH_VIDEO_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_WATCH_VIDEO_BUTTON_VUI_PRON
                                    })
                            }, createPlayMusicVideoAppbarAction: function createPlayMusicVideoAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia, {
                                        mediaItem: null, showAppBar: true
                                    }, MS.Entertainment.UI.AppBarActions.playMusicVideo, MS.Entertainment.UI.AutomationIds.playMusicVideo, String.id.IDS_WATCH_VIDEO_BUTTON_VUI_GUI, MS.Entertainment.UI.Icon.video11, {
                                        guiStringId: String.id.IDS_WATCH_VIDEO_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_WATCH_VIDEO_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_WATCH_VIDEO_BUTTON_VUI_PRON
                                    })
                            }, createPlayTopSongsAppbarAction: function createPlayTopSongsAppbarAction() {
                                var playTopSongsAction = new MS.Entertainment.UI.ToolbarAction;
                                playTopSongsAction.automationId = MS.Entertainment.UI.AutomationIds.appBarPlayTopSongs;
                                playTopSongsAction.id = MS.Entertainment.UI.AppBarActions.playTopSongs;
                                playTopSongsAction.title = String.load(String.id.IDS_MUSIC_PLAY_TOP_SONGS_BUTTON);
                                playTopSongsAction.icon = MS.Entertainment.UI.Icon.play;
                                playTopSongsAction.shouldAutomaticallySendTelemetry = false;
                                playTopSongsAction.executed = function playTopSongsAction_executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var mediaItems = mediaContext.mediaItem;
                                    var options = mediaContext.options || {};
                                    MS.Entertainment.Data.List.listToArray(mediaItems).done(function gotArray(arrayMediaItems) {
                                        var action;
                                        if (!_actionService)
                                            _actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                        action = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.playArtist);
                                        action.automationId = MS.Entertainment.UI.AutomationIds.appBarPlayTopSongs;
                                        action.parameter = {
                                            mediaItem: arrayMediaItems[0], actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                        };
                                        if (!MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.adService))
                                            return action.execute();
                                        var adService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.adService);
                                        adService.isStreamingBlocked(arrayMediaItems[0]).done(function isStreamingBlocked_complete(signInNeeded) {
                                            var promise;
                                            var dialogDismissed;
                                            promise = new WinJS.Promise(function(complete) {
                                                dialogDismissed = complete
                                            }).then(function playVideoAdIfRequired() {
                                                var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                                                return (!signInNeeded || signIn.isSignedIn) ? adService.playVideoAdIfRequired(arrayMediaItems[0]) : false
                                            }).done(function playVideoAdIfRequired_complete(adPlayedIfNeeded) {
                                                if (adPlayedIfNeeded)
                                                    action.execute()
                                            }, function playVideoAdIfRequired_failed(error) {
                                                MS.Entertainment.ViewModels.fail("playVideoAdIfRequired_failed: " + (error && error.message));
                                                action.execute()
                                            });
                                            if (signInNeeded) {
                                                var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                                                var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                                                if (!featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlayAnonymous))
                                                    MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signUp, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], dialogDismissed);
                                                else if (configurationManager.service.lastSignedInUserXuid)
                                                    MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_EXISTING_USER_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_EXISTING_USER_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signInNowForExistingUser, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], dialogDismissed);
                                                else
                                                    MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_NEW_USER_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_NEW_USER_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signInNowForNewUser, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], dialogDismissed)
                                            }
                                            else
                                                dialogDismissed()
                                        }, function isStreamingBlocked_failed(error) {
                                            MS.Entertainment.ViewModels.fail("isStreamingBlocked_failed: " + (error && error.message))
                                        })
                                    }, function failedToGetArray(error) {
                                        MS.Entertainment.ViewModels.fail("Failed to play top songs. Error message: " + error && error.message)
                                    })
                                };
                                playTopSongsAction.canExecute = function canExecute() {
                                    return true
                                };
                                return {action: playTopSongsAction}
                            }, createPlaySmartDJAppbarAction: function createPlaySmartDJAction() {
                                var playSmartDJAction = new MS.Entertainment.UI.ToolbarAction;
                                playSmartDJAction.id = MS.Entertainment.UI.AppBarActions.playSmartDJ;
                                playSmartDJAction.automationId = MS.Entertainment.UI.AutomationIds.appBarPlaySmartDJ;
                                playSmartDJAction.title = String.load(String.id.IDS_DETAILS_PLAY_SMART_DJ_MIX_NOW_LABEL);
                                playSmartDJAction.icon = MS.Entertainment.UI.Icon.smartDjAppbar;
                                playSmartDJAction.iconPressed = MS.Entertainment.UI.Icon.smartDjPressed;
                                playSmartDJAction.hideDefaultRing = true;
                                playSmartDJAction.shouldAutomaticallySendTelemetry = false;
                                playSmartDJAction.iconInfo = {className: "icon-smartDJ"};
                                if (playSmartDJAction.isVoiceEnabled) {
                                    playSmartDJAction.title = String.load(String.id.IDS_MUSIC2_PLAY_RADIO_BUTTON_VUI_GUI);
                                    playSmartDJAction.voicePhraseStringId = String.id.IDS_MUSIC2_PLAY_RADIO_BUTTON_VUI_ALM;
                                    playSmartDJAction.voicePhoneticPhraseStringId = String.id.IDS_MUSIC2_PLAY_RADIO_BUTTON_VUI_PRON
                                }
                                playSmartDJAction.executed = function playSmartDJAction_executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var mediaItems = mediaContext.mediaItem;
                                    var options = mediaContext.options || {};
                                    MS.Entertainment.Data.List.listToArray(mediaItems).done(function gotArray(arrayMediaItems) {
                                        var action;
                                        if (!_actionService)
                                            _actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                        action = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.playSmartDJ);
                                        action.automationId = MS.Entertainment.UI.AutomationIds.appBarPlaySmartDJ;
                                        action.parameter = {
                                            showImmersive: false, showAppBar: true, playCollectionItemsOnly: false, mediaItem: arrayMediaItems[0], actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                        };
                                        action.onComplete = function playSmartDJCompleted() {
                                            if (mediaContext)
                                                mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.playSmartDJ)
                                        };
                                        action.execute()
                                    })
                                };
                                playSmartDJAction.canExecute = function canExecute() {
                                    return true
                                };
                                return {action: playSmartDJAction}
                            }, createDeleteSmartDJAppbarAction: function createDeleteSmartDJAppbarAction() {
                                var deleteAction = new MS.Entertainment.UI.ToolbarAction;
                                deleteAction.id = MS.Entertainment.UI.AppBarActions.deleteSmartDJ;
                                deleteAction.automationId = MS.Entertainment.UI.AutomationIds.appBarDeleteSmartDJ;
                                deleteAction.title = String.load(String.id.IDS_DELETE_LABEL);
                                deleteAction.icon = MS.Entertainment.UI.Icon.delete;
                                deleteAction.shouldAutomaticallySendTelemetry = false;
                                deleteAction.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var mediaItems = mediaContext.mediaItem;
                                    var options = mediaContext.options || {};
                                    if (!_actionService)
                                        _actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                    var action = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.deleteSmartDJ);
                                    action.automationId = MS.Entertainment.UI.AutomationIds.appBarDeleteSmartDJ;
                                    action.parameter = {
                                        mediaItems: mediaItems, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                    };
                                    action.onComplete = function deleteCompleted() {
                                        if (mediaContext)
                                            mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.deleteSmartDJ, {
                                                deleted: true, mediaItems: mediaItems
                                            })
                                    };
                                    action.execute()
                                };
                                return {action: deleteAction}
                            }, createAppbarActionFromAction: function createAppbarActionFromAction(actionId, parameter, appBarActionId, automationId, titleId, icon, voiceInfo) {
                                if (!_actionService)
                                    _actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                var action = _actionService.getAction(actionId);
                                action.parameter = parameter || {};
                                var appBarAction = new MS.Entertainment.UI.ToolbarAction;
                                appBarAction.id = appBarActionId;
                                appBarAction.automationId = automationId;
                                if (titleId)
                                    appBarAction.title = String.load(titleId);
                                appBarAction.icon = icon;
                                appBarAction.wrappedAction = action;
                                appBarAction.parameter = parameter || {};
                                if (voiceInfo && appBarAction.isVoiceEnabled) {
                                    if (voiceInfo.guiStringId)
                                        appBarAction.title = String.load(voiceInfo.guiStringId);
                                    appBarAction.voicePhraseStringId = voiceInfo.voicePhraseStringId;
                                    appBarAction.voicePhoneticPhraseStringId = voiceInfo.voicePhoneticPhraseStringId;
                                    if (typeof(voiceInfo.voiceConfidence) === "number")
                                        appBarAction.voiceConfidence = voiceInfo.voiceConfidence
                                }
                                appBarAction.executed = function executed(parameter) {
                                    parameter = parameter || {};
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var options = mediaContext.options || {};
                                    appBarAction.parameter.actionType = MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction;
                                    appBarAction.executeLocation = options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection;
                                    appBarAction.shouldAutomaticallySendTelemetry = false;
                                    action.parameter.mediaItem = parameter.mediaItem || mediaContext.mediaItem;
                                    action.parameter.actionType = MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction;
                                    action.parameter.executeLocation = options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection;
                                    action.id = appBarAction.id;
                                    action.automationId = appBarAction.automationId;
                                    action.title = appBarAction.title;
                                    action.referenceContainer = appBarAction.referenceContainer;
                                    action.shouldAutomaticallySendTelemetry = false;
                                    WinJS.Promise.as(action.execute()).done(function actionFinished() {
                                        if (!parameter.skipCompleteHandler && mediaContext)
                                            mediaContext.dispatchEvent(appBarActionId)
                                    }, function handleError(error) {
                                        MS.Entertainment.ViewModels.fail("Executing app bar action has failed. ActionId: " + appBarActionId + " Error: " + (error && error.message))
                                    })
                                };
                                action.bind("isEnabled", function isEnabledChanged(isEnabled) {
                                    appBarAction.isEnabled = isEnabled
                                });
                                if (parameter.hasSubActions)
                                    action.bind("subActions", function subActionsChanged(newItems) {
                                        appBarAction.subActions = newItems
                                    });
                                appBarAction.canExecute = function canExecute(parameter) {
                                    if (!action.parameter.mediaItem || !action.parameter.isFlyoutAction) {
                                        var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                        action.parameter.mediaItem = mediaContext.mediaItem
                                    }
                                    action.requeryCanExecute();
                                    return action.isEnabled
                                };
                                return {action: appBarAction}
                            }, createPinToStartScreenAction: function createPinToStartScreenAction() {
                                var action = new MS.Entertainment.UI.ToolbarAction;
                                action.id = MS.Entertainment.UI.AppBarActions.pinToStartScreen;
                                action.automationId = MS.Entertainment.UI.AutomationIds.appBarPinToStartScreen;
                                action.title = String.load(String.id.IDS_PIN_TO_START_SCREEN_BUTTON);
                                action.icon = WinJS.UI.AppBarIcon.pin;
                                action.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var mediaItems = mediaContext.mediaItem;
                                    var mediaItem = Array.isArray(mediaItems) ? (mediaItems.length > 0 ? mediaItems[0] : null) : mediaItems;
                                    if (!mediaItem)
                                        throw new Error("Invalid mediaItems when createPinToStartScreenAction was executed");
                                    var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                                    MS.Entertainment.Data.List.getData(mediaItem, 0).done(function getData_complete(media) {
                                        return pinToStartScreenService.pinMediaAsync(media)
                                    })
                                };
                                action.onComplete = function pinToStartScreenCompleted() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    if (mediaContext)
                                        mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.pinToStartScreen)
                                };
                                return {action: action}
                            }, createUnpinFromStartScreenAction: function createUnpinFromStartScreenAction() {
                                var action = new MS.Entertainment.UI.ToolbarAction;
                                action.id = MS.Entertainment.UI.AppBarActions.unpinFromStartScreen;
                                action.automationId = MS.Entertainment.UI.AutomationIds.appBarUnpinFromStartScreen;
                                action.title = String.load(String.id.IDS_UNPIN_FROM_START_SCREEN_BUTTON);
                                action.icon = WinJS.UI.AppBarIcon.unpin;
                                action.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var mediaItems = mediaContext.mediaItem;
                                    var mediaItem = Array.isArray(mediaItems) ? (mediaItems.length > 0 ? mediaItems[0] : null) : mediaItems;
                                    if (!mediaItem)
                                        throw new Error("Invalid mediaItems when createUnpinFromStartScreenAction was executed");
                                    var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                                    MS.Entertainment.Data.List.getData(mediaItem, 0).done(function getData_complete(media) {
                                        return pinToStartScreenService.unpinMediaAsync(media)
                                    })
                                };
                                action.onComplete = function unPinToStartScreenCompleted() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    if (mediaContext)
                                        mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.unpinFromStartScreen)
                                };
                                return {action: action}
                            }, createGetOnXboxAction: function createGetOnXboxAction() {
                                var action = new MS.Entertainment.UI.ToolbarAction;
                                action.id = MS.Entertainment.UI.AppBarActions.getOnXbox360;
                                action.automationId = MS.Entertainment.UI.AutomationIds.appBarGetOnXbox360;
                                action.title = String.load(String.id.IDS_VIDEO_GET_FOR_XBOX360_APPBAR);
                                action.icon = MS.Entertainment.UI.Icon.shop;
                                action.shouldAutomaticallySendTelemetry = false;
                                action.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var mediaItems = mediaContext.mediaItem;
                                    var options = mediaContext.options || {};
                                    action.parameter = {
                                        actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                    };
                                    if (mediaItems === null)
                                        throw new Error("Invalid mediaItems when getGetOnXboxAction was executed");
                                    if (!_actionService)
                                        _actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                    var playToXboxAction = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.playOnXbox);
                                    var mediaItem = Array.isArray(mediaItems) ? mediaItems[0] : mediaItems;
                                    playToXboxAction.automationId = MS.Entertainment.UI.AutomationIds.appBarGetOnXbox360;
                                    playToXboxAction.parameter = {
                                        mediaItem: mediaItem, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                    };
                                    playToXboxAction.eventHandlers = {onFinishedEvent: function onFinishedEvent() {
                                            if (mediaContext)
                                                mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.getOnXbox360)
                                        }};
                                    playToXboxAction.execute()
                                };
                                return {action: action}
                            }, createDeleteMediaAppbarAction: function createDeleteMediaAppbarAction(appbarActionId, automationId, titleId) {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.deleteMedia, {
                                        mediaItem: null, skipCompleteHandler: true
                                    }, appbarActionId ? appbarActionId : MS.Entertainment.UI.AppBarActions.deleteMedia, automationId ? automationId : MS.Entertainment.UI.AutomationIds.smartButtonDeleteMedia, titleId ? titleId : String.id.IDS_DELETE_LABEL, MS.Entertainment.UI.Icon.delete, {
                                        guiStringId: String.id.IDS_DELETE_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_DELETE_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_DELETE_BUTTON_VUI_PRON, voiceConfidence: MS.Entertainment.Utilities.isApp2 ? MS.Entertainment.UI.App2.VoiceStatics.criticalActionConfidence : 0.0
                                    })
                            }, createDeleteSeriesAppbarAction: function createDeleteMediaAppbarAction() {
                                return this.createDeleteMediaAppbarAction(MS.Entertainment.UI.AppBarActions.deleteSeries, MS.Entertainment.UI.AutomationIds.smartButtonDeleteSeries, String.id.IDS_VIDEO_DELETE_SERIES_BUTTON)
                            }, createPurchaseAlbumAppbarAction: function createPurchaseAlbumAppbarAction() {
                                var action = new MS.Entertainment.UI.ToolbarAction;
                                action.id = MS.Entertainment.UI.AppBarActions.purchaseAlbumFromMarketplace;
                                action.automationId = MS.Entertainment.UI.AutomationIds.appBarPurchaseAlbumFromMarketplace;
                                action.title = String.load(String.id.IDS_DETAILS_BUY_ALBUM_LABEL);
                                action.icon = MS.Entertainment.UI.Icon.shop;
                                action.shouldAutomaticallySendTelemetry = false;
                                action.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var mediaItem = mediaContext.mediaItem;
                                    var options = mediaContext.options || {};
                                    action.parameter = {
                                        actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                    };
                                    if (!_actionService)
                                        _actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                    var purchaseAction = _actionService.getAction(MS.Entertainment.Utilities.isMusicApp1 ? MS.Entertainment.UI.Actions.ActionIdentifiers.albumPurchase : MS.Entertainment.UI.Actions.ActionIdentifiers.purchase);
                                    var albumToPurchase = Array.isArray(mediaItem) ? mediaItem[0] : mediaItem;
                                    purchaseAction.automationId = MS.Entertainment.UI.AutomationIds.appBarPurchaseAlbumFromMarketplace;
                                    purchaseAction.parameter = {
                                        mediaItem: albumToPurchase, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                    };
                                    purchaseAction.eventHandlers = {onFinishedEvent: function onFinishedEvent() {
                                            purchaseAction.parameter = {};
                                            if (mediaContext)
                                                mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.purchaseAlbumFromMarketplace)
                                        }};
                                    purchaseAction.execute()
                                };
                                action.canExecute = function canExecute() {
                                    return true
                                };
                                return {action: action}
                            }, createPurchaseTrackAppbarAction: function createPurchaseTrackAppbarAction() {
                                var action = new MS.Entertainment.UI.ToolbarAction;
                                action.id = MS.Entertainment.UI.AppBarActions.purchaseTrackFromMarketplace;
                                action.automationId = MS.Entertainment.UI.AutomationIds.appBarPurchaseTrackFromMarketplace;
                                action.title = String.load(String.id.IDS_DETAILS_BUY_SONG_LABEL);
                                action.icon = MS.Entertainment.UI.Icon.shop;
                                action.shouldAutomaticallySendTelemetry = false;
                                action.executed = function executed() {
                                    var hydratePromise;
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var mediaItem = mediaContext.mediaItem;
                                    var options = mediaContext.options || {};
                                    mediaItem = Array.isArray(mediaItem) ? mediaItem[0] : mediaItem;
                                    action.parameter = {
                                        actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                    };
                                    if (!_actionService)
                                        _actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                    var purchaseAction = _actionService.getAction(MS.Entertainment.Utilities.isMusicApp1 ? MS.Entertainment.UI.Actions.ActionIdentifiers.trackPurchase : MS.Entertainment.UI.Actions.ActionIdentifiers.purchase);
                                    purchaseAction.automationId = MS.Entertainment.UI.AutomationIds.appBarPurchaseTrackFromMarketplace;
                                    if (mediaItem.hydrate)
                                        hydratePromise = mediaItem.hydrate({forceUpdate: true});
                                    else
                                        hydratePromise = WinJS.Promise.wrap(mediaItem);
                                    hydratePromise.done(function hydrated() {
                                        var hasTrackPurchaseRights;
                                        var hasAlbumPurchaseRights;
                                        var mediaItemToPurchase;
                                        if (mediaItem.rights) {
                                            hasTrackPurchaseRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(mediaItem, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Purchase]);
                                            hasAlbumPurchaseRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(mediaItem, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.AlbumPurchase])
                                        }
                                        mediaItemToPurchase = (!hasTrackPurchaseRights && hasAlbumPurchaseRights) ? mediaItem.album : mediaItem;
                                        purchaseAction.parameter = {
                                            mediaItem: mediaItemToPurchase, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                        };
                                        purchaseAction.eventHandlers = {onFinishedEvent: function onFinishedEvent() {
                                                purchaseAction.parameter = {};
                                                if (mediaContext)
                                                    mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.purchaseTrackFromMarketplace)
                                            }};
                                        purchaseAction.execute()
                                    }, function hydrate_failed(){})
                                };
                                action.canExecute = function canExecute() {
                                    return true
                                };
                                return {action: action}
                            }, createNavigateBackAppbarAction: function createNavigateBackAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.navigate, MS.Entertainment.UI.Actions.navigate.NavigateLocation.back, MS.Entertainment.UI.AppBarActions.navigateBack, MS.Entertainment.UI.AutomationIds.appBarNavigateBack, String.id.IDS_ACC_BACK_BUTTON, WinJS.UI.AppBarIcon.back)
                            }, createPlayAndShowNowPlayingAppbarAction: function createPlayAndShowNowPlayingAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, {
                                        mediaItem: null, showDetails: false, autoPlay: true
                                    }, MS.Entertainment.UI.AppBarActions.playNow, MS.Entertainment.UI.AutomationIds.appBarPlayNow, String.id.IDS_DETAILS_PLAY_NOW_LABEL, MS.Entertainment.UI.Icon.play)
                            }, createSeriesDetailsAppbarAction: function createSeriesDetailsAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, {
                                        mediaItem: null, showDetails: true, autoPlay: false
                                    }, MS.Entertainment.UI.AppBarActions.seriesDetails, MS.Entertainment.UI.AutomationIds.appBarSeriesDetail, String.id.IDS_DETAILS_VIEW_FULL_SERIES, WinJS.UI.AppBarIcon.viewall)
                            }, createSeasonDetailsAppbarAction: function createSeasonDetailsAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, {
                                        mediaItem: null, showDetails: true, autoPlay: false, hub: "seasons"
                                    }, MS.Entertainment.UI.AppBarActions.seasonDetails, MS.Entertainment.UI.AutomationIds.appBarSeasonDetails, String.id.IDS_DETAILS_VIEW_FULL_SERIES, WinJS.UI.AppBarIcon.viewall)
                            }, createRemoveAppbarAction: function createRemoveAppbarAction(media, eventHandlers) {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadAgain, {mediaItem: null}, MS.Entertainment.UI.AppBarActions.removeFailedVideo, MS.Entertainment.UI.AutomationIds.appBarRemoveDownload, String.id.IDS_REMOVE_BUTTON, WinJS.UI.AppBarIcon.remove)
                            }, createMovieDetailsAppbarAction: function createMovieDetailsAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, {
                                        mediaItem: null, showDetails: true, autoPlay: false
                                    }, MS.Entertainment.UI.AppBarActions.movieDetails, MS.Entertainment.UI.AutomationIds.appBarMovieDetails, String.id.IDS_DETAILS_VIEW_FULL_MOVIE, MS.Entertainment.UI.Icon.details)
                            }, createPlayTrailerAppbarAction: function createPlayTrailerAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, {
                                        mediaItem: null, showDetails: MS.Entertainment.Utilities.isVideoApp1, autoPlay: true, playPreviewOnly: true
                                    }, MS.Entertainment.UI.AppBarActions.playTrailer, MS.Entertainment.UI.AutomationIds.appBarPlayTrailer, String.id.IDS_DETAILS_WATCH_TRAILER_LABEL, MS.Entertainment.UI.Icon.play)
                            }, createShowDownloadManagerAppbarAction: function createShowDownloadManagerAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.showVideoDownloadManager, {}, MS.Entertainment.UI.AppBarActions.showDownloadManager, MS.Entertainment.UI.AutomationIds.appBarShowDownloadManager, String.id.IDS_VIDEO_DOWNLOAD_MANAGER_LINK, WinJS.UI.AppBarIcon.download)
                            }, createRestartAppbarAction: function createRestartAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, {
                                        mediaItem: null, showDetails: false, autoPlay: true, startPositionMsec: 1
                                    }, MS.Entertainment.UI.AppBarActions.restartVideo, MS.Entertainment.UI.AutomationIds.appBarRestartVideo, String.id.IDS_RESTART_BUTTON, MS.Entertainment.UI.Icon.restart)
                            }, createResumeAppbarAction: function createResumeAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, {
                                        mediaItem: null, showDetails: false, autoPlay: true
                                    }, MS.Entertainment.UI.AppBarActions.resumeVideo, MS.Entertainment.UI.AutomationIds.appBarResumeVideo, String.id.IDS_RESUME_BUTTON, MS.Entertainment.UI.Icon.play)
                            }, createPlayDisabledAppbarAction: function createPlayDisabledAppbarAction(media, eventHandlers) {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.disabled, {}, MS.Entertainment.UI.AppBarActions.playDisabled, MS.Entertainment.UI.AutomationIds.appBarPlayDisabled, String.id.IDS_DETAILS_PLAY_NOW_LABEL, MS.Entertainment.UI.Icon.play)
                            }, createBuyEpisodeAppbarAction: function createBuyEpisodeAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, {
                                        mediaItem: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY
                                    }, MS.Entertainment.UI.AppBarActions.buyEpisode, MS.Entertainment.UI.AutomationIds.appBarBuyEpisode, String.id.IDS_DETAILS_BUY_EPISODE_POPOVER_LABEL, MS.Entertainment.UI.Icon.shop)
                            }, createPlayFreeOfferAppbarAction: function createPlayFreeOfferAppbarAction() {
                                var appbarAction = this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.playFreeOffer, {
                                        mediaItem: null, freeOfferId: null, freeOfferCurrencyCode: null, freeOfferSignedOffer: null, startPlayback: true
                                    }, MS.Entertainment.UI.AppBarActions.playFreeOffer, MS.Entertainment.UI.AutomationIds.appBarPlayFreeVideoOffer, String.id.IDS_PLAY_BUTTON_VIDEO, MS.Entertainment.UI.Icon.play);
                                appbarAction.subTitle = String.load(String.id.IDS_VIDEO_PRICE_FREE);
                                return appbarAction
                            }, createBuySeasonAppbarAction: function createBuySeasonAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, {
                                        mediaItem: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY
                                    }, MS.Entertainment.UI.AppBarActions.buySeason, MS.Entertainment.UI.AutomationIds.appBarBuySeason, String.id.IDS_DETAILS_BUY_SEASON_LABEL, MS.Entertainment.UI.Icon.shop)
                            }, createBuyCurrentSeasonAppbarAction: function createBuyCurrentSeasonAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, {
                                        mediaItem: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY
                                    }, MS.Entertainment.UI.AppBarActions.buyCurrentSeason, MS.Entertainment.UI.AutomationIds.appBarBuyCurrentSeason, String.id.IDS_DETAILS_BUY_SEASON_CURRENT_LABEL, MS.Entertainment.UI.Icon.shop)
                            }, createBuyVideoAppbarAction: function createBuyVideoAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, {
                                        mediaItem: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY
                                    }, MS.Entertainment.UI.AppBarActions.buyVideo, MS.Entertainment.UI.AutomationIds.appBarBuyVideo, String.id.IDS_BUY_BUTTON_VIDEO, MS.Entertainment.UI.Icon.shop)
                            }, createRentVideoAppbarAction: function createRentVideoAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, {
                                        mediaItem: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_RENT
                                    }, MS.Entertainment.UI.AppBarActions.rentVideo, MS.Entertainment.UI.AutomationIds.appBarRentVideo, String.id.IDS_RENT_BUTTON, MS.Entertainment.UI.Icon.shop)
                            }, createDownloadAppbarAction: function createDownloadAppbarAction(appBarActionId, automationId, titleId) {
                                var downloadAction = new MS.Entertainment.UI.ToolbarAction;
                                downloadAction.id = appBarActionId;
                                downloadAction.automationId = automationId;
                                if (titleId)
                                    downloadAction.title = String.load(titleId);
                                downloadAction.icon = WinJS.UI.AppBarIcon.download;
                                downloadAction.shouldAutomaticallySendTelemetry = false;
                                downloadAction.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var listOrMediaItem = mediaContext.mediaItem;
                                    var options = mediaContext.options || {};
                                    downloadAction.parameter = {
                                        actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                    };
                                    MS.Entertainment.Data.List.getData(listOrMediaItem, 0).then(function showImmersiveDetails(mediaItem) {
                                        MS.Entertainment.Platform.PurchaseHelpers.ensurePreownedMediaAddedAsync(mediaItem).then(function ensurePreownedMediaAddedAsync_complete(dbItems) {
                                            MS.Entertainment.Platform.PurchaseHelpers.downloadMedia(mediaItem, dbItems.dbMediaIds, dbItems.dbMediaTypes).then(function downloadMedia_complete(taskIds) {
                                                if (mediaContext)
                                                    mediaContext.dispatchEvent(appBarActionId)
                                            })
                                        })
                                    })
                                };
                                downloadAction.canExecute = function canExecute(parameter) {
                                    return this.isEnabled
                                };
                                return {action: downloadAction}
                            }, createDownloadFromMarketplaceAppbarAction: function createDownloadFromMarketplaceAppbarAction() {
                                var appBarButton = this.createDownloadAppbarAction(MS.Entertainment.UI.AppBarActions.downloadFromMarketplace, MS.Entertainment.UI.AutomationIds.appBarDownloadFromMarketplace, MS.Entertainment.Utilities.getDownloadDynamicText());
                                return appBarButton
                            }, createDownloadSeasonAppbarAction: function createDownloadSeasonAppbarAction() {
                                var appBarButton = this.createDownloadAppbarAction(MS.Entertainment.UI.AppBarActions.downloadSeason, MS.Entertainment.UI.AutomationIds.appBarDownloadSeason, MS.Entertainment.Utilities.getDownloadSeasonDynamicText());
                                return appBarButton
                            }, createTryDownloadAgainAppbarAction: function createTryDownloadAgainAppbarAction() {
                                var appBarButton = this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadAgain, {mediaItem: null}, MS.Entertainment.UI.AppBarActions.tryDownloadAgain, MS.Entertainment.UI.AutomationIds.appBarDownloadAgain, MS.Entertainment.Utilities.getTryAgainDynamicText(), WinJS.UI.AppBarIcon.download);
                                return appBarButton
                            }, createRemoveDownloadAppbarAction: function createRemoveDownloadAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadRemove, {mediaItem: null}, MS.Entertainment.UI.AppBarActions.removeFailedDownload, MS.Entertainment.UI.AutomationIds.appBarRemoveDownload, String.id.IDS_CLEAR_DOWNLOAD_ERROR_BUTTON, WinJS.UI.AppBarIcon.remove)
                            }, createSubscriptionDownloadAppbarAction: function createSubscriptionDownloadAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.subscriptionDownload, {mediaItem: null}, MS.Entertainment.UI.AppBarActions.subscriptionDownload, MS.Entertainment.UI.AutomationIds.appBarSubscriptionDownload, String.id.IDS_DETAILS_DOWNLOAD_LABEL, WinJS.UI.AppBarIcon.download)
                            }, createAddToMyMusicAppbarAction: function createAddToMyMusicAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.subscriptionAddToMyMusic, {mediaItem: null}, MS.Entertainment.UI.AppBarActions.addToMyMusic, MS.Entertainment.UI.AutomationIds.appBarAddToMyMusic, String.id.IDS_MUSIC_ADD_TO_MY_MUSIC_BUTTON, MS.Entertainment.UI.Icon.addToCollection, {
                                        guiStringId: String.id.IDS_MUSIC2_ADD_TO_COLLECTION_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_MUSIC2_ADD_TO_COLLECTION_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_MUSIC2_ADD_TO_COLLECTION_BUTTON_VUI_PRON
                                    })
                            }, createAddSongToAppbarAction: function createAddSongToAppbarAction() {
                                var appBarAction = this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.addTo, {
                                        mediaItem: null, hasSubActions: true
                                    }, MS.Entertainment.UI.AppBarActions.addTo, MS.Entertainment.UI.AutomationIds.appBarAddTo, String.id.IDS_MUSIC2_ADD_SONG_TO_LIST_BUTTON, WinJS.UI.AppBarIcon.add, {
                                        guiStringId: String.id.IDS_MUSIC2_ADD_SONG_TO_LIST_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_MUSIC2_ADD_SONG_TO_LIST_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_MUSIC2_ADD_SONG_TO_LIST_BUTTON_VUI_PRON
                                    });
                                appBarAction.action.hasSubActions = true;
                                return appBarAction
                            }, createAddToAppbarAction: function createAddToAppbarAction(stringID) {
                                var appBarAction = this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.addTo, {
                                        mediaItem: null, hasSubActions: true
                                    }, MS.Entertainment.UI.AppBarActions.addTo, MS.Entertainment.UI.AutomationIds.appBarAddTo, stringID, WinJS.UI.AppBarIcon.add);
                                appBarAction.action.hasSubActions = true;
                                return appBarAction
                            }, createEditMetadataAppbarAction: function createEditMetadataAppbarAction() {
                                return new MS.Entertainment.ViewModels.DelayLoadAction(MS.Entertainment.UI.Actions.ActionIdentifiers.editMetadata)
                            }, createRemoveFromListAppbarAction: function createRemoveFromListAppbarAction() {
                                return new MS.Entertainment.ViewModels.DelayLoadAction(MS.Entertainment.UI.Actions.ActionIdentifiers.removeFromList)
                            }, createMoveListItemUpAppbarAction: function createMoveListItemUpAppbarAction() {
                                return new MS.Entertainment.ViewModels.DelayLoadAction(MS.Entertainment.UI.Actions.ActionIdentifiers.moveListItemUp)
                            }, createMoveListItemDownAppbarAction: function createMoveListItemDownAppbarAction() {
                                return new MS.Entertainment.ViewModels.DelayLoadAction(MS.Entertainment.UI.Actions.ActionIdentifiers.moveListItemDown)
                            }, createMoveListItemToTopAppbarAction: function createMoveListItemToTopAppbarAction() {
                                return new MS.Entertainment.ViewModels.DelayLoadAction(MS.Entertainment.UI.Actions.ActionIdentifiers.moveListItemToTop)
                            }, createMoveListItemToBottomAppbarAction: function createMoveListItemToBottomAppbarAction() {
                                return new MS.Entertainment.ViewModels.DelayLoadAction(MS.Entertainment.UI.Actions.ActionIdentifiers.moveListItemToBottom)
                            }, createRemoveFromPlaylistAppbarAction: function createRemoveFromPlaylistAppbarAction() {
                                return MS.Entertainment.UI.Actions.Playlists.getRemoveFromPlaylistAction()
                            }, createRemoveFromNowPlayingAppbarAction: function createRemoveFromNowPlayingAppbarAction() {
                                return MS.Entertainment.UI.Actions.Playlists.getRemoveFromNowPlayingAction()
                            }, createShowPlaylistItemInformationAppbarAction: function createShowPlaylistItemInformationAppbarAction() {
                                return MS.Entertainment.UI.Actions.Playlists.getShowPlaylistItemInformationAction()
                            }, createImportPlaylistAppbarAction: function createImportPlaylistAppbarAction() {
                                return MS.Entertainment.UI.Actions.ImportPlaylists.getImportPlaylistsAction()
                            }, createDeletePlaylistAppbarAction: function createDeletePlaylistAppbarAction() {
                                return MS.Entertainment.UI.Actions.Playlists.getDeletePlaylistAction()
                            }, createRenamePlaylistAppbarAction: function createRenamePlaylistAppbarAction() {
                                return MS.Entertainment.UI.Actions.Playlists.getRenamePlaylistAction()
                            }, createSaveNowPlayingPlaylistAppbarAction: function createSaveNowPlayingPlaylistAppbarAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.saveNowPlayingPlaylist, {}, MS.Entertainment.UI.AppBarActions.saveNowPlayingPlaylist, MS.Entertainment.UI.AutomationIds.appBarSaveNowPlaying, String.id.IDS_MUSIC_SAVE_NOW_PLAYING_ACTION, WinJS.UI.AppBarIcon.save)
                            }, createClearSelectionAppbarAction: function createClearSelectionAppbarAction() {
                                var clearSelectionAction = new MS.Entertainment.UI.ToolbarAction;
                                clearSelectionAction.id = MS.Entertainment.UI.AppBarActions.clearSelection;
                                clearSelectionAction.automationId = MS.Entertainment.UI.AutomationIds.appBarClearSelection;
                                clearSelectionAction.title = String.load(String.id.IDS_CLEAR_SELECTION);
                                clearSelectionAction.icon = WinJS.UI.AppBarIcon.clear;
                                clearSelectionAction.shouldAutomaticallySendTelemetry = false;
                                clearSelectionAction.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var options = mediaContext.options || {};
                                    clearSelectionAction.parameter = {
                                        actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                    };
                                    if (mediaContext)
                                        mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.clearSelection)
                                };
                                clearSelectionAction.canExecute = function canExecute(parameter) {
                                    return true
                                };
                                return {action: clearSelectionAction}
                            }, createPlayOnXboxAppbarAction: function createPlayOnXboxAppbarAction(actionId) {
                                if (!_actionService)
                                    _actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                var playToXboxAction = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.playOnXbox);
                                var action = new MS.Entertainment.UI.ToolbarAction;
                                if (actionId === MS.Entertainment.UI.AppBarActions.playTrackOnXbox360) {
                                    action.id = MS.Entertainment.UI.AppBarActions.playTrackOnXbox360;
                                    playToXboxAction.automationId = action.automationId = MS.Entertainment.UI.AutomationIds.appBarPlayTrackOnXbox360;
                                    action.title = String.load(String.id.IDS_DETAILS_PLAY_SONG_ON_XBOX_APPBAR_LABEL)
                                }
                                else {
                                    action.id = MS.Entertainment.UI.AppBarActions.playOnXbox360;
                                    playToXboxAction.automationId = action.automationId = MS.Entertainment.UI.AutomationIds.appBarPlayOnXbox360;
                                    action.title = String.load(String.id.IDS_DETAILS_PLAY_ON_XBOX_APPBAR_LABEL)
                                }
                                action.icon = MS.Entertainment.UI.Icon.sendToXbox;
                                action.adornerMode = MS.Entertainment.UI.Controls.IconButtonMode.Custom;
                                action.adornerRing = MS.Entertainment.UI.Icon.sendToXboxAdornerAppbar;
                                action.shouldAutomaticallySendTelemetry = false;
                                action.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var mediaItem = mediaContext.mediaItem;
                                    var options = mediaContext.options || {};
                                    MS.Entertainment.Data.List.getData(mediaItem, 0).done(function getDataSucceeded(listMediaItem) {
                                        action.parameter = {
                                            actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                        };
                                        playToXboxAction.parameter = {
                                            mediaItem: listMediaItem, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                        };
                                        playToXboxAction.eventHandlers = {onFinishedEvent: function onFinishedEvent() {
                                                if (mediaContext)
                                                    mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.playOnXbox360)
                                            }};
                                        playToXboxAction.execute()
                                    }, function getDataFailed(error) {
                                        MS.Entertainment.ViewModels.fail("createPlayOnXboxAppbarAction_getData failed: " + error)
                                    })
                                };
                                action.canExecute = function canExecute() {
                                    return playToXboxAction.isEnabled
                                };
                                return {action: action}
                            }, createCollectionTrackAlbumPopoverAppbarAction: function createCollectionTrackAlbumPopoverAppbarAction(invokeTrack) {
                                var trackAlbumAction = new MS.Entertainment.UI.ToolbarAction;
                                trackAlbumAction.id = MS.Entertainment.UI.AppBarActions.exploreAlbum;
                                trackAlbumAction.automationId = MS.Entertainment.UI.AutomationIds.appBarExploreAlbum;
                                trackAlbumAction.title = String.load(String.id.IDS_MUSIC_COLLECTION_EXPLORE_ALBUM);
                                trackAlbumAction.icon = MS.Entertainment.Utilities.isMusicApp2 ? MS.Entertainment.UI.Icon.info : MS.Entertainment.UI.Icon.details;
                                trackAlbumAction.shouldAutomaticallySendTelemetry = false;
                                trackAlbumAction.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    MS.Entertainment.Data.List.dataFromIndex(mediaContext.mediaItem, 0).done(function gotMediaItem(mediaItem) {
                                        var options = mediaContext.options || {};
                                        trackAlbumAction.parameter = {
                                            actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                        };
                                        var popOverParameters = {itemConstructor: "MS.Entertainment.Pages.MusicTrackAlbumInlineDetails"};
                                        popOverParameters.dataContext = {
                                            data: mediaItem, location: MS.Entertainment.Data.ItemLocation.collection, invokeTrack: invokeTrack
                                        };
                                        var appBarControl = MS.Entertainment.globalControls.getControl(MS.Entertainment.globalControls.GlobalControl.appBar);
                                        if (appBarControl)
                                            appBarControl.hide();
                                        MS.Entertainment.UI.Controls.PopOver.showPopOver(popOverParameters);
                                        if (mediaContext)
                                            mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.exploreAlbum)
                                    }, function handleError(error) {
                                        MS.Entertainment.ViewModels.fail("Failed to show album popover in createCollectionTrackAlbumPopoverAppbarAction. Error message: " + error && error.message);
                                        if (mediaContext)
                                            mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.exploreAlbum)
                                    })
                                };
                                trackAlbumAction.canExecute = function canExecute() {
                                    return true
                                };
                                return {action: trackAlbumAction}
                            }, createArtistDetailsAppbarAction: function createArtistDetailsAppbarAction() {
                                var useImmersiveDetails = !MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.winJSNavigation);
                                var artistDetailAction = new MS.Entertainment.UI.ToolbarAction;
                                artistDetailAction.id = MS.Entertainment.UI.AppBarActions.artistDetails;
                                artistDetailAction.automationId = MS.Entertainment.UI.AutomationIds.appBarArtistDetails;
                                artistDetailAction.title = String.load(String.id.IDS_DETAILS_VIEW_FULL_ARTIST_APPBAR);
                                artistDetailAction.icon = (MS.Entertainment.Utilities.isMusicApp2 || !useImmersiveDetails) ? MS.Entertainment.UI.Icon.player : MS.Entertainment.UI.Icon.details;
                                artistDetailAction.shouldAutomaticallySendTelemetry = false;
                                if (artistDetailAction.isVoiceEnabled) {
                                    artistDetailAction.title = String.load(String.id.IDS_MUSIC2_EXPLORE_ARTIST_BUTTON_VUI_GUI);
                                    artistDetailAction.voicePhraseStringId = String.id.IDS_MUSIC2_EXPLORE_ARTIST_BUTTON_VUI_ALM;
                                    artistDetailAction.voicePhoneticPhraseStringId = String.id.IDS_MUSIC2_EXPLORE_ARTIST_BUTTON_VUI_PRON
                                }
                                artistDetailAction.executed = function executed() {
                                    if (!_actionService)
                                        _actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var mediaItems = mediaContext.mediaItem;
                                    var options = mediaContext.options || {};
                                    artistDetailAction.parameter = {
                                        actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                    };
                                    var action;
                                    var mediaItem = Array.isArray(mediaItems) ? mediaItems[0] : mediaItems;
                                    if (!useImmersiveDetails)
                                        MS.Entertainment.Data.List.getData(mediaItem, 0).then(function showArtistDetails(item) {
                                            if (item.mediaType !== Microsoft.Entertainment.Queries.ObjectType.person && item.artist)
                                                item = item.artist;
                                            action = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.artistDetailsNavigate);
                                            action.parameter = {
                                                data: item, location: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection, collectionFilter: mediaContext.collectionFilter
                                            };
                                            action.execute()
                                        });
                                    else {
                                        action = _actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails);
                                        action.parameter = {
                                            mediaItem: mediaItem, showDetails: true, autoPlay: false, automationId: MS.Entertainment.UI.AutomationIds.smartButtonArtistDetails, actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                        };
                                        action.execute()
                                    }
                                    if (mediaContext)
                                        mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.artistDetails)
                                };
                                artistDetailAction.canExecute = function canExecute() {
                                    return true
                                };
                                return {action: artistDetailAction}
                            }, createFindAlbumInfoAppbarAction: function createFindAlbumInfoAppbarAction() {
                                var findAlbumInfoAction = new MS.Entertainment.UI.ToolbarAction;
                                findAlbumInfoAction.id = MS.Entertainment.UI.AppBarActions.findAlbumInfo;
                                findAlbumInfoAction.automationId = MS.Entertainment.UI.AutomationIds.appBarFindAlbumInfo;
                                findAlbumInfoAction.title = String.load(String.id.IDS_MATCH_ALBUM_INFO_COMMAND);
                                findAlbumInfoAction.icon = MS.Entertainment.UI.Icon.search;
                                findAlbumInfoAction.shouldAutomaticallySendTelemetry = false;
                                findAlbumInfoAction.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var options = mediaContext.options || {};
                                    findAlbumInfoAction.parameter = {
                                        actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                    };
                                    MS.Entertainment.Data.List.getData(mediaContext.mediaItem, 0).then(function showFindAlbumInfoDialog(album) {
                                        var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                                        if (configurationManager.service.lastSignedInUserXuid && !MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn).isSignedIn)
                                            return MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_ADD_TO_MY_MUSIC_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signUp, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel]);
                                        return MS.Entertainment.Music.FindAlbumInfoDialog.show(album, MS.Entertainment.Music.FindAlbumInfo.MatchMode.library)
                                    }.bind(this)).done(function findAlbumInfoDialogClosed(dialog) {
                                        if (dialog && dialog.userControlInstance && dialog.userControlInstance.collectionUpdated && mediaContext)
                                            mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.findAlbumInfo)
                                    }, function findAlbumInfoDialogError() {
                                        MS.Entertainment.ViewModels.fail("FindAlbumInfo Dialog Failure")
                                    })
                                };
                                findAlbumInfoAction.canExecute = function canExecute(parameter) {
                                    return true
                                };
                                return {action: findAlbumInfoAction}
                            }, createAddToCloudAppbarAction: function createAddToCloudAppbarAction() {
                                var featureEnablement = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.featureEnablement);
                                var addToCloudActionTitle = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.devicesEnabled) ? String.id.IDS_MUSIC_ADD_TO_LIST_DEVICES_ACTION : String.id.IDS_MUSIC_ADD_TO_LIST_CLOUD_ACTION;
                                var addToCloudAction = new MS.Entertainment.UI.ToolbarAction;
                                addToCloudAction.id = MS.Entertainment.UI.AppBarActions.addToCloud;
                                addToCloudAction.automationId = MS.Entertainment.UI.AutomationIds.appBarAddToCloud;
                                addToCloudAction.title = String.load(addToCloudActionTitle);
                                addToCloudAction.icon = MS.Entertainment.UI.Icon.search;
                                addToCloudAction.shouldAutomaticallySendTelemetry = false;
                                addToCloudAction.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var options = mediaContext.options || {};
                                    var appView = Windows.UI.ViewManagement.ApplicationView;
                                    var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                                    if (uiStateService.isSnapped && !appView.tryUnsnap())
                                        return;
                                    addToCloudAction.parameter = {
                                        actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                    };
                                    MS.Entertainment.Data.List.getData(mediaContext.mediaItem, 0).then(function showFindAlbumInfoDialog(media) {
                                        var hydratePromise;
                                        var returnValue;
                                        if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.track)
                                            media = media.album;
                                        if (media.hydrate && !media.hydrated)
                                            hydratePromise = media.hydrate();
                                        if (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn).isSignedIn)
                                            returnValue = WinJS.Promise.as(hydratePromise).then(function hydrated() {
                                                return MS.Entertainment.Music.FindAlbumInfoDialog.show(media, MS.Entertainment.Music.FindAlbumInfo.MatchMode.cloud)
                                            }.bind(this));
                                        else
                                            returnValue = MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_ADD_TO_MY_MUSIC_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signUp, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel]);
                                        return returnValue
                                    }.bind(this)).done(function findAlbumInfoDialogClosed(dialog) {
                                        if (dialog && dialog.userControlInstance && dialog.userControlInstance.collectionUpdated && mediaContext)
                                            mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.findAlbumInfo)
                                    }, function findAlbumInfoDialogError() {
                                        MS.Entertainment.ViewModels.assert(false, "AddToCloud Dialog Failure")
                                    })
                                };
                                addToCloudAction.canExecute = function canExecute(parameter) {
                                    return true
                                };
                                return {action: addToCloudAction}
                            }, createMediaPropertiesAppbarAction: function createMediaPropertiesAppbarAction() {
                                var propertiesAction = new MS.Entertainment.UI.ToolbarAction;
                                propertiesAction.id = MS.Entertainment.UI.AppBarActions.mediaProperties;
                                propertiesAction.automationId = MS.Entertainment.UI.AutomationIds.appBarMediaProperties;
                                propertiesAction.title = String.load(String.id.IDS_MUSIC_PROPERTIES_DIALOG_TEXT);
                                propertiesAction.icon = MS.Entertainment.UI.Icon.details;
                                propertiesAction.shouldAutomaticallySendTelemetry = false;
                                propertiesAction.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var listOrMediaItem = mediaContext.mediaItem;
                                    var options = mediaContext.options || {};
                                    var appView = Windows.UI.ViewManagement.ApplicationView;
                                    var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                                    if (uiStateService.isSnapped && !appView.tryUnsnap())
                                        return;
                                    propertiesAction.parameter = {
                                        actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.selection
                                    };
                                    MS.Entertainment.Data.List.getData(listOrMediaItem, 0).then(function showPropertiesDialog(mediaItem) {
                                        if (!this.openPopup) {
                                            this.openPopup = MS.Entertainment.UI.Shell.createOverlay("MS.Entertainment.UI.Controls.MediaPropertiesFlyout", {mediaItem: mediaItem}, {
                                                top: "auto", right: "auto", left: "10px", bottom: "82px"
                                            });
                                            this.openPopup.show().then(function onPopupDismissed() {
                                                this.openPopup = null
                                            }.bind(this))
                                        }
                                        else
                                            this.openPopup.hide();
                                        if (mediaContext)
                                            mediaContext.dispatchEvent(MS.Entertainment.UI.AppBarActions.mediaProperties)
                                    }.bind(this))
                                };
                                propertiesAction.canExecute = function canExecute(parameter) {
                                    return true
                                };
                                return {action: propertiesAction}
                            }, createOverlayCloseAction: function createOverlayCloseAction() {
                                var closeAction = new MS.Entertainment.UI.ToolbarAction;
                                closeAction.id = MS.Entertainment.UI.AppBarActions.closeSmartGlassActivity;
                                closeAction.automationId = MS.Entertainment.UI.AutomationIds.appBarCloseSmartGlassActivity;
                                closeAction.title = String.load(String.id.IDS_ACTIVITY_CLOSE_BUTTON);
                                closeAction.icon = WinJS.UI.AppBarIcon.cancel;
                                closeAction.shouldAutomaticallySendTelemetry = false;
                                closeAction.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var options = (mediaContext && mediaContext.options) || {};
                                    closeAction.parameter = {
                                        actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.activity
                                    };
                                    MS.Entertainment.UI.Controls.ActivityOverlay.hide()
                                };
                                closeAction.canExecute = function canExecute() {
                                    return true
                                };
                                return {action: closeAction}
                            }, createControllerCloseAction: function createControllerCloseAction() {
                                var closeAction = new MS.Entertainment.UI.ToolbarAction;
                                closeAction.id = MS.Entertainment.UI.AppBarActions.closeSmartGlassController;
                                closeAction.automationId = MS.Entertainment.UI.AutomationIds.appBarCloseSmartGlassController;
                                closeAction.title = String.load(String.id.IDS_ACTIVITY_CLOSE_BUTTON);
                                closeAction.icon = WinJS.UI.AppBarIcon.cancel;
                                closeAction.shouldAutomaticallySendTelemetry = false;
                                closeAction.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var options = (mediaContext && mediaContext.options) || {};
                                    closeAction.parameter = {
                                        actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.canvas
                                    };
                                    MS.Entertainment.UI.Controls.XBoxControls.hide();
                                    MS.Entertainment.globalControls.getControl(MS.Entertainment.globalControls.GlobalControl.appBar).hide()
                                };
                                closeAction.canExecute = function canExecute() {
                                    return true
                                };
                                return {action: closeAction}
                            }, createToggleSwitcherAction: function createToggleSwitcherAction() {
                                var switcherAction = new MS.Entertainment.UI.ToolbarAction;
                                switcherAction.id = MS.Entertainment.UI.AppBarActions.toggleSwitcher;
                                switcherAction.automationId = MS.Entertainment.UI.AutomationIds.appBarToggleSwitcher;
                                switcherAction.title = String.load(String.id.IDS_CONTROLLER_SWITCHER_BUTTON);
                                switcherAction.icon = MS.Entertainment.UI.Icon.swap;
                                switcherAction.shouldAutomaticallySendTelemetry = false;
                                switcherAction.executed = function executed() {
                                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                                    var options = (mediaContext && mediaContext.options) || {};
                                    switcherAction.parameter = {
                                        actionType: MS.Entertainment.UI.Actions.ActionWrapperType.appbarAction, executeLocation: options.executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.canvas
                                    };
                                    var controls = mediaContext.options && mediaContext.options.xboxControls;
                                    controls.toggleSwitcher()
                                };
                                switcherAction.canExecute = function canExecute(parameter) {
                                    return true
                                };
                                return {action: switcherAction}
                            }, createCloseCommandingPopoverAction: function createCloseCommandingPopoverAction() {
                                var closeAction = new MS.Entertainment.UI.ToolbarAction;
                                closeAction.id = MS.Entertainment.UI.AppBarActions.closeDialog;
                                closeAction.automationId = MS.Entertainment.UI.AutomationIds.appBarCloseCommandingPopover;
                                closeAction.title = String.load(String.id.IDS_CLOSE_BUTTON_VUI_GUI);
                                closeAction.icon = WinJS.UI.AppBarIcon.cancel;
                                closeAction.shouldAutomaticallySendTelemetry = false;
                                closeAction.executed = function executed() {
                                    MS.Entertainment.UI.Controls.CommandingPopOver.hideCurrentCommandingPopover()
                                };
                                closeAction.canExecute = function canExecute() {
                                    return true
                                };
                                return {action: closeAction}
                            }, createSubscriptionSignupAction: function createSubscriptionSignupAction() {
                                return this.createAppbarActionFromAction(MS.Entertainment.UI.Actions.ActionIdentifiers.subscriptionSignup, {}, MS.Entertainment.UI.AppBarActions.subscriptionSignup, MS.Entertainment.UI.AutomationIds.subscriptionSignup, String.id.IDS_MUSIC_STREAMING_PLAY_LIMIT_PREMIUM_ACTION, MS.Entertainment.UI.Icon.xboxXenonLogo)
                            }
                    }
            })()
    })
})()
