﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.ViewModels");
(function() {
    "use strict";
    var _createBasicButton = function _createBasicButton(actionId, titleId, parameter, iconInfo, voiceInfo) {
            var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
            var button = actionService.getAction(actionId);
            if (titleId)
                button.title = String.load(titleId);
            if (parameter) {
                button.parameter = parameter;
                button.parameter.actionType = MS.Entertainment.UI.Actions.ActionWrapperType.button
            }
            if (iconInfo)
                button.iconInfo = iconInfo;
            if (button.parameter) {
                if (button.parameter.automationId)
                    button.automationId = button.parameter.automationId;
                if (button.parameter.id)
                    button.id = button.parameter.id
            }
            if (voiceInfo && button.isVoiceEnabled) {
                if (voiceInfo.guiStringId)
                    button.title = String.load(voiceInfo.guiStringId);
                button.voicePhraseStringId = voiceInfo.voicePhraseStringId;
                button.voicePhoneticPhraseStringId = voiceInfo.voicePhoneticPhraseStringId;
                if (voiceInfo.voiceConfidenceStringId)
                    button.voiceConfidenceStringId = voiceInfo.voiceConfidenceStringId;
                if (typeof(voiceInfo.voiceConfidence) === "number")
                    button.voiceConfidence = voiceInfo.voiceConfidence
            }
            return button
        };
    var _createButton = function createButton(actionId, titleId, parameter, iconInfo, voiceInfo) {
            var button = _createBasicButton(actionId, titleId, parameter, iconInfo, voiceInfo);
            if (button.parameter && button.parameter.dismissOverlayOnExecute) {
                var oldExecuted = button.executed.bind(button);
                button.executed = WinJS.Utilities.markSupportedForProcessing(function executed(actionData) {
                    var domEvent = document.createEvent("Event");
                    domEvent.initEvent("dismissoverlay", true, true);
                    var popover = document.querySelector(".overlayAnchor .popOver");
                    if (popover)
                        popover.dispatchEvent(domEvent);
                    oldExecuted(actionData)
                })
            }
            var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
            if (mediaContext.collectionFilter) {
                var previousExecuted = button.executed.bind(button);
                button.executed = WinJS.Utilities.markSupportedForProcessing(function executed(actionData, referenceElement) {
                    actionData.collectionFilter = mediaContext.collectionFilter;
                    previousExecuted(actionData, referenceElement)
                })
            }
            if (button.parameter && button.parameter.preRollVideoAdIfNeeded) {
                var lastExecuted = button.executed.bind(button);
                button.executed = WinJS.Utilities.markSupportedForProcessing(function executed(actionData, referenceElement) {
                    var collectionFilter = mediaContext.collectionFilter;
                    if (referenceElement)
                        MS.Entertainment.Utilities.Telemetry.logPageAction({domElement: referenceElement}, {
                            uri: MS.Entertainment.Utilities.getCurrentUserLocation(), pageTypeId: MS.Entertainment.Utilities.Telemetry.PageTypeId.Dash
                        });
                    if (collectionFilter === Microsoft.Entertainment.Platform.MediaAvailability.availableOffline || !MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.adService))
                        lastExecuted(actionData);
                    else {
                        var containingMedia = mediaContext.options && mediaContext.options.containingMedia;
                        var invokedMedia = (containingMedia && actionData.mediaItem === containingMedia.playbackItemSource) ? mediaContext.mediaItem : actionData.mediaItem;
                        var adService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.adService);
                        adService.isStreamingBlocked(invokedMedia).done(function isStreamingBlocked_complete(signInNeeded) {
                            var promise;
                            var dialogDismissed;
                            promise = new WinJS.Promise(function(c, e, p) {
                                dialogDismissed = c
                            }).then(function dialogDismissed() {
                                var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                                var collectionFilter = mediaContext && mediaContext.collectionFilter;
                                return (!signInNeeded || signIn.isSignedIn) ? adService.playVideoAdIfRequired(invokedMedia, collectionFilter) : false
                            }).done(function playVideoAdIfRequired_complete(adPlayedIfNeeded) {
                                if (adPlayedIfNeeded)
                                    lastExecuted(actionData)
                            }, function playVideoAdIfRequired_failed(error) {
                                MS.Entertainment.ViewModels.fail("playVideoAdIfRequired_failed: " + (error && error.message));
                                lastExecuted(actionData)
                            });
                            if (signInNeeded) {
                                var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                                var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                                if (!featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlayAnonymous))
                                    MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signUp, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], dialogDismissed);
                                else if (configurationManager.service.lastSignedInUserXuid)
                                    MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_EXISTING_USER_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_EXISTING_USER_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signInNowForExistingUser, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], dialogDismissed);
                                else
                                    MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_NEW_USER_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_NEW_USER_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signInNowForNewUser, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], dialogDismissed)
                            }
                            else
                                dialogDismissed()
                        }, function isStreamingBlocked_failed(error) {
                            MS.Entertainment.ViewModels.fail("isStreamingBlocked_failed: " + (error && error.message))
                        })
                    }
                })
            }
            return button
        };
    var _isFreeUser = function _isFreeUser() {
            var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
            var isSubscription = configurationManager.service.lastSignedInUserSubscription;
            return !isSubscription
        };
    WinJS.Namespace.define("MS.Entertainment.ViewModels.Actions", {GenericPlayButton: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function genericPlayButtonConstructor() {
            this.base();
            this.id = MS.Entertainment.UI.Actions.ActionIdentifiers.genericPlay;
            this.title = String.load(String.id.IDS_PLAY_BUTTON);
            this.automationId = MS.Entertainment.UI.AutomationIds.genericPlay;
            this.parameter = {
                automationId: this.automationId, executeLocation: MS.Entertainment.UI.Actions.ExecutionLocation.canvas
            };
            this.iconInfo = {
                icon: MS.Entertainment.UI.Icon.play, isToggleButton: false, isChecked: false, isDisabled: false
            }
        }, {
            _button: null, _executedSignal: null, _executedParam: null, executed: function executed(param) {
                    if (this._button)
                        return this._button.executed(param);
                    else if (!this._executedSignal) {
                        this._executedSignal = new MS.Entertainment.UI.Framework.Signal;
                        this._executedParam = param
                    }
                    return this._executedSignal.promise
                }, canExecute: function canExecute(param) {
                    return this._button ? this._button.canExecute(param) : true
                }, wrap: function wrap(button) {
                    MS.Entertainment.ViewModels.assert(button, "Button param expected");
                    this._button = button;
                    this.id = button.id;
                    this.title = button.title;
                    this.parameter = button.parameter;
                    this.iconInfo = button.iconInfo;
                    this.automationId = button.automationId;
                    if (button.parameter) {
                        if (button.parameter.automationId)
                            this.automationId = button.parameter.automationId;
                        if (button.parameter.id)
                            this.id = button.parameter.id
                    }
                    if (this._executedSignal) {
                        var executedSignal = this._executedSignal;
                        var executedParam = this._executedParam;
                        this._executedSignal = null;
                        this._executedParam = null;
                        WinJS.Promise.as(this.execute(executedParam)).done(function onExecuteComplete() {
                            executedSignal.complete()
                        }, function onExecuteError(error) {
                            executedSignal.error(error)
                        })
                    }
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {SmartBuyButtons: {
            createGenericPlayButton: function createGenericPlayButton() {
                return new MS.Entertainment.ViewModels.Actions.GenericPlayButton
            }, createButtonPlayDisabled: function createButtonPlayDisabled(executeLocation) {
                    var playStringId = MS.Entertainment.Utilities.isMusicApp ? String.id.IDS_PLAY_BUTTON : String.id.IDS_DETAILS_PLAY_NOW_LABEL;
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.disabled, playStringId, {
                            automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayDisabled, executeLocation: executeLocation, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplate"
                        }, {
                            icon: MS.Entertainment.UI.Icon.play, isToggleButton: true, isChecked: false, isDisabled: true
                        })
                }, createButtonDeleteMedia: function createButtonDeleteMedia(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.deleteMedia, String.id.IDS_DELETE_LABEL, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonDelete, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplate"
                        }, {icon: MS.Entertainment.UI.Icon.delete}, {
                            guiStringId: String.id.IDS_DELETE_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_DELETE_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_DELETE_BUTTON_VUI_PRON, voiceConfidence: MS.Entertainment.Utilities.isApp2 ? MS.Entertainment.UI.App2.VoiceStatics.criticalActionConfidence : 0.0
                        })
                }, createButtonTryAgain: function createButtonTryAgain(media, executeLocation, useVerboseDownloadText) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadAgain, useVerboseDownloadText ? String.id.IDS_MUSIC_DETAILS_RETRY_DOWNLOAD_ACTION : MS.Entertainment.Utilities.getTryAgainDynamicText(), {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonTryAgain
                        }, {icon: WinJS.UI.AppBarIcon.download});
                    return button
                }, createButtonPlayOnXbox: function createButtonPlayOnXbox(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playOnXbox, String.id.IDS_DETAILS_PLAY_ON_XBOX_LABEL, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayOnXbox, dismissOverlayOnExecute: true
                        }, {
                            icon: MS.Entertainment.UI.Icon.sendToXbox, adornerMode: MS.Entertainment.UI.Controls.IconButtonMode.Custom, adornerRing: MS.Entertainment.UI.Icon.sendToXboxAdorner, className: "enablePressedState"
                        })
                }, smartBuyButtonTemplateSelector: WinJS.Utilities.markSupportedForProcessing(function smartBuyButtonTemplateSelector(item) {
                    var result;
                    function loadItemTemplate(itemTemplate) {
                        return MS.Entertainment.UI.Framework.loadTemplate(itemTemplate, null, true).then(function(templateControl) {
                                return templateControl
                            })
                    }
                    if (item.parameter && item.parameter.itemTemplate)
                        return loadItemTemplate(item.parameter.itemTemplate);
                    else
                        return loadItemTemplate(this._itemTemplate)
                }), createButtonDownload: function createButtonDownload(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.download, MS.Entertainment.Utilities.getDownloadDynamicText(), {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonDownload
                        }, {icon: WinJS.UI.AppBarIcon.download});
                    return button
                }, createButtonPinToStartScreen: function createButtonPinToStartScreen(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.pinToStartScreen, String.id.IDS_PIN_TO_START_SCREEN_BUTTON, {
                            mediaItem: media, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPinToStartScreen, executeLocation: executeLocation, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplate"
                        }, {icon: WinJS.UI.AppBarIcon.pin});
                    return button
                }, createButtonUnpinFromStartScreen: function createButtonUnpinFromStartScreen(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.unpinFromStartScreen, String.id.IDS_UNPIN_FROM_START_SCREEN_BUTTON, {
                            mediaItem: media, automationId: MS.Entertainment.UI.AutomationIds.smartButtonUnpinFromStartScreen, executeLocation: executeLocation, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplate"
                        }, {icon: WinJS.UI.AppBarIcon.unpin});
                    return button
                }, _createButtonPinToDashsboardInternal: function _createButtonPinToDashsboardInternal(media, executeLocation, strings) {
                    var pinnedItemsList = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinnedList);
                    var hasPin = pinnedItemsList.hasPin(media);
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.pinMediaItem, hasPin ? strings.unpinButtonGuiStringId : strings.pinButtonGuiStringId, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPinToDashboard, strings: strings, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplate"
                        }, null, {
                            guiStringId: hasPin ? strings.unpinButtonGuiStringId : strings.pinButtonGuiStringId, voicePhraseStringId: hasPin ? strings.unpinButtonVoicePhraseStringId : strings.pinButtonVoicePhraseStringId, voicePhoneticPhraseStringId: hasPin ? strings.unpinButtonVoicePhoneticPhraseStringId : strings.pinButtonVoicePhoneticPhraseStringId, voiceConfidenceStringId: hasPin ? strings.unpinButtonVoiceConfidenceStringId : strings.pinButtonVoiceConfidenceStringId
                        });
                    button.parameter.button = button;
                    return button
                }, getSwitchOfferButtons: function getSwitchOfferButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.canvas;
                    var buttons = {
                            switchVideoDefinition: this.createButtonSwitchVideoDefinition(media, location), switchLanguage: this.createButtonSwitchLanguage(media, location)
                        };
                    return buttons
                }, getWatchNextEpisodeButtons: function getWatchNextEpisodeButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.canvas;
                    var buttons = {
                            buyEpisode: this.createButtonBuyEpisode(media, location), playFreeOffer: this.createButtonPlayFreeOffer(media, location), play: this.createButtonPlayAndShowNowPlaying(media, location), restart: this.createButtonRestart(media, location), resume: this.createButtonResume(media, location), playDisabled: this.createButtonPlayDisabled(location)
                        };
                    if (MS.Entertainment.Utilities.isVideoApp1) {
                        buttons.pinToStartScreen = this.createButtonPinToStartScreen(media, location);
                        buttons.unpinFromStartScreen = this.createButtonUnpinFromStartScreen(media, location)
                    }
                    return buttons
                }, getPostRollEpisodeButtons: function getPostRollEpisodeButtons(media) {
                    var buttons = {buyNextEpisode: this.createButtonBuyEpisodeInPostRoll(media)};
                    return buttons
                }, getPostRollSeasonButtons: function getPostRollSeasonButtons(media) {
                    var buttons = {
                            buyCurrentSeason: this.createButtonBuyCurrentSeasonInPostRoll(media), buyCurrentSeasonPass: this.createButtonBuyCurrentSeasonPassInPostRoll(media), buyNextSeason: this.createButtonBuyNextSeasonInPostRoll(media), buyNextSeasonPass: this.createButtonBuyNextSeasonPassInPostRoll(media)
                        };
                    return buttons
                }, getVideoDetailsButtons: function getVideoDetailsButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    var buttons = {
                            play: this.createButtonPlayAndShowNowPlaying(media, location), playTrailer: this.createButtonPlayTrailer(media, location), playFreeOffer: this.createButtonPlayFreeOffer(media, location), restart: this.createButtonRestart(media, location), resume: this.createButtonResume(media, location), watch: this.createButtonWatch(media, location), buy: this.createButtonBuyVideo(media, location), rent: this.createButtonRentVideo(media, location), switchVideoDefinition: this.createButtonSwitchVideoDefinition(media, location), switchLanguage: this.createButtonSwitchLanguage(media, location), seasonDetails: this.createButtonSeasonDetails(media, location), playDisabled: this.createButtonPlayDisabled(location)
                        };
                    if (MS.Entertainment.Utilities.isVideoApp1) {
                        buttons.playOnXbox = this.createButtonPlayOnXbox(media, location);
                        buttons.pinToStartScreen = this.createButtonPinToStartScreen(media, location);
                        buttons.unpinFromStartScreen = this.createButtonUnpinFromStartScreen(media, location);
                        buttons.downloadAction = this.createButtonDownload(media, location);
                        buttons.tryAgain = this.createButtonTryAgain(media, location);
                        buttons.showDownloadManager = this.createButtonDownloadManager(media, location)
                    }
                    else {
                        buttons.ratingPicker = this.createVideoButtonRatingPicker(media, location);
                        buttons.pinToDashboard = this.createVideoButtonPinToDashboard(media, location);
                        buttons.viewInCollection = this.createVideoButtonViewInCollection(media, location)
                    }
                    return buttons
                }, getTVDetailsButtons: function getTVDetailsButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    var buttons = {
                            buySeason: this.createButtonBuySeason(media, location), buyCurrentSeason: this.createButtonBuyCurrentSeason(media, location), playFreeOffer: this.createButtonPlayFreeOffer(media, location)
                        };
                    if (MS.Entertainment.Utilities.isVideoApp1) {
                        buttons.pinToStartScreen = this.createButtonPinToStartScreen(media, location);
                        buttons.unpinFromStartScreen = this.createButtonUnpinFromStartScreen(media, location)
                    }
                    return buttons
                }, getTVSeriesDetailsButtons: function getTVSeriesDetailsButtons(media, executeLocation, tvDetailsModel) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    var buttons = {};
                    if (MS.Entertainment.Utilities.isVideoApp2) {
                        buttons.ratingPicker = this.createVideoButtonRatingPicker(media, location);
                        buttons.pinToDashboard = this.createVideoButtonPinToDashboard(media, location)
                    }
                    var config = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                    if (MS.Entertainment.Utilities.isVideoApp2)
                        buttons.changeSeason = this.createVideoButtonChangeSeason(media, location, tvDetailsModel);
                    return buttons
                }, createVideoButtonRatingPicker: function createVideoButtonRatingPicker(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.actionPassThrough, String.id.IDS_VIDEO2_L3_RATE_BUTTON_VUI_GUI, {
                            mediaItem: media, automationId: MS.Entertainment.UI.AutomationIds.smartButtonRatingPicker, executeLocation: executeLocation, itemTemplate: "/Components/Video/VideoMarketplaceTemplates2.html#ratingPickerWrapperTemplate"
                        });
                    button.dataWinVoiceOverride = {
                        phrase: String.load(String.id.IDS_VIDEO2_L3_RATE_BUTTON_VUI_ALM), pronunciation: String.load(String.id.IDS_VIDEO2_L3_RATE_BUTTON_VUI_PRON), confidence: String.load(String.id.IDS_VIDEO2_L3_RATE_BUTTON_VUI_CONF)
                    };
                    button.onChange = WinJS.Utilities.markSupportedForProcessing(function onChange(event) {
                        if (media && event && event.detail && event.detail.hasOwnProperty("tentativeRating"))
                            media.userRating = event.detail.tentativeRating
                    });
                    button.eventHandlers = {onMessageEvent: function(message){}};
                    return button
                }, createVideoButtonChangeSeason: function createVideoButtonChangeSeason(media, executeLocation, tvDetailsModel) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.changeSeason, String.id.IDS_VIDEO2_TV_CHANGE_SEASON_BUTTON_VUI_GUI, {
                            mediaItem: media, automationId: MS.Entertainment.UI.AutomationIds.smartButtonChangeSeason, executeLocation: executeLocation, itemTemplate: "/Components/Video2/TvDetailsPage.html#changeSeasonButtonTemplate"
                        }, null, {
                            guiStringId: String.id.IDS_VIDEO2_TV_CHANGE_SEASON_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_TV_CHANGE_SEASON_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_TV_CHANGE_SEASON_BUTTON_VUI_PRON, voiceConfidenceStringId: String.id.IDS_VIDEO2_TV_CHANGE_SEASON_BUTTON_VUI_CONF
                        });
                    button.labelOverride = button.title;
                    button.model = tvDetailsModel;
                    return button
                }, createVideoButtonPinToDashboard: function createVideoButtonPinToDashboard(media, executeLocation) {
                    if (!MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.pinnedList))
                        return null;
                    var strings = {
                            pinningTitleStringId: String.id.IDS_VIDEO2_PIN_BUTTON_PINNING, unpinningTitleStringId: String.id.IDS_VIDEO2_PIN_BUTTON_UNPINNING, pinButtonGuiStringId: String.id.IDS_VIDEO2_L3_PIN_TO_HOME_BUTTON_VUI_GUI, pinButtonVoicePhraseStringId: String.id.IDS_VIDEO2_L3_PIN_TO_HOME_BUTTON_VUI_ALM, pinButtonVoicePhoneticPhraseStringId: String.id.IDS_VIDEO2_L3_PIN_TO_HOME_BUTTON_VUI_PRON, pinButtonVoiceConfidenceStringId: String.id.IDS_VIDEO2_L3_PIN_TO_HOME_BUTTON_VUI_CONF, unpinButtonGuiStringId: String.id.IDS_VIDEO2_L3_UNPIN_FROM_HOME_BUTTON_VUI_GUI, unpinButtonVoicePhraseStringId: String.id.IDS_VIDEO2_L3_UNPIN_FROM_HOME_BUTTON_VUI_ALM, unpinButtonVoicePhoneticPhraseStringId: String.id.IDS_VIDEO2_L3_UNPIN_FROM_HOME_BUTTON_VUI_PRON, unpinButtonVoiceConfidenceStringId: String.id.IDS_VIDEO2_L3_UNPIN_FROM_HOME_BUTTON_VUI_CONF
                        };
                    return MS.Entertainment.ViewModels.SmartBuyButtons._createButtonPinToDashsboardInternal(media, executeLocation, strings)
                }, createVideoButtonViewInCollection: function createVideoButtonViewInCollection(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.navigate, String.id.IDS_VIDEO2_L3_BUNDLE_GO_TO_COLLECTION_VUI_GUI, {
                            mediaItem: media, automationId: MS.Entertainment.UI.AutomationIds.smartButtonViewInCollection, executeLocation: executeLocation, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplate", page: MS.Entertainment.UI.Monikers.root
                        }, null, {
                            guiStringId: String.id.IDS_VIDEO2_L3_BUNDLE_GO_TO_COLLECTION_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_L3_BUNDLE_GO_TO_COLLECTION_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_L3_BUNDLE_GO_TO_COLLECTION_VUI_PRON, voiceConfidenceStringId: String.id.IDS_VIDEO2_L3_BUNDLE_GO_TO_COLLECTION_VUI_CONF
                        });
                    return button
                }, createButtonWatch: function createButtonWatch(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_DETAILS_BUY_EPISODE_LABEL, {
                            mediaItem: media, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY, automationId: MS.Entertainment.UI.AutomationIds.smartButtonWatch, executeLocation: executeLocation, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplateWithSubtext"
                        }, {icon: MS.Entertainment.UI.Icon.shop});
                    button.eventHandlers = {onMessageEvent: function(message){}};
                    return button
                }, createButtonBuyVideo: function createButtonBuyVideo(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_BUY_BUTTON_VIDEO, {
                            mediaItem: media, offer: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY, automationId: MS.Entertainment.UI.AutomationIds.smartButtonBuyVideo, executeLocation: executeLocation, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplateWithSubtext"
                        }, {
                            icon: MS.Entertainment.UI.Icon.shop, subTextVisibility: true
                        }, {
                            guiStringId: String.id.IDS_VIDEO2_L3_BUY_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_L3_BUY_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_L3_BUY_BUTTON_VUI_PRON
                        });
                    button.eventHandlers = {onMessageEvent: function(message){}};
                    return button
                }, createButtonBuyEpisode: function createButtonBuyEpisode(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_DETAILS_BUY_EPISODE_POPOVER_LABEL, {
                            mediaItem: media, offer: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY, automationId: MS.Entertainment.UI.AutomationIds.smartButtonBuyVideo, executeLocation: executeLocation, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplateWithSubtext"
                        }, {
                            icon: MS.Entertainment.UI.Icon.shop, subTextVisibility: true
                        }, {
                            guiStringId: String.id.IDS_VIDEO2_TV_BUY_EPISODE_POPOVER_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_TV_BUY_EPISODE_POPOVER_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_TV_BUY_EPISODE_POPOVER_BUTTON_VUI_PRON
                        });
                    button.eventHandlers = {onMessageEvent: function(message){}};
                    return button
                }, createButtonBuyEpisodeInPostRoll: function createButtonBuyEpisode(media) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_BUTTON_VUI_GUI, {
                            mediaItem: media, offer: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY, automationId: MS.Entertainment.UI.AutomationIds.smartButtonBuyVideo, executeLocation: MS.Entertainment.UI.Actions.ExecutionLocation.canvas
                        }, null, {
                            guiStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_BUTTON_VUI_PRON, voiceConfidenceStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_BUTTON_VUI_CONF
                        });
                    button.eventHandlers = {onMessageEvent: function(message){}};
                    return button
                }, createButtonBuyCurrentSeasonInPostRoll: function createButtonBuySeasonInPostRoll(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_VIDEO2_POSTROLL_BUY_THIS_SEASON_BUTTON_VUI_GUI, {
                            mediaItem: media, offer: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY, executeLocation: MS.Entertainment.UI.Actions.ExecutionLocation.canvas, automationId: MS.Entertainment.UI.AutomationIds.smartButtonBuySeason
                        }, null, {
                            guiStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_THIS_SEASON_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_THIS_SEASON_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_THIS_SEASON_BUTTON_VUI_PRON, voiceConfidenceStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_THIS_SEASON_BUTTON_VUI_CONF
                        })
                }, createButtonBuyCurrentSeasonPassInPostRoll: function createButtonBuyCurrentSeasonPassInPostRoll(media) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_VIDEO2_POSTROLL_BUY_THIS_SEASON_PASS_BUTTON_VUI_GUI, {
                            mediaItem: media, offer: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY, executeLocation: MS.Entertainment.UI.Actions.ExecutionLocation.canvas, automationId: MS.Entertainment.UI.AutomationIds.smartButtonBuySeason
                        }, null, {
                            guiStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_THIS_SEASON_PASS_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_THIS_SEASON_PASS_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_THIS_SEASON_PASS_BUTTON_VUI_PRON, voiceConfidenceStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_THIS_SEASON_PASS_BUTTON_VUI_CONF
                        })
                }, createButtonBuyNextSeasonInPostRoll: function createButtonBuyNextSeasonInPostRoll(media) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_SEASON_BUTTON_VUI_GUI, {
                            mediaItem: media, offer: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY, executeLocation: MS.Entertainment.UI.Actions.ExecutionLocation.canvas, automationId: MS.Entertainment.UI.AutomationIds.smartButtonBuySeason
                        }, null, {
                            guiStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_SEASON_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_SEASON_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_SEASON_BUTTON_VUI_PRON, voiceConfidenceStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_SEASON_BUTTON_VUI_CONF
                        })
                }, createButtonBuyNextSeasonPassInPostRoll: function createButtonBuyNextSeasonPassInPostRoll(media) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_SEASON_PASS_BUTTON_VUI_GUI, {
                            mediaItem: media, offer: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY, executeLocation: MS.Entertainment.UI.Actions.ExecutionLocation.canvas, automationId: MS.Entertainment.UI.AutomationIds.smartButtonBuySeason
                        }, null, {
                            guiStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_SEASON_PASS_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_SEASON_PASS_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_SEASON_PASS_BUTTON_VUI_PRON, voiceConfidenceStringId: String.id.IDS_VIDEO2_POSTROLL_BUY_NEXT_SEASON_PASS_BUTTON_VUI_CONF
                        })
                }, createButtonRentVideo: function createButtonRentVideo(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_RENT_BUTTON, {
                            mediaItem: media, offer: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_RENT, automationId: MS.Entertainment.UI.AutomationIds.smartButtonRentVideo, executeLocation: executeLocation, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplateWithSubtext"
                        }, {
                            icon: MS.Entertainment.UI.Icon.shop, subTextVisibility: true
                        }, {
                            guiStringId: String.id.IDS_VIDEO2_L3_RENT_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_L3_RENT_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_L3_RENT_BUTTON_VUI_PRON
                        });
                    button.eventHandlers = {onMessageEvent: function(message){}};
                    return button
                }, createButtonPlayFreeOffer: function createButtonPlayFreeOffer(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playFreeOffer, String.id.IDS_PLAY_BUTTON_VIDEO, {
                            mediaItem: media, offer: null, automationId: MS.Entertainment.UI.AutomationIds.smartBuyPlayFreeVideoOffer, executeLocation: executeLocation, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplateWithSubtext"
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_VIDEO2_L3_PLAY_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_L3_PLAY_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_L3_PLAY_BUTTON_VUI_PRON
                        });
                    button.subTitle = String.load(String.id.IDS_VIDEO_PRICE_FREE);
                    button.eventHandlers = {onMessageEvent: function(message){}};
                    return button
                }, createButtonPlayAndShowNowPlaying: function createButtonPlayAndShowNowPlaying(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, String.id.IDS_DETAILS_PLAY_NOW_LABEL, {
                            mediaItem: media, showDetails: false, autoPlay: true, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlay, executeLocation: executeLocation, dismissOverlayOnExecute: true, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplate"
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_VIDEO2_L3_PLAY_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_L3_PLAY_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_L3_PLAY_BUTTON_VUI_PRON
                        })
                }, createButtonRestart: function createButtonRestart(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, String.id.IDS_RESTART_BUTTON, {
                            mediaItem: media, showDetails: false, autoPlay: true, startPositionMsec: 1, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonRestart, dismissOverlayOnExecute: true, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplate"
                        }, {icon: MS.Entertainment.UI.Icon.restart}, {
                            guiStringId: String.id.IDS_VIDEO2_L3_RESTART_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_L3_RESTART_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_L3_RESTART_BUTTON_VUI_PRON
                        })
                }, createButtonResume: function createButtonResume(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, String.id.IDS_RESUME_BUTTON, {
                            mediaItem: media, showDetails: false, autoPlay: true, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonResume, dismissOverlayOnExecute: true, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplate"
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_VIDEO2_L3_RESUME_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_L3_RESUME_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_L3_RESUME_BUTTON_VUI_PRON
                        })
                }, createButtonDownloadManager: function createButtonDownloadManager(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.showVideoDownloadManager, String.id.IDS_VIDEO_DOWNLOAD_MANAGER_LINK, {
                            executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonShowDownloadManager
                        }, {icon: WinJS.UI.AppBarIcon.download})
                }, createButtonPauseDownload: function createButtonPauseDownload(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadPause, String.id.IDS_VIDEO_DOWNLOAD_MANAGER_PAUSE, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonDownloadPause
                        }, {icon: MS.Entertainment.UI.Icon.pause});
                    return button
                }, createButtonCancelDownload: function createButtonCancelDownload(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadCancel, String.id.IDS_VIDEO_DOWNLOAD_MANAGER_CANCEL, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonDownloadCancel
                        }, {icon: MS.Entertainment.UI.Icon.close});
                    return button
                }, createButtonResumeDownload: function createButtonResumeDownload(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadResume, String.id.IDS_VIDEO_DOWNLOAD_MANAGER_RESUME, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonDownloadResume
                        }, {icon: MS.Entertainment.UI.Icon.play});
                    return button
                }, createButtonRetryDownload: function createButtonRetryDownload(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadAgain, String.id.IDS_VIDEO_DOWNLOAD_MANAGER_RETRY, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonDownloadRetry
                        }, {icon: MS.Entertainment.UI.Icon.repeat});
                    return button
                }, createButtonPlayTrailer: function createButtonPlayTrailer(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, String.id.IDS_DETAILS_WATCH_TRAILER_LABEL, {
                            mediaItem: media, showDetails: MS.Entertainment.Utilities.isVideoApp1, autoPlay: true, playPreviewOnly: true, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayTrailer, dismissOverlayOnExecute: true, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplate"
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_VIDEO2_MOVIES_PLAY_TRAILER_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_MOVIES_PLAY_TRAILER_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_MOVIES_PLAY_TRAILER_BUTTON_VUI_PRON
                        })
                }, createButtonSwitchVideoDefinition: function createButtonSwitchVideoDefinition(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.switchVideoDefinition, String.id.IDS_VIDEO_PRICE_SWITCH_SD, {
                            mediaItem: media, executeLocation: executeLocation, initialized: false, automationId: MS.Entertainment.UI.AutomationIds.smartButtonSwitchVideoDefinition, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplate"
                        }, {icon: MS.Entertainment.UI.Icon.switchDefinition})
                }, createButtonSwitchLanguage: function createButtonSwitchLanguage(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.switchLanguage, String.id.IDS_VIDEO_PURCHASE_OPTIONS_LANGUAGE, {
                            mediaItem: media, executeLocation: executeLocation, initialized: false, automationId: MS.Entertainment.UI.AutomationIds.smartButtonSwitchLanguage, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplateWithSubtext"
                        }, {
                            icon: MS.Entertainment.UI.Icon.switchLanguage, subTextVisibility: true
                        })
                }, createButtonMovieDetails: function createButtonMovieDetails(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, String.id.IDS_DETAILS_VIEW_FULL_MOVIE, {
                            mediaItem: media, showDetails: true, autoPlay: false, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonMovieDetails, dismissOverlayOnExecute: true
                        }, {icon: MS.Entertainment.UI.Icon.details});
                    return button
                }, createButtonSeriesDetails: function createButtonSeriesDetails(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, String.id.IDS_DETAILS_VIEW_FULL_SERIES, {
                            mediaItem: media, showDetails: true, autoPlay: false, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonSeriesDetails, dismissOverlayOnExecute: true
                        }, {icon: WinJS.UI.AppBarIcon.viewall})
                }, createButtonSeasonDetails: function createButtonSeasonDetails(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, String.id.IDS_DETAILS_VIEW_FULL_SERIES, {
                            mediaItem: media, showDetails: true, autoPlay: false, hub: "seasons", executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonSeasonDetails, dismissOverlayOnExecute: true
                        }, {icon: WinJS.UI.AppBarIcon.viewall})
                }, createButtonBuySeason: function createButtonBuySeason(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_DETAILS_BUY_SEASON_LABEL, {
                            mediaItem: media, offer: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonBuySeason, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplateWithSubtext"
                        }, {
                            icon: MS.Entertainment.UI.Icon.shop, subTextVisibility: true
                        }, {
                            guiStringId: String.id.IDS_VIDEO2_TV_BUY_SEASON_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_TV_BUY_SEASON_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_TV_BUY_SEASON_BUTTON_VUI_PRON
                        })
                }, createButtonBuyCurrentSeason: function createButtonBuyCurrentSeason(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_DETAILS_BUY_SEASON_CURRENT_LABEL, {
                            mediaItem: media, offer: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, purchaseType: MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonBuyCurrentSeason, itemTemplate: "/Components/InlineDetails/ActionButtonsControl.html#actionButtonTemplateWithSubtext"
                        }, {
                            icon: MS.Entertainment.UI.Icon.shop, subTextVisibility: true
                        }, {
                            guiStringId: String.id.IDS_VIDEO2_TV_BUY_SEASON_PASS_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_TV_BUY_SEASON_PASS_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_TV_BUY_SEASON_PASS_BUTTON_VUI_PRON
                        })
                }, getVideoInlineDetailsButtons: function getVideoInlineDetailsButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    var buttons = {
                            play: this.createButtonPlayAndShowNowPlaying(media, location), playFreeOffer: this.createButtonPlayFreeOffer(media, location), restart: this.createButtonRestart(media, location), resume: this.createButtonResume(media, location), watch: this.createButtonWatch(media, location), buy: this.createButtonBuyVideo(media, location), buyEpisode: this.createButtonBuyEpisode(media, location), seriesDetails: this.createButtonSeriesDetails(media, location), rent: this.createButtonRentVideo(media, location), movieDetails: this.createButtonMovieDetails(media, location), playTrailer: this.createButtonPlayTrailer(media, location), deleteMedia: this.createButtonDeleteMedia(media, location), playDisabled: this.createButtonPlayDisabled(location), switchVideoDefinition: this.createButtonSwitchVideoDefinition(media, location), switchLanguage: this.createButtonSwitchLanguage(media, location)
                        };
                    if (MS.Entertainment.Utilities.isVideoApp1) {
                        buttons.playOnXbox = this.createButtonPlayOnXbox(media, location);
                        buttons.pinToStartScreen = this.createButtonPinToStartScreen(media, location);
                        buttons.unpinFromStartScreen = this.createButtonUnpinFromStartScreen(media, location);
                        buttons.downloadAction = this.createButtonDownload(media, location);
                        buttons.tryAgain = this.createButtonTryAgain(media, location);
                        buttons.showDownloadManager = this.createButtonDownloadManager(media, location)
                    }
                    return buttons
                }, getTVSeasonInlineDetailsButtons: function getTVSeasonInlineDetailsButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    return {
                            pinToStartScreen: this.createButtonPinToStartScreen(media, location), unpinFromStartScreen: this.createButtonUnpinFromStartScreen(media, location), playFreeOffer: this.createButtonPlayFreeOffer(media, location), buySeason: this.createButtonBuySeason(media, location), buyCurrentSeason: this.createButtonBuyCurrentSeason(media, location), seriesDetails: this.createButtonSeriesDetails(media, location)
                        }
                }, getTVSeriesInlineDetailsButtons: function getTVSeriesInlineDetailsButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    return {
                            pinToStartScreen: this.createButtonPinToStartScreen(media, location), unpinFromStartScreen: this.createButtonUnpinFromStartScreen(media, location), seasonDetails: this.createButtonSeasonDetails(media, location), seriesDetails: this.createButtonSeriesDetails(media, location)
                        }
                }, getVideoImmersiveDetailsHeroButtons: function getVideoImmersiveDetailsHeroButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.canvas;
                    return {playTrailer: this.createButtonPlayTrailer(media, location)}
                }, getVideoDownloadItemButtons: function getVideoDownloadItemButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    return {
                            pauseDownload: this.createButtonPauseDownload(media, location), resumeDownload: this.createButtonResumeDownload(media, location), cancelDownload: this.createButtonCancelDownload(media, location), retryDownload: this.createButtonRetryDownload(media, location)
                        }
                }
        }});
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {SmartBuyStateHandlers: {
            _canXboxAccessMedia: function _canXboxAccessMedia(media, rights) {
                if (!MS.Entertainment.Platform.PlaybackHelpers.isPlayToXboxFeatureEnabled(media))
                    return false;
                else if (!media || !media.hasServiceId)
                    return false;
                else if ((!media.rights || media.rights.length === 0) && media.hasCanonicalId)
                    return true;
                return MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(media, MS.Entertainment.ViewModels.SmartBuyStateHandlers.Tuner.Xbox360, rights)
            }, _mediaHasAnyRight: function _mediaHasAnyRight(media, tuner, rights, freeOnly, excludeFree, allowPartialRights, sortFunction, includeExpired, useUnfilteredRights) {
                    var possibleRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._getMatchingRights(media, tuner, rights, freeOnly, excludeFree, allowPartialRights, sortFunction, includeExpired, useUnfilteredRights);
                    return (possibleRights && possibleRights.length > 0)
                }, mediaHasAnyRight: function mediaHasAnyRight(media, tuner, rights, freeOnly, excludeFree) {
                    return MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(media, tuner, rights, freeOnly, excludeFree)
                }, getMatchingRights: function getMatchingRights(media, tuner, rights, freeOnly, excludeFree, allowPartialRights, sortFunction, includeExpired, useUnfilteredRights) {
                    return MS.Entertainment.ViewModels.SmartBuyStateHandlers._getMatchingRights(media, tuner, rights, freeOnly, excludeFree, allowPartialRights, sortFunction, includeExpired, useUnfilteredRights)
                }, _getMatchingRights: function _getMatchingRights(media, tuner, rights, freeOnly, excludeFree, allowPartialRights, sortFunction, includeExpired, useUnfilteredRights) {
                    if (!media || !media.rights)
                        return null;
                    var possibleRights = [];
                    var rightsToCheck = useUnfilteredRights ? media.unfilteredRights : media.rights;
                    var nowDate = Date.now();
                    if (!MS.Entertainment.Utilities.clientSupportsFreeJustPlays)
                        excludeFree = false;
                    for (var i = 0; i < rightsToCheck.length; i++) {
                        var right = rightsToCheck[i];
                        if (!right)
                            continue;
                        if (!MS.Entertainment.Utilities.rightSupportsTuner(right, tuner))
                            continue;
                        if (!rights || rights.length === 0) {
                            possibleRights.push(right);
                            continue
                        }
                        var foundRight = false;
                        for (var j = 0; j < rights.length; j++)
                            if (right.licenseRight === rights[j] && (MS.Entertainment.ViewModels.SmartBuyStateHandlers._isValidOffer(right, media.mediaType) || (allowPartialRights && !right.isFullRight) || right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Preview || right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.PreviewStream || right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.FreeStream || right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Subscription || right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Stream)) {
                                foundRight = true;
                                break
                            }
                        if (!foundRight)
                            continue;
                        if (right.hasOffer && right.startDate && right.endDate)
                            try {
                                if (!includeExpired) {
                                    var startDate = Date.parse(right.startDate);
                                    if (isNaN(startDate))
                                        startDate = parseInt(right.startDate.split("(")[1].split(")")[0]);
                                    var endDate = Date.parse(right.endDate);
                                    if (isNaN(endDate))
                                        endDate = parseInt(right.endDate.split("(")[1].split(")")[0]);
                                    if (startDate > nowDate || nowDate > endDate)
                                        continue
                                }
                            }
                            catch(e) {
                                MS.Entertainment.Data.fail("Invalid start or end date specified on media, service id = " + media.serviceId, null, MS.Entertainment.UI.Debug.errorLevel.low)
                            }
                        if (freeOnly && (!right.hasOffer || !MS.Entertainment.Utilities.isRightFree(right) || right.priceCurrencyCode === "MTK"))
                            continue;
                        if (excludeFree && (!right.hasOffer || MS.Entertainment.Utilities.isRightFree(right)))
                            continue;
                        if (MS.Entertainment.ViewModels.SmartBuyStateHandlers._filterRight(right, media))
                            continue;
                        possibleRights.push(right)
                    }
                    return possibleRights
                }, _filterRight: function _filterRight(right, media) {
                    return (MS.Entertainment.Utilities.isVideoApp2 && right.deliveryFormat === MS.Entertainment.Data.Augmenter.Marketplace.edsDeliveryFormat.download && media.mediaType !== Microsoft.Entertainment.Queries.ObjectType.tvSeason) || (MS.Entertainment.Utilities.isVideoApp1 && !right.signedOffer)
                }, _isValidOffer: function _isValidOffer(right, mediaType) {
                    var isValidOffer = right && right.offerId;
                    var isTrackAlbumPurchaseOffer = mediaType === Microsoft.Entertainment.Queries.ObjectType.track && right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.AlbumPurchase;
                    if (isValidOffer && !isTrackAlbumPurchaseOffer && (mediaType === Microsoft.Entertainment.Queries.ObjectType.track || mediaType === Microsoft.Entertainment.Queries.ObjectType.album)) {
                        var paymentTypes = right.paymentInstruments;
                        if (paymentTypes)
                            paymentTypes = Array.isArray(paymentTypes) ? paymentTypes : [paymentTypes];
                        isValidOffer = paymentTypes && paymentTypes.length && paymentTypes.some(function doesOfferSupportPurchaseType(paymentType) {
                            return (paymentType === MS.Entertainment.Data.Augmenter.Marketplace.edsPaymentType.creditCard)
                        })
                    }
                    return isValidOffer
                }, Tuner: {
                    All: MS.Entertainment.Data.Augmenter.Marketplace.edsDeviceType.all, Xbox360: MS.Entertainment.Data.Augmenter.Marketplace.edsDeviceType.xbox360, Windows: MS.Entertainment.Data.Augmenter.Marketplace.edsDeviceType.pc, Zune30: MS.Entertainment.Data.Augmenter.Marketplace.edsDeviceType.zuneDevice, ZuneMobile: MS.Entertainment.Data.Augmenter.Marketplace.edsDeviceType.zuneMobile
                }, MarketplaceRight: {
                    Rent: "Rent", RentStream: "RentStream", Purchase: "Purchase", PurchaseStream: "PurchaseStream", AlbumPurchase: "AlbumPurchase", Preview: "Preview", PreviewStream: "PreviewStream", Download: "Download", FreeStream: "FreeStream", SeasonPurchase: "SeasonPurchase", SeasonPurchaseStream: "SeasonPurchaseStream", Stream: "Stream", Subscription: "Subscription", SubscriptionFree: "SubscriptionFree", TransferToPortableDevice: "TransferToPortableDevice", Trial: "Trial"
                }, PaymentTypes: {
                    creditCard: "Credit Card", points: "Microsoft Points", mobileOperator: "Mobile Operator"
                }
        }})
})()
