﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    var MSE = WinJS.Namespace.define("MS.Entertainment", null);
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.ViewModels");
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {SmartBuyStateEngine: MS.Entertainment.defineOptionalObservable(function SmartBuyStateEngineConstructor(initialButtons) {
            this.currentButtons = initialButtons || [];
            this.currentAppbarActions = [];
            this._currentLocationSpecificButtons = {};
            this._eventProvider = new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell
        }, {
            buttons: null, appbarActions: null, suspendStateChanges: false, options: null, _eventProxy: null, _eventProvider: null, _mediaBindings: null, _mediaDeletedBindingMethod: null, _onStateChanged: null, _uniqueListenerId: null, _mediaRightChangedBindingMethod: null, _updateStateBindingMethod: null, _bindingsComplete: false, _cachedState: null, _lastUpdateStatePromise: null, _lastFireStateChangedPromise: null, _defaultDownloadInfo: null, _currentButtons: null, _currentLocationSpecificButtons: null, _unloaded: false, currentButtons: {
                    get: function() {
                        return this._currentButtons
                    }, set: function(value) {
                            if (value !== this._currentButtons) {
                                var oldValue = this._currentButtons;
                                this._currentButtons = value;
                                if (this.media)
                                    this._eventProvider.traceMediaStateEngine_ButtonsChanged(this.media.mediaType, this.media.serviceId, this.media.libraryId);
                                this.notify("currentButtons", value, oldValue);
                                this._fireCurrentButtonsChanged()
                            }
                        }
                }, eventProxy: {
                    get: function() {
                        return this._eventProxy
                    }, set: function(value) {
                            if (this._eventProxy !== value) {
                                this._eventProxy = value;
                                if (this.currentButtons && this.currentButtons.length)
                                    this._fireCurrentButtonsChanged()
                            }
                        }
                }, initialize: function initialize(media, buttons, onStateChanged, options) {
                    if (this.media)
                        throw"State engine already initialized.";
                    else if (!media)
                        throw"A valid media object is required.";
                    this.options = options || {};
                    this.media = media;
                    this._onStateChanged = onStateChanged;
                    this.buttons = buttons;
                    switch (media.mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.video:
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeries:
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeason:
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                        case Microsoft.Entertainment.Queries.ObjectType.playlist:
                            if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.fileTransfer)) {
                                this._uniqueListenerId = "SmartButtonListener_" + MS.Entertainment.Utilities.getSessionUniqueInteger();
                                var notifier = {
                                        add: this._onFileTransferAdd, pending: this._onFileTransferEvent, update: this._onFileTransferEvent, complete: this._onFileTransferEvent, error: this._onFileTransferEvent, cancel: this._onFileTransferEvent
                                    };
                                var fileTransferService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.fileTransfer);
                                fileTransferService.registerListener(this._uniqueListenerId, MS.Entertainment.Framework.DownloadInfo.getMarketplaceDownloadTaskKeyFunction(media.mediaType), null, notifier, this)
                            }
                            break
                    }
                    this._updateStateBindingMethod = this.updateState.bind(this);
                    this._mediaBindings = WinJS.Binding.bind(this.media, {
                        serviceId: this._updateStateBindingMethod, canonicalId: this._updateStateBindingMethod, isPinnedToStartScreen: this._updateStateBindingMethod, features: this._updateStateBindingMethod, trailerUrl: this._updateStateBindingMethod, defaultPlatformType: this._updateStateBindingMethod, libraryId: this._updateStateBindingMethod, rights: this._updateStateBindingMethod, rightsHydrated: this._updateStateBindingMethod, tracks: this._updateStateBindingMethod, episode: this._updateStateBindingMethod, episodes: this._updateStateBindingMethod, hydrated: this._updateStateBindingMethod, isFreeStreamRestrictedByMonthlyLimit: this._updateStateBindingMethod, isFreeStreamRestrictedByIndividualLimit: this._updateStateBindingMethod
                    });
                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn).bind("isSignedIn", this._updateStateBindingMethod);
                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser).bind("isSubscription", this._updateStateBindingMethod);
                    if (MS.Entertainment.Platform.PlaybackHelpers.isMusicTrackOrMusicVideo(this.media))
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).bind("primarySessionId", this._updateStateBindingMethod);
                    if (this.options.updateOnAppVisibilityChange)
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).bind("isAppVisible", this._updateStateBindingMethod);
                    if (this.options.updateOnAppSnappedModeChange)
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).bind("isSnapped", this._updateStateBindingMethod);
                    if (this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.video) {
                        var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                        var playbackSession = sessionMgr.nowPlayingSession;
                        playbackSession.bind("currentTransportState", this._updateStateBindingMethod);
                        var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                        uiStateService.bind("nowPlayingVisible", this._updateStateBindingMethod);
                        if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.mediaDeleted)) {
                            var deleteService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.mediaDeleted);
                            this._mediaDeletedBindingMethod = MS.Entertainment.Utilities.addEventHandlers(deleteService, {mediaDeleted: this._updateStateBindingMethod})
                        }
                    }
                    else if (this.media.mediaType !== undefined) {
                        var sessionManager = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                        this._smartDJBindings = WinJS.Binding.bind(sessionManager.primarySession, {smartDJSeed: this._updateStateBindingMethod})
                    }
                    if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.videoDetailsUiState)) {
                        var videoDetailsUiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.videoDetailsUiState);
                        videoDetailsUiStateService.addEventListener("downloadQualityChange", this._updateStateBindingMethod, false);
                        videoDetailsUiStateService.addEventListener("videoDefinitionChange", this._updateStateBindingMethod, false);
                        videoDetailsUiStateService.addEventListener("videoLanguageChange", this._updateStateBindingMethod, false)
                    }
                    try {
                        this._mediaRightChangedBindingMethod = this._mediaRightChanged.bind(this);
                        Microsoft.Entertainment.Marketplace.Marketplace.addEventListener("mediarightchanged", this._mediaRightChangedBindingMethod)
                    }
                    catch(e) {
                        var message = (e && e.message) || e;
                        var errorCode = e && e.number;
                        MS.Entertainment.ViewModels.fail("Microsoft.Entertainment.Marketplace.Marketplace.addEventListener failed with error:" + errorCode + "; message:" + message);
                        this._mediaRightChangedBindingMethod = null
                    }
                    this._eventProvider.traceMediaStateEngine_Initialized(this.media.mediaType, this.media.serviceId, this.media.libraryId);
                    this.bind("currentAppbarActions", function appbarActionsChanged() {
                        if (!this._bindingsComplete)
                            return;
                        this._eventProvider.traceMediaStateEngine_ActionsChanged(this.media.mediaType, this.media.serviceId, this.media.libraryId)
                    }.bind(this));
                    this._bindingsComplete = true;
                    if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.pinToStartScreen)) {
                        var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                        pinToStartScreenService.getTileForMediaAsync(this.media).then(function getTileForMediaAsync_complete(startScreenTile) {
                            this.media.isPinnedToStartScreen = !!startScreenTile
                        }.bind(this))
                    }
                    this.updateState();
                    if (this.options.updateImmediately)
                        this._fireStateChanged()
                }, unload: function unload() {
                    this._unloaded = true;
                    if (this._mediaRightChangedBindingMethod) {
                        Microsoft.Entertainment.Marketplace.Marketplace.removeEventListener("mediarightchanged", this._mediaRightChangedBindingMethod);
                        this._mediaRightChangedBindingMethod = null
                    }
                    if (this._updateStateBindingMethod) {
                        if (this._mediaBindings) {
                            this._mediaBindings.cancel();
                            this._mediaBindings = null
                        }
                        if (this._mediaDeletedBindingMethod) {
                            this._mediaDeletedBindingMethod.cancel();
                            this._mediaDeletedBindingMethod = null
                        }
                        if (this._smartDJBindings) {
                            this._smartDJBindings.cancel();
                            this._smartDJBindings = null
                        }
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn).unbind("isSignedIn", this._updateStateBindingMethod);
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser).unbind("isSubscription", this._updateStateBindingMethod);
                        if (MS.Entertainment.Platform.PlaybackHelpers.isMusicTrackOrMusicVideo(this.media))
                            MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).unbind("primarySessionId", this._updateStateBindingMethod);
                        if (this.options.updateOnAppVisibilityChange)
                            MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).unbind("isAppVisible", this._updateStateBindingMethod);
                        if (this.options.updateOnAppSnappedModeChange)
                            MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).unbind("isSnapped", this._updateStateBindingMethod);
                        if (this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.video) {
                            var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                            var playbackSession = sessionMgr.nowPlayingSession;
                            playbackSession.unbind("currentTransportState", this._updateStateBindingMethod);
                            var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                            uiStateService.unbind("nowPlayingVisible", this._updateStateBindingMethod)
                        }
                        if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.videoDetailsUiState)) {
                            var videoDetailsUiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.videoDetailsUiState);
                            videoDetailsUiStateService.removeEventListener("downloadQualityChange", this._updateStateBindingMethod, false);
                            videoDetailsUiStateService.removeEventListener("videoDefinitionChange", this._updateStateBindingMethod, false);
                            videoDetailsUiStateService.removeEventListener("videoLanguageChange", this._updateStateBindingMethod, false)
                        }
                        this._updateStateBindingMethod = null
                    }
                    if (this._uniqueListenerId) {
                        var fileTransferService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.fileTransfer);
                        fileTransferService.unregisterListener(this._uniqueListenerId);
                        this._uniqueListenerId = null
                    }
                    this.currentButtons = null;
                    this.currentAppbarActions = [];
                    this._clearLocationSpecificButtons();
                    this._currentLocationSpecificButtons = null;
                    this._eventProvider.traceMediaStateEngine_Unloaded(this.media && this.media.mediaType, this.media && this.media.serviceId, this.media && this.media.libraryId)
                }, getCurrentButtonsForLocation: function getCurrentButtonsForLocation(location) {
                    if (!this._currentLocationSpecificButtons[location])
                        this._currentLocationSpecificButtons[location] = new MS.Entertainment.ObservableArray;
                    return this._currentLocationSpecificButtons[location]
                }, _setLocationSpecificButtons: function _setLocationSpecificButtons(newLocationSpecificButtons) {
                    for (var newLocationKey in newLocationSpecificButtons) {
                        var buttonsForLocation = this.getCurrentButtonsForLocation(newLocationKey);
                        var newButtonsForLocation = newLocationSpecificButtons[newLocationKey];
                        if (!newButtonsForLocation)
                            continue;
                        var itemsChanged = newButtonsForLocation.length !== buttonsForLocation.length || newButtonsForLocation.some(function checkForButtonEquality(item, newLocationKey) {
                                return item !== buttonsForLocation.item(newLocationKey)
                            });
                        if (itemsChanged)
                            buttonsForLocation.spliceArray(0, buttonsForLocation.length, newButtonsForLocation)
                    }
                    for (var existingLocationKey in this._currentLocationSpecificButtons)
                        if (!newLocationSpecificButtons || !newLocationSpecificButtons[existingLocationKey])
                            if (this._currentLocationSpecificButtons[existingLocationKey])
                                this._currentLocationSpecificButtons[existingLocationKey].clear()
                }, _clearLocationSpecificButtons: function _clearLocationSpecificButtons() {
                    for (var existingLocationKey in this._currentLocationSpecificButtons)
                        this._currentLocationSpecificButtons[existingLocationKey] = null
                }, updateMediaItemProperties: function updateMediaItemProperties(updatedMedia) {
                    MS.Entertainment.ViewModels.assert(this.media && this.media.isEqual(updatedMedia), "Cannot update the engine's media item with properties from a different media item.");
                    MS.Entertainment.Utilities.copyAugmentedProperties(updatedMedia, this.media);
                    this.updateState()
                }, updateState: function updateState() {
                    if (!this.media || !this._bindingsComplete || this._unloaded || this.suspendStateChanges)
                        return;
                    this._eventProvider.traceMediaStateEngine_UpdateState(this.media.mediaType, this.media.serviceId, this.media.libraryId);
                    var updateStatePromise = function queryMediaStateAsync_complete(state) {
                            if (this._lastUpdateStatePromise === updateStatePromise && !this._unloaded) {
                                this._defaultDownloadInfo = null;
                                this._lastUpdateStatePromise = null;
                                this._cachedState = state;
                                this._fireStateChanged()
                            }
                        }.bind(this);
                    this._lastUpdateStatePromise = updateStatePromise;
                    return MS.Entertainment.ViewModels.SmartBuyStateEngine.queryMediaStateAsync(this.media, this._defaultDownloadInfo ? this._defaultDownloadInfo : (this._cachedState ? this._cachedState.download : null)).then(updateStatePromise)
                }, _fireCurrentButtonsChanged: function _fireCurrentButtonsChanged() {
                    if (this.eventProxy && this.eventProxy.dispatchEvent)
                        this.eventProxy.dispatchEvent("currentButtonsChanged", {
                            sender: this, value: this.currentButtons
                        })
                }, _fireStateChanged: function _fireStateChanged() {
                    var stateChangedPromise = WinJS.Promise.wrap();
                    if (this._onStateChanged)
                        if (this.options.invokeHandlerAsStatic)
                            stateChangedPromise = this._onStateChanged(this, this._cachedState) || stateChangedPromise;
                        else
                            stateChangedPromise = this._onStateChanged(this._cachedState) || stateChangedPromise;
                    var fireStateChangedPromise = function _onStateChanged_complete(requestButtons) {
                            if (this._unloaded)
                                return;
                            var subActionsChanged;
                            var newButtons = (requestButtons && requestButtons.smartButtons) || requestButtons;
                            var newActions = requestButtons && requestButtons.appbarActions;
                            var newLocationSpecificButtons = requestButtons && requestButtons.locationSpecificButtons;
                            if (this._lastFireStateChangedPromise === fireStateChangedPromise) {
                                if (!newButtons)
                                    this.currentButtons = [];
                                else if (newButtons.length !== this.currentButtons.length)
                                    this.currentButtons = newButtons;
                                else
                                    for (var i = 0; i < newButtons.length; i++)
                                        if (newButtons[i] !== this.currentButtons[i]) {
                                            this.currentButtons = newButtons;
                                            break
                                        }
                                subActionsChanged = !MS.Entertainment.UI.Actions.ActionArray.areSubActionsEqual(newActions, this.currentAppbarActions);
                                if (!newActions)
                                    this.currentAppbarActions = [];
                                else if (newActions.length !== this.currentAppbarActions.length || subActionsChanged)
                                    this.currentAppbarActions = newActions;
                                else
                                    for (var i = 0; i < newActions.length; i++)
                                        if (newActions[i] !== this.currentAppbarActions[i]) {
                                            this.currentAppbarActions = newActions;
                                            break
                                        }
                                this._setLocationSpecificButtons(newLocationSpecificButtons);
                                if (requestButtons && this.loading)
                                    this.loading = false
                            }
                        }.bind(this);
                    this._lastFireStateChangedPromise = fireStateChangedPromise;
                    stateChangedPromise.then(fireStateChangedPromise)
                }, _mediaRightChanged: function _mediaRightChanged(serviceMediaId) {
                    if (this._unloaded)
                        return;
                    MS.Entertainment.ViewModels.SmartBuyStateEngine.mediaContainsServiceMediaIdAsync(this.media, serviceMediaId).then(function mediaContainsServiceMediaIdAsync_complete(contained) {
                        if (contained)
                            this.updateState()
                    }.bind(this))
                }, _onFileTransferAdd: function _onFileTransferAdd(notification) {
                    if (this._unloaded)
                        return;
                    if (notification.transferType !== MS.Entertainment.UI.FileTransferType.download || !MS.Entertainment.Framework.DownloadInfo.isFileTransferTaskContainedByMedia(this.media, notification.task))
                        return;
                    if (notification.transferType === MS.Entertainment.UI.FileTransferType.download)
                        MS.Entertainment.ViewModels.MediaItemModel.hydrateLibraryInfoAsync(this.media).done(function hydrateLibraryInfoAsyncComplete() {
                            this._onFileTransferEvent(notification);
                            this.updateState()
                        }.bind(this), function hydrateLibraryInfoAsyncError(error) {
                            MS.Entertainment.ViewModels.assert(error.message === "Canceled", "Failed to get library id in the onFileTransferAdd callback. Error message: " + error.message);
                            this._onFileTransferEvent(notification)
                        }.bind(this))
                }, _onFileTransferEvent: function _onFileTransferEvent(notification) {
                    if (this._unloaded)
                        return;
                    if (notification.transferType !== MS.Entertainment.UI.FileTransferType.download || !MS.Entertainment.Framework.DownloadInfo.isFileTransferTaskContainedByMedia(this.media, notification.task))
                        return;
                    if (this._cachedState && this._cachedState.download.update(notification.task))
                        if (notification.task.taskStatus !== Microsoft.Entertainment.FileTransferStatus.completed && !notification.task.isClosed)
                            this._fireStateChanged();
                        else
                            this.updateState()
                }
        }, {
            currentAppbarActions: null, media: null, loading: true
        }, {
            FileContainerAccess: {
                None: "None", Partial: "Partial", All: "All"
            }, getSelectionStateFromMediaItem: function getSelectionStateFromMediaItem(mediaItem) {
                    var handler = null;
                    var selectedItemStateEngine = null;
                    switch (mediaItem && mediaItem.mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.playlist:
                            handler = MS.Entertainment.ViewModels.MusicStateHandlers.onMusicGalleryPlaylistSelectionStateChanged;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                            handler = MS.Entertainment.ViewModels.MusicStateHandlers.onMusicGallerySelectionStateChanged;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.video:
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeries:
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeason:
                        case Microsoft.Entertainment.Queries.ObjectType.folder:
                            selectedItemStateEngine = new MS.Entertainment.ViewModels.VideoSmartBuyStateEngine;
                            handler = MS.Entertainment.ViewModels.VideoSmartBuyStateEngine.prototype.onVideoGallerySelectionStateChanged;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.smartDJ:
                            handler = MS.Entertainment.ViewModels.MusicStateHandlers.onMusicGallerySmartDJSelectionStateChanged;
                            break;
                        case undefined:
                            break;
                        default:
                            MS.Entertainment.UI.Controls.fail("Unsupported media type: " + mediaItem && mediaItem.mediaType);
                            break
                    }
                    if (handler) {
                        if (!selectedItemStateEngine)
                            selectedItemStateEngine = new MS.Entertainment.ViewModels.SmartBuyStateEngine;
                        selectedItemStateEngine.initialize(mediaItem, null, handler)
                    }
                    return selectedItemStateEngine
                }, getMediaServiceIdsInCollectionAsync: function getMediaServiceIdsInCollectionAsync(media, collectionPath) {
                    var promise;
                    var collection = MS.Entertainment.Utilities.valueFromPropertyPath(media, collectionPath);
                    if (!collection || !collection.forEachAll || !media)
                        promise = WinJS.Promise.wrap([]);
                    else if (media.knownChildServiceIds)
                        promise = WinJS.Promise.wrap(media.knownChildServiceIds);
                    else {
                        var knownChildServiceIds = [];
                        promise = collection.forEachAll(function iteration(args) {
                            if (args.item.data.serviceId)
                                knownChildServiceIds.push(args.item.data.serviceId.toLowerCase())
                        }).then(function loadedIds() {
                            if (!media.knownChildServiceIds)
                                media.knownChildServiceIds = knownChildServiceIds;
                            knownChildServiceIds = null;
                            return media.knownChildServiceIds
                        }, function loadedIdsFailed() {
                            knownChildServiceIds = null;
                            return []
                        })
                    }
                    return promise
                }, findMediaServiceIdsInCollectionAsync: function findMediaServiceIdsInCollectionAsync(media, collectionPath, serviceId) {
                    var collection = MS.Entertainment.Utilities.valueFromPropertyPath(media, collectionPath);
                    if (!collection || !collection.forEachAll || !media)
                        return WinJS.Promise.wrap(false);
                    var promise;
                    var found = false;
                    serviceId = serviceId.toLowerCase();
                    if (media.knownChildServiceIds && media.knownChildServiceIds.length)
                        promise = WinJS.Promise.wrap(media.knownChildServiceIds.indexOf(serviceId) >= 0);
                    else
                        promise = collection.forEachAll(function iteration(args) {
                            var currentServiceId = args.item.data.serviceId;
                            if (args.item.data.serviceId && args.item.data.serviceId.toLowerCase() === serviceId) {
                                found = true;
                                args.stop = true
                            }
                        }).then(function finishedSearch() {
                            return found
                        }, function failedSearch() {
                            return false
                        });
                    return promise
                }, mediaContainsServiceMediaIdAsync: function mediaContainsServiceMediaIdAsync(media, serviceMediaId) {
                    if (!media || !serviceMediaId || MS.Entertainment.Utilities.isEmptyGuid(serviceMediaId))
                        return WinJS.Promise.wrap(false);
                    return MS.Entertainment.ViewModels.SmartBuyStateEngine.findMediaServiceIdsAsync(media, serviceMediaId)
                }, determineFileContainerAccess: function _determineFileContainerAccess(max, actual) {
                    if (actual === 0)
                        return MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.None;
                    else if (actual < max)
                        return MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.Partial;
                    else
                        return MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.All
                }, queryMediaStateAsync: function queryMediaStateAsync(media, downloadInfo) {
                    var promises = [];
                    var state = {};
                    state.marketplace = {};
                    state.collection = {};
                    promises.push(MS.Entertainment.ViewModels.SmartBuyStateEngine.getPlayabilityAsync(media).then(function getPlayabilityAsync_complete(playability) {
                        state.collection.playability = playability;
                        state.collection.canPlayLocally = playability.local;
                        state.collection.canPlayLocallyOrStreamFromCloud = playability.local || playability.cloud;
                        state.collection.canStreamFromCloud = playability.cloud
                    }));
                    if (MS.Entertainment.Platform.PlaybackHelpers.isMusicVideo(media) && media.track)
                        promises.push(media.track.hydrate().then(function onHydrate(track) {
                            return MS.Entertainment.ViewModels.MediaItemModel.hydrateLibraryInfoAsync(track)
                        }).then(function hydrateLibraryInfo_complete(track) {
                            state.track = {inCollection: track.inCollection}
                        }));
                    if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.video || media.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeries || media.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeason || media.mediaType === Microsoft.Entertainment.Queries.ObjectType.track || media.mediaType === Microsoft.Entertainment.Queries.ObjectType.album || media.mediaType === Microsoft.Entertainment.Queries.ObjectType.playlist)
                        promises.push(MS.Entertainment.ViewModels.SmartBuyStateEngine.getMarketplaceFileAccessAsync(media).then(function getMarketplaceFileAccessAsync_complete(marketplace) {
                            state.marketplace.mediaIds = marketplace.mediaIds;
                            state.marketplace.counts = marketplace.counts;
                            state.marketplace.canDownloadState = marketplace.canDownload;
                            state.marketplace.canDownload = (marketplace.canDownload !== MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.None);
                            state.marketplace.canStreamState = marketplace.canStream;
                            state.marketplace.canStream = (marketplace.canStream !== MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.None);
                            state.marketplace.exclusivelyRentedState = marketplace.exclusivelyRented;
                            state.marketplace.exclusivelyRented = (marketplace.exclusivelyRented === MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.All);
                            state.marketplace.isPresale = marketplace.isPresale;
                            state.marketplace.presaleFulfillmentDate = marketplace.presaleFulfillmentDate;
                            state.marketplace.rentalExpirations = marketplace.rentalExpirations;
                            state.marketplace.entitlements = marketplace.entitlements;
                            state.marketplace.hasPurchased = marketplace.hasPurchased;
                            state.marketplace.hasPurchasedSeason = marketplace.hasPurchasedSeason;
                            state.marketplace.libraryDefinitions = marketplace.libraryDefinitions
                        }));
                    if (downloadInfo)
                        state.download = downloadInfo;
                    else if (MS.Entertainment.Framework.DownloadInfo)
                        promises.push(MS.Entertainment.Framework.DownloadInfo.createAndInitializeAsync(media).then(function createAndInitializeAsync_complete(downloadInfo) {
                            state.download = downloadInfo
                        }));
                    else
                        state.download = {items: []};
                    return WinJS.Promise.join(promises).then(function queryMediaStateAsync_join_complete() {
                            state.canPlay = (state.marketplace && state.marketplace.canStream) || state.collection.canPlayLocallyOrStreamFromCloud;
                            return WinJS.Promise.wrap(state)
                        })
                }, getMediaServiceIdsAsync: function getMediaServiceIdsAsync(media) {
                    if (!media || !media.mediaType)
                        return WinJS.Promise.wrap([]);
                    switch (media.mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.video:
                            if (media.hasZuneId)
                                return WinJS.Promise.wrap([media.zuneId]);
                            else if (media.hasServiceId && media.serviceIdType === "ZuneCatalog")
                                return WinJS.Promise.wrap([media.serviceId]);
                            else
                                return WinJS.Promise.wrap([]);
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                            return WinJS.Promise.wrap(media.hasServiceId ? [media.serviceId] : []);
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                            return MS.Entertainment.ViewModels.SmartBuyStateEngine.getArtistTrackServiceIdsInCollectionAsync(media);
                        case Microsoft.Entertainment.Queries.ObjectType.playlist:
                            return MS.Entertainment.ViewModels.SmartBuyStateEngine.getPlaylistTrackServiceIdsInCollectionAsync(media);
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                            return MS.Entertainment.ViewModels.SmartBuyStateEngine.getMediaServiceIdsInCollectionAsync(media, "tracks");
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeries:
                            return WinJS.Promise.wrap([]);
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeason:
                            return MS.Entertainment.ViewModels.SmartBuyStateEngine.getMediaServiceIdsInCollectionAsync(media, "episodes");
                        case Microsoft.Entertainment.Queries.ObjectType.game:
                        case Microsoft.Entertainment.Queries.ObjectType.smartDJ:
                        case Microsoft.Entertainment.Queries.ObjectType.folder:
                            return WinJS.Promise.wrap([]);
                        default:
                            throw"The state engine does not support this media item type.";
                    }
                }, getArtistTrackServiceIdsInCollectionAsync: function getArtistTrackServiceIdsInCollectionAsync(mediaItem) {
                    if (!mediaItem || !mediaItem.libraryId)
                        return WinJS.Promise.wrap([]);
                    var query = new MS.Entertainment.Data.Query.libraryTracks;
                    query.artistId = mediaItem.libraryId;
                    return query.execute().then(function artistQuery_executed(q) {
                            return MS.Entertainment.ViewModels.SmartBuyStateEngine.getMediaServiceIdsInCollectionAsync(q.result, "items")
                        }, function artistQuery_failed() {
                            return WinJS.Promise.wrap([])
                        })
                }, getPlaylistTrackServiceIdsInCollectionAsync: function getPlaylistTrackServiceIdsInCollectionAsync(mediaItem) {
                    if (!mediaItem || !MS.Entertainment.Utilities.isValidLibraryId(mediaItem.libraryId))
                        return WinJS.Promise.as([]);
                    return MS.Entertainment.ViewModels.SmartBuyStateEngine.getMediaServiceIdsInCollectionAsync(mediaItem, "tracks").then(function onMediaServiceIdsRetrieved(mediaServiceIds) {
                            if (mediaServiceIds && mediaServiceIds.length > 0)
                                return mediaServiceIds;
                            else {
                                var query = new MS.Entertainment.Data.Query.libraryPlaylistMediaItems;
                                query.playlistId = mediaItem.libraryId;
                                return query.execute().then(function playlistQuery_executed(q) {
                                        return MS.Entertainment.ViewModels.SmartBuyStateEngine.getMediaServiceIdsInCollectionAsync(q.result, "items")
                                    }, function playlistQuery_failed() {
                                        return WinJS.Promise.as([])
                                    })
                            }
                        })
                }, findMediaServiceIdsAsync: function findMediaServiceIdsAsync(media, serviceId) {
                    if (!media || !media.mediaType || !serviceId)
                        return WinJS.Promise.wrap(false);
                    var promise;
                    switch (media.mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.video:
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                            promise = WinJS.Promise.wrap(!!media.serviceId && media.serviceId.toLowerCase() === serviceId.toLowerCase());
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                        case Microsoft.Entertainment.Queries.ObjectType.playlist:
                            promise = MS.Entertainment.ViewModels.SmartBuyStateEngine.findMediaServiceIdsInCollectionAsync(media, "tracks", serviceId);
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeason:
                            promise = MS.Entertainment.ViewModels.SmartBuyStateEngine.findMediaServiceIdsInCollectionAsync(media, "episodes", serviceId);
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeries:
                        case Microsoft.Entertainment.Queries.ObjectType.game:
                        case Microsoft.Entertainment.Queries.ObjectType.smartDJ:
                        case Microsoft.Entertainment.Queries.ObjectType.folder:
                            promise = WinJS.Promise.wrap(false);
                            break;
                        default:
                            throw"The state engine does not support this media item type.";
                    }
                    return promise
                }, _createPlayability: function _createPlayability(nativePlayabilityArray) {
                    var playability = {getItem: function getItem(media) {
                                if (!media || !this.items)
                                    return null;
                                var libraryId = media.inCollection ? media.libraryId : -1;
                                var serviceId = media.inCollection ? null : (media.hasServiceId ? media.serviceId.toLowerCase() : null);
                                if (libraryId === -1 && !serviceId)
                                    return null;
                                for (var i = 0; i < this.items.length; i++) {
                                    var item = this.items[i];
                                    if (libraryId !== -1) {
                                        if (item.libraryId === libraryId)
                                            return item
                                    }
                                    else if (serviceId)
                                        if (item.mediaId === serviceId)
                                            return item
                                }
                                return null
                            }};
                    playability.counts = {};
                    playability.counts.localOnly = 0;
                    playability.counts.localAndCloud = 0;
                    playability.counts.cloudOnly = 0;
                    playability.counts.neitherLocalNorCloud = 0;
                    playability.counts.localOrCloud = 0;
                    playability.counts.downloading = 0;
                    playability.items = [];
                    if (nativePlayabilityArray)
                        for (var i = 0; i < nativePlayabilityArray.length; i++) {
                            var item = nativePlayabilityArray[i];
                            if (item) {
                                playability.items.push({
                                    mediaType: item.mediaType, libraryId: item.libraryId, mediaId: item.mediaId, locallyPlayable: item.locallyPlayable, cloudStreamable: item.cloudStreamable, isDownloading: item.isDownloading
                                });
                                if (item.locallyPlayable)
                                    if (item.cloudStreamable)
                                        ++playability.counts.localAndCloud;
                                    else
                                        ++playability.counts.localOnly;
                                else if (item.cloudStreamable)
                                    ++playability.counts.cloudOnly;
                                else
                                    ++playability.counts.neitherLocalNorCloud;
                                if (item.isDownloading)
                                    ++playability.counts.downloading
                            }
                        }
                    playability.counts.local = playability.counts.localOnly + playability.counts.localAndCloud;
                    playability.counts.cloud = playability.counts.cloudOnly + playability.counts.localAndCloud;
                    playability.counts.localOrCloud = playability.counts.cloudOnly + playability.counts.localOnly + playability.counts.localAndCloud;
                    playability.local = (playability.counts.local > 0);
                    playability.cloud = (playability.counts.cloud > 0);
                    return playability
                }, getPlayabilityAsync: function getPlayabilityAsync(media, collectionFilter) {
                    if (!media || !media.inCollection || !media.mediaType)
                        return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(null));
                    collectionFilter = collectionFilter || Microsoft.Entertainment.Platform.MediaAvailability.undefined;
                    var mediaStore = new Microsoft.Entertainment.Platform.MediaStore;
                    switch (media.mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.video:
                            return mediaStore.videoProvider.getPlayabilityByLibraryIdAsync(media.libraryId).then(function getPlayabilityByLibraryIdAsync_complete(nativePlayability) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability([nativePlayability]))
                                }, function getPlayabilityByLibraryIdAsync_error(e) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(null))
                                });
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                            return mediaStore.trackProvider.getPlayabilityByLibraryIdAsync(media.libraryId).then(function getPlayabilityByLibraryIdAsync_complete(nativePlayability) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability([nativePlayability]))
                                }, function getPlayabilityByLibraryIdAsync_error(e) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(null))
                                });
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                            return mediaStore.albumProvider.getPlayabilityByLibraryIdAsync(media.libraryId, collectionFilter).then(function getPlayabilityByLibraryIdAsync_complete(nativePlayabilityArray) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(nativePlayabilityArray))
                                }, function getPlayabilityByLibraryIdAsync_error(e) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(null))
                                });
                        case Microsoft.Entertainment.Queries.ObjectType.playlist:
                            return mediaStore.playlistProvider.getPlayabilityByLibraryIdAsync(media.libraryId, collectionFilter).then(function getPlayabilityByLibraryIdAsync_complete(nativePlayabilityArray) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(nativePlayabilityArray))
                                }, function getPlayabilityByLibraryIdAsync_error(e) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(null))
                                });
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeries:
                            return mediaStore.seriesProvider.getPlayabilityByLibraryIdAsync(media.libraryId).then(function getPlayabilityByLibraryIdAsync_complete(nativePlayabilityArray) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(nativePlayabilityArray))
                                }, function getPlayabilityByLibraryIdAsync_error(e) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(null))
                                });
                        case Microsoft.Entertainment.Queries.ObjectType.tvSeason:
                            return mediaStore.seriesProvider.getPlayabilityBySeasonLibraryIdAsync(media.libraryId).then(function getPlayabilityBySeasonLibraryIdAsync_complete(nativePlayabilityArray) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(nativePlayabilityArray))
                                }, function getPlayabilityBySeasonLibraryIdAsync_error(e) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(null))
                                });
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                            return mediaStore.artistProvider.getPlayabilityByLibraryIdAsync(media.libraryId, collectionFilter).then(function getPlayabilityByLibraryIdAsync_complete(nativePlayabilityArray) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(nativePlayabilityArray))
                                }, function getPlayabilityByLibraryIdAsync_error(e) {
                                    return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(null))
                                });
                        case Microsoft.Entertainment.Queries.ObjectType.game:
                        case Microsoft.Entertainment.Queries.ObjectType.folder:
                            return WinJS.Promise.wrap(MS.Entertainment.ViewModels.SmartBuyStateEngine._createPlayability(null));
                        default:
                            throw"The state engine does not support this media item type.";
                    }
                }, getMarketplaceFileAccessAsync: function getMarketplaceFileAccessAsync(media) {
                    return MS.Entertainment.ViewModels.SmartBuyStateEngine.getMediaServiceIdsAsync(media).then(function getMediaServiceIdsAsync_complete(serviceIds) {
                            if (!serviceIds || !serviceIds.length)
                                return WinJS.Promise.wrap({
                                        mediaIds: [], isPresale: false, presaleFulfillmentDate: null, entitlements: [], canDownload: MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.None, canStream: MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.None, exclusivelyRented: MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.None, rentalExpirations: [], counts: {
                                                canDownload: 0, canStream: 0, exclusivelyRented: 0
                                            }
                                    });
                            return Microsoft.Entertainment.Marketplace.Marketplace.getMediaEntitlementsAsync(serviceIds).then(function getMediaEntitlementsAsync_complete(result) {
                                    result = JSON.parse(result).result;
                                    var parsedResult = {
                                            mediaIds: serviceIds, entitlements: result.entitlements, isPresale: result.entitlements.some(function isPresale_some(entitlement) {
                                                    return entitlement.isPresale
                                                }), presaleFulfillmentDate: null, canDownload: MS.Entertainment.ViewModels.SmartBuyStateEngine.determineFileContainerAccess(serviceIds.length, result.counts.canFullyDownload), canStream: MS.Entertainment.ViewModels.SmartBuyStateEngine.determineFileContainerAccess(serviceIds.length, result.counts.canFullyStream), exclusivelyRented: MS.Entertainment.ViewModels.SmartBuyStateEngine.determineFileContainerAccess(serviceIds.length, result.counts.exclusivelyRented), rentalExpirations: [], hasPurchased: result.entitlements.reduce(function hasPurchase_reduce(previousValue, currentValue) {
                                                    if (currentValue.hasPurchase)
                                                        previousValue += 1;
                                                    return previousValue
                                                }, 0), hasPurchasedSeason: result.entitlements.some(function hasPurchasedSeason_some(entitlement) {
                                                    return entitlement.hasPurchasedSeason
                                                }), counts: {
                                                    canDownload: result.counts.canFullyDownload, canStream: result.counts.canFullyStream, exclusivelyRented: result.counts.exclusivelyRented
                                                }
                                        };
                                    parsedResult.rentalExpirations.overall = {
                                        isExpired: false, latestDate: null
                                    };
                                    var hasExpiredRental = false;
                                    var hasNonExpiredRental = false;
                                    var hasInfiniteExpiration = false;
                                    function gatherOverallRentalExpiration(rental) {
                                        rental.expiration = MSE.Data.Factory.date(rental.expiration);
                                        if (rental.serviceMediaInstanceId && !MS.Entertainment.Utilities.isEmptyGuid(rental.serviceMediaInstanceId))
                                            if (rental.isExpired)
                                                hasExpiredRental = true;
                                            else {
                                                hasNonExpiredRental = true;
                                                if (!rental.expiration) {
                                                    hasInfiniteExpiration = true;
                                                    parsedResult.rentalExpirations.overall.latestDate = null
                                                }
                                                else if (!hasInfiniteExpiration && rental.expiration > parsedResult.rentalExpirations.overall.latestDate)
                                                    parsedResult.rentalExpirations.overall.latestDate = rental.expiration
                                            }
                                    }
                                    var edsResolutionFormat = MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat;
                                    parsedResult.libraryDefinitions = {};
                                    parsedResult.libraryDefinitions[edsResolutionFormat.xd] = false;
                                    parsedResult.libraryDefinitions[edsResolutionFormat.sd] = false;
                                    parsedResult.libraryDefinitions[edsResolutionFormat.hd] = false;
                                    parsedResult.libraryDefinitions[edsResolutionFormat.hd1080p] = false;
                                    function gatherAvailableVideoQuality(mediaEntitlement) {
                                        if (mediaEntitlement.serviceMediaInstanceId && !MS.Entertainment.Utilities.isEmptyGuid(mediaEntitlement.serviceMediaInstanceId))
                                            if (!mediaEntitlement.isExpired && mediaEntitlement.isAvailable && mediaEntitlement.format)
                                                parsedResult.libraryDefinitions[mediaEntitlement.format] = true
                                    }
                                    function gatherFulfillmentDate(entitlement) {
                                        if (entitlement.isPresale)
                                            if (entitlement.presaleFulfillmentDate && (!parsedResult.presaleFulfillmentDate || entitlement.presaleFulfillmentDate > parsedResult.presaleFulfillmentDate))
                                                parsedResult.presaleFulfillmentDate = entitlement.presaleFulfillmentDate
                                    }
                                    if (MS.Entertainment.Utilities.isVideoApp) {
                                        for (var i = 0; i < serviceIds.length; i++) {
                                            gatherOverallRentalExpiration(result.entitlements[i].rent.download);
                                            gatherOverallRentalExpiration(result.entitlements[i].rent.stream);
                                            gatherAvailableVideoQuality(result.entitlements[i].purchase.download);
                                            gatherAvailableVideoQuality(result.entitlements[i].purchase.stream);
                                            gatherAvailableVideoQuality(result.entitlements[i].rent.download);
                                            gatherAvailableVideoQuality(result.entitlements[i].rent.stream);
                                            gatherFulfillmentDate(result.entitlements[i]);
                                            parsedResult.rentalExpirations.push({
                                                download: {
                                                    date: result.entitlements[i].rent.download.expiration, isExpired: !!result.entitlements[i].rent.download.isExpired
                                                }, stream: {
                                                        date: result.entitlements[i].rent.stream.expiration, isExpired: !!result.entitlements[i].rent.stream.isExpired
                                                    }
                                            })
                                        }
                                        parsedResult.rentalExpirations.overall.isExpired = hasExpiredRental && !hasNonExpiredRental
                                    }
                                    return WinJS.Promise.wrap(parsedResult)
                                }, function getMediaEntitlements_error(e) {
                                    return WinJS.Promise.wrap({
                                            mediaIds: serviceIds, entitlements: [], isPresale: false, presaleFulfillmentDate: null, canDownload: MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.None, canStream: MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.None, exclusivelyRented: MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.None, rentalExpirations: [], counts: {
                                                    canDownload: 0, canStream: 0, exclusivelyRented: 0
                                                }
                                        })
                                })
                        })
                }
        })})
})()
