﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
var __extends = this.__extends || function(d, b) {
        for (var p in b)
            if (b.hasOwnProperty(p))
                d[p] = b[p];
        function __() {
            this.constructor = d
        }
        __.prototype = b.prototype;
        d.prototype = new __
    };
var MS;
(function(MS) {
    (function(Entertainment) {
        (function(ViewModels) {
            var Augmenter = MS.Entertainment.Data.Augmenter;
            var Marketplace = MS.Entertainment.Data.Augmenter.Marketplace;
            var EDSOfferRight = MS.Entertainment.Data.Augmenter.Marketplace.EDSOfferRight;
            var VideoSmartBuyStateEngine = (function(_super) {
                    __extends(VideoSmartBuyStateEngine, _super);
                    function VideoSmartBuyStateEngine() {
                        _super.apply(this, arguments)
                    }
                    VideoSmartBuyStateEngine.prototype.onPostRollEpisodeChanged = function(stateInfo) {
                        var buttons = [];
                        var actions = [];
                        if (this.media.hasServiceId && !this.media.hydrated) {
                            if (!this.media.hydrated && this.media.hydrate)
                                this.media.hydrate();
                            return WinJS.Promise.wrap(null)
                        }
                        var videoDefinitionPreference = this.options.videoDefinitionPreference || MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.hd;
                        var isOwned = (stateInfo.marketplace.canStream && !stateInfo.marketplace.exclusivelyRented);
                        var buyButtonOffer = VideoSmartBuyStateEngine.getDefaultOffer(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Purchase, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.PurchaseStream], false, false, null, videoDefinitionPreference);
                        if (buyButtonOffer && !isOwned) {
                            this.buttons.buyNextEpisode.subTitle = VideoSmartBuyStateEngine.getPriceString(buyButtonOffer);
                            this.buttons.buyNextEpisode.parameter.offer = buyButtonOffer;
                            buttons.push(this.buttons.buyNextEpisode)
                        }
                        return WinJS.Promise.wrap({smartButtons: buttons})
                    };
                    VideoSmartBuyStateEngine.prototype.onPostRollSeasonChanged = function(stateInfo) {
                        var _this = this;
                        var returnPromise = null;
                        var buttons = [];
                        var actions = [];
                        var isCurrentSeason = this.options.isCurrentSeason;
                        if (!this.media.hydrated) {
                            if (!this.media.hydrated && this.media.hydrate)
                                this.media.hydrate();
                            return WinJS.Promise.wrap(null)
                        }
                        var firstEpisodePromise;
                        var firstEpisode = this.media.firstEpisode;
                        if (firstEpisode && !this.media.inCollection && !firstEpisode.hydrated && firstEpisode.hydrate)
                            firstEpisodePromise = firstEpisode.hydrate();
                        return WinJS.Promise.as(firstEpisodePromise).then(function() {
                                var entireSeasonIsOwned = stateInfo.marketplace.hasPurchasedSeason || (_this.media.isComplete && stateInfo.marketplace.canStreamState === MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.All && stateInfo.marketplace.exclusivelyRentedState === MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.None);
                                var videoDefinitionPreference = _this.options.videoDefinitionPreference || MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.hd;
                                var seasonResolutions = VideoSmartBuyStateEngine.getSeasonResolutionsForRights(_this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp);
                                if (seasonResolutions && !seasonResolutions.hasHdRight)
                                    videoDefinitionPreference = MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.sd;
                                else if (seasonResolutions && !seasonResolutions.hasSdRight)
                                    videoDefinitionPreference = MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.hd;
                                var buyButtonOffer = VideoSmartBuyStateEngine.getDefaultSeasonBuyOffer(_this.media, videoDefinitionPreference, String.empty);
                                if (!entireSeasonIsOwned && buyButtonOffer) {
                                    var buySeasonButton = null;
                                    if (isCurrentSeason)
                                        if (_this.media.isComplete)
                                            buySeasonButton = _this.buttons.buyCurrentSeason;
                                        else
                                            buySeasonButton = _this.buttons.buyCurrentSeasonPass;
                                    else if (_this.media.isComplete)
                                        buySeasonButton = _this.buttons.buyNextSeason;
                                    else
                                        buySeasonButton = _this.buttons.buyNextSeasonPass;
                                    buySeasonButton.subTitle = VideoSmartBuyStateEngine.getPriceString(buyButtonOffer, videoDefinitionPreference);
                                    buySeasonButton.parameter.offer = buyButtonOffer;
                                    var ownedEpisodesCount = _this.media.libraryEpisodes.count;
                                    if (ownedEpisodesCount)
                                        buySeasonButton.metadata = String.load(String.id.IDS_VIDEO2_TV_SEASON_EPISODES_OWNED).format(ownedEpisodesCount);
                                    else
                                        buySeasonButton.metadata = String.empty;
                                    buttons.push(buySeasonButton)
                                }
                                return {
                                        smartButtons: buttons, appbarActions: null
                                    }
                            })
                    };
                    VideoSmartBuyStateEngine.prototype.onVideoDownloadItemStateChanged = function(stateInfo) {
                        var buttons = [];
                        this._updateFileTransferNotification(stateInfo);
                        if (stateInfo.download.hasPaused) {
                            buttons.push(this.buttons.resumeDownload);
                            buttons.push(this.buttons.cancelDownload)
                        }
                        else if (stateInfo.download.hasRetryable) {
                            buttons.push(this.buttons.retryDownload);
                            buttons.push(this.buttons.cancelDownload)
                        }
                        else if (stateInfo.download.hasFailed)
                            buttons.push(this.buttons.cancelDownload);
                        else if (stateInfo.download.hasActive || stateInfo.download.hasPending) {
                            buttons.push(this.buttons.pauseDownload);
                            buttons.push(this.buttons.cancelDownload)
                        }
                        return WinJS.Promise.wrap({
                                smartButtons: buttons, appbarActions: []
                            })
                    };
                    VideoSmartBuyStateEngine.prototype.onVideoInlineDetailsStateChanged = function(stateInfo) {
                        var _this = this;
                        var buttons = [];
                        var actions = [];
                        var returnPromise = null;
                        if (this.media.hasServiceId && !this.media.hydrated) {
                            if (!this.media.hydrated && this.media.hydrate)
                                this.media.hydrate();
                            return WinJS.Promise.wrap(null)
                        }
                        var appbarActions = MS.Entertainment.ViewModels.videoAppBarActionTable;
                        var isTVEpisode = MS.Entertainment.Platform.PlaybackHelpers.isTVEpisode(this.media);
                        var canPlayLocally = stateInfo.collection.canPlayLocally;
                        var canPlayTrailer = VideoSmartBuyStateEngine.canPlayVideoTrailer(this.media);
                        var canPlayOnXbox = VideoSmartBuyStateEngine.canPlayOnXbox(this.media);
                        var inVideoApp = (MS.Entertainment.Utilities.isVideoApp);
                        var isOwned = (stateInfo.marketplace.canStream && !stateInfo.marketplace.exclusivelyRented);
                        var getOnXbox = inVideoApp && canPlayOnXbox && !isOwned;
                        var showDownloadButton = (stateInfo.marketplace.counts.canDownload > (stateInfo.collection.playability.counts.local + stateInfo.download.items.length)) && MS.Entertainment.Utilities.isApp1;
                        var showDownloadManagerButton = (stateInfo.download.items.length > 0) && MS.Entertainment.Utilities.isApp1;
                        var showDeleteMediaButtonInPopover = (this.media && this.media.videoType === Microsoft.Entertainment.Queries.VideoType.tvEpisode);
                        if (inVideoApp && canPlayOnXbox)
                            canPlayOnXbox = isOwned;
                        this._updateOfflineAvailabilityNotification(stateInfo);
                        this._updateFileTransferNotification(stateInfo);
                        if (stateInfo.download.hasFailed) {
                            if (stateInfo.marketplace.canStream)
                                buttons.push(this.buttons.play);
                            else
                                buttons.push(this.buttons.playDisabled);
                            var isVideoMarketplaceNetworkEnabled = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.networkStatus).isEnabled(1);
                            if (!isTVEpisode && isVideoMarketplaceNetworkEnabled)
                                buttons.push(this.buttons.movieDetails);
                            if (canPlayOnXbox)
                                buttons.push(this.buttons.playOnXbox);
                            if (stateInfo.download.hasRetryable) {
                                if (stateInfo.marketplace.libraryDefinitions[MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd] || stateInfo.marketplace.libraryDefinitions[MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd1080p])
                                    this.buttons.tryAgain.title = String.load(MS.Entertainment.Utilities.getTryAgainDynamicText());
                                else
                                    this.buttons.tryAgain.title = String.load(String.id.IDS_DETAILS_TRY_AGAIN_LABEL);
                                buttons.push(this.buttons.tryAgain)
                            }
                        }
                        else if ((!stateInfo.marketplace.exclusivelyRented || (stateInfo.marketplace.rentalExpirations.overall && !stateInfo.marketplace.rentalExpirations.overall.isExpired)) && (stateInfo.download.hasAny || stateInfo.canPlay || stateInfo.marketplace.canDownload)) {
                            if (stateInfo.canPlay)
                                if (this.media.inCollection) {
                                    var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                                    var playbackSession = sessionMgr.getSession(MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying);
                                    var playIndex = buttons.length;
                                    returnPromise = this._getPlayButtonStates(this.media).then(function(result) {
                                        var playButtons = [];
                                        if (result.showPlay)
                                            playButtons.push(_this.buttons.play);
                                        if (result.showResume)
                                            playButtons.push(_this.buttons.resume);
                                        if (result.showRestart)
                                            playButtons.push(_this.buttons.restart);
                                        buttons.spliceArray(playIndex, 0, playButtons);
                                        buttons = VideoSmartBuyStateEngine.cutButtonsIfNecessary(buttons, _this.buttons);
                                        return {
                                                smartButtons: buttons, appbarActions: actions
                                            }
                                    })
                                }
                                else
                                    buttons.push(this.buttons.play);
                            else
                                buttons.push(this.buttons.playDisabled);
                            if (this.media.videoType === Microsoft.Entertainment.Queries.VideoType.tvEpisode || this.media.videoType === Microsoft.Entertainment.Queries.VideoType.movie) {
                                var isVideoMarketplaceNetworkEnabled = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.networkStatus).isEnabled(1);
                                if (!isTVEpisode && isVideoMarketplaceNetworkEnabled)
                                    buttons.push(this.buttons.movieDetails)
                            }
                            if (canPlayOnXbox)
                                buttons.push(this.buttons.playOnXbox);
                            if (showDownloadButton) {
                                var appBarService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar);
                                var downloadAction = this.buttons.downloadAction.title ? this.buttons.downloadAction : appBarService.getToolbarAction(appbarActions.downloadAction);
                                if (stateInfo.marketplace.libraryDefinitions[MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd] || stateInfo.marketplace.libraryDefinitions[MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd1080p])
                                    if (stateInfo.marketplace.exclusivelyRented)
                                        downloadAction.title = String.load(String.id.IDS_DETAILS_DOWNLOAD_LABEL_HD);
                                    else
                                        downloadAction.title = String.load(MS.Entertainment.Utilities.getDownloadDynamicText());
                                else
                                    downloadAction.title = String.load(String.id.IDS_DETAILS_DOWNLOAD_LABEL_SD);
                                buttons.push(this.buttons.downloadAction)
                            }
                            else if (showDownloadManagerButton)
                                buttons.push(this.buttons.showDownloadManager)
                        }
                        else {
                            if (this.media.type !== MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.MovieTrailer) {
                                var canChangeLanguage = false;
                                if (!this.buttons.switchLanguage.videoLanguagesAvailable || this.buttons.switchLanguage.videoLanguagesAvailable.length === 0)
                                    this.buttons.switchLanguage.setVideoLanguages(VideoSmartBuyStateEngine.getLanguagesForRights(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp));
                                if (this.buttons.switchLanguage.videoLanguagesAvailable && this.buttons.switchLanguage.videoLanguagesAvailable.length > 1) {
                                    this.buttons.switchLanguage.updateTitleForLanguageAndCount();
                                    canChangeLanguage = true
                                }
                                var bestFreeRight = VideoSmartBuyStateEngine.getBestFreeRight(this.media, false);
                                var buyButtonOffer = bestFreeRight ? null : VideoSmartBuyStateEngine.getDefaultBuyOffer(this.media, false, this.buttons.switchVideoDefinition.videoDefinitionPreference, this.buttons.switchLanguage.videoLanguagePreference);
                                var rentalButtonStreamingOffer = bestFreeRight ? null : VideoSmartBuyStateEngine.getDefaultRentStreamOffer(this.media, this.buttons.switchVideoDefinition.videoDefinitionPreference, this.buttons.switchLanguage.videoLanguagePreference);
                                var rentalButtonDownloadOffer = (MS.Entertainment.Utilities.isVideoApp2 || bestFreeRight) ? null : VideoSmartBuyStateEngine.getDefaultRentDownloadOffer(this.media, this.buttons.switchVideoDefinition.videoDefinitionPreference, this.buttons.switchLanguage.videoLanguagePreference);
                                if (bestFreeRight) {
                                    this.buttons.playFreeOffer.freeOffer = bestFreeRight;
                                    this.buttons.playFreeOffer.freeOfferId = bestFreeRight.offerId;
                                    this.buttons.playFreeOffer.freeOfferCurrencyCode = bestFreeRight.priceCurrencyCode;
                                    this.buttons.playFreeOffer.freeOfferPurchaseType = MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY;
                                    this.buttons.playFreeOffer.freeOfferSignedOffer = bestFreeRight.signedOffer;
                                    if (bestFreeRight.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Rent || bestFreeRight.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.RentStream)
                                        this.buttons.playFreeOffer.freeOfferPurchaseType = MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_RENT;
                                    this.buttons.playFreeOffer.startPlayback = true;
                                    this.buttons.playFreeOffer.title = String.load(!isTVEpisode ? String.id.IDS_MOVIE_PLAY_MOVIE_BUTTON : String.id.IDS_TV_PLAY_EPISODE_BUTTON);
                                    buttons.push(this.buttons.playFreeOffer)
                                }
                                if (buyButtonOffer)
                                    if (isTVEpisode) {
                                        this.buttons.buyEpisode.subTitle = VideoSmartBuyStateEngine.getPriceString(buyButtonOffer);
                                        this.buttons.buyEpisode.parameter.offer = buyButtonOffer;
                                        buttons.push(this.buttons.buyEpisode)
                                    }
                                    else {
                                        this.buttons.buy.subTitle = VideoSmartBuyStateEngine.getPriceString(buyButtonOffer);
                                        this.buttons.buy.parameter.offer = buyButtonOffer;
                                        buttons.push(this.buttons.buy)
                                    }
                                if (rentalButtonStreamingOffer || rentalButtonDownloadOffer) {
                                    this.buttons.rent.subTitle = VideoSmartBuyStateEngine.getPriceString(rentalButtonStreamingOffer || rentalButtonDownloadOffer);
                                    if (!rentalButtonStreamingOffer)
                                        this.buttons.rent.parameter.offer = rentalButtonDownloadOffer;
                                    else if (!rentalButtonDownloadOffer)
                                        this.buttons.rent.parameter.offer = rentalButtonStreamingOffer;
                                    else {
                                        this.buttons.rent.parameter.streamingOffer = rentalButtonStreamingOffer;
                                        this.buttons.rent.parameter.downloadOffer = rentalButtonDownloadOffer
                                    }
                                }
                                var resolutions = VideoSmartBuyStateEngine.getResolutionsForRights(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp);
                                if (resolutions.hasHdAndSdRights) {
                                    this.buttons.switchVideoDefinition.updateTitleForMediaResolution();
                                    buttons.push(this.buttons.switchVideoDefinition)
                                }
                                if (canChangeLanguage)
                                    buttons.push(this.buttons.switchLanguage)
                            }
                            var isVideoMarketplaceNetworkEnabled = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.networkStatus).isEnabled(1);
                            if (!isTVEpisode && isVideoMarketplaceNetworkEnabled)
                                buttons.push(this.buttons.movieDetails);
                            if (canPlayTrailer && buttons.length < 4)
                                buttons.push(this.buttons.playTrailer);
                            if (canPlayOnXbox && buttons.length < 4)
                                buttons.push(this.buttons.playOnXbox)
                        }
                        if (getOnXbox)
                            actions.push(appbarActions.getOnXbox);
                        if (this.media.isRemovable && (canPlayLocally || !isTVEpisode))
                            if (showDeleteMediaButtonInPopover)
                                buttons.push(this.buttons.deleteMedia);
                            else
                                actions.push(appbarActions.deleteMedia);
                        buttons = VideoSmartBuyStateEngine.cutButtonsIfNecessary(buttons, this.buttons);
                        if (returnPromise)
                            return returnPromise;
                        return WinJS.Promise.wrap({
                                smartButtons: buttons, appbarActions: actions
                            })
                    };
                    VideoSmartBuyStateEngine.prototype.onWatchNextStateChanged = function(stateInfo) {
                        var _this = this;
                        var buttons = [];
                        var actions = [];
                        var returnPromise = null;
                        if (this.media.hasServiceId && !this.media.hydrated) {
                            if (!this.media.hydrated && this.media.hydrate)
                                this.media.hydrate();
                            return WinJS.Promise.wrap(null)
                        }
                        var appbarActions = MS.Entertainment.ViewModels.videoAppBarActionTable;
                        var canPlayLocally = stateInfo.collection.canPlayLocally;
                        var canPlayTrailer = VideoSmartBuyStateEngine.canPlayVideoTrailer(this.media);
                        var canPlayOnXbox = VideoSmartBuyStateEngine.canPlayOnXbox(this.media);
                        var inVideoApp = (MS.Entertainment.Utilities.isVideoApp);
                        var isOwned = (stateInfo.marketplace.canStream && !stateInfo.marketplace.exclusivelyRented);
                        var getOnXbox = inVideoApp && canPlayOnXbox && !isOwned;
                        var showDownloadButton = (stateInfo.marketplace.counts.canDownload > (stateInfo.collection.playability.counts.local + stateInfo.download.items.length)) && MS.Entertainment.Utilities.isApp1;
                        var showDownloadManagerButton = (stateInfo.download.items.length > 0) && MS.Entertainment.Utilities.isApp1;
                        if (inVideoApp && canPlayOnXbox)
                            canPlayOnXbox = isOwned;
                        this._updateOfflineAvailabilityNotification(stateInfo);
                        this._updateFileTransferNotification(stateInfo);
                        if (this.media && this.media.episodeNumber) {
                            var updateTitle = function(button, stringId, voiceInfo) {
                                    if (button)
                                        if (voiceInfo && button.isVoiceEnabled) {
                                            if (voiceInfo.guiStringId)
                                                button.title = String.load(voiceInfo.guiStringId).format(_this.media.episodeNumber);
                                            if (voiceInfo.voicePhraseStringId)
                                                button.voicePhrase = String.load(voiceInfo.voicePhraseStringId).format(_this.media.episodeNumber);
                                            if (voiceInfo.voicePhoneticPhraseStringId) {
                                                var phoneticString = String.load(voiceInfo.voicePhoneticPhraseStringId);
                                                var voiceStatics = WinJS.Utilities.getMember("MS.Entertainment.UI.App2.VoiceStatics");
                                                if (phoneticString !== voiceStatics.emptyPronunciationValue)
                                                    button.voicePhoneticPhrase = phoneticString.format(_this.media.episodeNumber)
                                            }
                                            if (typeof(voiceInfo.voiceConfidence) === "number")
                                                button.voiceConfidence = voiceInfo.voiceConfidence
                                        }
                                        else if (stringId)
                                            button.title = String.load(stringId).format(_this.media.episodeNumber)
                                };
                            updateTitle(this.buttons.buyEpisode, String.id.IDS_VIDEO_WATCH_NEXT_BUY_EPISODE_BUTTON, {
                                guiStringId: String.id.IDS_VIDEO2_TV_BUY_EPISODE_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_TV_BUY_EPISODE_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_TV_BUY_EPISODE_BUTTON_VUI_PRON, voiceConfidence: String.id.IDS_VIDEO2_TV_BUY_EPISODE_BUTTON_VUI_CONF
                            });
                            updateTitle(this.buttons.play, String.id.IDS_VIDEO_WATCH_NEXT_PLAY_EPISODE_BUTTON, {
                                guiStringId: String.id.IDS_VIDEO2_TV_PLAY_EPISODE_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_TV_PLAY_EPISODE_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_TV_PLAY_EPISODE_BUTTON_VUI_PRON, voiceConfidence: String.id.IDS_VIDEO2_TV_PLAY_EPISODE_BUTTON_VUI_CONF
                            });
                            updateTitle(this.buttons.playDisabled, String.id.IDS_VIDEO_WATCH_NEXT_PLAY_EPISODE_BUTTON, {
                                guiStringId: String.id.IDS_VIDEO2_TV_PLAY_EPISODE_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_TV_PLAY_EPISODE_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_TV_PLAY_EPISODE_BUTTON_VUI_PRON, voiceConfidence: String.id.IDS_VIDEO2_TV_PLAY_EPISODE_BUTTON_VUI_CONF
                            });
                            updateTitle(this.buttons.resume, String.id.IDS_VIDEO_WATCH_NEXT_RESUME_EPISODE_BUTTON, {
                                guiStringId: String.id.IDS_VIDEO2_TV_RESUME_EPISODE_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_TV_RESUME_EPISODE_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_TV_RESUME_EPISODE_BUTTON_VUI_PRON, voiceConfidence: String.id.IDS_VIDEO2_TV_RESUME_EPISODE_BUTTON_VUI_CONF
                            });
                            updateTitle(this.buttons.restart, String.id.IDS_VIDEO_WATCH_NEXT_RESTART_EPISODE_BUTTON, {
                                guiStringId: String.id.IDS_VIDEO2_TV_RESTART_EPISODE_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_TV_RESTART_EPISODE_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_TV_RESTART_EPISODE_BUTTON_VUI_PRON, voiceConfidence: String.id.IDS_VIDEO2_TV_RESTART_EPISODE_BUTTON_VUI_CONF
                            });
                            updateTitle(this.buttons.playFreeOffer, String.id.IDS_VIDEO_WATCH_NEXT_PLAY_EPISODE_BUTTON, {
                                guiStringId: String.id.IDS_VIDEO2_TV_PLAY_EPISODE_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_VIDEO2_TV_PLAY_EPISODE_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_VIDEO2_TV_PLAY_EPISODE_BUTTON_VUI_PRON, voiceConfidence: String.id.IDS_VIDEO2_TV_PLAY_EPISODE_BUTTON_VUI_CONF
                            })
                        }
                        if (stateInfo.download.hasFailed)
                            if (stateInfo.marketplace.canStream)
                                buttons.push(this.buttons.play);
                            else
                                buttons.push(this.buttons.playDisabled);
                        else if (stateInfo.download.hasAny || stateInfo.canPlay || stateInfo.marketplace.canDownload)
                            if (stateInfo.canPlay)
                                if (this.media.inCollection) {
                                    var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                                    var playbackSession = sessionMgr.getSession(MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying);
                                    var playIndex = buttons.length;
                                    returnPromise = this._getPlayButtonStates(this.media).then(function(result) {
                                        var playButtons = [];
                                        if (result.showPlay)
                                            playButtons.push(_this.buttons.play);
                                        if (result.showResume)
                                            playButtons.push(_this.buttons.resume);
                                        if (result.showRestart && (playButtons.length === 0 || MS.Entertainment.Utilities.isVideoApp1))
                                            playButtons.push(_this.buttons.restart);
                                        buttons.spliceArray(playIndex, 0, playButtons);
                                        return {
                                                smartButtons: buttons, appbarActions: actions
                                            }
                                    })
                                }
                                else
                                    buttons.push(this.buttons.play);
                            else
                                buttons.push(this.buttons.playDisabled);
                        else {
                            var bestFreeRight = VideoSmartBuyStateEngine.getBestFreeRight(this.media, false);
                            var buyButtonOffer = bestFreeRight ? null : VideoSmartBuyStateEngine.getDefaultBuyOffer(this.media, false, this.buttons.switchVideoDefinition.videoDefinitionPreference, this.buttons.switchLanguage.videoLanguagePreference);
                            if (bestFreeRight) {
                                this.buttons.playFreeOffer.freeOffer = bestFreeRight;
                                this.buttons.playFreeOffer.freeOfferId = bestFreeRight.offerId;
                                this.buttons.playFreeOffer.freeOfferCurrencyCode = bestFreeRight.priceCurrencyCode;
                                this.buttons.playFreeOffer.freeOfferPurchaseType = MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY;
                                this.buttons.playFreeOffer.freeOfferSignedOffer = bestFreeRight.signedOffer;
                                this.buttons.playFreeOffer.startPlayback = true;
                                buttons.push(this.buttons.playFreeOffer)
                            }
                            if (buyButtonOffer) {
                                this.buttons.buyEpisode.subTitle = VideoSmartBuyStateEngine.getPriceString(buyButtonOffer);
                                this.buttons.buyEpisode.parameter.offer = buyButtonOffer;
                                buttons.push(this.buttons.buyEpisode)
                            }
                        }
                        if (returnPromise)
                            return returnPromise;
                        return WinJS.Promise.wrap({
                                smartButtons: buttons, appbarActions: actions
                            })
                    };
                    VideoSmartBuyStateEngine.prototype._getPlayButtonStates = function(media) {
                        var ms = new Microsoft.Entertainment.Platform.MediaStore;
                        var pendingBookmarkWriteOperations;
                        if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.bookmarkOperationsWatcher)) {
                            var bookmarkWatcher = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.bookmarkOperationsWatcher);
                            pendingBookmarkWriteOperations = bookmarkWatcher.waitForPendingOperations()
                        }
                        return WinJS.Promise.as(pendingBookmarkWriteOperations).then(function() {
                                return WinJS.Promise.join({
                                        bookmark: ms.videoProvider.getBookmarkAsync(media.libraryId), played: ms.videoProvider.getPlayedStatusAsync(media.libraryId)
                                    })
                            }).then(function(result) {
                                var bookmark = result.bookmark.value || 0;
                                var played = result.played.value || 0;
                                return VideoSmartBuyStateEngine.getPlayButtonStatesFromBookmark(bookmark, !!played)
                            })
                    };
                    VideoSmartBuyStateEngine.prototype._addSeriesDetailsButtonIfAppropriate = function(buttons) {
                        MS.Entertainment.ViewModels.assert(this, "_addSeriesDetailsButtonIfAppropriate must be called with valid this object");
                        var inSeriesDetails = false;
                        var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                        var logicalPage = null;
                        if (MS.Entertainment.Utilities.isVideoApp1)
                            logicalPage = navigationService.currentPage;
                        else {
                            var previousLocation = navigationService.getPreviousLocation();
                            if (previousLocation)
                                logicalPage = previousLocation.page
                        }
                        if (logicalPage && logicalPage.iaNode && logicalPage.iaNode.moniker) {
                            var moniker = logicalPage.iaNode.moniker;
                            inSeriesDetails = (moniker === MS.Entertainment.UI.Monikers.immersiveDetails)
                        }
                        if (!inSeriesDetails && this.buttons.seriesDetails)
                            buttons.push(this.buttons.seriesDetails)
                    };
                    VideoSmartBuyStateEngine.prototype.onVideoDetailsTwoButtonStateChanged = function(stateInfo) {
                        var _this = this;
                        var buttons = [];
                        var actions = [];
                        var returnPromise = null;
                        if (!this.media.hydrated) {
                            if (!this.media.hydrated && this.media.hydrate)
                                this.media.hydrate();
                            return WinJS.Promise.wrap(null)
                        }
                        var isBundle = this.media.isBundle;
                        var isOwnedPresale = false;
                        if (stateInfo.marketplace.isPresale) {
                            var fulfillmentDate = new Date(stateInfo.marketplace.presaleFulfillmentDate);
                            var spanInMilliseconds = fulfillmentDate.getTime() - Date.now();
                            isOwnedPresale = (spanInMilliseconds > 0)
                        }
                        var appbarActions = MS.Entertainment.ViewModels.videoAppBarActionTable;
                        var isSignedIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn).isSignedIn;
                        var canPlayOnXbox = VideoSmartBuyStateEngine.canPlayOnXbox(this.media);
                        var isOwned = (stateInfo.marketplace.canStream && !stateInfo.marketplace.exclusivelyRented);
                        var getOnXbox = canPlayOnXbox && !isOwned;
                        var showDownloadButton = (stateInfo.marketplace.counts.canDownload > (stateInfo.collection.playability.counts.local + stateInfo.download.items.length) && MS.Entertainment.Utilities.isApp1);
                        var showDownloadManagerButton = (stateInfo.download.items.length > 0) && MS.Entertainment.Utilities.isApp1;
                        var canPlayTrailer = VideoSmartBuyStateEngine.canPlayVideoTrailer(this.media);
                        var isTVEpisode = MS.Entertainment.Platform.PlaybackHelpers.isTVEpisode(this.media);
                        var canPinToDashboard = false;
                        var canRate = false;
                        var rentalStateIsExpired = WinJS.Utilities.getMember("marketplace.rentalExpirations.overall.isExpired", stateInfo);
                        var purchaseHistoryService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.purchaseHistory);
                        var hasGrovelEverCompleted = !!(purchaseHistoryService && purchaseHistoryService.hasGrovelEverCompleted);
                        var showCollectionActionsForExpiredRentals = rentalStateIsExpired && !hasGrovelEverCompleted;
                        var canPlayContent = stateInfo.canPlay || showCollectionActionsForExpiredRentals;
                        if (MS.Entertainment.Utilities.isVideoApp2 && !isBundle) {
                            canPinToDashboard = isSignedIn && MS.Entertainment.Utilities.appRegionMatchesMachineRegion();
                            canRate = isSignedIn
                        }
                        if (canPlayOnXbox)
                            canPlayOnXbox = isOwned;
                        this._updateOfflineAvailabilityNotification(stateInfo);
                        this._updateFileTransferNotification(stateInfo);
                        if (stateInfo.download.hasFailed && stateInfo.download.hasRetryable) {
                            if (stateInfo.marketplace.libraryDefinitions[MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd] || stateInfo.marketplace.libraryDefinitions[MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd1080p])
                                this.buttons.tryAgain.title = String.load(MS.Entertainment.Utilities.getTryAgainDynamicText());
                            else
                                this.buttons.tryAgain.title = String.load(String.id.IDS_DETAILS_TRY_AGAIN_LABEL);
                            buttons.push(this.buttons.tryAgain)
                        }
                        if ((!stateInfo.marketplace.exclusivelyRented || !rentalStateIsExpired) && (stateInfo.download.hasAny || canPlayContent) && !isOwnedPresale) {
                            if (canPlayContent) {
                                var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                                var playbackSession = sessionMgr.getSession(MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying);
                                var isCurrentlyLoaded = !playbackSession.isPreview && playbackSession.isMediaCurrentlyLoaded(this.media);
                                if (!isCurrentlyLoaded || !MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).nowPlayingVisible)
                                    if (this.media.inCollection) {
                                        var playIndex = buttons.length;
                                        returnPromise = this._getPlayButtonStates(this.media).then(function(result) {
                                            var playButtons = [];
                                            if (result.showPlay)
                                                playButtons.push(_this.buttons.play);
                                            if (result.showResume)
                                                playButtons.push(_this.buttons.resume);
                                            if (result.showRestart)
                                                playButtons.push(_this.buttons.restart);
                                            buttons.spliceArray(playIndex, 0, playButtons);
                                            buttons = VideoSmartBuyStateEngine.cutButtonsIfNecessary(buttons, _this.buttons);
                                            return {
                                                    smartButtons: buttons, appbarActions: actions
                                                }
                                        })
                                    }
                                    else
                                        buttons.push(this.buttons.play)
                            }
                            else
                                buttons.push(this.buttons.playDisabled);
                            if (canPlayOnXbox && buttons.length < 2)
                                buttons.push(this.buttons.playOnXbox)
                        }
                        else if (stateInfo.marketplace.canDownload && !isOwnedPresale) {
                            if (canPlayOnXbox && buttons.length < 2)
                                buttons.push(this.buttons.playOnXbox)
                        }
                        else {
                            if (this.media.type !== MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.MovieTrailer) {
                                var canChangeLanguage = false;
                                if ((!this.buttons.switchLanguage.videoLanguagesAvailable || this.buttons.switchLanguage.videoLanguagesAvailable.length === 0) && this.buttons.switchLanguage.setVideoLanguages)
                                    this.buttons.switchLanguage.setVideoLanguages(VideoSmartBuyStateEngine.getLanguagesForRights(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp));
                                if (this.buttons.switchLanguage.videoLanguagesAvailable && this.buttons.switchLanguage.videoLanguagesAvailable.length > 1 && this.buttons.switchLanguage.updateTitleForLanguageAndCount) {
                                    this.buttons.switchLanguage.updateTitleForLanguageAndCount();
                                    canChangeLanguage = true
                                }
                                var freeWithAds = false;
                                var bestFreeRight = VideoSmartBuyStateEngine.getBestFreeRight(this.media, false);
                                if (bestFreeRight) {
                                    var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                                    var freeVideoWithAds = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.videoFreeWithAds);
                                    if (freeVideoWithAds) {
                                        var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                                        if (configurationManager.video.offerIdsVideoFreeWithAds) {
                                            var freeOfferWithAds = configurationManager.video.offerIdsVideoFreeWithAds.split(',');
                                            var offerId = bestFreeRight.offerId.toLowerCase();
                                            for (var count = 0; count < freeOfferWithAds.length; count++)
                                                if (offerId === freeOfferWithAds[count].toLowerCase()) {
                                                    freeWithAds = true;
                                                    break
                                                }
                                        }
                                    }
                                }
                                var buyButtonOffer = (!freeWithAds && bestFreeRight) ? null : VideoSmartBuyStateEngine.getDefaultBuyOffer(this.media, false, this.buttons.switchVideoDefinition.videoDefinitionPreference, this.buttons.switchLanguage.videoLanguagePreference);
                                var rentalButtonStreamingOffer = bestFreeRight ? null : VideoSmartBuyStateEngine.getDefaultRentStreamOffer(this.media, this.buttons.switchVideoDefinition.videoDefinitionPreference, this.buttons.switchLanguage.videoLanguagePreference);
                                var rentalButtonDownloadOffer = (MS.Entertainment.Utilities.isVideoApp2 || bestFreeRight) ? null : VideoSmartBuyStateEngine.getDefaultRentDownloadOffer(this.media, this.buttons.switchVideoDefinition.videoDefinitionPreference, this.buttons.switchLanguage.videoLanguagePreference);
                                if (bestFreeRight) {
                                    this.buttons.playFreeOffer.freeOffer = bestFreeRight;
                                    this.buttons.playFreeOffer.freeOfferId = bestFreeRight.offerId;
                                    this.buttons.playFreeOffer.freeOfferCurrencyCode = bestFreeRight.priceCurrencyCode;
                                    this.buttons.playFreeOffer.freeOfferPurchaseType = MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY;
                                    this.buttons.playFreeOffer.freeOfferSignedOffer = bestFreeRight.signedOffer;
                                    if (bestFreeRight.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Rent || bestFreeRight.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.RentStream)
                                        this.buttons.playFreeOffer.freeOfferPurchaseType = MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_RENT;
                                    this.buttons.playFreeOffer.startPlayback = true;
                                    this.buttons.playFreeOffer.title = String.load(!isTVEpisode ? String.id.IDS_MOVIE_PLAY_MOVIE_BUTTON : String.id.IDS_TV_PLAY_EPISODE_BUTTON);
                                    if (freeWithAds)
                                        this.buttons.playFreeOffer.subTitle = String.load(String.id.IDS_VIDEO_PRICE_FREE_WITH_ADS);
                                    buttons.push(this.buttons.playFreeOffer)
                                }
                                if (buyButtonOffer && !isOwnedPresale) {
                                    if (isBundle) {
                                        if (this.buttons.buy.isVoiceEnabled) {
                                            this.buttons.buy.title = String.load(String.id.IDS_VIDEO2_L3_BUY_BUNDLE_BUTTON_VUI_GUI);
                                            this.buttons.buy.voicePhrase = String.load(String.id.IDS_VIDEO2_L3_BUY_BUNDLE_BUTTON_VUI_ALM);
                                            this.buttons.buy.voicePhoneticPhrase = String.load(String.id.IDS_VIDEO2_L3_BUY_BUNDLE_BUTTON_VUI_PRON);
                                            this.buttons.buy.voiceConfidence = String.load(String.id.IDS_VIDEO2_L3_BUY_BUNDLE_BUTTON_VUI_CONF)
                                        }
                                        else
                                            this.buttons.buy.title = String.load(String.id.IDS_DETAILS_BUY_BUNDLE);
                                        this.buttons.buy.subTitle = VideoSmartBuyStateEngine.getPriceString(buyButtonOffer);
                                        this.buttons.buy.parameter.offer = buyButtonOffer
                                    }
                                    else if (buyButtonOffer.distributionType === MS.Entertainment.Data.Augmenter.Marketplace.edsOfferDistributionType.presale) {
                                        this.buttons.buy.title = String.load(String.id.IDS_VIDEO2_L3_PRE_ORDER_BUTTON_VUI_GUI);
                                        if (this.buttons.buy.isVoiceEnabled) {
                                            this.buttons.buy.voicePhrase = String.load(String.id.IDS_VIDEO2_L3_PRE_ORDER_BUTTON_VUI_ALM);
                                            this.buttons.buy.voicePhoneticPhrase = String.load(String.id.IDS_VIDEO2_L3_PRE_ORDER_BUTTON_VUI_PRON);
                                            this.buttons.buy.voiceConfidence = String.load(String.id.IDS_VIDEO2_L3_PRE_ORDER_BUTTON_VUI_CONF)
                                        }
                                        this.buttons.buy.subTitle = VideoSmartBuyStateEngine.getPriceString(buyButtonOffer);
                                        this.buttons.buy.parameter.offer = buyButtonOffer
                                    }
                                    else {
                                        this.buttons.buy.title = String.load(String.id.IDS_BUY_BUTTON_VIDEO);
                                        if (this.buttons.buy.isVoiceEnabled) {
                                            this.buttons.buy.voicePhrase = String.load(String.id.IDS_VIDEO2_L3_BUY_BUTTON_VUI_ALM);
                                            this.buttons.buy.voicePhoneticPhrase = String.load(String.id.IDS_VIDEO2_L3_BUY_BUTTON_VUI_PRON);
                                            this.buttons.buy.voiceConfidence = String.load(String.id.IDS_VIDEO2_L3_BUY_BUTTON_VUI_CONF)
                                        }
                                        this.buttons.buy.subTitle = VideoSmartBuyStateEngine.getPriceString(buyButtonOffer);
                                        this.buttons.buy.parameter.offer = buyButtonOffer
                                    }
                                    buttons.push(this.buttons.buy)
                                }
                                if (!isBundle && !isOwnedPresale && (rentalButtonStreamingOffer || rentalButtonDownloadOffer)) {
                                    this.buttons.rent.subTitle = VideoSmartBuyStateEngine.getPriceString(rentalButtonStreamingOffer || rentalButtonDownloadOffer);
                                    if (!rentalButtonStreamingOffer)
                                        this.buttons.rent.parameter.offer = rentalButtonDownloadOffer;
                                    else if (!rentalButtonDownloadOffer)
                                        this.buttons.rent.parameter.offer = rentalButtonStreamingOffer;
                                    else {
                                        this.buttons.rent.parameter.streamingOffer = rentalButtonStreamingOffer;
                                        this.buttons.rent.parameter.downloadOffer = rentalButtonDownloadOffer
                                    }
                                    buttons.push(this.buttons.rent)
                                }
                                if (canPlayTrailer)
                                    buttons.push(this.buttons.playTrailer)
                            }
                            var resolutions = VideoSmartBuyStateEngine.getResolutionsForRights(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp);
                            if (resolutions.hasHdAndSdRights) {
                                if (this.buttons.switchVideoDefinition.updateTitleForMediaResolution)
                                    this.buttons.switchVideoDefinition.updateTitleForMediaResolution();
                                buttons.push(this.buttons.switchVideoDefinition)
                            }
                            if (canChangeLanguage)
                                buttons.push(this.buttons.switchLanguage);
                            if (canPlayOnXbox && buttons.length < 2)
                                buttons.push(this.buttons.playOnXbox)
                        }
                        if (showDownloadButton) {
                            if (stateInfo.marketplace.libraryDefinitions[MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd] || stateInfo.marketplace.libraryDefinitions[MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd1080p])
                                if (stateInfo.marketplace.exclusivelyRented)
                                    this.buttons.downloadAction.title = String.load(String.id.IDS_DETAILS_DOWNLOAD_LABEL_HD);
                                else
                                    this.buttons.downloadAction.title = String.load(MS.Entertainment.Utilities.getDownloadDynamicText());
                            else
                                this.buttons.downloadAction.title = String.load(String.id.IDS_DETAILS_DOWNLOAD_LABEL_SD);
                            buttons.push(this.buttons.downloadAction)
                        }
                        if (canRate)
                            buttons.push(this.buttons.ratingPicker);
                        if (canPinToDashboard) {
                            buttons.push(this.buttons.pinToDashboard);
                            if (this.buttons.pinToDashboard.refresh)
                                this.buttons.pinToDashboard.refresh()
                        }
                        if (appbarActions) {
                            this._addNavigateBackButtonIfAppropriate(actions, appbarActions);
                            if (getOnXbox)
                                actions.push(appbarActions.getOnXbox);
                            if (showDownloadManagerButton)
                                buttons.push(this.buttons.showDownloadManager);
                            if (this.media.isRemovable)
                                actions.push(appbarActions.deleteMedia)
                        }
                        if (isBundle && this.media.allBundledItemsInCollection)
                            buttons.push(this.buttons.viewInCollection);
                        buttons = VideoSmartBuyStateEngine.cutButtonsIfNecessary(buttons, this.buttons);
                        if (returnPromise)
                            return returnPromise;
                        return WinJS.Promise.wrap({
                                smartButtons: buttons, appbarActions: actions
                            })
                    };
                    VideoSmartBuyStateEngine.prototype._addNavigateBackButtonIfAppropriate = function(actions, appbarActions) {
                        MS.Entertainment.ViewModels.assert(this, "_addNavigateBackButtonIfAppropriate must be called with valid this object");
                        var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                        if (MS.Entertainment.Utilities.isVideoApp1 && uiStateService && uiStateService.nowPlayingVisible && appbarActions && appbarActions.navigateBack)
                            actions.push(appbarActions.navigateBack)
                    };
                    VideoSmartBuyStateEngine.prototype.onTVSeriesInlineDetailsStateChangedInternal = function(stateInfo) {
                        return {
                                smartButtons: [], appbarActions: [this.buttons.deleteSeriesAction]
                            }
                    };
                    VideoSmartBuyStateEngine.prototype.onTVSeriesInlineDetailsStateChanged = function(stateInfo) {
                        return WinJS.Promise.wrap(this.onTVSeriesInlineDetailsStateChangedInternal(stateInfo))
                    };
                    VideoSmartBuyStateEngine.prototype.onTVSeasonInlineDetailsStateChangedInternal = function(stateInfo) {
                        var _this = this;
                        var returnPromise = null;
                        var buttons = [];
                        var actions = [];
                        if (!this.media.hydrated) {
                            if (!this.media.hydrated && this.media.hydrate)
                                this.media.hydrate();
                            return WinJS.Promise.wrap(null)
                        }
                        var entireSeasonIsOwned = stateInfo.marketplace.hasPurchasedSeason || (this.media.isComplete && stateInfo.marketplace.canStreamState === MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.All && stateInfo.marketplace.exclusivelyRentedState === MS.Entertainment.ViewModels.SmartBuyStateEngine.FileContainerAccess.None);
                        var promise;
                        var firstEpisode = this.media.firstEpisode;
                        if (firstEpisode && !entireSeasonIsOwned && !firstEpisode.hydrated && firstEpisode.hydrate)
                            promise = firstEpisode.hydrate();
                        return WinJS.Promise.as(promise).then(function() {
                                var appbarActions = MS.Entertainment.ViewModels.videoAppBarActionTable;
                                var isOwned = (stateInfo.marketplace.canStream && !stateInfo.marketplace.exclusivelyRented);
                                var showDownloadAppbarButton = (stateInfo.marketplace.counts.canDownload > (stateInfo.collection.playability.counts.local + stateInfo.download.items.length)) && MS.Entertainment.Utilities.isApp1;
                                var showDownloadManagerAppbarButton = (stateInfo.download.items.length > 0) && MS.Entertainment.Utilities.isApp1;
                                var bestFreeRight = !entireSeasonIsOwned && VideoSmartBuyStateEngine.getBestFreeSeasonRight(_this.media);
                                var buyButtonOffer = (bestFreeRight || entireSeasonIsOwned) ? null : VideoSmartBuyStateEngine.getDefaultSeasonBuyOffer(_this.media, _this.buttons.switchVideoDefinition.videoDefinitionPreference, _this.buttons.switchLanguage.videoLanguagePreference);
                                if (bestFreeRight) {
                                    _this.buttons.playFreeOffer.freeOffer = bestFreeRight;
                                    _this.buttons.playFreeOffer.freeOfferId = bestFreeRight.offerId;
                                    _this.buttons.playFreeOffer.freeOfferCurrencyCode = bestFreeRight.priceCurrencyCode;
                                    _this.buttons.playFreeOffer.freeOfferPurchaseType = MS.Entertainment.Platform.PurchaseHelpers.PURCHASE_TYPE_BUY;
                                    _this.buttons.playFreeOffer.freeOfferSignedOffer = bestFreeRight.signedOffer;
                                    _this.buttons.playFreeOffer.startPlayback = true;
                                    _this.buttons.playFreeOffer.title = String.load(String.id.IDS_DETAILS_PLAY_SEASON_BUTTON);
                                    buttons.push(_this.buttons.playFreeOffer)
                                }
                                var canSwitchDefinition = false;
                                var videoDefinitionPreference = null;
                                var canChangeLanguage = false;
                                var seasonResolutions = VideoSmartBuyStateEngine.getSeasonResolutionsForRights(_this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp);
                                if (seasonResolutions && !seasonResolutions.hasHdRight)
                                    videoDefinitionPreference = MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.sd;
                                else if (seasonResolutions && !seasonResolutions.hasSdRight)
                                    videoDefinitionPreference = MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.hd;
                                else {
                                    videoDefinitionPreference = _this.buttons.switchVideoDefinition.videoDefinitionPreference;
                                    canSwitchDefinition = true
                                }
                                if (buyButtonOffer)
                                    if (_this.media.isComplete) {
                                        _this.buttons.buySeason.subTitle = VideoSmartBuyStateEngine.getPriceString(buyButtonOffer, videoDefinitionPreference);
                                        _this.buttons.buySeason.parameter.offer = buyButtonOffer;
                                        buttons.push(_this.buttons.buySeason)
                                    }
                                    else {
                                        _this.buttons.buyCurrentSeason.subTitle = VideoSmartBuyStateEngine.getPriceString(buyButtonOffer, videoDefinitionPreference);
                                        _this.buttons.buyCurrentSeason.parameter.offer = buyButtonOffer;
                                        buttons.push(_this.buttons.buyCurrentSeason)
                                    }
                                if (buyButtonOffer) {
                                    if (canSwitchDefinition) {
                                        _this.buttons.switchVideoDefinition.updateTitleForMediaResolution();
                                        buttons.push(_this.buttons.switchVideoDefinition)
                                    }
                                    if (canChangeLanguage)
                                        buttons.push(_this.buttons.switchLanguage)
                                }
                                if (showDownloadAppbarButton) {
                                    var appBarService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar);
                                    var downloadSeasonAction = appBarService.getToolbarAction(appbarActions.downloadSeason);
                                    MS.Entertainment.UI.Controls.ChooseDownloadQualityOverlay.mediaHasHDandSD(_this.media).then(function hasHD(info) {
                                        if (info.hasHD && info.hasSD)
                                            downloadSeasonAction.title = String.load(MS.Entertainment.Utilities.getDownloadSeasonDynamicText());
                                        else if (info.hasHD)
                                            downloadSeasonAction.title = String.load(String.id.IDS_TV_DOWNLOAD_SEASON_BUTTON_HD);
                                        else
                                            downloadSeasonAction.title = String.load(String.id.IDS_TV_DOWNLOAD_SEASON_BUTTON_SD)
                                    });
                                    actions.push(appbarActions.downloadSeason)
                                }
                                if (showDownloadManagerAppbarButton)
                                    actions.push(appbarActions.showDownloadManager);
                                buttons = VideoSmartBuyStateEngine.cutButtonsIfNecessary(buttons, _this.buttons);
                                return {
                                        smartButtons: buttons, appbarActions: actions
                                    }
                            })
                    };
                    VideoSmartBuyStateEngine.prototype.onTVSeriesDetailsStateChanged = function(stateInfo) {
                        var buttons = [];
                        var actions = [];
                        if (!this.media.hydrated) {
                            if (!this.media.hydrated && this.media.hydrate)
                                this.media.hydrate();
                            return WinJS.Promise.wrap(null)
                        }
                        var isSignedIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn).isSignedIn;
                        var canPinToDashboard = false;
                        var canRate = false;
                        if (MS.Entertainment.Utilities.isVideoApp2) {
                            canPinToDashboard = isSignedIn && MS.Entertainment.Utilities.appRegionMatchesMachineRegion();
                            canRate = isSignedIn
                        }
                        var config = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                        if (MS.Entertainment.Utilities.isVideoApp2)
                            buttons.push(this.buttons.changeSeason);
                        if (canRate)
                            buttons.push(this.buttons.ratingPicker);
                        if (canPinToDashboard) {
                            buttons.push(this.buttons.pinToDashboard);
                            this.buttons.pinToDashboard.refresh()
                        }
                        return WinJS.Promise.wrap({
                                smartButtons: buttons, appbarActions: actions
                            })
                    };
                    VideoSmartBuyStateEngine.prototype.onTVSeasonInlineDetailsStateChanged = function(stateInfo) {
                        return WinJS.Promise.wrap(this.onTVSeasonInlineDetailsStateChangedInternal(stateInfo))
                    };
                    VideoSmartBuyStateEngine.prototype.onVideoGallerySelectionStateChanged = function(stateInfo) {
                        var actionButtons;
                        var buttonPromise;
                        var that = this;
                        this.buttons = MS.Entertainment.ViewModels.videoAppBarActionTable;
                        if (this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeries)
                            actionButtons = this.onTVSeriesInlineDetailsStateChangedInternal(stateInfo);
                        else if (this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeason)
                            actionButtons = this.onTVSeasonInlineDetailsStateChangedInternal(stateInfo);
                        else
                            buttonPromise = this.onVideoInlineDetailsStateChanged(stateInfo);
                        if (buttonPromise)
                            return buttonPromise.then(function(actionButtons) {
                                    return WinJS.Promise.wrap({
                                            smartButtons: [], appbarActions: that._concatAppBarActions(actionButtons)
                                        })
                                });
                        return WinJS.Promise.wrap({
                                smartButtons: [], appbarActions: this._concatAppBarActions(actionButtons)
                            })
                    };
                    VideoSmartBuyStateEngine.prototype._concatAppBarActions = function(actionButtons) {
                        var buttons = null;
                        if (actionButtons && actionButtons.smartButtons)
                            buttons = actionButtons.smartButtons.concat(actionButtons.appbarActions);
                        return buttons
                    };
                    VideoSmartBuyStateEngine.prototype._updateOfflineAvailabilityNotification = function(stateInfo) {
                        var notAvailableOffline = false;
                        if (!stateInfo.collection.canPlayLocally && !stateInfo.download.hasAny)
                            if (this.media.inCollection)
                                notAvailableOffline = true;
                            else
                                notAvailableOffline = stateInfo.marketplace.canStream || stateInfo.marketplace.canDownload;
                        if (notAvailableOffline)
                            if (!this.media.contentNotifications)
                                MS.Entertainment.Utilities.BindingAgnostic.setProperty(this.media, "contentNotifications", new MS.Entertainment.UI.ContentNotification.ObservableNotificationArray);
                            else {
                                var label = String.load(String.id.IDS_NOT_AVAILABLE_OFFLINE);
                                if (this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.video)
                                    if (stateInfo.marketplace.canStream)
                                        label = stateInfo.marketplace.canDownload ? String.load(String.id.IDS_VIDEO_STREAMING_DOWNLOAD_TO_WATCH_OFFLINE) : String.load(String.id.IDS_VIDEO_STREAMING_NOTIFICATION);
                                    else if (stateInfo.marketplace.canDownload)
                                        label = String.load(String.id.IDS_VIDEO_DOWNLOAD_TO_WATCH_OFFLINE);
                                if (stateInfo.marketplace.canStream)
                                    this.media.contentNotifications.sendNotification(MS.Entertainment.UI.ContentNotification.NotificationType.offlineAvailabilityLabel, new MS.Entertainment.UI.ContentNotification.Notification(MS.Entertainment.UI.Icon.inlineStreaming, null, label))
                            }
                        else if (this.media.contentNotifications)
                            if (stateInfo.collection.canPlayLocally && !stateInfo.download.hasActive)
                                this.media.contentNotifications.sendNotification(MS.Entertainment.UI.ContentNotification.NotificationType.offlineAvailabilityLabel, new MS.Entertainment.UI.ContentNotification.Notification(null, null, String.load(String.id.IDS_VIDEO_AVAILABLE_OFFLINE)));
                            else
                                this.media.contentNotifications.sendNotification(MS.Entertainment.UI.ContentNotification.NotificationType.offlineAvailabilityLabel, null)
                    };
                    VideoSmartBuyStateEngine.prototype._updateFileTransferNotification = function(stateInfo) {
                        if (!stateInfo.download.hasAny)
                            if (this.media.contentNotifications)
                                this.media.contentNotifications.sendNotification(MS.Entertainment.UI.ContentNotification.NotificationType.marketplaceDownloadStatus, null)
                    };
                    VideoSmartBuyStateEngine.getSeasonResolutionsForRights = function(media, tuner) {
                        if (media && media.firstEpisode)
                            return VideoSmartBuyStateEngine.getResolutionsForRights(media.firstEpisode, tuner);
                        return null
                    };
                    VideoSmartBuyStateEngine.getSeasonLanguagesForRights = function(media, tuner) {
                        if (media && media.firstEpisode)
                            return VideoSmartBuyStateEngine.getLanguagesForRights(media.firstEpisode, tuner);
                        return []
                    };
                    VideoSmartBuyStateEngine.getResolutionsForRights = function(media, tuner) {
                        var hasHdRight = false;
                        var hasSdRight = false;
                        var rights = [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Purchase, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.PurchaseStream, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Rent, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.RentStream];
                        var matchingRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._getMatchingRights(media, tuner, rights, false);
                        if (matchingRights && matchingRights.length > 0)
                            for (var i = 0; i < matchingRights.length; i++) {
                                if (!matchingRights[i].videoDefinition)
                                    continue;
                                hasSdRight = hasSdRight || matchingRights[i].videoDefinition === MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.xd || matchingRights[i].videoDefinition === MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.sd;
                                hasHdRight = hasHdRight || matchingRights[i].videoDefinition === MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd || matchingRights[i].videoDefinition === MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd1080p;
                                if (hasHdRight && hasSdRight)
                                    break
                            }
                        return {
                                hasHdRight: hasHdRight, hasSdRight: hasSdRight, hasHdAndSdRights: hasHdRight && hasSdRight
                            }
                    };
                    VideoSmartBuyStateEngine.getLanguagesForRights = function(media, tuner) {
                        var languages = [];
                        var rights = [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Purchase, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.PurchaseStream, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Rent, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.RentStream];
                        var matchingRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._getMatchingRights(media, tuner, rights, false);
                        if (matchingRights && matchingRights.length > 0)
                            for (var i = 0; i < matchingRights.length; i++) {
                                var found = false;
                                for (var j = 0; j < languages.length && !found; j++)
                                    if (languages[j] === matchingRights[i].primaryAudioLanguage)
                                        found = true;
                                if (!found)
                                    languages.push(matchingRights[i].primaryAudioLanguage)
                            }
                        return languages
                    };
                    VideoSmartBuyStateEngine.findSeasonOffer = function(season, episodeOffer) {
                        for (var i = 0; i < season.rights.length; i++)
                            if (season.rights[i].offerId === episodeOffer.offerId && season.rights[i].offerInstanceId === episodeOffer.offerInstanceId)
                                return season.rights[i];
                        return null
                    };
                    VideoSmartBuyStateEngine.getBestFreeSeasonRight = function(season) {
                        var episode = null;
                        Trace.assert(season.firstEpisode && season.firstEpisode.hydrated, "First episode not hydrated");
                        if (season.firstEpisode && season.firstEpisode.hydrated)
                            episode = season.firstEpisode;
                        var bestOffer = VideoSmartBuyStateEngine.getBestFreeRight((episode !== null) ? episode : season, true);
                        if (bestOffer && episode)
                            bestOffer = MS.Entertainment.ViewModels.VideoSmartBuyStateEngine.findSeasonOffer(season, bestOffer);
                        return bestOffer
                    };
                    VideoSmartBuyStateEngine.getBestFreeRight = function(media, isSeason) {
                        var bestFreeRight = null;
                        if (!MS.Entertainment.Utilities.clientSupportsFreeJustPlays)
                            return bestFreeRight;
                        var tuner = MS.Entertainment.Utilities.defaultClientTypeFromApp;
                        var rights = null;
                        if (isSeason)
                            rights = [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.SeasonPurchase, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.SeasonPurchaseStream];
                        else
                            rights = [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Purchase, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.PurchaseStream, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Rent, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.RentStream];
                        var matchingRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._getMatchingRights(media, tuner, rights, true);
                        if (!matchingRights || matchingRights.length === 0)
                            return null;
                        var edsResolutionFormat = MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat;
                        var highestFreeResolution = String.empty;
                        var i = 0;
                        while (i < matchingRights.length && matchingRights[i]) {
                            if (MS.Entertainment.Utilities.isRightFree(matchingRights[i])) {
                                var videoDefinition = matchingRights[i].videoDefinition;
                                var betterRight = false;
                                switch (videoDefinition) {
                                    case MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.xd:
                                        betterRight = (highestFreeResolution === String.empty);
                                        break;
                                    case MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.sd:
                                        betterRight = (highestFreeResolution === String.empty || highestFreeResolution === MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.xd);
                                        break;
                                    case MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd:
                                        betterRight = (highestFreeResolution === String.empty || highestFreeResolution === MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.xd || highestFreeResolution === MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.sd);
                                        break;
                                    case MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd1080p:
                                        betterRight = (highestFreeResolution === String.empty || highestFreeResolution === MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.xd || highestFreeResolution === MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.sd || highestFreeResolution === MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd);
                                        break
                                }
                                if (!bestFreeRight || betterRight) {
                                    bestFreeRight = matchingRights[i];
                                    highestFreeResolution = bestFreeRight.videoDefinition
                                }
                            }
                            i++
                        }
                        return bestFreeRight
                    };
                    VideoSmartBuyStateEngine.getDefaultSeasonBuyOffer = function(season, videoDefinitionPreference, videoLanguagePreference) {
                        var episode = null;
                        Trace.assert(season.firstEpisode && season.firstEpisode.hydrated, "First episode not hydrated");
                        if (season.firstEpisode && season.firstEpisode.hydrated)
                            episode = season.firstEpisode;
                        var bestOffer = MS.Entertainment.ViewModels.VideoSmartBuyStateEngine.getDefaultBuyOffer((episode !== null) ? episode : season, true, videoDefinitionPreference, videoLanguagePreference);
                        if (bestOffer && episode)
                            bestOffer = MS.Entertainment.ViewModels.VideoSmartBuyStateEngine.findSeasonOffer(season, bestOffer);
                        return bestOffer
                    };
                    VideoSmartBuyStateEngine.getDefaultBuyOffer = function(media, isSeason, videoDefinitionPreference, videoLanguagePreference) {
                        var rights;
                        if (isSeason)
                            rights = [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.SeasonPurchase, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.SeasonPurchaseStream];
                        else
                            rights = [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Purchase, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.PurchaseStream];
                        return VideoSmartBuyStateEngine.getDefaultOffer(media, MS.Entertainment.Utilities.defaultClientTypeFromApp, rights, false, MS.Entertainment.Utilities.clientSupportsFreeJustPlays, null, videoDefinitionPreference, videoLanguagePreference)
                    };
                    VideoSmartBuyStateEngine.getDefaultRentStreamOffer = function(media, videoDefinitionPreference, videoLanguagePreference) {
                        return VideoSmartBuyStateEngine.getDefaultOffer(media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Rent, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.RentStream], false, MS.Entertainment.Utilities.clientSupportsFreeJustPlays, MS.Entertainment.Data.Augmenter.Marketplace.edsDeliveryFormat.streaming, videoDefinitionPreference, videoLanguagePreference)
                    };
                    VideoSmartBuyStateEngine.getDefaultRentDownloadOffer = function(media, videoDefinitionPreference, videoLanguagePreference) {
                        return VideoSmartBuyStateEngine.getDefaultOffer(media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Rent, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.RentStream], false, MS.Entertainment.Utilities.clientSupportsFreeJustPlays, MS.Entertainment.Data.Augmenter.Marketplace.edsDeliveryFormat.download, videoDefinitionPreference, videoLanguagePreference)
                    };
                    VideoSmartBuyStateEngine.getDefaultOffer = function(media, tuner, rights, freeOnly, excludeFree, deliveryFormat, videoDefinitionPreference, videoLanguagePreference) {
                        return this.getPreferredQualityWithLowestPriceOffer(media, tuner, rights, freeOnly, excludeFree, deliveryFormat, videoDefinitionPreference, videoLanguagePreference)
                    };
                    VideoSmartBuyStateEngine.getPreferredQualityWithLowestPriceOffer = function(media, tuner, rights, freeOnly, excludeFree, deliveryFormat, videoDefinitionPreference, videoLanguagePreference) {
                        var resolutionFormat = MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat;
                        var preferHigherQuality;
                        if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeason)
                            videoLanguagePreference = null;
                        var qualityPreference = [];
                        if (videoDefinitionPreference === MS.Entertainment.Data.Augmenter.Marketplace.videoDefinition.sd) {
                            qualityPreference = [resolutionFormat.hd1080p, resolutionFormat.hd, resolutionFormat.xd, resolutionFormat.sd];
                            preferHigherQuality = false
                        }
                        else {
                            qualityPreference = [resolutionFormat.xd, resolutionFormat.sd, resolutionFormat.hd, resolutionFormat.hd1080p];
                            preferHigherQuality = true
                        }
                        var qualityOrder = {};
                        for (var i = 0; i < qualityPreference.length; i++)
                            qualityOrder[qualityPreference[i]] = i + 1;
                        var sortTypeQualityPriceFunction = function sortTypeQualityPriceFunction(a, b) {
                                var resolutionA = a ? qualityOrder[a.videoDefinition] : null;
                                var resolutionB = b ? qualityOrder[b.videoDefinition] : null;
                                var priceA = a ? a.price : 0;
                                var priceB = b ? b.price : 0;
                                var offerDistributionTypeA = a ? a.distributionType : null;
                                var offerDistributionTypeB = b ? b.distributionType : null;
                                MS.Entertainment.ViewModels.assert(offerDistributionTypeA && offerDistributionTypeB && ((offerDistributionTypeA === MS.Entertainment.Data.Augmenter.Marketplace.edsOfferDistributionType.full) || (offerDistributionTypeA === MS.Entertainment.Data.Augmenter.Marketplace.edsOfferDistributionType.presale)) && ((offerDistributionTypeB === MS.Entertainment.Data.Augmenter.Marketplace.edsOfferDistributionType.full) || (offerDistributionTypeB === MS.Entertainment.Data.Augmenter.Marketplace.edsOfferDistributionType.presale)), "Both items must be valid and have valid distribution Types");
                                if (offerDistributionTypeA !== offerDistributionTypeB)
                                    return ((offerDistributionTypeA === MS.Entertainment.Data.Augmenter.Marketplace.edsOfferDistributionType.full) ? 1 : -1);
                                if (!resolutionA || !resolutionB)
                                    return preferHigherQuality ? (priceA === priceB) ? 0 : ((priceA > priceB) ? 1 : -1) : (priceA === priceB) ? 0 : ((priceB > priceA) ? 1 : -1);
                                var resolutionOrder = (resolutionA === resolutionB) ? 0 : ((resolutionA > resolutionB) ? 1 : -1);
                                if (resolutionOrder === 0)
                                    if (priceA !== priceB)
                                        return ((priceA < priceB) ? 1 : -1);
                                    else if (a.videoEncoding === MS.Entertainment.Data.Augmenter.Marketplace.edsVideoEncoding.h264)
                                        return (new Microsoft.Entertainment.Configuration.ConfigurationManager).fulfillment.preferH264Content ? 1 : -1;
                                    else
                                        return (new Microsoft.Entertainment.Configuration.ConfigurationManager).fulfillment.preferH264Content ? -1 : 1;
                                else
                                    return resolutionOrder
                            };
                        return this.getOfferWithSortFunction(media, tuner, rights, freeOnly, excludeFree, deliveryFormat, videoLanguagePreference, sortTypeQualityPriceFunction)
                    };
                    VideoSmartBuyStateEngine.getLowestPriceOffer = function(media, tuner, rights, freeOnly, excludeFree) {
                        var sortPriceFunction = function sortPricesLowToHigh(a, b) {
                                var priceA = a ? a.price : 0;
                                var priceB = b ? b.price : 0;
                                return ((priceA === priceB) ? 0 : ((priceA < priceB) ? 1 : -1))
                            };
                        return this.getOfferWithSortFunction(media, tuner, rights, freeOnly, excludeFree, null, null, sortPriceFunction)
                    };
                    VideoSmartBuyStateEngine.getOfferWithSortFunction = function(media, tuner, rights, freeOnly, excludeFree, deliveryFormat, language, sortFunction) {
                        if (!MS.Entertainment.Utilities.clientSupportsFreeJustPlays)
                            excludeFree = false;
                        MS.Entertainment.ViewModels.assert(MS.Entertainment.Utilities.isApp1 || !deliveryFormat || deliveryFormat === MS.Entertainment.Data.Augmenter.Marketplace.edsDeliveryFormat.streaming, "Streaming is the only acceptable delivery format for Video 2");
                        var matchingRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._getMatchingRights(media, tuner, rights, freeOnly, excludeFree);
                        if (!matchingRights || matchingRights.length === 0)
                            return null;
                        var possibleRights = [];
                        var foundCurrencyCode = String.empty;
                        for (var i = 0; i < matchingRights.length; i++) {
                            var right = matchingRights[i];
                            if (deliveryFormat && right.deliveryFormat !== deliveryFormat)
                                continue;
                            if (language && right.primaryAudioLanguage && MS.Entertainment.Utilities.getDisplayLanguageFromLanguageCode(right.primaryAudioLanguage) !== MS.Entertainment.Utilities.getDisplayLanguageFromLanguageCode(language))
                                continue;
                            if (right.hasOffer && right.priceCurrencyCode) {
                                var localCurrencyFilters = (new Microsoft.Entertainment.Configuration.ConfigurationManager).service.localCurrencyFilters.split(",");
                                if (localCurrencyFilters) {
                                    var unwantedCurrencyCode = false;
                                    for (var f = 0; f < localCurrencyFilters.length; f++)
                                        if (right.priceCurrencyCode === localCurrencyFilters[f]) {
                                            unwantedCurrencyCode = true;
                                            break
                                        }
                                    if (unwantedCurrencyCode)
                                        continue
                                }
                                if (!foundCurrencyCode)
                                    foundCurrencyCode = right.priceCurrencyCode;
                                if (right.priceCurrencyCode !== foundCurrencyCode)
                                    return null;
                                possibleRights.push(right)
                            }
                        }
                        if (!possibleRights || possibleRights.length === 0)
                            return null;
                        else {
                            possibleRights.sort(sortFunction);
                            return possibleRights[possibleRights.length - 1]
                        }
                    };
                    VideoSmartBuyStateEngine.getPriceString = function(offer, videoDefinitionPreference) {
                        if (!offer)
                            return null;
                        var displayPrice = offer.displayPrice;
                        if (displayPrice === "")
                            displayPrice = "0";
                        else if (MS.Entertainment.Utilities.isApp2 && MS.Entertainment.Utilities.isRightFree(offer))
                            displayPrice = String.load(String.id.IDS_VIDEO_PRICE_FREE);
                        var priceString = null;
                        if (MS.Entertainment.Utilities.isRightFree(offer))
                            priceString = String.load(String.id.IDS_VIDEO_PRICE_FREE);
                        else {
                            var videoDefinition = videoDefinitionPreference || offer.videoDefinition;
                            switch (videoDefinition) {
                                case MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd:
                                case MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.hd1080p:
                                    priceString = String.load(String.id.IDS_VIDEO_PRICE_HD).format(displayPrice);
                                    break;
                                case MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.sd:
                                case MS.Entertainment.Data.Augmenter.Marketplace.edsResolutionFormat.xd:
                                    priceString = String.load(String.id.IDS_VIDEO_PRICE_SD).format(displayPrice);
                                    break;
                                case String.empty:
                                    priceString = displayPrice;
                                    break;
                                default:
                                    MS.Entertainment.Framework.fail("SmartBuyStateHandler: Unknown Video Resolution: " + videoDefinition);
                                    break
                            }
                        }
                        return priceString
                    };
                    VideoSmartBuyStateEngine.canPlayVideoTrailer = function(media) {
                        var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                        var isTVEpisode = MS.Entertainment.Platform.PlaybackHelpers.isTVEpisode(media);
                        var canPlayTrailer = !isTVEpisode && featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.movieTrailersMarketplace) && ((media.hasCanonicalId && media.videoPreviewUrl) || (media.hasServiceId && MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Preview, MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.PreviewStream])));
                        return canPlayTrailer
                    };
                    VideoSmartBuyStateEngine.canPlayOnXbox = function(media) {
                        return media.type !== MS.Entertainment.Data.Augmenter.Marketplace.EditorialType.MovieTrailer && MS.Entertainment.ViewModels.SmartBuyStateHandlers._canXboxAccessMedia(media, null) && MS.Entertainment.Utilities.isApp1 && !MS.Entertainment.Utilities.isWindowsBlue
                    };
                    VideoSmartBuyStateEngine.cutButtonsIfNecessary = function(buttons, buttonSet) {
                        if (MS.Entertainment.Utilities.isVideoApp2 && buttons.length > this.maxVideo2Buttons)
                            buttons = buttons.slice(0, this.maxVideo2Buttons);
                        return buttons
                    };
                    VideoSmartBuyStateEngine.getPlayButtonStatesFromBookmark = function(bookmark, played) {
                        return {
                                showPlay: (bookmark === 0) && !played, showResume: (bookmark > 0) && !played, showRestart: (bookmark === 0 && played) || (bookmark > 0)
                            }
                    };
                    VideoSmartBuyStateEngine.maxVideo2Buttons = 6;
                    return VideoSmartBuyStateEngine
                })(MS.Entertainment.ViewModels.SmartBuyStateEngine);
            ViewModels.VideoSmartBuyStateEngine = VideoSmartBuyStateEngine;
            ViewModels.videoAppBarActionTable = {
                play: MS.Entertainment.UI.AppBarActions.playNow, playFreeOffer: MS.Entertainment.UI.AppBarActions.playFreeOffer, playOnXbox: MS.Entertainment.UI.AppBarActions.playOnXbox360, tryAgain: MS.Entertainment.UI.AppBarActions.tryDownloadAgain, remove: MS.Entertainment.UI.AppBarActions.removeFailedVideo, movieDetails: MS.Entertainment.UI.AppBarActions.movieDetails, playTrailer: MS.Entertainment.UI.AppBarActions.playTrailer, restart: MS.Entertainment.UI.AppBarActions.restartVideo, resume: MS.Entertainment.UI.AppBarActions.resumeVideo, playDisabled: MS.Entertainment.UI.AppBarActions.playDisabled, buyEpisode: MS.Entertainment.UI.AppBarActions.buyEpisode, buy: MS.Entertainment.UI.AppBarActions.buyVideo, rent: MS.Entertainment.UI.AppBarActions.rentVideo, seriesDetails: MS.Entertainment.UI.AppBarActions.seriesDetails, downloadAction: MS.Entertainment.UI.AppBarActions.downloadFromMarketplace, downloadSeason: MS.Entertainment.UI.AppBarActions.downloadSeason, showDownloadManager: MS.Entertainment.UI.AppBarActions.showDownloadManager, deleteMedia: MS.Entertainment.UI.AppBarActions.deleteMedia, deleteSeriesAction: MS.Entertainment.UI.AppBarActions.deleteSeries, getOnXbox: MS.Entertainment.UI.AppBarActions.getOnXbox360, deleteAction: MS.Entertainment.UI.AppBarActions.deleteMedia, pinToStartScreen: MS.Entertainment.UI.AppBarActions.pinToStartScreen, unpinFromStartScreen: MS.Entertainment.UI.AppBarActions.unpinFromStartScreen, seasonDetails: MS.Entertainment.UI.AppBarActions.seasonDetails, buySeason: MS.Entertainment.UI.AppBarActions.buySeason, buyCurrentSeason: MS.Entertainment.UI.AppBarActions.buyCurrentSeason, navigateBack: MS.Entertainment.UI.AppBarActions.navigateBack
            }
        })(Entertainment.ViewModels || (Entertainment.ViewModels = {}));
        var ViewModels = Entertainment.ViewModels
    })(MS.Entertainment || (MS.Entertainment = {}));
    var Entertainment = MS.Entertainment
})(MS || (MS = {}))
