﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.ViewModels");
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {Node: MS.Entertainment.UI.Framework.derive("MS.Entertainment.UI.Framework.ObservableBase", function Node(id, label, value, secondaryLabel) {
            MS.Entertainment.ViewModels.assert(id, "Attempted to construct a Node without an ID.");
            this.id = id;
            this.label = label || String.empty;
            this.value = value || {};
            this.secondaryLabel = secondaryLabel || String.empty
        }, {
            selected: MS.Entertainment.UI.Framework.observableProperty("selected", false), isRoot: MS.Entertainment.UI.Framework.observableProperty("isRoot", true), label: MS.Entertainment.UI.Framework.observableProperty("label", null), secondaryLabel: MS.Entertainment.UI.Framework.observableProperty("secondaryLabel", null), value: MS.Entertainment.UI.Framework.observableProperty("value", null), items: MS.Entertainment.UI.Framework.observableProperty("items", null)
        })});
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {
        NodeValues: MS.Entertainment.UI.Framework.define(function nodeValues(query, queryOptions, modelOptions, modifierOptions, pivotOptions, trackQuery, trackQueryOptions) {
            this.query = query || null;
            this.queryOptions = queryOptions || {};
            this.modelOptions = modelOptions || {};
            this.modifierOptions = modifierOptions || {};
            this.pivotOptions = pivotOptions || {};
            this.trackQuery = trackQuery || null;
            this.trackQueryOptions = trackQueryOptions || {}
        }, {
            query: null, queryOptions: null, modelOptions: null, modifierOptions: null, filterOptions: null, pivotOptions: null, trackQuery: null, trackQueryOptions: null, subPivotOptions: null
        }, {create: function create(options) {
                var nodeValue = new MS.Entertainment.ViewModels.NodeValues;
                WinJS.UI.setOptions(nodeValue, options);
                return nodeValue
            }}), ActionCell: MS.Entertainment.UI.Framework.derive("MS.Entertainment.UI.Framework.ObservableBase", function actionCell(actionId, actionOptions, actionParams) {
                this._actionParams = actionParams;
                this._actionOptions = actionOptions;
                this._actionId = actionId
            }, {
                _actionId: null, _actionOptions: null, _actionParam: null, isAction: true, icon: MS.Entertainment.UI.Framework.observableProperty("icon", null), text: MS.Entertainment.UI.Framework.observableProperty("text", null), stringId: null, hideDefaultRing: false, groupHeader: String.empty, automationId: String.empty, action: {get: function() {
                            if (!this._action)
                                this._initializeAction();
                            return this._action
                        }}, _initializeAction: function _initializeAction() {
                        if (this._actionId)
                            this._action = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).getAction(this._actionId);
                        if (this._actionOptions && this._action)
                            WinJS.UI.setOptions(this._action, this._actionOptions);
                        if (this._actionParams && this._action)
                            this._action.parameter = this._actionParams;
                        this._actionOptions = null
                    }
            })
    });
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {ModifierActionCell: MS.Entertainment.UI.Framework.derive("MS.Entertainment.ViewModels.ActionCell", function modifierActionCell(modifier, groupHeader, modifierTitleId, forcedModifier) {
            var modifierCount = modifier && modifier.dataSource && modifier.dataSource.length;
            if (modifierCount > 1) {
                var originalIndex = modifier.selectedIndex;
                this._action = {
                    execute: function executedPlay(param) {
                        return MS.Entertainment.UI.Controls.ModifierPopup.createModifierPopup("primaryPanelModifierPopup", modifier).show().then(function modifierChanged() {
                                return WinJS.Promise.wrap(originalIndex !== modifier.selectedIndex)
                            })
                    }, canExecute: function canExecutePlay(param) {
                            return true
                        }
                };
                this.enabled = true
            }
            else {
                this._action = {
                    execute: function executedPlay(param) {
                        return WinJS.Promise.wrap(false)
                    }, canExecute: function canExecutePlay(param) {
                            return true
                        }
                };
                this.enabled = false
            }
            if (modifierCount > 0)
                this.text = modifier.selectedItem ? modifier.selectedItem.label : modifier.dataSource.item(0).label;
            else
                this.text = forcedModifier;
            this.icon = null;
            if (modifierTitleId)
                this.title = modifierTitleId;
            if (groupHeader)
                this.groupHeader = groupHeader
        }, {
            title: null, isModifier: true, action: {get: function() {
                        if (!this._action)
                            var action;
                        return this._action
                    }}
        })});
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {NotificationActionCell: MS.Entertainment.UI.Framework.derive("MS.Entertainment.ViewModels.ActionCell", function notificationActionCell(category, actionId, actionOptions, clearActionId) {
            this.category = category;
            this._clearActionId = clearActionId;
            this._cloudMatchDialogAction = actionOptions && actionOptions.consentDialog ? true : false;
            MS.Entertainment.ViewModels.ActionCell.prototype.constructor.call(this, actionId, actionOptions, {category: this.category});
            this.notificationClass = category.name
        }, {
            _clearActionId: null, _cloudMatchDialogAction: false, category: null, clearIcon: MS.Entertainment.UI.Icon.close, isNotification: true, subText: MS.Entertainment.UI.Framework.observableProperty("subText", null), notificationClass: null, clearAction: {get: function() {
                        if (!this._clearAction)
                            this._initializeClearAction();
                        return this._clearAction
                    }}, _initializeClearAction: function _initializeClearAction() {
                    this._clearAction = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).getAction(this._clearActionId);
                    this._clearAction.title = String.load(String.id.IDS_CLOSE_BUTTON);
                    this._clearAction.optOut = this._cloudMatchDialogAction;
                    this._clearAction.parameter = {
                        category: this.category, actionType: this.category.name
                    }
                }, setNotification: function setNotification(notification) {
                    if (!notification) {
                        MS.Entertainment.ViewModels.fail("NotificationActionCell::setNotification() Unable to set notification since notification was null.");
                        return
                    }
                    this._actionId = notification.action;
                    this._actionOptions = notification.actionParams;
                    this.category = notification.category;
                    this._actionParams = {category: this.category};
                    this._clearActionId = notification.dismissAction;
                    this.icon = notification.icon;
                    this.text = notification.title;
                    this.subText = notification.subTitle;
                    this._cloudMatchDialogAction = !!(this._actionOptions && this._actionOptions.consentDialog);
                    var oldAction = this._action;
                    this._initializeAction();
                    this.notify("action", this._action, oldAction);
                    var oldClearAction = this._clearAction;
                    this._initializeClearAction();
                    this.notify("clearAction", this._clearAction, oldClearAction)
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {QueryViewModelBase: MS.Entertainment.UI.Framework.derive("MS.Entertainment.UI.Framework.ObservableBase", null, {
            items: MS.Entertainment.UI.Framework.observableProperty("items", null), emptyGalleryModel: MS.Entertainment.UI.Framework.observableProperty("emptyGalleryModel", null), modifierSelectionManager: MS.Entertainment.UI.Framework.observableProperty("modifierSelectionManager", null), secondaryModifierSelectionManager: MS.Entertainment.UI.Framework.observableProperty("secondaryModifierSelectionManager", null), pivotsSelectionManager: MS.Entertainment.UI.Framework.observableProperty("pivotsSelectionManager", null), filterSelectionManager: MS.Entertainment.UI.Framework.observableProperty("filterSelectionManager", null), isFailed: MS.Entertainment.UI.Framework.observableProperty("isFailed", false), isLoading: MS.Entertainment.UI.Framework.observableProperty("isLoading", false), largeItemIndex: MS.Entertainment.UI.Framework.observableProperty("largeItemIndex", -1), selectedTemplate: MS.Entertainment.UI.Framework.observableProperty("selectedTemplate", null), titleOverride: MS.Entertainment.UI.Framework.observableProperty("titleOverride", null), titleFormatValues: MS.Entertainment.UI.Framework.observableProperty("titleFormatValues", null), title: MS.Entertainment.UI.Framework.observableProperty("title", null), pivotSelectedIndexOverride: MS.Entertainment.UI.Framework.observableProperty("pivotSelectedIndexOverride", null), modelActions: MS.Entertainment.UI.Framework.observableProperty("modelActions", null), containingMedia: MS.Entertainment.UI.Framework.observableProperty("containingMedia", null), selectionHandlers: MS.Entertainment.UI.Framework.observableProperty("selectionHandlers", null), filterValue: MS.Entertainment.UI.Framework.observableProperty("filterValue", null), primaryItemIndex: MS.Entertainment.UI.Framework.observableProperty("primaryItemIndex", -1), initialInvokedItem: -1
        })});
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {QueryViewModel: MS.Entertainment.UI.Framework.derive("MS.Entertainment.ViewModels.QueryViewModelBase", function queryViewModelConstructor(view) {
            MS.Entertainment.ViewModels.QueryViewModelBase.prototype.constructor.call(this);
            this._fileTransferListenerId = "" + Date.now();
            this._fileTransferListenerId = this._viewModelId + this._fileTransferListenerId + "_" + Math.random();
            this._workingModifierSelectionManager = new MS.Entertainment.UI.Framework.SelectionManager(null, this._defaultModifierSelection, null, this._overrideModifierSetting);
            this._workingSecondaryModifierSelectionManager = new MS.Entertainment.UI.Framework.SelectionManager(null, this._defaultSecondaryModifierSelection);
            this._workingFilterSelectionManager = new MS.Entertainment.UI.Framework.SelectionManager(null, this._defaultFilterIndex, null, this._overrideFilterSettingOnce);
            this._workingPivotsSelectionManager = new MS.Entertainment.UI.Framework.SelectionManager(null, this._defaultPivotIndex);
            this.selectedTemplate = {};
            this.initialize();
            this.view = view
        }, {
            _defaultModifierSelection: 0, _overrideModifierSetting: false, _overrideFilterSettingOnce: false, _defaultSecondaryModifierSelection: 0, _defaultFilterIndex: 0, _defaultPivotIndex: 0, _viewModelId: null, _queryWatcher: null, _fileTransferListenerId: null, _queryWatcherString: "marketplace", _loadingDoneString: "loadingDone", _pendingViewChange: null, _pendingQueryExecute: null, _lastUsedPivotItem: null, _lastUsedModifierItem: null, _lastUsedSecondaryModifierItem: null, _lastUsedFilterItem: null, _lastUsedView: null, _lastUsedQueryType: null, _lastUsedQuery: null, _lastUsedModifierDefinition: null, _lastUsedSecondaryModifierDefinition: null, _lastUsedFilterDefinition: null, _lastUsedPivotDefinition: null, _lastUsedSubPivotDefinition: null, _modifiersTrimmed: false, _workingModifierSelectionManager: null, _workingSecondaryModifierSelectionManager: null, _workingFilterSelectionManager: null, _workingPivotsSelectionManager: null, _recreateQueries: false, _view: null, _enabled: false, _disposed: false, _modifierEvents: null, _secondaryModifierEvents: null, _filterEvents: null, _pivotEvents: null, _refreshing: false, _queryEventHandlers: null, _isDelayInitialized: false, enableDelayInitialize: false, debugId: null, handleQueryChanges: true, autoHideInvalidModifiers: false, failOnEmpty: false, propertyKey: null, taskKeyGetter: null, notifier: null, addActionCellsToList: true, forceEmpty: false, completedQuery: null, mediaContext: null, pausableQuery: {get: function() {
                        if (this.getQuery)
                            return this.getQuery;
                        else if (this.completedQuery && this.completedQuery.pause && this.completedQuery.unpause)
                            return this.completedQuery;
                        return null
                    }}, isOnline: {get: function() {
                        return MS.Entertainment.UI.NetworkStatusService.isOnline()
                    }}, initialize: function initialize() {
                    MS.Entertainment.ViewModels.assert(!this._modifierEvents && !this._pivotEvents && !this._secondaryModifierEvents, "These events shouldn't exist");
                    this._modifierEvents = MS.Entertainment.Utilities.addEventHandlers(this._workingModifierSelectionManager, {selectedItemChanged: this._handleModifierSelectionChanged.bind(this)});
                    this._secondaryModifierEvents = MS.Entertainment.Utilities.addEventHandlers(this._workingSecondaryModifierSelectionManager, {selectedItemChanged: this._handleSecondaryModifierSelectionChanged.bind(this)});
                    this._filterEvents = MS.Entertainment.Utilities.addEventHandlers(this._workingFilterSelectionManager, {selectedItemChanged: this._handleFilterSelectionChanged.bind(this)});
                    this._pivotEvents = MS.Entertainment.Utilities.addEventHandlers(this._workingPivotsSelectionManager, {selectedItemChanged: this._handlePivotSelectionChanged.bind(this)});
                    if (!this.enableDelayInitialize)
                        this.delayInitialize()
                }, isDelayInitialized: {get: function get_isDelayInitialized() {
                        return this._isDelayInitialized
                    }}, delayInitialize: function delayInitialize() {
                    this._isDelayInitialized = true;
                    this._updateSelectionHandlers()
                }, dispose: function dispose() {
                    if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.fileTransfer)) {
                        var fileTransferService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.fileTransfer);
                        fileTransferService.unregisterListener(this._fileTransferListenerId)
                    }
                    this._disposeQuery();
                    if (this._pendingViewChange) {
                        this._pendingViewChange.cancel();
                        this._pendingViewChange = null
                    }
                    if (this._pendingQueryExecute) {
                        this._pendingQueryExecute.cancel();
                        this._pendingQueryExecute = null
                    }
                    if (this._modifierEvents) {
                        this._modifierEvents.cancel();
                        this._modifierEvents = null
                    }
                    if (this._filterEvents) {
                        this._filterEvents.cancel();
                        this._filterEvents = null
                    }
                    if (this._pivotEvents) {
                        this._pivotEvents.cancel();
                        this._pivotEvents = null
                    }
                    if (this._workingModifierSelectionManager) {
                        this._workingModifierSelectionManager.dispose();
                        this._workingModifierSelectionManager = null
                    }
                    if (this._workingSecondaryModifierSelectionManager) {
                        this._workingSecondaryModifierSelectionManager.dispose();
                        this._workingSecondaryModifierSelectionManager = null
                    }
                    if (this._workingFilterSelectionManager) {
                        this._workingFilterSelectionManager.dispose();
                        this._workingFilterSelectionManager = null
                    }
                    if (this._workingPivotsSelectionManager) {
                        this._workingPivotsSelectionManager.dispose();
                        this._workingPivotsSelectionManager = null
                    }
                    this.selectedTemplate = {};
                    this.containingMedia = null;
                    this.selectionHandlers = null;
                    this._lastUsedPivotItem = null;
                    this._lastUsedModifierItem = null;
                    this._lastUsedSecondaryModifierItem = null;
                    this._lastUsedFilterItem = null;
                    this._disposed = true
                }, view: {
                    get: function() {
                        return this._view
                    }, set: function(value) {
                            if (this._view !== value && !this._disposed) {
                                var oldValue = this._view;
                                this._view = value;
                                this._handleViewChange();
                                this.notify("view", value, oldValue)
                            }
                        }
                }, impressionGuid: {get: function() {
                        return WinJS.Utilities.getMember("_lastUsedQuery.result.impressionGuid", this) || null
                    }}, isCurrentQuery: function isCurrentQuery() {
                    return !this._pendingQueryExecute
                }, getViewDefinition: function(view) {
                    return {}
                }, getPivotDefinition: function(view) {
                    return null
                }, getSubPivotDefinition: function(view, pivot) {
                    return null
                }, getModifierDefinition: function(view) {
                    return null
                }, getSecondaryModifierDefinition: function getSecondaryModifierDefinition(view, modifier) {
                    return null
                }, getFilterDefinition: function(view) {
                    return null
                }, getFilterOptions: function getFilterOptions(view) {
                    var definition = this.getViewDefinition(view);
                    return definition && definition.filterOptions
                }, getModifierOptions: function getModifierOptions(view) {
                    var definition = this.getViewDefinition(view);
                    return definition && definition.modifierOptions
                }, getSecondaryModifierOptions: function getSecondaryModifierOptions(view, modifier) {
                    var definition = this.getViewDefinition(view);
                    return definition && definition.secondaryModifierOptions
                }, getSecondaryModifierItemQueryOptions: function getSecondaryModifierItemQueryOptions(view, modifier) {
                    var definition = this.getViewDefinition(view);
                    return MS.Entertainment.Utilities.uniteObjects(WinJS.Binding.unwrap(modifier && modifier.value && modifier.value.itemQueryOptions), WinJS.Binding.unwrap(definition && definition.itemQueryOptions))
                }, getSubPivotItemQueryOptions: function getSubPivotItemQueryOptions(view, pivot) {
                    var definition = this.getViewDefinition(view);
                    return MS.Entertainment.Utilities.uniteObjects(WinJS.Binding.unwrap(pivot && pivot.value && pivot.value.itemQueryOptions), WinJS.Binding.unwrap(definition && definition.itemQueryOptions))
                }, getPivotOptions: function getPivotOptions(view) {
                    var definition = this.getViewDefinition(view);
                    return definition && definition.pivotOptions
                }, getSubPivotOptions: function getSubPivotOptions(view, pivot) {
                    var definition = this.getViewDefinition(view);
                    return definition && definition.subPivotOptions
                }, getQueryType: function getQueryType(view, pivot, modifier) {
                    var QueryType = null;
                    if (view.query)
                        QueryType = view.query;
                    else if (modifier.value.query)
                        QueryType = modifier.value.query;
                    else if (pivot.value.query)
                        QueryType = pivot.value.query;
                    return QueryType
                }, getQueryOptions: function getQueryOptions(view, pivot, modifier, secondaryModifier, filter) {
                    var options = MS.Entertainment.Utilities.uniteObjects(WinJS.Binding.unwrap(pivot.value.queryOptions), WinJS.Binding.unwrap(modifier.value.queryOptions));
                    options = MS.Entertainment.Utilities.uniteObjects(options, WinJS.Binding.unwrap(secondaryModifier.value.queryOptions));
                    options = MS.Entertainment.Utilities.uniteObjects(options, WinJS.Binding.unwrap(filter.value.queryOptions));
                    options = MS.Entertainment.Utilities.uniteObjects(options, WinJS.Binding.unwrap(view.queryOptions));
                    return options
                }, createContainingMedia: function createContainingMedia() {
                    return null
                }, createSelectionHandlers: function createSelectionHandlers() {
                    return null
                }, makeEmpty: function makeEmpty() {
                    this._clear(true)
                }, clear: function clear() {
                    this._clear(false)
                }, _clear: function _clear(setEmptyResult) {
                    this._disposeQuery();
                    this._resetLastUsed();
                    this._setIsFailed(false);
                    this._setItems(setEmptyResult ? new MS.Entertainment.Data.VirtualList : null)
                }, refresh: function refresh() {
                    if (this._disposed)
                        return;
                    this._resetLastUsed();
                    this._enabled = true;
                    return this._refresh()
                }, refreshIfNeeded: function refreshIfNeeded() {
                    if (this._disposed)
                        return;
                    this._enabled = true;
                    return this._refresh()
                }, begin: function begin() {
                    var promise;
                    if (!this._enabled) {
                        this._enabled = true;
                        promise = this._refresh()
                    }
                    return WinJS.Promise.as(promise)
                }, waitForLoad: function waitForLoad() {
                    return WinJS.Promise.as(this._pendingViewChange).then(function hideResult(){})
                }, setCurrentFilter: function setCurrentFilter(filterId) {
                    if (this.filterSelectionManager && this.filterSelectionManager.dataSource) {
                        var foundFilter = false;
                        for (var i = 0; i < this.filterSelectionManager.dataSource.length; i++)
                            if (this.filterSelectionManager.dataSource[i].id === filterId) {
                                foundFilter = true;
                                this.filterSelectionManager.selectedIndex = i;
                                break
                            }
                        MS.Entertainment.ViewModels.assert(foundFilter, "setCurrentFilter: Filter not found: '" + filterId + "'")
                    }
                }, _raiseDelayLoadedEvent: function _raiseDelayLoadedEvent() {
                    this.dispatchEvent(MS.Entertainment.ViewModels.QueryViewModel.events.delayLoaded, {sender: this})
                }, _raiseShouldClearSelection: function _raiseShouldClearSelection() {
                    if (!this._disposed)
                        this.dispatchEvent(MS.Entertainment.ViewModels.QueryViewModel.events.shouldClearSelection, {sender: this})
                }, _handleModifierSelectionChanged: function _handleModifierSelectionChanged(args) {
                    if (!this.view || args.detail.oldSelection.index < 0 || this._refreshing)
                        return;
                    this._lastUsedSecondaryModifierDefinition = null;
                    this._resetSecondaryModifier();
                    this._beginQuery()
                }, _handleSecondaryModifierSelectionChanged: function _handleSecondaryModifierSelectionChanged(args) {
                    if (!this.view || args.detail.oldSelection.index < 0 || this._refreshing)
                        return;
                    this._beginQuery()
                }, _handleFilterSelectionChanged: function _handleFilterSelectionChanged(args) {
                    if (!this.view || args.detail.oldSelection.index < 0 || this._refreshing)
                        return;
                    this._beginQuery()
                }, _handlePivotSelectionChanged: function _handlePivotSelectionChanged(args) {
                    if (!this.view || args.detail.oldSelection.index < 0 || this._refreshing) {
                        if (this.isHorizontalLayout)
                            this._resetModifiers();
                        return
                    }
                    if (args.detail.newSelection.index >= 0 && this._modifiersTrimmed)
                        this._lastUsedModifierDefinition = null;
                    this._lastUsedSubPivotDefinition = null;
                    this._resetSubPivots();
                    this._refresh()
                }, _handleViewChange: function _handleViewChange() {
                    this._setItems(null);
                    this._lastUsedView = null;
                    this._lastUsedPivotDefinition = null;
                    this._lastUsedSubPivotDefinition = null;
                    this._lastUsedModifierDefinition = null;
                    this._lastUsedSecondaryModifierDefinition = null;
                    this._lastUsedFilterDefinition = null;
                    return this._refresh()
                }, _refresh: function _refresh() {
                    if (!this.view || !this._enabled || this._refreshing || this._disposed)
                        return WinJS.Promise.wrap();
                    this._refreshing = true;
                    if (this._pendingViewChange) {
                        this._pendingViewChange.cancel();
                        this._pendingViewChange = null
                    }
                    var promise = this._pendingViewChange = WinJS.Promise.join({
                            pivots: this._resetPivots(), modifier: this._resetModifiers(), filters: this._resetFilters()
                        }).then(this._viewLoadCompleted.bind(this), this._viewLoadFailed.bind(this));
                    this._refreshing = false;
                    return promise
                }, _resetPivots: function _resetPivots() {
                    var promise;
                    var staticPromise;
                    var queryPromise;
                    var pivotsQuery;
                    var pivots = this.getPivotDefinition(this.view);
                    var workingDataSource;
                    if (pivots !== this._lastUsedPivotDefinition) {
                        this._lastUsedPivotDefinition = pivots;
                        workingDataSource = new MS.Entertainment.ObservableArray;
                        this._lastUsedPivotItem = null;
                        MS.Entertainment.Utilities.BindingAgnostic.setProperties(this._workingPivotsSelectionManager, this.getPivotOptions(this.view));
                        this.pivotsSelectionManager = this._workingPivotsSelectionManager;
                        if (pivots) {
                            if (pivots.itemFactory) {
                                workingDataSource.spliceArray(workingDataSource.length, 0, pivots.itemFactory());
                                staticPromise = WinJS.Promise.wrap(true)
                            }
                            if (pivots.itemQuery) {
                                pivotsQuery = new pivots.itemQuery;
                                MS.Entertainment.Utilities.BindingAgnostic.setProperties(pivotsQuery, pivots.itemQueryOptions);
                                queryPromise = WinJS.Promise.as(staticPromise).then(function itemsArrayAdded() {
                                    return pivotsQuery.execute()
                                }.bind(this)).then(function queryFinished(q) {
                                    return MS.Entertainment.Data.List.listToArray(q.result.items)
                                }.bind(this), function queryFailed(error) {
                                    if (this._lastUsedPivotDefinition === pivots)
                                        this._lastUsedPivotDefinition = null;
                                    return WinJS.Promise.wrapError(error)
                                }.bind(this)).then(function gotArray(array) {
                                    workingDataSource.spliceArray(workingDataSource.length, 0, array);
                                    return true
                                }.bind(this))
                            }
                        }
                        this._workingPivotsSelectionManager.dataSource = workingDataSource;
                        promise = WinJS.Promise.as(staticPromise || queryPromise);
                        promise = promise.then(function refreshSubPivots() {
                            this._lastUsedSubPivotDefinition = null;
                            return this._resetSubPivots()
                        }.bind(this))
                    }
                    return WinJS.Promise.as(promise)
                }, _resetSubPivots: function _resetSubPivots() {
                    var promise;
                    var staticPromise;
                    var queryPromise;
                    var subPivotQuery;
                    var selectedPivot = this._workingPivotsSelectionManager.selectedItem;
                    var subPivot = this.getSubPivotDefinition(this.view, selectedPivot);
                    var workingDataSource;
                    if (subPivot !== this._lastUsedSubPivotDefinition) {
                        this._lastUsedSubPivotDefinition = subPivot;
                        var workingDataSource = [];
                        var subPivotOptions = this.getSubPivotOptions(this.view, selectedPivot) || {};
                        if (subPivot && selectedPivot) {
                            if (subPivot.itemFactory) {
                                workingDataSource = subPivot.itemFactory().concat(workingDataSource);
                                staticPromise = WinJS.Promise.wrap(true)
                            }
                            if (subPivot.itemQuery) {
                                subPivotQuery = new subPivot.itemQuery;
                                MS.Entertainment.Utilities.BindingAgnostic.setProperties(subPivotQuery, this.getSubPivotItemQueryOptions(this.view, selectedPivot));
                                this._createSubItemAugmentation(subPivotQuery, selectedPivot);
                                queryPromise = WinJS.Promise.as(staticPromise).then(function itemsArrayAdded() {
                                    return subPivotQuery.execute()
                                }).then(function queryFinished(q) {
                                    return MS.Entertainment.Data.List.listToArray(q.result.items)
                                }, function queryFailed(error) {
                                    if (this._lastUsedSubPivotDefinition === subPivot)
                                        this._lastUsedSubPivotDefinition = null;
                                    if (error.name === "Canceled")
                                        return WinJS.Promise.wrapError(error);
                                    else
                                        return []
                                }.bind(this)).then(function gotArray(array) {
                                    workingDataSource = workingDataSource.concat(array);
                                    return true
                                })
                            }
                            promise = WinJS.Promise.as(staticPromise || queryPromise);
                            promise.then(null, function ignoreError(error){}).done(function setItems() {
                                if (this._lastUsedSubPivotDefinition === subPivot && (typeof subPivotOptions.minItems !== "number" || isNaN(subPivotOptions.minItems) || subPivotOptions.minItems <= workingDataSource.length))
                                    selectedPivot.items = workingDataSource;
                                else if (selectedPivot.items)
                                    selectedPivot.items = null
                            }.bind(this))
                        }
                        else if (selectedPivot && selectedPivot.items)
                            selectedPivot.items = null
                    }
                    return WinJS.Promise.as(promise)
                }, _createSubItemAugmentation: function _createSubItemAugmentation(query, parent) {
                    query.resultAugmentation = MS.Entertainment.Data.AugmentQuery.modifyListResultAugmentation(query.resultAugmentation, {parent: parent})
                }, _resetLastUsed: function _resetLastUsed() {
                    this._lastUsedPivotItem = null;
                    this._lastUsedModifierItem = null;
                    this._lastUsedSecondaryModifierItem = null;
                    this._lastUsedFilterItem = null;
                    this._lastUsedView = null
                }, _resetModifiers: function _resetModifiers() {
                    var promise;
                    var staticPromise;
                    var queryPromise;
                    var modifierQuery;
                    var modifier = this.getModifierDefinition(this.view);
                    var workingDataSource;
                    if (modifier !== this._lastUsedModifierDefinition) {
                        this._lastUsedModifierDefinition = modifier;
                        workingDataSource = new MS.Entertainment.ObservableArray;
                        this._lastUsedModifierItem = null;
                        this._lastUsedSecondaryModifierItem = null;
                        this._modifiersTrimmed = false;
                        MS.Entertainment.Utilities.BindingAgnostic.setProperties(this._workingModifierSelectionManager, this.getModifierOptions(this.view));
                        this.modifierSelectionManager = this._workingModifierSelectionManager;
                        if (modifier) {
                            if (modifier.itemFactory) {
                                workingDataSource.spliceArray(workingDataSource.length, 0, modifier.itemFactory());
                                staticPromise = WinJS.Promise.wrap(true)
                            }
                            if (modifier.itemQuery) {
                                modifierQuery = new modifier.itemQuery;
                                MS.Entertainment.Utilities.BindingAgnostic.setProperties(modifierQuery, modifier.itemQueryOptions);
                                queryPromise = WinJS.Promise.as(staticPromise).then(function itemsArrayAdded() {
                                    return modifierQuery.execute()
                                }.bind(this)).then(function queryFinished(q) {
                                    return MS.Entertainment.Data.List.listToArray(q.result.items)
                                }.bind(this), function queryFailed(error) {
                                    if (this._lastUsedModifierDefinition === modifier)
                                        this._lastUsedModifierDefinition = null;
                                    if (error.name === "Canceled")
                                        return WinJS.Promise.wrapError(error);
                                    else
                                        return []
                                }.bind(this)).then(function gotArray(array) {
                                    workingDataSource.spliceArray(workingDataSource.length, 0, array);
                                    return true
                                }.bind(this))
                            }
                            if (modifier.options)
                                this._workingModifierSelectionManager.toggleModifier = modifier.options.toggleModifier;
                            promise = staticPromise || queryPromise
                        }
                        this._workingModifierSelectionManager.dataSource = workingDataSource;
                        this._lastUsedSecondaryModifierDefinition = null;
                        if (promise)
                            promise = promise.then(null, function ignoreError(error) {
                                MS.Entertainment.Data.fail("Failed loading the modifier.  Error: " + (error && error.message))
                            }).then(this._resetSecondaryModifier.bind(this))
                    }
                    return WinJS.Promise.as(promise)
                }, _resetFilters: function _resetFilters() {
                    var filters = this.getFilterDefinition(this.view);
                    if (filters !== this._lastUsedFilterDefinition) {
                        this._lastUsedFilterDefinition = filters;
                        this._lastUsedFilterItem = null;
                        MS.Entertainment.Utilities.BindingAgnostic.setProperties(this._workingFilterSelectionManager, this.getFilterOptions(this.view));
                        if (filters && filters.itemFactory)
                            this._workingFilterSelectionManager.dataSource = filters.itemFactory();
                        else if (filters && filters.options)
                            this._workingFilterSelectionManager.dataSource = filters.options;
                        else
                            this._workingFilterSelectionManager.dataSource = null;
                        this.filterSelectionManager = this._workingFilterSelectionManager
                    }
                    return WinJS.Promise.as(true)
                }, _resetSecondaryModifier: function _resetSecondaryModifier() {
                    var promise;
                    var staticPromise;
                    var queryPromise;
                    var secondaryModifierQuery;
                    var selectedModifier = this._workingModifierSelectionManager.selectedItem;
                    var secondaryModifier = this.getSecondaryModifierDefinition(this.view, selectedModifier);
                    var workingDataSource;
                    this.secondaryModifierSelectionManager = this._workingSecondaryModifierSelectionManager;
                    if (secondaryModifier !== this._lastUsedSecondaryModifierDefinition) {
                        this._lastUsedSecondaryModifierDefinition = secondaryModifier;
                        var workingDataSource = new MS.Entertainment.ObservableArray;
                        MS.Entertainment.Utilities.BindingAgnostic.setProperties(this._workingModifierSelectionManager, this.getSecondaryModifierOptions(this.view, selectedModifier));
                        if (secondaryModifier) {
                            if (secondaryModifier.itemFactory) {
                                workingDataSource.spliceArray(workingDataSource.length, 0, secondaryModifier.itemFactory());
                                staticPromise = WinJS.Promise.wrap(true)
                            }
                            if (secondaryModifier.itemQuery) {
                                secondaryModifierQuery = new secondaryModifier.itemQuery;
                                MS.Entertainment.Utilities.BindingAgnostic.setProperties(secondaryModifierQuery, this.getSecondaryModifierItemQueryOptions(this.view, selectedModifier));
                                queryPromise = WinJS.Promise.as(staticPromise).then(function itemsArrayAdded() {
                                    return secondaryModifierQuery.execute()
                                }.bind(this)).then(function queryFinished(q) {
                                    return MS.Entertainment.Data.List.listToArray(q.result.items)
                                }.bind(this), function queryFailed(error) {
                                    if (this._lastUsedSecondaryModifierDefinition === secondaryModifier)
                                        this._lastUsedSecondaryModifierDefinition = null;
                                    if (error.name === "Canceled")
                                        return WinJS.Promise.wrapError(error);
                                    else
                                        return []
                                }.bind(this)).then(function gotArray(array) {
                                    workingDataSource.spliceArray(workingDataSource.length, 0, array);
                                    return true
                                }.bind(this))
                            }
                            promise = staticPromise || queryPromise
                        }
                        this._workingSecondaryModifierSelectionManager.dataSource = workingDataSource
                    }
                    else if (this._workingSecondaryModifierSelectionManager.dataSource)
                        this._workingSecondaryModifierSelectionManager.dataSource = null;
                    return WinJS.Promise.as(promise)
                }, _viewLoadExit: function _viewLoadExit(results) {
                    this._pendingViewChange = null
                }, _viewLoadCompleted: function _viewLoadCompleted(results) {
                    this._viewLoadExit(results);
                    if (results.modifier && (!this._workingModifierSelectionManager.dataSource || !this._workingModifierSelectionManager.dataSource.length))
                        this.modifierSelectionManager = null;
                    if (results.secondaryModifier && (!this._workingSecondaryModifierSelectionManager.dataSource || !this._workingSecondaryModifierSelectionManager.dataSource.length))
                        this.secondaryModifierSelectionManager = null;
                    if (results.filter && (!this._workingFilterSelectionManager.dataSource || !this._workingFilterSelectionManager.dataSource.length))
                        this.filterSelectionManager = null;
                    if (results.pivots && (!this._workingPivotsSelectionManager.dataSource || !this._workingPivotsSelectionManager.dataSource.length))
                        this.pivotsSelectionManager = null;
                    return this._beginQuery()
                }, _viewLoadFailed: function _viewLoadFailed(results) {
                    this._viewLoadExit({
                        modifier: [], pivots: []
                    });
                    this._setItems(null);
                    this._setIsFailed(true);
                    return WinJS.Promise.wrapError(new Error("Pivots, modifiers or filters failed to load"))
                }, _updateSelectionHandlers: function _updateSelectionHandlers() {
                    if (this.isDelayInitialized && (this.completedQuery || this.items))
                        this.selectionHandlers = this.createSelectionHandlers()
                }, createActionCells: function createActionCells() {
                    return null
                }, createModifierActionCells: function createModifierActionCells(groupHeader, modifierTitleIds, showIfEmptyModifiers) {
                    var modifiers = [this.modifierSelectionManager, this.secondaryModifierSelectionManager];
                    var actionCells = [];
                    modifiers.forEach(function createModifierCell(modifier, i) {
                        if ((modifier.dataSource && modifier.dataSource.length > 1) || (showIfEmptyModifiers && showIfEmptyModifiers[i])) {
                            var newActionCell = new MS.Entertainment.ViewModels.ModifierActionCell(modifier, groupHeader, modifierTitleIds ? modifierTitleIds[i] : null, showIfEmptyModifiers ? showIfEmptyModifiers[i] : null);
                            actionCells.push(newActionCell)
                        }
                    });
                    return actionCells
                }, cloneCurrentQuery: function cloneCurrentQuery() {
                    return null
                }, refreshQuery: function refreshQuery() {
                    this._lastUsedView = null;
                    return this._beginQuery()
                }, _disposeQuery: function _disposeQuery() {
                    if (this._lastUsedQuery && this._lastUsedQuery.dispose) {
                        this._lastUsedQuery.dispose();
                        this._lastUsedQuery = null
                    }
                    if (this._queryWatcher) {
                        this._queryWatcher.clearQueries();
                        this._queryWatcher = null
                    }
                    this._unregisterForQueryEvents()
                }, _beginQuery: function _beginQuery() {
                    if (this._disposed)
                        return WinJS.Promise.wrap();
                    return WinJS.Promise.as(this._onDelayBeginQuery()).then(function _beginQueryWork() {
                            var pivot;
                            var modifier;
                            var secondaryModifier;
                            var filter;
                            var pivotItem = this._workingPivotsSelectionManager.selectedItem;
                            var modifierItem = this._workingModifierSelectionManager.selectedItem;
                            var secondaryModifierItem = this._workingSecondaryModifierSelectionManager.selectedItem;
                            var filterItem = this._workingFilterSelectionManager.selectedItem;
                            var view = this.getViewDefinition(this.view);
                            if (!view || !this._enabled)
                                return WinJS.Promise.wrap();
                            if (this._lastUsedView === view && this._lastUsedPivotItem === pivotItem && this._lastUsedModifierItem === modifierItem && this._lastUsedSecondaryModifierItem === secondaryModifierItem && this._lastUsedFilterItem == filterItem)
                                return WinJS.Promise.wrap();
                            pivot = this._workingPivotsSelectionManager.selectedItem;
                            modifier = this._workingModifierSelectionManager.selectedItem;
                            secondaryModifier = this._workingSecondaryModifierSelectionManager.selectedItem;
                            filter = this._workingFilterSelectionManager.selectedItem;
                            this._lastUsedView = view;
                            this._lastUsedPivotItem = pivotItem;
                            this._lastUsedModifierItem = modifierItem;
                            this._lastUsedSecondaryModifierItem = secondaryModifier;
                            this._lastUsedFilterItem = filterItem;
                            pivot = pivot || {value: {}};
                            modifier = modifier || {value: {}};
                            secondaryModifier = secondaryModifier || {value: {}};
                            filter = filter || {value: {}};
                            var options;
                            options = MS.Entertainment.Utilities.uniteObjects(WinJS.Binding.unwrap(pivot.value.modelOptions), WinJS.Binding.unwrap(modifier.value.modelOptions));
                            options = MS.Entertainment.Utilities.uniteObjects(options, WinJS.Binding.unwrap(secondaryModifier.modelOptions));
                            options = MS.Entertainment.Utilities.uniteObjects(options, WinJS.Binding.unwrap(filter.value.modelOptions));
                            options = MS.Entertainment.Utilities.uniteObjects(options, WinJS.Binding.unwrap(view.modelOptions));
                            MS.Entertainment.Utilities.BindingAgnostic.setProperties(this, options);
                            return this._handleBeginQuery(view, pivot, modifier, secondaryModifier, filter)
                        }.bind(this))
                }, _handleBeginQuery: function _handleBeginQuery(view, pivot, modifier, secondaryModifier, filter) {
                    var options;
                    var sender;
                    var QueryType;
                    var promise;
                    if (this._pendingQueryExecute) {
                        this._pendingQueryExecute.cancel();
                        this._pendingQueryExecute = null
                    }
                    this.completedQuery = null;
                    this.containingMedia = null;
                    this.selectionHandlers = null;
                    this._unregisterForQueryEvents();
                    if (!this.forceEmpty)
                        QueryType = this.getQueryType(view, modifier, pivot);
                    if ((this._lastUsedQueryType !== QueryType || this._recreateQueries) && QueryType) {
                        this._disposeQuery();
                        this._lastUsedQuery = new QueryType;
                        this._lastUsedQueryType = QueryType;
                        this._queryWatcher = new MS.Entertainment.Framework.QueryWatcher(this._queryWatcherString + this.view);
                        this._queryWatcher.registerQuery(this._lastUsedQuery);
                        if (this.taskKeyGetter && this.propertyKey && this.notifier) {
                            var notifications = new MS.Entertainment.UI.ContentNotification.NotificationModification(MS.Entertainment.UI.ContentNotification.listResult(), MS.Entertainment.UI.FileTransferService.keyFromProperty(this.propertyKey, false, true));
                            sender = notifications.createSender();
                            notifications.modifyQuery(this._lastUsedQuery)
                        }
                    }
                    else if (!QueryType) {
                        this._disposeQuery();
                        promise = WinJS.Promise.wrap()
                    }
                    if (this._lastUsedQuery) {
                        options = this.getQueryOptions(view, pivot, modifier, secondaryModifier, filter);
                        MS.Entertainment.Utilities.BindingAgnostic.setProperties(this._lastUsedQuery, options);
                        this._onBeginQuery(this._lastUsedQuery);
                        this._setItems(null);
                        var queryFinishedOnExecute = false;
                        promise = this._pendingQueryExecute = this._lastUsedQuery.execute();
                        promise.then(function queryCompleted(q) {
                            if (promise !== this._pendingQueryExecute)
                                return;
                            if (this.failOnEmpty && this._isEmptyListFromQuery(q))
                                this._handleQueryFailure();
                            else {
                                queryFinishedOnExecute = true;
                                this._pendingQueryExecute = null;
                                this.completedQuery = q;
                                if (this.completedQuery.autoUpdateProperties && this.completedQuery.autoUpdateProperties.enabled)
                                    this.completedQuery.autoUpdateProperties.enabled = false;
                                this.completedQuery.enabled = true;
                                if (sender) {
                                    var fileTransferService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.fileTransfer);
                                    fileTransferService.registerListener(this._fileTransferListenerId, this.taskKeyGetter, sender, this.notifier)
                                }
                                var eventProvider = new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell;
                                eventProvider.traceNavigable_Loading_Done(this._loadingDoneString);
                                this._setItemsFromQuery(q);
                                this._registerForQueryEvents(q);
                                this._onQueryCompleted(q);
                                this._updateSelectionHandlers()
                            }
                        }.bind(this), function queryFailed(error) {
                            if (promise !== this._pendingQueryExecute)
                                return;
                            this._onQueryFailed(error);
                            this._handleQueryFailure(error);
                            return WinJS.Promise.wrapError(error)
                        }.bind(this))
                    }
                    else
                        this._setItems(null);
                    if (queryFinishedOnExecute)
                        this._pendingQueryExecute = null;
                    MS.Entertainment.Data.assert(!!promise, "We should be returning the query's execute promise");
                    return WinJS.Promise.as(promise)
                }, _handleQueryFailure: function _handleQueryFailure(error) {
                    this._pendingQueryExecute = null;
                    if (!error || !WinJS.Promise.isCanceledError(error)) {
                        if (!this._attemptAnotherQuery()) {
                            this._setIsFailed(true);
                            this._setItems(null)
                        }
                        if (error && error.details && error.details.query) {
                            error.details.query.enabled = true;
                            this._registerForQueryEvents(error.details.query)
                        }
                    }
                }, _isEmptyListFromQuery: function _isEmptyListFromQuery(query) {
                    return !query || !query.result || !query.result.items || query.result.items.count <= 0
                }, _setItemsFromQuery: function _setItemsFromQuery(queryOrEvent) {
                    var result;
                    var error;
                    if (queryOrEvent) {
                        result = queryOrEvent.detail ? queryOrEvent.detail.result : queryOrEvent.result;
                        error = queryOrEvent.detail ? queryOrEvent.detail.error : queryOrEvent.error
                    }
                    if (error) {
                        this._onQueryFailed(error);
                        this._setIsFailed(true);
                        this._setItems(null)
                    }
                    else if (result) {
                        if (!result.items)
                            result.items = new MS.Entertainment.Data.VirtualList(null, []);
                        this._setIsFailed(false);
                        this._setItems(result.items)
                    }
                }, _registerForQueryEvents: function _registerForQueryEvents(query) {
                    this._unregisterForQueryEvents();
                    if (query && this.handleQueryChanges)
                        this._queryEventHandlers = MS.Entertainment.Utilities.addEventHandlers(query, {resultChanged: this._setItemsFromQuery.bind(this)})
                }, _unregisterForQueryEvents: function _unregisterForQueryEvents() {
                    if (this._queryEventHandlers) {
                        this._queryEventHandlers.cancel();
                        this._queryEventHandlers = null
                    }
                }, _setIsFailed: function _setIsFailed(value) {
                    if (this.isFailed !== value) {
                        var oldValue = this.isFailed;
                        this.isFailed = value
                    }
                }, _setItems: function _setItems(items) {
                    if (this.items !== items) {
                        var actionCells;
                        if (items) {
                            actionCells = this.createActionCells(items.count);
                            if (actionCells && !Array.isArray(actionCells))
                                actionCells = [actionCells];
                            if (actionCells && actionCells.length > 0 && this.addActionCellsToList) {
                                actionCells = actionCells.map(function(currentAction) {
                                    return new MS.Entertainment.Data.Factory.ListActionItemWrapper(currentAction)
                                });
                                if (this.addActionCellsAtEnd)
                                    items.insertRangeAtEnd(actionCells);
                                else
                                    items.insertRangeAt(0, actionCells)
                            }
                            else
                                this.modelActions = actionCells
                        }
                        this._onItemsChanging(items, this.items);
                        var oldItems = this.items;
                        this.items = items;
                        this._onItemsChanged(this.items, oldItems);
                        this.containingMedia = this.items ? this.createContainingMedia() : null
                    }
                }, _onItemsChanging: function _onItemsChanging(newValue, oldValue){}, _onItemsChanged: function _onItemsChanged(newValue, oldValue){}, _setLargeItemIndex: function _setLargeItemIndex(value) {
                    if (this.largeItemIndex !== value) {
                        var oldValue = this.largeItemIndex;
                        this.largeItemIndex = value
                    }
                }, _onDelayBeginQuery: function _onDelayBeginQuery() {
                    return WinJS.Promise.as()
                }, _onBeginQuery: function _onBeginQuery(query){}, _onQueryFailed: function _onQueryFailed(error){}, _onQueryCompleted: function _onQueryCompleted(query){}, _attemptAnotherQuery: function _attemptAnotherQuery() {
                    var changedModifier = false;
                    var oldDataSource;
                    if (this.autoHideInvalidModifiers && !this._disposed && this._workingModifierSelectionManager) {
                        if (this._workingModifierSelectionManager.dataSource && this._workingModifierSelectionManager.selectedIndex >= 0) {
                            changedModifier = true;
                            this._modifiersTrimmed = true;
                            this._workingModifierSelectionManager.dataSource.removeAt(this._workingModifierSelectionManager.selectedIndex)
                        }
                        if (this._workingModifierSelectionManager.dataSource && this._workingModifierSelectionManager.dataSource.getArray)
                            oldDataSource = WinJS.Binding.unwrap(this._workingModifierSelectionManager.dataSource.getArray());
                        else
                            oldDataSource = [];
                        this._workingModifierSelectionManager.dataSource = new MS.Entertainment.ObservableArray(oldDataSource)
                    }
                    return changedModifier
                }
        }, {events: {
                modifierClick: "modifierClick", secondaryModifierClick: "secondaryModifierClick", shouldClearSelection: "shouldClearSelection", delayLoaded: "delayLoaded"
            }})})
})()
