﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
(function() {
    "use strict";
    scriptValidator();
    var modifierAlreadyOpen = false;
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Controls");
    WinJS.Namespace.define("MS.Entertainment.UI.Controls", {
        Modifier: MS.Entertainment.UI.Framework.defineUserControl("/Controls/Modifier.html#modifierTemplate", function modifierConstructor() {
            this._observableArrayChanged = this._observableArrayChanged.bind(this)
        }, {
            _selectionManagerEventHandlers: null, openPopup: null, minItems: 0, showDropDownArrow: true, showDescription: true, _tabPanel: null, isBeingDismissed: false, _selectionManager: null, calculateModifierPopoverPosition: true, allowSingleItemToShowList: false, _overrideDescription: false, _labelOverride: String.empty, _automaticFocusOverride: String.empty, _selectedItem: null, _items: null, popupShouldOpenAboveButton: false, deferUpdatesUntilInitialized: false, useSelectionManagerDataSourceAsItems: false, identifier: "", initialize: function initialize() {
                    if (!this.useSelectionManagerDataSourceAsItems)
                        this.selectionManager = new MS.Entertainment.UI.Framework.SelectionManager;
                    this.bind("descriptionLabel", this._descriptionLabelChanged.bind(this));
                    this.bind("descriptionLabelText", this._descriptionLabelChanged.bind(this));
                    this.bind("isRoamingSetting", this._settingsChanged.bind(this));
                    this.bind("settingsKey", this._settingsChanged.bind(this));
                    if (this.deferUpdatesUntilInitialized)
                        this._observableArrayChanged()
                }, unload: function unload() {
                    this.isBeingDismissed = true;
                    if (this.openPopup)
                        this.openPopup.hide();
                    if (this._selectionManagerEventHandlers) {
                        this._selectionManagerEventHandlers.cancel();
                        this._selectionManagerEventHandlers = null
                    }
                    MS.Entertainment.UI.Framework.UserControl.prototype.unload.call(this)
                }, freeze: function freeze() {
                    if (!MS.Entertainment.Utilities.isApp2) {
                        this.isBeingDismissed = true;
                        if (this.openPopup)
                            this.openPopup.hide()
                    }
                    MS.Entertainment.UI.Framework.UserControl.prototype.freeze.call(this)
                }, thaw: function thaw() {
                    MS.Entertainment.UI.Framework.UserControl.prototype.thaw.call(this);
                    this.isBeingDismissed = false
                }, selectionManager: {
                    get: function() {
                        return this._selectionManager
                    }, set: function(value) {
                            if (this._selectionManager !== value) {
                                var oldValue = this._selectionManager;
                                this._selectionManager = value;
                                if (this._selectionManagerEventHandlers) {
                                    this._selectionManagerEventHandlers.cancel();
                                    this._selectionManagerEventHandlers = null
                                }
                                this._selectionManagerEventHandlers = MS.Entertainment.Utilities.addEventHandlers(this._selectionManager, {
                                    selectedItemChanged: this._selectionManagerSelectionChanged.bind(this), dataSourceChanged: this._selectionManagerDataContextChanged.bind(this)
                                });
                                this._selectionManagerDataContextChanged();
                                this._selectionManagerSelectionChanged();
                                this.notify("selectionManager", value, oldValue)
                            }
                        }
                }, selectedItem: {
                    get: function() {
                        return this._selectedItem
                    }, set: function(value) {
                            if (value !== WinJS.Binding.unwrap(this._selectedItem)) {
                                var oldValue = this._selectedItem;
                                this._selectedItem = value;
                                this._selectedItemChanged();
                                this.notify("selectedItem", value, oldValue)
                            }
                        }
                }, items: {
                    get: function() {
                        return this._items
                    }, set: function(value) {
                            if (value !== WinJS.Binding.unwrap(this._items)) {
                                var oldValue = this._items;
                                this._items = value;
                                this._itemsChanged(value, oldValue);
                                this.notify("items", value, oldValue)
                            }
                        }
                }, overrideDescription: {
                    get: function() {
                        return this._overrideDescription
                    }, set: function(value) {
                            this.updateAndNotify("overrideDescription", value);
                            if (this.overrideDescription)
                                this.descriptionLabelVisible = true
                        }
                }, labelOverride: {
                    get: function() {
                        return this._labelOverride
                    }, set: function(value) {
                            this._labelOverride = value;
                            if (this.labelControl && !this.overrideDescription)
                                this.labelControl.textContent = value;
                            else if (this.labelDescription && this.overrideDescription)
                                this.labelDescription.textContent = value;
                            if (this.isVoiceEnabled && this.labelOverride)
                                this.voicePhrase = this.labelOverride
                        }
                }, automaticFocusOverride: {
                    get: function() {
                        return this._automaticFocusOverride
                    }, set: function(value) {
                            this._automaticFocusOverride = value;
                            var focusItem = this.domElement.querySelector(".win-focusable");
                            if (focusItem)
                                focusItem.setAttribute("data-win-focus", this._automaticFocusOverride)
                        }
                }, _settingsChanged: function settingsChanged() {
                    if (this._selectionManager && this.settingsKey) {
                        this._selectionManager.isRoamingSetting = !MS.Entertainment.Utilities.isVideoApp2 && this.isRoamingSetting;
                        this._selectionManager.settingsKey = this.settingsKey
                    }
                }, _itemsChanged: function _itemsChanged(newValue, oldValue) {
                    if (oldValue && oldValue instanceof MS.Entertainment.ObservableArray)
                        oldValue.removeChangeListener(this._observableArrayChanged);
                    if (newValue && newValue instanceof MS.Entertainment.ObservableArray)
                        newValue.addChangeListener(this._observableArrayChanged);
                    if (this.selectionManager && !this.useSelectionManagerDataSourceAsItems)
                        if (this.selectionManager.dataSource !== this.items) {
                            this.selectionManager.settingsKey = this.settingsKey;
                            this.selectionManager.dataSource = this.items
                        }
                    this._observableArrayChanged()
                }, _observableArrayChanged: function _observableArrayChanged() {
                    if (this.deferUpdatesUntilInitialized && !this._initialized)
                        return;
                    this._constrainSelectedItem();
                    this._updateArrowVisibility();
                    this._updateControlVisibility();
                    if (!isNaN(this.tabIndex)) {
                        var elementToAttribute = this.domElement;
                        if (this._modifierContainer)
                            elementToAttribute = this._modifierContainer;
                        if (elementToAttribute)
                            if (this.items && (this.items.length > 1 || (this.items.length == 1 && this.allowSingleItemToShowList))) {
                                elementToAttribute.setAttribute("tabindex", this.tabIndex);
                                elementToAttribute.setAttribute("aria-haspopup", true);
                                elementToAttribute.disabled = false
                            }
                            else {
                                elementToAttribute.setAttribute("tabindex", -1);
                                elementToAttribute.removeAttribute("aria-haspopup");
                                elementToAttribute.disabled = true
                            }
                    }
                }, _selectedItemChanged: function _selectedItemChanged() {
                    var label;
                    var labelType;
                    this._constrainSelectedItem();
                    if (this.selectionManager)
                        this.selectionManager.selectedItem = this.selectedItem;
                    label = String.empty;
                    if (this.selectedItem && (this.selectedItem.label || this.selectedItem.title))
                        label = this.selectedItem.label ? this.selectedItem.label : this.selectedItem.title;
                    labelType = typeof label;
                    switch (labelType) {
                        case"string":
                            if (String.id[label])
                                label = String.load(label);
                            break;
                        default:
                            MS.Entertainment.UI.Controls.assert(false, "Unrecognized label type in modifier control.");
                            break
                    }
                    if (this.labelControl && (!this.labelOverride || this.overrideDescription))
                        this.labelControl.textContent = label;
                    if (this.isVoiceEnabled && !this.labelOverride)
                        this.voicePhrase = label;
                    (new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell).traceModifierControl_SelectionChanged(label, this.identifier)
                }, _selectionManagerSelectionChanged: function _selectionManagerSelectionChanged() {
                    var newSelectedItem = this.selectionManager ? this.selectionManager.selectedItem : null;
                    if (!this.selectedItem || this.selectedItem !== newSelectedItem)
                        this.selectedItem = newSelectedItem
                }, _selectionManagerDataContextChanged: function _selectionManagerDataContextChanged() {
                    if (this.useSelectionManagerDataSourceAsItems)
                        this.items = this._selectionManager && this._selectionManager.dataSource
                }, _descriptionLabelChanged: function _descriptionLabelChanged() {
                    if (!this.showDescription || !this.labelDescription || this.overrideDescription)
                        return;
                    if (this.descriptionLabel) {
                        this.labelDescription.textContent = String.load(this.descriptionLabel);
                        this.descriptionLabelVisible = true
                    }
                    else if (this.descriptionLabelText) {
                        this.labelDescription.textContent = this.descriptionLabelText;
                        this.descriptionLabelVisible = true
                    }
                    else
                        this.descriptionLabelVisible = false
                }, _constrainSelectedItem: function _constrainSelectedItem() {
                    if (this.useSelectionManagerDataSourceAsItems) {
                        this.selectionItem = this.selectionManager && this.selectionManager.selectedItem;
                        return
                    }
                    if (!this.items)
                        this.selectedItem = null;
                    else if (this.items.length <= this.minItems)
                        this.selectedItem = null;
                    else if (this.items instanceof MS.Entertainment.ObservableArray) {
                        var found = false;
                        for (var i = 0; i < this.items.length; i++)
                            if (this.items.item(i) === this.selectedItem || this.items.item(i) === WinJS.Binding.unwrap(this.selectedItem)) {
                                found = true;
                                break
                            }
                        if (!found)
                            this.selectedItem = (this.selectionManager && this.selectionManager.selectedItem) || this.items.item(0)
                    }
                    else if (this.items.indexOf(WinJS.Binding.unwrap(this.selectedItem)) < 0) {
                        var found = false;
                        for (var i = 0; i < this.items.length; i++)
                            if (this.items[i] === this.selectedItem || this.items[i] === WinJS.Binding.unwrap(this.selectedItem)) {
                                found = true;
                                break
                            }
                        if (!found)
                            this.selectedItem = (this.selectionManager && this.selectionManager.selectedItem)
                    }
                }, _updateControlVisibility: function _updateControlVisibility() {
                    if (!this.overrideControlVisibility)
                        if (!this.items || this.items.length <= this.minItems)
                            WinJS.Utilities.addClass(this.domElement, "removeFromDisplay");
                        else
                            WinJS.Utilities.removeClass(this.domElement, "removeFromDisplay")
                }, _updateArrowVisibility: function _updateArrowVisibility() {
                    var elementToAttribute = this.domElement;
                    if (this._modifierContainer)
                        elementToAttribute = this._modifierContainer;
                    if (!this.items || (!this.allowSingleItemToShowList && this.items.length <= 1))
                        WinJS.Utilities.addClass(elementToAttribute, "dropDownDisabled");
                    else
                        WinJS.Utilities.removeClass(elementToAttribute, "dropDownDisabled");
                    if (!this.dropDownArrow)
                        return null;
                    if (this.showDropDownArrow && (!MS.Entertainment.Utilities.isMusicApp2 || (this.items && this.items.length >= this.minItems)))
                        WinJS.Utilities.removeClass(this.dropDownArrow, "removeFromDisplay");
                    else
                        WinJS.Utilities.addClass(this.dropDownArrow, "removeFromDisplay")
                }, setTabPanel: function setTabPanel(tabPanel) {
                    this._tabPanel = tabPanel
                }, onKeyDown: function onKeyDown(event) {
                    if (event.keyCode === WinJS.Utilities.Key.enter || event.keyCode === WinJS.Utilities.Key.space) {
                        this.onClicked();
                        event.stopPropagation();
                        event.preventDefault()
                    }
                }, onClicked: function onClicked() {
                    var boundingRect;
                    if (!this.items || (!this.allowSingleItemToShowList && this.items.length <= 1))
                        return;
                    var elementToAttribute = this.domElement;
                    if (this._modifierContainer)
                        elementToAttribute = this._modifierContainer;
                    if (this.toggleModifier) {
                        var selectedItem = this.selectionManager.selectedItem;
                        var currentSelectedIndex = Math.max(this.selectionManager.dataSource.indexOf(selectedItem), 0);
                        var items = this.selectionManager.dataSource;
                        var newIndex = (currentSelectedIndex + 1) % 2;
                        if (items instanceof MS.Entertainment.ObservableArray)
                            this.selectionManager.selectedItem = items.item(newIndex);
                        else if (Array.isArray(items))
                            this.selectionManager.selectedItem = items[newIndex];
                        else
                            MS.Entertainment.UI.Controls.fail("Unrecognized items list type in modifier control.")
                    }
                    else if (!this.openPopup && !modifierAlreadyOpen) {
                        var anchorElement;
                        if (this.calculateModifierPopoverPosition)
                            anchorElement = elementToAttribute;
                        this.openPopup = MS.Entertainment.UI.Controls.ModifierPopup.createModifierPopup(this.popupStyle, this.selectionManager, {
                            elementToAnchorTo: anchorElement, modifierControl: this, popupShouldOpenAboveButton: this.popupShouldOpenAboveButton
                        });
                        if (!this.isBeingDismissed) {
                            modifierAlreadyOpen = true;
                            if (elementToAttribute)
                                WinJS.Utilities.addClass(elementToAttribute, "showingList");
                            this.openPopup.show().then(function traceModifierPopupDismissed() {
                                if (elementToAttribute)
                                    WinJS.Utilities.removeClass(elementToAttribute, "showingList");
                                modifierAlreadyOpen = false;
                                this.openPopup = null;
                                (new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell).traceModifierControl_PopupClosed(this.identifier)
                            }.bind(this))
                        }
                    }
                }, getItem: function getItem(index) {
                    var items = WinJS.Binding.unwrap(this.items);
                    if (items instanceof MS.Entertainment.ObservableArray)
                        return items.item(index);
                    else if (Array.isArray(items))
                        return items[index];
                    else
                        MS.Entertainment.UI.Controls.fail("Unrecognized items list type in modifier control.")
                }
        }, {
            tabIndex: 0, descriptionLabel: null, descriptionLabelText: null, descriptionLabelVisible: false, popupStyle: null, settingsKey: null, isRoamingSetting: !MS.Entertainment.Utilities.isVideoApp2, overrideControlVisibility: false
        }), ModifierPopup: MS.Entertainment.UI.Framework.defineUserControl("/Controls/Modifier.html#modifierPopupTemplate", function modifierPopupConstructor(){}, {
                modifierControl: null, customStyle: null, topPadding: 0, bottomPadding: 0, selectedItemIndex: 0, _guesstimatedItemHeightPx: 50, _keyboardNavigationManager: null, _eventHandlers: null, _uiStateHandlers: null, descriptionLabel: null, anchorElement: null, _parentOverlay: null, initialize: function initialize() {
                        this._keyboardNavigationManager = new MS.Entertainment.Framework.KeyboardNavigationManager(this.scrollerElement, this.scrollerElement);
                        if (Array.isArray(this.customStyle))
                            this.customStyle.forEach(function(style) {
                                WinJS.Utilities.addClass(this.domElement, style)
                            }, this);
                        else if (this.customStyle)
                            WinJS.Utilities.addClass(this.domElement, this.customStyle);
                        this._eventHandlers = MS.Entertainment.Utilities.addEventHandlers(this.domElement, {
                            keypress: function onKeyPress(event) {
                                if (event.keyCode === WinJS.Utilities.Key.escape)
                                    this.modifierControl.openPopup.hide()
                            }.bind(this), keyup: function onKeyUp(event) {
                                    if (event.keyCode === WinJS.Utilities.Key.dismissButton) {
                                        event.stopPropagation();
                                        this.modifierControl.openPopup.hide()
                                    }
                                }.bind(this)
                        });
                        if (this.modifierControl.descriptionLabel)
                            this.descriptionLabel = String.load(this.modifierControl.descriptionLabel);
                        else if (this.modifierControl.descriptionLabelText)
                            this.descriptionLabel = this.modifierControl.descriptionLabelText;
                        if (this.anchorElement) {
                            var uiState = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                            this._uiStateHandlers = MS.Entertainment.UI.Framework.addEventHandlers(uiState, {windowresize: this.hide.bind(this)})
                        }
                        this.buildItems()
                    }, unload: function unload() {
                        if (this._eventHandlers) {
                            this._eventHandlers.cancel();
                            this._eventHandlers = null
                        }
                        if (this._uiStateHandlers) {
                            this._uiStateHandlers.cancel();
                            this._uiStateHandlers = null
                        }
                        this._parentOverlay = null;
                        MS.Entertainment.UI.Framework.UserControl.prototype.unload.call(this)
                    }, hide: function hide() {
                        if (this._parentOverlay)
                            this._parentOverlay.hide()
                    }, setOverlay: function setOverlay(overlay) {
                        this._parentOverlay = overlay;
                        overlay.lightDismissEnabled = true
                    }, scrollerElement: {get: function() {
                            if (this.scroller && this.scroller._element)
                                return this.scroller._element;
                            return this.scroller
                        }}, buildItems: function buildItems() {
                        var items = [];
                        var i;
                        var selectedItem;
                        var indexOfSelectedItem = -1;
                        var entriesLoaded = 0;
                        this.domElement.addEventListener("ModifierPopupEntryLoaded", function entryLoaded(event) {
                            event.stopPropagation();
                            entriesLoaded++;
                            if (entriesLoaded === (indexOfSelectedItem + 1)) {
                                window.requestAnimationFrame(function() {
                                    if (this.entryList)
                                        this.entryList.bringItemIntoView(items[indexOfSelectedItem]);
                                    if (this.scrollerElement)
                                        this._keyboardNavigationManager.setFocusedItem(this.scrollerElement.querySelector(".initialSelectedModifierPopupEntry"))
                                }.bind(this));
                                (new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell).traceModifierControl_PopupOpened(this.modifierControl.identifier)
                            }
                        }.bind(this));
                        if (this.selectionManager && this.selectionManager.dataSource) {
                            selectedItem = WinJS.Binding.unwrap(this.selectionManager.selectedItem);
                            for (i = 0; i < this.selectionManager.dataSource.length; i++) {
                                var listItem = {
                                        item: this.selectionManager.dataSource.item ? this.selectionManager.dataSource.item(i) : this.selectionManager.dataSource[i], modifierControl: this.modifierControl, selectionManager: this.selectionManager
                                    };
                                if (!listItem.item.canExecute || listItem.item.canExecute()) {
                                    if (indexOfSelectedItem === -1 && (!selectedItem || listItem.item === selectedItem)) {
                                        indexOfSelectedItem = i;
                                        this.selectedItemIndex = i;
                                        listItem.isInitialSelected = true
                                    }
                                    items.push(listItem)
                                }
                            }
                            if (indexOfSelectedItem === -1 && items.length > 0) {
                                items[0].isInitialSelected = true;
                                indexOfSelectedItem = 0
                            }
                            var screenItems = Math.round(((MS.Entertainment.Utilities.getWindowHeight() - 150) / this._guesstimatedItemHeightPx));
                            this.entryList.longListDeferalItemLimit = screenItems;
                            this.entryList.dataSource = items
                        }
                    }
            }, {items: null}, {
                calculatePositionFromBoundingRect: function calculatePositionFromBoundingRect(boundingRect, popupShouldOpenAboveButton) {
                    var left;
                    var right;
                    switch (MS.Entertainment.Utilities.getTextDirection()) {
                        case MS.Entertainment.Utilities.TextDirections.LeftToRight:
                            left = Math.round(boundingRect.left) + "px";
                            right = "auto";
                            break;
                        case MS.Entertainment.Utilities.TextDirections.RightToLeft:
                            right = (document.body.clientWidth - Math.round(boundingRect.right)) + "px";
                            left = "auto";
                            break
                    }
                    var top = popupShouldOpenAboveButton ? "auto" : Math.round(boundingRect.top + boundingRect.height) + "px";
                    var bottom = popupShouldOpenAboveButton ? document.body.clientHeight - Math.round(boundingRect.top) + "px" : "auto";
                    return {
                            left: left, top: top, right: right, bottom: bottom
                        }
                }, createModifierPopup: function createModifierPopup(popupStyle, selectionManager, optionalParameters) {
                        var anchorElement = optionalParameters && optionalParameters.elementToAnchorTo;
                        var boundingRect;
                        var modifierControl = optionalParameters && optionalParameters.modifierControl;
                        var openAbove = optionalParameters && optionalParameters.popupShouldOpenAboveButton;
                        if (anchorElement)
                            boundingRect = anchorElement.getBoundingClientRect();
                        boundingRect = boundingRect || {
                            left: 0, width: 0, right: 0, top: 0, bottom: 0, height: 0
                        };
                        var position = MS.Entertainment.UI.Controls.ModifierPopup.calculatePositionFromBoundingRect(boundingRect, openAbove);
                        var fakeModifierControl = false;
                        if (!modifierControl) {
                            modifierControl = {};
                            fakeModifierControl = true
                        }
                        var popupClass = "MS.Entertainment.UI.Controls.ModifierPopup";
                        if (MS.Entertainment.Utilities.isApp2)
                            popupClass = "MS.Entertainment.UI.Controls.App2ModifierPopup";
                        var openPopup = MS.Entertainment.UI.Shell.createOverlay(popupClass, {
                                modifierControl: modifierControl, selectionManager: selectionManager, customStyle: popupStyle, topPadding: Math.round(boundingRect.top), bottomPadding: MS.Entertainment.Utilities.getWindowHeight() - Math.round(boundingRect.bottom), anchorElement: anchorElement
                            }, {
                                left: position.left, right: position.right, bottom: position.bottom, top: position.top, customStyle: "modifierPopupContainer", autoSetFocus: false, lightDismissEnabled: MS.Entertainment.Utilities.isApp2, enableKeyboardLightDismiss: MS.Entertainment.Utilities.isApp2, focusPreviouslyFocusedElement: false, showAnimation: function() {
                                        return WinJS.Promise.as()
                                    }
                            });
                        if (fakeModifierControl)
                            modifierControl.openPopup = openPopup;
                        return openPopup
                    }, selectTemplateOverride: function selectTemplateOverride(item) {
                        var template = this.itemTemplate;
                        if (item && item.item && item.item.templateOverride)
                            template = item.item.templateOverride;
                        return MS.Entertainment.UI.Framework.loadTemplate(template, null, true).then(function(templateControl) {
                                return templateControl
                            })
                    }
            }), ModifierPopupEntry: MS.Entertainment.UI.Framework.defineUserControl("/Controls/Modifier.html#modifierPopupEntryTemplate", null, {
                modifierControl: null, _skipDefer: true, item: null, isInitialSelected: false, preventHideDuringInitialize: true, allowAnimations: false, _gotData: function _gotData(data) {
                        var domEvent;
                        var label = String.empty;
                        if (data.item && (data.item.label || data.item.title))
                            label = data.item.label ? data.item.label : data.item.title;
                        var labelType = typeof label;
                        switch (labelType) {
                            case"string":
                                if (String.id[label])
                                    this.text.textContent = String.load(label);
                                else
                                    this.text.textContent = label;
                                if (this.isVoiceEnabled && data.item) {
                                    this.voicePhrase = data.item.voicePhrase || label;
                                    this.voicePhoneticPhrase = data.item.voicePhoneticPhrase || MS.Entertainment.UI.App2.VoiceStatics.emptyPronunciationValue;
                                    this.voiceConfidence = data.item.voiceConfidence || MS.Entertainment.UI.App2.VoiceStatics.defaultConfidence
                                }
                                break;
                            default:
                                MS.Entertainment.UI.Controls.fail("Unrecognized label type in modifier control popup entry: " + labelType);
                                break
                        }
                        if (data.isInitialSelected)
                            WinJS.Utilities.addClass(this.background, "initialSelectedModifierPopupEntry");
                        if (data.modifierControl._tabPanel)
                            MS.Entertainment.Framework.AccUtils.createAndAddAriaLink(this.domElement, data.modifierControl._tabPanel, "aria-controls");
                        domEvent = document.createEvent("Event");
                        domEvent.initEvent("ModifierPopupEntryLoaded", true, true);
                        this.domElement.dispatchEvent(domEvent)
                    }, onClicked: function onClicked() {
                        this.data.selectionManager.selectedItem = this.data.item;
                        WinJS.Utilities.addClass(this.background, "selectedModifierPopupEntry");
                        MS.Entertainment.Utilities.Telemetry.logModifierClicked(this.data.item.id, this.text.textContent);
                        if (this.data.modifierControl.openPopup)
                            this.data.modifierControl.openPopup.hide();
                        else
                            this.data.modifierControl.backingData.openPopup.hide()
                    }, _data: null, data: {
                        get: function() {
                            return this._data
                        }, set: function(value) {
                                if (value === this._data)
                                    return;
                                this._data = value;
                                if (!value)
                                    return;
                                this._gotData(value)
                            }
                    }
            })
    });
    WinJS.Namespace.define("MS.Entertainment.UI.Controls", {ModifierPopupEntryAdditionalStyles: MS.Entertainment.UI.Framework.deriveUserControl("MS.Entertainment.UI.Controls.ModifierPopupEntry", "/Controls/Modifier.html#modifierPopupEntryTemplate", null, {_gotData: function _gotData(data) {
                var addedStyle = WinJS.Utilities.getMember("item.styleName", data);
                if (addedStyle && this.domElement)
                    WinJS.Utilities.addClass(this.domElement, addedStyle);
                MS.Entertainment.UI.Controls.ModifierPopupEntry.prototype._gotData.call(this, data)
            }})});
    WinJS.Namespace.define("MS.Entertainment.UI.Controls", {App2ModifierPopup: MS.Entertainment.UI.Framework.deriveUserControl("MS.Entertainment.UI.Controls.ModifierPopup", "/Controls/Modifier.html#app2ModifierPopupTemplate", function app2ModifierPopupConstructor(){}, {onCloseClick: function onCloseClick() {
                var domEvent = document.createEvent("Event");
                domEvent.initEvent("dismissoverlay", true, true);
                this.domElement.dispatchEvent(domEvent)
            }})});
    WinJS.Namespace.define("MS.Entertainment.UI.Controls", {ModifierButton: MS.Entertainment.UI.Framework.deriveUserControl("MS.Entertainment.UI.Controls.Modifier", "/Controls/Modifier.html#modifierButtonTemplate", function modifierButtonConstructor(){}, {
            calculateModifierPopoverPosition: false, initialize: function() {
                    this.onClickWithContext = WinJS.Utilities.markSupportedForProcessing(this.onClickWithContext.bind(this));
                    MS.Entertainment.UI.Controls.Modifier.prototype.initialize.call(this)
                }, onClickWithContext: function onClickWithContext(args) {
                    this.onClicked(args)
                }
        }, {buttonText: null})});
    WinJS.Namespace.define("MS.Entertainment.UI.Controls", {TemplatelessModifier: MS.Entertainment.UI.Framework.deriveUserControl("MS.Entertainment.UI.Controls.Modifier", null, function templateLessModifier() {
            this.labelControl = this.domElement
        }, {
            _clickHandler: null, _keyDownHandler: null, _activateEventHandlers: null, templateStorage: null, initialize: function inititialize() {
                    MS.Entertainment.UI.Controls.Modifier.prototype.initialize.call(this);
                    this._activateEventHandlers = MS.Entertainment.Utilities.addEventHandlers(this.domElement, {
                        click: this.onClicked.bind(this), keydown: this.onKeyDown.bind(this)
                    })
                }, unload: function unload() {
                    if (this._activateEventHandlers)
                        this._activateEventHandlers.cancel();
                    MS.Entertainment.UI.Controls.Modifier.prototype.unload.call(this)
                }
        })})
})();
(function runModifierVoiceMixins() {
    if (WinJS.Utilities.getMember("MS.Entertainment.UI.App2.VoicePropertyMixin")) {
        WinJS.Class.mix(MS.Entertainment.UI.Controls.Modifier, MS.Entertainment.UI.App2.VoicePropertyMixin);
        WinJS.Class.mix(MS.Entertainment.UI.Controls.ModifierPopupEntry, MS.Entertainment.UI.App2.VoicePropertyMixin)
    }
})()
