﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
(function() {
    "use strict";
    scriptValidator();
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Controls");
    WinJS.Namespace.define("MS.Entertainment.UI.Controls", {
        Overlay: MS.Entertainment.UI.Framework.defineUserControl("/Controls/Overlay.html#overlayTemplate", function overlayConstructor(element, options) {
            this._handleLightDismissChanged = this._handleLightDismissChanged.bind(this);
            WinJS.Utilities.addClass(this.domElement, "removeFromDisplay");
            var keyEvent = "keydown";
            if (MS.Entertainment.Utilities.isApp2)
                keyEvent = "keyup";
            this.domElement.addEventListener(keyEvent, this.keyDown.bind(this));
            WinJS.Utilities.addClass(this.domElement, "overlayAnchor");
            document.body.appendChild(this.domElement);
            this.bind("customStyle", function applyCustomStyle(newValue, oldValue) {
                if (oldValue)
                    WinJS.Utilities.removeClass(this.domElement, oldValue);
                if (newValue)
                    WinJS.Utilities.addClass(this.domElement, newValue)
            }.bind(this));
            if (!this.persistOnNavigate) {
                var navigationService;
                if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.navigation))
                    navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                if (navigationService) {
                    this._eventHandlers = MS.Entertainment.Utilities.addEvents(navigationService, {currentPageChanged: this._dismissOnNavigate.bind(this)});
                    this._bindings = WinJS.Binding.bind(navigationService, {currentPage: {options: this._dismissOnNavigate.bind(this)}})
                }
                this._firstBindOnDismiss = false
            }
            if (this.dontWaitForContent)
                this._contentInitialized = true
        }, {
            criticalTemplate: false, _eventHandlers: null, _bindings: null, _skipDefer: true, _presentedToUser: false, _focusSuppressedElements: null, _navigating: false, _contentDismiss: false, _currentPage: null, _presetPreOverlayFocus: null, setFocusToNavigationService: false, _oldFocusRoot: null, _saveFocusOnShow: true, _overlayIdentifier: null, _uiStateBindings: null, result: null, visible: false, userControlInstance: null, resolvedUserControlConstructor: null, ignoreChildrenInitialization: true, autoSetFocus: true, excludeEndpointElements: true, focusPreviouslyFocusedElement: true, restorePreviouslyFocusedElement: true, focusFirstElementOnNavigate: true, userControlOptions: undefined, dontWaitForContent: false, dismissOnAppBarShown: false, enableKeyboardLightDismiss: false, listViewItemToFocusOnClose: null, hideReason: String.empty, onInitializeFinished: null, useGoBackForVoice: false, allowTypeToSearch: true, initialize: function initialize() {
                    this._tabConstrainerHelper = new MS.Entertainment.UI.Framework.TabConstrainer(this.domElement);
                    MS.Entertainment.UI.Controls.assert(this.overlayBackground, "Overlay: Element with data-ent-member='overlayBackground' not found");
                    MS.Entertainment.UI.Controls.assert(this.overlayContainer, "Overlay: Element with data-ent-member='overlayContainer' not found");
                    MS.Entertainment.UI.Controls.assert(this.overlayContent, "Overlay: Element with data-ent-member='overlayContent' not found");
                    if (this.setFocusToNavigationService && MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.navigation))
                        this._currentPage = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).currentPage;
                    this.bind("lightDismissEnabled", this._handleLightDismissChanged);
                    var that = this;
                    this._overlayIdentifier = MS.Entertainment.Utilities.Telemetry.TelemetryParameterValues.Overlay;
                    this.overlayContent.addEventListener("UserControlInitialized", function userControlInitialized(event) {
                        if (event.userControl instanceof that.resolvedUserControlConstructor) {
                            if (event.userControl && event.userControl.setOverlay)
                                event.userControl.setOverlay(that);
                            if (event.userControl && event.userControl.templateName)
                                this._overlayIdentifier = event.userControl.templateName;
                            that._contentInitialized = true;
                            if (that.visible)
                                that._show();
                            if (this.dontWaitForContent)
                                this._sendShowTelemetryEvent()
                        }
                    }.bind(this));
                    this._uiStateBindings = WinJS.Binding.bind(MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState), {appBarVisible: function onAppBarVisibleChanged(visible, oldVisible) {
                            if (visible && !oldVisible && this.dismissOnAppBarShown && this.visible) {
                                this.hideReason = MS.Entertainment.UI.Controls.OverlayHideReasons.lightDismiss;
                                this.hide()
                            }
                        }.bind(this)});
                    var UserControlConstructor = this.userControl;
                    var controlAttribute;
                    if (typeof UserControlConstructor === "string") {
                        UserControlConstructor = WinJS.Utilities.getMember(this.userControl);
                        controlAttribute = this.userControl
                    }
                    else
                        controlAttribute = "MS.Entertainment.UI.Framework.UserControl";
                    this.resolvedUserControlConstructor = UserControlConstructor;
                    if (this.userControlOptions)
                        this.userControlOptions._skipDefer = true;
                    MS.Entertainment.UI.Controls.assert(typeof UserControlConstructor === "function", "Overlay: " + this.userControl + " is not a function");
                    this.userControlInstance = new UserControlConstructor(this.overlayContent, this.userControlOptions);
                    this.overlayContent.setAttribute("data-win-control", controlAttribute);
                    this.overlayBackground.addEventListener("click", this.lightDismiss.bind(this));
                    if (this.onInitializeFinished)
                        this.onInitializeFinished()
                }, unload: function unload() {
                    if (this._eventHandlers) {
                        this._eventHandlers.cancel();
                        this._eventHandlers = null
                    }
                    if (this._bindings) {
                        this._bindings.cancel();
                        this._bindings = null
                    }
                    if (this._uiStateBindings) {
                        this._uiStateBindings.cancel();
                        this._uiStateBindings = null
                    }
                    if (this._focusSuppressedElements) {
                        MS.Entertainment.Utilities.restoreSubtreeKeyboardFocus(this._focusSuppressedElements);
                        this._focusSuppressedElements = null
                    }
                    MS.Entertainment.UI.Framework.UserControl.prototype.unload.call(this)
                }, lightDismiss: function lightDismiss() {
                    if (this.lightDismissEnabled && this.visible) {
                        this.hideReason = MS.Entertainment.UI.Controls.OverlayHideReasons.lightDismiss;
                        this.hide();
                        return true
                    }
                    return false
                }, keyDown: function(e) {
                    if (!this.enableKeyboardLightDismiss)
                        return;
                    if (e.keyCode !== WinJS.Utilities.Key.escape && e.keyCode !== WinJS.Utilities.Key.dismissButton && (!MS.Entertainment.Utilities.isApp2 || (e.keyCode !== WinJS.Utilities.Key.backspace || MS.Entertainment.Utilities.doesElementSupportKeyboardInput(document.activeElement))))
                        return;
                    if (this.lightDismiss()) {
                        e.stopPropagation();
                        e.preventDefault()
                    }
                }, _handleLightDismissChanged: function _handleLightDismissChanged(newValue, oldValue) {
                    if (oldValue && !newValue) {
                        MS.Entertainment.Utilities.setAccessibilityText(this.overlayBackground, null);
                        this.overlayBackground.removeAttribute("tabindex");
                        MS.Entertainment.Framework.AccUtils.checkAndSetAriaAttribute(null, this.overlayBackground, "role")
                    }
                    else if (!oldValue && newValue) {
                        MS.Entertainment.Utilities.setAccessibilityTextFromStringId(this.overlayBackground, String.id.IDS_CLOSE_BUTTON);
                        this.overlayBackground.setAttribute("tabindex", -1);
                        MS.Entertainment.Framework.AccUtils.checkAndSetAriaAttribute("button", this.overlayBackground, "role")
                    }
                }, show: function show() {
                    if (this.visible)
                        throw"Dialog.show error - dialog is already visible!";
                    if (this._showCancelled)
                        return WinJS.Promise.wrap();
                    if (!this.allowTypeToSearch && MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.typeToSearch))
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.typeToSearch).disableTypeToSearch();
                    this.visible = true;
                    this._navigating = false;
                    var eventProvider = new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell;
                    eventProvider.traceOverlay_Shown("Title: [" + this.title + "], UserControl: [" + this.userControl + "]");
                    var pageContainer = document.getElementById("pageContainer");
                    if (pageContainer)
                        pageContainer.setAttribute("aria-hidden", "true");
                    var newMusicPageContainer = document.querySelector(".control-app");
                    if (newMusicPageContainer)
                        newMusicPageContainer.setAttribute("aria-hidden", "true");
                    MS.Entertainment.UI.Controls.assert(!!pageContainer || !!newMusicPageContainer, "Overlay: The 'pageContainer' element was not found. Can't set aria-hidden on root container");
                    if (WinJS.Utilities.getMember("MS.Entertainment.UI.Framework.currentOverlayContainer.winControl.unregisterGlobalCommands"))
                        MS.Entertainment.UI.Framework.currentOverlayContainer.winControl.unregisterGlobalCommands();
                    this.registerGlobalCommands();
                    if (this._contentInitialized)
                        this._show();
                    this._hideCompletePromise = new WinJS.Promise(function(c, e, p) {
                        this._hideComplete = c
                    }.bind(this));
                    return this._hideCompletePromise
                }, hide: function hide() {
                    var completePromise = WinJS.Promise.wrap();
                    if (!this._contentInitialized && !this.visible) {
                        this._showCancelled = true;
                        return completePromise
                    }
                    if (!this._navigating && !this._contentDismiss && this._currentPage)
                        WinJS.Binding.unwrap(this._currentPage).specialFocusedElement = null;
                    if (this.visible) {
                        var eventProvider = new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell;
                        eventProvider.traceOverlay_Dismissed("Title: [" + this.title + "], UserControl: [" + this.userControl + "]");
                        if (MS.Entertainment.Utilities.isApp2)
                            MS.Entertainment.Utilities.playOverlayOut();
                        MS.Entertainment.UI.Framework.removeOverlayContainer(this.domElement);
                        this.hideAnimation(this.overlayContainer, null).done(function() {
                            if (!this.allowTypeToSearch && MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.typeToSearch))
                                MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.typeToSearch).enableTypeToSearch();
                            var focusInOverlay = MS.Entertainment.UI.Framework.focusedItemInContainer(this.domElement);
                            document.body.removeChild(this.domElement);
                            if (this._oldFocusRoot === this.domElement || !document.body.contains(this._oldFocusRoot))
                                this._oldFocusRoot = MS.Entertainment.UI.Framework.currentFocusContainer;
                            if (!this.isStartupDialog || (this._oldFocusRoot && this._oldFocusRoot !== document.body)) {
                                MS.Entertainment.UI.Framework.setFocusRoot(this._oldFocusRoot);
                                this._restorePreviouslyFocusedItem(focusInOverlay)
                            }
                            var pageContainer = document.getElementById("pageContainer");
                            if (pageContainer)
                                pageContainer.removeAttribute("aria-hidden");
                            var newMusicPageContainer = document.querySelector(".control-app");
                            if (newMusicPageContainer)
                                newMusicPageContainer.removeAttribute("aria-hidden");
                            MS.Entertainment.UI.Controls.assert(!!pageContainer || !!newMusicPageContainer, "Overlay: The 'pageContainer' element was not found. Can't remove aria-hidden on root container");
                            if (this._hideComplete)
                                this._hideComplete(this);
                            if (this.isVoiceEnabled)
                                this.refreshVoiceElements()
                        }.bind(this));
                        this.dispatchEvent("close", {sender: this});
                        completePromise = this._hideCompletePromise;
                        MS.Entertainment.UI.Controls.Overlay._visibleCount--;
                        if (MS.Entertainment.UI.Controls.Overlay._visibleCount === 0)
                            MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).overlayVisible = false
                    }
                    else
                        this._showCancelled = true;
                    this.unregisterGlobalCommands();
                    if (WinJS.Utilities.getMember("MS.Entertainment.UI.Framework.currentOverlayContainer.winControl.registerGlobalCommands"))
                        MS.Entertainment.UI.Framework.currentOverlayContainer.winControl.registerGlobalCommands();
                    this.visible = false;
                    return completePromise
                }, registerGlobalCommands: function registerGlobalCommands() {
                    if (!MS.Entertainment.Utilities.isApp2)
                        return;
                    WinJS.Utilities.addClass(document.body, "overlayDisplayed");
                    if (!this.allowBackNavigate) {
                        MS.Entertainment.UI.Shell.unregisterGlobalBackCommands();
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.backButton).hideBackButton();
                        if (this.lightDismissEnabled || this.useGoBackForVoice)
                            MS.Entertainment.UI.Shell.registerGlobalOverlayBackCommands(this.useGoBackForVoice)
                    }
                }, unregisterGlobalCommands: function unregisterGlobalCommands() {
                    if (!MS.Entertainment.Utilities.isApp2)
                        return;
                    WinJS.Utilities.removeClass(document.body, "overlayDisplayed");
                    if (!this.allowBackNavigate) {
                        if (this.lightDismissEnabled || this.useGoBackForVoice)
                            MS.Entertainment.UI.Shell.unregisterGlobalOverlayBackCommands(this.useGoBackForVoice);
                        MS.Entertainment.UI.Shell.registerGlobalBackCommands();
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.backButton).showBackButton()
                    }
                }, _dismissOnNavigate: function _dismissOnNavigate(newVal, oldVal) {
                    if (!this._firstBindOnDismiss && !this._navigating) {
                        this.hideReason = MS.Entertainment.UI.Controls.OverlayHideReasons.navigate;
                        this._navigating = true;
                        if (this.dismissOnNavigateDelay)
                            WinJS.Promise.timeout(this.dismissOnNavigateDelay).done(this.hide.bind(this));
                        else
                            this.hide()
                    }
                }, showAnimation: function showAnimation(element) {
                    return WinJS.UI.Animation.showPopup(element)
                }, hideAnimation: function hideAnimation(element) {
                    if (this.overlayContainer && this.overlayContainer.style)
                        this.overlayContainer.style.opacity = 0;
                    if (this._navigating)
                        return WinJS.Promise.as();
                    else
                        return WinJS.UI.Animation.hidePopup(element)
                }, setAccessibilityTitle: function setAccessibilityTitle(title) {
                    MS.Entertainment.Utilities.setAccessibilityText(this.overlayContainer, title)
                }, previouslyFocusedElement: {
                    get: function get_previouslyFocusedItem() {
                        return this._itemWithFocusBeforeOverlayShown
                    }, set: function set_previouslyFocusedItem(element) {
                            MS.Entertainment.UI.Controls.assert(!this.visible, "Overlay: previouslyFocusedElement can only be set before show() is called!");
                            this._itemWithFocusBeforeOverlayShown = element;
                            this._saveFocusOnShow = false
                        }
                }, _show: function _show() {
                    if (this._presentedToUser)
                        return;
                    this._presentedToUser = true;
                    this._dismissFlyouts();
                    if (this._saveFocusOnShow)
                        this._saveCurrentFocusedItem();
                    MS.Entertainment.UI.Controls.Overlay._visibleCount++;
                    if (MS.Entertainment.UI.Controls.Overlay._visibleCount > 0)
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).overlayVisible = true;
                    WinJS.Utilities.removeClass(this.domElement, "removeFromDisplay");
                    if (MS.Entertainment.Utilities.isApp2)
                        MS.Entertainment.Utilities.playOverlayIn();
                    if (WinJS.UI.AutomaticFocus) {
                        this._oldFocusRoot = WinJS.UI.AutomaticFocus.focusRoot;
                        WinJS.UI.AutomaticFocus.focusRoot = null
                    }
                    MS.Entertainment.UI.Framework.addOverlayContainer(this.domElement);
                    this.showAnimation(this.overlayContainer, null).then(function complete() {
                        MS.Entertainment.UI.Framework.setFocusRoot(this.domElement);
                        if (this.autoSetFocus)
                            MS.Entertainment.UI.Framework.focusFirstInSubTree(this.domElement, this.excludeEndpointElements);
                        this.dispatchEvent(this.events.overlayVisible, {sender: this});
                        if (!this.dontWaitForContent)
                            this._sendShowTelemetryEvent()
                    }.bind(this));
                    this.domElement.addEventListener("dismissoverlay", function() {
                        this._contentDismiss = true;
                        this.hide()
                    }.bind(this))
                }, _dismissFlyouts: function _dismissFlyouts() {
                    var flyouts = document.querySelectorAll(".win-flyout");
                    if (!flyouts)
                        return;
                    for (var i = 0; i < flyouts.length; i++) {
                        var flyout = flyouts[i] && flyouts[i].winControl;
                        if (flyout && flyout.hide)
                            flyout.hide()
                    }
                }, _sendShowTelemetryEvent: function _sendShowTelemetryEvent() {
                    var location = MS.Entertainment.Utilities.getCurrentUserLocation();
                    MS.Entertainment.Utilities.Telemetry.logPageView(this.overlayContainer, {
                        uri: this._overlayIdentifier, pageTypeId: MS.Entertainment.Utilities.Telemetry.PageTypeId.Popup
                    }, {uri: location})
                }, _saveCurrentFocusedItem: function _saveCurrentFocusedItem() {
                    if (this._presetPreOverlayFocus)
                        this._itemWithFocusBeforeOverlayShown = this._presetPreOverlayFocus;
                    else {
                        this._itemWithFocusBeforeOverlayShown = document.activeElement;
                        var containingListView;
                        var listViews = document.querySelectorAll(".win-listview");
                        for (var i = 0; i < listViews.length; i++)
                            if (listViews[i].contains(this._itemWithFocusBeforeOverlayShown))
                                containingListView = listViews[i].winControl;
                        if (containingListView) {
                            var focusIsInListViewSurface = false;
                            if (containingListView.element) {
                                var listViewSurface = containingListView.element.querySelector(".win-surface");
                                if (listViewSurface && listViewSurface.contains(this._itemWithFocusBeforeOverlayShown))
                                    focusIsInListViewSurface = true
                            }
                            if (focusIsInListViewSurface)
                                this._itemWithFocusBeforeOverlayShown = {
                                    listView: containingListView, item: containingListView.currentItem
                                }
                        }
                    }
                    if (this._currentPage)
                        WinJS.Binding.unwrap(this._currentPage).specialFocusedElement = this._itemWithFocusBeforeOverlayShown;
                    if (!containingListView && this.autoSetFocus && this._itemWithFocusBeforeOverlayShown && this.focusPreviouslyFocusedElement)
                        MS.Entertainment.UI.Framework.focusFirstFocusableAncestor(this._itemWithFocusBeforeOverlayShown)
                }, _restorePreviouslyFocusedItem: function _restorePreviouslyFocusedItem(focusInOverlay) {
                    var focusFirstInSubTree = false;
                    var previousItem = this._itemWithFocusBeforeOverlayShown;
                    if (!this.restorePreviouslyFocusedElement || (!previousItem || !focusInOverlay) && MS.Entertainment.Utilities.isApp1)
                        return;
                    var target = previousItem;
                    if (target && target.listView)
                        target = target.listView._element;
                    if (!MS.Entertainment.UI.Framework.canMoveFocus(target)) {
                        if (MS.Entertainment.UI.Framework.currentFocusContainer !== MS.Entertainment.UI.Framework.currentContentContainer)
                            return;
                        target = null
                    }
                    if (!target) {
                        if (MS.Entertainment.Utilities.isApp1 || !this.focusFirstElementOnNavigate)
                            return;
                        previousItem = target = MS.Entertainment.UI.Framework.currentFocusContainer;
                        focusFirstInSubTree = true
                    }
                    var listViewItem = this.listViewItemToFocusOnClose || (previousItem && previousItem.item);
                    if (previousItem && previousItem.listView && listViewItem) {
                        if (!this._navigating)
                            WinJS.Promise.timeout(100).then(function() {
                                if (MS.Entertainment.Framework.KeyboardInteractionListener.showKeyboardFocus)
                                    listViewItem.showFocus = true;
                                previousItem.listView.currentItem = listViewItem
                            })
                    }
                    else if (!focusFirstInSubTree)
                        MS.Entertainment.UI.Framework.focusFirstFocusableAncestor(previousItem);
                    else
                        MS.Entertainment.UI.Framework.tryAndFocusElementInSubTreeWithTimer(previousItem, 0)
                }, _tabConstrainerHelper: null, _itemWithFocusBeforeOverlayShown: null, _fadedElements: null, _hideComplete: null, _hideCompletePromise: null, _contentInitialized: false, _showCancelled: false, _firstBindOnDismiss: true
        }, {
            top: "20%", left: "20%", bottom: "20%", right: "20%", userControl: null, lightDismissEnabled: true, allowBackNavigate: false, customStyle: null, showContainerAnimationClass: "animShowOverlayContainer", hideContainerAnimationClass: "animHideOverlayContainer", showBackgroundAnimationClass: "animShowOverlayBackground", hideBackgroundAnimationClass: "animHideOverlayBackground", events: {overlayVisible: "overlayVisible"}
        }, {
            anyVisible: function anyVisible() {
                return MS.Entertainment.UI.Controls.Overlay._visibleCount > 0
            }, _visibleCount: 0, _dismissOnNavigateDelay: 750, _dismissOnNowPlaying: 2000, showLoadingOverlay: function showLoadingOverlay(loadingOptions) {
                    loadingOptions = loadingOptions || {};
                    if (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isSnapped)
                        return WinJS.Promise.as();
                    var showAnimation = function showAnimation() {
                            return WinJS.Promise.timeout()
                        };
                    if (loadingOptions.showAnimation)
                        showAnimation = loadingOptions.showAnimation;
                    var options = {
                            userControl: loadingOptions.userControl, top: 0, left: 0, bottom: 0, right: 0, customStyle: loadingOptions.overlayCustomStyle, persistOnNavigate: false, dontWaitForContent: false, showAnimation: showAnimation, enableKeyboardLightDismiss: !!loadingOptions.enableKeyboardLightDismiss, dismissOnNavigateDelay: MS.Entertainment.UI.Controls.Overlay._dismissOnNavigateDelay, setFocusToNavigationService: true, userControlOptions: loadingOptions.userControlOptions
                        };
                    if (loadingOptions.hideAnimation)
                        options.hideAnimation = loadingOptions.hideAnimation;
                    var newControl = document.createElement("div");
                    var overlay = new MS.Entertainment.UI.Controls.Overlay(newControl, options);
                    WinJS.Utilities.addClass(overlay.domElement, "loadingOverlay");
                    var progressControl = document.createElement("progress");
                    WinJS.Utilities.addClass(progressControl, "loadingRing win-ring win-large");
                    var completion = null;
                    var showPopover = function showPopover() {
                            if (loadingOptions.hideOnNowPlaying) {
                                var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                                if (navigationService.checkUserLocation(MS.Entertainment.UI.Monikers.fullScreenNowPlaying))
                                    WinJS.Promise.timeout(this._dismissOnNowPlaying).done(overlay.hide.bind(overlay))
                            }
                            var closeTimeoutPromise = null;
                            if (loadingOptions.closeTimeoutMS)
                                closeTimeoutPromise = WinJS.Promise.timeout(loadingOptions.closeTimeoutMS).then(function closeOverlay() {
                                    overlay.hide()
                                });
                            return overlay.show().then(function onComplete(params) {
                                    if (completion)
                                        completion(params);
                                    if (closeTimeoutPromise)
                                        closeTimeoutPromise.cancel()
                                })
                        }.bind(this);
                    var preShowPopover = function preShowPopover() {
                            if (loadingOptions.onInitialize)
                                return loadingOptions.onInitialize(overlay.overlayContent).then(showPopover);
                            else
                                return showPopover()
                        };
                    if (overlay.overlayContent)
                        return preShowPopover();
                    else {
                        overlay.onInitializeFinished = preShowPopover;
                        return new WinJS.Promise(function(c, e, p) {
                                completion = c
                            })
                    }
                }
        }), OverlayHideReasons: {
                other: "other", lightDismiss: "lightDismiss", navigate: "navigate"
            }
    })
})()
