﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
(function() {
    "use strict";
    scriptValidator("/Framework/corefx.js", "/Framework/debug.js", "/Framework/utilities.js");
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Controls");
    WinJS.Namespace.define("MS.Entertainment.UI.Controls", {DownloadManagerGroupActions: {
            PauseAll: "PauseAll", ResumeAll: "ResumeAll", CancelAll: "CancelAll"
        }});
    WinJS.Namespace.define("MS.Entertainment.UI.Controls", {
        VideoDownloadManagerDialog: MS.Entertainment.UI.Framework.defineUserControl("/Controls/Video/VideoDownloadManager.html#videoDownloadManagerDialogTemplate", function VideoDownloadManagerDialog() {
            this.templateSelectorConstructor = MS.Entertainment.UI.Controls.VideoDownloadTemplateSelector
        }, {
            _hasPopulatedList: false, _updatingGalleryItems: false, _overlay: null, _refreshTimer: null, _refreshInterval: 10, setOverlay: function setOverlay(instance) {
                    this._overlay = instance;
                    var that = this;
                    this._overlay.buttons = [WinJS.Binding.as({
                            title: String.load(String.id.IDS_VIDEO_DOWNLOAD_MANAGER_PAUSE_ALL), isEnabled: true, isAvailable: false, execute: function onPauseAll() {
                                    that._onPauseAll()
                                }
                        }), WinJS.Binding.as({
                            title: String.load(String.id.IDS_VIDEO_DOWNLOAD_MANAGER_RESUME_All), isEnabled: true, isAvailable: false, execute: function onResumeAll() {
                                    that._onResumeAll()
                                }
                        }), WinJS.Binding.as({
                            title: String.load(String.id.IDS_VIDEO_DOWNLOAD_MANAGER_CANCEL_ALL), isEnabled: true, isAvailable: false, execute: function onCancelAll() {
                                    that._onCancelAll()
                                }
                        }), WinJS.Binding.as({
                            title: String.load(String.id.IDS_VIDEO_DOWNLOAD_MANAGER_CLOSE), isEnabled: true, execute: function onClose() {
                                    that._onClose()
                                }
                        })];
                    this._overlay.defaultButtonIndex = 2;
                    this._overlay.cancelButtonIndex = 2;
                    this._setInitialState()
                }, _setInitialState: function _setInitialState() {
                    this._enablePauseAllButton(false);
                    this._enableResumeAllButton(false);
                    this._enableCancelAllButton(false);
                    this._queryDatabaseItems()
                }, _applyGroupAction: function applyGroupAction(groupActionType) {
                    return this.getMarketplaceNativeFileDownloadsAsync().then(function getMarketplaceNativeFileDownloadsAsync_complete(nativeItems) {
                            if (!nativeItems)
                                return;
                            for (var i = 0; i < nativeItems.length; i++) {
                                var nativeItem = nativeItems[i];
                                if (nativeItem.libraryTypeId !== Microsoft.Entertainment.Queries.ObjectType.video)
                                    continue;
                                if (nativeItem.taskStatus !== Microsoft.Entertainment.FileTransferStatus.completed && nativeItem.taskStatus !== Microsoft.Entertainment.FileTransferStatus.canceled && !nativeItem.isClosed)
                                    switch (groupActionType) {
                                        case MS.Entertainment.UI.Controls.DownloadManagerGroupActions.PauseAll:
                                            nativeItem.pause();
                                            break;
                                        case MS.Entertainment.UI.Controls.DownloadManagerGroupActions.CancelAll:
                                            nativeItem.cancel();
                                            break;
                                        case MS.Entertainment.UI.Controls.DownloadManagerGroupActions.ResumeAll:
                                            nativeItem.resume();
                                            break
                                    }
                            }
                        }.bind(this))
                }, _queryDatabaseItems: function queryDatabaseItems() {
                    return this.getMarketplaceNativeFileDownloadsAsync().then(function getMarketplaceNativeFileDownloadsAsync_complete(nativeDownloadTasks) {
                            var downloadTasks = [];
                            var videoLibraryIds = [];
                            if (!nativeDownloadTasks)
                                return;
                            for (var i = 0; i < nativeDownloadTasks.length; i++) {
                                var downloadTask = nativeDownloadTasks[i];
                                if (downloadTask.libraryTypeId !== Microsoft.Entertainment.Queries.ObjectType.video)
                                    continue;
                                if (downloadTask.taskStatus !== Microsoft.Entertainment.FileTransferStatus.completed && downloadTask.taskStatus !== Microsoft.Entertainment.FileTransferStatus.canceled && !downloadTask.isClosed) {
                                    downloadTasks.push(downloadTask);
                                    videoLibraryIds.push(downloadTask.libraryId)
                                }
                            }
                            return this._getVideoDetails(videoLibraryIds).then(function getVideoDetailsComplete(dbItems) {
                                    if (!dbItems)
                                        return;
                                    var dbItemArray = [];
                                    dbItems.forEach(function(currentDbItem) {
                                        var currentMediaItem = currentDbItem.item.data;
                                        downloadTasks.forEach(function(currentDownloadTask) {
                                            if (currentDownloadTask.mediaId === currentMediaItem.serviceId) {
                                                currentMediaItem.downloadTask = currentDownloadTask;
                                                currentMediaItem.hydrated = true;
                                                dbItemArray.push(currentMediaItem)
                                            }
                                        })
                                    });
                                    dbItemArray.sort(function(a, b) {
                                        if (a.downloadTask.taskStatus < b.downloadTask.taskStatus)
                                            return -1;
                                        else if (a.downloadTask.taskStatus === b.downloadTask.taskStatus)
                                            return b.downloadTask.percentage - a.downloadTask.percentage;
                                        else
                                            return 1
                                    });
                                    if (this._galleryControl)
                                        this._galleryControl.dataSource = new MS.Entertainment.Data.VirtualList(null, dbItemArray);
                                    this._hasPopulatedList = true;
                                    this._updateGroupActions(downloadTasks);
                                    this._resetTimer()
                                }.bind(this), function getVideoDetailsFailed(error) {
                                    MS.Entertainment.UI.Controls.fail("Error while querying the DB for the active downloads in the Download Manager.")
                                })
                        }.bind(this))
                }, _updateGalleryItems: function updateGalleryItems(newItems) {
                    if (!this._hasPopulatedList || !this._galleryControl || !this._galleryControl.dataSource || this._updatingGalleryItems)
                        return;
                    this._updatingGalleryItems = true;
                    this.getMarketplaceNativeFileDownloadsAsync().then(function getMarketplaceNativeFileDownloadsAsync_complete(nativeDownloadTasks) {
                        var downloadTasks = [];
                        if (!nativeDownloadTasks || nativeDownloadTasks.length === 0) {
                            if (this._galleryControl && this._galleryControl.dataSource) {
                                this._galleryControl.dataSource.setSource([]);
                                this._updateGroupActions(downloadTasks)
                            }
                            return
                        }
                        for (var i = 0; i < nativeDownloadTasks.length; i++) {
                            var downloadTask = nativeDownloadTasks[i];
                            if (downloadTask.libraryTypeId !== Microsoft.Entertainment.Queries.ObjectType.video)
                                continue;
                            if (downloadTask.taskStatus !== Microsoft.Entertainment.FileTransferStatus.canceled && downloadTask.taskStatus !== Microsoft.Entertainment.FileTransferStatus.completed && !downloadTask.isClosed)
                                downloadTasks.push(downloadTask)
                        }
                        downloadTasks.sort(function(a, b) {
                            if (a.taskStatus < b.taskStatus)
                                return -1;
                            else if (a.taskStatus === b.taskStatus)
                                return b.percentage - a.percentage;
                            else
                                return 1
                        });
                        for (var listIndex = 0; this._galleryControl && (listIndex < this._galleryControl.dataSource.count); listIndex++) {
                            var currentItem = this._galleryControl.dataSource.getItem(listIndex);
                            var found = false;
                            if (!currentItem || !currentItem.data)
                                continue;
                            for (var taskIndex = 0; taskIndex < downloadTasks.length; taskIndex++)
                                if (downloadTasks[taskIndex].mediaId === currentItem.data.serviceId) {
                                    currentItem.data.downloadTask = downloadTasks[taskIndex];
                                    found = true;
                                    break
                                }
                            if (!found) {
                                this._galleryControl.clearSelection();
                                this._galleryControl.dataSource.removeAt(listIndex);
                                listIndex--
                            }
                        }
                        for (var sortIndex = 0; sortIndex < downloadTasks.length; sortIndex++) {
                            var currentTask = downloadTasks[sortIndex];
                            for (var listIndex = 0; listIndex < this._galleryControl.dataSource.count; listIndex++) {
                                var listItem = this._galleryControl.dataSource.getItem(listIndex);
                                if (listItem && currentTask.mediaId === listItem.data.serviceId) {
                                    if (listIndex !== sortIndex)
                                        this._galleryControl.dataSource.moveAt(listIndex, sortIndex);
                                    break
                                }
                            }
                        }
                        this._updateGroupActions(downloadTasks);
                        this._resetTimer()
                    }.bind(this)).done(function() {
                        this._updatingGalleryItems = false
                    }.bind(this), function() {
                        this._updatingGalleryItems = false
                    }.bind(this))
                }, _getVideoDetails: function getVideoDetails(libraryIds) {
                    var query = new MS.Entertainment.Data.Query.libraryVideos;
                    query.objectIds = libraryIds;
                    return query.execute().then(function getVideoDetailsComplete(q) {
                            return q.result.items
                        }.bind(this), function getVideoDetailsFailed(error) {
                            MS.Entertainment.UI.Controls.fail("Error while querying the DB for the active downloads in the Download Manager.")
                        })
                }, _updateGroupActions: function updateGroupActions(downloadTasks) {
                    var active = 0;
                    var canRetry = 0;
                    var failed = 0;
                    var pending = 0;
                    var paused = 0;
                    for (var i = 0; i < downloadTasks.length; i++) {
                        var downloadTask = downloadTasks[i];
                        if (downloadTask.libraryTypeId !== Microsoft.Entertainment.Queries.ObjectType.video)
                            continue;
                        if (downloadTask.taskStatus !== Microsoft.Entertainment.FileTransferStatus.completed && downloadTask.taskStatus !== Microsoft.Entertainment.FileTransferStatus.canceled && !downloadTask.isClosed)
                            switch (downloadTask.taskStatus) {
                                case Microsoft.Entertainment.FileTransferStatus.pending:
                                case Microsoft.Entertainment.FileTransferStatus.notStarted:
                                    ++pending;
                                    break;
                                case Microsoft.Entertainment.FileTransferStatus.error:
                                    if (downloadTask.canRetry)
                                        ++canRetry;
                                    ++failed;
                                    break;
                                case Microsoft.Entertainment.FileTransferStatus.canceled:
                                    break;
                                case Microsoft.Entertainment.FileTransferStatus.paused:
                                    ++paused;
                                    break;
                                default:
                                    ++active;
                                    break
                            }
                    }
                    var hasPending = (pending > 0);
                    var hasActive = (active > 0);
                    var hasFailed = (failed > 0);
                    var hasRetryable = (canRetry > 0);
                    var hasPaused = (paused > 0);
                    this._enablePauseAllButton(hasPending || hasActive);
                    this._enableResumeAllButton(hasPaused);
                    this._enableCancelAllButton(hasPending || hasActive || hasFailed || hasPaused);
                    this.showDownloads = (this._galleryControl && this._galleryControl.dataSource && this._galleryControl.dataSource.count > 0)
                }, getMarketplaceNativeFileDownloadsAsync: function getMarketplaceNativeFileDownloadsAsync() {
                    if (Microsoft.Entertainment.FileTransferQuery) {
                        var query = new Microsoft.Entertainment.FileTransferQuery;
                        query.requestType = Microsoft.Entertainment.FileTransferRequestType.marketPlace;
                        if (query && Microsoft.Entertainment.FileTransferManager)
                            return Microsoft.Entertainment.FileTransferManager.getFileDownloadsByQueryAsync(query).then(function getFileDownloadsByQueryAsync_complete(nativeItems) {
                                    if (nativeItems)
                                        return WinJS.Promise.wrap(nativeItems);
                                    return WinJS.Promise.wrap([])
                                }, function getFileDownloadsByQueryAsync_error(e) {
                                    return WinJS.Promise.wrap([])
                                });
                        else
                            return WinJS.Promise.wrap([])
                    }
                    else
                        return WinJS.Promise.wrap([])
                }, _enablePauseAllButton: function enablePauseAllButton(enable) {
                    if (this._overlay)
                        this._overlay.buttons[0].isAvailable = enable
                }, _enableResumeAllButton: function enableResumeAllButton(enable) {
                    if (this._overlay)
                        this._overlay.buttons[1].isAvailable = enable
                }, _enableCancelAllButton: function enableCancelAllButton(enable) {
                    if (this._overlay)
                        this._overlay.buttons[2].isAvailable = enable
                }, _resetTimer: function _restartTimer() {
                    window.clearTimeout(this._timer);
                    var refreshListIntervalMs = this._refreshInterval * 1000;
                    this._timer = window.setTimeout(function() {
                        this._updateGalleryItems()
                    }.bind(this), refreshListIntervalMs)
                }, _onPauseAll: function onPauseAll() {
                    return this._applyGroupAction(MS.Entertainment.UI.Controls.DownloadManagerGroupActions.PauseAll)
                }, _onResumeAll: function onResumeAll() {
                    return this._applyGroupAction(MS.Entertainment.UI.Controls.DownloadManagerGroupActions.ResumeAll)
                }, _onCancelAll: function onCancelAll() {
                    return this._applyGroupAction(MS.Entertainment.UI.Controls.DownloadManagerGroupActions.CancelAll)
                }, _onClose: function onClose() {
                    if (this._overlay)
                        this._overlay.hide()
                }
        }, {
            showDownloads: true, templateSelectorConstructor: MS.Entertainment.UI.Controls.GalleryTemplateSelector
        }), VideoDownloadTemplateSelector: MS.Entertainment.UI.Framework.derive("MS.Entertainment.UI.Controls.GalleryTemplateSelector", function videoDownloadTemplateSelector(galleryView) {
                MS.Entertainment.UI.Controls.GalleryTemplateSelector.prototype.constructor.call(this);
                this._galleryView = galleryView;
                this.addTemplate("episode", "/Controls/Video/VideoDownloadItem.html#episodeDownloadItemHost");
                this.addTemplate("movie", "/Controls/Video/VideoDownloadItem.html#movieDownloadItemHost")
            }, {onSelectTemplate: function onSelectTemplate(item) {
                    var template = null;
                    if (item && item.data) {
                        var data = item.data || {};
                        switch (data.videoType) {
                            case Microsoft.Entertainment.Queries.VideoType.tvEpisode:
                                template = "episode";
                                break;
                            case Microsoft.Entertainment.Queries.VideoType.movie:
                                template = "movie";
                                break;
                            default:
                                throw new Error("videoType not recognized for video download manager!");
                        }
                    }
                    this.ensureTemplatesLoaded([template]);
                    return this.getTemplateProvider(template)
                }}), ShowVideoDownloadManagerDialogAction: MS.Entertainment.deferredDerive(MS.Entertainment.UI.Actions.Action, null, {
                automationId: MS.Entertainment.UI.AutomationIds.showDownloadManager, canExecute: function canExecute() {
                        return true
                    }, executed: function() {
                        MS.Entertainment.UI.Shell.showDialog(String.load(String.id.IDS_VIDEO_DOWNLOAD_MANAGER_TITLE), "MS.Entertainment.UI.Controls.VideoDownloadManagerDialog", {
                            width: null, height: null, customStyle: "videoDownloadManagerDialog"
                        })
                    }
            })
    });
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.showVideoDownloadManager, function() {
        return new MS.Entertainment.UI.Controls.ShowVideoDownloadManagerDialogAction
    })
})()
